/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package benchmark;

import static simulation.parser.InstanceTokens.AGENT;
import static simulation.parser.InstanceTokens.AGENTS;
import static simulation.parser.InstanceTokens.ARITY;
import static simulation.parser.InstanceTokens.BENCHMARK;
import static simulation.parser.InstanceTokens.CONSTRAINT;
import static simulation.parser.InstanceTokens.CONSTRAINTS;
import static simulation.parser.InstanceTokens.DCOP;
import static simulation.parser.InstanceTokens.DEFAULT_COST;
import static simulation.parser.InstanceTokens.DESCRIPTION;
import static simulation.parser.InstanceTokens.DOMAIN;
import static simulation.parser.InstanceTokens.DOMAINS;
import static simulation.parser.InstanceTokens.ID;
import static simulation.parser.InstanceTokens.INFINITY;
import static simulation.parser.InstanceTokens.INITIAL_COST;
import static simulation.parser.InstanceTokens.INSTANCE;
import static simulation.parser.InstanceTokens.INSTANCES;
import static simulation.parser.InstanceTokens.MAXIMAL_COST;
import static simulation.parser.InstanceTokens.MODEL;
import static simulation.parser.InstanceTokens.NAME;
import static simulation.parser.InstanceTokens.NB_AGENTS;
import static simulation.parser.InstanceTokens.NB_CONSTRAINTS;
import static simulation.parser.InstanceTokens.NB_DOMAINS;
import static simulation.parser.InstanceTokens.NB_INSTANCES;
import static simulation.parser.InstanceTokens.NB_RELATIONS;
import static simulation.parser.InstanceTokens.NB_TUPLES;
import static simulation.parser.InstanceTokens.NB_VALUES;
import static simulation.parser.InstanceTokens.NB_VARIABLES;
import static simulation.parser.InstanceTokens.REFERENCE;
import static simulation.parser.InstanceTokens.RELATION;
import static simulation.parser.InstanceTokens.RELATIONS;
import static simulation.parser.InstanceTokens.SCOPE;
import static simulation.parser.InstanceTokens.SEMANTICS;
import static simulation.parser.InstanceTokens.SIMPLE_MODEL;
import static simulation.parser.InstanceTokens.SOFT;
import static simulation.parser.InstanceTokens.TKC;
import static simulation.parser.InstanceTokens.TYPE;
import static simulation.parser.InstanceTokens.VARIABLE;
import static simulation.parser.InstanceTokens.VARIABLES;
import static util.Commands.AO;
import static util.Commands.Algorithm;
import static util.Commands.Algorithms;
import static util.Commands.Communication;
import static util.Commands.Communications;
import static util.Commands.Experiments;
import static util.Commands.GuiPresentation;
import static util.Commands.MaxNCCCs;
import static util.Commands.MaxURC;
import static util.Commands.MinURC;
import static util.Commands.NbRun;
import static util.Commands.Results;
import static util.Commands.channelType;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import kernel.VarIdentifier;
import kernel.order.AgentOrdering;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import simulation.DisProblem;
import simulation.DisSolverSimulator;
import simulation.parser.InstanceTokens;
import util.Logger;
import util.Tools;

public class DataSetTeamcore extends Generator {

	public static final String TEAMCORE = "TEAMCORE";
	protected static final String THE_AGENT = "AGENT";
	protected static final String THE_VARIABLE = "VARIABLE";
	protected static final String THE_CONSTRAINT = "CONSTRAINT";
	protected static final String F = "F";
	protected static final String TIMESLOTS = "TIMESLOTS";
	protected static final String PARTICIPANT = "PARTICIPANT";
	protected static final String EVENT = "EVENT";
	protected static final String WHITESPACE = " "; //" \t\r\n\"";
	protected static final String CLASS = "class";

	private String path = null;

	public static final String fixKeys = CLASS;
	private static final String PEAV = "PEAV";
	private static final String EAV = "EAV";
	public static int nbHeaderLine = 4;

	private final String[] classes = new String[] {"caseA", "caseB", "caseC", "caseD"};
	
	/** identifier, list of variables */
	protected Map<Integer, List<Integer>> clusters = null;

	/** The index of the cluster each vertex belongs to */
	protected Map<Integer, Integer> clusterOf = null;
	
	Map<Integer, Integer> mapOfVariables = null; 
	Map<Integer, Pair> mapOfConstraints = null;
	Map<Integer, Map<Pair, Integer>> mapOfRelations = null; 
	
	private int constraintIndex = 0;
	private int nbInstances = 30;
	private String formulation = null;
	private Integer event;
	
	public DataSetTeamcore(String path, String formulation) {
		super(TEAMCORE, SIMPLE_MODEL, TKC);
		this.path = path;
		this.formulation  = formulation;
		this.clusters = new HashMap<Integer, List<Integer>>();
		this.clusterOf = new HashMap<Integer, Integer>();
		this.mapOfVariables = new HashMap<Integer, Integer>();
		this.mapOfConstraints = new HashMap<Integer, Pair>();
		this.mapOfRelations = new HashMap<Integer, Map<Pair,Integer>>();
	}
	
	/**
	 * Create the thread for running experiments 
	 * @param experimentation : the configuration file of experimentation
	 */
	public DataSetTeamcore(File file) {
		super(file);
	}

	public static List<String> readFile(String fileName) {		
		List<String> list = new ArrayList<String>();
		try {
			BufferedReader br = new BufferedReader(new FileReader(fileName));
			String line = br.readLine();
			while (line != null) {
				list.add(line);
				line = br.readLine();
			}
			br.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

	public static String fileNameEAV(String fileName) {
		return fileName+".EAV";
	}

	public static String fileNamePEAV(String fileName) {
		return fileName+".PEAV";
	}

	public static String fileNameSCH(String fileName) {
		return fileName+".SCH";
	}
	
	public void parse(String fileName) {
		this.clusters = new HashMap<Integer, List<Integer>>();
		this.clusterOf = new HashMap<Integer, Integer>();
		this.mapOfVariables = new HashMap<Integer, Integer>();
		this.mapOfConstraints = new HashMap<Integer, Pair>();
		this.mapOfRelations = new HashMap<Integer, Map<Pair,Integer>>();
		this.constraintIndex = 0;
		List<String> dcop = null;
		if(this.formulation.equals(EAV)) {
			dcop = readFile(fileNameEAV(fileName));
		} else if(this.formulation.equals(PEAV)) {
			dcop = readFile(fileNamePEAV(fileName));
		}
		for(int i = 0 ; i<dcop.size() ; ) {
			String line = dcop.get(i++);
			if(line.startsWith(THE_AGENT)) {
				parseAgent(line);
			} else if(line.startsWith(THE_VARIABLE)) {
				parseVariable(line);
			} else if(line.startsWith(THE_CONSTRAINT)) {
				Map<Pair, Integer> relation = parseConstraint(line);
				while(i<dcop.size()) {
					line = dcop.get(i++);
					if(line.startsWith(F)) {
						parseCost(line, relation);
					} else {
						break;
					}
				}
				--i;
			}
		}
		if(this.formulation.equals(PEAV)) {
			dcop = readFile(fileNameSCH(fileName));
			this.event = 0;
			for(int i = 0 ; i<dcop.size() ; ) {
				String line = dcop.get(i++);
				if(line.startsWith(PARTICIPANT)) {
					parseParticipant(line);
				} 
				if(line.startsWith(EVENT)) {
					parseEvent(line);
				} 
			}
		}
	}

	private void parseAgent(String line) {
		String[] tokens = line.split(WHITESPACE);
		//tokens[0] AGENT
		assert (tokens[0].equals(THE_AGENT)) : "can not parse the following line as agent: "+line;
	}
	
	private void parseParticipant(String line) {
		assert (line.startsWith(PARTICIPANT)) : "can not parse the following line as PARTICIPANT: "+line;
		String[] tokens = line.split(WHITESPACE);
		//tokens[0] PARTICIPANT
		Integer agId = Integer.parseInt(tokens[1]) + 1; // add 1 to avoid 0
		this.clusters.put(agId, new ArrayList<Integer>());
	}

	private void parseEvent(String line) {
		assert (line.startsWith(EVENT)) : "can not parse the following line as EVENT: "+line;
		String e = line.substring(line.indexOf('[')+1, line.indexOf(']')).trim();
		String[] tokens = e.split(WHITESPACE);
		//tokens[0] EVENT 
		this.event++;
		//this.clusters.put(this.event, new ArrayList<Integer>());
		for (String token : tokens){
			int agId = Integer.parseInt(token)+1;
			this.clusters.get(agId).add(this.event);
			this.clusterOf.put(this.event, agId);
		}
		System.out.println(this.clusters);
	}
	
	private void parseVariable(String line) {
		String[] tokens = line.split(WHITESPACE);
		//tokens[0] VARIABLE
		assert (tokens[0].equals(THE_VARIABLE)) : "can not parse the following line as variable: "+line;
		//tokens[1] the variables ID 
		Integer id = Integer.parseInt(tokens[1]) + 1; // add 1 to avoid 0
		//tokens[2] the agent owner of the variable (1)
		//tokens[3] the number of possible values for the variable 
		// the variable can range from 0 to nbValues. 
		Integer nbValues = new Integer(tokens[3]) - 1; // to index the last value in the domain
		this.mapOfVariables.put(id, nbValues);		
		if(this.formulation.equals(EAV)) {
			if(!this.clusters.containsKey(id)) {
				this.clusters.put(id, new ArrayList<Integer>());
			}
			this.clusters.get(id).add(id);
			this.clusterOf.put(id, id);
		}
	}
	
	private Map<Pair, Integer> parseConstraint(String line) {
		String[] tokens = line.split(WHITESPACE);
		//tokens[0] CONSTRAINT
		assert (tokens[0].equals(THE_CONSTRAINT)) : "can not parse the following line as constraint: "+line;
		this.constraintIndex++;
		//tokens[1] the ID of the first constrained variable 
		Integer var1 = new Integer(tokens[1]) + 1; // add 1 to avoid 0
		//tokens[2] the ID of the second constrained variable 
		Integer var2 = new Integer(tokens[2]) + 1; // add 1 to avoid 0
		//tokens[3] the threshold for this constraint
		Pair constraint = new Pair(var1, var2);
		this.mapOfConstraints.put(constraintIndex, constraint);
		Map<Pair, Integer> relation = new HashMap<Pair, Integer>();
		this.mapOfRelations.put(constraintIndex, relation);
		return relation;
	}
	
	private void parseCost(String line, Map<Pair, Integer> relation) {
		String[] tokens = line.split(WHITESPACE);
		//tokens[0] F
		assert (tokens[0].equals(F)) : "\ncan not parse the following line as relation: "+line+ "\n"+relation+"\n";
		//tokens[1] the ID of the first constrained variable 
		Integer val1 = new Integer(tokens[1]);
		//tokens[2] the ID of the second constrained variable 
		Integer val2 = new Integer(tokens[2]);
		//tokens[3] the cost incurred when the variables take those values.
		Integer cost = new Integer(tokens[3]);
		Pair combination = new Pair(val1, val2);
		relation.put(combination, cost);

	}

	public String variablesToString() {
		StringBuffer sb = new StringBuffer();
		for(Entry<Integer, Integer> entry : this.mapOfVariables.entrySet()) {
			Integer agId = entry.getKey();
			VarIdentifier varId = new VarIdentifier(agId);
			int nbValues = entry.getValue();
			sb.append("variable ");
			sb.append(varId);
			sb.append(" domain ");
			sb.append("0.."+nbValues);
			sb.append("\n");
		}
		return sb.toString();
	}
	
	public String constraintsToString() {
		StringBuffer sb = new StringBuffer();
		for(Entry<Integer, Pair> entry : this.mapOfConstraints.entrySet()) {
			Integer rel = entry.getKey();
			sb.append("constraint ");
			sb.append(rel);
			sb.append(" scope ");
			Pair scope = entry.getValue();
			Integer var1 = scope.getFirst();
			Integer var2 = scope.getSecond();
			VarIdentifier varId1 = new VarIdentifier(var1); 
			VarIdentifier varId2 = new VarIdentifier(var2); 
			sb.append(varId1);
			sb.append(" ");
			sb.append(varId2);
			sb.append("\n");
		}
		return sb.toString();
	}

	public String relationsToString() {
		StringBuffer sb = new StringBuffer();
		for(Entry<Integer, Map<Pair, Integer>> entry : this.mapOfRelations.entrySet()) {
			Integer rel = entry.getKey();
			sb.append("relation ");
			sb.append(rel);
			sb.append(" combinations ");
			for(Entry<Pair, Integer> e : entry.getValue().entrySet()) {
				Integer cost = e.getValue();
				Pair key = e.getKey();
				Integer val1 = key.getFirst();
				Integer val2 = key.getSecond();
				sb.append(cost);
				sb.append(":");
				sb.append(val1);
				sb.append(" ");
				sb.append(val2);
				sb.append("|");
			}
			sb.append("\n");
		}
		return sb.toString();
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(this.variablesToString());
		sb.append(this.constraintsToString());
		sb.append(this.relationsToString());
		return sb.toString();
	}
	
	
	/** The XDisCSP format part */

	/**
	 * This method define the set of agents in the XDisCSP format
	 * 
	 * @return org.jdom2.Element the agents element
	 */
	public Element getAgents() {
		int nbAgents = this.clusters.size();
		Element agents = new Element(AGENTS);
		agents.setAttribute(NB_AGENTS, Integer.toString(nbAgents));
		for(Integer agId : this.clusters.keySet()) {
			Element agent = new Element(AGENT);
			agent.setAttribute(NAME, "A" + agId);
			agent.setAttribute(ID, "" + agId);
			agent.setAttribute(DESCRIPTION, AGENT + "-" + agId);
			agents.addContent(agent);
		}
		return agents;
	}

	/**
	 * This method define the domains in the XDisCSP format. 
	 * One domain with <tt>domainSize</tt> values ranged from 
	 * <tt>1</tt> to <tt>domainSize</tt>.
	 * 
	 * @param domainSize the number of values in the domain
	 * @return org.jdom2..Element the domains element
	 */
	public Element getDomaines() {
		int nbDomains = this.mapOfVariables.size();
		Element domains = new Element(DOMAINS);
		domains.setAttribute(NB_DOMAINS, Integer.toString(nbDomains));
		for(Entry<Integer, Integer> entry : this.mapOfVariables.entrySet()) {
			Integer varId = entry.getKey();
			Integer nbValues = entry.getValue();
			Element domain = new Element(DOMAIN);
			domain.setAttribute(NAME, "D" + varId);
			domain.setAttribute(NB_VALUES, Integer.toString(nbValues+1));
			domain.setText("0.." + nbValues);
			domains.addContent(domain);
		}
		return domains;
	}

	/**
	 * This method define the set of variables/vertices in the XDisCSP format
	 * 
	 * @return org.jdom2..Element the variables element
	 */
	public Element getVariables() {
		int nbVariables = this.mapOfVariables.size();
		Element variables = new Element(VARIABLES);
		variables.setAttribute(NB_VARIABLES, Integer.toString(nbVariables));
		for(Entry<Integer, List<Integer>> entry : this.clusters.entrySet()) {
			Integer agId = entry.getKey();
			List<Integer> vars = entry.getValue();
			for(int i = 0 ; i<vars.size() ; i++) {
				int varId = vars.get(i)+1;
				int index = i+1;
				Element variable = new Element(VARIABLE);
				variable.setAttribute(AGENT, "A" + Integer.toString(agId));
				variable.setAttribute(NAME, "X" + agId + "." + index);
				variable.setAttribute(ID, Integer.toString(index));
				variable.setAttribute(DOMAIN, "D"+varId);
				variable.setAttribute(DESCRIPTION, VARIABLE + "-" + agId + "." + index);
				variables.addContent(variable);
			}
		}
		return variables;
	}

	/**
	 * This method defines the set of constraints/edges in the XDisCSP format
	 * 
	 * @return org.jdom2..Element the constraints element
	 */
	public Element getConstrains() {
		int nbConstraint = this.mapOfConstraints.size();
		Element constraints = new Element(CONSTRAINTS);
		constraints.setAttribute(INITIAL_COST, "0");
		constraints.setAttribute(MAXIMAL_COST, INFINITY);
		constraints.setAttribute(NB_CONSTRAINTS, "" + nbConstraint);
		for(Entry<Integer, Pair> entry : this.mapOfConstraints.entrySet()) {
			Integer id = entry.getKey();
			Element constraint = new Element(CONSTRAINT);
			constraint.setAttribute(NAME, "C" + id);
			constraint.setAttribute(MODEL, TKC); // Totally Known Constraints
			constraint.setAttribute(ARITY, Integer.toString(2));
			Pair scope = entry.getValue();
			Integer agId1 = scope.getFirst();
			Integer agId2 = scope.getSecond();
			Integer index1 = 1;
			Integer index2 = 1;
			constraint.setAttribute(SCOPE, "X" + agId1 + "." + index1 + " X" + agId2 + "." + index2);
			constraint.setAttribute(REFERENCE, "R" + id);
			constraints.addContent(constraint);
		}
		return constraints;
	}
	
	/**
	 * This method defines the set of relations in the XDisCSP format. 
	 * Thus, for each constraint we generate a relation. 
	 */
	public Element getRelations() {
		int nbConstraint = this.mapOfRelations.size();
		Element relations = new Element(RELATIONS);
		relations.setAttribute(NB_RELATIONS, "" + nbConstraint);
		for(Entry<Integer, Map<Pair, Integer>> entry : this.mapOfRelations.entrySet()) {
			Integer id = entry.getKey();
			Element relation = new Element(RELATION);
			relation.setAttribute(NAME, "R" + id);
			relation.setAttribute(ARITY, Integer.toString(2));
			relation.setAttribute(SEMANTICS, SOFT);
			relation.setAttribute(DEFAULT_COST, INFINITY);
			Map<Pair, Integer> tuples = entry.getValue();
			relation.setAttribute(NB_TUPLES, Integer.toString(tuples.size()));
			StringBuffer sb = new StringBuffer();
			for(Entry<Pair, Integer> e : entry.getValue().entrySet()) {
				Integer cost = e.getValue();
				Pair key = e.getKey();
				Integer val1 = key.getFirst();
				Integer val2 = key.getSecond();
				sb.append(cost);
				sb.append(InstanceTokens.COST_SEPARATOR);
				sb.append(val1);
				sb.append(InstanceTokens.VALUE_SEPARATOR);
				sb.append(val2);
				sb.append(InstanceTokens.TUPLES_SEPARATOR);
			}
			relation.setText(sb.substring(0, sb.length()-1));
			relations.addContent(relation);
		}
		return relations;
	}
	
	/**
	 * This method return the description of the instance. This will be used in the 
	 * Graphical User Interface. 
	 * 
	 * @param nbOfInstance : the number of instance
	 * @param p1 : the network connectivity defined as the ratio of existing binary constraints,
	 * @param p2 : is the constraint tightness defined as the ratio of forbidden value pairs. 
	 * @param nbConstraint : the number of constraint in the instance
	 * @param nbTuples : the number of tuples 
	 * @return
	 */
	private Element getGuiPresentation(String name) {
		int nbAgents = this.clusters.size();
		int nbConstraints = this.mapOfConstraints.size();
		Element presentation = new Element(GuiPresentation);
		presentation.setAttribute(TYPE, DCOP);
		presentation.setAttribute(BENCHMARK, this.typeOfBenchmark);
		presentation.setAttribute(NAME, name);
		presentation.setAttribute(MODEL, this.constraintsModel);
		presentation.setAttribute(NB_AGENTS, Integer.toString(nbAgents));
		presentation.setAttribute(NB_CONSTRAINTS, Integer.toString(nbConstraints));
		return presentation;
	}

	private Element generateInstance(String name, int numInstance) {
		Element instance = new Element(INSTANCE);
		instance.addContent(this.getPresentation(numInstance, DCOP));
		instance.addContent(getAgents());
		instance.addContent(getDomaines());
		instance.addContent(getVariables());
		instance.addContent(getRelations());
		instance.addContent(getConstrains());
		instance.addContent(this.getGuiPresentation(name));
		return instance;
	}
	
	public void generate() {
		String fileBase = "OUT00";
		Element experiments = new Element(Experiments);
		Document confDocument = new Document(experiments);
		experiments.setAttribute(TYPE, DCOP);
		experiments.setAttribute(BENCHMARK, this.typeOfBenchmark);
		Element instances = new Element(INSTANCES);
		instances.setAttribute(TYPE, DCOP);
		instances.setAttribute(BENCHMARK, this.typeOfBenchmark);
		instances.setAttribute(NAME, fileBase+this.formulation);
		
		for(String pathExtention : this.classes) {
			String problemPath = this.path + File.separator + pathExtention;
			for(int i = 0 ; i<this.nbInstances  ; i++) {
				String fileName = fileBase+this.formulation + String.format("%02d", i);
				String absoluteFileName = problemPath + File.separator + fileBase + String.format("%02d", i);
				this.parse(absoluteFileName);
				int cmp = 0;
				for(List<Integer> value : clusters.values()) {
					cmp+= value.size();
				}
				System.out.println(cmp);
				System.exit(0);
				Element instance = this.generateInstance(fileName, i+1);
				this.save(problemPath, fileName+".xml", new Document(instance));
			}
		}
		Element set = new Element(CLASS);
		set.setAttribute(NAME, fileBase+this.formulation);
		set.setAttribute(InstanceTokens.NB_INSTANCES, Integer.toString(nbInstances) + "");
		set.setText(Arrays.toString(this.classes));
		instances.addContent(set);
		experiments.addContent(instances);
		String experimentsConfFileName = Experiments + "_" + this.formulation + "_" +this.typeOfBenchmark + "_" + this.nbInstances + ".xml";
		this.saveXML(path, experimentsConfFileName, confDocument);
	}

	class Pair implements Comparable<Pair>{
		protected Integer first;
		protected Integer second;

		public Pair(Integer v1, Integer v2) {
			if ((v1 == null) || (v2 == null)) throw new IllegalArgumentException("Pair cannot contain null values");
			this.first = v1;
			this.second = v2;
		}
		public Integer getFirst() {
			return this.first;
		}
		public Integer getSecond() {
			return this.second;
		}
		@Override
		public int compareTo(Pair pair) {
			int cmp1 = this.first - pair.getFirst();
			int cmp2 = this.second - pair.getSecond();
			return cmp1 * 1000 + cmp2;
		}
		public String toString() {
			return getFirst()+" "+getSecond();
		}
	}
	
	
	@Override
	public void run() {
		int nbInstances;
		ArrayList<String> keys = null;
		String filePath = this.experimentation.getParent();
		String directory = "";
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(this.experimentation);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Communications).getChildren();
		List<Element> algorithms = xp.getChild(Algorithms).getChildren();
		int nbAlgorithms = algorithms.size();
		String[] protocols = new String[nbAlgorithms];
		String[] orderings = new String[nbAlgorithms];
		for(int a = 0 ; a < nbAlgorithms ; a++) {
			Element algorithm = algorithms.get(a);
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(AO);
		}
		int nbCommunications = communications.size();
		for(int c = 0 ; c < nbCommunications ; c++) {
			Element communication = communications.get(c);
			String commType = communication.getAttributeValue(TYPE);
			String cht = communication.getAttributeValue(channelType);
			int minURC = new Integer(communication.getAttributeValue(MinURC));
			int maxURC = new Integer(communication.getAttributeValue(MaxURC));
			int nbRun = new Integer(communication.getAttributeValue(NbRun));
			long maxNCCCs = new Long(communication.getAttributeValue(MaxNCCCs));
			String baseName = instances.getAttribute(NAME).getValue();
			Element setElement = instances.getChild(CLASS);
			nbInstances = new Integer(setElement.getAttributeValue(NB_INSTANCES));
			String[] sets = util.Tools.arrayFromString(setElement.getText(), String.class);
			directory = filePath + File.separator + Results + "_" + nbInstances;
			try {
				File f = new File(directory);
				if (!f.exists()) {
					f.mkdir();
				}
			} catch (Exception e) {}
			directory += File.separator + commType + "_" + cht;
			try {
				File f = new File(directory);
				if (!f.exists()) {
					f.mkdir();
				}
			} catch (Exception e) {}
			for(int p = 0 ; p < protocols.length ; p++) { // Protocols
				Logger allResuls = new Logger(directory, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
				Logger results = new Logger(directory, protocols[p] + "-" + orderings[p] + ".csv");
				for(String set : sets) {
					HashMap<String, Double> setStatistics = new HashMap<String, Double>();
					for(int i = 0 ; i < nbInstances ; i++) {
						String problem = filePath + File.separator + set + File.separator + baseName + String.format("%02d", i) + ".xml.bz2";
						HashMap<String, Double> instanceStatistics = new HashMap<String, Double>();
						for(int r = 1 ; r <= nbRun ; r++) {
							DisProblem disProb = new DisProblem(problem, protocols[p]);
							DisSolverSimulator solver = new DisSolverSimulator(disProb);
							solver.setCutOffNCCCs(maxNCCCs);
							solver.addCommunicationMetric();
							solver.setMessageDelay(minURC, maxURC);
							solver.setCentralizedAO(AgentOrdering.getCentralizedAO(orderings[p]));
							solver.solve();
							if (keys == null) {
								keys = disProb.master.getStatisticsKeys();
							}
							HashMap<String, Double> runStatistics = solver.getStatistics();
							if (i == 0 && r == 1 && set.equals(sets[0])) {
								String header = TEAMCORE + COMMA;
								header += Algorithm + COMMA + protocols[p] + COMMA + AO + COMMA + orderings[p] + "\n";
								header += NB_INSTANCES + COMMA + nbInstances + "\n";
								header += channelType + COMMA + cht + COMMA + Communication + COMMA + commType + COMMA;
								header += MinURC + COMMA + minURC + COMMA + MaxURC + COMMA + maxURC + "\n\n";
								header += CLASS + COMMA + INSTANCE + COMMA + NbRun + COMMA + Tools.toString(keys);
								allResuls.printToLogln(header);
								header = TEAMCORE + COMMA;
								header += Algorithm + COMMA + protocols[p] + COMMA + AO + COMMA + orderings[p] + "\n";
								header += NB_INSTANCES + COMMA + nbInstances + "\n";
								header += channelType + COMMA + cht + COMMA + Communication + COMMA + commType + COMMA;
								header += MinURC + COMMA + minURC + COMMA + MaxURC + COMMA + maxURC + "\n\n";
								header += CLASS + COMMA + Tools.toString(keys);
								results.printToLogln(header);
							}
							allResuls.printToLogln(set + COMMA + i + COMMA + r + COMMA + Tools.statisticsToString(keys, runStatistics));
							instanceStatistics = Tools.merge(runStatistics, instanceStatistics);
							disProb.clear();
							disProb.setAgents(null);
							disProb = null;
							System.gc();
						}
						Tools.average(instanceStatistics, nbRun);
						setStatistics = Tools.merge(instanceStatistics, setStatistics);
					}
					Tools.average(setStatistics, nbInstances);
					results.printToLogln(set + COMMA + Tools.statisticsToString(keys, setStatistics));
				} // sets
				results.close();
				allResuls.close();
				keys = null;
			} // protocols
			ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
			printer.setFixKeys(fixKeys);
			ArrayList<String> paths = printer.printToGnuPlot(directory, "", commType);
			printer.plot(directory, paths);
			String header = TEAMCORE + COMMA + "\n";
			header += "\n#";
			header += channelType + COMMA + cht + COMMA + Communication + COMMA + commType + COMMA;
			header += MinURC + COMMA + minURC + COMMA + MaxURC + COMMA + maxURC + COMMA + NB_INSTANCES + COMMA + nbInstances;
			printer.printAllResults(directory, header);
		} //Communications
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		args = new String[] {"./samples/DATASET/SensorCases"};
		if (args.length!=1) {
			System.out.println("Usage: DataSetTeamCore path\nwhere path is the path where your files are stored.");
			System.exit(-1);
		}
		DataSetTeamcore dataset = new DataSetTeamcore(args[0], PEAV);
		dataset.generate();
	}
}
