/******************************************************************

 ******************************************************************/

package benchmark.nqueen;

import static simulation.parser.InstanceTokens.AGENT;
import static simulation.parser.InstanceTokens.AGENTS;
import static simulation.parser.InstanceTokens.ARITY;
import static simulation.parser.InstanceTokens.BENCHMARK;
import static simulation.parser.InstanceTokens.CONFLICTS;
import static simulation.parser.InstanceTokens.CONSTRAINT;
import static simulation.parser.InstanceTokens.CONSTRAINTS;
import static simulation.parser.InstanceTokens.DESCRIPTION;
import static simulation.parser.InstanceTokens.DOMAIN;
import static simulation.parser.InstanceTokens.DOMAINS;
import static simulation.parser.InstanceTokens.DisCSP;
import static simulation.parser.InstanceTokens.EXPRESSION;
import static simulation.parser.InstanceTokens.EXTENSION;
import static simulation.parser.InstanceTokens.FUNCTIONAL;
import static simulation.parser.InstanceTokens.ID;
import static simulation.parser.InstanceTokens.INSTANCE;
import static simulation.parser.InstanceTokens.INSTANCES;
import static simulation.parser.InstanceTokens.MODEL;
import static simulation.parser.InstanceTokens.NAME;
import static simulation.parser.InstanceTokens.NB_AGENTS;
import static simulation.parser.InstanceTokens.NB_CONSTRAINTS;
import static simulation.parser.InstanceTokens.NB_DOMAINS;
import static simulation.parser.InstanceTokens.NB_PREDICATES;
import static simulation.parser.InstanceTokens.NB_RELATIONS;
import static simulation.parser.InstanceTokens.NB_TUPLES;
import static simulation.parser.InstanceTokens.NB_VALUES;
import static simulation.parser.InstanceTokens.NB_VARIABLES;
import static simulation.parser.InstanceTokens.PARAMETERS;
import static simulation.parser.InstanceTokens.PREDICATE;
import static simulation.parser.InstanceTokens.PREDICATES;
import static simulation.parser.InstanceTokens.REFERENCE;
import static simulation.parser.InstanceTokens.RELATION;
import static simulation.parser.InstanceTokens.RELATIONS;
import static simulation.parser.InstanceTokens.SCOPE;
import static simulation.parser.InstanceTokens.SEMANTICS;
import static simulation.parser.InstanceTokens.SIMPLE_MODEL;
import static simulation.parser.InstanceTokens.TKC;
import static simulation.parser.InstanceTokens.TYPE;
import static simulation.parser.InstanceTokens.VARIABLE;
import static simulation.parser.InstanceTokens.VARIABLES;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import kernel.order.AgentOrdering;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import simulation.DisProblem;
import simulation.DisSolverSimulator;
import simulation.parser.InstanceParser;
import util.Commands;
import util.Logger;
import util.Tools;
import benchmark.Generator;
import benchmark.ResultPrinter;

public class NQueenGenerator extends Generator {

	private int minQueen = 4; // the number of queen
	private int maxQueen = 4; // the number of queen
	private int step = 1;
	private static final String NQueen = "NQueen";
	private static final String MIN_QUEEN = "MIN_QUEEN";
	private static final String MAX_QUEEN = "MAX_QUEEN";
	private static final String STEP = "STEP";

	private String constraintType = EXTENSION;
	private File directory = new File("./" + NQueen + "/");
	public final static String fixKeys = NB_AGENTS; // nbNodes, nbColours,

	public final static int nbHeaderLine = 3;

	public NQueenGenerator(File directory, int minQueen, int maxQueen, int step) {
		super(NQueen, SIMPLE_MODEL, TKC);
		this.directory = directory;
		if (!this.directory.isDirectory()) {
			this.directory.mkdir();
		}
		this.minQueen = minQueen;
		this.maxQueen = maxQueen;
		this.step = step;
	}
	
	/**
	 * Create the thread for running experiments 
	 * @param experimentation : the configuration file of experimentation
	 */
	public NQueenGenerator(File file) {
		super(file);
	}

	public NQueenGenerator(int n, String constraintType) {
		super(NQueen, SIMPLE_MODEL, TKC);
		this.minQueen = this.maxQueen = n;
		this.constraintType = constraintType;
	}

	public void setType(String constraintType) {
		this.constraintType = constraintType;
	}

	public Element getInstance(int nbQueen) {
		Element instance = new Element(INSTANCE);
		instance.addContent(this.getPresentation(1, DisCSP));
		instance.addContent(this.getAgents(nbQueen));
		instance.addContent(this.getDomaines(nbQueen));
		instance.addContent(this.getVariables(nbQueen));
		if (this.constraintType.equals(EXTENSION)) {
			instance.addContent(this.getExtentionRelations(nbQueen));
			instance.addContent(this.getExtentionConstraints(nbQueen));
		} else {
			instance.addContent(this.getIntentionPredicates(nbQueen));
			instance.addContent(this.getIntentionConstraints(nbQueen));
		}
		return instance;
	}

	private Element getAgents(int nbQueen) {
		Element agents = new Element(AGENTS);
		agents.setAttribute(NB_AGENTS, "" + nbQueen);
		for (int id = 1 ; id <= nbQueen ; id++) {
			Element agent = new Element(AGENT);
			agent.setAttribute(NAME, "Q" + id);
			agent.setAttribute(ID, "" + id);
			agent.setAttribute(DESCRIPTION, "Queen" + id);
			agents.addContent(agent);
		}
		return agents;
	}

	private Element getDomaines(int nbQueen) {
		Element domains = new Element(DOMAINS);
		domains.setAttribute(NB_DOMAINS, "1");
		Element domain = new Element(DOMAIN);
		domain.setAttribute(NAME, "D1");
		domain.setAttribute(NB_VALUES, "" + nbQueen);
		domain.setText("1.." + nbQueen);
		domains.addContent(domain);
		return domains;
	}

	private Element getVariables(int nbQueen) {
		Element variables = new Element(VARIABLES);
		variables.setAttribute(NB_VARIABLES, "" + nbQueen);
		for (int id = 1 ; id <= nbQueen ; id++) {
			Element variable = new Element(VARIABLE);
			variable.setAttribute(AGENT, "Q" + id);
			variable.setAttribute(NAME, "Q" + id + ".0");
			variable.setAttribute(ID, "0");
			variable.setAttribute(DOMAIN, "D1");
			variable.setAttribute(DESCRIPTION, "Queen" + id + ".0");
			variables.addContent(variable);
		}
		return variables;
	}

	private Element getExtentionRelations(int nbQueen) {
		Element relations = new Element(RELATIONS);
		relations.setAttribute(NB_RELATIONS, "" + (nbQueen - 1));
		int X = 3 * nbQueen - 2;
		for (int Q = 1 ; Q < nbQueen && X > nbQueen ; Q++) {
			Element relation = new Element(RELATION);
			relation.setAttribute(NAME, "R" + Q);
			relation.setAttribute(ARITY, "2");
			relation.setAttribute(NB_TUPLES, "" + X);
			relation.setAttribute(SEMANTICS, CONFLICTS);
			X = X - 2;
			StringBuffer tuples = new StringBuffer();
			for (int i = 1 ; i <= nbQueen ; i++) {
				for (int j = 1 ; j <= nbQueen ; j++) {
					if (!(i != j && (Math.abs(i - j) != Q))) {
						tuples = tuples.append("" + i + " " + j + ((i + j != 2 * nbQueen)? "|" : ""));
					}
				}
			}
			relation.setText(tuples.toString());
			relations.addContent(relation);
		}
		return relations;
	}

	/**
	 * 
	 */
	private Element getExtentionConstraints(int nbQueen) {
		Element constraints = new Element(CONSTRAINTS);
		constraints.setAttribute(NB_CONSTRAINTS, "" + nbQueen * (nbQueen - 1) / 2);
		int k = 1;
		for (int qi = 1 ; qi <= nbQueen ; qi++) {
			for (int qj = qi + 1 ; qj <= nbQueen ; qj++) {
				Element constraint = new Element(CONSTRAINT);
				constraint.setAttribute(NAME, "C" + (k++));
				constraint.setAttribute(MODEL, TKC);
				constraint.setAttribute(ARITY, "2");
				// constraint.setAttribute(SCOPE_AGENTS, "Q"+qi+" Q"+qj);
				constraint.setAttribute(SCOPE, "Q" + qi + ".0 Q" + qj + ".0");
				constraint.setAttribute(REFERENCE, "R" + (qj - qi));
				constraints.addContent(constraint);
			}
		}
		return constraints;
	}

	private Element getIntentionPredicates(int nbQueen) {
		Element predicates = new Element(PREDICATES);
		predicates.setAttribute(NB_PREDICATES, "" + 1);
		Element predicate = new Element(PREDICATE);
		predicate.setAttribute(NAME, "P1");
		Element parameters = new Element(PARAMETERS);
		parameters.setText(" int X int Y int Z ");
		predicate.addContent(parameters);
		Element expression = new Element(EXPRESSION);
		Element functional = new Element(FUNCTIONAL);
		functional.setText(" and(ne(X,Y),ne(abs(sub(X,Y)),Z)) ");
		expression.addContent(functional);
		predicate.addContent(expression);
		predicates.addContent(predicate);
		return predicates;
	}

	/**
	 * 
	 */
	private Element getIntentionConstraints(int nbQueen) {
		Element constraints = new Element(CONSTRAINTS);
		constraints.setAttribute(NB_CONSTRAINTS, "" + nbQueen * (nbQueen - 1) / 2);
		int k = 1;
		for (int i = 1 ; i <= nbQueen ; i++) {
			for (int j = i + 1 ; j <= nbQueen ; j++) {
				Element constraint = new Element(CONSTRAINT);
				constraint.setAttribute(NAME, "C" + (k++));
				constraint.setAttribute(MODEL, TKC);
				constraint.setAttribute(ARITY, "2");
				// constraint.setAttribute(SCOPE_AGENTS, "Q"+i+" Q"+j);
				constraint.setAttribute(SCOPE, "Q" + i + ".0 Q" + j + ".0");
				constraint.setAttribute(REFERENCE, "P1");
				Element parameters = new Element(PARAMETERS);
				parameters.setText(" Q" + i + ".0 Q" + j + ".0 " + (j - i) + " ");
				constraint.addContent(parameters);
				constraints.addContent(constraint);
			}
		}
		return constraints;
	}

	public void generate() {
		Element experiments = new Element(Commands.Experiments);
		Document confDocument = new Document(experiments);
		experiments.setAttribute(TYPE, DisCSP);
		experiments.setAttribute(BENCHMARK, this.typeOfBenchmark);
		Element instances = new Element(INSTANCES);
		instances.setAttribute(TYPE, DisCSP);
		instances.setAttribute(BENCHMARK, this.typeOfBenchmark);
		instances.setAttribute(NAME, NQueen);
		instances.setAttribute(MIN_QUEEN, "" + this.minQueen);
		instances.setAttribute(MAX_QUEEN, "" + this.maxQueen);
		instances.setAttribute(STEP, "" + this.step);
		experiments.addContent(instances);
		for (int nbQueen = this.minQueen ; nbQueen <= this.maxQueen ; nbQueen += this.step) {
			Element instance = this.getInstance(nbQueen);
			Document document = new Document(instance);
			this.save(NQueen + "_" + nbQueen + ".xml", document);
		}
		String experimentsConfFileName = Commands.Experiments + "_" + this.minQueen + "_" + this.maxQueen + ".xml";
		this.saveXML(directory.getPath(), experimentsConfFileName, confDocument);
	}

	public static void main(String[] args) throws IOException {
		String[] arg = {"-e", "8"}; // extension
		args = arg;
		if (args.length != 2) {
			System.out.println("Usage : simulation.NQueenGenerator -type N ");
			System.out.println("        type: -e for extension or -i for instension");
			System.exit(-1);
		}
		String type = args[0];
		int N = Integer.parseInt(args[1]);
		if (N < 4) {
			System.out.println("N must be >= 4");
			System.exit(-1);
		}
		if (!(type.equals("-e") || type.equals("-i"))) {
			System.out.println("type must be -e:extension or -i:intension");
			System.exit(-1);
		}
		if (type.equals("-e"))
			type = "ext";
		else
			type = "int";
		NQueenGenerator generator = new NQueenGenerator(N, type);
		generator.generate();
	}
	
	@Override
	public void run() {
		String filePath = this.experimentation.getParent();
		ArrayList<String> keys = null;
		String directory = filePath + File.separator + Commands.Results;
		try {
			File d = new File(directory);
			if (!d.exists()) {
				d.mkdir();
			}
		} catch (Exception e) {}
		// //////////////////////////////////////////////////////////////////////////////////////////////////////////
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(this.experimentation);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Commands.Communications).getChildren();
		List<Element> algorithms = xp.getChild(Commands.Algorithms).getChildren();
		String[] protocols = new String[algorithms.size()];
		String[] orderings = new String[algorithms.size()];
		int a = 0;
		Iterator<Element> it = algorithms.iterator();
		while (it.hasNext()) {
			Element algorithm = it.next();
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(Commands.AO);
			a++;
		}
		it = communications.iterator();
		while (it.hasNext()) {
			// /////////////////////////////////////////////////////////////////
			Element comm = it.next();
			String commType = comm.getAttributeValue(TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			long maxNCCCs = new Long(comm.getAttributeValue(Commands.MaxNCCCs));
			// /////////////////////////////////////////////////////////////////
			directory = filePath + File.separator + Commands.Results + File.separator + commType + "_" + channelType;
			try {
				File f = new File(directory);
				if (!f.exists()) {
					f.mkdir();
				}
			} catch (Exception e) {}
			String name = instances.getAttribute(NAME).getValue();
			int minQueen = new Integer(instances.getAttributeValue(MIN_QUEEN));
			int maxQueen = new Integer(instances.getAttributeValue(MAX_QUEEN));
			int step = new Integer(instances.getAttributeValue(STEP));
			for (int p = 0 ; p < protocols.length ; p++) { // Protocols
				Logger allResults = new Logger(directory, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
				Logger results = new Logger(directory, protocols[p] + "-" + orderings[p] + ".csv");
				for (int nbQueen = minQueen ; nbQueen <= maxQueen ; nbQueen += step) { // TODO
					HashMap<String, Double> instanceStatistics = new HashMap<String, Double>();
					DisSolverSimulator solver = null;
					DisProblem disProb = null;
					InstanceParser parser = new InstanceParser(name + "_" + nbQueen);
					String problem = filePath + File.separator + name + "_" + nbQueen + ".xml.bz2";
					parser.loadInstance(problem);
					parser.parse(false);
					for (int r = 0 ; r < nbRun ; r++) {
						AgentOrdering.getCentralizedAO(orderings[p]);
						disProb = new DisProblem(parser, protocols[p]);
						solver = new DisSolverSimulator(disProb);
						solver.setCutOffNCCCs(maxNCCCs);
						solver.solve();
						if(keys == null ){
							keys = disProb.master.getStatisticsKeys();
						}
						HashMap<String, Double> runStatistics = solver.getStatistics();
						if (nbQueen == minQueen && r == 0) {
							String s = ",";
							String header = NQueen + s;
							header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
							header +=
									channelType + s + channelType + s + Commands.Communication + s + commType + s + Commands.MinURC + s + minURC + s
											+ Commands.MaxURC + s + maxURC + "\n\n";
							header += NB_AGENTS + s + Commands.NbRun + s + Tools.toString(keys);
							allResults.printToLogln(header);
							header = NQueen + s;
							header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
							header +=
									channelType + s + channelType + s + Commands.Communication + s + commType + s + Commands.MinURC + s + minURC + s
											+ Commands.MaxURC + s + maxURC + "\n\n";
							header += NB_AGENTS + s + Tools.toString(keys);
							results.printToLogln(header);
						}
						instanceStatistics = Tools.merge(runStatistics, instanceStatistics);
						allResults.printToLogln(nbQueen + "," + (r + 1) + "," + Tools.statisticsToString(keys, instanceStatistics));
						disProb = null;
					}
					Tools.average(instanceStatistics, nbRun);
					results.printToLogln(nbQueen + "," + instanceStatistics);

					parser = null;
				}
				keys = null;
				allResults.close();
				results.close();
			}// Protocols
			String header =
					channelType + "," + channelType + "," + Commands.Communication + "," + commType + "," + Commands.MinURC + "," + minURC + ","
							+ Commands.MaxURC + "," + maxURC;
			ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
			printer.setFixKeys(fixKeys);
			ArrayList<String> paths = printer.printToGnuPlot(directory, "", commType);
			printer.plot(directory, paths);
			printer.printAllResults(directory, header);
		}// Communication
	}

}
