/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package benchmark.sensornetwork;

import static simulation.parser.InstanceTokens.AGENT;
import static simulation.parser.InstanceTokens.AGENTS;
import static simulation.parser.InstanceTokens.ARITY;
import static simulation.parser.InstanceTokens.BENCHMARK;
import static simulation.parser.InstanceTokens.CONSTRAINT;
import static simulation.parser.InstanceTokens.CONSTRAINTS;
import static simulation.parser.InstanceTokens.DESCRIPTION;
import static simulation.parser.InstanceTokens.DOMAIN;
import static simulation.parser.InstanceTokens.DOMAINS;
import static simulation.parser.InstanceTokens.DisCSP;
import static simulation.parser.InstanceTokens.EXPRESSION;
import static simulation.parser.InstanceTokens.FUNCTIONAL;
import static simulation.parser.InstanceTokens.ID;
import static simulation.parser.InstanceTokens.INSTANCE;
import static simulation.parser.InstanceTokens.INSTANCES;
import static simulation.parser.InstanceTokens.MODEL;
import static simulation.parser.InstanceTokens.NAME;
import static simulation.parser.InstanceTokens.NB_AGENTS;
import static simulation.parser.InstanceTokens.NB_CONSTRAINTS;
import static simulation.parser.InstanceTokens.NB_DOMAINS;
import static simulation.parser.InstanceTokens.NB_INSTANCES;
import static simulation.parser.InstanceTokens.NB_PREDICATES;
import static simulation.parser.InstanceTokens.NB_RELATIONS;
import static simulation.parser.InstanceTokens.NB_TUPLES;
import static simulation.parser.InstanceTokens.NB_VALUES;
import static simulation.parser.InstanceTokens.NB_VARIABLES;
import static simulation.parser.InstanceTokens.PARAMETERS;
import static simulation.parser.InstanceTokens.PREDICATE;
import static simulation.parser.InstanceTokens.PREDICATES;
import static simulation.parser.InstanceTokens.REFERENCE;
import static simulation.parser.InstanceTokens.RELATION;
import static simulation.parser.InstanceTokens.RELATIONS;
import static simulation.parser.InstanceTokens.SCOPE;
import static simulation.parser.InstanceTokens.SEMANTICS;
import static simulation.parser.InstanceTokens.SUPPORTS;
import static simulation.parser.InstanceTokens.TYPE;
import static simulation.parser.InstanceTokens.VARIABLE;
import static simulation.parser.InstanceTokens.VARIABLES;
import static util.Commands.DIRECTORY;
import static util.Commands.GRID;
import static util.Commands.GuiPresentation;
import static util.Commands.KC;
import static util.Commands.KV;
import static util.Commands.MAX_PV;
import static util.Commands.MIN_PV;
import static util.Commands.NB_SENSORS;
import static util.Commands.NB_TARGETS;
import static util.Commands.P_C;
import static util.Commands.P_V;
import static util.Commands.REAL;
import static util.Commands.STEP_PV;
import static util.Commands.Sensor;
import static util.Commands.Sensors;
import static util.Commands.Target;
import static util.Commands.Targets;
import static util.Commands.X_axis;
import static util.Commands.Y_axis;
import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;
import kernel.order.AgentOrdering;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import simulation.DisProblem;
import simulation.DisSolverSimulator;
import util.Commands;
import util.IntArray;
import util.Logger;
import util.Tools;
import benchmark.Generator;
import benchmark.ResultPrinter;
import benchmark.meetingscheduling.Place;

/**
 * This class implements the Uniform Random Grid-based or Real Sensor Networks Problem 
 * as described in <tt>[Bejar et al., 2005]</tt> <br/>
 * <b>R. Bejar , C. Domshlak, C. Fernandez, C. Gomes, B. Krishnamachari, B. Selman, and M. Valls</b>. 
 * <i>Sensor networks and distributed csp: communication, computation and complexity</i>. 
 * Artificial Intelligence (2005). 
 * 
 * @author Mohamed Wahbi
 *
 */
public class SensorDCSPGenerator extends Generator  {

	public static final String fixKeys = P_V;
	public HashMap<Integer, Sensor> mySensors = null; 
	public HashMap<Integer, Target> myTargets = null; 
	public static int nbHeaderLine = 4;
	/** number of instances to generate for each pair (Pc,Pv) */
	private int nbInstances = 25;
	/** number of sensors */
	private int nbSensors;
	/** number of targets */
	private int nbTargets;
	/** Kc-restricted compatibility */
	private int minKc = 2, maxKc = 5, stepKc = 1;
	/** Kv-restricted visibility */
	private int minKv = 2, maxKv = 5, stepKv = 1; 
	/** Density of visibility (Pv) */
	private int minPv = 10, maxPv = 90, stepPv = 10;
	/** Density of communication (Pc) */
	private int minPc = 10, maxPc = 90, stepPc = 10;
	/** The geography properties (number of x and y axis). */
	private int x_axis = 10, y_axis = 10;
	private int pc, pv, kc, kv;
	private File directory = null;
	private Place[] targetsPlaces = null;
	private Place[] sensorsPlaces = null;
	private IntArray[] sensorsNeighbors = null;
	private IntArray[] targetsNeighbors = null;
	private Random rand = null;
	private int lineWidth = 2;
	private int margin = 50;
	private int width = 500;
	private int height = 500;
	private double unitX = 1;
	private double unitY = 1;
	
	/** The the type of the local layout where the sensors are installed: REAL or GRID */
	private String layout = REAL;
	
	private static RenderingHints hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

	/**
	 * Constructor
	 * @param benchmark : type of the benchmark: GSensorDCP or RSensorDCSP. 
	 * @param model : agents model, i.e., <tt>Simple</tt> or <tt>Complex</tt>.
	 * @param constraintModel : constraints model, i.e., <tt>TKC</tt> or <tt>PKC</tt>.
	 * @param layout : the format of the local layout where the sensors are installed.
	 */
	public SensorDCSPGenerator(String benchmark, String model, String constraintModel, String layout) {
		super(benchmark, model, constraintModel);
		this.setLayout(layout);
		this.rand = new Random();
		this.rand.setSeed(System.currentTimeMillis());
	}
	
	/**
	 * Constructor
	 * @param benchmark : type of the benchmark: GSensorDCP or RSensorDCSP. 
	 * @param model : agents model, i.e., <tt>Simple</tt> or <tt>Complex</tt>.
	 * @param constraintModel : constraints model, i.e., <tt>TKC</tt> or <tt>PKC</tt>.
	 * @param layout : the format of the local layout where the sensors are installed.
	 * @param nbSensors : the number on sensors
	 * @param nbTargets : the number on targets
	 */
	public SensorDCSPGenerator(String benchmark, String model, String constraintModel, String layout, 
			int nbSensors, int nbTargets) {
		this(benchmark, model, constraintModel, layout);
		this.nbSensors = nbSensors;
		this.nbTargets = nbTargets;
	}
	
	/**
	 * Constructor
	 * @param benchmark : type of the benchmark: GSensorDCP or RSensorDCSP. 
	 * @param model : agents model, i.e., <tt>Simple</tt> or <tt>Complex</tt>.
	 * @param constraintModel : constraints model, i.e., <tt>TKC</tt> or <tt>PKC</tt>.
	 * @param layout : the format of the local layout where the sensors are installed.
	 * @param directory : the directory where the instances will be stored
	 * @param nbInstances : the number on instances
	 * @param nbSensors : the number on sensors
	 * @param nbTargets : the number on targets
	 */
	public SensorDCSPGenerator(String benchmark, String model, String constraintModel, String layout, 
			File directory, int nbInstances, int nbSensors, int nbTargets) {
		this(benchmark, model, constraintModel, layout, nbSensors, nbTargets);
		this.directory = directory;
		if (!this.directory.isDirectory()) {
			this.directory.mkdir();
		}
		this.nbInstances = nbInstances;
	}
	
	/**
	 * Constructor
	 * @param benchmark : type of the benchmark: GSensorDCP or RSensorDCSP. 
	 * @param model : agents model, i.e., <tt>Simple</tt> or <tt>Complex</tt>.
	 * @param constraintModel : constraints model, i.e., <tt>TKC</tt> or <tt>PKC</tt>.
	 * @param layout : the format of the local layout where the sensors are installed.
	 * @param directory : the directory where the instances will be stored
	 * @param nbInstances : the number on instances
	 * @param nbSensors : the number on sensors
	 * @param nbTargets : the number on targets
	 * @param minPv : minimal density of visibility (Pv)
	 * @param maxPv : maximal density of visibility (Pv)
	 * @param stepPv : step of density of visibility (Pv)
	 * @param minKv : minimal Kv-restricted visibility
	 * @param maxKv : maximal Kv-restricted visibility
	 * @param stepKv : step of Kv-restricted visibility
	 * @param minPc : minimal density of communication between sensors (Pc)
	 * @param maxPc : maximal density of communication between sensors (Pc)
	 * @param stepPc : step of density of communication between sensors (Pc)
	 * @param minKc : minimal Kc-restricted compatibility
	 * @param maxKc : maximal Kc-restricted compatibility
	 * @param stepKc : step of Kc-restricted compatibility
	 * @param x_axis : number of x axis
	 * @param y_axis : number of y axis
	 * @param format : REAL or GRID
	 * @param model : type on instances to generate Simple or Complex. 
	 */
	public SensorDCSPGenerator(
			String benchmark, String model, String constraintModel, String layout, 
			File directory, int nbInstances, int nbSensors, int nbTargets, 
			int minPv, int maxPv, int stepPv, 
			int minKv, int maxKv, int stepKv, 
			int minPc, int maxPc, int stepPc, 
			int minKc, int maxKc, int stepKc,
			int x_axis, int y_axis) {
		this(benchmark, model, constraintModel, layout, directory, nbInstances, nbSensors, nbTargets);
		this.setVisibilityProperties(minPv, maxPv, stepPv, minKv, maxKv, stepKv);
		this.setCommunicationProperties(minPc, maxPc, stepPc, minKc, maxKc, stepKc);
		this.setGeographyProperties(x_axis, y_axis);
	}
	
	/**
	 * Create the thread for running experiments 
	 * @param experimentation : the configuration file of experimentation
	 */
	public SensorDCSPGenerator(File file) {
		super(file);
	}

	/**
	 * This method sets the visibility properties
	 * @param minPv : minimal density of visibility (Pv)
	 * @param maxPv : maximal density of visibility (Pv)
	 * @param stepPv : step of density of visibility (Pv)
	 * @param minKv : minimal Kv-restricted visibility
	 * @param maxKv : maximal Kv-restricted visibility
	 * @param stepKv : step of Kv-restricted visibility
	 */
	public void setVisibilityProperties(int minPv, int maxPv, int stepPv, int minKv, int maxKv, int stepKv) {
		this.minPv = minPv;
		this.maxPv = maxPv;
		this.stepPv = stepPv;
		this.minKv = minKv;
		this.maxKv = maxKv;
		this.stepKv = stepKv;
	}

	/**
	 * This method sets the compatibility properties
	 * @param minPc : minimal density of communication between sensors (Pc)
	 * @param maxPc : maximal density of communication between sensors (Pc)
	 * @param stepPc : step of density of communication between sensors (Pc)
	 * @param minKc : minimal Kc-restricted compatibility
	 * @param maxKc : maximal Kc-restricted compatibility
	 * @param stepKc : step of Kc-restricted compatibility
	 */
	public void setCommunicationProperties(int minPc, int maxPc, int stepPc, int minKc, int maxKc, int stepKc) {
		this.minPc = minPc;
		this.maxPc = maxPc;
		this.stepPc = stepPc;
		this.minKc = minKc;
		this.maxKc = maxKc;
		this.stepKc = stepKc;
	}

	/**
	 * This method sets the geography properties (number of x and y axis). 
	 * @param x_axis : number of x axis
	 * @param y_axis : number of y axis
	 */
	public void setGeographyProperties(int x_axis, int y_axis) {
		this.x_axis = x_axis;
		this.y_axis = y_axis;
	}
	
	/**
	 * This method sets the type of the local layout, i.e., REAL or GRID
	 * @param layout : the format of the local layout where the sensors are installed. 
	 */
	public void setLayout(String layout) {
		this.layout = layout;
	}

	public void generate() {
		String path = this.directory.getPath();
		Element experiments = new Element(Commands.Experiments);
		Document confDocument = new Document(experiments);
		experiments.setAttribute(TYPE, DisCSP);
		experiments.setAttribute(BENCHMARK, this.typeOfBenchmark);
		Element instances = new Element(INSTANCES);
		experiments.addContent(instances);
		instances.setAttribute(TYPE, DisCSP);
		instances.setAttribute(BENCHMARK, this.typeOfBenchmark);
		instances.setAttribute(NB_SENSORS, "" + this.nbSensors);
		instances.setAttribute(NB_TARGETS, "" + this.nbTargets);
		instances.setAttribute(X_axis, "" + this.x_axis);
		instances.setAttribute(Y_axis, "" + this.y_axis);
		String name = Commands.SensorDCSP + "_" + this.nbSensors + "_" + this.nbTargets + "_";
		instances.setAttribute(NAME, name);
		for (kc = this.minKc ; kc <= this.maxKc ; kc += this.stepKc) {
			for (kv = this.minKv ; kv <= this.maxKv ; kv += this.stepKv) {
				Element dir = new Element(DIRECTORY);
				String kckv = KC + "_" + kc + "-" + KV + "_" + kv;
				File folder = new File(path + File.separator + kckv);
				if (!folder.exists()) {
					folder.mkdir();
				}
				dir.setAttribute(NAME, KC + "_" + kc + "-" + KV + "_" + kv);
				dir.setAttribute(KC, "" + kc);
				dir.setAttribute(KV, "" + kv);
				for (pc = this.minPc ; pc <= this.maxPc ; pc += this.stepPc) {
					String subDensity = P_C + "_" + ((pc < 10)? "0" + pc : pc);
					File subFolder = new File(folder + File.separator + subDensity);
					if (!subFolder.exists()) {
						subFolder.mkdir();
					}
					Element cDensity = new Element(P_C);
					cDensity.setAttribute(NAME, subDensity);
					cDensity.setAttribute(P_C, "" + ((pc < 10)? "0" + pc : pc));
					Element vDensity = new Element(P_V);
					vDensity.setAttribute(MIN_PV, ""+((this.minPv < 10)? "0" + this.minPv : this.minPv));
					vDensity.setAttribute(MAX_PV, ""+((this.maxPv < 10)? "0" + this.maxPv : this.maxPv));
					vDensity.setAttribute(STEP_PV, "" + ((this.stepPv < 10)? "0" + this.stepPv : this.stepPv));
					vDensity.setAttribute(NB_INSTANCES, this.nbInstances + "");
					cDensity.addContent(vDensity);
					for (pv = this.minPv ; pv <= this.maxPv ; pv += this.stepPv) {
						String subVisibility = "" + ((pv < 10)? "0" + pv : pv);
						File subsubFolder = new File(subFolder.getPath() + File.separator + subVisibility);
						if (!subsubFolder.exists()) {
							subsubFolder.mkdir();
						}
						for (int i = 0 ; i < this.nbInstances ; i++) {
							Element instance = null;
							if (model.equals(Commands.SIMPLE)) {
								instance = this.generateSimpleInstance();
							} else {
								instance = this.generateComplexInstance();
							}
							Document document = new Document(instance);
							this.save(subsubFolder.getPath(), name + (i + 1) + ".xml", document);
						}
					}
					dir.addContent(cDensity);
				}
				instances.addContent(dir);
			}
		}
		String experimentsConfFileName = Commands.Experiments + "_" + this.nbInstances + ".xml";
		this.saveXML(path, experimentsConfFileName, confDocument);
		
	}

	/**
	 * This method generates compatibility graph. 
	 */
	private void generateCompatibilityGraph() {
		this.sensorsNeighbors = new IntArray[this.nbSensors];
		for (Place sensor : this.sensorsPlaces) {
			int s = sensor.getID();
			this.sensorsNeighbors[s] = new IntArray();
			if (layout.equals(GRID)) {
				int x = (int) sensor.getX();
				int y = (int) sensor.getY();
				for (Place neighbor : this.sensorsPlaces) {
					int neig = neighbor.getID();
					int i = (int) neighbor.getX();
					int j = (int) neighbor.getY();
					if ( (i>=(x-kc)) && (i<=(x+kc)) && (j>=(y-kc)) && (j<=(y+kc)) && neig!=s){
						this.sensorsNeighbors[s].add(neig);
					}
				}
			} else { // Real 
				for (Place neighbor : this.sensorsPlaces) {
					int neig = neighbor.getID();
					if (sensor.distance(neighbor) <= kc && neig!=s) {
						this.sensorsNeighbors[s].add(neig);
					}
				}
			}
		}
//		if(model.equals(GRID)){//TODO
//			for (int s = 0 ; s < this.nbSensors ; s++) {
//				int nbNeigh = (this.sensorsNeighbors[s].size() * pc) / 100;
//				for (int i = 0 ; i < nbNeigh ; i++) {
//					int selected = this.rand.nextInt(this.sensorsNeighbors[s].size());
//					this.sensorsNeighbors[s].removeAt(selected);
//				}
//			}
//		} else {
			ArrayList<Couple> combination = new ArrayList<Couple>(); // combination
			for (Place sensor : this.sensorsPlaces) {
				int s = sensor.getID();
				for(int u : this.sensorsNeighbors[s]){
					Couple c = new Couple(s, u);
					if(!combination.contains(c)){
						combination.add(c);
					}
				}
			}
			int toRemove = (combination.size() * pc) / 100;
			for(int i=0 ; i<toRemove ; i++){
				int selected = this.rand.nextInt(combination.size());
				Couple c = combination.remove(selected);
				this.sensorsNeighbors[c.x].remove(c.y); 
				this.sensorsNeighbors[c.y].remove(c.x); 
			}
////			for(Couple c : combination){
////				System.out.println(c);
////			}
//		}
	}

	/**
	 * This method search the index of the given sensor. 
	 * @param sensor : 
	 * @return integer : the index of the given sensor; -1 if the sensor does not belong to the set of sensors. 
	 */
	protected int indexOfSensor(Place sensor) {
		for (int i = 0 ; i < this.sensorsPlaces.length ; i++) {
			if (this.sensorsPlaces[i].equals(sensor)) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * This method generates a random place for every target
	 */
	private void generateTargets() {
		this.targetsPlaces = new Place[this.nbTargets];
		for (int t = 0 ; t < this.targetsPlaces.length ; t++) {
			double x = this.rand.nextFloat() * (this.x_axis - 1);
			double y = this.rand.nextFloat() * (this.y_axis - 1);
			this.targetsPlaces[t] = new Place(t, x, y, Target + (t + 1));
		}
	}

	/**
	 * This method generates a random place for every sensor
	 */
	private void generateSensors() {
		this.sensorsPlaces = new Place[this.nbSensors];
		if (this.layout.equals(GRID)) {
			for (int s = 0 ; s < this.nbSensors ; s++) {
				int x = s % this.x_axis;
				int y = s / this.x_axis;
				this.sensorsPlaces[s] = new Place(s, x, y, Sensor + (s + 1));
			}
		} else{ // Real Layout
			for (int s = 0 ; s < this.sensorsPlaces.length; s++) {
				double x = this.rand.nextDouble() * (this.x_axis - 1);
				double y = this.rand.nextDouble() * (this.y_axis - 1);
				this.sensorsPlaces[s] = new Place(s, x, y, Sensor + (s + 1));
			}
		}
	}

	/**
	 * This method generates compatibility graph. 
	 * @return
	 */
	private boolean generateVisibilityGraph() {
		this.targetsNeighbors = new IntArray[this.nbTargets];
		for (Place target : this.targetsPlaces) {
			int t = target.getID();
			this.targetsNeighbors[t] = new IntArray();
			if(this.layout.equals(GRID)){
				for (Place sensor : this.sensorsPlaces) {
					double x = Math.floor(target.getX());
					double y = Math.floor(target.getY());
					int id = sensor.getID();
					int i = (int) sensor.getX();
					int j = (int) sensor.getY();
					if ( (i>(x-kv) && i<=(x+kv)) && (j>(y-kv) && j<=(y+kv)) ) {
						this.targetsNeighbors[t].add(id);
					}
				} 
			} else { // Real layout
				for (Place sensor : this.sensorsPlaces) {
					if (target.distance(sensor) <= kv) {
						this.targetsNeighbors[t].add(sensor.getID());
					}
				}
			}
		}
		if(this.layout.equals(GRID)){
			for (int t = 0 ; t < this.targetsNeighbors.length ; t++) {
				int nbNeigh = (int) (this.targetsNeighbors[t].size() * ((double) pv / 100));
				for (int i = 0 ; i < nbNeigh ; i++) {
					int selected = this.rand.nextInt(this.targetsNeighbors[t].size());
					this.targetsNeighbors[t].removeAt(selected);
				}
				if(this.targetsNeighbors[t].isEmpty()){
					return false;
				}
			}
		} else { // Real layout
			IntArray combination = new IntArray();// combination 
			for (Place target : this.targetsPlaces) {
				int t = target.getID();
				for(int s : this.targetsNeighbors[t]){
					combination.add(t*nbSensors + s);
				}
			}
			int toRemove = (int) (combination.size() * ((double) pv / 100));
			for (int i = 0 ; i < toRemove ; i++) {
				int index = this.rand.nextInt(combination.size());
				int selected = combination.get(index);
				int t = selected/nbSensors;
				int s = selected%nbSensors;
				this.targetsNeighbors[t].remove(s);
				combination.removeAt(index);
			}
			for (Place target : this.targetsPlaces) {
				int t = target.getID();
				if(this.targetsNeighbors[t].isEmpty()){
					return false;
				}
			}
		}
		return true;//TODO
	}

	private Element generateSimpleInstance() {// TODO Auto-generated method stub
		/*
		this.generateTargets();
		this.generateSensors();
		this.generateCompatibilityGraph();
		this.generateVisibilityGraph();
		Element instance = new Element(INSTANCE);
		instance.addContent(this.getPresentation(Commands.SIMPLE));
		instance.addContent(this.getAgents());
		this.toSimpleModel();
		System.err.println("Not yet implemented for SIMPLE agents");
		throw new Error("");
		*/
		/*
		 * instance.addContent(this.getSimpleDomaines()); instance.addContent(this.getSimpleVariables());
		 * instance.addContent(this.getSimplePredicates()); instance.addContent(this.getSimpleRelations());
		 * instance.addContent(this.getSimpleConstraints());
		 * instance.addContent(this.getGuiPresentation(Commands.SIMPLE));
		 */
		return null;
	}

	public Element generateComplexInstance() {
		boolean uniform = false;
		int nbRun = 0;
		do {
//			System.out.println("nbRun : "+nbRun);
			this.generateTargets();
			this.generateSensors();
			this.generateCompatibilityGraph();
			uniform = this.generateVisibilityGraph();//TODO
			if (nbRun++ >= 100000) { 
				throw new Error("\n" +
						"You can not generate uniform problem with this properties:\n" + 
						"nbSensors:" + this.nbSensors + 
						" nbTargets" + this.nbTargets + 
						" Pc:" + pc + " Pv:" + pv + " Kc:" + kc + " Kv:" + kv);
			}
		} while (!uniform);
		Element instance = new Element(INSTANCE);
		instance.addContent(this.getPresentation(this.nbInstances, DisCSP));
		instance.addContent(this.getAgents());
		instance.addContent(this.getDomaines());
		instance.addContent(this.getVariables());
		instance.addContent(this.getPredicates());
		instance.addContent(this.getRelations());
		instance.addContent(this.getConstraints());
		instance.addContent(this.getGuiPresentation());
		return instance;
	}
	
	private Element getAgents() {
		Element agents = new Element(AGENTS);
		agents.setAttribute(NB_AGENTS, "" + this.nbTargets);
		for (int t = 1 ; t <= this.nbTargets ; t++) {
			Element agent = new Element(AGENT);
			agent.setAttribute(NAME, "T" + t);
			agent.setAttribute(Commands.ID, "" + t);
			agent.setAttribute(DESCRIPTION, this.targetsPlaces[t - 1].pretty());
			agents.addContent(agent);
		}
		return agents;
	}

	private Element getDomaines() {
		Element domains = new Element(DOMAINS);
		domains.setAttribute(NB_DOMAINS, "" + this.nbTargets);
		for (int t = 0 ; t < this.nbTargets ; t++) {
			Element domain = new Element(DOMAIN);
			domain.setAttribute(NAME, "D" + (t + 1));
			domain.setAttribute(NB_VALUES, "" + this.targetsNeighbors[t].size());
			String values = this.targetsNeighbors[t].toString();
			domain.setText(values.substring(values.indexOf("[") + 1, values.indexOf("]")));
			domains.addContent(domain);
		}
		return domains;
	}

	private Element getVariables() {
		int nbSensorPerTarget = 3;
		Element variables = new Element(VARIABLES);
		variables.setAttribute(NB_VARIABLES, "" + this.nbTargets * nbSensorPerTarget);
		for (int t = 1 ; t <= this.nbTargets ; t++) {
			for (int s = 0 ; s < nbSensorPerTarget ; s++) {
				Element variable = new Element(VARIABLE);
				variable.setAttribute(AGENT, "T" + t);
				variable.setAttribute(NAME, "S" + t + "." + s);
				variable.setAttribute(ID, "" + s);
				variable.setAttribute(DOMAIN, "D" + t);
				variable.setAttribute(DESCRIPTION, Sensor + "_" + t + "." + s);
				variables.addContent(variable);
			}
		}
		return variables;
	}

	/**
	 * 
	 * @return
	 */
	private Element getPredicates() {
		Element predicates = new Element(PREDICATES);
		predicates.setAttribute(NB_PREDICATES, "" + 1);
		Element neqPredicate = new Element(PREDICATE);
		neqPredicate.setAttribute(NAME, "P0");
		Element eqParameters = new Element(PARAMETERS);
		eqParameters.setText("int Si int Sj ");
		neqPredicate.addContent(eqParameters);
		Element eqExpression = new Element(EXPRESSION);
		Element eqFunctional = new Element(FUNCTIONAL);
		eqFunctional.setText(" ne(Si,Sj) ");
		eqExpression.addContent(eqFunctional);
		neqPredicate.addContent(eqExpression);
		predicates.addContent(neqPredicate);
		return predicates;
	}

	/**
	 * 
	 * @return
	 */
	private Element getRelations() {
		Element relations = new Element(RELATIONS);
		for (int t = 0 ; t < this.targetsNeighbors.length ; t++) {
			IntArray sensors = this.targetsNeighbors[t];
			int nbTuples = 0;
			Element relation = new Element(RELATION);
			relation.setAttribute(NAME, "R" + t);
			relation.setAttribute(ARITY, "2");
			String tuples = "";
			for (int i = 0 ; i < sensors.size() - 1 ; i++) {
				for (int j = i + 1 ; j < sensors.size() ; j++) {
					int si = sensors.get(i);
					int sj = sensors.get(j);
					if (this.sensorsNeighbors[si].contains(sj) || this.sensorsNeighbors[sj].contains(si)) {
						tuples += si + " " + sj + "|" + sj + " " + si + "|";
						nbTuples += 2;
					}
				}
			}
			//if (nbTuples > 0) {
				relation.setAttribute(NB_TUPLES, "" + nbTuples);
				relation.setAttribute(SEMANTICS, SUPPORTS);
				relation.setText(tuples.substring(0, Math.max(tuples.length() - 1, 0)));
				relations.addContent(relation);
			//}
		}
		relations.setAttribute(NB_RELATIONS, "" + this.targetsNeighbors.length);
		return relations;
	}

	/**
	 * 
	 * @return
	 */
	private Element getConstraints() {
		Element constraints = new Element(CONSTRAINTS);
		int nbConstraints = 0;
		for (int t = 0 ; t < this.targetsNeighbors.length ; t++) {
			for (int i = 0 ; i < 2 ; i++) {
				for (int j = i + 1 ; j < 3 ; j++) {
					Element constraint = new Element(CONSTRAINT);
					constraint.setAttribute(NAME, "C" + (nbConstraints));
					constraint.setAttribute(MODEL, Commands.TKC);
					constraint.setAttribute(ARITY, "2");
					constraint.setAttribute(SCOPE, "S" + (t + 1) + "." + i + " S" + (t + 1) + "." + j);
					constraint.setAttribute(REFERENCE, "R" + t);
					constraints.addContent(constraint);
					nbConstraints++;
				}
			}
		}
		for (int ti = 0 ; ti < this.targetsNeighbors.length - 1 ; ti++) {
			IntArray neig_i = this.targetsNeighbors[ti];
			for (int tj = ti + 1 ; tj < this.targetsNeighbors.length ; tj++) {
				IntArray neig_j = this.targetsNeighbors[tj];
				if (IntArray.intersection(neig_i, neig_j).size() > 0) {
					for (int i = 0 ; i < 3 ; i++) {
						for (int j = 0 ; j < 3 ; j++) {
							Element constraint = new Element(CONSTRAINT);
							constraint.setAttribute(NAME, "C" + (nbConstraints));
							constraint.setAttribute(MODEL, Commands.TKC);
							constraint.setAttribute(ARITY, "2");
							constraint.setAttribute(SCOPE, "S" + (ti + 1) + "." + i + " S" + (tj + 1) + "." + j);
							constraint.setAttribute(REFERENCE, "P0");
							Element parameters = new Element(PARAMETERS);
							parameters.setText("S" + (ti + 1) + "." + i + " S" + (tj + 1) + "." + j);
							constraint.addContent(parameters);
							constraints.addContent(constraint);
							nbConstraints++;
						}
					}
				}
			}
		}
		constraints.setAttribute(NB_CONSTRAINTS, "" + nbConstraints);
		return constraints;
	}
	
	/**
	 * 
	 * @param Pc
	 * @param Pv
	 * @param Kc
	 * @param Kv
	 */
	protected void generateOneProblem(int Pc, int Pv, int Kc, int Kv) {
		kc = Kc;
		kv = Kv;
		pc = Pc;
		pv = Pv;
		boolean uniform = false;
		int nbRun = 0;
		do {
//			System.out.println("nbRun : " + nbRun);
			this.generateTargets();
			this.generateSensors();
			this.generateCompatibilityGraph();
			uniform = this.generateVisibilityGraph();
			if (nbRun++ >= 10000) { 
				throw new Error("\nCan not generate uniform problem with this properties:\n" + 
						"nbSensors:" + this.nbSensors + " nbTargets" + this.nbTargets + 
						" Pc:" + Pc + " Pv:" + Pv + " Kc:" + Kc + " Kv:" + Kv); 
			}
		} while (!uniform);
		/*
		 * / this.generateTargets(); this.generateSensors(); this.generateCompatibilityGraph(); this.generateVisibilityGraph();
		 */
	}

	protected void plot() {
		for (UIManager.LookAndFeelInfo laf : UIManager.getInstalledLookAndFeels()) {
			if ("Mac OS X".equals(laf.getName())) {
				try {
					UIManager.setLookAndFeel(laf.getClassName());
				} catch (Exception e) {}
			} else {
				if ("Nimbus".equals(laf.getName())) {
					try {
						UIManager.setLookAndFeel(laf.getClassName());
					} catch (Exception e) {}
				}
			}
		}
		Toolkit kit = Toolkit.getDefaultToolkit();
		JFrame frame = new JFrame();
		//Insets insets = kit.getScreenInsets(frame.getGraphicsConfiguration());
		//frame.setSize(new Dimension(this.width + 2 * this.margin, this.height + 2 * this.margin + insets.top + insets.bottom));
		frame.setSize(new Dimension(this.width, this.height));
		frame.add(new GridSensor(frame.getSize()), BorderLayout.CENTER);
		Dimension dim = kit.getScreenSize();
		// Determine the new location of the window
		int w = frame.getSize().width;
		int h = frame.getSize().height;
		int x = (dim.width - w) / 2;
		int y = (dim.height - h) / 2;
		// Move the window
		frame.setLocation(x, y);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}
	/**
	 * @return
	 */
	private Element getGuiPresentation() {
		Element presentation = new Element(GuiPresentation);
		presentation.setAttribute(NAME, Commands.SensorNetworks);
		presentation.setAttribute(TYPE, DisCSP);
		presentation.setAttribute(BENCHMARK, this.typeOfBenchmark);
		presentation.setAttribute(MODEL, this.model);
		presentation.setAttribute(KC, "" + this.kc);
		presentation.setAttribute(KV, "" + this.kv);
		String pc = "" + ((this.pc < 10)? "0" + this.pc : this.pc);
		presentation.setAttribute(P_C, "" + pc);
		String pv = "" + ((this.pv < 10)? "0" + this.pv : this.pv);
		presentation.setAttribute(P_V, "" + pv);
		presentation.setAttribute(X_axis, "" + this.x_axis);
		presentation.setAttribute(Y_axis, "" + this.y_axis);
		Element sensors = new Element(Sensors);
		sensors.setAttribute(NB_SENSORS, ""+this.sensorsPlaces.length);
		for(Place s : this.sensorsPlaces){
			int id = s.getID();
			Element sensor = new Element(Sensor);
			sensor.setAttribute(NAME, s.getName());
			sensor.setAttribute(ID, ""+id);
			sensor.setAttribute(X_axis, ""+s.getX());
			sensor.setAttribute(Y_axis, ""+s.getY());
			sensor.setText(""+this.sensorsNeighbors[id]); //Neighbors
			sensors.addContent(sensor);
		}
		Element tagets = new Element(Targets);
		tagets.setAttribute(NB_TARGETS, ""+this.nbTargets);
		for(Place t : this.targetsPlaces){
			Element target = new Element(Target);
			int id = t.getID();
			target.setAttribute(NAME, t.getName());
			target.setAttribute(ID, ""+id);
			target.setAttribute(X_axis, ""+t.getX());
			target.setAttribute(Y_axis, ""+t.getY());
			target.setText(""+this.targetsNeighbors[id]); //Neighbors
			tagets.addContent(target);
		}
		presentation.addContent(sensors);
		presentation.addContent(tagets);
		return presentation;
	}

	class Target implements MouseListener {
		protected int id;
		protected Shape shape;
		protected String name;
		protected FontMetrics fontMetrics;
		protected int rad;
		protected int len;
		protected int width = 14;
		protected int height = 14;
		protected Component c;
		protected Color selectedColor = Color.red;
		protected Color selectedFillColor = Color.green;
		protected Color normalColor = Color.black;
		protected Color normalFillColor = Color.red;
		protected RenderingHints hints;
		protected IntArray neighbors = null;
		protected final float alpha = 0.15f;
		protected final Color zoneColor = new Color(0.5f, 0.5f, 0, alpha);
		protected int X;
		protected int Y;

		public Target(Component c, Place target, IntArray neighbors) {
			hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			hints.add(new RenderingHints(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY));
			X = (int) (unitX + margin + (target.getX() * unitX));
			Y = (int) (unitY + 3 * margin / 2 + (target.getY() * unitY));
			this.c = c;
			this.id = target.getID();
			this.name = target.getName();
			fontMetrics = c.getGraphics().getFontMetrics();
			rad = fontMetrics.getAscent();
			len = fontMetrics.stringWidth(name);
			shape = new Rectangle2D.Double(X - width / 2, Y - height / 2, width, height);
			c.addMouseListener(this);
			this.neighbors = neighbors;
		}

		public void paint(Graphics g) {
			paintNormal();
		}

		public void mouseClicked(MouseEvent e) {}

		public void mouseEntered(MouseEvent e) {}

		public void mouseExited(MouseEvent e) {}

		public void mousePressed(MouseEvent e) {
			if (shape.contains(e.getPoint())) {
				paintNeighbors();
				paintSelected();
				paintZone();
			}
		}

		public void mouseReleased(MouseEvent e) {
			c.repaint();
		}

		protected Graphics getGraphics() {
			return c.getGraphics();
		}

		/**
			 * 
			 */
		private void paintNormal() {
			Graphics2D gr = ((Graphics2D) getGraphics());
			gr.setRenderingHints(hints);
			gr.setStroke(new BasicStroke(2));
			gr.setColor(normalFillColor);
			gr.fill(shape);
			gr.setColor(normalColor);
			gr.draw(shape);
			gr.dispose();
		}

		/**
			 * 
			 */
		protected void paintSelected() {
			Graphics2D gr = (Graphics2D) getGraphics();
			gr.setRenderingHints(hints);
			gr.setStroke(new BasicStroke(2));
			gr.setColor(selectedFillColor);
			gr.fill(shape);
			int x = (int) (shape.getBounds2D().getCenterX() - len / 2);
			int y = (int) (shape.getBounds2D().getCenterY() - height - (rad - fontMetrics.getDescent()) / 2);
			Shape zone = new Rectangle2D.Double(x, y - rad, len, (rad - fontMetrics.getDescent()) / 2 + height);
			gr.setColor(Color.white);
			gr.fill(zone);
			gr.setColor(selectedColor);
			gr.drawString(name, x, y);
			gr.draw(shape);
			gr.dispose();
		}

		/**
			 * 
			 */
		protected void paintNeighbors() {
			for (int neig : neighbors) {
				mySensors.get(neig).selected();
			}
		}

		/**
			 * 
			 */
		protected void selected() {
			Graphics2D gr = (Graphics2D) getGraphics();
			gr.setRenderingHints(hints);
			gr.setStroke(new BasicStroke(2));
			gr.setColor(selectedFillColor);
			gr.fill(shape);
			gr.setColor(selectedColor);
			gr.draw(shape);
			gr.dispose();
		}
		protected void paintZone() {
			int w = (int) ((2 * (kv - 1) + 1) * unitX);
			int h = (int) ((2 * (kv - 1) + 1) * unitY);
			int xx = (int) ((shape.getBounds2D().getCenterX() - margin) / unitX);
			int yy = (int) ((shape.getBounds2D().getCenterY() - 3 * margin / 2) / unitY);
			int x = (int) (xx * unitX - width + margin - ((w - unitX) / 2));
			int y = (int) (yy * unitY - height + 3 * margin / 2 - ((h - unitY) / 2));
			Graphics2D gr = (Graphics2D) getGraphics();
			Composite composite = gr.getComposite();
			gr.setColor(Color.yellow);
			gr.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
			Ellipse2D zone = new Ellipse2D.Double(x, y, w + 2 * width, h + 2 * height);
			gr.fill(zone);
			gr.setComposite(composite);
		}
	}
	
	class Sensor extends Target {
		
		public Sensor(Component c, Place sensor, IntArray neighbors) {
			super(c, sensor, neighbors);
			this.selectedColor = Color.blue;
			this.selectedFillColor = Color.yellow;
			this.normalColor = Color.black;
			this.normalFillColor = Color.blue;
			shape = new Ellipse2D.Double(X - width / 2, Y - height / 2, width, height);
		}
		protected void paintZone() {
			int x = (int) (shape.getBounds2D().getCenterX());
			int y = (int) (shape.getBounds2D().getCenterY());
			int xx = (int) (x - kc * unitX - width);
			int lengthX = (int) (x + kc * unitX + width) - xx;
			int yy = (int) (y - kc * unitY - height);
			int lengthY = (int) (y + kc * unitY + height) - yy;
			Graphics2D gr = (Graphics2D) getGraphics();
			gr.setPaint(Color.yellow);
			gr.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, Math.min(alpha,1.0f ) ));
			Ellipse2D zone = new Ellipse2D.Double(xx, yy, lengthX, lengthY);
			gr.fill(zone);
			gr.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
		}
	}

	public void maximize(JFrame frame) {
		Toolkit kit = Toolkit.getDefaultToolkit();
		Insets insets = kit.getScreenInsets(frame.getGraphicsConfiguration());
		Dimension screen = kit.getScreenSize();
		int w = (int) (screen.getWidth() - insets.left - insets.right);
		int h = (int) (screen.getHeight() - insets.top - insets.bottom);
		int x = (int) (insets.left);
		int y = (int) (insets.top);
		Dimension dimension = new Dimension(w, h);
		frame.setSize(dimension);
		frame.setLocation(x, y);
	}

	/**
	 * 
	 * @param combination
	 * @param n
	 * @return
	 */
	public boolean nextCombination(int[] combination, int n) {
		int i = combination.length - 1;
		++combination[i];
		while ((i > 0) && (combination[i] >= n - combination.length + 1 + i)) {
			--i;
			++combination[i];
		}
		 /* Combination (n-k, n-k+1,..., n) reached */
		if (combination[0] > n - combination.length)
			return false; /* No more combinations can be generated */
		/*
		 * comb now looks like (...,x,n,n,n,...,n). Turn it into (...,x,x +1, x + 2, ...)
		 */
		for (i = i + 1 ; i < combination.length ; ++i)
			combination[i] = combination[i - 1] + 1;
		return true;
	}

	/**
	 * 
	 * @param n
	 * @param k
	 * @return
	 */
	public ArrayList<int[]> getAllCombinations(int n, int k) {
		ArrayList<int[]> list = new ArrayList<int[]>();
		int combination[] = new int[k];
		for (int i = 0 ; i < k ; ++i) {
			combination[i] = i;
		}
		do {
			list.add(combination.clone());
		} while (nextCombination(combination, n));
		return list;
	}

	/**
	 * 
	 * @param n
	 * @param k
	 * @return
	 */
	public ArrayList<ArrayList<Integer>> getAllCombinations(IntArray values, int k) {
		ArrayList<ArrayList<Integer>> list = new ArrayList<ArrayList<Integer>>();
		int combination[] = new int[k];
		for (int i = 0 ; i < k ; ++i) {
			combination[i] = i;
		}
		do {
			ArrayList<Integer> array = new ArrayList<Integer>();
			for (int i = 0 ; i < k ; ++i) {
				array.add(i, values.get(combination[i]));
			}
			list.add(array);
		} while (nextCombination(combination, values.size()));
		return list;
	}

	protected void toSimpleModel() {// TODO Auto-generated method stub
		// TODO to check
		System.out.println("***************************************************");
		HashMap<Integer, IntArray> domains = new HashMap<Integer, IntArray>();
		// HashMap<Integer, String> constraints = new HashMap<Integer,
		// String>();
		IntArray sensors = new IntArray();
		for (int s = 0 ; s < this.nbSensors ; s++) {
			sensors.add(s, s);
		}
		System.out.print("Domains : \n[\n");
		if (sensors.size() < 3) { return; }
		ArrayList<ArrayList<Integer>> sensorCombinations = this.getAllCombinations(sensors, 3);
		for (int t = 0 ; t < this.nbTargets ; t++) {
			IntArray domain = new IntArray();
			domains.put(t, domain);
			IntArray neighbors = this.targetsNeighbors[t];
			ArrayList<ArrayList<Integer>> targetCombinations = null;
			if (neighbors.size() >= 3) {
				targetCombinations = this.getAllCombinations(neighbors, 3);
				for (int i = 0 ; i < targetCombinations.size() ; i++) {
					ArrayList<Integer> combination = targetCombinations.get(i);
					int index = sensorCombinations.indexOf(combination);
					domain.add(index);
				}
			}
			System.out.print(t + " : " + domain.size() + " : [");
			for (int v : domain) {
				// System.out.print(v+"{"+sensorCombinations.get(v)+"} ");
				System.out.print(v + " ");
			}
			System.out.println("]");
		}
		System.out.print("Constraints : \n[\n");
		HashMap<Couple, ArrayList<Couple>> constraints = new HashMap<Couple, ArrayList<Couple>>();
		for (int t1 = 0 ; t1 < this.nbTargets ; t1++) {
			IntArray neighbors1 = this.targetsNeighbors[t1];
			// IntArray domain1 = domains.get(t1);
			for (int t2 = t1 + 1 ; t2 < this.nbTargets ; t2++) {
				Couple c = new Couple(t1, t2);
				ArrayList<Couple> tuples = new ArrayList<Couple>();
				constraints.put(c, tuples);
				System.out.print("(" + t1 + "," + t2 + "): ");
				// IntArray domain2 = domains.get(t2);
				IntArray neighbors2 = this.targetsNeighbors[t2];
				for (int s1 : neighbors1) {
					IntArray values1 = new IntArray();
					IntArray values2 = new IntArray();
					for (int v1 : domains.get(t1)) {
						if (sensorCombinations.get(v1).contains(s1)) {
							values1.add(v1);
						}
					}
					IntArray N1 = this.sensorsNeighbors[s1];
					IntArray intersection = IntArray.intersection(N1, neighbors2);
					for (int s2 : intersection) {
						for (int v2 : domains.get(t2)) {
							if (sensorCombinations.get(v2).contains(s2)) {
								values2.add(v2);
							}
						}
						// System.out.print("|"+s1+","+s2+"| ");
					}
					for (int v1 : values1) {
						for (int v2 : values2) {
							Couple tuple = new Couple(v1, v2);
							if (!tuples.contains(tuple)) {
								tuples.add(tuple);
							}
						}
					}
				}
				System.out.print(tuples.size() + " : [");
				for (Couple tuple : tuples) {
					System.out.print(tuple + "|");
				}
				System.out.print("]\n");
			}
		}
		System.out.println("]");
	}

	protected void plotTargetNeighbors() {
		System.out.println("***************************************************");
		for (int t = 0 ; t < this.nbTargets ; t++) {
			System.out.println("target : " + t + " = " + this.targetsNeighbors[t]);
		}
	}

	protected void plotSensorNeighbors() {
		System.out.println("***************************************************");
		for (int s = 0 ; s < this.nbSensors ; s++) {
			System.out.println("sensor : " + s + " = " + this.sensorsNeighbors[s]);
		}
	}

	public class Couple { // couple
		int x;
		int y;

		public Couple(int x, int y) {
			this.x = x;
			this.y = y;
		}

		public boolean equals(Object obj) {
			Couple c = (Couple) obj;
			if (this.x == c.x && this.y == c.y) return true;
			return (this.x == c.y && this.y == c.x);
		}

		public String toString() {
			return this.x + " " + this.y;
		}
	}
	
	@Override
	public void run() {
		int nbInstance, minPv, maxPv, stepPv;
		ArrayList<String> keys = null;
		String pc;
		String filePath = this.experimentation.getParent();
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(this.experimentation);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Commands.Communications).getChildren();
		List<Element> algorithms = xp.getChild(Commands.Algorithms).getChildren();
		String type = instances.getAttributeValue(TYPE);
		String[] protocols = new String[algorithms.size()];
		String[] orderings = new String[algorithms.size()];
		int a = 0;
		Iterator<Element> it = algorithms.iterator();
		while (it.hasNext()) {
			Element algorithm = it.next();
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(Commands.AO);
			a++;
		}
		it = communications.iterator();
		int nbSensors = new Integer(instances.getAttributeValue(NB_SENSORS));
		int nbTargets = new Integer(instances.getAttributeValue(NB_TARGETS));
		int x_axis = new Integer(instances.getAttributeValue(X_axis));
		int y_axis = new Integer(instances.getAttributeValue(Y_axis));
		String name = instances.getAttribute(NAME).getValue();
		it = communications.iterator();
		while (it.hasNext()) {
			Element comm = it.next();
			String commType = comm.getAttributeValue(TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			long maxNCCCs = new Long(comm.getAttributeValue(Commands.MaxNCCCs));
			List<Element> dir = instances.getChildren();
			for (int d = 0 ; d < dir.size() ; d++) {
				Element KcKv = dir.get(d);
				int Kc = new Integer(KcKv.getAttributeValue(KC));
				int Kv = new Integer(KcKv.getAttributeValue(KV));
				String KcKvName = KcKv.getAttributeValue(NAME);
				List<Element> cDensities = KcKv.getChildren();
				for (int compatibility = 0 ; compatibility < cDensities.size() ; compatibility++) { // compatibility density
					Element PC = cDensities.get(compatibility);
					Element visibilityDensity = PC.getChild(P_V);
					minPv = new Integer(visibilityDensity.getAttributeValue(MIN_PV));
					maxPv = new Integer(visibilityDensity.getAttributeValue(MAX_PV));
					stepPv = new Integer(visibilityDensity.getAttributeValue(STEP_PV));
					nbInstance = new Integer(visibilityDensity.getAttributeValue(NB_INSTANCES));
					String PCName = PC.getAttributeValue(NAME);
					pc = PC.getAttributeValue(P_C);
					String directory = filePath + File.separator + Commands.Results+"_"+nbSensors+"_"+nbTargets+"_"+nbInstance;
					try {
						File f = new File(directory);
						if (!f.exists()) { f.mkdir(); }
					} catch (Exception e) {}
					directory += File.separator + commType + "_" + channelType;
					try {
						File f = new File(directory);
						if (!f.exists()) { f.mkdir(); }
					} catch (Exception e) {}
					File folder = new File(directory + File.separator + KcKvName);
					try { if (!folder.exists()) { folder.mkdir(); }
					} catch (Exception e) {}
					String subFolder = folder + File.separator + PCName;
					String problemsFolder = filePath + File.separator + KcKvName + File.separator + PCName;
					try {
						File f = new File(subFolder);
						if (!f.exists()) {
							f.mkdir();
						}
					} catch (Exception e) {}
					for (int p = 0 ; p < protocols.length ; p++) { // Protocols
						Logger allResuls = new Logger(subFolder, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
						Logger results = new Logger(subFolder, protocols[p] + "-" + orderings[p] + ".csv");
						for (int pv = minPv ; pv <= maxPv ; pv += stepPv) {
							String Pv = (pv < 10)? "0" + pv : "" + pv;
							HashMap<String, Double> densityStatistics = new HashMap<String, Double>();
							for (int i = 1 ; i <= nbInstance ; i++) {
								String problem = problemsFolder + File.separator + Pv + File.separator + name + i + ".xml.bz2";
								HashMap<String, Double> instanceStatistics = new HashMap<String, Double>();
								for (int r = 1 ; r <= nbRun ; r++) {
									DisProblem disProb = new DisProblem(problem, protocols[p]);
									DisSolverSimulator solver = new DisSolverSimulator(disProb);
									solver.setCutOffNCCCs(maxNCCCs);
									solver.addCommunicationMetric();
									solver.setMessageDelay(minURC, maxURC);
									// solver.CheckTheReturnedSolution(true);
									solver.setCentralizedAO(AgentOrdering.getCentralizedAO(orderings[p]));
									solver.solve();
									if(keys == null ){
										keys = disProb.master.getStatisticsKeys();
									}
									HashMap<String, Double> runStatistics = solver.getStatistics();
									if (i == 1 && r == 1 && pv == minPv) {
										String s = ",";
										String header = type + s;
										header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
										header += P_C + s + "0."+pc + s + NB_SENSORS + s + nbSensors + s + NB_TARGETS + s + nbTargets + s;
										header += X_axis + s + x_axis + s + Y_axis + s + y_axis + s + KC + s + Kc + s + KV + s + Kv +s;
										header += NB_INSTANCES+s+nbInstance + s + Commands.NbRun+s+nbRun+"\n";
										header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s;
										header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
										header += P_V + s + INSTANCE + s + Commands.NbRun + s + Tools.toString(keys);
										allResuls.printToLogln(header);
										header = type + s;
										header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
										header += P_C + s + "0."+pc + s + NB_SENSORS + s + nbSensors + s + NB_TARGETS + s + nbTargets + s;
										header += X_axis + s + x_axis + s + Y_axis + s + y_axis + s + KC + s + Kc + s + KV + s + Kv +s;
										header += NB_INSTANCES+s+nbInstance + s + Commands.NbRun+s+nbRun+"\n";
										header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s;
										header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
										header += P_V + s + Tools.toString(keys);
										results.printToLogln(header);
									}
									allResuls.printToLogln("0."+Pv + "," + i + "," + r + "," + Tools.statisticsToString(keys, runStatistics));
									instanceStatistics = Tools.merge(runStatistics, instanceStatistics);
									disProb.setAgents(null);
									System.gc();
								}
								Tools.average(instanceStatistics, nbRun);
								densityStatistics = Tools.merge(instanceStatistics, densityStatistics);
							}
							Tools.average(densityStatistics, nbInstance);
							results.printToLogln("0."+Pv + "," + Tools.statisticsToString(keys, densityStatistics));
						}
						results.close();
						allResuls.close();
						keys = null;
					}
					String pathName = folder + File.separator + PC.getAttributeValue(NAME);
					ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
					printer.setFixKeys(fixKeys);
					ArrayList<String> paths = printer.printToGnuPlot(pathName, "", commType);
					printer.plot(pathName, paths);
					String s = ",";
					String header = type + s + NB_SENSORS + s + nbSensors + s + NB_TARGETS + s + nbTargets + s;
					header += "\n#";
					header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s;
					header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + s + P_C + s + pc;
					printer.printAllResults(pathName, header);
				}// compatibility density
			}
		}
	}
	
	class GridSensor extends JPanel implements MouseWheelListener, MouseListener, MouseMotionListener {
		
		private static final long serialVersionUID = 5352838628433546561L;
		public static final double DEFAULT_ZOOM_PERCENTAGE = 1.05;
		private Dimension preferredSize = null;
		private Dimension minimumSize = null;
		private Dimension maximumSize = null; 
		public HashMap<Integer, Sensor> mySensors = null;
		public HashMap<Integer, Target> myTargets = null;
		private Target pick;
		private boolean pickfixed;
		private Rectangle rect;
		
		public GridSensor(Dimension size){
			super();
			minimumSize = new Dimension(size.width-margin/2, size.height-margin);
			preferredSize = minimumSize;
			maximumSize = new Dimension(preferredSize.width*2, preferredSize.height*2);
			mySensors = new HashMap<Integer, Sensor>();
			myTargets = new HashMap<Integer, Target>();
			setBackground(Color.white);
			addMouseWheelListener(this);
			addMouseListener(this);
			setSize(preferredSize);
			createShapes();
		}
		
		public void mouseClicked(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		
		public void mousePressed(MouseEvent e) {
			Point p = e.getPoint();
			addMouseMotionListener(this);
			for(int s : mySensors.keySet()){
				Sensor sensor =  mySensors.get(s);
				if(sensor.contains(p)){
					pick = sensor;
					break;
				}
			}
			for(int t : myTargets.keySet()){
				Target target = myTargets.get(t);
				if(target.contains(p)){
					pick = target;
					break;
				}
			}
			if (pick != null) {
				pickfixed = pick.selected;
				pick.selected = true;
			}
			repaint();
			e.consume();
		}

		public void mouseReleased(MouseEvent e) {
			removeMouseMotionListener(this);
			if (pick != null) {
				pick.selected = pickfixed;
				pick = null;
			}
			repaint();
			e.consume();
		}
		public void mouseDragged(MouseEvent e) {
			if (pick != null) {
//				pick.setCenter(e.getPoint());
			}
			repaint();
			e.consume();
		}
		public void mouseMoved(MouseEvent e) {}
		
		public void mouseWheelMoved(MouseWheelEvent e) {
			updatePreferredSize(e.getWheelRotation(), e.getPoint());
		}
		
		public Dimension getPreferredSize() {
			return preferredSize;
		}

		private void updatePreferredSize(int n, Point p) {
			double d = (double) n * DEFAULT_ZOOM_PERCENTAGE;
			d = (n > 0)? (1 / d) : (-d);
			int w = (int) (getPreferredSize().getWidth() * d);
			int h = (int) (getPreferredSize().getHeight() * d);
			if(w < minimumSize.width){
				preferredSize = minimumSize;
			} else if(w > maximumSize.width){
				preferredSize = maximumSize;
			} else{
				preferredSize = new Dimension(w, h);
			}
			createShapes();
			getParent().doLayout();
			revalidate();
			repaint();
		}

		public void paintComponent(Graphics g) {
			createShapes();
			Graphics2D gr = (Graphics2D) g;
	        gr.setRenderingHints(hints);  
	        gr.setColor(Color.white);
			gr.fillRect(0, 0, getWidth(), getHeight());
			gr.setColor(Color.black);
			if(layout.equals(GRID)) {
				drawGrid(gr);
			}
	        gr.setStroke(new BasicStroke(lineWidth));
			gr.draw(rect);
			Font font = gr.getFont();
			font = new Font(font.getName(), Font.BOLD, font.getSize()+1);
			gr.setFont(font);
			FontMetrics fontMetric = gr.getFontMetrics();
			String desc = NB_SENSORS+"="+nbSensors+",  "+NB_TARGETS+"="+nbTargets+",  ";
			desc += KC+"="+kc+",  "+KV+"="+kv+",  "+P_C+"="+pc+",  "+P_V+"="+pv+",  ";
			desc += X_axis+"="+x_axis+",  "+Y_axis+"="+y_axis;
			gr.drawString(desc, (getWidth()-fontMetric.stringWidth(desc))/2, margin - lineWidth);
			for(int id : myTargets.keySet()){
				myTargets.get(id).paint(gr);
			}
			for(int id : mySensors.keySet()){
		        mySensors.get(id).paint(gr);
			}
			if(pick!=null){
				pick.paint(gr);
			}
			gr.dispose();
		}

		private void createShapes() {
			int m = margin - lineWidth;
			rect = new Rectangle(m, m + m / 2, getWidth() - 2 * m, getHeight() - 2 * m);
			unitX = (rect.width - 2*lineWidth) / (x_axis+1);
			unitY = (rect.height - 2*lineWidth) / (y_axis+1);
			mySensors = new HashMap<Integer, Sensor>();
			myTargets = new HashMap<Integer, Target>();
			for (Place sensor : sensorsPlaces) {
				int id = sensor.getID();
				double x = (sensor.getX() * unitX);
				double y = (sensor.getY() * unitY);
				String name = sensor.getName();
				double xx = x + rect.x + lineWidth + unitX ;
				double yy = y + rect.y + lineWidth + unitY ;
				mySensors.put(id, new Sensor(id, name, xx, yy, sensorsNeighbors[id]));
			}
			for (Place target : targetsPlaces) {
				int id = target.getID();
				double x = unitX + margin + (target.getX() * unitX);
				double y = unitY + 3*margin/2 + (target.getY() * unitY);
				String name = target.getName();
				myTargets.put(id, new Target(id, name, x, y, targetsNeighbors[id]));
			}
		}
		
		/**
		 * @param gr
		 */
		private void drawGrid(Graphics2D gr) {
			//========================================================================
	        gr.setStroke(new BasicStroke(0.5f));
			int w = rect.width - 2*lineWidth;
			int h = rect.height- 2*lineWidth;
			int unitX = h / (x_axis+1);
			int unitY = w / (y_axis+1);
			int x = rect.x + lineWidth ;
			int y = rect.y + lineWidth ;
			for (int r = 1 ; r <= x_axis ; r++){
				gr.drawLine(x, y+r* unitX, x+w, y+r * unitX);
			}
			for (int r = 1 ; r <= y_axis ; r++){
				gr.drawLine(x+r * unitY, y, x+r * unitY, y+h);
			}
			//========================================================================
		}

		class Target {
			protected int id;
			protected Shape shape;
			protected String name;
			protected FontMetrics fontMetrics;
			protected int rad;
			protected int len;
			protected int width = 14;
			protected int height = 14;
			protected Color selectedColor = Color.black;
			protected Color selectedFillNeigh = Color.green;
			protected Color selectedFillColor = Color.orange;
			protected Color normalColor = Color.black;
			protected Color normalFillColor = Color.red;
			protected RenderingHints hints;
			protected IntArray neighbors = null;
			protected final float alpha = 0.2f;
			protected boolean selected ;

			protected Composite composite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
			protected Composite defaultComposite;
			
			public Target(int id, String name, double x, double y, IntArray neighbors) {
				hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				hints.add(new RenderingHints( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY));
				this.id = id;
				this.name = name;
				shape = new Rectangle2D.Double(x-width/2, y-height/2, width, height);
				this.neighbors = neighbors;
			}

			public void paint(Graphics g) {
				Graphics2D gr = ((Graphics2D) getGraphics());
				defaultComposite = gr.getComposite();
				fontMetrics = gr.getFontMetrics();
				rad = fontMetrics.getAscent();
				len = fontMetrics.stringWidth(name);
				if(selected){
					paintZone(gr);
					paintNeighbors(gr);
					paintSelected(gr);
				} else {
					paintNormal(gr);
				}
				gr.dispose();
			}

			/**
			 * 
			 */
			private void paintNormal(Graphics2D gr) {
				gr.setRenderingHints(hints);
				gr.setStroke(new BasicStroke(1.8f));
				gr.setColor(normalFillColor);
				gr.fill(shape);
				gr.setColor(normalColor);
				gr.draw(shape);
			}
			/**
			 * 
			 */
			protected void paintSelected(Graphics2D gr) {
				gr.setRenderingHints(hints);
		        gr.setStroke(new BasicStroke(1.8f));
				gr.setColor(selectedFillColor);
				gr.fill(shape);
				int x = (int) (shape.getBounds2D().getCenterX() - len / 2);
				int y = (int) (shape.getBounds2D().getCenterY() - height - (rad - fontMetrics.getDescent()) / 2);
				Shape zone = new Rectangle2D.Double(x, y-rad, len, (rad - fontMetrics.getDescent())/2+height);
				gr.setColor(Color.white);
				gr.fill(zone);
				gr.setColor(Color.black);
		        gr.setStroke(new BasicStroke(1.8f));
				gr.drawString(name, x, y);
				gr.draw(shape);
			}
			/**
			 * 
			 */
			protected void paintNeighbors(Graphics2D gr) {
				for(int neig : neighbors){
					mySensors.get(neig).selected(gr);
				}
			}

			/**
			 * 
			 */
			protected void selected(Graphics2D gr) {
				gr.setRenderingHints(hints);
		        gr.setStroke(new BasicStroke(1.8f));
				gr.setColor(selectedFillNeigh);
				gr.fill(shape);
				gr.setColor(selectedColor);
				gr.draw(shape);
			}
			/**
			 * 
			 */
			protected void paintZone(Graphics2D gr) {
				int w, h, x, y;
				Shape zone = null;
				if(layout.equals(REAL)){
					w = (int) (2 * kv*unitX + width);
					h = (int) (2 * kv*unitY + height);
					x = (int) (shape.getBounds2D().getCenterX()) - w/2;
					y = (int) (shape.getBounds2D().getCenterY()) - h/2;
					zone = new Ellipse2D.Double(x, y, w, h);
				} else { //Real
//					w = (int) ((2*(kv - 1) + 1) * unitX + width);
//					h = (int) ((2*(kv - 1) + 1) * unitY + height);
//					int xx = (int) ((int)(shape.getBounds2D().getCenterX() - rect.x) / unitX);
//					int yy = (int) ((int)(shape.getBounds2D().getCenterY() - rect.y) / unitY);
//					x = rect.x + (int) (xx * unitX + unitX/2) - w/2;
//					y = rect.y + (int) (yy * unitY + unitY/2) - h/2;
//					zone = new Rectangle2D.Double(x, y, w, h);
					w = (int) ((2 * (kv - 1) + 1) * unitX)+ width;
					h = (int) ((2 * (kv - 1) + 1) * unitY)+ height;
					int xx = (int) ((shape.getBounds2D().getCenterX() - margin) / unitX);
					int yy = (int) ((shape.getBounds2D().getCenterY() - 3 * margin / 2) / unitY);
					x = (int) (xx * unitX + margin - ((w - unitX) / 2));
					y = (int) (yy * unitY + 3 * margin / 2 - ((h - unitY) / 2));
					zone = new Rectangle2D.Double(x, y, w, h);
				}
		        gr.setRenderingHints(hints);  
				gr.setComposite(composite);
		        gr.setColor(Color.yellow);
		        gr.fill(zone);
		        gr.setColor(Color.black);
		        gr.setStroke(new BasicStroke(1f));
		        gr.draw(zone);
		        gr.setComposite(defaultComposite);
			}
			/**
			 * @param p
			 * @return
			 */
			public boolean contains(Point p) {
				return shape.contains(p);
			}
		}


		class Sensor extends Target {

			public Sensor(int id, String name, double x, double y, IntArray neighbors) {
				super(id, name, x, y, neighbors);
				this.selectedColor = Color.blue;
				this.selectedFillNeigh = Color.cyan;
				this.normalColor = Color.black;
				this.normalFillColor = Color.gray;
				shape = new Ellipse2D.Double(x-width/2, y-height/2, width, height);
			}
			/**
			 * 
			 */
			protected void paintZone(Graphics2D gr) {
				int w = (int) (2 * kc*unitX + width);
				int h = (int) (2 * kc*unitY + height);
				int x = (int) (shape.getBounds2D().getCenterX()) - w/2;
				int y = (int) (shape.getBounds2D().getCenterY()) - h/2;
		        gr.setRenderingHints(hints);
		        gr.setColor(Color.yellow);
				gr.setComposite(composite);
				Shape zone = null;
				if(layout.equals(REAL)){
					zone = new Ellipse2D.Double(x, y, w, h);
				} else{
					zone = new Rectangle2D.Double(x, y, w, h);
				}
		        gr.fill(zone);
		        gr.setColor(Color.black);
		        gr.setStroke(new BasicStroke(1f));
		        gr.draw(zone);
		        gr.setComposite(defaultComposite);
			}
		}
	}
	
}
