/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package dischoco;

import java.util.SortedMap;
import java.util.TreeMap;
import kernel.AbstractMaster;
import simple.dcop.adopt.MasterAdopt;
import simple.dcop.bnbadopt.MasterBnBAdopt;
import simple.dcop.synchbb.SBBMaster;
import simple.dcsp.abtfamily.abt.ABTMaster;
import simple.dcsp.abtfamily.abt.abtac.ABTacMaster;
import simple.dcsp.abtfamily.abtdac.ABTdacMaster;
import simple.dcsp.abtfamily.abtdo.ABTDOMaster;
import simple.dcsp.abtfamily.abtdoretro.ABTDORetroMaster;
import simple.dcsp.abtfamily.abtdoretroz.ZABTDORetroMaster;
import simple.dcsp.abtfamily.abthyb.ABTHybMaster;
import simple.dcsp.afcfamily.afc.AFCMaster;
import simple.dcsp.afcfamily.afc.domdeg.AFCDomDegMaster;
import simple.dcsp.afcfamily.afc.domdeg.AFCMinDomMaster;
import simple.dcsp.afcfamily.afc.estimDom.AFCEstimatedMaster;
import simple.dcsp.afcfamily.afc.triggerednogood.AFCTriggeredNogoodMaster;
import simple.dcsp.afcfamily.afc_dbj.AFCDBJMaster;
import simple.dcsp.afcfamily.afcng.AFCngMaster;
import simple.dcsp.afcfamily.afcng.afctree.AFCTreeMaster;
import simple.dcsp.afcfamily.afcng.approximateDom.AFCngApproximateMinDomMaster;
import simple.dcsp.afcfamily.afcng.domDeg.AFCngDomDegMaster;
import simple.dcsp.afcfamily.afcng.domDeg.AFCngMinDomMaster;
import simple.dcsp.afcfamily.afcng.triggerednogood.AFCngTriggeredNogoodMaster;
import simple.dcsp.afcfamily.ddbj.DDBJMaster;
import simple.dcsp.agile.exp.AgileMaster;
import simple.dcsp.agileabt.explanation.AgileABTMaster;
import simple.dcsp.dba.DBAmaster;
import simple.dcsp.sbt.SBTMaster;

/**
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@users.sourceforge.net &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@users.sourceforge.net?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public final class Protocols {

	public static final String SBT = "SBT";
	/** ABT Bessiere et al. 2005 */
	public static final String ABT = "ABT";
	/**  */
	public static final String ABTHyb = "ABT-hyb";
	/**  */
	public static final String ABTdac = "ABTdac";
	/**  */
	public static final String ABTac = "ABTac";

	/** ABT with Dynamic Ordering Family Algorithms */
	public static final String ABTDOng = "ABTDO-ng";
	/**  */
	public static final String ABTDOMinDom = "ABTDOMinDom";
	/**  */
	public static final String ABTDORandom = "ABTDORandom";
	/**  */
	public static final String ABTDOMinDomS = "ABTDOMinDom-S";
	/**  */
	public static final String ABTDORandomS = "ABTDORandom-S";

	/** MinDomain retroactive Heuristics for ABT */
	public static final String ABTDORetro = "ABTDORetro";
	/** Minimum Domain */
	public static final String ABTDORetroMinDom = "ABTDORetroMinDom";
	/** After Second Last */
	public static final String ABTDORetroA2ndL = "ABTDORetroA2ndL";
	/** Domain Position Single */
	public static final String ABTDORetroDomPS = "ABTDORetroDomPS";
	/** Domain Position All */
	public static final String ABTDORetroDomPAll = "ABTDORetroDomPAll";
	/** Highest */
	public static final String ABTDORetroHighest = "ABTDORetroHighest";

	public static final String AFC = "AFC";
	public static final String AFCTriggeredng = "AFCTriggeredng";
	public static final String AFCMinDom = "AFCMinDom";
	public static final String AFCDomDeg = "AFCDomDeg";
	public static final String AFCEstMinDom = "AFCEstMinDom";
	public static final String AFCEstDomDeg = "AFCEstDomDeg";

	/** Dynamic Distributed BackJumping: Viet Nguyen, Djamila Sam-Haroud, and Boi Faltings*/
	public static final String AFCdbj = "AFC-dbj";
	public static final String DDBJ = "DDBJ";

	public static final String AFCng = "AFC-ng";
	public static final String AFCngTriggeredng = "AFC-ng-TriggeredNogood";
	public static final String AFCngMinDom = "AFC-ng-MinDom";
	public static final String AFCngDomDeg = "AFC-ng-DomDeg";
	public static final String AFCngMaxDeg = "AFC-ng-MaxDeg";
	public static final String AFCngApproximatedMinDom = "AFC-ng-ApproximatedMinDom";
	public static final String AFCngApproximatedDomDeg = "AFC-ng-ApproximatedDomDeg";

	public static final String AFCtree = "AFC-tree";
	
	// ====================================================================

	public static final String DisFC = "DisFC";
	public static final String DisFC1ph = "DisFC1ph";
	public static final String DisFC2ph = "DisFC2ph";
	public static final String DisFClies = "DisFClies";

	public static final String SynchBB = "SBB";
	public static final String OriginalAdopt = "Adopt-original";
	public static final String Adopt = "Adopt";
	public static final String BnBAdopt = "BnBAdopt";
	public static final String AdoptPlus = "Adopt+";
	public static final String BnBAdoptPlus = "BnBAdopt+";
	public static final String BnBAdoptPlusDP2 = "BnBAdopt+DP2";

	public static final String AFB = "AFB";
	public static final String AFB_minC = "AFB-minC";

	public static final String AFB_BJ = "AFB-BJ";
	
	public static final String ConcFB = "ConcFB";

	public static final String DBA = "DBA";

	public static final String ABTcf = "ABTcf";

	public static final String SDisCSP = "SDisCSP";
	public static final String SDCOP = "SDCOP";
	public static final String CDisCSP = "CDisCSP";
	public static final String CDCOP = "CDCOP";

	/** Agile ABT Family : best heuristic sending only in */
	public static final String AgileMinDom = "AgileABTMinDom";
	public static final String AgileDomDeg = "AgileABTDomDeg";
	public static final String Agile = AgileMinDom;
	public static final String Agile_ALL = "AgileABT-All";
	public static final String Agile_O = "AgileABT-O";
	public static final String Agile_N = "AgileABT-N";
	public static final String Agile_ON = "AgileABT-ON";
	public static final String AgileMinDom_ID = "AgileABTMinDomTopoID";
	public static final String AgileMinDom_Deg = "AgileABTMinDomTopoDeg";
	public static final String AgileMinDom_MaxSucc = "AgileABTMinDomTopoMaxSucc";
	public static final String AgileDomDeg_ID = "AgileABTDomDegTopoID";
	public static final String AgileDomDeg_Deg = "AgileABTDomDegTopoDeg";
	public static final String AgileDomDeg_MaxSucc = "AgileABTDomDegTopoMaxSucc";

	public static final String Agiledom = "AgileABTdom";
	public static final String Agiledom_ID = "AgileABTdom+id";
	public static final String Agiledom_Deg = "AgileABTdom+deg";
	public static final String Agiledom_MaxSucc = "AgileABTdom+maxSucc";

	public static final String Agiledomdeg = "AgileABTdomdeg";
	public static final String Agiledomdeg_ID = "AgileABTdomdeg+id";
	public static final String Agiledomdeg_Deg = "AgileABTdomdeg+deg";
	public static final String Agiledomdeg_MaxSucc = "AgileABTdomdeg+maxSucc";

	public static final String Agiledomwdeg = "AgileABTdomwdeg";
	public static final String Agiledomwdeg_ID = "AgileABTdomwdeg+id";
	public static final String Agiledomwdeg_Deg = "AgileABTdomwdeg+deg";
	public static final String Agiledomwdeg_MaxSucc = "AgileABTdomwdeg+maxSucc";

	/** MinDomain Retroactive Heuristics for ABT **/
	public static final String ZABTDORetro = "ZABTDORetro";
	/** Minimum Domain */
	public static final String ZABTDORetroMinDom = "ZABTDORetroMinDom";
	/** After Second Last */
	public static final String ZABTDORetroA2ndL = "ZABTDORetroA2ndL";
	/** Domain Position Single */
	public static final String ZABTDORetroDomPS = "ZABTDORetroDomPS";
	/** Domain Position All */
	public static final String ZABTDORetroDomPAll = "ZABTDORetroDomPAll";
	/** Highest */
	public static final String ZABTDORetroHighest = "ZABTDORetroHighest";

	private static final SortedMap<String, Integer> protocols = new TreeMap<String, Integer>();
	private static final SortedMap<String, Integer> simpleDisCSPProtocols = new TreeMap<String, Integer>();
	private static final SortedMap<String, Integer> complexDisCSPProtocols = new TreeMap<String, Integer>();
	private static final SortedMap<String, Integer> simpleDCOPProtocols = new TreeMap<String, Integer>();
	private static final SortedMap<String, Integer> complexDCOPProtocols = new TreeMap<String, Integer>();	

	static {
		simpleDisCSPProtocols.put(SBT, 0);
		simpleDisCSPProtocols.put(ABT, 1);
		simpleDisCSPProtocols.put(ABTHyb, 2);
		simpleDisCSPProtocols.put(ABTac, 3);
		simpleDisCSPProtocols.put(ABTdac, 4);

		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(AFC, 101);
		simpleDisCSPProtocols.put(AFCMinDom, 102);
		simpleDisCSPProtocols.put(AFCDomDeg, 103);
		simpleDisCSPProtocols.put(AFCTriggeredng, 104);
		simpleDisCSPProtocols.put(AFCEstMinDom, 105);
		simpleDisCSPProtocols.put(AFCEstDomDeg, 106);
		// ------------------------------------------------------------------

		simpleDisCSPProtocols.put(AFCdbj, 108);
		simpleDisCSPProtocols.put(DDBJ, 109);
		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(AFCng, 141);
		simpleDisCSPProtocols.put(AFCngMinDom, 142);
		simpleDisCSPProtocols.put(AFCngDomDeg, 143);
		simpleDisCSPProtocols.put(AFCngTriggeredng, 144);
		simpleDisCSPProtocols.put(AFCngApproximatedMinDom, 145);
		simpleDisCSPProtocols.put(AFCngApproximatedDomDeg, 146);
		
		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(AFCtree, 161);

		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(ABTDOng, 211);
		simpleDisCSPProtocols.put(ABTDOMinDom, 212);
		simpleDisCSPProtocols.put(ABTDORandom, 213);
		simpleDisCSPProtocols.put(ABTDOMinDomS, 214);
		simpleDisCSPProtocols.put(ABTDORandomS, 215);
		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(ABTDORetro, 221);
		simpleDisCSPProtocols.put(ABTDORetroMinDom, 222);
		simpleDisCSPProtocols.put(ABTDORetroA2ndL, 223);
		simpleDisCSPProtocols.put(ABTDORetroDomPS, 224);
		simpleDisCSPProtocols.put(ABTDORetroDomPAll, 225);
		simpleDisCSPProtocols.put(ABTDORetroHighest, 226);
		/* ===================================================================================================== */
		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(ABTDOng, 211);
		simpleDisCSPProtocols.put(ABTDOMinDom, 212);
		simpleDisCSPProtocols.put(ABTDORandom, 213);
		simpleDisCSPProtocols.put(ABTDOMinDomS, 214);
		simpleDisCSPProtocols.put(ABTDORandomS, 215);
		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(ABTDORetro, 221);
		simpleDisCSPProtocols.put(ABTDORetroMinDom, 222);
		simpleDisCSPProtocols.put(ABTDORetroA2ndL, 223);
		simpleDisCSPProtocols.put(ABTDORetroDomPS, 224);
		simpleDisCSPProtocols.put(ABTDORetroDomPAll, 225);
		simpleDisCSPProtocols.put(ABTDORetroHighest, 226);
		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(AgileMinDom, 231);
		simpleDisCSPProtocols.put(Agile_ALL, 232);
		simpleDisCSPProtocols.put(Agile_O, 233);
		simpleDisCSPProtocols.put(Agile_N, 234);
		simpleDisCSPProtocols.put(Agile_ON, 235);
		simpleDisCSPProtocols.put(AgileMinDom_ID, 236);
		simpleDisCSPProtocols.put(AgileMinDom_Deg, 237);
		simpleDisCSPProtocols.put(AgileMinDom_MaxSucc, 238);
		simpleDisCSPProtocols.put(AgileDomDeg, 241);
		simpleDisCSPProtocols.put(AgileDomDeg_ID, 242);
		simpleDisCSPProtocols.put(AgileDomDeg_Deg, 243);
		simpleDisCSPProtocols.put(AgileMinDom_MaxSucc, 244);
		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(Agiledom, 251);
		simpleDisCSPProtocols.put(Agiledom_ID, 252);
		simpleDisCSPProtocols.put(Agiledom_Deg, 253);
		simpleDisCSPProtocols.put(Agiledom_MaxSucc, 254);

		simpleDisCSPProtocols.put(Agiledomdeg, 255);
		simpleDisCSPProtocols.put(Agiledomdeg_ID, 256);
		simpleDisCSPProtocols.put(Agiledomdeg_Deg, 257);
		simpleDisCSPProtocols.put(Agiledomdeg_MaxSucc, 258);

		simpleDisCSPProtocols.put(Agiledomwdeg, 261);
		simpleDisCSPProtocols.put(Agiledomwdeg_ID, 262);
		simpleDisCSPProtocols.put(Agiledomwdeg_Deg, 263);
		simpleDisCSPProtocols.put(Agiledomwdeg_MaxSucc, 264);
		/* ===================================================================================================== */
		simpleDisCSPProtocols.put(ZABTDORetro, 321);
		simpleDisCSPProtocols.put(ZABTDORetroMinDom, 322);
		simpleDisCSPProtocols.put(ZABTDORetroA2ndL, 323);
		simpleDisCSPProtocols.put(ZABTDORetroDomPS, 324);
		simpleDisCSPProtocols.put(ZABTDORetroDomPAll, 325);
		simpleDisCSPProtocols.put(ZABTDORetroHighest, 326);
		/* ===================================================================================================== */

		simpleDCOPProtocols.put(OriginalAdopt, 1001);
		simpleDCOPProtocols.put(Adopt, 1002);
		simpleDCOPProtocols.put(AdoptPlus, 1003);
		// simpleDCOPProtocols.put(MyAdopt, 1004);
		simpleDCOPProtocols.put(BnBAdopt, 1011);
		simpleDCOPProtocols.put(BnBAdoptPlus, 1012);
		simpleDCOPProtocols.put(BnBAdoptPlusDP2, 1013);
		

		simpleDCOPProtocols.put(SynchBB, 1051);
		
		simpleDCOPProtocols.put(AFB, 1261);
		simpleDCOPProtocols.put(AFB_minC, 1262);
		simpleDCOPProtocols.put(AFB_BJ, 1263);

		simpleDCOPProtocols.put(ConcFB, 1280);

		simpleDisCSPProtocols.put(DBA, 2000);

		complexDisCSPProtocols.put(ABTcf, 3000);


		protocols.putAll(simpleDisCSPProtocols);
		protocols.putAll(simpleDCOPProtocols);
		protocols.putAll(complexDisCSPProtocols);
		protocols.putAll(complexDCOPProtocols);
	}

	public static AbstractMaster getMaster(String protocol) {
		if (protocols.keySet().contains(protocol)) {
			switch (protocols.get(protocol)) {

			/* ==========================================================================simpleDisCSPProtocols.================= */
				case 0 :
					return new SBTMaster();
					/* ===================================================================================================== */
				case 1 :
					return new ABTMaster();
				case 2 :
					return new ABTHybMaster();
				case 3 :
					return new ABTacMaster();
				case 4 :
					return new ABTdacMaster();
					/* ===================================================================================================== */
				case 101 :
					return new AFCMaster(false);
				case 102 :
					return new AFCMinDomMaster();
				case 103 :
					return new AFCDomDegMaster();
				case 104 :
					return new AFCTriggeredNogoodMaster();
				case 105 :
					return new AFCEstimatedMaster(AFCEstimatedMaster.MinDom);
				case 106 :
					return new AFCEstimatedMaster(AFCEstimatedMaster.DomDeg);
					// --------------------
				case 108 :
					return new AFCDBJMaster();
				case 109 :
					return new DDBJMaster();
					/* ===================================================================================================== */
				case 121 :
					return new AFCMaster(true);
					
					/* ===================================================================================================== */
				case 141 :
					return new AFCngMaster();
				case 142 :
					return new AFCngMinDomMaster();
				case 143 :
					return new AFCngDomDegMaster();
				case 144 :
					return new AFCngTriggeredNogoodMaster();
				case 145 :
					return new AFCngApproximateMinDomMaster(AFCngApproximateMinDomMaster.MinDom);
				case 146 :
					return new AFCngApproximateMinDomMaster(AFCngApproximateMinDomMaster.MinDomDeg);
					/* ===================================================================================================== */
				case 161 :
					return new AFCTreeMaster();
					/* ===================================================================================================== */
					/* ===================================================================================================== */
				case 211 :
					return new ABTDOMaster(ABTDOMaster.Nogood_triggered);
				case 212 :
					return new ABTDOMaster(ABTDOMaster.MinDom);
				case 213 :
					return new ABTDOMaster(ABTDOMaster.Random);
				case 214 :
					return new ABTDOMaster(ABTDOMaster.MinDom_Single);
				case 215 :
					return new ABTDOMaster(ABTDOMaster.Random_Single);
					/* ===================================================================================================== */
				case 221 :
					return new ABTDORetroMaster(ABTDORetroMaster.MinDomain);
				case 222 :
					return new ABTDORetroMaster(ABTDORetroMaster.MinDomain);
				case 223 :
					return new ABTDORetroMaster(ABTDORetroMaster.After_Second_Last);
				case 224 :
					return new ABTDORetroMaster(ABTDORetroMaster.Dom_Pos_Single);
				case 225 :
					return new ABTDORetroMaster(ABTDORetroMaster.Dom_Pos_All);
				case 226 :
					return new ABTDORetroMaster(ABTDORetroMaster.Highest);
					/* ===================================================================================================== */
				case 231 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.MinDom);
				case 232 :
					return new AgileABTMaster(AgileABTMaster.All);
				case 233 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly);
				case 234 :
					return new AgileABTMaster(AgileABTMaster.Neighbors);
				case 235 :
					return new AgileABTMaster(AgileABTMaster.OrderNeigh);
				case 236 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.MinDom, AgileABTMaster.Topological_ID);
				case 237 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.MinDom, AgileABTMaster.Topological_Deg);
				case 238 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.MinDom, AgileABTMaster.Topological_MaxSucc);
				case 241 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.DomDeg);
				case 242 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.DomDeg, AgileABTMaster.Topological_ID);
				case 243 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.DomDeg, AgileABTMaster.Topological_Deg);
				case 244 :
					return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.DomDeg, AgileABTMaster.Topological_MaxSucc);
					/* ===================================================================================================== */
					/* ===================================================================================================== */
				case 251 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.MinDom);
				case 252 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.MinDom, AgileMaster.Topological_ID);
				case 253 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.MinDom, AgileMaster.Topological_Deg);
				case 254 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.MinDom, AgileMaster.Topological_MaxSucc);
				case 255 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomDeg);
				case 256 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomDeg, AgileMaster.Topological_ID);
				case 257 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomDeg, AgileMaster.Topological_Deg);
				case 258 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomDeg, AgileMaster.Topological_MaxSucc);
				case 261 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomWDeg);
				case 262 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomWDeg, AgileMaster.Topological_ID);
				case 263 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomWDeg, AgileMaster.Topological_Deg);
				case 264 :
					return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomWDeg, AgileMaster.Topological_MaxSucc);
					/* ===================================================================================================== */
					/* ===================================================================================================== */
				case 321 :
					return new ZABTDORetroMaster(ZABTDORetroMaster.MinDomain);
				case 322 :
					return new ZABTDORetroMaster(ZABTDORetroMaster.MinDomain);
				case 323 :
					return new ZABTDORetroMaster(ZABTDORetroMaster.After_Second_Last);
				case 324 :
					return new ZABTDORetroMaster(ZABTDORetroMaster.Dom_Pos_Single);
				case 325 :
					return new ZABTDORetroMaster(ZABTDORetroMaster.Dom_Pos_All);
				case 326 :
					return new ZABTDORetroMaster(ZABTDORetroMaster.Highest);
					/* ===================================================================================================== */

				case 1001 : return new MasterAdopt(protocol); // (OriginalAdopt);
				case 1002 : return new MasterAdopt(protocol); // (Adopt);
				case 1003 : return new MasterAdopt(protocol); // (AdoptPlus);

				case 1011 : return new MasterBnBAdopt(protocol); // (BnBAdopt);
				case 1012 : return new MasterBnBAdopt(protocol); // (BnBAdoptPlus);
				case 1013 : return new MasterBnBAdopt(BnBAdoptPlusDP2); // (BnBAdoptPlusDP2);
				case 1051 : return new SBBMaster(); 
				
				case 1261 : return new simple.dcop.afb.AFBMaster(AFB);
				case 1262 : return new simple.dcop.afb.AFBMaster(AFB_minC);
				case 1263 : return new simple.dcop.afb.AFBMaster(AFB_BJ);
				
				/* ===================================================================================================== */

				case 2000 :
					return new DBAmaster();
				default :
					throw new Error("Unknown protocol: " + protocol);
			}
		}
		throw new Error("Unknown protocol: " + protocol);
	}

	public static SortedMap<String, Integer> getProtocols(String type) {
		if (type.equals(SDisCSP)) {
			return simpleDisCSPProtocols;
		}
		if (type.equals(SDCOP)) {
			return simpleDCOPProtocols;
		}
		if (type.equals(CDisCSP)) {
			return complexDisCSPProtocols;
		}
		if (type.equals(CDCOP)) {
			return complexDCOPProtocols;
		}
		throw new Error("Invalid protocol type : " + type);
	}

	public static String[] getProtocolsName(String type) {
		SortedMap<String, Integer> protocols = getProtocols(type);
		return protocols.keySet().toArray(new String[protocols.size()]);
	}
}
