/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;

import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

/**
 * 
 * @authors <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and <a
 *          href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 * 
 */
public class Console extends JPanel implements Runnable {

	private static final long serialVersionUID = 1L;
	private JTextPane console = null;
	private Thread thread1;
	private Thread thread2;
	private Thread errorThrower;
	private final PipedInputStream p_input1 = new PipedInputStream();
	private final PipedInputStream p_input2 = new PipedInputStream();
	private Style Default = null;
	private Style Warning = null;
	private static int position = 0;
	private boolean active = true;
	private PrintStream outStream = null;
	private PrintStream errStream = null;

	PrintStream standarOut = System.out;
	PrintStream standarErr = System.err;

	public Console(boolean active) {
		super();
		this.active = active;
		this.setLayout(new BorderLayout());
		this.add(getConsole(), BorderLayout.CENTER);
		try {
			PipedOutputStream p_output1 = new PipedOutputStream(p_input1);
			outStream = new PrintStream(p_output1, true);
			PipedOutputStream p_output2 = new PipedOutputStream(p_input2);
			errStream = new PrintStream(p_output2, true);
		} catch (IOException io) {
			console.setText("STDERR can't be displayed in the GuiConsole Object\n" + io.getMessage());
		}
		// this.redirectOut();
	}

	private JTextPane getConsole() {
		if (console == null) {
			console = new JTextPane();
			console.setSelectedTextColor(Color.blue);
			console.setSelectionColor(Color.yellow);
			Style d = console.getStyle("default");
			Default = console.addStyle("Default", d);
			StyleConstants.setForeground(Default, Color.BLACK);
			StyleConstants.setFontSize(Default, 12);
			StyleConstants.setBold(Default, true);
			Warning = console.addStyle("Warning", d);
			StyleConstants.setForeground(Warning, Color.RED);
			StyleConstants.setFontSize(Warning, 12);
			StyleConstants.setBold(Warning, true);
		}
		return console;
	}

	public synchronized void run() {
		try {
			while (active && Thread.currentThread() == thread1) {
				try {
					wait(10);
				} catch (InterruptedException e) {
					printWarning("ERROR\n" + e.getMessage());
				}
				if (p_input1.available() != 0) {
					printText(readLine(p_input1));
					console.setCaretPosition(console.getDocument().getLength());
				}
				// if (!active) { return; }
			}
			while (active && Thread.currentThread() == thread2) {
				try {
					wait(10);
				} catch (InterruptedException e) {
					printWarning("ERROR\n" + e.getMessage());
				}
				if (p_input2.available() != 0) {
					printText(readLine(p_input2));
					console.setCaretPosition(console.getDocument().getLength());
				}
				// if (!active) { return; }
			}
		} catch (Exception e) {
			printWarning("\nConsole reports an Internal error.");
			printWarning("The error is: " + e);
		}
	}

	// private String read(PipedInputStream p_input) throws IOException {
	// int size = p_input.available();
	// byte tab[] = new byte[size];
	// p_input.read(tab);
	// return new String(tab, 0, tab.length);
	// }

	protected synchronized String readLine(PipedInputStream in) throws IOException {
		String input = "";
		do {
			int available = in.available();
			if (available == 0)
				break;
			byte b[] = new byte[available];
			in.read(b);
			input = input + new String(b, 0, b.length);
		} while (!input.endsWith("\n") && !input.endsWith("\r\n") && active);
		return input;
	}

	public void printText(String str) {
		position = console.getDocument().getLength();
		StyledDocument sDoc = (StyledDocument) console.getDocument();
		try {
			sDoc.insertString(position, str, Default);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		position += str.length();
	}

	public void println(String str) {
		this.printText(str + "\n");
	}

	public void printWarning(String str) {
		position = console.getDocument().getLength();
		StyledDocument sDoc = (StyledDocument) console.getDocument();
		try {
			sDoc.insertString(position, str, Warning);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		position += str.length();
	}

	public void printlnWarning(String str) {
		this.printWarning(str + "\n");
	}

	public void setActive(boolean active) {
		this.active = active;
		this.redirectOut();
	}

	public void redirectOut() {
		if (this.active) {
			System.setOut(outStream);
			System.setErr(errStream);
			thread1 = new Thread(this);
			thread1.setDaemon(true);
			thread1.start();
			thread2 = new Thread(this);
			thread2.setDaemon(true);
			thread2.start();
			errorThrower = new Thread(this);
			errorThrower.setDaemon(true);
			errorThrower.start();
		} else {
			System.setErr(standarErr);
			System.setOut(standarOut);
		}
	}
}
