/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * @version 2.0
 *          DisChoco
 *          .java
 */
package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

/**
 * 
 * 
 * @authors <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and <a
 *          href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 * 
 */
public class GuiConsole extends JPanel {

	public static final long serialVersionUID = 21362469L;
	private Style Default = null;
	private Style Warning = null;
	private static int position = 0;
	private TextConsole textConsole;
	private boolean active;
	final private PrintStream standarOut = System.out;
	final private PrintStream standarErr = System.err;
	private PrintStream outStream;
	private PrintStream errStream;

	public GuiConsole(boolean active) {
		super(new BorderLayout());
		this.add(getConsole(), BorderLayout.CENTER);
		this.setActive(active);
	}

	private TextConsole getConsole() {
		if (textConsole == null) {
			textConsole = new TextConsole();
		}
		return textConsole;
	}

	private OutputStream getOutputStream() {
		OutputStream out = new OutputStream() {

			@Override
			public void write(final int b) throws IOException {
				updateTextPane(String.valueOf((char) b));
			}

			@Override
			public void write(byte[] b, int off, int len) throws IOException {
				updateTextPane(new String(b, off, len));
			}

			@Override
			public void write(byte[] b) throws IOException {
				write(b, 0, b.length);
			}
		};
		return out;
	}

	public PrintStream getOutStream() {
		if (outStream == null) {
			outStream = new PrintStream(getOutputStream(), true);
		}
		return outStream;
	}

	public PrintStream getErrStream() {
		if (errStream == null) {
			errStream = new PrintStream(getOutputStream(), true);
		}
		return errStream;
	}

	private void updateTextPane(final String str) {
		SwingUtilities.invokeLater(new Runnable() {

			public void run() {
				position = textConsole.getDocument().getLength();
				StyledDocument sDoc = (StyledDocument) textConsole.getDocument();
				try {
					sDoc.insertString(position, str, Default);
				} catch (BadLocationException e) {
					throw new RuntimeException(e);
				}
				position += str.length();
			}
		});
	}

	public void printText(String str) {
		position = textConsole.getDocument().getLength();
		StyledDocument sDoc = (StyledDocument) textConsole.getDocument();
		try {
			sDoc.insertString(position, str, Default);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		position += str.length();
	}

	public void println(String str) {
		this.printText(str + "\n");
	}

	public void printWarning(String str) {
		position = textConsole.getDocument().getLength();
		StyledDocument sDoc = (StyledDocument) textConsole.getDocument();
		try {
			sDoc.insertString(position, str, Warning);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		position += str.length();
	}

	public void printlnWarning(String str) {
		this.printWarning(str + "\n");
	}

	public void setActive(boolean active) {
		this.active = active;
		this.redirectOut();
	}

	public void redirectOut() {
		if (this.active) {
			System.setOut(getOutStream());
			System.setErr(getErrStream());
		} else {
			System.setErr(standarErr);
			System.setOut(standarOut);
		}
	}

	@SuppressWarnings("serial")
	public class TextConsole extends JTextPane implements Printable {

		public TextConsole() {
			super();
			setSelectedTextColor(Color.blue);
			setSelectionColor(Color.yellow);
			Style d = getStyle("Default");
			Default = addStyle("Default", d);
			StyleConstants.setForeground(Default, Color.BLACK);
			StyleConstants.setFontSize(Default, 12);
			StyleConstants.setBold(Default, true);
			Warning = addStyle("Warning", d);
			StyleConstants.setForeground(Warning, Color.RED);
			StyleConstants.setFontSize(Warning, 12);
			StyleConstants.setBold(Warning, true);
			setEditable(false);
		}

		public void paintComponent(Graphics g) {
			super.paintComponent(g);
		}

		public int print(Graphics g, PageFormat pf, int pIndex) {
			if (pIndex > 0)
				return Printable.NO_SUCH_PAGE;

			Graphics2D g2d = (Graphics2D) g;
			g2d.translate(pf.getImageableX(), pf.getImageableY());

			/* Now print the window and its visible contents */
			printAll(g);
			return Printable.PAGE_EXISTS;
		}
	};
}
