/** 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 * 
 * @version 2.0
 * DisChoco
 * HJDgs.java
 */
package gui;

import static dischoco.DisChoco.ICONS_PATH;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * 
 * 
 * @authors <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and <a
 *          href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 * 
 */

public class LabelAccessory extends JLabel implements PropertyChangeListener {
	
	public final static ImageIcon xmlIcon = new ImageIcon(ClassLoader.getSystemClassLoader().getResource(ICONS_PATH + "blog_post.png"));
	public final static ImageIcon csvIcon = new ImageIcon(ClassLoader.getSystemClassLoader().getResource(ICONS_PATH + "blog_post.png"));
	public final static ImageIcon bz2Icon = new ImageIcon(ClassLoader.getSystemClassLoader().getResource(ICONS_PATH + "archive.png"));
	public final static ImageIcon pdfIcon = new ImageIcon(ClassLoader.getSystemClassLoader().getResource(ICONS_PATH + "pdf.png"));
	public final static ImageIcon folderIcon = new ImageIcon(ClassLoader.getSystemClassLoader().getResource(ICONS_PATH + "folder.png"));
	public final static ImageIcon homeIcon = new ImageIcon(ClassLoader.getSystemClassLoader().getResource(ICONS_PATH + "home.png"));
	
	private static final long serialVersionUID = 8356298286736204201L;
	private static final int PREFERRED_WIDTH = 150;
	private static final int PREFERRED_HEIGHT = 150;
	private File file = null;
	private JLabel label;
	private int maxImgWidth;


	public LabelAccessory(final JFileChooser chooser) {
		setLayout(new BorderLayout());
		setVerticalAlignment(JLabel.CENTER);
		setHorizontalAlignment(JLabel.CENTER);
	    JPanel northPanel = new JPanel(new BorderLayout());
	    JButton homeButton = new JButton(homeIcon);
	    homeButton.setToolTipText("Home");
	    homeButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent paramActionEvent) {
				chooser.setCurrentDirectory(new File(System.getProperty("user.home", ".")));
			}
	    });
	    JButton newFButton = new JButton(folderIcon);
	    newFButton.setToolTipText("New Folder");
	    northPanel.add(homeButton, BorderLayout.NORTH);
	    northPanel.add(newFButton, BorderLayout.SOUTH);
	    northPanel.setPreferredSize(new Dimension(PREFERRED_WIDTH, PREFERRED_HEIGHT/2));
	    add(northPanel, BorderLayout.NORTH);
	    JPanel centerPanel = new JPanel();
	    centerPanel.setLayout(new BorderLayout(5,5));
	    centerPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    centerPanel.add(new JLabel("Preview:"), BorderLayout.NORTH);
		label = new JLabel();
		label.setBackground(Color.WHITE);
		label.setOpaque(true);
		maxImgWidth = 145;
		label.setBorder(BorderFactory.createEtchedBorder());
		centerPanel.add(label, BorderLayout.CENTER);
		centerPanel.setPreferredSize(new Dimension(PREFERRED_WIDTH, PREFERRED_HEIGHT/2));
	    add(centerPanel, BorderLayout.CENTER);
		setPreferredSize(new Dimension(PREFERRED_WIDTH, PREFERRED_HEIGHT));
		chooser.addPropertyChangeListener(this);
	}

	public void propertyChange(PropertyChangeEvent evt) {
		ImageIcon icon = null;
		if(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(evt.getPropertyName())) {
			File newFile = (File) evt.getNewValue();
			if(newFile != null) {
				String path = newFile.getAbsolutePath();
				if(path.endsWith(".gif") || path.endsWith(".jpg") || path.endsWith(".png") || path.endsWith(".bmp") ) {
					try {
						BufferedImage img = ImageIO.read(newFile);
						float width = img.getWidth();
						float height = img.getHeight();
						float scale = height / width;
						width = maxImgWidth;
						height = (width * scale); // height should be scaled from new width							
						icon = new ImageIcon(img.getScaledInstance(Math.max(1, (int)width), Math.max(1, (int)height), Image.SCALE_SMOOTH));
					}
					catch(IOException e) { }
				} else{
					if (file != null) {
						icon = getIconOf(file);//
						if (icon == null) {
							icon = new ImageIcon(file.getPath());
						}
						if (icon.getIconWidth() > PREFERRED_WIDTH) {
							icon = new ImageIcon(icon.getImage().getScaledInstance(PREFERRED_WIDTH, -1, Image.SCALE_DEFAULT));
							if (icon.getIconHeight() > PREFERRED_HEIGHT) {
								icon = new ImageIcon(icon.getImage().getScaledInstance(-1, PREFERRED_HEIGHT, Image.SCALE_DEFAULT));
							}
						}
					}
				}
			}
				
			label.setIcon(icon);
			this.repaint();
					
		}
//		String changeName = changeEvent.getPropertyName();
//		if (changeName.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
//			File file = (File) changeEvent.getNewValue();
//			if (file != null) {
////				ImageIcon icon = JavaFileView.getIconOf(file);//
////				if (icon == null) {
////					icon = new ImageIcon(file.getPath());
////				}
////				if (icon.getIconWidth() > PREFERRED_WIDTH) {
////					icon = new ImageIcon(icon.getImage().getScaledInstance(PREFERRED_WIDTH, -1, Image.SCALE_DEFAULT));
////					if (icon.getIconHeight() > PREFERRED_HEIGHT) {
////						icon = new ImageIcon(icon.getImage().getScaledInstance(-1, PREFERRED_HEIGHT, Image.SCALE_DEFAULT));
////					}
////				}
////				setIcon(icon);
//				thumbnail = null;
//	            if (isShowing()) {
//	                loadImage();
//	                repaint();
//	            }
//			}
//		}
	}
	/*
    protected void paintComponent(Graphics g) {
        if (thumbnail == null) {
            loadImage();
        }
        if (thumbnail != null) {
            int x = getWidth()/2 - thumbnail.getIconWidth()/2;
            int y = getHeight()/2 - thumbnail.getIconHeight()/2;

            if (y < 0) {
                y = 0;
            }

            if (x < 5) {
                x = 5;
            }
            thumbnail.paintIcon(this, g, x, y);
        }
    }
    

    public void loadImage() {
        if (file == null) {
            thumbnail = null;
            return;
        }

        //Don't use createImageIcon (which is a wrapper for getResource)
        //because the image we're trying to load is probably not one
        //of this program's own resources.
        ImageIcon tmpIcon = new ImageIcon(file.getPath());
        if (tmpIcon != null) {
            if (tmpIcon.getIconWidth() > 20) {
                thumbnail = new ImageIcon(tmpIcon.getImage().getScaledInstance(90, -1, Image.SCALE_DEFAULT));
            } else { //no need to miniaturize
                thumbnail = tmpIcon;
            }
        }
    }
    */
	
	public static ImageIcon getIconOf(File file) {
		ImageIcon icon = folderIcon;
		if (file.isDirectory()) { 
			return icon;
		}
		String filename = file.getName().toLowerCase();
		if (filename.endsWith(".pdf")) {
			icon = pdfIcon;
		} else if (filename.endsWith(".xml")) {
			icon = xmlIcon;
		} else if (filename.endsWith(".bz2")) {
			icon = bz2Icon;
		} else if (filename.endsWith(".csv")) {
			icon = csvIcon;
		} else {
			icon = new ImageIcon(file.getAbsolutePath());
		}
		return icon;
	}
}
