package gui.experiments;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

import javax.swing.AbstractCellEditor;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EtchedBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.jdom2.DataConversionException;
import org.jdom2.Element;

import simulation.parser.InstanceTokens;
import util.Commands;

@SuppressWarnings("serial")
public class CommunicationPanel extends JPanel {

	private CommunicationEditable model = null;
	private JTable table;
	private JPanel buttonsPanel = null;
	private JButton addButton = null;
	private JButton removeButton = null;

	public CommunicationPanel(Dimension d) {
		super(new BorderLayout());
		Dimension size = new Dimension(d.width * 2/3, d.height);
		setSize(size);
		setMinimumSize(size);
		add(getButtonsPanel(), BorderLayout.SOUTH);
		add(new JScrollPane(getTable()), BorderLayout.CENTER);
	}
	/**
	 * 
	 * @return
	 */
	private JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			buttonsPanel = new JPanel();
			Dimension size = new Dimension(getWidth(), 40);
			buttonsPanel.setSize(size);
			buttonsPanel.add(getAddButton());
			buttonsPanel.add(getRemoveButton());
		}
		return buttonsPanel;
	}

	private JTable getTable() {
		if (table == null) {
			model = new CommunicationEditable();
			table = new JTable(model);
			table.setRowHeight(22);
			table.setIntercellSpacing(new Dimension(1, 1));
			table.getTableHeader().setReorderingAllowed(false);
			table.getTableHeader().setResizingAllowed(false);
			table.setShowVerticalLines(true);
			table.setShowHorizontalLines(true);
			table.setGridColor(Color.gray);
			TableCellRenderer tbch = getTableHeaderRenderer();
			for (int i = 0 ; i < table.getColumnCount() ; i++) {
				TableColumn tc = table.getColumnModel().getColumn(i);
				tc.setHeaderRenderer(tbch);
			}
			TableColumn column0 = table.getColumnModel().getColumn(0);
			column0.setMinWidth(175);
			column0.setCellEditor(new CommTypeEditor());
			TableColumn column1 = table.getColumnModel().getColumn(1);
			column1.setMinWidth(125);
			column1.setCellEditor(new ChannelTypeEditor());
			TableColumn column2 = table.getColumnModel().getColumn(2);
			column2.setMinWidth(80);
			column2.setCellEditor(new MinURCEditor());
			TableColumn column3 = table.getColumnModel().getColumn(3);
			column3.setMinWidth(80);
			column3.setCellEditor(new MaxURCEditor());
			TableColumn column4 = table.getColumnModel().getColumn(4);
			column4.setMinWidth(80);
			column4.setCellEditor(new NbRunEditor());
			TableColumn column5 = table.getColumnModel().getColumn(5);
			column5.setMinWidth(160);
			column5.setCellEditor(new MaxNCCCsEditor());
		}
		Dimension size = new Dimension(getWidth(), getHeight() -96);
		table.setMinimumSize(size);
		table.setSize(size);
		table.setPreferredScrollableViewportSize(size);
		return table;
	}

	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton("Add");
			addButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					model.addCommElement(new CommElement(Commands.FAST_COM, Commands.FIFO, 0, 0, 1, 100000000D));
					model.fireTableDataChanged();
				}
			});
		}
		return addButton;
	}

	private JButton getRemoveButton() {
		if (removeButton == null) {
			removeButton = new JButton("Remove");
			removeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int[] selection = table.getSelectedRows();
					for (int i = selection.length - 1 ; i >= 0 ; i--) {
						model.removeCommElement(selection[i]);
					}
					model.fireTableDataChanged();
				}
			});
		}
		return removeButton;
	}

	public static TableCellRenderer getTableHeaderRenderer() {
		return new TableCellRenderer() {
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				JLabel lbl = new JLabel();
				lbl.setBorder(new EtchedBorder());
				lbl.setHorizontalAlignment(JLabel.CENTER);
				Font font = new java.awt.Font("", Font.BOLD, 12);
				lbl.setOpaque(false);
				lbl.setFont(font);
				lbl.setForeground(Color.BLUE);
				lbl.setText((String) value);
				return lbl;
			}
		};
	}

	public Element getXMLElement() {
		Element communications = new Element(Commands.Communications);
		communications.setAttribute(Commands.nbCommunications, "" + this.table.getRowCount());
		for (int i = 0 ; i < this.table.getRowCount() ; i++) {
			CommElement row = model.getCommElement(i);
			Element communication = new Element(Commands.Communication);
			communication.setAttribute(InstanceTokens.TYPE, row.getCommType());
			communication.setAttribute(Commands.channelType, row.getChannelType());
			communication.setAttribute(Commands.MinURC, "" + row.getMinURC());
			communication.setAttribute(Commands.MaxURC, "" + row.getMaxURC());
			communication.setAttribute(Commands.NbRun, "" + row.getNbRun());
			communication.setAttribute(Commands.MaxNCCCs, "" + row.getMaxNCCCs().longValue());
			communications.addContent(communication);
		}
		return communications;
	}

	public void printToTable(Element communications) throws DataConversionException {
		List<Element> rows = communications.getChildren();
		this.clearTable();
		for (int r = 0 ; r < rows.size() ; r++) {
			Element comm = rows.get(r);
			String type = comm.getAttributeValue(InstanceTokens.TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			double maxNCCCs = new Double(comm.getAttributeValue(Commands.MaxNCCCs));
			model.addCommElement(new CommElement(type, channelType, minURC, maxURC, nbRun, maxNCCCs));
		}
	}

	public int getNbRows() {
		return this.table.getRowCount();
	}

	private void clearTable() {
		model.elements.clear();
		model.fireTableDataChanged();
	}
}

@SuppressWarnings("serial")
class CommunicationEditable extends AbstractTableModel {

	protected final List<CommElement> elements = new ArrayList<CommElement>();
	static Object[][] data = {};
	protected final static String[] headers =
			{"Type of communication", "Type of channel", "<html><center>Min Uniform<br/>Random<br/>Cost (CCCs)</center></html>",
					"<html><center>Max Uniform<br/>Random<br/>Cost (CCCs)</center></html>",
					"<html><center>Number<br/>of run for<br/>instance</center></html>", "max NCCCs"};

	public CommunicationEditable() {
		super();
		// elements.add(new CommElement(Commands.commTypes[0], Commands.FIFO,0,
		// 0, 1, 100000000));
	}

	public int getRowCount() {
		return elements.size();
	}

	public int getColumnCount() {
		return headers.length;
	}

	public String getColumnName(int columnIndex) {
		return headers[columnIndex];
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 0 :
				return ((CommElement) elements.get(rowIndex)).getCommType();
			case 1 :
				return ((CommElement) elements.get(rowIndex)).getChannelType();
			case 2 :
				return ((CommElement) elements.get(rowIndex)).getMinURC();
			case 3 :
				return ((CommElement) elements.get(rowIndex)).getMaxURC();
			case 4 :
				return ((CommElement) elements.get(rowIndex)).getNbRun();
			case 5 :
				return ((CommElement) elements.get(rowIndex)).getMaxNCCCs();
			default :
				return new Object(); // Must never happens
		}
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
			case 0 :
				return String.class;
			case 1 :
				return String.class;
			case 2 :
				return Integer.class;
			case 3 :
				return Integer.class;
			case 4 :
				return Integer.class;
			case 5 :
				return Double.class;
			default :
				return Object.class;
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return true;
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if (aValue != null) {
			CommElement element = (CommElement) elements.get(rowIndex);
			switch (columnIndex) {
				case 0 :
					String type = (String) aValue;
					element.setCommType(type);
					if (type.equals(Commands.FAST_COM)) {
						element.setMinURC(0);
						element.setMaxURC(0);
					} else {
						if (type.equals(Commands.MEDIUM_COM)) {
							element.setMinURC(50);
							element.setMaxURC(100);
						} else {
							element.setMinURC(500);
							element.setMaxURC(1000);
						}
					}
				break;
				case 1 :
					element.setChannelType((String) aValue);
				break;
				case 2 :
					int minURC = (Integer) aValue;
					element.setMinURC(minURC);
					element.setMaxURC(Math.max(minURC, element.getMaxURC()));
				break;
				case 3 :
					int maxURC = (Integer) aValue;
					element.setMaxURC(maxURC);
					element.setMinURC(Math.min(maxURC, element.getMinURC()));
				break;
				case 4 :
					element.setNbRun((Integer) aValue);
				break;
				case 5 :
					element.setMaxNCCCs((Double) aValue);
				break;
			}
			this.fireTableDataChanged();
		}
	}

	public void addCommElement(CommElement element) {
		elements.add(element);
		fireTableRowsInserted(elements.size() - 1, elements.size() - 1);
	}

	public CommElement getCommElement(int row) {
		return elements.get(row);
	}

	public void removeCommElement(int rowIndex) {
		if (rowIndex >= 0 && rowIndex < elements.size()) {
			elements.remove(rowIndex);
			fireTableRowsDeleted(rowIndex, rowIndex);
		}
	}
}

class CommElement {
	private String commType;
	private String channelType;
	private int minURC;
	private int maxURC;
	private int nbRun;
	private Double maxNCCCs;

	public CommElement(String comm, String channel, int minURC, int maxURC, int nbRun, Double maxNCCCs) {
		super();
		this.commType = comm;
		this.channelType = channel;
		this.minURC = minURC;
		this.maxURC = maxURC;
		this.nbRun = nbRun;
		this.maxNCCCs = maxNCCCs;
	}

	public String getCommType() {
		return commType;
	}

	public void setCommType(String comm) {
		this.commType = comm;
	}

	public String getChannelType() {
		return channelType;
	}

	public void setChannelType(String channel) {
		this.channelType = channel;
	}

	public int getMinURC() {
		return minURC;
	}

	public void setMinURC(int minURC) {
		this.minURC = minURC;
	}

	public int getMaxURC() {
		return maxURC;
	}

	public void setMaxURC(int maxURC) {
		this.maxURC = maxURC;
	}

	public int getNbRun() {
		return nbRun;
	}

	public void setNbRun(int nbRun) {
		this.nbRun = nbRun;
	}

	public Double getMaxNCCCs() {
		return maxNCCCs;
	}

	public void setMaxNCCCs(Double maxNCCCs) {
		this.maxNCCCs = maxNCCCs;
	}

}

@SuppressWarnings("serial")
class CommTypeEditor extends DefaultCellEditor {
	public CommTypeEditor() {
		super(new JComboBox(Commands.communicationTypes));
	}
}

@SuppressWarnings("serial")
class ChannelTypeEditor extends DefaultCellEditor {
	public ChannelTypeEditor() {
		super(new JComboBox(Commands.channelsTypes));
	}
}

@SuppressWarnings("serial")
class MinURCEditor extends AbstractCellEditor implements TableCellEditor {
	final JSpinner spinner = new JSpinner(new SpinnerNumberModel(1, 0, 1000, 1));

	// Initializes the spinner.
	public MinURCEditor() {}

	// Prepares the spinner component and returns it.
	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		spinner.setValue(value);
		return spinner;
	}

	// Enables the editor only for double-clicks.
	public boolean isCellEditable(EventObject evt) {
		// if (evt instanceof MouseEvent) {
		// return ((MouseEvent) evt).getClickCount() >= 2;
		// }
		return true;
	}

	// Returns the spinners current value.
	public Object getCellEditorValue() {
		return spinner.getValue();
	}
}

@SuppressWarnings("serial")
class MaxURCEditor extends AbstractCellEditor implements TableCellEditor {
	final JSpinner spinner = new JSpinner(new SpinnerNumberModel(1, 0, 10000, 1));

	// Initializes the spinner.
	public MaxURCEditor() {}

	// Prepares the spinner component and returns it.
	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		spinner.setValue(value);
		return spinner;
	}

	// Enables the editor only for double-clicks.
	public boolean isCellEditable(EventObject evt) {
		// if (evt instanceof MouseEvent) {
		// return ((MouseEvent) evt).getClickCount() >= 2;
		// }
		return true;
	}

	// Returns the spinners current value.
	public Object getCellEditorValue() {
		return spinner.getValue();
	}
}

@SuppressWarnings("serial")
class NbRunEditor extends AbstractCellEditor implements TableCellEditor {
	JSpinner spinner = null;

	public NbRunEditor() {
		spinner = new JSpinner(new SpinnerNumberModel(1, 1, 100, 1));
	}

	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		spinner.setValue(value);
		return spinner;
	}

	public boolean isCellEditable(EventObject evt) {
		return true;
	}

	public Object getCellEditorValue() {
		return spinner.getValue();
	}
}

@SuppressWarnings("serial")
class MaxNCCCsEditor extends AbstractCellEditor implements TableCellEditor {
	JSpinner spinner = null;

	public MaxNCCCsEditor() {
		spinner = new JSpinner(new SpinnerNumberModel(100000000, 1000000, Long.MAX_VALUE, 250000));
	}

	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		spinner.setValue(value);
		return spinner;
	}

	public boolean isCellEditable(EventObject evt) {
		return true;
	}

	public Object getCellEditorValue() {
		return spinner.getValue();
	}
}
