package gui.experiments;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.jdom2.DataConversionException;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import dischoco.DisChoco;
import gui.Filter;
import util.Commands;

public class ConfigurationPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	private JPanel centerPanel = null;
	private JPanel buttonsPanel = null;
	private CommunicationPanel communicationPanel = null;
	private ProtocolsPanel protocolsPanel = null;
	private JButton loadButton = null;
	private JButton mergeButton = null;
	public JFileChooser fileChooser = null;
	public File file = null;

	/**
	 * This is the default constructor
	 */
	public ConfigurationPanel(Dimension d) {
		super(new BorderLayout());
		setMinimumSize(d);
		setMaximumSize(d);
		setPreferredSize(d);
		setSize(d);
		add(getButtonsPanel(), BorderLayout.NORTH);
		add(getCenterPanel(), BorderLayout.CENTER);
		setBorder(BorderFactory.createEtchedBorder());
	}

	/**
	 * This method initializes centerPanel
	 * 
	 * @param height
	 * @param width
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getCenterPanel() {
		if (centerPanel == null) {
			centerPanel = new JPanel(new BorderLayout());
			centerPanel.add(getProtocolsPanel(), BorderLayout.WEST);
			centerPanel.add(getCommunicationPanel(), BorderLayout.CENTER);
		}
		return centerPanel;
	}
	
	public void paintComponents(Graphics g){
		getCenterPanel();
	}
	
	public void update(Graphics g){
		paintComponents(g);
	}
	
	/**
	 * @return
	 */
	private CommunicationPanel getCommunicationPanel() {
		Dimension size = new Dimension(getWidth()-2, getHeight() - buttonsPanel.getHeight());
		if(communicationPanel == null){
			communicationPanel = new CommunicationPanel(size);
		}
		return communicationPanel;
	}
	
	/**
	 * @return
	 */
	private ProtocolsPanel getProtocolsPanel() {
		Dimension size = new Dimension(getWidth()-2, getHeight() - buttonsPanel.getHeight());
		if(protocolsPanel == null){
			protocolsPanel = new ProtocolsPanel(size);
		}
		return protocolsPanel;
	}
	/**
	 * This method initializes buttonsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			buttonsPanel = new JPanel();
			Dimension size = new Dimension(getWidth(), 40);
			buttonsPanel.setMinimumSize(size);
			buttonsPanel.setMaximumSize(size);
			buttonsPanel.setPreferredSize(size);
			buttonsPanel.add(getLoadButton());
			buttonsPanel.add(getMergeButton());
		}
		return buttonsPanel;
	}

	/**
	 * This method initializes loadButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getLoadButton() {
		if (loadButton == null) {
			loadButton = new JButton("Load problems file");
			loadButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Container c = getParent();
					while(!(c instanceof JFrame)){
						c = c.getParent();
					}
					String description = "experiments file (XML)";
					String[] extension = new String[] {"xml"};
					Filter filter = new Filter(extension, description);
					fileChooser = new JFileChooser(DisChoco.filePath);
					fileChooser.setFileFilter(filter);
//					fileChooser.setFileView(new JavaFileView());
					fileChooser.setPreferredSize(new Dimension(660, 440));
					int returnVal = fileChooser.showOpenDialog(c);
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						file = fileChooser.getSelectedFile();
						DisChoco.filePath = file.getAbsolutePath();
						try {
							printToTable(file);
						} catch (DataConversionException e1) {}
						System.out.println("The file \"" + file + "\" was successfelly loaded!");
					}
				}
			});
		}
		return loadButton;
	}

	private void printToTable(File file) throws DataConversionException {
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(file);
		} catch (Exception ex) {}
		Element experiments = document.getRootElement();
		if (experiments.getChild(Commands.Communications) != null) {
			communicationPanel.printToTable(experiments.getChild(Commands.Communications));
		}
		if (experiments.getChild(Commands.Algorithms) != null) {
			protocolsPanel.printToTable(experiments.getChild(Commands.Algorithms));
		}
	}

	protected ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = ConfigurationPanel.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			Container c = getParent();
			while( !(c instanceof JFrame) ){
				c = c.getParent();
			}
			((DisChoco) c).getConsole().printlnWarning("Couldn't find file: " + path);
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	/**
	 * This method initializes mergeButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getMergeButton() {
		if (mergeButton == null) {
			mergeButton = new JButton("Merge");
			mergeButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					Container c = getParent();
					while(!(c instanceof JFrame)){
						c = c.getParent();
					}
					if (file == null) {
						String msg = "Load problems configuration file";
						((DisChoco) c).getConsole().printlnWarning(msg);
						JOptionPane.showMessageDialog(c, msg, "DisChoco", JOptionPane.WARNING_MESSAGE);
						return;
					}
					if (communicationPanel.getNbRows() == 0) {
						String msg = "You must choose at least one communication configuration";
						((DisChoco) c).getConsole().printlnWarning(msg);
						JOptionPane.showMessageDialog(c, msg, "DisChoco", JOptionPane.WARNING_MESSAGE);
						return;
					}
					if (protocolsPanel.getNbRows() == 0) {
						String msg = "You must choose at least one algorithm to test";
						((DisChoco) c).getConsole().printlnWarning(msg);
						JOptionPane.showMessageDialog(c, msg, "DisChoco", JOptionPane.WARNING_MESSAGE);
						return;
					}
					Document document = null;
					SAXBuilder sxb = new SAXBuilder();
					try {
						document = sxb.build(file);
					} catch (Exception ex) {}
					Element experiments = document.getRootElement();
					experiments.removeChild(Commands.Communications);
					experiments.removeChild(Commands.Algorithms);
					experiments.addContent(communicationPanel.getXMLElement());
					experiments.addContent(protocolsPanel.getXMLElement());
					try {
						XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
						out.output(document, new FileOutputStream(file.getPath()));
					} catch (java.io.IOException ex) {}
					String msg = "The configuration finished with success";
					JOptionPane.showMessageDialog(null, msg, "DisChoco", JOptionPane.INFORMATION_MESSAGE);
					// guiConsole.println(msg);
					System.out.println(msg);
				}
			});
		}
		return mergeButton;
	}

	// /**
	// * This method initializes consoleScrollPane
	// * @param height
	// * @param width
	// *
	// * @return javax.swing.JScrollPane
	// */
	// private JScrollPane getConsoleScrollPane(int width, int height) {
	// if (consoleScrollPane == null) {
	// consoleScrollPane = new
	// JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
	// JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	// consoleScrollPane.setMinimumSize(new Dimension(width, height));
	// consoleScrollPane.setPreferredSize(new Dimension(width, height));
	// consoleScrollPane.setMaximumSize(new Dimension(width, height));
	// consoleScrollPane.setViewportView(getGuiConsole());
	// }
	// return consoleScrollPane;
	// }
	//	
	// private GuiConsole getGuiConsole() {
	// if (guiConsole == null) {
	// guiConsole = new GuiConsole();
	// }
	// return guiConsole;
	// }

}
