package gui.experiments;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.jdom2.DataConversionException;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import gui.Filter;
import util.Commands;

public class ExperimentsPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	private JPanel commPanel = null;
	private JPanel algoPanel = null;
	private JPanel centerPanel = null;
	private JPanel buttonsPanel = null;
	private CommunicationPanel communicationPanel = null;
	private ProtocolsPanel protocolsPanel = null;
	private JButton loadButton = null;
	private JButton mergeButton = null;
	private JFileChooser fileChooser = null;
	private File file = null;

	/**
	 * This is the default constructor
	 */
	public ExperimentsPanel() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setLayout(new BorderLayout());
		Dimension d = new Dimension(1024, 500);
		this.add(getCenterPanel(d), BorderLayout.CENTER);
		this.setMinimumSize(new Dimension(1024, 500));
		this.setMaximumSize(new Dimension(1024, 500));
		this.setPreferredSize(new Dimension(1024, 500));
		this.setBorder(BorderFactory.createEtchedBorder());
	}

	/**
	 * This method initializes centerPanel
	 * 
	 * @param height
	 * @param width
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getCenterPanel(Dimension d) {
		if (centerPanel == null) {
			centerPanel = new JPanel();
			centerPanel.setLayout(new BorderLayout());
			centerPanel.add(getButtonsPanel(700, 40), BorderLayout.NORTH);
			centerPanel.add(getAlgoPanel(d), BorderLayout.WEST);
			centerPanel.add(getCommPanel(new Dimension(702, d.height - 100)), BorderLayout.CENTER);
		}
		return centerPanel;
	}

	/**
	 * This method initializes commPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getCommPanel(Dimension d) {
		if (commPanel == null) {
			commPanel = new JPanel();
			commPanel.setLayout(new BorderLayout());
			commPanel.add(getCommunicationPanel(d), BorderLayout.CENTER);
			commPanel.setMaximumSize(d);
			commPanel.setMinimumSize(d);
			commPanel.setPreferredSize(d);
		}
		return commPanel;
	}

	private CommunicationPanel getCommunicationPanel(Dimension d) {
		if (communicationPanel == null) {
			communicationPanel = new CommunicationPanel(d);
		}
		return communicationPanel;
	}

	/**
	 * This method initializes algoPanel
	 * 
	 * @param height
	 * @param width
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getAlgoPanel(Dimension d) {
		if (algoPanel == null) {
			algoPanel = new JPanel();
			Dimension size = new Dimension(d.width - 715, d.height - 100);
			algoPanel.setLayout(new BorderLayout());
			algoPanel.add(getProtocolsPanel(size), BorderLayout.CENTER);
			algoPanel.setMaximumSize(size);
			algoPanel.setMinimumSize(size);
			algoPanel.setPreferredSize(size);
		}
		return algoPanel;
	}

	private ProtocolsPanel getProtocolsPanel(Dimension d) {
		if (protocolsPanel == null) {
			protocolsPanel.setMaximumSize(d);
			protocolsPanel.setMinimumSize(d);
			protocolsPanel.setPreferredSize(d);
			protocolsPanel = new ProtocolsPanel(d);
		}
		return protocolsPanel;
	}
	
	/**
	 * This method initializes buttonsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getButtonsPanel(int width, int height) {
		if (buttonsPanel == null) {
			buttonsPanel = new JPanel();
			buttonsPanel.add(getLoadButton());
			buttonsPanel.add(getMergeButton());
			buttonsPanel.setMinimumSize(new Dimension(width, height));
			buttonsPanel.setMaximumSize(new Dimension(width, height));
			buttonsPanel.setPreferredSize(new Dimension(width, height));
		}
		return buttonsPanel;
	}

	/**
	 * This method initializes loadButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getLoadButton() {
		if (loadButton == null) {
			loadButton = new JButton("Load problems file");// ,
			loadButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String description = "configuration file (XML)";
					String[] extension = new String[] {"xml"};
					Filter filter = new Filter(extension, description);
					fileChooser = new JFileChooser(".");
					fileChooser.setFileFilter(filter);
					Container c = getParent();
					while(!(c instanceof JFrame)){
						c = c.getParent();
					}
					int returnVal = fileChooser.showSaveDialog(c);
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						file = fileChooser.getSelectedFile();
						try {
							printToTable(file);
						} catch (DataConversionException e1) {}
						System.out.println("The file \"" + file + "\" was successfelly uploaded!");
					}
				}
			});
		}
		return loadButton;
	}

	private void printToTable(File file) throws DataConversionException {
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(file);
		} catch (Exception ex) {}
		Element experiments = document.getRootElement();
		if (experiments.getChild(Commands.Communications) != null) {
			communicationPanel.printToTable(experiments.getChild(Commands.Communications));
		}
		if (experiments.getChild(Commands.Algorithms) != null) {
			protocolsPanel.printToTable(experiments.getChild(Commands.Algorithms));
		}
	}

	protected ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = ExperimentsPanel.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			// guiConsole.printlnWarning("Couldn't find file: " + path);
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	/**
	 * This method initializes mergeButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getMergeButton() {
		if (mergeButton == null) {
			mergeButton = new JButton("Merge");
			mergeButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (file == null) {
						String msg = "Load problems configuration file";
						System.err.println(msg);
						// guiConsole.printlnWarning(msg);
						JOptionPane.showMessageDialog(null, msg, "DisChoco", JOptionPane.WARNING_MESSAGE);
						return;
					}
					if (communicationPanel.getNbRows() == 0) {
						String msg = "You must choose at least one communication configuration";
						System.err.println(msg);
						// guiConsole.printlnWarning(msg);
						JOptionPane.showMessageDialog(null, msg, "DisChoco", JOptionPane.WARNING_MESSAGE);
						return;
					}
					if (protocolsPanel.getNbRows() == 0) {
						String msg = "You must choose at least one algorithm to test";
						System.err.println(msg);
						// guiConsole.printlnWarning(msg);
						JOptionPane.showMessageDialog(null, msg, "DisChoco", JOptionPane.WARNING_MESSAGE);
						return;
					}
					Document document = null;
					SAXBuilder sxb = new SAXBuilder();
					try {
						document = sxb.build(file);
					} catch (Exception ex) {}
					Element experiments = document.getRootElement();
					experiments.removeChild(Commands.Communications);
					experiments.removeChild(Commands.Algorithms);
					experiments.addContent(communicationPanel.getXMLElement());
					experiments.addContent(protocolsPanel.getXMLElement());
					// TODO
					// String msg = "Give the name of experiments file";
					// String name = JOptionPane.showInputDialog(null, msg,
					// "DisChoco", JOptionPane.OK_OPTION);
					// System.out.println(name);
					try {
						XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
						out.output(document, new FileOutputStream(file.getPath()));
					} catch (java.io.IOException ex) {}
					String msg = "The configuration finished with success";
					JOptionPane.showMessageDialog(null, msg, "DisChoco", JOptionPane.INFORMATION_MESSAGE);
					// guiConsole.println(msg);
					System.out.println(msg);
				}
			});
		}
		return mergeButton;
	}

	// /**
	// * This method initializes consoleScrollPane
	// * @param height
	// * @param width
	// *
	// * @return javax.swing.JScrollPane
	// */
	// private JScrollPane getConsoleScrollPane(int width, int height) {
	// if (consoleScrollPane == null) {
	// consoleScrollPane = new
	// JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
	// JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	// consoleScrollPane.setMinimumSize(new Dimension(width, height));
	// consoleScrollPane.setPreferredSize(new Dimension(width, height));
	// consoleScrollPane.setMaximumSize(new Dimension(width, height));
	// consoleScrollPane.setViewportView(getGuiConsole());
	// }
	// return consoleScrollPane;
	// }
	//	
	// private GuiConsole getGuiConsole() {
	// if (guiConsole == null) {
	// guiConsole = new GuiConsole();
	// }
	// return guiConsole;
	// }

}
