package gui.experiments;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.EtchedBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.jdom2.Element;

import dischoco.Protocols;

import kernel.order.AgentOrdering;
import util.Commands;

@SuppressWarnings("serial")
public class ProtocolsPanel extends JPanel {

	private ModelEditable model = null;
	private JTable table;
	private JPanel buttonsPanel = null;
	private JPanel typePanel = null;
	private JComboBox algorithmTypes = null;
	private String[] algorithms = null;
	private String[] orderings = null;
	private JButton addButton = null;
	private JButton removeButton = null;
	private JPanel centerPanel;

	public ProtocolsPanel(Dimension d) {
		super(new BorderLayout());
		Dimension size = new Dimension(d.width -(d.width * 2/3), d.height);	
		setSize(size);
		setMinimumSize(size);
		add(getTypePanel(), BorderLayout.NORTH);
		algorithms = getAlgorithms();
		orderings = AgentOrdering.getOrderTypes();
		add(getCentralPanel(), BorderLayout.CENTER);
	}

	private JPanel getTypePanel() {
		if (typePanel == null) {
			typePanel = new JPanel();
			typePanel.add(getAlgorithmTypes(), BorderLayout.CENTER);
		}
		return typePanel;
	}
	
	/**
	 * 
	 */
	private JPanel getCentralPanel() {
		if(centerPanel == null){
			centerPanel = new JPanel(new BorderLayout());
			centerPanel.setSize(getWidth(), getHeight()-40);
			centerPanel.add(getButtonsPanel(), BorderLayout.SOUTH);
			centerPanel.add(new JScrollPane(createTable()), BorderLayout.CENTER);
		}
		createTable();
		return centerPanel;
	}

	private JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			buttonsPanel = new JPanel();
			Dimension size = new Dimension(getWidth(), 40);
			buttonsPanel.setSize(size);
			buttonsPanel.setMinimumSize(size);
			buttonsPanel.add(getAddButton());
			buttonsPanel.add(getRemoveButton());
		}
		return buttonsPanel;
	}

	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton("Add");
			addButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					model.addAlgoElement(new AlgoElement(algorithms[0], orderings[0]));
					model.fireTableDataChanged();
				}
			});
		}
		return addButton;
	}

	private JButton getRemoveButton() {
		if (removeButton == null) {
			removeButton = new JButton("Remove");
			removeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int[] selection = table.getSelectedRows();
					for (int i = selection.length - 1 ; i >= 0 ; i--) {
						model.removeAlgoElement(selection[i]);
					}
					model.fireTableDataChanged();
				}
			});
		}
		return removeButton;
	}

	private JTable createTable() {
		Dimension size = new Dimension(getWidth(), getHeight()-106);
		if(table == null) {
			model = new ModelEditable();
			table = new JTable(model);
			table.setRowHeight(22);
			TableCellRenderer tbch = getTableHeaderRenderer();
			for (int i = 0 ; i < table.getColumnCount() ; i++) {
				TableColumn tc = table.getColumnModel().getColumn(i);
				tc.setHeaderRenderer(tbch);
			}
			table.setIntercellSpacing(new Dimension(1, 1));
			table.getTableHeader().setReorderingAllowed(false);
			table.getTableHeader().setResizingAllowed(false);
			table.setShowVerticalLines(true);
			table.setShowHorizontalLines(true);
			table.setGridColor(Color.gray);
			TableColumn algoColumn = table.getColumnModel().getColumn(0);
			int width = (int) (size.width*0.39);
			algoColumn.setPreferredWidth(width );
			algoColumn.setCellEditor(new AlgorithmCellEditor(this.algorithms));
			TableColumn orderColumn = table.getColumnModel().getColumn(1);
			orderColumn.setPreferredWidth(size.width - width);
			orderColumn.setCellEditor(new OrderingCellEditor(this.orderings));
		}
		table.setMinimumSize(size);
		table.setSize(size);
		table.setPreferredScrollableViewportSize(size);
		return table;
	}

	/**
	 * This method initializes algorithmTypes
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getAlgorithmTypes() {
		if (algorithmTypes == null) {
			String[] types = {Protocols.SDisCSP, Protocols.SDCOP, Protocols.CDisCSP, Protocols.CDCOP};
			algorithmTypes = new JComboBox(types);
			algorithmTypes.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					clearTable();
//					String selected = (String) algorithmTypes.getSelectedItem();
//					algorithms = Protocols.getProtocolsName(selected).clone();
					TableColumn algoColumn = createTable().getColumnModel().getColumn(0);
					algoColumn.setCellEditor(new AlgorithmCellEditor(algorithms));
				}
			});
		}
		return algorithmTypes;
	}

	private String[] getAlgorithms() {
		String type = (String) algorithmTypes.getSelectedItem();
		return Protocols.getProtocolsName(type);
	}

	public static TableCellRenderer getTableHeaderRenderer() {
		return new TableCellRenderer() {
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				JLabel lbl = new JLabel();
				lbl.setBorder(new EtchedBorder());
				lbl.setHorizontalAlignment(JLabel.CENTER);
				Font font = new java.awt.Font("", Font.BOLD, 12);
				lbl.setOpaque(false);
				lbl.setFont(font);
				lbl.setForeground(Color.BLUE);
				lbl.setText((String) value);
				return lbl;
			}
		};
	}

	public Element getXMLElement() {
		Element algorithms = new Element(Commands.Algorithms);
		algorithms.setAttribute(Commands.nbAlgorithms, "" + this.table.getRowCount());
		for (int i = 0 ; i < this.table.getRowCount() ; i++) {
			AlgoElement row = model.getAlgoElement(i);
			Element algorithm = new Element(Commands.Algorithm);
			algorithm.setText(row.getAlgorithm());
			algorithm.setAttribute(Commands.AO, row.getOrdering());
			algorithms.addContent(algorithm);
		}
		return algorithms;
	}

	public int getNbRows() {
		return this.table.getRowCount();
	}

	public void printToTable(Element algorithms) {
		List<Element> rows = algorithms.getChildren();
		this.clearTable();
		for (int r = 0 ; r < rows.size() ; r++) {
			Element comm = rows.get(r);
			String algorithm = comm.getText();
			String agentsOrdering = comm.getAttributeValue(Commands.AO);
			model.addAlgoElement(new AlgoElement(algorithm, agentsOrdering));
		}
	}

	private void clearTable() {
		model.elements.clear();
		model.fireTableDataChanged();
		algorithms = getAlgorithms();
	}
}

@SuppressWarnings("serial")
class ModelEditable extends AbstractTableModel {

	protected final List<AlgoElement> elements = new ArrayList<AlgoElement>();
	protected final String[] headers = {"Algorithms", Commands.AO};

	public ModelEditable() {
		super();
	}

	public int getRowCount() {
		return elements.size();
	}

	public int getColumnCount() {
		return headers.length;
	}

	public String getColumnName(int columnIndex) {
		return headers[columnIndex];
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 0 :
				return ((AlgoElement) elements.get(rowIndex)).getAlgorithm();
			case 1 :
				return ((AlgoElement) elements.get(rowIndex)).getOrdering();
			default :
				return new String(); // Must never happens
		}
	}

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		switch (columnIndex) {
			case 0 :
				return String.class;
			case 1 :
				return String.class;
			default :
				return Object.class;
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return true;
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if (aValue != null) {
			AlgoElement element = (AlgoElement) elements.get(rowIndex);
			switch (columnIndex) {
				case 0 :
					element.setAlgorithm((String) aValue);
				break;
				case 1 :
					element.setOrdering((String) aValue);
				break;
			}
		}
	}

	public void addAlgoElement(AlgoElement element) {
		elements.add(element);
		fireTableRowsInserted(elements.size() - 1, elements.size() - 1);
	}

	public AlgoElement getAlgoElement(int row) {
		return elements.get(row);
	}

	public void removeAlgoElement(int rowIndex) {
		if (rowIndex >= 0 && rowIndex < elements.size()) {
			elements.remove(rowIndex);
			fireTableRowsDeleted(rowIndex, rowIndex);
		}
	}
}

class AlgoElement {
	private String algorithm;
	private String ordering;

	public AlgoElement(String algorithm, String ordering) {
		super();
		this.algorithm = algorithm;
		this.ordering = ordering;
	}

	public String getAlgorithm() {
		return algorithm;
	}

	public void setAlgorithm(String algorithm) {
		this.algorithm = algorithm;
	}

	public String getOrdering() {
		return ordering;
	}

	public void setOrdering(String ordering) {
		this.ordering = ordering;
	}

}

@SuppressWarnings("serial")
class AlgorithmCellEditor extends DefaultCellEditor {
	public AlgorithmCellEditor(String[] items) {
		super(new JComboBox(items));
	}
}

@SuppressWarnings("serial")
class OrderingCellEditor extends DefaultCellEditor {
	public OrderingCellEditor(String[] items) {
		super(new JComboBox(items));
	}
}
