/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/
package gui.generators;

import static simulation.parser.InstanceTokens.EXTENSION;
import static simulation.parser.InstanceTokens.INTENSION;
import static util.Commands.DisponibilityOfAgents;
import static util.Commands.DurationOfMeetings;
import static util.Commands.NumberOfAgentPerMeetings;
import static util.Commands.NumberOfAgents;
import static util.Commands.NumberOfDays;
import static util.Commands.NumberOfHours;
import static util.Commands.NumberOfInstances;
import static util.Commands.NumberOfMeetings;
import gui.LabelAccessory;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import util.Commands;
import benchmark.meetingscheduling.DCOPMeetingScheduling;
import benchmark.meetingscheduling.DCOPMeetingScheduling.Mode;
import dischoco.DisChoco;

@SuppressWarnings("serial")
public class DCOPMeetingsSchedulingPanel extends JPanel {

	public static final String TOTALLY_KNOWN_CONSTRAINT = Commands.TOTALLY_KNOWN_CONSTRAINT;
	public static final String PARTIALLY_KNOWN_CONSTRAINT = Commands.PARTIALLY_KNOWN_CONSTRAINT;
	public static final String SUCCEFULLY = Commands.SUCCEFULLY;
	public static final String BROWSE = Commands.BROWSE;
	public static final String GENERATE = Commands.GENERATE;
	public static final String CONSTRAINT_MODEL = Commands.CONSTRAINT_MODEL;
	public static final String FROM = Commands.FROM;
	public static final String TO = Commands.TO;
	public static final String STEP = Commands.STEP;
	public static final String DOMAIN_SIZE = Commands.SizeOfDomain;
	public static final String CONSTRAINTS_TIGHTNESS = Commands.CONSTRAINTS_TIGHTNESS;
	
	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JPanel parametersPanel = null;
	private JButton browseButton = null;
	private JButton generateButton = null;
	private JFileChooser fileChooser = null;
	private JFrame mainFrame = null;
	private File file = null;

	private JSpinner nbInstances = null;
	private JSpinner maxCost = null;
	private JSpinner duration = null;
	private JSpinner nbDays = null;
	private JSpinner nbHours = null;
	private JSpinner minNbAgents = null;
	private JSpinner maxNbAgents = null;
	private JSpinner stepNbAgents = null;
	private JSpinner minNbOfMeeting = null;
	private JSpinner maxNbOfMeeting = null;
	private JSpinner stepNbOfMeeting = null;
	private JSpinner minAgentsPerMeeting = null;
	private JSpinner maxAgentsPerMeeting = null;
	private JSpinner stepAgentsPerMeeting = null;
	private JSpinner disponibility = null;
	private JComboBox mode = null;
	private JComboBox constraintsModel = null;
	private JComboBox constraintsType = null;
	
	/**
	 * Create the panel.
	 */
	public DCOPMeetingsSchedulingPanel(JFrame dischoco) {
		super(new BorderLayout());
		this.mainFrame = dischoco;
		this.setPreferredSize(new Dimension(600, 400));
		this.add(createNorthPanel(), BorderLayout.NORTH);
		this.add(createCenterPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes northPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createNorthPanel() {
		northPanel = new JPanel(new FlowLayout());
		browseButton = new JButton();
		browseButton.setText(BROWSE);
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				fileChooser = new JFileChooser(DisChoco.filePath);
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fileChooser.setAccessory(new LabelAccessory(fileChooser));
				int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getSelectedFile();
					if(!file.isDirectory()) {
						file = fileChooser.getCurrentDirectory();
					}
					DisChoco.filePath = file.getAbsolutePath();
					System.out.println("Path : " + file.getAbsolutePath() + "\n");
				}
			}
		});
		northPanel.add(browseButton);
		
		generateButton = new JButton();
		generateButton.setText(GENERATE);
		generateButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (file == null) {
					JOptionPane.showMessageDialog(mainFrame, "Choose firstly a directoty", "DisChoco", JOptionPane.WARNING_MESSAGE);
					return;
				}
				EventQueue.invokeLater(new Runnable() {
					int nbInstances = getNbInstances();
					int minNbMeetings = getMinNbMeetings();
					int maxNbMeetings = getMaxNbMeeting();
					int stepNbMeetings = getStepNbMeeting();

					int minNbAgents = getMinNbAgents();
					int maxNbAgents = getMaxNbAgents();
					int stepNbAgents = getStepNbAgents();

					int minMeetingsPerAgent = getMinAgentsPerMeeting();
					int maxMeetingsPerAgent = getMaxAgentsPerMeeting();
					int stepMeetingsPerAgent = getStepAgentsPerMeeting();

					int duration = getDuration();
					int nbDays = getNbDays();
					int nbHours = getNbHours();
					int disponibility = getDisponibility();

					Integer maxCost = getMaxCost();
					Mode mode = getSelectedMode();

					String constraintsModel = getConstraintsModel();
					boolean extention = constraintsType.getSelectedItem().equals(INTENSION); 

					DCOPMeetingScheduling msp = new DCOPMeetingScheduling(constraintsModel, 
							file, extention, 
							nbInstances, duration, nbDays, nbHours, disponibility, 
							minNbMeetings, maxNbMeetings, stepNbMeetings, 
							minNbAgents, maxNbAgents, stepNbAgents, 
							minMeetingsPerAgent, maxMeetingsPerAgent, stepMeetingsPerAgent, 
							maxCost, mode);

					public void run() {
						try {
							msp.generate();
							JOptionPane.showMessageDialog(mainFrame, SUCCEFULLY);
							file = null;
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		northPanel.add(generateButton);
		
		return northPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createCenterPanel() {
		centerPanel = new JPanel(new FlowLayout());
		centerPanel.add(createParametersPanel());
		return centerPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createParametersPanel() {
		parametersPanel = new JPanel(null);
		parametersPanel.setPreferredSize(new Dimension(600, 350));
		parametersPanel.setMinimumSize(new Dimension(600, 350));
		parametersPanel.setMaximumSize(new Dimension(600, 350));

		JLabel lblConstraintModel = new JLabel(CONSTRAINT_MODEL, JLabel.RIGHT);
		lblConstraintModel.setBounds(new Rectangle(145, 10, 120, 22));
		parametersPanel.add(lblConstraintModel);
		
		String[] models = {TOTALLY_KNOWN_CONSTRAINT, PARTIALLY_KNOWN_CONSTRAINT};
		constraintsModel = new JComboBox(models);
		constraintsModel.setSelectedItem(TOTALLY_KNOWN_CONSTRAINT);
		constraintsModel.setBounds(new Rectangle(275, 10, 210, 24));
		parametersPanel.add(constraintsModel);
		
		JLabel lblType = new JLabel("Constraint Type", JLabel.RIGHT);
		lblType.setBounds(new Rectangle(315, 55, 100, 22));
		parametersPanel.add(lblType); 

		String[] types = {EXTENSION, INTENSION};
		constraintsType = new JComboBox(types);
		constraintsType.setSelectedItem(EXTENSION);
		constraintsType.setBounds(new Rectangle(415, 55, 120, 24)); 
		parametersPanel.add(constraintsType); 

		JLabel lblNbInstance = new JLabel(NumberOfInstances, JLabel.RIGHT);
		lblNbInstance.setBounds(new Rectangle(65, 95, 150, 22));
		parametersPanel.add(lblNbInstance);
		
		nbInstances = new JSpinner(new SpinnerNumberModel(25, 1, 200, 1));
		nbInstances.setBounds(new Rectangle(220, 95, 60, 24));
		parametersPanel.add(nbInstances);
		
		JLabel lblMaxCost = new JLabel(Commands.MAXIMUM_COST, JLabel.RIGHT);
		lblMaxCost.setBounds(new Rectangle(295, 95, 150, 22));
		parametersPanel.add(lblMaxCost);
		
		maxCost = new JSpinner(new SpinnerNumberModel(100, 0, Integer.MAX_VALUE, 10));
		maxCost.setBounds(450, 95, 80, 22);
		parametersPanel.add(maxCost);
		
		JLabel lblMode = new JLabel("DiMES Mode", JLabel.RIGHT);
		lblMode.setBounds(new Rectangle(75, 55, 120, 22));
		parametersPanel.add(lblMode);
		
		Mode[] category = {Mode.EAV, Mode.EASV, Mode.PEAV};
		mode = new JComboBox(category);
		mode.setSelectedItem(Mode.PEAV);
		mode.setBounds(new Rectangle(195, 55, 90, 24));
		parametersPanel.add(mode);

		JLabel lblDays = new JLabel(NumberOfDays, JLabel.RIGHT);
		lblDays.setBounds(new Rectangle(65, 135, 150, 22));
		parametersPanel.add(lblDays);
		
		nbDays = new JSpinner(new SpinnerNumberModel(1, 1, 30, 1));
		nbDays.setBounds(new Rectangle(220, 135, 60, 22));
		parametersPanel.add(nbDays);
		
		JLabel lblHours = new JLabel(NumberOfHours, JLabel.RIGHT);
		lblHours.setBounds(new Rectangle(325, 135, 140, 22));
		parametersPanel.add(lblHours);

		nbHours = new JSpinner(new SpinnerNumberModel(8, 2, 24, 1));
		nbHours.setBounds(new Rectangle(470, 135, 60, 24));
		parametersPanel.add(nbHours);
		
		JLabel lblDuration = new JLabel(DurationOfMeetings, JLabel.RIGHT);
		lblDuration.setBounds(new Rectangle(325, 170, 140, 22));
		parametersPanel.add(lblDuration);

		duration = new JSpinner(new SpinnerNumberModel(1, 1, 8, 1));
		duration.setBounds(new Rectangle(470, 170, 60, 24));
		parametersPanel.add(duration);
		//==============================================================
		
		JLabel minLabel = new JLabel(FROM, JLabel.CENTER);
		minLabel.setBounds(new Rectangle(275, 200, 80, 22));
		parametersPanel.add(minLabel);
		
		JLabel maxLabel = new JLabel(TO, JLabel.CENTER);
		maxLabel.setBounds(new Rectangle(455, 200, 80, 22));
		parametersPanel.add(maxLabel);
		
		JLabel stepLabel = new JLabel(STEP, JLabel.CENTER);
		stepLabel.setBounds(new Rectangle(380, 200, 60, 22));
		parametersPanel.add(stepLabel);

		JLabel lblNbAgent = new JLabel(NumberOfAgents, JLabel.RIGHT);
		lblNbAgent.setBounds(new Rectangle(145, 270, 130, 22));
		parametersPanel.add(lblNbAgent);
		
		minNbAgents = new JSpinner(new SpinnerNumberModel(10, 3, 50, 1));
		minNbAgents.setBounds(new Rectangle(285, 270, 60, 24));
		parametersPanel.add(minNbAgents);

		maxNbAgents = new JSpinner(new SpinnerNumberModel(20, 3, 50, 1));
		maxNbAgents.setBounds(new Rectangle(470, 270, 60, 22));
		parametersPanel.add(maxNbAgents);

		stepNbAgents = new JSpinner(new SpinnerNumberModel(5, 1, 25, 1));
		stepNbAgents.setBounds(new Rectangle(380, 270, 60, 24));
		parametersPanel.add(stepNbAgents);
		
		JLabel lblNbOfMeeting = new JLabel(NumberOfMeetings, JLabel.RIGHT);
		lblNbOfMeeting.setBounds(new Rectangle(145, 230, 130, 22));
		parametersPanel.add(lblNbOfMeeting);
		
		minNbOfMeeting = new JSpinner(new SpinnerNumberModel(9, 5, 50, 1));
		minNbOfMeeting.setBounds(new Rectangle(285, 230, 60, 24));
		parametersPanel.add(minNbOfMeeting);

		maxNbOfMeeting = new JSpinner(new SpinnerNumberModel(11, 5, 50, 1));
		maxNbOfMeeting.setBounds(new Rectangle(470, 230, 60, 22));
		parametersPanel.add(maxNbOfMeeting);

		stepNbOfMeeting = new JSpinner(new SpinnerNumberModel(2, 0, 10, 1));
		stepNbOfMeeting.setBounds(new Rectangle(380, 230, 60, 22));
		parametersPanel.add(stepNbOfMeeting);


		JLabel lblAgentsPerMeeting = new JLabel(NumberOfAgentPerMeetings, JLabel.RIGHT);
		lblAgentsPerMeeting.setBounds(new Rectangle(75, 310, 200, 22));
		parametersPanel.add(lblAgentsPerMeeting);

		minAgentsPerMeeting = new JSpinner(new SpinnerNumberModel(3, 2, 8, 1));
		minAgentsPerMeeting.setBounds(new Rectangle(285, 310, 60, 22));
		parametersPanel.add(minAgentsPerMeeting);
		
		maxAgentsPerMeeting = new JSpinner(new SpinnerNumberModel(5, 2, 20, 1));
		maxAgentsPerMeeting.setBounds(new Rectangle(470, 310, 60, 22));
		parametersPanel.add(maxAgentsPerMeeting);

		stepAgentsPerMeeting = new JSpinner(new SpinnerNumberModel(2, 0, 25, 1));
		stepAgentsPerMeeting.setBounds(new Rectangle(380, 310, 60, 22));
		parametersPanel.add(stepAgentsPerMeeting);

		disponibility = new JSpinner(new SpinnerNumberModel(100, 1, 100, 1));
		disponibility.setBounds(new Rectangle(220, 169, 60, 24));
		parametersPanel.add(disponibility);

		JLabel lblDisponibility = new JLabel(DisponibilityOfAgents, JLabel.RIGHT);
		lblDisponibility.setBounds(new Rectangle(60, 169, 150, 22));
		parametersPanel.add(lblDisponibility);

		JLabel lblPercent = new JLabel("%", JLabel.LEFT);
		lblPercent.setBounds(new Rectangle(285, 169, 20, 22));
		parametersPanel.add(lblPercent);
		
		return parametersPanel;
	}

	
	public Integer getNbInstances() {
		return (Integer) this.nbInstances.getValue();
	}
	
	public Integer getMinNbMeetings() {
		return (Integer) this.minNbOfMeeting.getValue();
	}
	
	public Integer getMaxNbMeeting() {
		return (Integer) this.maxNbOfMeeting.getValue();
	}
	
	public Integer getStepNbMeeting() {
		return (Integer) this.stepNbOfMeeting.getValue();
	}
	
	public Integer getMinNbAgents() {
		return (Integer) this.minNbAgents.getValue();
	}
	
	public Integer getMaxNbAgents() {
		return (Integer) this.maxNbAgents.getValue();
	}
	
	public Integer getStepNbAgents() {
		return (Integer) this.stepNbAgents.getValue();
	}
	
	public Integer getMinAgentsPerMeeting() {
		return (Integer) this.minAgentsPerMeeting.getValue();
	}
	
	public Integer getMaxAgentsPerMeeting() {
		return (Integer) this.maxAgentsPerMeeting.getValue();
	}
	
	public Integer getStepAgentsPerMeeting() {
		return (Integer) this.stepAgentsPerMeeting.getValue();
	}

	public Integer getDuration() {
		return (Integer) this.duration.getValue();
	}

	public Integer getNbDays() {
		return (Integer) this.nbDays.getValue();
	}

	public Integer getNbHours() {
		return (Integer) this.nbHours.getValue();
	}

	public Integer getDisponibility() {
		return (Integer) this.disponibility.getValue();
	}
	
	public Integer getMaxCost() {
		return (Integer) this.maxCost.getValue();
	}
	
	private Mode getSelectedMode() {
		return (Mode) this.mode.getSelectedItem();
	}
	
	public String getConstraintsModel() {
		String model = (String) constraintsModel.getSelectedItem();
		if(model.equals(TOTALLY_KNOWN_CONSTRAINT)) {
			return Commands.TKC;
		}
		return Commands.PKC;
	}
}
