/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package gui.generators;

import gui.LabelAccessory;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import util.Commands;
import benchmark.meetingscheduling.DCSPMeetingScheduling;
import dischoco.DisChoco;

@SuppressWarnings("serial")
public class DCSPMeetingsSchedulingPanel extends JPanel {
	
	public static final String SUCCEFULLY = Commands.SUCCEFULLY;
	public static final String BROWSE = Commands.BROWSE;
	public static final String GENERATE = Commands.GENERATE;
	public static final String NB_INSTANCES = Commands.NumberOfInstances;
	public static final String NB_AGENTS = Commands.NumberOfAgents;
	public static final String DOMAIN_SIZE = Commands.SizeOfDomain;
	public static final String GRID_SIZE = Commands.SizeOfGrid;
	
	public static final String DISPONIBILITY = Commands.DisponibilityOfAgents;
	public static final String DURATION = Commands.DurationOfMeetings;
	public static final String MAXIMUM = Commands.MAXIMUM;
	public static final String MINIMUM = Commands.MINIMUM;
	public static final String STEP = Commands.STEP;
	public static final String NB_DAYS = Commands.NumberOfDays;
	public static final String NB_HOURS = Commands.NumberOfHours;
	public static final String NB_MEETINGS = Commands.NumberOfMeetings;
	public static final String NB_MEETINGS_PER_AGENT = Commands.NumberOfMeetingsPerAgent;
	
	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JPanel parametersPanel = null;
	private JButton browseButton = null;
	private JButton generateButton = null;
	private JFileChooser fileChooser = null;
	private JFrame mainFrame = null;
	private File file = null;
	
	private JSpinner nbInstances = null;
	private JSpinner duration = null;
	private JSpinner nbDays = null;
	private JSpinner nbHours = null;
	private JSpinner minNbAgents = null;
	private JSpinner maxNbAgents = null;
	private JSpinner stepNbAgents = null;
	private JSpinner minNbOfMeeting = null;
	private JSpinner maxNbOfMeeting = null;
	private JSpinner stepNbOfMeeting = null;
	private JSpinner minMeetingPerAgent = null;
	private JSpinner maxMeetingPerAgent = null;
	private JSpinner stepMeetingPerAgent = null;
	private JSpinner minGridSize = null;
	private JSpinner maxGridSize = null;
	private JSpinner stepGridSize = null;
	private JSpinner disponibility = null;
	
	/**
	 * Create the panel.
	 */
	public DCSPMeetingsSchedulingPanel(JFrame dischoco) {
		super(new BorderLayout());
		this.mainFrame = dischoco;
		this.setPreferredSize(new Dimension(600, 425));
		this.add(createNorthPanel(), BorderLayout.NORTH);
		this.add(createCenterPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes northPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createNorthPanel() {
		northPanel = new JPanel(new FlowLayout());
		browseButton = new JButton();
		browseButton.setText(BROWSE);
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				fileChooser = new JFileChooser(DisChoco.filePath);
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fileChooser.setAccessory(new LabelAccessory(fileChooser));
				int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getSelectedFile();
					if(!file.isDirectory()) {
						file = fileChooser.getCurrentDirectory();
					}
					DisChoco.filePath = file.getAbsolutePath();
					System.out.println("Path : " + file.getAbsolutePath() + "\n");
				}
			}
		});
		northPanel.add(browseButton);
		
		generateButton = new JButton();
		generateButton.setText(GENERATE);
		generateButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (file == null) {
					JOptionPane.showMessageDialog(mainFrame, "Choose firstly a directoty", "DisChoco", JOptionPane.WARNING_MESSAGE);
					return;
				}
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							int nbInstances = getNbInstances();
							int minNbMeetings = getMinNbMeetings();
							int maxNbMeetings = getMaxNbMeeting();
							int stepNbMeetings = getStepNbMeeting();
							int minNbAgents = getMinNbAgents();
							int maxNbAgents = getMaxNbAgents();
							int stepNbAgents = getStepNbAgents();
							int minMeetingsPerAgent = getMinMeetingPerAgent();
							int maxMeetingsPerAgent = getMaxMeetingPerAgent();
							int stepMeetingsPerAgent = getStepMeetingPerAgent();
							int duration = getDuration();
							int nbDays = getNbDays();
							int nbHours = getNbHours();
							int minGridSize = getMinGridSize();
							int maxGridSize = getMaxGridSize();
							int stepGridSize = getStepGridSize();
							int disponibility = getDisponibility();
							DCSPMeetingScheduling msp = new DCSPMeetingScheduling(file, nbInstances, duration, 
									minNbMeetings, maxNbMeetings, stepNbMeetings, 
									minNbAgents, maxNbAgents, stepNbAgents, 
									minMeetingsPerAgent, maxMeetingsPerAgent, stepMeetingsPerAgent, 
									minGridSize, maxGridSize, stepGridSize, 
									nbDays, nbHours, disponibility);
							msp.generate();
							JOptionPane.showMessageDialog(mainFrame, SUCCEFULLY);
							file = null;
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		northPanel.add(generateButton);
		return northPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createCenterPanel() {
		centerPanel = new JPanel(new FlowLayout());
		centerPanel.add(createParametersPanel());
		return centerPanel;
	}
	
	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createParametersPanel() {
		parametersPanel = new JPanel(null);
		parametersPanel.setPreferredSize(new Dimension(600, 375));
		parametersPanel.setMinimumSize(new Dimension(600, 375));
		parametersPanel.setMaximumSize(new Dimension(600, 375));

		JLabel lblNbInstance = new JLabel(NB_INSTANCES, JLabel.RIGHT);
		lblNbInstance.setBounds(new Rectangle(20, 25, 150, 22));
		parametersPanel.add(lblNbInstance);
		
		nbInstances = new JSpinner(new SpinnerNumberModel(25, 1, 200, 1));
		nbInstances.setBounds(new Rectangle(175, 25, 60, 24));
		parametersPanel.add(nbInstances);

		duration = new JSpinner(new SpinnerNumberModel(1, 1, 8, 1));
		duration.setBounds(new Rectangle(400, 25, 60, 24));
		parametersPanel.add(duration);
		
		JLabel lblDuration = new JLabel(DURATION, JLabel.RIGHT);
		lblDuration.setBounds(new Rectangle(255, 25, 140, 22));
		parametersPanel.add(lblDuration);

		JLabel lblDays = new JLabel(NB_DAYS, JLabel.RIGHT);
		lblDays.setBounds(new Rectangle(20, 65, 150, 22));
		parametersPanel.add(lblDays);
		
		nbDays = new JSpinner(new SpinnerNumberModel(2, 1, 30, 1));
		nbDays.setBounds(new Rectangle(175, 65, 60, 22));
		parametersPanel.add(nbDays);
		
		JLabel lblHours = new JLabel(NB_HOURS, JLabel.RIGHT);
		lblHours.setBounds(new Rectangle(255, 65, 140, 22));
		parametersPanel.add(lblHours);

		nbHours = new JSpinner(new SpinnerNumberModel(10, 2, 24, 1));
		nbHours.setBounds(new Rectangle(400, 65, 60, 24));
		parametersPanel.add(nbHours);
		//==============================================================
		
		JLabel minLabel = new JLabel(MINIMUM, JLabel.CENTER);
		minLabel.setBounds(new Rectangle(210, 105, 80, 22));
		parametersPanel.add(minLabel);
		
		JLabel maxLabel = new JLabel(MAXIMUM, JLabel.CENTER);
		maxLabel.setBounds(new Rectangle(295, 105, 80, 22));
		parametersPanel.add(maxLabel);
		
		JLabel stepLabel = new JLabel(STEP, JLabel.CENTER);
		stepLabel.setBounds(new Rectangle(400, 105, 60, 22));
		parametersPanel.add(stepLabel);

		JLabel lblNbAgent = new JLabel(NB_AGENTS, JLabel.RIGHT);
		lblNbAgent.setBounds(new Rectangle(80, 175, 130, 22));
		parametersPanel.add(lblNbAgent);
		
		minNbAgents = new JSpinner(new SpinnerNumberModel(16, 3, 50, 1));
		minNbAgents.setBounds(new Rectangle(220, 175, 60, 24));
		parametersPanel.add(minNbAgents);

		maxNbAgents = new JSpinner(new SpinnerNumberModel(20, 5, 50, 1));
		maxNbAgents.setBounds(new Rectangle(310, 175, 60, 22));
		parametersPanel.add(maxNbAgents);

		stepNbAgents = new JSpinner(new SpinnerNumberModel(2, 0, 25, 1));
		stepNbAgents.setBounds(new Rectangle(400, 175, 60, 24));
		parametersPanel.add(stepNbAgents);
		
		minNbOfMeeting = new JSpinner(new SpinnerNumberModel(9, 5, 50, 1));
		minNbOfMeeting.setBounds(new Rectangle(220, 135, 60, 24));
		parametersPanel.add(minNbOfMeeting);
		
		maxMeetingPerAgent = new JSpinner(new SpinnerNumberModel(7, 2, 20, 1));
		maxMeetingPerAgent.setBounds(new Rectangle(310, 215, 60, 22));
		parametersPanel.add(maxMeetingPerAgent);

		stepMeetingPerAgent = new JSpinner(new SpinnerNumberModel(1, 0, 25, 1));
		stepMeetingPerAgent.setBounds(new Rectangle(400, 215, 60, 22));
		parametersPanel.add(stepMeetingPerAgent);
		
		JLabel lblNbOfMeeting = new JLabel(NB_MEETINGS, JLabel.RIGHT);
		lblNbOfMeeting.setBounds(new Rectangle(80, 135, 130, 22));
		parametersPanel.add(lblNbOfMeeting);

		minMeetingPerAgent = new JSpinner(new SpinnerNumberModel(2, 2, 8, 1));
		minMeetingPerAgent.setBounds(new Rectangle(220, 215, 60, 22));
		parametersPanel.add(minMeetingPerAgent);

		maxNbOfMeeting = new JSpinner(new SpinnerNumberModel(11, 5, 50, 1));
		maxNbOfMeeting.setBounds(new Rectangle(310, 135, 60, 22));
		parametersPanel.add(maxNbOfMeeting);

		stepNbOfMeeting = new JSpinner(new SpinnerNumberModel(1, 0, 10, 1));
		stepNbOfMeeting.setBounds(new Rectangle(400, 135, 60, 22));
		parametersPanel.add(stepNbOfMeeting);

		JLabel lblMeetingPerAgent = new JLabel(NB_MEETINGS_PER_AGENT, JLabel.RIGHT);
		lblMeetingPerAgent.setBounds(new Rectangle(10, 215, 200, 22));
		parametersPanel.add(lblMeetingPerAgent);

		JLabel lblGridSize = new JLabel(GRID_SIZE, JLabel.RIGHT);
		lblGridSize.setBounds(new Rectangle(60, 255, 150, 22));
		parametersPanel.add(lblGridSize);
		
		minGridSize = new JSpinner(new SpinnerNumberModel(3, 1, 1000, 1));
		minGridSize.setBounds(new Rectangle(220, 255, 60, 24));
		parametersPanel.add(minGridSize);

		maxGridSize = new JSpinner(new SpinnerNumberModel(21, 1, 2000, 1));
		maxGridSize.setBounds(new Rectangle(310, 255, 60, 24));
		parametersPanel.add(maxGridSize);
		
		stepGridSize = new JSpinner(new SpinnerNumberModel(2, 1, 2000, 1));
		stepGridSize.setBounds(new Rectangle(400, 255, 60, 24));
		parametersPanel.add(stepGridSize);

		disponibility = new JSpinner(new SpinnerNumberModel(100, 1, 100, 1));
		disponibility.setBounds(new Rectangle(300, 300, 60, 24));
		parametersPanel.add(disponibility);

		JLabel lblDisponibility = new JLabel(DISPONIBILITY, JLabel.RIGHT);
		lblDisponibility.setBounds(new Rectangle(140, 300, 150, 22));
		parametersPanel.add(lblDisponibility);

		JLabel lblPercent = new JLabel("%", JLabel.LEFT);
		lblPercent.setBounds(new Rectangle(365, 300, 20, 22));
		parametersPanel.add(lblPercent);
		
		return parametersPanel;
	}
	
	public Integer getNbInstances() {
		return (Integer) this.nbInstances.getValue();
	}
	
	public Integer getMinNbMeetings() {
		return (Integer) this.minNbOfMeeting.getValue();
	}
	
	public Integer getMaxNbMeeting() {
		return (Integer) this.maxNbOfMeeting.getValue();
	}
	
	public Integer getStepNbMeeting() {
		return (Integer) this.stepNbOfMeeting.getValue();
	}
	
	public Integer getMinNbAgents() {
		return (Integer) this.minNbAgents.getValue();
	}
	
	public Integer getMaxNbAgents() {
		return (Integer) this.maxNbAgents.getValue();
	}
	
	public Integer getStepNbAgents() {
		return (Integer) this.stepNbAgents.getValue();
	}
	
	public Integer getMinMeetingPerAgent() {
		return (Integer) this.minMeetingPerAgent.getValue();
	}
	
	public Integer getMaxMeetingPerAgent() {
		return (Integer) this.maxMeetingPerAgent.getValue();
	}
	
	public Integer getStepMeetingPerAgent() {
		return (Integer) this.stepMeetingPerAgent.getValue();
	}

	public Integer getDuration() {
		return (Integer) this.duration.getValue();
	}

	public Integer getNbDays() {
		return (Integer) this.nbDays.getValue();
	}

	public Integer getNbHours() {
		return (Integer) this.nbHours.getValue();
	}

	public Integer getMinGridSize() {
		return (Integer) this.minGridSize.getValue();
	}

	public Integer getMaxGridSize() {
		return (Integer) this.maxGridSize.getValue();
	}

	public Integer getStepGridSize() {
		return (Integer) this.stepGridSize.getValue();
	}

	public Integer getDisponibility() {
		return (Integer) this.disponibility.getValue();
	}
	
}
