package gui.generators;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import util.Commands;

@SuppressWarnings("serial")
public class GeneratorsPanel extends JPanel {

	public static final String RANDOM_DISCSP = Commands.URDisCSP;
	public static final String DCSP_MEETING_SCHEDULING = Commands.DMSP;
	public static final String SENSOR_NETWORK = Commands.DSNP;
	public static final String GRAPH_COLORING = Commands.URGColoring;
	public static final String NQUEEN = Commands.DisNQueen;
	//------------------------------------------------------------------------
	public static final String RANDOM_DCOP = Commands.URDCOP;
	public static final String RANDOM_MaxDisCSP = Commands.URMaxDisCSP;
	public static final String DCOP_MEETING_SCHEDULING = Commands.DIMES;
	public static final String SOFT_GRAPH_COLORING = Commands.SoftGraphColoring;
	//------------------------------------------------------------------------
	public static final String RANDOM_COMPLEX_DISCSP = Commands.URComplexDisCSP;
	public static final String COMPLEX_GRAPH_COLORING = Commands.URComplexGColoring;
	//------------------------------------------------------------------------
	public static final String RANDOM_COMPLEX_DCOP = Commands.URComplexDCOP;

	private JPanel westPanel = null;
	private JPanel centerPanel = null;
	private JList listOfGenerators = null;

	/**
	 * This is the default constructor
	 * @param disChoco 
	 * 
	 * @param aPanel
	 * @param panel
	 * @param disChocoXP
	 */
	public GeneratorsPanel(JFrame disChoco) {
		super();
		this.setLayout(new BorderLayout());
		this.add(createWestPanel(), BorderLayout.WEST);
		this.add(createCenterPanel(disChoco), BorderLayout.CENTER);
		this.setMinimumSize(new Dimension(1024, 520));
		this.setMaximumSize(new Dimension(1024, 520));
		this.setPreferredSize(new Dimension(1024, 520));
		this.setBorder(BorderFactory.createEtchedBorder());
	}
	
	/**
	 * This method initializes the West Panel
	 * 
	 * @return javax.swing.JPanel
	 */
	public JPanel createWestPanel() {
		westPanel = new JPanel(new BorderLayout());
		westPanel.setBorder(BorderFactory.createEmptyBorder(6, 6, 3, 3));
		DefaultListModel lm = new DefaultListModel();

		lm.addElement(RANDOM_DISCSP); 		// 1. "Random DisCSP"
		lm.addElement(DCSP_MEETING_SCHEDULING); 			// 2. "Distributed Meeting Scheduling"
		lm.addElement(SENSOR_NETWORK); 			// 3. "Distributed Sensor Networking"
		lm.addElement(GRAPH_COLORING);	// 4. "Random Graph Coloring";
		lm.addElement(NQUEEN);		// 5. "Distributed N-Queen"
		//------------------------------------------------------------
		lm.addElement(RANDOM_DCOP);				// 6. "Random DCOP"
		lm.addElement(RANDOM_MaxDisCSP);				// 7. "Random Max-DisCSP"
		lm.addElement(DCOP_MEETING_SCHEDULING);				// 8. "DiMES"
		lm.addElement(SOFT_GRAPH_COLORING);				// 9. "Soft GC"
		//------------------------------------------------------------
		lm.addElement(RANDOM_COMPLEX_DISCSP);	// 9. "Random Complex DisCSP"
		lm.addElement(COMPLEX_GRAPH_COLORING);	// 10. "Complex Graph Coloring"
		//------------------------------------------------------------
		lm.addElement(RANDOM_COMPLEX_DCOP);		// 11. "Random Complex DCOP"	
		
		listOfGenerators = new JList(lm);
		listOfGenerators.setFont(new Font("TimesRoman", Font.PLAIN, 13));
//		listOfGenerators.setForeground(Color.BLUE);
//		listOfGenerators.setBackground(westPanel.getBackground());
		listOfGenerators.setSelectedIndex(0);
		listOfGenerators.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listOfGenerators.setLayoutOrientation(JList.VERTICAL);
		listOfGenerators.setFixedCellWidth(250);
		listOfGenerators.setFixedCellHeight(22);
		this.listOfGenerators.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				String selected = (String) ((JList) e.getSource()).getSelectedValue();
				selectGenerator(selected);
			}
		});
		westPanel.setBorder(BorderFactory.createTitledBorder("List Of Generators"));
		JScrollPane westcontent = new JScrollPane(listOfGenerators);
		westcontent.setBorder(null);
		westPanel.add(westcontent, BorderLayout.CENTER);
		return westPanel;
	}

	/**
	 * This method initializes the Center Panel
	 * 
	 * @return javax.swing.JPanel
	 */
	public JPanel createCenterPanel(JFrame disChoco) {
		centerPanel = new JPanel();
		centerPanel.setLayout(new BorderLayout());
		centerPanel = new JPanel(new CardLayout());
		JPanel discsp = new RandomDCSPPanel(disChoco);
		discsp.setBorder(BorderFactory.createTitledBorder(Commands.URDisCSP_LONG));
		centerPanel.add(discsp, RANDOM_DISCSP);
		
		JPanel meeting = new DCSPMeetingsSchedulingPanel(disChoco);
		meeting.setBorder(BorderFactory.createTitledBorder(DCSP_MEETING_SCHEDULING));
		centerPanel.add(meeting, DCSP_MEETING_SCHEDULING);
		
		JPanel sensor = new SensorNetworksPanel(disChoco);
		sensor.setBorder(BorderFactory.createTitledBorder(SENSOR_NETWORK));
		centerPanel.add(sensor, SENSOR_NETWORK);
		JPanel gc = new GraphColoringPanel(disChoco);
		gc.setBorder(BorderFactory.createTitledBorder(GRAPH_COLORING));
		centerPanel.add(gc, GRAPH_COLORING);
		JPanel nqueen = new NQueenPanel(disChoco);
		nqueen.setBorder(BorderFactory.createTitledBorder(NQUEEN));
		centerPanel.add(nqueen, NQUEEN);
		//------------------------------------------------------------
		JPanel dcop = new RandomDCOPPanel(disChoco);
		dcop.setBorder(BorderFactory.createTitledBorder(Commands.URDCOP_LONG));
		centerPanel.add(dcop, RANDOM_DCOP);
		
		JPanel maxdiscsp = new RandomMaxDisCSPPanel(disChoco);
		maxdiscsp.setBorder(BorderFactory.createTitledBorder(Commands.URMaxDisCSP_LONG));
		centerPanel.add(maxdiscsp, RANDOM_MaxDisCSP);

		JPanel dcopMSP = new DCOPMeetingsSchedulingPanel(disChoco);
		dcopMSP.setBorder(BorderFactory.createTitledBorder(Commands.DIMES_LONG));
		centerPanel.add(dcopMSP, DCOP_MEETING_SCHEDULING);
		
		JPanel sgcp = new SoftGraphColoringPanel(disChoco);
		sgcp.setBorder(BorderFactory.createTitledBorder(Commands.URSGCP_LONG));
		centerPanel.add(sgcp, SOFT_GRAPH_COLORING);
		//------------------------------------------------------------
		JPanel complex_discsp = new RandomDCSPComplexPanel(disChoco);
		complex_discsp.setBorder(BorderFactory.createTitledBorder(Commands.URComplexDisCSP_LONG));
		centerPanel.add(complex_discsp, RANDOM_COMPLEX_DISCSP);
		JPanel complex_gc = new GraphColoringPanel(disChoco);
		complex_gc.setBorder(BorderFactory.createTitledBorder(COMPLEX_GRAPH_COLORING));
		centerPanel.add(complex_gc, COMPLEX_GRAPH_COLORING);
		//------------------------------------------------------------
		JPanel complex_dcop = new RandomDCOPComplexPanel(disChoco);
		complex_dcop.setBorder(BorderFactory.createTitledBorder(Commands.URComplexDCOP_LONG));
		centerPanel.add(complex_dcop, RANDOM_COMPLEX_DCOP);
		//------------------------------------------------------------
		return centerPanel;
	}

	public void selectGenerator(String selected) {
		CardLayout cards = (CardLayout) centerPanel.getLayout();
		cards.show(centerPanel, selected);
		listOfGenerators.setSelectedValue(selected, true);
//		centerPanel.repaint();
	}
}
