package gui.generators;

import static simulation.parser.InstanceTokens.EXTENSION;
import static simulation.parser.InstanceTokens.INTENSION;
import gui.LabelAccessory;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import util.Commands;
import benchmark.nqueen.NQueenGenerator;
import dischoco.DisChoco;

@SuppressWarnings("serial")
public class NQueenPanel extends JPanel {
	
	public static final String TOTALLY_KNOWN_CONSTRAINT = Commands.TOTALLY_KNOWN_CONSTRAINT;
	public static final String PARTIALLY_KNOWN_CONSTRAINT = Commands.PARTIALLY_KNOWN_CONSTRAINT;
	public static final String SUCCEFULLY = Commands.SUCCEFULLY;
	public static final String BROWSE = Commands.BROWSE;
	public static final String GENERATE = Commands.GENERATE;
	public static final String NB_QUEENS = "Number of Queens";
	public static final String FROM = Commands.FROM;
	public static final String TO = Commands.TO;
	public static final String STEP = Commands.STEP;
	
	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JPanel parametersPanel = null;
	private JButton browseButton;
	private JButton generateButton;
	private JFileChooser fileChooser = null;
	private File file = null;
	private JFrame mainFrame = null;
	private JComboBox constraintsType;
	private JSpinner minQueen;
	private JSpinner maxQueen;
	private JSpinner stepQueen;
	
	/**
	 * Create the panel.
	 */
	public NQueenPanel(JFrame dischoco) {
		super(new BorderLayout());
		this.mainFrame = dischoco;
		this.setPreferredSize(new Dimension(600, 400));
		this.add(createNorthPanel(), BorderLayout.NORTH);
		this.add(createCenterPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes northPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createNorthPanel() {
		northPanel = new JPanel(new FlowLayout());
		browseButton = new JButton();
		browseButton.setText(BROWSE);
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				fileChooser = new JFileChooser(DisChoco.filePath);
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fileChooser.setAccessory(new LabelAccessory(fileChooser));
				int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getSelectedFile();
					if(!file.isDirectory()) {
						file = fileChooser.getCurrentDirectory();
					}
					DisChoco.filePath = file.getAbsolutePath();
					System.out.println("Path : " + file.getAbsolutePath() + "\n");
				}
			}
		});
		northPanel.add(browseButton);
		
		generateButton = new JButton();
		generateButton.setText(GENERATE);
		generateButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (file == null) {
					JOptionPane.showMessageDialog(mainFrame, "Choose firstly a directoty", "DisChoco", JOptionPane.WARNING_MESSAGE);
					return;
				}
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							int minQ = getMinNbQueen();
							int maxQ = getMaxNbQueen();
							int stepQ = getStepNbQueen();
							assert minQ < maxQ : "The min must be less or equal the max";
							NQueenGenerator nQueen = new NQueenGenerator(file, minQ, maxQ, stepQ);
							nQueen.setType(getConstraintsType());
							nQueen.generate();
							JOptionPane.showMessageDialog(mainFrame, SUCCEFULLY);
							file = null;
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		northPanel.add(generateButton);
		
		return northPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createCenterPanel() {
		centerPanel = new JPanel(new FlowLayout());
		centerPanel.add(createParametersPanel());
		return centerPanel;
	}


	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createParametersPanel() {
		parametersPanel = new JPanel(null);
		parametersPanel.setPreferredSize(new Dimension(600, 350));
		parametersPanel.setMinimumSize(new Dimension(600, 350));
		parametersPanel.setMaximumSize(new Dimension(600, 350));
		
		JLabel lblType = new JLabel("Type of constraints", JLabel.CENTER);
		lblType.setBounds(new Rectangle(215, 20, 150, 22));
		parametersPanel.add(lblType); 

		String[] types = {EXTENSION, INTENSION};
		constraintsType = new JComboBox(types);
		constraintsType.setSelectedItem(EXTENSION);
		constraintsType.setBounds(new Rectangle(225, 55, 120, 24)); 
		parametersPanel.add(constraintsType); 
		
		JLabel lblFrom = new JLabel(FROM);
		lblFrom.setBounds(new Rectangle(215, 165, 50, 22));
		lblFrom.setHorizontalAlignment(JLabel.CENTER);
		parametersPanel.add(lblFrom);
		
		JLabel lblTo = new JLabel(TO);
		lblTo.setBounds(new Rectangle(395, 165, 50, 22));
		lblTo.setHorizontalAlignment(JLabel.CENTER);
		parametersPanel.add(lblTo);
		
		JLabel lblStep = new JLabel(STEP);
		lblStep.setBounds(new Rectangle(305, 165, 50, 22));
		lblStep.setHorizontalAlignment(JLabel.CENTER);
		parametersPanel.add(lblStep);
		
		JLabel lblP1 = new JLabel(NB_QUEENS);
		lblP1.setBounds(new Rectangle(65, 200, 140, 22));
		lblP1.setHorizontalAlignment(JLabel.RIGHT);
		parametersPanel.add(lblP1);
		
		minQueen = new JSpinner(new SpinnerNumberModel(10, 4, 500, 1));
		minQueen.setBounds(new Rectangle(210, 200, 60, 22));
		parametersPanel.add(minQueen);
		
		maxQueen = new JSpinner(new SpinnerNumberModel(15, 4, 1000, 1));
		maxQueen.setBounds(new Rectangle(390, 200, 60, 22));
		parametersPanel.add(maxQueen);
	
		stepQueen = new JSpinner(new SpinnerNumberModel(1, 1, 500, 1));
		stepQueen.setBounds(new Rectangle(300, 200, 60, 22));
		parametersPanel.add(stepQueen);
		
		return parametersPanel;
	}
	
	public Integer getMinNbQueen() {
		return (Integer) this.minQueen.getValue();
	}
	
	public Integer getMaxNbQueen() {
		return (Integer) this.maxQueen.getValue();
	}
	
	public Integer getStepNbQueen() {
		return (Integer) this.stepQueen.getValue();
	}
	
	public String getConstraintsType() {
		return (String) this.constraintsType.getSelectedItem();
	}
}
