package gui.generators;

import gui.LabelAccessory;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import util.Commands;
import dischoco.DisChoco;

@SuppressWarnings("serial")
public class RandomDCOPComplexPanel extends JPanel {

	public static final String TOTALLY_KNOWN_CONSTRAINT = Commands.TOTALLY_KNOWN_CONSTRAINT;
	public static final String PARTIALLY_KNOWN_CONSTRAINT = Commands.PARTIALLY_KNOWN_CONSTRAINT;
	public static final String SUCCEFULLY = Commands.SUCCEFULLY;
	public static final String BROWSE = Commands.BROWSE;
	public static final String GENERATE = Commands.GENERATE;
	public static final String CONSTRAINT_MODEL = Commands.CONSTRAINT_MODEL;
	public static final String NB_INSTANCES = Commands.NumberOfInstances;
	public static final String NB_AGENTS = Commands.NumberOfAgents;
	public static final String DOMAIN_SIZE = Commands.SizeOfDomain;
	public static final String FROM = Commands.FROM;
	public static final String TO = Commands.TO;
	public static final String STEP = Commands.STEP;
	public static final String CONSTRAINTS_TIGHTNESS = Commands.CONSTRAINTS_TIGHTNESS;
	public static final String NB_VAR_PER_AGENT = "Number of Var per Agent";
	public static final String INTER_AGENT_DENSITY = "Inter-agent Connection Density";
	public static final String DENSITY_OF_CONNECTION = "Density of Connections";
	public static final String INTRA_AGENT_DENSITY = "Intra-agent Density";
	public static final String MINIMUM_COST = Commands.MINIMUM_COST;
	public static final String MAXIMUM_COST = Commands.MAXIMUM_COST;

	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JPanel parametersPanel = null;
	private JComboBox constraintsModel = null;
	private JButton browseButton;
	private JButton generateButton;
	private JFileChooser fileChooser = null;
	private File file = null;
	private JFrame mainFrame = null;
	private JSpinner minP1 = null;
	private JSpinner maxP1 = null;
	private JSpinner stepP1 = null;
	private JSpinner minP2 = null;
	private JSpinner maxP2 = null;
	private JSpinner stepP2 = null;
	private JSpinner nbInstances = null;
	private JSpinner nbAgents = null;
	private JSpinner nbVarsPerAgent = null;
	private JSpinner domainSize = null;
	private JSpinner minCost = null;
	private JSpinner maxCost = null;
	private JSpinner minIntraAgentDensity = null;
	private JSpinner maxIntraAgentDensity = null;
	private JSpinner stepIntraAgentDensity = null;
	private JSpinner minConnextionDensity = null;
	private JSpinner maxConnextionDensity = null;
	private JSpinner stepConnextionDensity = null;

	
	/**
	 * Create the panel.
	 */
	public RandomDCOPComplexPanel(JFrame dischoco) {
		super(new BorderLayout());
		this.mainFrame = dischoco;
		this.setPreferredSize(new Dimension(600, 400));
		this.add(createNorthPanel(), BorderLayout.NORTH);
		this.add(createCenterPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes northPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createNorthPanel() {
		northPanel = new JPanel(new FlowLayout());
		browseButton = new JButton();
		browseButton.setText(BROWSE);
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				fileChooser = new JFileChooser(DisChoco.filePath);
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fileChooser.setAccessory(new LabelAccessory(fileChooser));
				int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getSelectedFile();
					if(!file.isDirectory()) {
						file = fileChooser.getCurrentDirectory();
					}
					DisChoco.filePath = file.getAbsolutePath();
					System.out.println("Path : " + file.getAbsolutePath() + "\n");
				}
			}
		});
		northPanel.add(browseButton);
		
		generateButton = new JButton();
		generateButton.setText(GENERATE);
		generateButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (file == null) {
					JOptionPane.showMessageDialog(mainFrame, "Choose firstly a directoty", "DisChoco", JOptionPane.WARNING_MESSAGE);
					return;
				}
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
//							Integer i =  (Integer) (nbInstances.getValue());
//							Integer n = (Integer) (nbAgents.getValue());
//							Integer d = (Integer) (domainSize.getValue());
//							Integer min1 = (Integer) (minP1.getValue());
//							Integer max1 = (Integer) (maxP1.getValue());
//							Integer step1 = (Integer) (stepP1.getValue());
//							Integer min2 = (Integer) (minP2.getValue());
//							Integer max2 = (Integer) (maxP2.getValue());
//							Integer step2 = (Integer) (stepP2.getValue());
//							Integer minC = (Integer) (minCost.getValue());
//							Integer maxC = (Integer) (maxCost.getValue());
//							
//							String model = (String) constraintsModel.getSelectedItem();
//							boolean tkc = model.equals(TKC);
//							DCOPGenerator g = new DCOPGenerator(file, tkc, i, n, d, 
//									min1, max1, step1, min2, max2, step2, minC, maxC);
//							g.generate();
//							System.out.println(SUCCEFULLY);
							//JOptionPane.showMessageDialog(mainFrame, SUCCEFULLY);
							//file = null;
							System.out.println("Not yet implemented");
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		northPanel.add(generateButton);
		
		return northPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createCenterPanel() {
		centerPanel = new JPanel(new FlowLayout());
		centerPanel.add(createParametersPanel());
		return centerPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createParametersPanel() {
		parametersPanel = new JPanel(null);
		parametersPanel.setPreferredSize(new Dimension(600, 350));
		parametersPanel.setMinimumSize(new Dimension(600, 350));
		parametersPanel.setMaximumSize(new Dimension(600, 350));

		JLabel lblConstraintModel = new JLabel(CONSTRAINT_MODEL, JLabel.RIGHT);
		lblConstraintModel.setBounds(new Rectangle(150, 15, 120, 22));
		parametersPanel.add(lblConstraintModel);
		
		String[] models = {TOTALLY_KNOWN_CONSTRAINT, PARTIALLY_KNOWN_CONSTRAINT};
		constraintsModel = new JComboBox(models);
		constraintsModel.setSelectedItem(TOTALLY_KNOWN_CONSTRAINT);
		constraintsModel.setBounds(new Rectangle(280, 15, 210, 24));
		parametersPanel.add(constraintsModel);

		JLabel lblNbInstances = new JLabel(NB_INSTANCES, JLabel.RIGHT);
		lblNbInstances.setBounds(new Rectangle(75, 60, 160, 22));
		parametersPanel.add(lblNbInstances);
		
		nbInstances = new JSpinner(new SpinnerNumberModel(25, 1, 100, 1));
		nbInstances.setBounds(new Rectangle(240, 60, 60, 22));
		parametersPanel.add(nbInstances);
		
		JLabel lblNbAgent = new JLabel(NB_AGENTS, JLabel.RIGHT);
		lblNbAgent.setBounds(new Rectangle(330, 60, 120, 22));
		parametersPanel.add(lblNbAgent);

		nbAgents = new JSpinner(new SpinnerNumberModel(5, 1, 50, 1));
		nbAgents.setBounds(new Rectangle(455, 60, 60, 22));
		parametersPanel.add(nbAgents);
		
		JLabel lblNbVarsPerAgent = new JLabel(NB_VAR_PER_AGENT, JLabel.RIGHT);
		lblNbVarsPerAgent.setBounds(new Rectangle(75, 100, 160, 22));
		parametersPanel.add(lblNbVarsPerAgent);

		nbVarsPerAgent = new JSpinner(new SpinnerNumberModel(2, 1, 10, 1));
		nbVarsPerAgent.setBounds(new Rectangle(240, 100, 60, 22));
		parametersPanel.add(nbVarsPerAgent);
		
		JLabel lblDomainSize = new JLabel(DOMAIN_SIZE, JLabel.RIGHT);
		lblDomainSize.setBounds(new Rectangle(75, 140, 160, 22));
		parametersPanel.add(lblDomainSize);
		
		domainSize = new JSpinner(new SpinnerNumberModel(10, 2, 100, 1));
		domainSize.setBounds(new Rectangle(240, 140, 60, 22));
		parametersPanel.add(domainSize);

		JLabel lblMinCost = new JLabel(MINIMUM_COST);
		lblMinCost.setHorizontalAlignment(SwingConstants.RIGHT);
		lblMinCost.setBounds(330, 100, 120, 22);
		parametersPanel.add(lblMinCost);

		minCost = new JSpinner(new SpinnerNumberModel(0, 0, 1000, 10));
		minCost.setBounds(455, 100, 60, 22);
		parametersPanel.add(minCost);

		JLabel lblMaxCost = new JLabel(MAXIMUM_COST);
		lblMaxCost.setHorizontalAlignment(SwingConstants.RIGHT);
		lblMaxCost.setBounds(330, 140, 120, 22);
		parametersPanel.add(lblMaxCost);

		maxCost = new JSpinner(new SpinnerNumberModel(100, 0, 10000, 10));
		maxCost.setBounds(455, 140, 60, 22);
		parametersPanel.add(maxCost);
		
		JLabel lblFrom = new JLabel(FROM, JLabel.CENTER);
		lblFrom.setBounds(new Rectangle(270, 175, 50, 25));
		parametersPanel.add(lblFrom);
		
		JLabel lblTo = new JLabel(TO, JLabel.CENTER);
		lblTo.setBounds(new Rectangle(365, 175, 50, 25));
		parametersPanel.add(lblTo);
		
		JLabel lblStep = new JLabel(STEP, JLabel.CENTER);
		lblStep.setBounds(new Rectangle(455, 175, 50, 25));
		parametersPanel.add(lblStep);
		
		JLabel lblLocalDensity = new JLabel(INTRA_AGENT_DENSITY+" (%)", JLabel.RIGHT);
		lblLocalDensity.setBounds(new Rectangle(35, 205, 230, 20));
		parametersPanel.add(lblLocalDensity);

		minP1 = new JSpinner(new SpinnerNumberModel(20, 1, 100, 1));
		minP1.setBounds(new Rectangle(270, 205, 60, 22));
		parametersPanel.add(minP1);

		maxP1 = new JSpinner(new SpinnerNumberModel(80, 1, 100, 1));
		maxP1.setBounds(new Rectangle(365, 205, 60, 22));
		parametersPanel.add(maxP1);
		
		stepP1 = new JSpinner(new SpinnerNumberModel(10, 1, 100, 1));
		stepP1.setBounds(new Rectangle(455, 205, 60, 22));
		parametersPanel.add(stepP1);

		JLabel lblConnections = new JLabel(INTER_AGENT_DENSITY+" (%)", JLabel.RIGHT);
		lblConnections.setBounds(new Rectangle(35, 235, 230, 22));
		parametersPanel.add(lblConnections);

		minIntraAgentDensity = new JSpinner(new SpinnerNumberModel(40, 1, 100, 1));
		minIntraAgentDensity.setBounds(new Rectangle(270, 235, 60, 22));
		parametersPanel.add(minIntraAgentDensity);
		
		maxIntraAgentDensity = new JSpinner(new SpinnerNumberModel(70, 1, 100, 1));
		maxIntraAgentDensity.setBounds(new Rectangle(365, 235, 60, 22));
		parametersPanel.add(maxIntraAgentDensity);

		stepIntraAgentDensity = new JSpinner(new SpinnerNumberModel(30, 1, 100, 1));
		stepIntraAgentDensity.setBounds(new Rectangle(455, 235, 60, 22));
		parametersPanel.add(stepIntraAgentDensity);		
		
		JLabel lblConstraintPerConnection = new JLabel(DENSITY_OF_CONNECTION+" (%)", JLabel.RIGHT);
		lblConstraintPerConnection.setBounds(new Rectangle(35, 265, 230, 22));
		parametersPanel.add(lblConstraintPerConnection);

		minConnextionDensity = new JSpinner(new SpinnerNumberModel(40, 1, 100, 1));
		minConnextionDensity.setBounds(new Rectangle(270, 265, 60, 22));
		parametersPanel.add(minConnextionDensity);

		maxConnextionDensity = new JSpinner(new SpinnerNumberModel(70, 1, 100, 1));
		maxConnextionDensity.setBounds(new Rectangle(365, 265, 60, 22));
		parametersPanel.add(maxConnextionDensity);

		stepConnextionDensity = new JSpinner(new SpinnerNumberModel(30, 1, 100, 1));
		stepConnextionDensity.setBounds(new Rectangle(455, 265, 60, 22));
		parametersPanel.add(stepConnextionDensity);
		
		JLabel lblThigthness = new JLabel(CONSTRAINTS_TIGHTNESS+" (%)", JLabel.RIGHT);
		lblThigthness.setBounds(new Rectangle(35, 295, 230, 20));
		parametersPanel.add(lblThigthness);

		minP2 = new JSpinner(new SpinnerNumberModel(10, 1, 100, 1));
		minP2.setBounds(new Rectangle(270, 295, 60, 22));
		parametersPanel.add(minP2);
		
		maxP2 = new JSpinner(new SpinnerNumberModel(90, 1, 100, 1));
		maxP2.setBounds(new Rectangle(365, 295, 60, 22));
		parametersPanel.add(maxP2);

		stepP2 = new JSpinner(new SpinnerNumberModel(10, 1, 100, 1));
		stepP2.setBounds(new Rectangle(455, 295, 60, 22));
		parametersPanel.add(stepP2);
		
		return parametersPanel;
	}
	
	public String getConstraintsModel() {
		String model = (String) constraintsModel.getSelectedItem();
		if(model.equals(TOTALLY_KNOWN_CONSTRAINT)) {
			return Commands.TKC;
		}
		return Commands.PKC;
	}
	
}
