package gui.generators;

import gui.LabelAccessory;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import util.Commands;
import benchmark.random.DCOPGenerator;
import dischoco.DisChoco;

@SuppressWarnings("serial")
public class RandomDCOPPanel extends JPanel {

	public static final String TOTALLY_KNOWN_CONSTRAINT = Commands.TOTALLY_KNOWN_CONSTRAINT;
	public static final String PARTIALLY_KNOWN_CONSTRAINT = Commands.PARTIALLY_KNOWN_CONSTRAINT;
	public static final String SUCCEFULLY = Commands.SUCCEFULLY;
	public static final String BROWSE = Commands.BROWSE;
	public static final String GENERATE = Commands.GENERATE;
	public static final String CONSTRAINT_MODEL = Commands.CONSTRAINT_MODEL;
	public static final String NB_INSTANCES = Commands.NumberOfInstances;
	public static final String NB_AGENTS = Commands.NumberOfAgents;
	public static final String DOMAIN_SIZE = Commands.SizeOfDomain;
	public static final String FROM = Commands.FROM;
	public static final String TO = Commands.TO;
	public static final String STEP = Commands.STEP;
	public static final String DENSITY_OF_GRAPH = Commands.DENSITY_OF_GRAPH;
	public static final String CONSTRAINTS_TIGHTNESS = Commands.CONSTRAINTS_TIGHTNESS;
	public static final String MINIMUM_COST = Commands.MINIMUM_COST;
	public static final String MAXIMUM_COST = Commands.MAXIMUM_COST;

	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JPanel parametersPanel = null;
	private JComboBox constraintsModel = null;
	private JButton browseButton;
	private JButton generateButton;
	private JFileChooser fileChooser = null;
	private File file = null;
	private JFrame mainFrame = null;
	private JSpinner minP1 = null;
	private JSpinner maxP1 = null;
	private JSpinner stepP1 = null;
	private JSpinner nbInstances = null;
	private JSpinner nbAgents = null;
	private JSpinner domainSize = null;
	private JSpinner minCost = null;
	private JSpinner maxCost = null;

	/**
	 * Create the panel.
	 */
	public RandomDCOPPanel(JFrame dischoco) {
		super(new BorderLayout());
		this.mainFrame = dischoco;
		this.setPreferredSize(new Dimension(600, 400));
		this.add(createNorthPanel(), BorderLayout.NORTH);
		this.add(createCenterPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes northPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel createNorthPanel() {
		northPanel = new JPanel(new FlowLayout());
		browseButton = new JButton();
		browseButton.setText(BROWSE);
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				fileChooser = new JFileChooser(DisChoco.filePath);
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fileChooser.setAccessory(new LabelAccessory(fileChooser));
				int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getSelectedFile();
					if(!file.isDirectory()) {
						file = fileChooser.getCurrentDirectory();
					}
					DisChoco.filePath = file.getAbsolutePath();
					System.out.println("Path : " + file.getAbsolutePath() + "\n");
				}
			}
		});
		northPanel.add(browseButton);

		generateButton = new JButton();
		generateButton.setText(GENERATE);
		generateButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (file == null) {
					JOptionPane.showMessageDialog(mainFrame, "Choose firstly a directoty", "DisChoco",
							JOptionPane.WARNING_MESSAGE);
					return;
				}
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							Integer nbInstances = getNbInstances();
							Integer nbAgents = getNbAgents();
							Integer domainSize = getDomainSize();
							Integer minP1 = getMinP1();
							Integer maxP1 = getMaxP1();
							Integer stepP1 = getStepP1();
							Integer minCost = getMinCost();
							Integer maxCost = getMaxCost();
							String constraintModel = getConstraintsModel();
							DCOPGenerator g = new DCOPGenerator(file, constraintModel, 
									nbInstances, nbAgents, domainSize, 
									minP1, maxP1, stepP1, 
									minCost, maxCost);
							g.generate();
							JOptionPane.showMessageDialog(mainFrame, SUCCEFULLY);
							file = null;
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		northPanel.add(generateButton);

		return northPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel createCenterPanel() {
		centerPanel = new JPanel(new FlowLayout());
		centerPanel.add(createParametersPanel());
		return centerPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel createParametersPanel() {
		parametersPanel = new JPanel(null);

		parametersPanel.setPreferredSize(new Dimension(600, 350));
		parametersPanel.setMinimumSize(new Dimension(600, 350));
		parametersPanel.setMaximumSize(new Dimension(600, 350));

		JLabel lblConstraintModel = new JLabel(CONSTRAINT_MODEL, JLabel.RIGHT);
		lblConstraintModel.setBounds(new Rectangle(150, 15, 120, 22));
		parametersPanel.add(lblConstraintModel);
		
		String[] models = {TOTALLY_KNOWN_CONSTRAINT, PARTIALLY_KNOWN_CONSTRAINT};
		constraintsModel = new JComboBox(models);
		constraintsModel.setSelectedItem(TOTALLY_KNOWN_CONSTRAINT);
		constraintsModel.setBounds(new Rectangle(280, 15, 210, 24));
		parametersPanel.add(constraintsModel);

		JLabel lblNbInstances = new JLabel(NB_INSTANCES, JLabel.RIGHT);
		lblNbInstances.setBounds(new Rectangle(200, 65, 150, 22));
		parametersPanel.add(lblNbInstances);

		nbInstances = new JSpinner(new SpinnerNumberModel(25, 1, 200, 1));
		nbInstances.setBounds(new Rectangle(360, 65, 60, 22));
		parametersPanel.add(nbInstances);

		JLabel lblNbAgents = new JLabel(NB_AGENTS, JLabel.RIGHT);
		lblNbAgents.setBounds(new Rectangle(200, 110, 150, 22));
		parametersPanel.add(lblNbAgents);

		nbAgents = new JSpinner(new SpinnerNumberModel(20, 5, 50, 1));
		nbAgents.setBounds(new Rectangle(360, 110, 60, 22));
		parametersPanel.add(nbAgents);

		JLabel lblDomainSize = new JLabel(DOMAIN_SIZE, JLabel.RIGHT);
		lblDomainSize.setBounds(new Rectangle(200, 150, 150, 22));
		parametersPanel.add(lblDomainSize);

		domainSize = new JSpinner(new SpinnerNumberModel(10, 2, 30, 1));
		domainSize.setBounds(new Rectangle(360, 150, 60, 22));
		parametersPanel.add(domainSize, null);

		JLabel lblMinCost = new JLabel(MINIMUM_COST, JLabel.RIGHT);
		lblMinCost.setBounds(100, 255, 150, 22);
		parametersPanel.add(lblMinCost);

		minCost = new JSpinner(new SpinnerNumberModel(0, 0, 1000, 1));
		minCost.setBounds(255, 255, 60, 22);
		parametersPanel.add(minCost);

		JLabel lblMaxCost = new JLabel(MAXIMUM_COST, JLabel.RIGHT);
		lblMaxCost.setBounds(330, 255, 100, 22);
		parametersPanel.add(lblMaxCost);

		maxCost = new JSpinner(new SpinnerNumberModel(100, 0, 10000, 10));
		maxCost.setBounds(430, 255, 60, 22);
		parametersPanel.add(maxCost);

		JLabel lblFrom = new JLabel(FROM, JLabel.CENTER);
		lblFrom.setBounds(new Rectangle(260, 180, 50, 22));
		parametersPanel.add(lblFrom);

		JLabel lblTo = new JLabel(TO, JLabel.CENTER);
		lblTo.setBounds(new Rectangle(435, 180, 50, 22));
		parametersPanel.add(lblTo);

		JLabel lblStep = new JLabel(STEP, JLabel.CENTER);
		lblStep.setBounds(new Rectangle(350, 180, 50, 22));
		parametersPanel.add(lblStep);
		
		JLabel lblP1 = new JLabel(DENSITY_OF_GRAPH+" (%)", JLabel.RIGHT);
		lblP1.setBounds(new Rectangle(100, 215, 150, 22));
		parametersPanel.add(lblP1);
		
		minP1 = new JSpinner(new SpinnerNumberModel(20, 1, 100, 1));
		minP1.setBounds(new Rectangle(255, 215, 60, 22));
		parametersPanel.add(minP1);
		
		maxP1 = new JSpinner(new SpinnerNumberModel(80, 1, 100, 1));
		maxP1.setBounds(new Rectangle(430, 215, 60, 22));
		parametersPanel.add(maxP1);
	
		stepP1 = new JSpinner(new SpinnerNumberModel(30, 1, 100, 1));
		stepP1.setBounds(new Rectangle(345, 215, 60, 22));
		parametersPanel.add(stepP1);

		return parametersPanel;
	}
	
	public Integer getNbInstances() {
		return (Integer) this.nbInstances.getValue();
	}
	
	public Integer getNbAgents() {
		return (Integer) this.nbAgents.getValue();
	}
	
	public Integer getDomainSize() {
		return (Integer) this.domainSize.getValue();
	}
	
	public Integer getMinP1() {
		return (Integer) this.minP1.getValue();
	}
	
	public Integer getMaxP1() {
		return (Integer) this.maxP1.getValue();
	}
	
	public Integer getStepP1() {
		return (Integer) this.stepP1.getValue();
	}
	
	public Integer getMinCost() {
		return (Integer) this.minCost.getValue();
	}
	
	public Integer getMaxCost() {
		return (Integer) this.maxCost.getValue();
	}
	
	public String getConstraintsModel() {
		String model = (String) constraintsModel.getSelectedItem();
		if(model.equals(TOTALLY_KNOWN_CONSTRAINT)) {
			return Commands.TKC;
		}
		return Commands.PKC;
	}

}
