package gui.generators;

import gui.LabelAccessory;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import util.Commands;
import benchmark.random.ComplexDisCSPGenerator;
import dischoco.DisChoco;

@SuppressWarnings("serial")
public class RandomDCSPComplexPanel extends JPanel {

	public static final String TOTALLY_KNOWN_CONSTRAINT = Commands.TOTALLY_KNOWN_CONSTRAINT;
	public static final String PARTIALLY_KNOWN_CONSTRAINT = Commands.PARTIALLY_KNOWN_CONSTRAINT;
	public static final String SUCCEFULLY = Commands.SUCCEFULLY;
	public static final String BROWSE = Commands.BROWSE;
	public static final String GENERATE = Commands.GENERATE;
	public static final String CONSTRAINT_MODEL = Commands.CONSTRAINT_MODEL;
	public static final String NB_INSTANCES = Commands.NumberOfInstances;
	public static final String NB_AGENTS = Commands.NumberOfAgents;
	public static final String DOMAIN_SIZE = Commands.SizeOfDomain;
	public static final String FROM = Commands.FROM;
	public static final String TO = Commands.TO;
	public static final String STEP = Commands.STEP;
	public static final String CONSTRAINTS_TIGHTNESS = Commands.CONSTRAINTS_TIGHTNESS;
	public static final String NB_VAR_PER_AGENT = "Number of Var per Agent";
	public static final String INTER_AGENT_DENSITY = "Inter-agent Connection Density";
	public static final String DENSITY_OF_CONNECTION = "Density of Connections";
	public static final String INTRA_AGENT_DENSITY = "Intra-agent Density";
	

	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JPanel parametersPanel = null;
	private JComboBox constraintsModel = null;
	private JButton browseButton;
	private JButton generateButton;
	private JFileChooser fileChooser = null;
	private File file = null;
	private JFrame mainFrame = null;
	private JSpinner minP1 = null;
	private JSpinner maxP1 = null;
	private JSpinner stepP1 = null;
	private JSpinner minP2 = null;
	private JSpinner maxP2 = null;
	private JSpinner stepP2 = null;
	private JSpinner nbInstances = null;
	private JSpinner nbAgents = null;
	private JSpinner nbVarsPerAgent = null;
	private JSpinner domainSize = null;
	private JSpinner minIntraAgentDensity = null;
	private JSpinner maxIntraAgentDensity = null;
	private JSpinner stepIntraAgentDensity = null;
	private JSpinner minConnextionDensity = null;
	private JSpinner maxConnextionDensity = null;
	private JSpinner stepConnextionDensity = null;

	
	/**
	 * Create the panel.
	 */
	public RandomDCSPComplexPanel(JFrame dischoco) {
		super(new BorderLayout());
		this.mainFrame = dischoco;
		this.setPreferredSize(new Dimension(600, 400));
		this.add(createNorthPanel(), BorderLayout.NORTH);
		this.add(createCenterPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes northPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createNorthPanel() {
		northPanel = new JPanel(new FlowLayout());
		browseButton = new JButton();
		browseButton.setText(BROWSE);
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				fileChooser = new JFileChooser(DisChoco.filePath);
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fileChooser.setAccessory(new LabelAccessory(fileChooser));
				int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getSelectedFile();
					if(!file.isDirectory()) {
						file = fileChooser.getCurrentDirectory();
					}
					DisChoco.filePath = file.getAbsolutePath();
					System.out.println("Path : " + file.getAbsolutePath() + "\n");
				}
			}
		});
		northPanel.add(browseButton);
		
		generateButton = new JButton();
		generateButton.setText(GENERATE);
		generateButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (file == null) {
					JOptionPane.showMessageDialog(mainFrame, "Choose firstly a directoty", "DisChoco", JOptionPane.WARNING_MESSAGE);
					return;
				}
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {

							Integer nbInstances = getNbInstances();
							Integer nbAgents = getNbAgents();
							Integer nbVarsPerAgent = getNbVarsPerAgent();
							Integer domainSize = getDomainSize();
							Integer minP1 = getMinP1();
							Integer maxP1 = getMaxP1();
							Integer stepP1 = getStepP1();
							Integer minP2 = getMinP2();
							Integer maxP2 = getMaxP2();
							Integer stepP2 = getStepP2();
							String constraintsModel = getConstraintsModel();
							Integer minIntra = getMinIntraAgentDensity();
							Integer maxIntra = getMaxIntraAgentDensity();
							Integer stepIntra = getStepIntraAgentDensity();
							Integer minConnection = getMinConnextionDensity();
							Integer maxConnection = getMaxConnextionDensity();
							Integer stepConnection = getStepConnextionDensity();
							ComplexDisCSPGenerator g = new ComplexDisCSPGenerator(file, constraintsModel, 
									nbInstances, nbAgents, nbVarsPerAgent, domainSize, 
									minIntra, maxIntra, stepIntra, 
									minConnection, maxConnection, stepConnection,
									minP1, maxP1, stepP1, 
									minP2, maxP2, stepP2);
							g.generate();
							JOptionPane.showMessageDialog(mainFrame, SUCCEFULLY);
							file = null;
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		northPanel.add(generateButton);
		return northPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createCenterPanel() {
		centerPanel = new JPanel(new FlowLayout());
		centerPanel.add(createParametersPanel());
		return centerPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createParametersPanel() {
		parametersPanel = new JPanel(null);
		parametersPanel.setPreferredSize(new Dimension(600, 350));
		parametersPanel.setMinimumSize(new Dimension(600, 350));
		parametersPanel.setMaximumSize(new Dimension(600, 350));

		JLabel lblConstraintModel = new JLabel(CONSTRAINT_MODEL, JLabel.RIGHT);
		lblConstraintModel.setBounds(new Rectangle(150, 15, 120, 22));
		parametersPanel.add(lblConstraintModel);
		
		String[] models = {TOTALLY_KNOWN_CONSTRAINT, PARTIALLY_KNOWN_CONSTRAINT};
		constraintsModel = new JComboBox(models);
		constraintsModel.setSelectedItem(TOTALLY_KNOWN_CONSTRAINT);
		constraintsModel.setBounds(new Rectangle(280, 15, 210, 24));
		parametersPanel.add(constraintsModel);

		JLabel lblNbInstances = new JLabel(NB_INSTANCES, JLabel.RIGHT);
		lblNbInstances.setBounds(new Rectangle(85, 60, 150, 22));
		parametersPanel.add(lblNbInstances);
		
		nbInstances = new JSpinner(new SpinnerNumberModel(5, 1, 100, 1));
		nbInstances.setBounds(new Rectangle(240, 60, 60, 22));
		parametersPanel.add(nbInstances);
		
		JLabel lblNbAgent = new JLabel(NB_AGENTS, JLabel.RIGHT);
		lblNbAgent.setBounds(new Rectangle(330, 60, 120, 22));
		parametersPanel.add(lblNbAgent);

		nbAgents = new JSpinner(new SpinnerNumberModel(10, 1, 50, 1));
		nbAgents.setBounds(new Rectangle(455, 60, 60, 22));
		parametersPanel.add(nbAgents);
		
		JLabel lblNbVarsPerAgent = new JLabel(NB_VAR_PER_AGENT, JLabel.RIGHT);
		lblNbVarsPerAgent.setBounds(new Rectangle(55, 100, 180, 25));
		parametersPanel.add(lblNbVarsPerAgent);

		nbVarsPerAgent = new JSpinner(new SpinnerNumberModel(4, 1, 10, 1));
		nbVarsPerAgent.setBounds(new Rectangle(240, 100, 60, 22));
		parametersPanel.add(nbVarsPerAgent);
		
		JLabel lblDomainSize = new JLabel(DOMAIN_SIZE, JLabel.RIGHT);
		lblDomainSize.setBounds(new Rectangle(330, 100, 120, 22));
		parametersPanel.add(lblDomainSize);
		
		domainSize = new JSpinner(new SpinnerNumberModel(10, 2, 100, 1));
		domainSize.setBounds(new Rectangle(455, 100, 60, 22));
		parametersPanel.add(domainSize);
		
		JLabel lblFrom = new JLabel(FROM, JLabel.CENTER);
		lblFrom.setBounds(new Rectangle(255, 145, 50, 25));
		parametersPanel.add(lblFrom);
		
		JLabel lblTo = new JLabel(TO, JLabel.CENTER);
		lblTo.setBounds(new Rectangle(455, 145, 50, 25));
		parametersPanel.add(lblTo);
		
		JLabel lblStep = new JLabel(STEP, JLabel.CENTER);
		lblStep.setBounds(new Rectangle(355, 145, 50, 25));
		parametersPanel.add(lblStep);
		
		JLabel lblLocalDensity = new JLabel(INTRA_AGENT_DENSITY+" (%)", JLabel.RIGHT);
		lblLocalDensity.setBounds(new Rectangle(20, 175, 230, 20));
		parametersPanel.add(lblLocalDensity);

		minP1 = new JSpinner(new SpinnerNumberModel(30, 1, 100, 1));
		minP1.setBounds(new Rectangle(255, 175, 60, 22));
		parametersPanel.add(minP1);

		maxP1 = new JSpinner(new SpinnerNumberModel(70, 1, 100, 1));
		maxP1.setBounds(new Rectangle(455, 175, 60, 22));
		parametersPanel.add(maxP1);
		
		stepP1 = new JSpinner(new SpinnerNumberModel(40, 1, 100, 1));
		stepP1.setBounds(new Rectangle(355, 175, 60, 22));
		parametersPanel.add(stepP1);

		JLabel lblConnections = new JLabel(INTER_AGENT_DENSITY+" (%)", JLabel.RIGHT);
		lblConnections.setBounds(new Rectangle(20, 215, 230, 22));
		parametersPanel.add(lblConnections);

		minIntraAgentDensity = new JSpinner(new SpinnerNumberModel(30, 1, 100, 1));
		minIntraAgentDensity.setBounds(new Rectangle(255, 215, 60, 22));
		parametersPanel.add(minIntraAgentDensity);
		
		maxIntraAgentDensity = new JSpinner(new SpinnerNumberModel(70, 1, 100, 1));
		maxIntraAgentDensity.setBounds(new Rectangle(455, 215, 60, 22));
		parametersPanel.add(maxIntraAgentDensity);

		stepIntraAgentDensity = new JSpinner(new SpinnerNumberModel(40, 1, 100, 1));
		stepIntraAgentDensity.setBounds(new Rectangle(355, 215, 60, 22));
		parametersPanel.add(stepIntraAgentDensity);		
		
		JLabel lblConstraintPerConnection = new JLabel(DENSITY_OF_CONNECTION+" (%)", JLabel.RIGHT);
		lblConstraintPerConnection.setBounds(new Rectangle(20, 250, 230, 22));
		parametersPanel.add(lblConstraintPerConnection);

		minConnextionDensity = new JSpinner(new SpinnerNumberModel(30, 1, 100, 1));
		minConnextionDensity.setBounds(new Rectangle(255, 250, 60, 22));
		parametersPanel.add(minConnextionDensity);

		maxConnextionDensity = new JSpinner(new SpinnerNumberModel(70, 1, 100, 1));
		maxConnextionDensity.setBounds(new Rectangle(455, 250, 60, 22));
		parametersPanel.add(maxConnextionDensity);

		stepConnextionDensity = new JSpinner(new SpinnerNumberModel(40, 1, 100, 1));
		stepConnextionDensity.setBounds(new Rectangle(355, 250, 60, 22));
		parametersPanel.add(stepConnextionDensity);
		
		JLabel lblThigthness = new JLabel(CONSTRAINTS_TIGHTNESS+" (%)", JLabel.RIGHT);
		lblThigthness.setBounds(new Rectangle(20, 285, 230, 20));
		parametersPanel.add(lblThigthness);

		minP2 = new JSpinner(new SpinnerNumberModel(10, 1, 100, 1));
		minP2.setBounds(new Rectangle(255, 285, 60, 22));
		parametersPanel.add(minP2);
		
		maxP2 = new JSpinner(new SpinnerNumberModel(90, 1, 100, 1));
		maxP2.setBounds(new Rectangle(455, 285, 60, 22));
		parametersPanel.add(maxP2);

		stepP2 = new JSpinner(new SpinnerNumberModel(20, 1, 100, 1));
		stepP2.setBounds(new Rectangle(355, 285, 60, 22));
		parametersPanel.add(stepP2);
		
		return parametersPanel;
	}
	
	public Integer getNbInstances() {
		return (Integer) this.nbInstances.getValue();
	}
	
	public Integer getNbAgents() {
		return (Integer) this.nbAgents.getValue();
	}
	
	public Integer getNbVarsPerAgent() {
		return (Integer) this.nbVarsPerAgent.getValue();
	}
	
	public Integer getDomainSize() {
		return (Integer) this.domainSize.getValue();
	}
	
	public Integer getMinP1() {
		return (Integer) this.minP1.getValue();
	}
	
	public Integer getMaxP1() {
		return (Integer) this.maxP1.getValue();
	}
	
	public Integer getStepP1() {
		return (Integer) this.stepP1.getValue();
	}
	
	public Integer getMinIntraAgentDensity() {
		return (Integer) this.minIntraAgentDensity.getValue();
	}
	
	public Integer getMaxIntraAgentDensity() {
		return (Integer) this.maxIntraAgentDensity.getValue();
	}
	
	public Integer getStepIntraAgentDensity() {
		return (Integer) this.stepIntraAgentDensity.getValue();
	}
	
	public Integer getMinP2() {
		return (Integer) this.minP2.getValue();
	}
	
	public Integer getMaxP2() {
		return (Integer) this.maxP2.getValue();
	}
	
	public Integer getStepP2() {
		return (Integer) this.stepP2.getValue();
	}
	
	public Integer getMinConnextionDensity() {
		return (Integer) this.minConnextionDensity.getValue();
	}
	
	public Integer getMaxConnextionDensity() {
		return (Integer) this.maxConnextionDensity.getValue();
	}
	
	public Integer getStepConnextionDensity() {
		return (Integer) this.stepConnextionDensity.getValue();
	}
	
	public String getConstraintsModel() {
		String model = (String) constraintsModel.getSelectedItem();
		if(model.equals(TOTALLY_KNOWN_CONSTRAINT)) {
			return Commands.TKC;
		}
		return Commands.PKC;
	}
}
