/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package gui.generators;

import gui.LabelAccessory;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import simulation.parser.InstanceTokens;
import util.Commands;
import benchmark.sensornetwork.SensorDCSPGenerator;
import dischoco.DisChoco;

@SuppressWarnings("serial")
public class SensorNetworksPanel extends JPanel {

	public static final String GRID = Commands.GRID;
	public static final String REAL = Commands.REAL;
	public static final String SUCCEFULLY = Commands.SUCCEFULLY;
	public static final String BROWSE = Commands.BROWSE;
	public static final String GENERATE = Commands.GENERATE;
	public static final String NB_INSTANCES = Commands.NumberOfInstances;
	public static final String NB_SENSORS = Commands.NumberOfSensors;
	public static final String NB_TARGETS = Commands.NumberOfTargets;
	public static final String NB_X_AXIS = Commands.NumberOfXAxis;
	public static final String NB_Y_AXIS = Commands.NumberOfYAxis;
	public static final String FROM = Commands.FROM;
	public static final String TO = Commands.TO;
	public static final String STEP = Commands.STEP;

	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JPanel parametersPanel = null;
	private JButton browseButton;
	private JButton generateButton;
	private JFileChooser fileChooser = null;
	private File file = null;
	private JFrame mainFrame = null;
	private JSpinner nbInstances = null;
	private JSpinner x_axis = null;
	private JSpinner y_axis = null;
	private JSpinner nbTargets = null;
	private JSpinner nbSensors = null;
	private JSpinner minKc = null;
	private JSpinner maxKc = null;
	private JSpinner stepKc = null;
	private JSpinner minKv = null;
	private JSpinner maxKv = null;
	private JSpinner stepKv = null;
	private JSpinner minPc = null;
	private JSpinner maxPc = null;
	private JSpinner stepPc = null;
	private JSpinner minPv = null;
	private JSpinner maxPv = null;
	private JSpinner stepPv = null;
	/** Format: REAL or GRID */
	private JComboBox localLayout = null;
	/** <b>Complex</b>: several variables per agent or<br/> <b>Simple</b>: a variable per agent */
	private JComboBox model = null;
	
	/**
	 * Create the panel.
	 */
	public SensorNetworksPanel(JFrame dischoco) {
		super(new BorderLayout());
		this.mainFrame = dischoco;
		this.setPreferredSize(new Dimension(600, 400));
		this.add(createNorthPanel(), BorderLayout.NORTH);
		this.add(createCenterPanel(), BorderLayout.CENTER);
	}
	
	/**
	 * 
	 * @return
	 */
	private JComboBox createLayouts() {
		String[] layouts = {GRID, REAL};
		localLayout = new JComboBox(layouts);
		localLayout.setSelectedItem(GRID);
		localLayout.setBounds(new Rectangle(165, 5, 120, 25));
		return localLayout;
	}

	/**
	 * 
	 * @return
	 */
	private JComboBox createModel() {
		String[] models = {InstanceTokens.SIMPLE_MODEL, InstanceTokens.COMPLEX_MODEL};
		model = new JComboBox(models);
		model.setSelectedItem(InstanceTokens.COMPLEX_MODEL);
		model.setBounds(new Rectangle(305, 5, 120, 25));
		return model;
	}
	
	/**
	 * This method initializes northPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createNorthPanel() {
		northPanel = new JPanel(new FlowLayout());
		browseButton = new JButton();
		browseButton.setText(BROWSE);
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				fileChooser = new JFileChooser(DisChoco.filePath);
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fileChooser.setAccessory(new LabelAccessory(fileChooser));
				int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getCurrentDirectory();
					DisChoco.filePath = file.getAbsolutePath();
					System.out.println("Path : " + file.getAbsolutePath() + "\n");
				}
			}
		});
		northPanel.add(browseButton);
		
		generateButton = new JButton();
		generateButton.setText(GENERATE);
		generateButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (file == null) {
					JOptionPane.showMessageDialog(mainFrame, "Choose firstly a directoty", "DisChoco", JOptionPane.WARNING_MESSAGE);
					return;
				}
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							Integer nbInstances = getNbInstances();
							Integer x_axis = getX_axis();
							Integer y_axis = getY_axis();
							Integer nbSensors = getNbSensors();
							Integer nbTargets = getNbTargets();
							Integer minKv = getMinKv();
							Integer maxKv = getMaxKv();
							Integer stepKv = getStepKv();
							Integer minKc = getMinKc();
							Integer maxKc = getMaxKc();
							Integer stepKc = getStepKc();
							Integer minPv = getMinPv();
							Integer maxPv = getMaxPv();
							Integer stepPv = getStepPv();
							Integer minPc = getMinPc();
							Integer maxPc = getMaxPc();
							Integer stepPc = getStepPc();
							String layout = getLocalLayout();
							String benchmark = Commands.GSensorDCSP; 
							String model = getModel();
							String constraintModel = InstanceTokens.TKC;
							if(layout.equals(REAL)) {
								benchmark = Commands.SensorDCSP; 
							}
							SensorDCSPGenerator sn = new SensorDCSPGenerator(
									benchmark, model, constraintModel, layout, 
									file, nbInstances, nbSensors, nbTargets, 
									minPv, maxPv, stepPv, 
									minKv, maxKv, stepKv, 
									minPc, maxPc, stepPc, 
									minKc, maxKc, stepKc, 
									x_axis, y_axis);
							sn.generate();
							JOptionPane.showMessageDialog(mainFrame, SUCCEFULLY);
							file = null;
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		northPanel.add(generateButton);
		
		return northPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createCenterPanel() {
		centerPanel = new JPanel(new FlowLayout());
		centerPanel.add(createParametersPanel());
		return centerPanel;
	}


	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createParametersPanel() {
		parametersPanel = new JPanel(null);
		parametersPanel.setPreferredSize(new Dimension(600, 350));
		parametersPanel.setMinimumSize(new Dimension(600, 350));
		parametersPanel.setMaximumSize(new Dimension(600, 350));

		parametersPanel.add(createLayouts());
		parametersPanel.add(createModel());
		
		JLabel lblNbInstance = new JLabel(NB_INSTANCES, JLabel.RIGHT);
		lblNbInstance.setBounds(new Rectangle(200, 40, 150, 22));
		parametersPanel.add(lblNbInstance);
		
		nbInstances = new JSpinner(new SpinnerNumberModel(100, 1, 1000, 1));
		nbInstances.setBounds(new Rectangle(360, 40, 60, 22));
		parametersPanel.add(nbInstances);

		JLabel lblXAxis = new JLabel(NB_X_AXIS, JLabel.LEFT);
		lblXAxis.setBounds(new Rectangle(80, 75, 120, 22));
		parametersPanel.add(lblXAxis);

		x_axis = new JSpinner(new SpinnerNumberModel(5, 1, 100, 1));
		x_axis.setBounds(new Rectangle(210, 75, 60, 22));
		parametersPanel.add(x_axis);

		JLabel lblYAxis = new JLabel(NB_X_AXIS, SwingConstants.RIGHT);
		lblYAxis.setBounds(new Rectangle(315, 75, 120, 22));
		parametersPanel.add(lblYAxis);

		y_axis = new JSpinner(new SpinnerNumberModel(5, 1, 100, 1));
		y_axis.setBounds(new Rectangle(440, 75, 60, 22));
		parametersPanel.add(y_axis);
		
		JLabel lblNbSensors = new JLabel(NB_SENSORS, JLabel.RIGHT);
		lblNbSensors.setBounds(new Rectangle(80, 110, 120, 22));
		parametersPanel.add(lblNbSensors);

		nbSensors = new JSpinner(new SpinnerNumberModel(25, 5, 1000, 1));
		nbSensors.setBounds(new Rectangle(210, 110, 60, 22));
		parametersPanel.add(nbSensors);
		
		JLabel lblNbTargets = new JLabel(NB_TARGETS, JLabel.RIGHT);
		lblNbTargets.setBounds(new Rectangle(315, 110, 120, 22));
		parametersPanel.add(lblNbTargets);
		
		nbTargets = new JSpinner(new SpinnerNumberModel(5, 3, 100, 1));
		nbTargets.setBounds(new Rectangle(440, 110, 60, 22));
		parametersPanel.add(nbTargets);
		
		JLabel lblFrom = new JLabel(FROM, JLabel.CENTER);
		lblFrom.setBounds(new Rectangle(240, 150, 50, 22));
		parametersPanel.add(lblFrom);
		
		JLabel lblTo = new JLabel(TO, JLabel.CENTER);
		lblTo.setBounds(new Rectangle(440, 150, 60, 22));
		parametersPanel.add(lblTo);
		
		JLabel lblStep = new JLabel(STEP, JLabel.CENTER);
		lblStep.setBounds(new Rectangle(339, 150, 60, 22));
		parametersPanel.add(lblStep);
		
		JLabel lblKC = new JLabel("Kc-restricted compatibility", JLabel.RIGHT);
		lblKC.setBounds(new Rectangle(45, 185, 185, 22));
		parametersPanel.add(lblKC);
		
		minKc = new JSpinner(new SpinnerNumberModel(2, 1, 50, 1));
		minKc.setBounds(new Rectangle(235, 185, 60, 22));
		parametersPanel.add(minKc);
		
		maxKc = new JSpinner(new SpinnerNumberModel(2, 2, 50, 1));
		maxKc.setBounds(new Rectangle(440, 185, 60, 22));
		parametersPanel.add(maxKc);
		
		stepKc = new JSpinner(new SpinnerNumberModel(1, 1, 20, 1));
		stepKc.setBounds(new Rectangle(339, 185, 60, 22));
		parametersPanel.add(stepKc);
		
		JLabel lblKV = new JLabel("Kv-restricted visibility", JLabel.RIGHT);
		lblKV.setBounds(new Rectangle(45, 220, 185, 22));
		parametersPanel.add(lblKV);
		
		minKv = new JSpinner(new SpinnerNumberModel(2, 1, 50, 1));
		minKv.setBounds(new Rectangle(235, 220, 60, 22));
		parametersPanel.add(minKv);
		
		maxKv = new JSpinner(new SpinnerNumberModel(2, 2, 50, 1));
		maxKv.setBounds(new Rectangle(440, 220, 60, 22));
		parametersPanel.add(maxKv);
		
		stepKv = new JSpinner(new SpinnerNumberModel(1, 1, 20, 1));
		stepKv.setBounds(new Rectangle(339, 220, 60, 22));
		parametersPanel.add(stepKv);
		
		JLabel Pclabel = new JLabel("Density of communication (Pc)", JLabel.RIGHT);
		Pclabel.setBounds(new Rectangle(35, 255, 195, 22));
		parametersPanel.add(Pclabel);

		minPc = new JSpinner(new SpinnerNumberModel(20, 1, 100, 1));
		minPc.setBounds(new Rectangle(235, 255, 60, 22));
		parametersPanel.add(minPc);

		maxPc = new JSpinner(new SpinnerNumberModel(80, 1, 100, 1));
		maxPc.setBounds(new Rectangle(440, 255, 60, 22));
		parametersPanel.add(maxPc);

		stepPc = new JSpinner(new SpinnerNumberModel(10, 1, 100, 1));
		stepPc.setBounds(new Rectangle(339, 255, 60, 22));
		parametersPanel.add(stepPc);

		JLabel Pvlabel = new JLabel("Density of visibility (Pv)", JLabel.RIGHT);
		Pvlabel.setBounds(new Rectangle(35, 290, 195, 22));
		parametersPanel.add(Pvlabel);		

		minPv = new JSpinner(new SpinnerNumberModel(10, 1, 100, 1));
		minPv.setBounds(new Rectangle(235, 290, 60, 22));
		parametersPanel.add(minPv);
		
		maxPv = new JSpinner(new SpinnerNumberModel(90, 1, 100, 1));
		maxPv.setBounds(new Rectangle(440, 290, 60, 22));
		parametersPanel.add(maxPv);

		stepPv = new JSpinner(new SpinnerNumberModel(5, 1, 100, 1));
		stepPv.setBounds(new Rectangle(339, 290, 60, 22));
		parametersPanel.add(stepPv);
		
		return parametersPanel;
	}
	
	public Integer getNbInstances() {
		return (Integer) this.nbInstances.getValue();
	}
	
	public Integer getNbSensors() {
		return (Integer) this.nbSensors.getValue();
	}

	public Integer getNbTargets() {
		return (Integer) this.nbTargets.getValue();
	}

	public Integer getX_axis() {
		return (Integer) this.x_axis.getValue();
	}
	public Integer getY_axis() {
		return (Integer) this.y_axis.getValue();
	}
	
	public Integer getMinKc() {
		return (Integer) this.minKc.getValue();
	}
	
	public Integer getMaxKc() {
		return (Integer) this.maxKc.getValue();
	}
	
	public Integer getStepKc() {
		return (Integer) this.stepKc.getValue();
	}
	
	public Integer getMinKv() {
		return (Integer) this.minKv.getValue();
	}
	
	public Integer getMaxKv() {
		return (Integer) this.maxKv.getValue();
	}
	
	public Integer getStepKv() {
		return (Integer) this.stepKv.getValue();
	}
	
	public Integer getMinPc() {
		return (Integer) this.minPc.getValue();
	}
	
	public Integer getMaxPc() {
		return (Integer) this.maxPc.getValue();
	}
	
	public Integer getStepPc() {
		return (Integer) this.stepPc.getValue();
	}
	
	public Integer getMinPv() {
		return (Integer) this.minPv.getValue();
	}
	
	public Integer getMaxPv() {
		return (Integer) this.maxPv.getValue();
	}
	
	public Integer getStepPv() {
		return (Integer) this.stepPv.getValue();
	}
	
	public String getLocalLayout() {
		return (String) this.localLayout.getSelectedItem();
	}
	
	public String getModel() {
		return (String) this.model.getSelectedItem();
	}

}
