package gui.generators;

import gui.LabelAccessory;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import simulation.parser.InstanceTokens;
import util.Commands;
import benchmark.graphcoloring.URSoftGraphColoringGenerator;
import dischoco.DisChoco;

@SuppressWarnings("serial")
public class SoftGraphColoringPanel extends JPanel {

	public static final String SIMPLE_MODEL = InstanceTokens.SIMPLE_MODEL;
	public static final String COMPLEX_MODEL = InstanceTokens.COMPLEX_MODEL;
	public static final String TOTALLY_KNOWN_CONSTRAINT = Commands.TOTALLY_KNOWN_CONSTRAINT;
	public static final String PARTIALLY_KNOWN_CONSTRAINT = Commands.PARTIALLY_KNOWN_CONSTRAINT;
	public static final String SUCCEFULLY = Commands.SUCCEFULLY;
	public static final String BROWSE = Commands.BROWSE;
	public static final String GENERATE = Commands.GENERATE;
	public static final String CONSTRAINT_MODEL = Commands.CONSTRAINT_MODEL;
	public static final String NB_INSTANCES = Commands.NumberOfInstances;
	public static final String NB_AGENTS = Commands.NumberOfAgents;
	public static final String DOMAIN_SIZE = Commands.SizeOfDomain;
	public static final String NB_COLORS = Commands.NumberOfColors;
	public static final String FROM = Commands.FROM;
	public static final String TO = Commands.TO;
	public static final String STEP = Commands.STEP;
	public static final String DENSITY_OF_GRAPH = Commands.DENSITY_OF_GRAPH;
	public static final String CONSTRAINTS_TIGHTNESS = Commands.CONSTRAINTS_TIGHTNESS;
	public static final String NB_VAR_PER_AGENT = "Number of Var per Agent";

	private JPanel northPanel = null;
	private JPanel centerPanel = null;
	private JPanel parametersPanel = null;
	private JButton browseButton;
	private JButton generateButton;
	private JFileChooser fileChooser = null;
	private File file = null;
	private JFrame mainFrame = null;
	
	private JSpinner nbInstances = null;
	private JSpinner minNbColors = null;
	private JSpinner maxNbColors = null;
	private JSpinner stepNbColors = null;
	private JSpinner minDensity = null;
	private JSpinner maxDensity = null;
	private JSpinner stepDensity = null;
	private JSpinner minNbAgents = null;
	private JSpinner maxNbAgents = null;
	private JSpinner stepNbAgents = null;
	private JSpinner nbVarsPerAgent = null;
	private JComboBox agentModel = null;
	private JLabel lblNbVarsPerAgent =  null;
	
	/**
	 * Create the panel.
	 */
	public SoftGraphColoringPanel(JFrame dischoco) {
		super(new BorderLayout());
		this.mainFrame = dischoco;
		this.setPreferredSize(new Dimension(500, 425));
		this.add(createNorthPanel(), BorderLayout.NORTH);
		this.add(createCenterPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes northPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createNorthPanel() {
		northPanel = new JPanel(new FlowLayout());
		browseButton = new JButton();
		browseButton.setText(BROWSE);
		browseButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				fileChooser = new JFileChooser(DisChoco.filePath);
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fileChooser.setAccessory(new LabelAccessory(fileChooser));
				int returnVal = fileChooser.showSaveDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					file = fileChooser.getSelectedFile();
					if(!file.isDirectory()) {
						file = fileChooser.getCurrentDirectory();
					}
					DisChoco.filePath = file.getAbsolutePath();
					System.out.println("Path : " + file.getAbsolutePath() + "\n");
				}
			}
		});
		northPanel.add(browseButton);
		
		generateButton = new JButton();
		generateButton.setText(GENERATE);
		generateButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (file == null) {
					JOptionPane.showMessageDialog(mainFrame, "Choose firstly a directoty", "DisChoco", JOptionPane.WARNING_MESSAGE);
					return;
				}
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							int nbInstances = getNbInstances();
							int minNbAgents = getMinNbAgents();
							int maxNbAgents = getMaxNbAgents();
							int stepNbAgents = getStepNbAgents();
							int minNbColors = getMinNbColors();
							int maxNbColors = getMaxNbColors();
							int stepNbColors = getStepNbColors();
							int minDensity = getMinDensity();
							int maxDensity = getMaxDensity();
							int stepDensity = getStepDensity();
							String model = getAgentModel();
							URSoftGraphColoringGenerator gc = new URSoftGraphColoringGenerator(file, nbInstances, model, 
									minNbAgents, stepNbAgents, maxNbAgents, 
									minDensity, stepDensity, maxDensity,
									minNbColors, stepNbColors, maxNbColors);
							if (model.equals(COMPLEX_MODEL)) {
								gc.setNbVarsPerAgent(getNbVarsPerAgent());
							}
							gc.generate();
							JOptionPane.showMessageDialog(mainFrame, SUCCEFULLY);
							file = null;
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		northPanel.add(generateButton);
		
		return northPanel;
	}

	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createCenterPanel() {
		centerPanel = new JPanel(new FlowLayout());
		centerPanel.add(createParametersPanel());
		return centerPanel;
	}


	/**
	 * This method initializes parametersPanel
	 * @return javax.swing.JPanel 
	 */
	protected JPanel createParametersPanel() {
		parametersPanel = new JPanel(null);
		parametersPanel.setPreferredSize(new Dimension(500, 375));
		parametersPanel.setMinimumSize(new Dimension(500, 375));
		parametersPanel.setMaximumSize(new Dimension(500, 375));

		parametersPanel.add(createAgentModel());

		JLabel lblNbInstance = new JLabel(NB_INSTANCES, JLabel.RIGHT);
		lblNbInstance.setBounds(new Rectangle(140, 60, 150, 22));
		parametersPanel.add(lblNbInstance);
		
		nbInstances = new JSpinner(new SpinnerNumberModel(50, 1, 500, 1));
		nbInstances.setBounds(new Rectangle(300, 60, 55, 22));
		parametersPanel.add(nbInstances);
	
		JLabel lblFrom = new JLabel(FROM, JLabel.CENTER);
		lblFrom.setBounds(new Rectangle(190, 105, 55, 22));
		parametersPanel.add(lblFrom);

		JLabel lblTo = new JLabel(TO, JLabel.CENTER);
		lblTo.setBounds(new Rectangle(270, 105, 55, 22));
		parametersPanel.add(lblTo);

		JLabel lblStep = new JLabel(STEP, JLabel.CENTER);
		lblStep.setBounds(new Rectangle(350, 105, 55, 22));
		parametersPanel.add(lblStep);

		JLabel lblDomainSize = new JLabel(NB_COLORS, JLabel.RIGHT);
		lblDomainSize.setBounds(new Rectangle(60, 140, 120, 22));
		parametersPanel.add(lblDomainSize);

		minNbColors = new JSpinner(new SpinnerNumberModel(8, 2, 50, 1));
		minNbColors.setBounds(new Rectangle(190, 140, 60, 22));
		parametersPanel.add(minNbColors);

		maxNbColors = new JSpinner(new SpinnerNumberModel(10, 2, 50, 1));
		maxNbColors.setBounds(new Rectangle(270, 140, 60, 22));
		parametersPanel.add(maxNbColors);

		stepNbColors = new JSpinner(new SpinnerNumberModel(2, 1, 10, 1));
		stepNbColors.setBounds(new Rectangle(350, 140, 60, 22));
		parametersPanel.add(stepNbColors);

		JLabel lblDensity = new JLabel(DENSITY_OF_GRAPH, JLabel.RIGHT);
		lblDensity.setBounds(new Rectangle(60, 180, 120, 22));
		parametersPanel.add(lblDensity);
		
		minDensity = new JSpinner(new SpinnerNumberModel(40, 1, 100, 1));
		minDensity.setBounds(new Rectangle(190, 180, 60, 22));
		parametersPanel.add(minDensity);

		maxDensity = new JSpinner(new SpinnerNumberModel(70, 1, 100, 1));
		maxDensity.setBounds(new Rectangle(270, 180, 60, 22));
		parametersPanel.add(maxDensity);

		stepDensity = new JSpinner(new SpinnerNumberModel(30, 1, 80, 1));
		stepDensity.setBounds(new Rectangle(350, 180, 60, 22));
		parametersPanel.add(stepDensity);

		JLabel lblNbAgents = new JLabel(NB_AGENTS, JLabel.RIGHT);
		lblNbAgents.setBounds(new Rectangle(60, 220, 120, 22));
		parametersPanel.add(lblNbAgents);

		minNbAgents = new JSpinner(new SpinnerNumberModel(10, 5, 20, 1));
		minNbAgents.setBounds(new Rectangle(190, 220, 60, 22));
		parametersPanel.add(minNbAgents);

		maxNbAgents = new JSpinner(new SpinnerNumberModel(20, 5, 1000, 1));
		maxNbAgents.setBounds(new Rectangle(270, 220, 60, 22));
		parametersPanel.add(maxNbAgents);

		stepNbAgents = new JSpinner(new SpinnerNumberModel(5, 1, 100, 1));
		stepNbAgents.setBounds(new Rectangle(350, 220, 60, 22));
		parametersPanel.add(stepNbAgents);
		
		lblNbVarsPerAgent = new JLabel(NB_VAR_PER_AGENT, JLabel.CENTER);
		lblNbVarsPerAgent.setBounds(new Rectangle(150, 270, 200, 22));
		lblNbVarsPerAgent.setVisible(false);
		parametersPanel.add(lblNbVarsPerAgent);
		
		nbVarsPerAgent = new JSpinner(new SpinnerNumberModel(2, 1, 10, 1));
		nbVarsPerAgent.setBounds(new Rectangle(220, 300, 50, 22));
		nbVarsPerAgent.setVisible(false);
		parametersPanel.add(nbVarsPerAgent);
		
		return parametersPanel;
	}


	/**
	 * This method initializes format
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox createAgentModel() {
		agentModel = new JComboBox();
		String[] formats = {SIMPLE_MODEL, COMPLEX_MODEL};
		agentModel = new JComboBox(formats);
		agentModel.setSelectedItem(SIMPLE_MODEL);
		agentModel.setBounds(new Rectangle(180, 20, 150, 25));
		agentModel.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					String selected = (String) e.getItem();
					if (selected.equals(Commands.SIMPLE)) {
						lblNbVarsPerAgent.setVisible(false);
						nbVarsPerAgent.setVisible(false);
					} else if (selected.equals(Commands.COMPLEX)) {
						lblNbVarsPerAgent.setVisible(true);
						nbVarsPerAgent.setVisible(true);
					}
				}
			}
		});
		return agentModel;
	}
	
	public Integer getNbInstances() {
		return (Integer) this.nbInstances.getValue();
	}
	
	public Integer getNbVarsPerAgent() {
		return (Integer) this.nbVarsPerAgent.getValue();
	}
	
	public Integer getMinNbAgents() {
		return (Integer) this.minNbAgents.getValue();
	}
	
	public Integer getMaxNbAgents() {
		return (Integer) this.maxNbAgents.getValue();
	}
	
	public Integer getStepNbAgents() {
		return (Integer) this.stepNbAgents.getValue();
	}
	
	public Integer getMinNbColors() {
		return (Integer) this.minNbColors.getValue();
	}
	
	public Integer getMaxNbColors() {
		return (Integer) this.maxNbColors.getValue();
	}
	
	public Integer getStepNbColors() {
		return (Integer) this.stepNbColors.getValue();
	}
	
	public Integer getMinDensity() {
		return (Integer) this.minDensity.getValue();
	}
	
	public Integer getMaxDensity() {
		return (Integer) this.maxDensity.getValue();
	}
	
	public Integer getStepDensity() {
		return (Integer) this.stepDensity.getValue();
	}
	
	public String getAgentModel() {
		return (String) this.agentModel.getSelectedItem();
	}
	
}
