package gui.graph;

import gui.graph.presentation.GraphPanel;
import gui.graph.presentation.GraphPresentation;
import gui.tree.presentation.TreePanel.Orientation;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import kernel.VarIdentifier;
import util.graph.AbstractGraph;
import util.graph.UndirectedGraph;
import util.graph.Vertex;
/**
 * The Frame for visualizing the graph. 
 * 
 * @author Mohamed Wahbi
 * 
 */
@SuppressWarnings("serial")
public class GraphFrame extends JFrame {

	
	private final static String title = "DisChoco: Visalising constraint graphs";

	private static final String ABOUT_MESSAGE = 
			"<html><p align=center><h3>DisChoco 2.0</h3><br> "
			+ "Version 2.0, Freeware, 01-01-2013<br><br>"
			+ "<hr>This program is designed for creating and editing different graph types.<br>"
			+ "Graphs can be loaded and saved.<br>You can undo or redo up to 10 most recent actions.<br>"
			+ "There several graph algorythms that can be executed on your graph.<br><hr><br> "
			+ "Author: Mohamed Wahbi (<a href=\"mailto:wahbi@lirmm.fr\">wahbi@lirmm.fr</a>)<br> "
			+ "Designed for: </p></html>";
		
	private final GraphPanel<VarIdentifier> graphPanel;
	//private final TreePanel<String> graphPanel;
	private final JComponent mainComponent;
	
	/**
	 * Creates the frame
	 */
	public GraphFrame() {
		super(title);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout());
		Dimension size = new Dimension(800, 500);
//		//TreePresentation<String> graph = makeGraph();
//		//TreePresentation<String> graph = makeGraphPresentation();
		//TreePresentation<String> graph = getOneComponentGraph();
		//this.graphPanel = new TreePanel<String>(graph, size);
		//TreePresentation<IVarIdentifier> graph = getConstraintGraph();
		//this.graphPanel = new TreePanel<IVarIdentifier>(graph, size);
		UndirectedGraph<VarIdentifier> graph = getConstraintGraph();
		System.out.println("cyclic: "+graph.isCyclic());
		//graph = ConstraintGraph.getConstraintGraph();
		GraphPresentation<VarIdentifier> graphPresentation = new GraphPresentation<VarIdentifier>(graph, false, false);
		this.graphPanel = new GraphPanel<VarIdentifier>(graphPresentation, size, Orientation.VERTICAL);
		
		this.mainComponent = new JComponent() {};
		this.mainComponent.setLayout(new BorderLayout());
		this.mainComponent.add(graphPanel, BorderLayout.CENTER);
		setJMenuBar(createMainMenu());
		getContentPane().add(mainComponent, BorderLayout.CENTER);
		setSize(size);
		setPreferredSize(size);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((int)(screenSize.getWidth()-size.getWidth())/2, (int)(screenSize.getHeight()-size.getHeight())/2);
		pack();
		setVisible(true);
	}

	private JMenuBar createMainMenu() {
		JMenuBar mainMenu = new JMenuBar();
		mainMenu.add(createLayoutsMenu());
		mainMenu.add(createConvertionMenu());
		mainMenu.add(createHelpMenu());
		return mainMenu;
	}

	private JMenu createLayoutsMenu() {
		JMenu algorythmMenu = new JMenu("Layouts");  
//		algorythmMenu.add(graphComponent.getLayoutFactory().getPredefinedAction());
//		algorythmMenu.add(graphComponent.getLayoutFactory().getCircleAction());
		return algorythmMenu;
	}

	private JMenu createConvertionMenu() {
		JMenu convertionMenu = new JMenu("Convert");
//		convertionMenu.add(graphComponent.getConvertionActionsFactory().getToWeightedAction());
//		convertionMenu.add(graphComponent.getConvertionActionsFactory().getToUnweightedAction());
//		convertionMenu.add(graphComponent.getConvertionActionsFactory().getToUndirectedAction());
		return convertionMenu;
	}

	private JMenu createHelpMenu() {
		JMenu helpMenu = new JMenu("Help");
		helpMenu.add(new AbstractAction("About") {
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(GraphFrame.this, ABOUT_MESSAGE, "About", JOptionPane.QUESTION_MESSAGE);
			}
		});
		return helpMenu;
	}
	
	public static GraphPresentation<String> makeGraphPresentation() {
		AbstractGraph<String> graph = new UndirectedGraph<String>();
		Vertex<String> A = new Vertex<String>("A");
		Vertex<String> B = new Vertex<String>("B");
		Vertex<String> C = new Vertex<String>("C");
		Vertex<String> D = new Vertex<String>("D");
		Vertex<String> E = new Vertex<String>("E");
		Vertex<String> F = new Vertex<String>("F");
		Vertex<String> G = new Vertex<String>("G");
		Vertex<String> H = new Vertex<String>("H");
		graph.addVertex(A);
		graph.addVertex(B);
		graph.addVertex(C);
		graph.addVertex(D);
		graph.addVertex(E);
		graph.addVertex(F);
		graph.addVertex(G);
		graph.addVertex(H);
		graph.addEdge(A, B);
		graph.addEdge(A, C);
		graph.addEdge(C, D);
		graph.addEdge(D, E);
		graph.addEdge(D, G);
		graph.addEdge(E, G);
		graph.addEdge(H, C);
		graph.addEdge(H, F);
		graph.addEdge(E, F);
		return new GraphPresentation<String>(graph, false, false);
	}
	
	public static GraphPresentation<String> makeGraph(){
		UndirectedGraph<String> graph = new UndirectedGraph<String>();
		// let's throw in a clique, too
		for (int i = 1; i <= 10; i++) {
			for (int j = i + 1; j <= 10; j++) {
				Vertex<String> v1 = new Vertex<String>(Integer.toString(i));
				Vertex<String> v2 = new Vertex<String>(Integer.toString(j));
				graph.addEdge(v1, v2, (int) Math.pow(i+2,j));
			}
		}

		// and, last, a partial clique
		for (int i = 11; i <= 20; i++) {
			for (int j = i + 1; j <= 20; j++) {
				if (Math.random() > 0.6)
					continue;
				Vertex<String> v1 = new Vertex<String>(Integer.toString(i));
				Vertex<String> v2 = new Vertex<String>(Integer.toString(j));
				graph.addEdge(v1, v2, (int) Math.pow(i+2,j));
			}
		}

		ArrayList<Vertex<String>> index = new ArrayList<Vertex<String>>(graph.getVertexes());
		// and one edge to connect them all
		for (int i = 0; i < index.size() - 1; i++) {
			Vertex<String> v1 = index.get(i);
			Vertex<String> v2 = index.get(i+1);
			graph.addEdge(v1, v2, i);
		}
		//System.out.println(graph.toString());
		//System.exit(0);
		return new GraphPresentation<String>(graph, false, false);
	}
	
	public static GraphPresentation<String> getOneComponentGraph() {
		int weight = 0;
		UndirectedGraph<String> g = new UndirectedGraph<String>();
		// let's throw in a clique, too
		for (int i = 1; i <= 10; i++) {
			for (int j = i + 1; j <= 10; j++) {
				String i1 = "" + i;
				String i2 = "" + j;
				g.addEdge(new Vertex<String>(i1), new Vertex<String>(i2), weight++);
			}
		}

		// and, last, a partial clique
		for (int i = 11; i <= 20; i++) {
			for (int j = i + 1; j <= 20; j++) {
				if (Math.random() > 0.3) {
					String i1 = Integer.toString(i);
					String i2 = Integer.toString(j);
					g.addEdge(new Vertex<String>(i1), new Vertex<String>(i2), weight++);
				}
			}
		}

		List<Vertex<String>> index = new ArrayList<Vertex<String>>();
		index.addAll(g.getVertexes());
		// and one edge to connect them all
		for (int i = 0; i < index.size() - 1; i++) 
		    g.addEdge(index.get(i), index.get(i+1), weight++);
		return new GraphPresentation<String>(g, false, false);
	}
	
	public static UndirectedGraph<VarIdentifier> getConstraintGraph(){
		UndirectedGraph<VarIdentifier> graph = new UndirectedGraph<VarIdentifier>();
		VarIdentifier x1 = new VarIdentifier(1, 1);
		VarIdentifier x2 = new VarIdentifier(2, 1);
		VarIdentifier x3 = new VarIdentifier(3, 1);
		VarIdentifier x4 = new VarIdentifier(4, 1);
		VarIdentifier x5 = new VarIdentifier(5, 1);
		graph.addEdge(x1, x2, "C1");
		graph.addEdge(x1, x3, "C2", 0.1);
		graph.addEdge(x1, x5, "C3");
		graph.addEdge(x2, x4, "C4");
		graph.addEdge(x2, x5, "C5");
		graph.addEdge(x3, x4, "C6");
		graph.addEdge(x3, x5, "C7");
		//graph.addEdge(x2, x3, "C8");
		return graph;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
//		Vector<Vertex<String>> vertexes = new Vector<Vertex<String>>();
//		HashSet<String> set = new HashSet<String>();
//		for(int i = 1 ; i < 4 ; i++) {
//			for(int j = i+1 ; j <4 ; j++) {
//				set.add(Integer.toString(i));
//				set.add(Integer.toString(j));
//				Vertex<String> vi = new Vertex<String>(Integer.toString(i));
//				if(!vertexes.contains(vi)) {
//					vertexes.add(vi);
//				}
//				Vertex<String> vj = new Vertex<String>(Integer.toString(j));
//				if(!vertexes.contains(vj)) {
//					vertexes.add(vj);
//				}
//			}
//		}
//		System.out.println(vertexes);
//		System.out.println(set);
		new GraphFrame();
		//TreeFrame frame = new TreeFrame();
		//frame.setVisible(true);
	}

}
