/*
 * Created on Jul 19, 2005
 * Copyright (c) 2005, the JUNG Project and the Regents of the University
 * of California
 * All rights reserved.
 * This software is open-source under the BSD license; see either
 * "license.txt" or
 * http://jung.sourceforge.net/license.txt for a description.
 */
package gui.graph.layout;

import gui.graph.presentation.GraphPresentation;
import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Random;

/**
 * Generates a random location within the bounds of the Dimension property. 
 * 
 * @author Mohamed Wahbi
 * @param <T>
 */
@SuppressWarnings("serial")
public class RandomLocations<T> extends HashMap<T, Point2D> {

	private Dimension size;
	private Random random;

	/**
	 * Creates an instance with the specified size which uses the current time 
	 * as the random seed.
	 */
	public RandomLocations(Collection<T> vertexes, Dimension size) {
		this(vertexes, size, new Date().getTime());
	}

	/**
	 * Creates an instance with the specified dimension and random seed.
	 * @param size
	 * @param seed
	 */
	public RandomLocations(Collection<T> vertexes, final Dimension size, long seed) {
		this.size = size;
		this.random = new Random(seed);
		for(T vertex : vertexes) {
			this.put(vertex, randomLocation(vertex));
		}
	}

	public Point2D randomLocation(T vertex) {
		double x = random.nextDouble() * size.width;
		double y = random.nextDouble() * size.height;
		x = Math.min(x, size.width-GraphPresentation.SHIFT);
		x = Math.max(x, GraphPresentation.SHIFT);
		y = Math.min(y, size.height-GraphPresentation.SHIFT);
		y = Math.max(y, GraphPresentation.SHIFT);
		return new Point2D.Double(x, y);
	}
}
