package gui.results;

import gui.Filter;
import gui.LabelAccessory;
import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.Map.Entry;
import java.util.SortedMap;
import java.util.TreeMap;
import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.renderer.category.LineAndShapeRenderer;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.ui.RectangleEdge;
import util.Commands;
import benchmark.ResultPrinter;
import com.lowagie.text.Document;
import com.lowagie.text.pdf.DefaultFontMapper;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Example of a ResultsPanel. Use the Add button to add entries that will be displayed in the list and the table
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class ResultsPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -961204332596835431L;
	private static final String GIFPICTURE = "GIF picture";
	private static final String BMPPICTURE = "BMP picture";
	private static final String JPGPICTURE = "JPG picture";
	private static final String PNGPICTURE = "PNG picture";
	private static final String PDFPICTURE = "PDF picture";
	private JFrame mainFrame;
	private JScrollPane detailsScrollPane = null;
	private JTable detailsTable = null;
	private TableColumn detailsTableColumn1 = null;
	private TableColumn detailsTableColumn2 = null;
	private TableColumn detailsTableColumn3 = null;
	private JTabbedPane algorithmPane = null;
	private JScrollPane scrollPane = null;
	private JList algorithmList = null;
	private JButton removeButton = null;
	private JButton addButton = null;
	private JPanel principalPanel = null;
	private JPanel westPanel = null;
	private JPanel eastPanel = null;
	private JPanel northEastPanel = null;
	private JPanel southWestPanel = null;
	private JPanel northWestPanel = null;

	private JFileChooser fileChooser;
	private File[] files;
	private String path = ".";

	private ChartPanel chartPanel = null;
	private JFreeChart chart = null;
	private JButton replotButton = null;
	private JButton mergeButton = null;
	private JMenuItem exportAsMenu;
	private JMenuItem JPGMenu;
	private JMenuItem GIFMenu;
	private JMenuItem BMPMenu;
	private JMenuItem PNGMenu;
	private JMenuItem PDFMenu;

	private JList allKeys;
	private JList selectedKeys;
	private JButton selectButton;
	private JButton unselectButton;
	DefaultListModel list1 = new DefaultListModel();
	DefaultListModel list2 = new DefaultListModel();
	public JScrollPane jScrollPane1 = null;
	public JScrollPane jScrollPane2 = null;
	private JDialog choicesDialog;
	private JPanel northPanel;
	private JPanel centerPanel = null;
	private JPanel southPanel = null;
	private JLabel allKeysLabel = null;
	private JLabel selectedKeysLabel = null;
	private JButton browseButton = null;
	private JButton okButton = null;
	private JButton cancelButton = null;
	private String gnuplot = Commands.GNUPLOT;
	private ResultPrinter printer;
	private JTextField gnuplotPath = null;

	public ResultsPanel(JFrame frame, JButton replotButton, JButton mergeButton, JMenuItem exportAsMenuItem) {
		super();
		mainFrame = frame;
		initialize();
		setReplotButton(replotButton);
		setMergeButton(mergeButton);
		setExportAsMenu(exportAsMenuItem);
	}

	/**
	 * Initialize the class.
	 */
	private void initialize() {
		this.setLayout(new BorderLayout());
		this.add(getPrincipalPanel(), BorderLayout.CENTER);
		this.setMinimumSize(new Dimension(1024, 320));
		this.setMaximumSize(new Dimension(1024, 320));
		this.setPreferredSize(new Dimension(1024, 320));
	}

	/**
	 * @param replotButton
	 */
	private void setReplotButton(JButton plotButton) {
		replotButton = plotButton;
		replotButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(java.awt.event.ActionEvent e) {
				plotFigure();
			}
		});
	}

	/**
	 * @param replotButton
	 */
	private void setMergeButton(JButton mButton) {
		mergeButton = mButton;
		mergeButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(java.awt.event.ActionEvent e) {
				mergeResults();
			}
		});
	}

	/**
	 * @param exportAsMenuItem
	 */
	private void setExportAsMenu(JMenuItem exportAsMenu) {
		this.exportAsMenu = exportAsMenu;
		this.exportAsMenu.add(getGIFMenu());
		this.exportAsMenu.add(getBMPMenu());
		this.exportAsMenu.add(getJPGMenu());
		this.exportAsMenu.add(getPNGMenu());
		this.exportAsMenu.add(getPDFMenu());
	}

	private JMenuItem getGIFMenu() {
		if (GIFMenu == null) {
			GIFMenu = new JMenuItem("GIF picture");
			GIFMenu.addActionListener(new ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					saveAs(new String[] {"gif", "GIF"}, GIFPICTURE);
				}
			});
		}
		return GIFMenu;
	}

	private JMenuItem getBMPMenu() {
		if (BMPMenu == null) {
			BMPMenu = new JMenuItem("BMP picture");
			BMPMenu.addActionListener(new ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					saveAs(new String[] {"bmp", "BMP"}, BMPPICTURE);
				}
			});
		}
		return BMPMenu;
	}

	private JMenuItem getJPGMenu() {
		if (JPGMenu == null) {
			JPGMenu = new JMenuItem("JPG picture");
			JPGMenu.addActionListener(new ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					saveAs(new String[] {"jpeg", "JPEG", "jpg", "JPG"}, JPGPICTURE);
				}
			});
		}
		return JPGMenu;
	}

	private JMenuItem getPNGMenu() {
		if (PNGMenu == null) {
			PNGMenu = new JMenuItem("PNG picture");
			PNGMenu.addActionListener(new ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					saveAs(new String[] {"png", "PNG"}, PNGPICTURE);
				}
			});
		}
		return PNGMenu;
	}

	private JMenuItem getPDFMenu() {
		if (PDFMenu == null) {
			PDFMenu = new JMenuItem("PDF picture");
			PDFMenu.addActionListener(new ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					saveAs(new String[] {"pdf", "PDF"}, PDFPICTURE);
				}
			});
		}
		return PDFMenu;
	}

	/**
	 * Return the JFrameContentPane property value.
	 * 
	 * @return JPanel
	 */
	private JPanel getPrincipalPanel() {
		if (principalPanel == null) {
			principalPanel = new JPanel();
			principalPanel.setLayout(new BorderLayout());
			principalPanel.add(getWestPanel(), BorderLayout.WEST);
			principalPanel.add(getEastPanel(), BorderLayout.CENTER);
		}
		return principalPanel;
	}

	/**
	 * This method initializes detailsScrollPane
	 * 
	 * @return JScrollPane
	 */
	private JScrollPane getDetailsScrollPane() {
		if (detailsScrollPane == null) {
			detailsScrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			detailsScrollPane.setViewportView(getDetailsTable());
			detailsScrollPane.setBounds(10, 130, 229, 146);
		}
		return detailsScrollPane;
	}

	/**
	 * Static inner class for the table model
	 */
	public static class ResultsPanelModel extends AbstractTableModel {

		/**
		 * 
		 */
		private static final long serialVersionUID = 2974340238895070918L;
		ArrayList<String[]> data = new ArrayList<String[]>();

		public String getValueAt(int row, int column) {
			return data.get(row)[column];
		}

		public String getPathAt(int row) {
			return data.get(row)[1];
		}

		public String[] getAllPath() {
			String[] paths = new String[data.size()];
			for(int i = 0 ; i < paths.length ; i++) {
				paths[i] = data.get(i)[1];
			}
			return paths;
		}

		public int getColumnCount() {
			return 3;
		}

		public int getRowCount() {
			return data.size();
		}

		public String getColumnName(int c) {
			return ("");
		}

		public void addElement(String[] element) {
			data.add(element);
			fireTableDataChanged();
		}

		public void removeElement(int row) {
			data.remove(row);
			fireTableDataChanged();
		}

		public String getAlgorithmNameAt(int row) {
			return data.get(row)[0];
		}

		public ArrayList<String> getAlgorithmNames() {
			ArrayList<String> list = new ArrayList<String>();
			for(String[] str : this.data) {
				list.add(str[0]);
			}
			return list;
		}
	}

	/**
	 * This method initializes detailsTable
	 * 
	 * @return JTable
	 */
	private JTable getDetailsTable() {
		if (detailsTable == null) {
			detailsTable = new JTable();
			detailsTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			detailsTable.addColumn(getDetailsTableColumn1());
			detailsTable.addColumn(getDetailsTableColumn2());
			detailsTable.addColumn(getDetailsTableColumn3());
			detailsTable.setAutoCreateColumnsFromModel(false);
			detailsTable.setModel(new ResultsPanelModel());
		}
		return detailsTable;
	}

	/**
	 * This method initializes detailsTableColumn1
	 * 
	 * @return table.TableColumn
	 */
	private TableColumn getDetailsTableColumn1() {
		if (detailsTableColumn1 == null) {
			detailsTableColumn1 = new TableColumn();
			detailsTableColumn1.setHeaderValue("Algorithm");
			detailsTableColumn1.setPreferredWidth(100);
			// detailsTableColumn1.setResizable(false);
			detailsTableColumn1.setModelIndex(0);
		}
		return detailsTableColumn1;
	}

	/**
	 * This method initializes detailsTableColumn2
	 * 
	 * @return table.TableColumn
	 */
	private TableColumn getDetailsTableColumn2() {
		if (detailsTableColumn2 == null) {
			detailsTableColumn2 = new TableColumn();
			detailsTableColumn2.setHeaderValue("Absolute Path");
			detailsTableColumn2.setPreferredWidth(250);
			detailsTableColumn2.setModelIndex(1);
		}
		return detailsTableColumn2;
	}

	/**
	 * This method initializes detailsTableColumn3
	 * 
	 * @return table.TableColumn
	 */
	private TableColumn getDetailsTableColumn3() {
		if (detailsTableColumn3 == null) {
			detailsTableColumn3 = new TableColumn();
			detailsTableColumn3.setHeaderValue("Time Added");
			detailsTableColumn3.setPreferredWidth(150);
			detailsTableColumn3.setModelIndex(2);
		}
		return detailsTableColumn3;
	}

	/**
	 * This method initializes algorithmPane
	 * 
	 * @return JTabbedPane
	 */
	private JTabbedPane getAlgorithmPane() {
		if (algorithmPane == null) {
			algorithmPane = new JTabbedPane();
			algorithmPane.addTab("Algorithm List", null, getScrollPane(), null);
			algorithmPane.addTab("Details", null, getDetailsScrollPane(), null);
			algorithmPane.setMinimumSize(new Dimension(350, 280));
			algorithmPane.setMaximumSize(new Dimension(350, 280));
			algorithmPane.setPreferredSize(new Dimension(350, 280));
			algorithmPane.setBorder(BorderFactory.createEtchedBorder());
		}
		return algorithmPane;
	}

	/**
	 * This method initializes scrollPane
	 * 
	 * @return JScrollPane
	 */
	private JScrollPane getScrollPane() {
		if (scrollPane == null) {
			scrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			scrollPane.setViewportView(getAlgorithmList());
		}
		return scrollPane;
	}

	/**
	 * This method initializes algorithmList
	 * 
	 * @return JList
	 */
	private JList getAlgorithmList() {
		if (algorithmList == null) {
			algorithmList = new JList();
			algorithmList.setModel(new DefaultListModel());
		}
		return algorithmList;
	}

	/**
	 * This method initializes westPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getWestPanel() {
		if (westPanel == null) {
			westPanel = new JPanel();
			westPanel.setLayout(new BorderLayout());
			westPanel.setMinimumSize(new Dimension(350, 520));
			westPanel.setMaximumSize(new Dimension(350, 520));
			westPanel.setPreferredSize(new Dimension(350, 520));
			westPanel.add(getSouthWestPanel(), BorderLayout.SOUTH);
			westPanel.add(getNorthWestPanel(), BorderLayout.CENTER);
		}
		return westPanel;
	}

	/**
	 * This method initializes northEastPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getNorthEastPanel() {
		if (northEastPanel == null) {
			northEastPanel = new JPanel();
			Dimension size = new Dimension(674, 280);
			northEastPanel.setMinimumSize(size);
			northEastPanel.setMaximumSize(size);
			northEastPanel.setPreferredSize(size);
			northEastPanel.setBorder(BorderFactory.createEtchedBorder());
		}
		return northEastPanel;
	}

	private void mergeResults() {
		ResultsPanelModel model = (ResultsPanelModel) getDetailsTable().getModel();
		String[] names = new String[model.getRowCount()];
		for(int i = 0 ; i < names.length ; i++) {
			names[i] = model.getPathAt(i);
		}
		printer = new ResultPrinter(names);
		JDialog choicesDialog = getChoicesDialog();
		choicesDialog.pack();
		Point loc = mainFrame.getLocation();
		Point location = mainFrame.getLocation();
		Dimension size = mainFrame.getSize();
		loc.setLocation(size.getWidth() / 2 + location.getX(), size.getHeight() / 2 + location.getY());
		loc.translate(-choicesDialog.getWidth() / 2, -choicesDialog.getHeight() / 2);
		choicesDialog.setLocation(loc);
		choicesDialog.setVisible(true);
	}

	/**
	 * 
	 * @param key
	 */
	public void plotFigure() {
		getChartPanel();
		ResultsPanelModel model = (ResultsPanelModel) getDetailsTable().getModel();
		ResultPrinter printer = new ResultPrinter(model.getAllPath());
		ArrayList<String> listKeys = printer.getKeys();
		String[] keys = new String[listKeys.size()];
		for(int i = 0 ; i < keys.length ; i++) {
			keys[i] = listKeys.get(i);
		}
		String key =
				(String) JOptionPane.showInputDialog(this, "Select what to plot", "DisChoco: Select what to plot", JOptionPane.QUESTION_MESSAGE,
						null, keys, keys[0]);
		if (key == null) {
			return;
		}
		Object[] options = {"Yes", "No"};
		boolean histogram = false;
		int n =
				JOptionPane.showOptionDialog(this, "Would you like to plot histogram?", Commands.DisChoco_TITLE, JOptionPane.YES_NO_OPTION,
						JOptionPane.QUESTION_MESSAGE, null, // do not use a custom Icon
						options, // the titles of buttons
						options[1]); // default button title
		if (n == JOptionPane.YES_OPTION) {
			histogram = true;
		}
		// String[] algoNames = new String[model.getRowCount()];
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();
		for(int i = 0 ; i < model.getRowCount() ; i++) {
			String algoName = model.getAlgorithmNameAt(i).trim();
			SortedMap<String, Double> map = new TreeMap<String, Double>(printer.getData(algoName, key));
			for(Entry<String, Double> entry : map.entrySet()) {
				dataset.addValue(entry.getValue(), algoName, entry.getKey());
			}
		}
		if (histogram) {
			this.plotBarChart(printer.getFixKey(), key, dataset);
		} else {
			this.plotChart(printer.getFixKey(), key, dataset);
		}
		exportAsMenu.setEnabled(true);
		// ////////////////////////////////////////////////////////////////
	}

	/**
	 * 
	 * @param printer
	 * @param key
	 * @param dataset
	 */
	@SuppressWarnings("deprecation")
	private void plotChart(String fixKey, String key, DefaultCategoryDataset dataset) {
		chart = ChartFactory.createLineChart("", fixKey, key, dataset, PlotOrientation.VERTICAL, true, false, false);
		chart.setBackgroundPaint(Color.white);
		chart.setBorderVisible(true);
		chart.getLegend().setPosition(RectangleEdge.TOP);
		chart.setBackgroundPaint(Color.lightGray);
		CategoryPlot plot = (CategoryPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.gray);

		// customise the range axis...
		final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
		rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
		rangeAxis.setAutoRangeIncludesZero(true);

		LineAndShapeRenderer renderer = (LineAndShapeRenderer) plot.getRenderer();
		renderer.setShapesVisible(true);
		renderer.setDrawOutlines(true);
		renderer.setUseFillPaint(true);
		// renderer.setFillPaint(Color.yellow);
		renderer.setSeriesPaint(0, Color.blue);
		renderer.setSeriesPaint(1, Color.red);
		renderer.setSeriesPaint(2, Color.green);
		renderer.setSeriesPaint(3, Color.black);
		renderer.setSeriesPaint(4, Color.orange);
		renderer.setSeriesPaint(5, Color.pink);
		renderer.setSeriesPaint(6, Color.lightGray);
		renderer.setSeriesPaint(7, Color.cyan);
		renderer.setSeriesPaint(8, Color.darkGray);
		renderer.setSeriesPaint(9, Color.gray);
		
		for(int i = 0 ; i < 10 ; i++) {
			if (i % 2 == 0) {
				renderer.setSeriesStroke(i, new BasicStroke(1.5f));
			} else {
				renderer.setSeriesStroke(i, new BasicStroke(2.0f, i % 3, i % 3, 1.0f, new float[] {i * 4.0f, i * 5.0f}, 0.0f));
			}
		}
		chartPanel.setChart(chart);
	}

	/**
	 * 
	 * @param printer
	 * @param key
	 * @param dataset
	 */
	private void plotBarChart(String fixKey, String key, DefaultCategoryDataset dataset) {
		chart = ChartFactory.createBarChart("", // Title
				fixKey, key, dataset, PlotOrientation.VERTICAL, true, false, false);
		chart.setBackgroundPaint(Color.white);
		chart.setBorderVisible(true);
		chart.getLegend().setPosition(RectangleEdge.TOP);
		CategoryPlot plot = chart.getCategoryPlot();
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.black);
		plot.setRangeCrosshairVisible(true);
		BarRenderer renderer = (BarRenderer) plot.getRenderer();
		renderer.setDrawBarOutline(true);
		renderer.setItemMargin(0);
		renderer.setSeriesPaint(0, Color.blue);
		renderer.setSeriesPaint(1, Color.red);
		renderer.setSeriesPaint(2, Color.green);
		renderer.setSeriesPaint(3, Color.black);
		renderer.setSeriesPaint(4, Color.orange);
		renderer.setSeriesPaint(5, Color.pink);
		renderer.setSeriesPaint(6, Color.lightGray);
		renderer.setSeriesPaint(7, Color.cyan);
		renderer.setSeriesPaint(8, Color.darkGray);
		renderer.setSeriesPaint(9, Color.gray);
		final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
		rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
		final CategoryAxis domainAxis = plot.getDomainAxis();
		domainAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_45);
		chartPanel.setChart(chart);
	}

	/**
	 * 
	 */
	private ChartPanel getChartPanel() {
		if (chartPanel == null) {
			chartPanel = new ChartPanel(chart);
			Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
			chartPanel.setPreferredSize(size);
			chartPanel.setPopupMenu(null);
			GroupLayout mainPanelLayout = new GroupLayout(getNorthEastPanel());
			getNorthEastPanel().setLayout(mainPanelLayout);
			mainPanelLayout.setAutoCreateContainerGaps(true);
			mainPanelLayout.setAutoCreateGaps(true);
			GroupLayout.SequentialGroup hGroup = mainPanelLayout.createSequentialGroup();
			hGroup.addComponent(chartPanel);
			mainPanelLayout.setHorizontalGroup(hGroup);
			GroupLayout.SequentialGroup vGroup = mainPanelLayout.createSequentialGroup();
			vGroup.addComponent(chartPanel);
			mainPanelLayout.setVerticalGroup(vGroup);
		}
		return chartPanel;
	}

	/**
	 * This method initializes figurePanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getEastPanel() {
		if (eastPanel == null) {
			eastPanel = new JPanel();
			eastPanel.setLayout(new BorderLayout());
			eastPanel.add(getNorthEastPanel(), BorderLayout.CENTER);
			eastPanel.setMinimumSize(new Dimension(674, 320));
			eastPanel.setMaximumSize(new Dimension(674, 320));
			eastPanel.setPreferredSize(new Dimension(674, 320));
		}
		return eastPanel;
	}

	/**
	 * DOCUMENT ME!
	 */
	public void saveAs(String[] extentions, String description) {
		fileChooser = new JFileChooser(new File(path));
		fileChooser.setFileFilter(new Filter(extentions, description));
		// fileChooser.setFileView(new JavaFileView());
		fileChooser.setAccessory(new LabelAccessory(fileChooser));
		fileChooser.changeToParentDirectory();
		fileChooser.setPreferredSize(new Dimension(660, 440));
		fileChooser.setMultiSelectionEnabled(false);
		if (fileChooser.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
			String fileName = fileChooser.getSelectedFile().getAbsolutePath();
			String fileDesc = fileChooser.getFileFilter().getDescription();
			if (fileDesc.equals(PDFPICTURE)) {
				String ext = ".pdf";
				if (!fileName.endsWith(ext)) {
					fileName += ext;
				}
				saveChartToPDF(fileName);
			} else if (fileDesc.equals(JPGPICTURE)) {
				String ext = "jpg";
				if (!fileName.endsWith(ext)) {
					fileName += "." + ext;
				}
				saveChart(fileName, ext);
			} else if (fileDesc.equals(PNGPICTURE)) {
				String ext = "png";
				if (!fileName.endsWith(ext)) {
					fileName += "." + ext;
				}
				saveChart(fileName, ext);
			} else if (fileDesc.equals(GIFPICTURE)) {
				String ext = "gif";
				if (!fileName.endsWith(ext)) {
					fileName += "." + ext;
				}
				saveChart(fileName, ext);
			} else if (fileDesc.equals(BMPPICTURE)) {
				String ext = "bmp";
				if (!fileName.endsWith(ext)) {
					fileName += "." + ext;
				}
				saveChart(fileName, ext);
			}
		}
	}

	public void saveChart(String fileName, String ext) {
		if (chart != null) {
			int width = chartPanel.getWidth();
			int height = chartPanel.getHeight();
			BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			Graphics2D g2d = bi.createGraphics();
			Rectangle2D r2D = new Rectangle2D.Double(0, 0, width, height);
			chart.draw(g2d, r2D, null);
			g2d.dispose();
			try {
				ImageIO.write(bi, ext, new File(fileName));
				System.err.println("Successfully generating the figure: " + fileName);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void saveChartToPDF(String fileName) {
		if (chart != null) {
			BufferedOutputStream out = null;
			try {
				out = new BufferedOutputStream(new FileOutputStream(fileName));
				int width = 600;
				int height = 400;
				Document document = new Document();
				try {
					PdfWriter writer = PdfWriter.getInstance(document, out);
					document.addAuthor("Mohamed Wahbi");
					document.open();
					PdfContentByte cb = writer.getDirectContent();
					PdfTemplate tp = cb.createTemplate(width, height);
					Graphics2D g2 = tp.createGraphics(width, height, new DefaultFontMapper());
					Rectangle2D r2D = new Rectangle2D.Double(0, 0, width, height);
					chart.draw(g2, r2D, null);
					g2.dispose();
					cb.addTemplate(tp, 0, 0);
				} finally {
					document.close();
					out.close();
					System.err.println("Successfully generating the figure: " + fileName);
				}
			} catch (Exception e) {}
		}
	}

	/**
	 * This method initializes southWestPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getSouthWestPanel() {
		if (southWestPanel == null) {
			southWestPanel = new JPanel();
			southWestPanel.add(getAddButton(), null);
			southWestPanel.add(getRemoveButton(), null);
			southWestPanel.setMinimumSize(new Dimension(350, 40));
			southWestPanel.setMaximumSize(new Dimension(350, 40));
			southWestPanel.setPreferredSize(new Dimension(350, 40));
		}
		return southWestPanel;
	}

	/**
	 * This method initializes northWestPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getNorthWestPanel() {
		if (northWestPanel == null) {
			northWestPanel = new JPanel(new BorderLayout());
			northWestPanel.add(this.getAlgorithmPane(), BorderLayout.CENTER);
			northWestPanel.setMinimumSize(new Dimension(350, 280));
			northWestPanel.setMaximumSize(new Dimension(350, 280));
			northWestPanel.setPreferredSize(new Dimension(350, 280));
		}
		return northWestPanel;
	}

	/**
	 * This method initializes addButton
	 * 
	 * @return javax.swing.JButton
	 */
	public JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton("  add  ");
			addButton.addActionListener(new ActionListener() {

				@SuppressWarnings("resource")
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String description = "Results file (csv)";
					String[] extension = new String[] {"csv"};
					Filter filter = new Filter(extension, description);
					fileChooser = new JFileChooser(path);
					fileChooser.setMultiSelectionEnabled(true);
					fileChooser.setFileFilter(filter);
					// fileChooser.setFileView(new JavaFileView());
					fileChooser.setAccessory(new LabelAccessory(fileChooser));
					fileChooser.changeToParentDirectory();
					fileChooser.setPreferredSize(new Dimension(660, 440));
					Container c = getParent();
					while (!(c instanceof JFrame)) {
						c = c.getParent();
					}
					int returnVal = fileChooser.showOpenDialog(c);
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						files = fileChooser.getSelectedFiles();
						for(int i = 0 ; i < files.length ; i++) {
							String newAlgo = "";
							try {
								String line = new BufferedReader(new FileReader(files[i])).readLine();
								String[] str = line.split(",");
								newAlgo = str[2].trim();
							} catch (Exception exc) {}
							String newFile = files[i].getAbsolutePath();
							String calendar = new GregorianCalendar().getTime().toString();
							String[] element = {newAlgo, newFile, calendar};
							if (!newAlgo.isEmpty()) {
								DefaultListModel listModel = (DefaultListModel) getAlgorithmList().getModel();
								listModel.addElement(newAlgo);
								ResultsPanelModel model = (ResultsPanelModel) getDetailsTable().getModel();
								model.addElement(element);
							}
							System.err.println("add : " + newFile + " where algo=" + newAlgo);
						}
						path = files[0].getPath();
						ResultsPanelModel model = (ResultsPanelModel) getDetailsTable().getModel();
						if (model.getRowCount() > 0) {
							replotButton.setEnabled(true);
							mergeButton.setEnabled(true);
						}
					}
				}
			});
		}
		return addButton;
	}

	/**
	 * This method initializes removeButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getRemoveButton() {
		if (removeButton == null) {
			removeButton = new JButton("remove ");
			removeButton.addActionListener(new java.awt.event.ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					int[] indexes = getAlgorithmList().getSelectedIndices();
					if (indexes.length == 0)
						return;
					ResultsPanelModel model = (ResultsPanelModel) getDetailsTable().getModel();
					for(int i = indexes.length - 1 ; i >= 0 ; i--) {
						((DefaultListModel) getAlgorithmList().getModel()).removeElementAt(indexes[i]);
						String name = model.getAlgorithmNameAt(indexes[i]);
						model.removeElement(indexes[i]);
						System.out.println("remove " + name + " successfully ");
					}
					files = new File[model.getRowCount()];
					for(int i = 0 ; i < model.getRowCount() ; i++) {
						files[i] = new File(model.getPathAt(i));
					}
					if (model.getRowCount() == 0) {
						replotButton.setEnabled(false);
						mergeButton.setEnabled(false);
					}
				}
			});
		}
		return removeButton;
	}

	/**
	 * @param printer 
	 * @return
	 */
	private JDialog getChoicesDialog() {
		if (choicesDialog == null) {
			choicesDialog = new JDialog(mainFrame, true);
			choicesDialog.getContentPane().setLayout(new BorderLayout());
			choicesDialog.getContentPane().add(getNorthPanel(), BorderLayout.NORTH);
			choicesDialog.getContentPane().add(getCenterPanel(), BorderLayout.CENTER);
			choicesDialog.getContentPane().add(getSouthPanel(), BorderLayout.SOUTH);
			choicesDialog.setPreferredSize(new Dimension(500, 400));
			choicesDialog.setResizable(false);
		}
		return choicesDialog;
	}

	/**
	 * @return
	 */
	private JPanel getNorthPanel() {
		if (northPanel == null) {
			northPanel = new JPanel();
			northPanel.setLayout(new FlowLayout());
			northPanel.setPreferredSize(new Dimension(500, 40));
			northPanel.add(new JLabel("gnuplot path :", JLabel.RIGHT));
			northPanel.add(getGnuplotPath());
			northPanel.add(getBrowseButton());
		}
		return northPanel;
	}

	/**
	 * @return
	 */
	private JTextField getGnuplotPath() {
		if (gnuplotPath == null) {
			gnuplotPath = new JTextField();
			gnuplotPath.setText(gnuplot);
			gnuplotPath.setPreferredSize(new Dimension(250, 22));
			gnuplotPath.setEditable(false);
		}
		return gnuplotPath;
	}

	/**
	 * @return
	 */
	private JButton getBrowseButton() {
		if (browseButton == null) {
			browseButton = new JButton("Browse");
			browseButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent arg0) {
					fileChooser = new JFileChooser(new File(gnuplot));
					// fileChooser.setFileView(new JavaFileView());
					fileChooser.setAccessory(new LabelAccessory(fileChooser));
					fileChooser.setPreferredSize(new Dimension(660, 440));
					fileChooser.setMultiSelectionEnabled(false);
					if (fileChooser.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
						gnuplot = fileChooser.getSelectedFile().getAbsolutePath();
						getGnuplotPath().setText(gnuplot);
						printer.setGnuplot(gnuplot);
					}
				}
			});
		}
		return browseButton;
	}

	/**
	 * @param printer 
	 * @return
	 */
	private JPanel getCenterPanel() {
		if (centerPanel == null) {
			allKeysLabel = new JLabel("All statistics", JLabel.CENTER);
			allKeysLabel.setBounds(new Rectangle(15, 10, 180, 24));
			selectedKeysLabel = new JLabel("Selected statistics", JLabel.CENTER);
			selectedKeysLabel.setBounds(new Rectangle(300, 10, 180, 24));
			centerPanel = new JPanel();
			centerPanel.setLayout(null);
			centerPanel.setPreferredSize(new Dimension(500, 300));
			centerPanel.add(getJScrollPane1(), null);
			centerPanel.add(getJScrollPane2(), null);
			centerPanel.add(getSelectButton(), null);
			centerPanel.add(getUnSelectButton(), null);
			centerPanel.add(allKeysLabel, null);
			centerPanel.add(selectedKeysLabel, null);
		}
		return centerPanel;
	}

	/**
	 * @param printer 
	 * @param printer 
	 * @return
	 */
	private JPanel getSouthPanel() {
		if (southPanel == null) {
			southPanel = new JPanel();
			southPanel.setLayout(new FlowLayout()); // Generated
			southPanel.setPreferredSize(new Dimension(500, 40));
			southPanel.add(getOkButton());
			southPanel.add(getCancelButton());
		}
		return southPanel;
	}

	/**
	 * @param printer 
	 * @param printer 
	 * @return
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton("generate plots");
			okButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					plotSelectedKeys();
					choicesDialog.dispose();
				}
			});
		}
		return okButton;
	}

	private void plotSelectedKeys() {
		String[] keys = new String[list2.size()];
		for(int i = 0 ; i < keys.length ; i++) {
			keys[i] = (String) list2.getElementAt(i);
		}
		printer.plot(keys);
	}

	/**
	 * @return
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton("cancel");
			cancelButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					choicesDialog.dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * @param printer 
	 * @return
	 */
	private JButton getSelectButton() {
		if (selectButton == null) {
			selectButton = new JButton("Select");
			selectButton.setBounds(new Rectangle(210, 100, 80, 25));
			selectButton.addActionListener(new java.awt.event.ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					transfert(allKeys, selectedKeys);
				}
			});
		}
		return selectButton;
	}

	/**
	 * @param printer 
	 * @return
	 */
	private JButton getUnSelectButton() {
		if (unselectButton == null) {
			unselectButton = new JButton("Remove");
			unselectButton.setBounds(new Rectangle(210, 160, 80, 25));
			unselectButton.addActionListener(new java.awt.event.ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					transfert(selectedKeys, allKeys);
				}
			});
		}
		return unselectButton;
	}

	public void transfert(JList L1, JList L2) {
		int[] indexes = L1.getSelectedIndices(); // indexes
		if (indexes.length == 0)
			return;
		for(int i = indexes.length - 1 ; i >= 0 ; i--) {
			((DefaultListModel) L2.getModel()).addElement(((DefaultListModel) L1.getModel()).elementAt(indexes[i]));
			((DefaultListModel) L1.getModel()).removeElementAt(indexes[i]);
		}
	}

	/**
	 * This method initializes initialAlgos	
	 * 	
	 * @return javax.swing.JList	
	 */
	public JList getAllKeys(ArrayList<String> keys) {
		if (allKeys == null) {
			allKeys = new JList(list1);
			allKeys.setBounds(new Rectangle(15, 50, 150, 120)); // Generated
			setInitialKeys(keys);
		}
		return allKeys;
	}

	private void setInitialKeys(ArrayList<String> keys) {
		list1.clear();
		list2.clear();
		for(String key : keys) {
			list1.addElement(key);
		}
		list1.removeElement(Commands.MSGs);
		list2.addElement(Commands.MSGs);
		list1.removeElement(Commands.NCCCs);
		list2.addElement(Commands.NCCCs);
		list1.removeElement(Commands.SOLVABLE);
		list2.addElement(Commands.SOLVABLE);
	}

	/**
	 * This method initializes selectedAlgos	
	 * 	
	 * @return javax.swing.JList	
	 */
	public JList getSelectedKeys() {
		if (selectedKeys == null) {
			selectedKeys = new JList(list2);
			selectedKeys.setBounds(new Rectangle(300, 50, 150, 128)); // Generated
		}
		return selectedKeys;
	}

	/**
	 * This method initializes jScrollPane1
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	public JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setMinimumSize(new Dimension(180, 250));
			jScrollPane1.setBounds(new Rectangle(15, 45, 180, 250));
			jScrollPane1.setViewportView(getAllKeys(printer.getKeys()));
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jScrollPane2	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	public JScrollPane getJScrollPane2() {
		if (jScrollPane2 == null) {
			jScrollPane2 = new JScrollPane();
			jScrollPane2.setMinimumSize(new Dimension(180, 250));
			jScrollPane2.setBounds(new Rectangle(300, 45, 180, 250));
			jScrollPane2.setViewportView(getSelectedKeys());
		}
		return jScrollPane2;
	}
}
