/** 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 * 
 * @version 2.0
 * DisChoco
 * SNPanel.java
 */ 
package gui.simulation;

import static util.Commands.DISPONIBILITY;
import static util.Commands.DisponibilityOfAgents;
import static util.Commands.NB_DAYS;
import static util.Commands.NB_HOURS;
import static util.Commands.NB_MEETINGS;
import static util.Commands.NB_MEETINGS_PER_AGENT;
import static util.Commands.NumberOfAgents;
import static util.Commands.NumberOfDays;
import static util.Commands.NumberOfHours;
import static util.Commands.NumberOfMeetings;
import static util.Commands.NumberOfMeetingsPerAgent;
import static util.Commands.NumberOfXAxis;
import static util.Commands.NumberOfYAxis;
import static util.Commands.X_axis;
import static util.Commands.Y_axis;
import static simulation.parser.InstanceTokens.NB_AGENTS;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.File;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.jdom2.Element;

import util.Commands;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 * 
 */

public class SimulationOnMeetingSchedulingPanel extends SimulationOnBenchmarks {


	protected static final long serialVersionUID = -4353556871723921003L;
	protected JLabel typeLabel = null;
	protected JLabel prepocessingLabel = null;
	
	private JLabel nbMeetLabel = null;
	private JLabel nbMeetingsLabel = null;
	private JLabel nbAgLabel = null;
	private JLabel nbAgentsLabel = null;
	private JLabel nbMPALabel = null;
	private JLabel nbMeetingPerAgentLabel = null;
	private JLabel dispALabel = null;
	private JLabel disponibilityOfAgentsLabel = null;
	private JLabel nbDLabel = null;
	private JLabel nbDaysLabel = null;
	private JLabel nbHLabel = null;
	private JLabel nbHoursLabel = null;	
	private JLabel randomDCSPLabel = null;
	private JLabel nbXLabel = null;
	private JLabel nbXAxisLabel = null;
	private JLabel nbYLabel = null;
	private JLabel nbYAxisLabel = null;
	
	private int nbMeetings;
	private int nbAgents;
	private int nbMeetingsPerAgent;
	private int disponibilityOfAgents;
	private int nbDays;
	private int nbHours;
	private int xaxis;
	private int yaxis;
	
	/**
	 * @param file
	 * @param size
	 * @param typeOfVisualization
	 */
	public SimulationOnMeetingSchedulingPanel(File file, Dimension dimension, short typeVisualization) {
		super(file, dimension, typeVisualization);
	}
	
	/**
	 * 
	 */
	public void setPresentation(Element presentation) {
		this.nbMeetings = new Integer(presentation.getAttributeValue(NB_MEETINGS));
		this.nbAgents = new Integer(presentation.getAttributeValue(NB_AGENTS));
		this.nbMeetingsPerAgent = new Integer(presentation.getAttributeValue(NB_MEETINGS_PER_AGENT));
		this.disponibilityOfAgents = new Integer(presentation.getAttributeValue(DISPONIBILITY));
		this.nbDays = new Integer(presentation.getAttributeValue(NB_DAYS));
		this.nbHours = new Integer(presentation.getAttributeValue(NB_HOURS));
		this.xaxis = new Integer(presentation.getAttributeValue(X_axis));
		this.yaxis = new Integer(presentation.getAttributeValue(Y_axis));
	}

	/**
	 * This method initializes propertiesPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel getPropertiesPanel() {
		if (propertiesPanel == null) {
			propertiesPanel = new JPanel(null);
			Dimension d = new Dimension(EAST_PANEL_WIDTH, size.height);
			propertiesPanel.setSize(d);
			propertiesPanel.setPreferredSize(d);
			prepocessingLabel = new JLabel("Prepocessing Ordering", JLabel.CENTER);
			prepocessingLabel.setBounds(new Rectangle(80, 15, 200, 22));
			typeLabel = new JLabel("Type: ", JLabel.RIGHT);
			typeLabel.setBounds(new Rectangle(70, 80, 155, 22));
			randomDCSPLabel = new JLabel(Commands.RMSP, JLabel.LEFT);
			randomDCSPLabel.setBounds(new Rectangle(230, 80, 150, 22));
			nbMeetLabel = new JLabel(NumberOfMeetings+": ", JLabel.RIGHT);
			nbMeetLabel.setBounds(new Rectangle(70, 115, 155, 22));
			nbMeetingsLabel = new JLabel("", JLabel.LEFT);
			nbMeetingsLabel.setBounds(new Rectangle(230, 115, 150, 22));
			nbAgLabel = new JLabel(NumberOfAgents+": ", JLabel.RIGHT);
			nbAgLabel.setBounds(new Rectangle(70, 150, 155, 22));
			nbAgentsLabel = new JLabel("", JLabel.LEFT);
			nbAgentsLabel.setBounds(new Rectangle(230, 150, 150, 22));
			nbMPALabel = new JLabel(NumberOfMeetingsPerAgent+": ", JLabel.RIGHT);
			nbMPALabel.setBounds(new Rectangle(10, 185, 215, 22));
			nbMeetingPerAgentLabel = new JLabel("", JLabel.LEFT);
			nbMeetingPerAgentLabel.setBounds(new Rectangle(230, 185, 150, 22));
			dispALabel = new JLabel(DisponibilityOfAgents+": ", JLabel.RIGHT);
			dispALabel.setBounds(new Rectangle(70, 220, 155, 22));
			disponibilityOfAgentsLabel = new JLabel("", JLabel.LEFT);
			disponibilityOfAgentsLabel.setBounds(new Rectangle(230, 220, 150, 22));
			nbDLabel = new JLabel(NumberOfDays+": ", JLabel.RIGHT);
			nbDLabel.setBounds(new Rectangle(70, 255, 155, 22));
			nbDaysLabel = new JLabel("", JLabel.LEFT);
			nbDaysLabel.setBounds(new Rectangle(230, 255, 150, 22));
			nbHLabel = new JLabel(NumberOfHours+": ", JLabel.RIGHT);
			nbHLabel.setBounds(new Rectangle(70, 290, 155, 22));
			nbHoursLabel = new JLabel("", JLabel.LEFT);
			nbHoursLabel.setBounds(new Rectangle(230, 290, 150, 22));
			nbXLabel = new JLabel(NumberOfXAxis+": ", JLabel.RIGHT);
			nbXLabel.setBounds(new Rectangle(70, 325, 155, 22));
			nbXAxisLabel = new JLabel("", JLabel.LEFT);
			nbXAxisLabel.setBounds(new Rectangle(230, 325, 150, 22));
			nbYLabel = new JLabel(NumberOfYAxis+": ", JLabel.RIGHT);
			nbYLabel.setBounds(new Rectangle(70, 360, 155, 22));
			nbYAxisLabel = new JLabel("", JLabel.LEFT);
			nbYAxisLabel.setBounds(new Rectangle(230, 360, 150, 22));
			propertiesPanel.add(prepocessingLabel);
			propertiesPanel.add(getAgentsOrdering());
			propertiesPanel.add(typeLabel);
			propertiesPanel.add(randomDCSPLabel);
			propertiesPanel.add(nbMeetLabel);
			propertiesPanel.add(nbMeetingsLabel);
			propertiesPanel.add(nbAgLabel);
			propertiesPanel.add(nbAgentsLabel);
			propertiesPanel.add(nbMPALabel);
			propertiesPanel.add(dispALabel);
			propertiesPanel.add(nbMeetingPerAgentLabel);
			propertiesPanel.add(disponibilityOfAgentsLabel);
			propertiesPanel.add(nbDLabel);
			propertiesPanel.add(nbDaysLabel);
			propertiesPanel.add(nbHLabel);
			propertiesPanel.add(nbHoursLabel);
			propertiesPanel.add(nbXLabel);
			propertiesPanel.add(nbXAxisLabel);
			propertiesPanel.add(nbYLabel);
			propertiesPanel.add(nbYAxisLabel);
			nbMeetingsLabel.setText(this.nbMeetings + "");
			nbAgentsLabel.setText(this.nbAgents + "");
			nbMeetingPerAgentLabel.setText(this.nbMeetingsPerAgent + "");
			disponibilityOfAgentsLabel.setText(this.disponibilityOfAgents + " %");
			nbDaysLabel.setText(this.nbDays + "");
			nbHoursLabel.setText(this.nbHours + "");
			nbXAxisLabel.setText(this.xaxis + "");
			nbYAxisLabel.setText(this.yaxis + "");
		}
		return propertiesPanel;
	}
	
}
