/**
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with this program; if not, write to
 * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * @version 2.0 DisChoco RandomDisCSPGeneratorPanel.java
 */
package gui.simulation;

import static simulation.parser.InstanceTokens.NB_AGENTS;
import static simulation.parser.InstanceTokens.NB_CONSTRAINTS;
import static simulation.parser.InstanceTokens.NB_TUPLES;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.File;
import javax.swing.JLabel;
import javax.swing.JPanel;
import org.jdom2.Element;
import util.Commands;
/**
 * @authors <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and <a
 *          href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 */

public class SimulationOnRBDisCSPPanel extends SimulationOnBenchmarks {

	private static final long serialVersionUID = -4353556871723921003L;
	private JLabel typeLabel = null;
	private JLabel prepocessingLabel = null;
	private JLabel randomDCSPLabel = null;
	private JLabel nbAgLabel = null;
	private JLabel nbAgentsLabel = null;
	private JLabel domLabel = null;
	private JLabel domainSizeLabel = null;
	private JLabel cstsdensityLabel = null;
	private JLabel P1Label = null;
	private JLabel cststightnessLabel = null;
	private JLabel P2Label = null;
	private JLabel nbCstsLabel = null;
	private JLabel nbConstrantsLabel = null;
	private JLabel tuplesLabel = null;
	private JLabel nbTuplesLabel = null;
	
	private int nbAgents;
	private int domainSize;
	private double p1;
	private double p2;
	private int nbConstraints;
	private int nbTuples;
	
	/**
	 * Constructor
	 * @param file
	 * @param size
	 * @param typeOfVisualization
	 */
	public SimulationOnRBDisCSPPanel(File file, Dimension dimension, short typeVisualization) {
		super(file, dimension, typeVisualization);
	}

	/**
	 * 
	 */
	protected void setPresentation(Element presentation) {
		this.nbAgents = new Integer(presentation.getAttributeValue(NB_AGENTS));
		this.domainSize = new Integer(presentation.getAttributeValue(Commands.DomainSize));
		this.p1 = new Double(presentation.getAttributeValue(Commands.DENSITY))/100;
		this.p2 = new Double(presentation.getAttributeValue(Commands.TIGHTNESS))/100;
		this.nbConstraints = new Integer(presentation.getAttributeValue(NB_CONSTRAINTS));
		this.nbTuples = new Integer(presentation.getAttributeValue(NB_TUPLES));
	}

	/**
	 * This method initializes propertiesPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel getPropertiesPanel() {
		if (propertiesPanel == null) {
			propertiesPanel = new JPanel(null);
			Dimension d = new Dimension(EAST_PANEL_WIDTH, size.height);
			propertiesPanel.setSize(d);
			propertiesPanel.setPreferredSize(d);
			typeLabel = new JLabel("Type :", JLabel.RIGHT);
			typeLabel.setBounds(new Rectangle(25, 100, 150, 22));
			prepocessingLabel = new JLabel("Prepocessing Ordering", JLabel.CENTER);
			prepocessingLabel.setBounds(new Rectangle(80, 15, 200, 22));
			randomDCSPLabel = new JLabel(Commands.RandomDisCSP, JLabel.LEFT);
			randomDCSPLabel.setBounds(new Rectangle(190, 100, 150, 22));
			nbAgLabel = new JLabel("Number of Agents :", JLabel.RIGHT);
			nbAgLabel.setBounds(new Rectangle(25, 135, 150, 22));
			nbAgentsLabel = new JLabel("", JLabel.LEFT);
			nbAgentsLabel.setBounds(new Rectangle(190, 135, 150, 22));
			domLabel = new JLabel("Domain size :", JLabel.RIGHT);
			domLabel.setBounds(new Rectangle(25, 170, 150, 22));
			domainSizeLabel = new JLabel("", JLabel.LEFT);
			domainSizeLabel.setBounds(new Rectangle(190, 170, 150, 22));
			cstsdensityLabel = new JLabel("Constraint density :", JLabel.RIGHT);
			cstsdensityLabel.setBounds(new Rectangle(25, 205, 150, 22));
			P1Label = new JLabel("", JLabel.LEFT);
			P1Label.setBounds(new Rectangle(190, 205, 150, 22));
			cststightnessLabel = new JLabel("Constraints tightness :", JLabel.RIGHT);
			cststightnessLabel.setBounds(new Rectangle(25, 240, 150, 22));
			P2Label = new JLabel("", JLabel.LEFT);
			P2Label.setBounds(new Rectangle(190, 240, 150, 22));
			nbCstsLabel = new JLabel("Number of constraints :", JLabel.RIGHT);
			nbCstsLabel.setBounds(new Rectangle(25, 275, 150, 22));
			nbConstrantsLabel = new JLabel("", JLabel.LEFT);
			nbConstrantsLabel.setBounds(new Rectangle(190, 275, 150, 22));
			tuplesLabel = new JLabel("Number of tuples (frb) :", JLabel.RIGHT);
			tuplesLabel.setBounds(new Rectangle(25, 310, 150, 22));
			nbTuplesLabel = new JLabel("", JLabel.LEFT);
			nbTuplesLabel.setBounds(new Rectangle(190, 310, 150, 22));
			propertiesPanel.add(prepocessingLabel);
			propertiesPanel.add(getAgentsOrdering());
			propertiesPanel.add(typeLabel);
			propertiesPanel.add(randomDCSPLabel);
			propertiesPanel.add(nbAgLabel);
			propertiesPanel.add(nbAgentsLabel);
			propertiesPanel.add(domLabel);
			propertiesPanel.add(domainSizeLabel);
			propertiesPanel.add(cstsdensityLabel);
			propertiesPanel.add(cststightnessLabel);
			propertiesPanel.add(P1Label);
			propertiesPanel.add(P2Label);
			propertiesPanel.add(nbCstsLabel);
			propertiesPanel.add(nbConstrantsLabel);
			propertiesPanel.add(tuplesLabel);
			propertiesPanel.add(nbTuplesLabel);
			nbAgentsLabel.setText(nbAgents + "");
			domainSizeLabel.setText(domainSize + "");
			P1Label.setText(p1 + "");
			P2Label.setText(p2 + "");
			nbConstrantsLabel.setText(nbConstraints + "");
			nbTuplesLabel.setText(nbTuples + "");
		}
		return propertiesPanel;
	}
}
