/** 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 * 
 * @version 2.0
 * DisChoco
 * SNPanel.java
 */ 
package gui.simulation;

import static util.Commands.GRID;
import static util.Commands.KC;
import static util.Commands.KV;
import static util.Commands.NB_SENSORS;
import static util.Commands.NB_TARGETS;
import static util.Commands.P_C;
import static util.Commands.P_V;
import static util.Commands.REAL;
import static util.Commands.Sensors;
import static util.Commands.Targets;
import static util.Commands.X_axis;
import static util.Commands.Y_axis;
import static simulation.parser.InstanceTokens.ID;
import static simulation.parser.InstanceTokens.MODEL;
import static simulation.parser.InstanceTokens.NAME;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.HashMap;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.jdom2.Element;

import util.Commands;
import util.IntArray;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 * 
 */

public class SimulationOnSensorNetworkPanel extends SimulationOnBenchmarks {
	
	/** 
	 * This program is free software; you can redistribute it and/or modify 
	 * it under the terms of the GNU General Public License as published by 
	 * the Free Software Foundation; either version 2 of the License, or 
	 * (at your option) any later version. 
	 *  
	 * This program is distributed in the hope that it will be useful, 
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
	 * GNU General Public License for more details. 
	 *  
	 * You should have received a copy of the GNU General Public License 
	 * along with this program; if not, write to the Free Software 
	 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
	 * 
	 * @version 2.0
	 * DisChoco
	 * SNPanel.java
	 */ 
	private static final long serialVersionUID = 6052326216599337737L;

	public static final String SensorGrid = "SensorGrid";

	private static RenderingHints hints = null;
	private int x_axis;
	private int y_axis;
	private double unitX = 1;
	private double unitY = 1;
	private int lineWidth = 2;
	private int margin = 50;
	private int nbTargets;
	private int nbSensors;
	private int Kc;
	private int Kv;
	private double Pc;
	private double Pv;
	private JLabel typeLabel;
	private JLabel prepocessingLabel;
	private JLabel sensorDCSPLabel;
	private JLabel nbSLabel;
	private JLabel nbSensorsLabel;
	private JLabel nbTLabel;
	private JLabel nbTargetsLabel;
	private JLabel compatibilityDensityLabel;
	private JLabel PcLabel;
	private JLabel visibilityDensityLabel;
	private JLabel PvLabel;
	private JLabel nbKcLabel;
	private JLabel kcLabel;
	private JLabel nbKvLabel;
	private JLabel kvLabel;
	private List<Element> sensors;
	private List<Element> targets;
	private String model = REAL;
		
	public static Color sensorColor = Color.blue;
	public static Color targetColor = Color.red;
	
	/**
	 * Constructor
	 * @param file
	 * @param dimension
	 * @param typeVisualization
	 */
	public SimulationOnSensorNetworkPanel(File file, Dimension dimension, short typeVisualization) {
		super(file, dimension, typeVisualization);
		hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		hints.add(new RenderingHints( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY));
	}
	
	/**
	 *
	 */
	@Override
	protected JPanel getPropertiesPanel() {
		if (propertiesPanel == null) {
			propertiesPanel = new JPanel(null);
			propertiesPanel.setMinimumSize(new Dimension(350, size.height));
			propertiesPanel.setMaximumSize(new Dimension(350, size.height));
			propertiesPanel.setPreferredSize(new Dimension(350, size.height));
			typeLabel = new JLabel("Type :", JLabel.RIGHT);
			typeLabel.setBounds(new Rectangle(25, 100, 150, 22));
			propertiesPanel.add(typeLabel, null);
			prepocessingLabel = new JLabel("Prepocessing Ordering", JLabel.CENTER);
			prepocessingLabel.setBounds(new Rectangle(80, 15, 200, 22));
			propertiesPanel.add(prepocessingLabel, null);
			propertiesPanel.add(getAgentsOrdering());
			sensorDCSPLabel = new JLabel(Commands.GSensorDCSP, JLabel.LEFT);
			sensorDCSPLabel.setBounds(new Rectangle(190, 100, 150, 22));
			propertiesPanel.add(sensorDCSPLabel, null);
			nbSLabel = new JLabel("Number of Sensors :", JLabel.RIGHT);
			nbSLabel.setBounds(new Rectangle(25, 135, 150, 22));
			propertiesPanel.add(nbSLabel, null);
			nbSensorsLabel = new JLabel(""+this.nbSensors, JLabel.LEFT);
			nbSensorsLabel.setBounds(new Rectangle(190, 135, 150, 22));
			propertiesPanel.add(nbSensorsLabel, null);
			nbTLabel = new JLabel("Number of Targets :", JLabel.RIGHT);
			nbTLabel.setBounds(new Rectangle(25, 170, 150, 22));
			propertiesPanel.add(nbTLabel, null);
			nbTargetsLabel = new JLabel(""+this.nbTargets, JLabel.LEFT);
			nbTargetsLabel.setBounds(new Rectangle(190, 170, 150, 22));
			propertiesPanel.add(nbTargetsLabel, null);
			compatibilityDensityLabel = new JLabel("compatibility density :", JLabel.RIGHT);
			compatibilityDensityLabel.setBounds(new Rectangle(25, 205, 150, 22));
			propertiesPanel.add(compatibilityDensityLabel, null);
			PcLabel = new JLabel(""+this.Pc, JLabel.LEFT);
			PcLabel.setBounds(new Rectangle(190, 205, 150, 22));
			propertiesPanel.add(PcLabel, null);
			visibilityDensityLabel = new JLabel("visibility density :", JLabel.RIGHT);
			visibilityDensityLabel.setBounds(new Rectangle(25, 240, 150, 22));
			propertiesPanel.add(visibilityDensityLabel, null);
			PvLabel = new JLabel(""+this.Pv, JLabel.LEFT);
			PvLabel.setBounds(new Rectangle(190, 240, 150, 22));
			propertiesPanel.add(PvLabel, null);
			nbKcLabel = new JLabel("Kc :", JLabel.RIGHT);
			nbKcLabel.setBounds(new Rectangle(25, 275, 150, 22));
			propertiesPanel.add(nbKcLabel, null);
			kcLabel = new JLabel(""+this.Kc, JLabel.LEFT);
			kcLabel.setBounds(new Rectangle(190, 275, 150, 22));
			propertiesPanel.add(kcLabel, null);
			nbKvLabel = new JLabel("Kv :", JLabel.RIGHT);
			nbKvLabel.setBounds(new Rectangle(25, 310, 150, 22));
			propertiesPanel.add(nbKvLabel, null);
			kvLabel = new JLabel(this.Kv+"", JLabel.LEFT);
			kvLabel.setBounds(new Rectangle(190, 310, 150, 22));
			propertiesPanel.add(kvLabel, null);
		}
		return propertiesPanel;
	}


//	/**
//	 * This method initializes figurePanel
//	 * 
//	 * @return javax.swing.JPanel
//	 */
//	protected JPanel getEastPanel() {
//		if (eastPanel == null) {
//			Dimension d = new Dimension(674, size.height);
//			eastPanel = new JPanel(new CardLayout());
//			eastPanel.setMinimumSize(d);
//			eastPanel.setMaximumSize(d);
//			eastPanel.setPreferredSize(d);
//			eastPanel.setBackground(Color.white);
//			eastPanel.add(getSimulatorPanel(d.width), BorderLayout.NORTH);
//			eastPanel.add(new JScrollPane(new GridSensor(d)), SensorGrid);
//		}
//		return eastPanel;
//	}

	public JComboBox getAgentsOrdering() {
		if (agentsOrdering == null) {
			super.getAgentsOrdering();
			String[] items = {SensorGrid, Commands.LexicoAO, Commands.RandomAO, Commands.MinDomOverDegAO, Commands.TreeAO};
			agentsOrdering = new JComboBox(items);
			agentsOrdering.setBounds(new Rectangle(100, 50, 150, 22));
			agentsOrdering.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					chart = null;
					if (e.getStateChange() == ItemEvent.SELECTED) {
						plotEastPanel((String) e.getItem());
					}
				}
			});
		}
		return agentsOrdering;
	}

	protected void plotEastPanel(String selectedItem) {
		if (selectedItem.equals(SensorGrid)) {
			this.plotSensorGrid();
		} else {
			super.plotEastPanel(selectedItem);
		}
	}

	/**
	 * 
	 */
	private void plotSensorGrid() {
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), SensorGrid);
		this.stop();
	}
	
	public void setPresentation(Element presentation){
		this.nbTargets = new Integer(presentation.getChild(Targets).getAttributeValue(NB_TARGETS));
		this.nbSensors = new Integer(presentation.getChild(Sensors).getAttributeValue(NB_SENSORS));
		this.Kc = new Integer(presentation.getAttributeValue(KC));
		this.Kv = new Integer(presentation.getAttributeValue(KV));
		this.Pc = new Double(presentation.getAttributeValue(P_C));
		this.Pv = new Double(presentation.getAttributeValue(P_V));
		this.x_axis = new Integer(presentation.getAttributeValue(X_axis));
		this.y_axis = new Integer(presentation.getAttributeValue(Y_axis));
		this.model = presentation.getAttributeValue(MODEL);
		sensors = presentation.getChild(Sensors).getChildren();
		targets = presentation.getChild(Targets).getChildren();
	}
	
	class GridSensor extends JPanel implements MouseWheelListener, MouseListener, MouseMotionListener {
		
		private static final long serialVersionUID = 5352838628433546561L;
		public static final double DEFAULT_ZOOM_PERCENTAGE = 1.05;
		private Dimension preferredSize = null;
		private Dimension minimumSize = null;
		private Dimension maximumSize = null; 
		public HashMap<Integer, Sensor> mySensors = null;
		public HashMap<Integer, Target> myTargets = null;
		private Target pick;
		private boolean pickfixed;
		private Rectangle rect;
		
		public GridSensor(Dimension size){
			super();
			minimumSize = new Dimension(size.width-margin/2, size.height-margin);
			preferredSize = minimumSize;
			maximumSize = new Dimension(preferredSize.width*2, preferredSize.height*2);
			mySensors = new HashMap<Integer, Sensor>();
			myTargets = new HashMap<Integer, Target>();
			setBackground(Color.white);
			addMouseWheelListener(this);
			addMouseListener(this);
			setSize(preferredSize);
			createShapes();
		}
		
		public void mouseClicked(MouseEvent e) {}
		public void mouseEntered(MouseEvent e) {}
		public void mouseExited(MouseEvent e) {}
		
		public void mousePressed(MouseEvent e) {
			Point p = e.getPoint();
			addMouseMotionListener(this);
			for(int s : mySensors.keySet()){
				Sensor sensor =  mySensors.get(s);
				if(sensor.contains(p)){
					pick = sensor;
					break;
				}
			}
			for(int t : myTargets.keySet()){
				Target target = myTargets.get(t);
				if(target.contains(p)){
					pick = target;
					break;
				}
			}
			if (pick != null) {
				pickfixed = pick.selected;
				pick.selected = true;
			}
			repaint();
			e.consume();
		}

		public void mouseReleased(MouseEvent e) {
			removeMouseMotionListener(this);
			if (pick != null) {
				pick.selected = pickfixed;
				pick = null;
			}
			repaint();
			e.consume();
		}
		public void mouseDragged(MouseEvent e) {
			if (pick != null) {
//				pick.setCenter(e.getPoint());
			}
			repaint();
			e.consume();
		}
		public void mouseMoved(MouseEvent e) {}
		
		public void mouseWheelMoved(MouseWheelEvent e) {
			updatePreferredSize(e.getWheelRotation(), e.getPoint());
		}
		
		public Dimension getPreferredSize() {
			return preferredSize;
		}

		private void updatePreferredSize(int n, Point p) {
			double d = (double) n * DEFAULT_ZOOM_PERCENTAGE;
			d = (n > 0)? (1 / d) : (-d);
			int w = (int) (getPreferredSize().getWidth() * d);
			int h = (int) (getPreferredSize().getHeight() * d);
			if(w < minimumSize.width){
				preferredSize = minimumSize;
			} else if(w > maximumSize.width){
				preferredSize = maximumSize;
			} else{
				preferredSize = new Dimension(w, h);
			}
			createShapes();
			getParent().doLayout();
			revalidate();
			repaint();
		}

		public void paintComponent(Graphics g) {
			createShapes();
			Graphics2D gr = (Graphics2D) g;
	        gr.setRenderingHints(hints);  
	        gr.setColor(Color.white);
			gr.fillRect(0, 0, getWidth(), getHeight());
			gr.setColor(Color.black);
			if(model.equals(GRID)) drawGrid(gr);
	        gr.setStroke(new BasicStroke(lineWidth));
			gr.draw(rect);
			Font font = gr.getFont();
			font = new Font(font.getName(), Font.BOLD, font.getSize()+1);
			gr.setFont(font);
			FontMetrics fontMetric = gr.getFontMetrics();
			String desc = NB_SENSORS+"="+nbSensors+",  "+NB_TARGETS+"="+nbTargets+",  ";
			desc += KC+"="+Kc+",  "+KV+"="+Kv+",  "+P_C+"="+Pc+",  "+P_V+"="+Pv+",  ";
			desc += X_axis+"="+x_axis+",  "+Y_axis+"="+y_axis;
			gr.drawString(desc, (getWidth()-fontMetric.stringWidth(desc))/2, margin - lineWidth);
			for(int id : myTargets.keySet()){
				myTargets.get(id).paint(gr);
			}
			for(int id : mySensors.keySet()){
		        mySensors.get(id).paint(gr);
			}
			if(pick!=null){
				pick.paint(gr);
			}
			gr.dispose();
		}

		private void createShapes() {
			int m = margin - lineWidth;
			rect = new Rectangle(m, m + m / 2, getWidth() - 2 * m, getHeight() - 2 * m);
			unitX = (rect.width - 2*lineWidth) / (x_axis+1);
			unitY = (rect.height - 2*lineWidth) / (y_axis+1);
			mySensors = new HashMap<Integer, Sensor>();
			myTargets = new HashMap<Integer, Target>();
			for(Element sensor : sensors){
				int id = new Integer(sensor.getAttributeValue(ID));
				String name = sensor.getAttributeValue(NAME);
				IntArray neighbors = IntArray.fromString(sensor.getText());
				double xx = new Double(sensor.getAttributeValue(X_axis));
				double yy = new Double(sensor.getAttributeValue(Y_axis));
				double x = rect.x + lineWidth + unitX + (xx * unitX);
				double y = rect.y + lineWidth + unitY + (yy * unitY);
				mySensors.put(id, new Sensor(id, name, x, y, neighbors));
			}
			for(Element target : targets){
				int id = new Integer(target.getAttributeValue(ID));
				String name = target.getAttributeValue(NAME);
				IntArray neighbors = IntArray.fromString(target.getText());
				double xx = new Double(target.getAttributeValue(X_axis));
				double yy = new Double(target.getAttributeValue(Y_axis));
				double x = rect.x + lineWidth + unitX + (xx * unitX);
				double y = rect.y + lineWidth + unitY + (yy * unitY);
				myTargets.put(id, new Target(id, name, x, y, neighbors));
			}
		}
		
		/**
		 * @param gr
		 */
		private void drawGrid(Graphics2D gr) {
			//========================================================================
	        gr.setStroke(new BasicStroke(0.5f));
			int w = rect.width - 2*lineWidth;
			int h = rect.height- 2*lineWidth;
			int unitX = h / (x_axis+1);
			int unitY = w / (y_axis+1);
			int x = rect.x + lineWidth ;
			int y = rect.y + lineWidth ;
			for (int r = 1 ; r <= x_axis ; r++){
				gr.drawLine(x, y+r* unitX, x+w, y+r * unitX);
			}
			for (int r = 1 ; r <= y_axis ; r++){
				gr.drawLine(x+r * unitY, y, x+r * unitY, y+h);
			}
			//========================================================================
		}

		class Target {
			protected int id;
			protected Shape shape;
			protected String name;
			protected FontMetrics fontMetrics;
			protected int rad;
			protected int len;
			protected int width = 14;
			protected int height = 14;
			protected Color selectedColor = Color.black;
			protected Color selectedFillNeigh = Color.green;
			protected Color selectedFillColor = Color.orange;
			protected Color normalColor = Color.black;
			protected Color normalFillColor = Color.red;
			protected RenderingHints hints;
			protected IntArray neighbors = null;
			protected final float alpha = 0.2f;
			protected boolean selected ;

			protected Composite composite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
			protected Composite defaultComposite;
			
			public Target(int id, String name, double x, double y, IntArray neighbors) {
				hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				hints.add(new RenderingHints( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY));
				this.id = id;
				this.name = name;
				shape = new Rectangle2D.Double(x-width/2, y-height/2, width, height);
				this.neighbors = neighbors;
			}

			public void paint(Graphics g) {
				Graphics2D gr = ((Graphics2D) getGraphics());
				defaultComposite = gr.getComposite();
				fontMetrics = gr.getFontMetrics();
				rad = fontMetrics.getAscent();
				len = fontMetrics.stringWidth(name);
				if(selected){
					paintZone(gr);
					paintNeighbors(gr);
					paintSelected(gr);
				} else {
					paintNormal(gr);
				}
				gr.dispose();
			}

			/**
			 * 
			 */
			private void paintNormal(Graphics2D gr) {
				gr.setRenderingHints(hints);
				gr.setStroke(new BasicStroke(1.8f));
				gr.setColor(normalFillColor);
				gr.fill(shape);
				gr.setColor(normalColor);
				gr.draw(shape);
			}
			/**
			 * 
			 */
			protected void paintSelected(Graphics2D gr) {
				gr.setRenderingHints(hints);
		        gr.setStroke(new BasicStroke(1.8f));
				gr.setColor(selectedFillColor);
				gr.fill(shape);
				int x = (int) (shape.getBounds2D().getCenterX() - len / 2);
				int y = (int) (shape.getBounds2D().getCenterY() - height - (rad - fontMetrics.getDescent()) / 2);
				Shape zone = new Rectangle2D.Double(x, y-rad, len, (rad - fontMetrics.getDescent())/2+height);
				gr.setColor(Color.white);
				gr.fill(zone);
				gr.setColor(Color.black);
		        gr.setStroke(new BasicStroke(1.8f));
				gr.drawString(name, x, y);
				gr.draw(shape);
			}
			/**
			 * 
			 */
			protected void paintNeighbors(Graphics2D gr) {
				for(int neig : neighbors){
					mySensors.get(neig).selected(gr);
				}
			}

			/**
			 * 
			 */
			protected void selected(Graphics2D gr) {
				gr.setRenderingHints(hints);
		        gr.setStroke(new BasicStroke(1.8f));
				gr.setColor(selectedFillNeigh);
				gr.fill(shape);
				gr.setColor(selectedColor);
				gr.draw(shape);
			}
			/**
			 * 
			 */
			protected void paintZone(Graphics2D gr) {
				int w, h, x, y;
				Shape zone = null;
				if(model.equals(REAL)){
					w = (int) (2 * Kv*unitX + width);
					h = (int) (2 * Kv*unitY + height);
					x = (int) (shape.getBounds2D().getCenterX()) - w/2;
					y = (int) (shape.getBounds2D().getCenterY()) - h/2;
					zone = new Ellipse2D.Double(x, y, w, h);
				} else{
					w = (int) ((2 * (Kv - 1) + 1) * unitX)+ width;
					h = (int) ((2 * (Kv - 1) + 1) * unitY)+ height;
					int xx = (int) ((shape.getBounds2D().getCenterX() - margin) / unitX);
					int yy = (int) ((shape.getBounds2D().getCenterY() - 3 * margin / 2) / unitY);
					x = (int) (xx * unitX + margin - ((w - unitX) / 2));
					y = (int) (yy * unitY + 3 * margin / 2 - ((h - unitY) / 2));
					zone = new Rectangle2D.Double(x, y, w, h);
				}
		        gr.setRenderingHints(hints);  
				gr.setComposite(composite);
		        gr.setColor(Color.yellow);
		        gr.fill(zone);
		        gr.setColor(Color.black);
		        gr.setStroke(new BasicStroke(1f));
		        gr.draw(zone);
		        gr.setComposite(defaultComposite);
			}
			/**
			 * @param p
			 * @return
			 */
			public boolean contains(Point p) {
				return shape.contains(p);
			}
		}


		class Sensor extends Target {

			public Sensor(int id, String name, double x, double y, IntArray neighbors) {
				super(id, name, x, y, neighbors);
				this.selectedColor = Color.blue;
				this.selectedFillNeigh = Color.cyan;
				this.normalColor = Color.black;
				this.normalFillColor = Color.gray;
				shape = new Ellipse2D.Double(x-width/2, y-height/2, width, height);
			}
			/**
			 * 
			 */
			protected void paintZone(Graphics2D gr) {
				int w = (int) (2 * Kc*unitX + width);
				int h = (int) (2 * Kc*unitY + height);
				int x = (int) (shape.getBounds2D().getCenterX()) - w/2;
				int y = (int) (shape.getBounds2D().getCenterY()) - h/2;
		        gr.setRenderingHints(hints);
		        gr.setColor(Color.yellow);
				gr.setComposite(composite);
				Shape zone = null;
				if(model.equals(REAL)){
					zone = new Ellipse2D.Double(x, y, w, h);
				} else{
					zone = new Rectangle2D.Double(x, y, w, h);
				}
		        gr.fill(zone);
		        gr.setColor(Color.black);
		        gr.setStroke(new BasicStroke(1f));
		        gr.draw(zone);
		        gr.setComposite(defaultComposite);
			}
		}
	}
	
}
