package gui.simulation;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import simulation.parser.InstanceParser;
import util.Commands;

public class SimulationsPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	public final static String EMPTY_PANEL = "empty_Panel";
	public final static String Random_DisCSP_PANEL = "Random_DisCSP_Panel"; 
	public final static String SN_PANEL = "SN_Panel";
	public final static String MS_PANEL = "MS_Panel";
	public final static String TreeAO_PANEL = "TreeAO_Panel";
	public final static String RandomAO_PANEL = "RandomAO_Panel";
	public final static String LexicoAO_PANEL = "LexicoAO_Panel";
	public final static String MinDomDegAO_PANEL = "MinDomDegAO_Panel";

	private JPanel emptyPanel = null;
	private JPanel principalPanel = null;
	private File problem = null;

	/**
	 * This is the default constructor
	 */
	public SimulationsPanel() {
		super();
		initialize();
	}

	/**
	 * Initialize the class.
	 */
	private void initialize() {
		this.setLayout(new BorderLayout());
		this.add(getPrincipalPanel(), BorderLayout.CENTER);
		this.setMinimumSize(new Dimension(1024, 520));
		this.setMaximumSize(new Dimension(1024, 520));
		this.setPreferredSize(new Dimension(1024, 520));
	}

	/**
	 * Return the JFrameContentPane property value.
	 * 
	 * @return JPanel
	 */
	private JPanel getPrincipalPanel() {
		if (principalPanel == null) {
			principalPanel = new JPanel();
			principalPanel.setLayout(new CardLayout());
			principalPanel.setMinimumSize(new Dimension(1024, 520));
			principalPanel.setMaximumSize(new Dimension(1024, 520));
			principalPanel.setPreferredSize(new Dimension(1024, 520));
			principalPanel.add(getEmptyPanel(), EMPTY_PANEL);
			principalPanel.setBorder(BorderFactory.createEtchedBorder());
		}
		return principalPanel;
	}
	
	/**
	 * This method initializes westPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getEmptyPanel() {
		if (emptyPanel == null) {
			emptyPanel = new JPanel();
			emptyPanel.setMinimumSize(new Dimension(1024, 520));
			emptyPanel.setMaximumSize(new Dimension(1024, 520));
			emptyPanel.setPreferredSize(new Dimension(1024, 520));
		}
		return emptyPanel;
	}
	/**
	 * 
	 * @param file
	 * @param typeVisualization
	 */
	public void setProblem(File file, short typeOfVisualization) {
		this.problem = file;
		String problemFile = this.problem.getAbsolutePath();
		InstanceParser parser = new InstanceParser(problemFile);
		parser.loadInstance(problemFile);
		parser.parse(false);
		String benchmark = parser.getBenchmark();
		if (benchmark.equals(Commands.RandomDisCSP) || benchmark.equals(Commands.RandomDCOP) || 
				benchmark.equals(Commands.RandomMaxDisCSP)|| benchmark.equals(Commands.ComplexRandomTKC)) { 
			getPrincipalPanel().add(new SimulationOnRBDisCSPPanel(file, getPrincipalPanel().getSize(), typeOfVisualization), Random_DisCSP_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), Random_DisCSP_PANEL);
		} else if (benchmark.equals(Commands.GSensorDCSP) || benchmark.equals(Commands.SensorDCSP)) {
			getPrincipalPanel().add(new SimulationOnSensorNetworkPanel(file, getPrincipalPanel().getSize(), typeOfVisualization), SN_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), SN_PANEL);
		} else if (benchmark.equals(Commands.MeetingScheduling) || benchmark.equals(Commands.DIMES)) {
			getPrincipalPanel().add(new SimulationOnMeetingSchedulingPanel(file, getPrincipalPanel().getSize(), typeOfVisualization), MS_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), MS_PANEL);
		} else {
			System.err.println("Todo for : " + benchmark);// TODO
		}
	}
}
