package gui.tree;

import gui.tree.presentation.TreePanel;
import gui.tree.presentation.TreePresentation;
import gui.tree.presentation.TreePanel.Orientation;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import kernel.VarIdentifier;
import util.graph.UndirectedGraph;
import util.tree.AbstractTree;

/**
 * The Frame for visualizing the graph. 
 * 
 * @author Mohamed Wahbi
 * 
 */
@SuppressWarnings("serial")
public class TreeFrame extends JFrame {

	private final static String title = "DisChoco: Visalising constraint graphs";

	private static final String ABOUT_MESSAGE = "<html><p align=center><h3>DisChoco 2.0</h3><br> " + "Version 2.0, Freeware, 01-01-2013<br><br>"
			+ "<hr>This program is designed for creating and editing different graph types.<br>"
			+ "Trees can be loaded and saved.<br>You can undo or redo up to 10 most recent actions.<br>"
			+ "There several graph algorythms that can be executed on your graph.<br><hr><br> "
			+ "Author: Mohamed Wahbi (<a href=\"mailto:wahbi@lirmm.fr\">wahbi@lirmm.fr</a>)<br> " + "Designed for: </p></html>";

	private final TreePanel<VarIdentifier> treePanel;
	// private final TreePanel<String> treePanel;
	private final JComponent mainComponent;

	/**
	 * Creates the frame
	 */
	public TreeFrame() {
		super(title);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout());
		Dimension size = new Dimension(800, 500);
		// //TreePresentation<String> graph = makeTree();
		// //TreePresentation<String> graph = makeTreePresentation();
		// TreePresentation<String> graph = getOneComponentTree();
		// this.treePanel = new TreePanel<String>(graph, size);
		// TreePresentation<IVarIdentifier> graph = getConstraintTree();
		// this.treePanel = new TreePanel<IVarIdentifier>(graph, size);
//		UndirectedGraph<VarIdentifier> graph = getConstraintGraph();
//		String fileName = "dotgraph.dot";
//		UndirectedGraph.toDotFile(graph, fileName);
//		UndirectedGraph.fromDOTtoPDF(fileName);
//		//graph = ConstraintGraph.getSimpleConstraintGraph();
//		AbstractTree<VarIdentifier> tree = graph.MCSDFS();
//		//tree = graph.MaxDegreeDFS();
//		fileName = "mcstree.dot";
//		AbstractTree.toDotFile(tree, fileName);
//		UndirectedGraph.fromDOTtoPDF(fileName);
//		tree = graph.MaxDegreeDFS();
//		fileName = "MLSPTree.dot";
//		AbstractTree.toDotFile(tree, fileName);
//		UndirectedGraph.fromDOTtoPDF(fileName);
//		fileName = "maxdegtree.dot";
//		AbstractTree.toDotFile(tree, fileName);
//		UndirectedGraph.fromDOTtoPDF(fileName);
//		tree = graph.MLSPTree();
		String //string = "[{7.1; {8.1; {9.1; {4.1; {1.1}}}; {10.1}}; {6.1; {3.1; {5.1}}}; {2.1}}]";
//		string = "[{15.1; {16.1; {17.1; {9.1; {8.1; {24.1; {23.1; {20.1; {19.1; {21.1; {2.1; {1.1; {12.1; {11.1; {25.1; {26.1}; {18.1}}}; {13.1; {14.1; {4.1; {3.1}; {27.1}}}}}}; {5.1}}}}}}; {22.1}}}}}}; {6.1; {7.1; {10.1}}}}]";
		string = "[{2.1; {6.1; {9.1; {8.1; {1.1; {5.1; {10.1}; {7.1}}}}; {4.1}}; {3.1}}}]";
		AbstractTree<VarIdentifier> tree = AbstractTree.fromString(string , VarIdentifier.class);
		TreePresentation<VarIdentifier> treePresentation = new TreePresentation<VarIdentifier>(tree, false, false);
		this.treePanel = new TreePanel<VarIdentifier>(treePresentation, size, Orientation.VERTICAL);

		this.mainComponent = new JComponent() {};
		this.mainComponent.setLayout(new BorderLayout());
		this.mainComponent.add(treePanel, BorderLayout.CENTER);
		setJMenuBar(createMainMenu());
		getContentPane().add(mainComponent, BorderLayout.CENTER);
		setSize(size);
		setPreferredSize(size);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((int) (screenSize.getWidth() - size.getWidth()) / 2, (int) (screenSize.getHeight() - size.getHeight()) / 2);
		pack();
		setVisible(true);
	}

	public static UndirectedGraph<VarIdentifier> getConstraintGraph() {
		int weight = 0;
		UndirectedGraph<VarIdentifier> graph = new UndirectedGraph<VarIdentifier>();
		// let's throw in a clique, too

		VarIdentifier x1 = new VarIdentifier(1, 1);
		VarIdentifier x2 = new VarIdentifier(2, 1);
		VarIdentifier x3 = new VarIdentifier(3, 1);
		VarIdentifier x4 = new VarIdentifier(4, 1);
		VarIdentifier x5 = new VarIdentifier(5, 1);
		VarIdentifier x6 = new VarIdentifier(6, 1);
		VarIdentifier x7 = new VarIdentifier(7, 1);
		VarIdentifier x8 = new VarIdentifier(8, 1);
		VarIdentifier x9 = new VarIdentifier(9, 1);
		VarIdentifier x10 = new VarIdentifier(10, 1);
		VarIdentifier x11 = new VarIdentifier(11, 1);
		VarIdentifier x12 = new VarIdentifier(12, 1);
		VarIdentifier x13 = new VarIdentifier(13, 1);

		graph.addEdge(x1, x2, "C"+weight, weight++);
		graph.addEdge(x1, x3, "C"+weight, weight++);
		graph.addEdge(x1, x4, "C"+weight, weight++);
		graph.addEdge(x1, x8, "C"+weight, weight++);
		graph.addEdge(x1, x11, "C"+weight, weight++);
		graph.addEdge(x2, x3, "C"+weight, weight++);
		graph.addEdge(x2, x4, "C"+weight, weight++);
		graph.addEdge(x2, x5, "C"+weight, weight++);
		graph.addEdge(x2, x6, "C"+weight, weight++);
		graph.addEdge(x2, x7, "C"+weight, weight++);
		graph.addEdge(x2, x8, "C"+weight, weight++);
		graph.addEdge(x3, x5, "C"+weight, weight++);
		graph.addEdge(x4, x6, "C"+weight, weight++);
		graph.addEdge(x5, x7, "C"+weight, weight++);
		graph.addEdge(x6, x9, "C"+weight, weight++);
		graph.addEdge(x8, x10, "C"+weight, weight++);
		graph.addEdge(x8, x11, "C"+weight, weight++);
		graph.addEdge(x10, x12, "C"+weight, weight++);
		graph.addEdge(x10, x13, "C"+weight, weight++);
		
		return graph;
	}

	private JMenuBar createMainMenu() {
		JMenuBar mainMenu = new JMenuBar();
		mainMenu.add(createLayoutsMenu());
		mainMenu.add(createConvertionMenu());
		mainMenu.add(createHelpMenu());
		return mainMenu;
	}

	private JMenu createLayoutsMenu() {
		JMenu algorythmMenu = new JMenu("Layouts");
		// algorythmMenu.add(graphComponent.getLayoutFactory().getPredefinedAction());
		// algorythmMenu.add(graphComponent.getLayoutFactory().getCircleAction());
		return algorythmMenu;
	}

	private JMenu createConvertionMenu() {
		JMenu convertionMenu = new JMenu("Convert");
		// convertionMenu.add(graphComponent.getConvertionActionsFactory().getToWeightedAction());
		// convertionMenu.add(graphComponent.getConvertionActionsFactory().getToUnweightedAction());
		// convertionMenu.add(graphComponent.getConvertionActionsFactory().getToUndirectedAction());
		return convertionMenu;
	}

	private JMenu createHelpMenu() {
		JMenu helpMenu = new JMenu("Help");
		helpMenu.add(new AbstractAction("About") {

			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(TreeFrame.this, ABOUT_MESSAGE, "About", JOptionPane.QUESTION_MESSAGE);
			}
		});
		return helpMenu;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// Vector<Vertex<String>> vertexes = new Vector<Vertex<String>>();
		// HashSet<String> set = new HashSet<String>();
		// for(int i = 1 ; i < 4 ; i++) {
		// for(int j = i+1 ; j <4 ; j++) {
		// set.add(Integer.toString(i));
		// set.add(Integer.toString(j));
		// Vertex<String> vi = new Vertex<String>(Integer.toString(i));
		// if(!vertexes.contains(vi)) {
		// vertexes.add(vi);
		// }
		// Vertex<String> vj = new Vertex<String>(Integer.toString(j));
		// if(!vertexes.contains(vj)) {
		// vertexes.add(vj);
		// }
		// }
		// }
		// System.out.println(vertexes);
		// System.out.println(set);
		new TreeFrame();
		// TreeFrame frame = new TreeFrame();
		// frame.setVisible(true);
	}

}
