package gui.tree.presentation;

import util.graph.Edge;
import gui.graph.presentation.EdgePresentation;
import gui.graph.presentation.VertexPresentation;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.util.Map;

/**
 * This class provides a presentation for <tt>Edge<T></tt>.
 * 
 * @author Mohamed Wahbi
 * 
 */
public class BackEdgePresentation<T extends Comparable<? super T>> extends EdgePresentation<T>{

	/**
	 * Creates edge presentation with given start and end points.
	 * 
	 * @param edge edge
	 * @param vertexesMap map from vertexes to their presentations
	 * @param isDirected true if you want directed edge; false otherwise
	 * @param isWeighted true if you want weighted edge; false otherwise
	 */
	public BackEdgePresentation(Edge<T> edge, Map<T, VertexPresentation<T>> vertexesMap, boolean isDirected, boolean isWeighted) {
		super(edge, vertexesMap, isDirected, isWeighted);
		this.color = Color.gray;
		this.colorSelection = Color.darkGray;
		this.show = false;
	}
	
	/**
	 * Creates edge presentation with given start and end points.
	 * 
	 * @param edge edge
	 * @param source source vertex presentation
	 * @param target source vertex presentation	 
	 * @param isDirected true if you want directed edge; false otherwise
	 * @param isWeighted true if you want weighted edge; false otherwise
	 */
	public BackEdgePresentation(Edge<T> edge, VertexPresentation<T> source, VertexPresentation<T> target, boolean isDirected, boolean isWeighted) {
		super(edge, source, target, isDirected, isWeighted);
		this.color = Color.gray;
		this.colorSelection = Color.darkGray;
		this.show = false;
	}

	/**
	 * Creates edge presentation with given start and end points.
	 * 
	 * @param edge edge
	 * @param source source vertex presentation
	 * @param target source vertex presentation
	 * @param isDirected true if you want directed edge; false otherwise
	 * @param isWeighted true if you want weighted edge; false otherwise
	 */
	public BackEdgePresentation(Edge<T> edge, VertexPresentation<T> source, VertexPresentation<T> target, boolean isWeighted) {
		this(edge, source, target, false, isWeighted);
		this.color = Color.gray;
		this.colorSelection = Color.darkGray;
		this.show = false;
	}

	/**
	 * Creates a new edge presentation and new edge. 
	 * 
	 * @param source source vertex presentation
	 * @param target source vertex presentation
	 * @param isDirected true if you want directed edge; false otherwise
	 * @param isWeighted true if you want weighted edge; false otherwise
	 */
	public BackEdgePresentation(VertexPresentation<T> source, VertexPresentation<T> target, boolean isDirected, boolean isWeighted) {
		super(source, target, isDirected, isWeighted);
		this.color = Color.gray;
		this.colorSelection = Color.darkGray;
		this.show = false;
	}

	/**
	 * Paints this edge using given graphics
	 * 
	 * @param gr graphics
	 * @param bgColor background color
	 */
	@Override
	public void paint(Graphics2D gr, Color bgColor) {
		if(show) {
			Stroke actualStroke = gr.getStroke();
			gr.setStroke(DASHED_STROKE);
			super.paint(gr, bgColor);
			gr.setStroke(actualStroke);
		}
	}

}
