/*
 * DisChoco: Distributed Constraint Reasoning Solver
 * Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi
 * DisChoco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * DisChoco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * To contact the authors: <http://dischoco.sourceforge.net/>
 */
package gui.visualizing;

import static util.Commands.GRID;
import static util.Commands.KC;
import static util.Commands.KV;
import static util.Commands.NB_SENSORS;
import static util.Commands.NB_TARGETS;
import static util.Commands.P_C;
import static util.Commands.P_V;
import static util.Commands.REAL;
import static util.Commands.X_axis;
import static util.Commands.Y_axis;
import static simulation.parser.InstanceTokens.ID;
import static simulation.parser.InstanceTokens.NAME;
import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.HashMap;
import javax.swing.JPanel;
import org.jdom2.Element;
import util.IntArray;

/**
 * @version dischoco 2.5.3<br/>
 * benchmark.sensornetwork.gui.GridSensor.java
 * @author Mohamed Wahbi
 */

public class GridSensor extends JPanel implements MouseWheelListener, MouseListener, MouseMotionListener {

	private static final long serialVersionUID = 5352838628433546561L;
	public static final double DEFAULT_ZOOM_PERCENTAGE = 1.05;
	private Dimension preferredSize = null;
	private Dimension minimumSize = null;
	private Dimension maximumSize = null;
	private HashMap<Integer, Sensor> mySensors = null;
	private HashMap<Integer, Target> myTargets = null;
	private Target pick;
	private boolean pickfixed;
	private Rectangle rect;
	private double unitX = 1;
	private double unitY = 1;
	private int lineWidth = 2;
	private int margin = 50;
	// private List<Element> sensors;
	// private List<Element> targets;
	private SensorNetworkPanel model;
	String layout = GRID;
	private static RenderingHints hints = null;

	public GridSensor(SensorNetworkPanel model, String layout, Dimension size) {
		super();
		this.model = model;
		this.layout = layout;
		hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		hints.add(new RenderingHints(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY));
		minimumSize = new Dimension(size.width - margin / 2, size.height - margin);
		preferredSize = minimumSize;
		maximumSize = new Dimension(preferredSize.width * 2, preferredSize.height * 2);
		mySensors = new HashMap<Integer, Sensor>();
		myTargets = new HashMap<Integer, Target>();
		setBackground(Color.white);
		addMouseWheelListener(this);
		addMouseListener(this);
		setSize(preferredSize);
		createShapes();
	}

	public void mouseClicked(MouseEvent e) {}

	public void mouseEntered(MouseEvent e) {}

	public void mouseExited(MouseEvent e) {}

	public void mousePressed(MouseEvent e) {
		Point p = e.getPoint();
		addMouseMotionListener(this);
		for(int s : mySensors.keySet()) {
			Sensor sensor = mySensors.get(s);
			if (sensor.contains(p)) {
				pick = sensor;
				break;
			}
		}
		for(int t : myTargets.keySet()) {
			Target target = myTargets.get(t);
			if (target.contains(p)) {
				pick = target;
				break;
			}
		}
		if (pick != null) {
			pickfixed = pick.selected;
			pick.selected = true;
		}
		repaint();
		e.consume();
	}

	public void mouseReleased(MouseEvent e) {
		removeMouseMotionListener(this);
		if (pick != null) {
			pick.selected = pickfixed;
			pick = null;
		}
		repaint();
		e.consume();
	}

	public void mouseDragged(MouseEvent e) {
		if (pick != null) {
			// pick.setCenter(e.getPoint());
		}
		repaint();
		e.consume();
	}

	public void mouseMoved(MouseEvent e) {}

	public void mouseWheelMoved(MouseWheelEvent e) {
		updatePreferredSize(e.getWheelRotation(), e.getPoint());
	}

	public Dimension getPreferredSize() {
		return preferredSize;
	}

	private void updatePreferredSize(int n, Point p) {
		double d = (double) n * DEFAULT_ZOOM_PERCENTAGE;
		d = (n > 0)? (1 / d) : (-d);
		int w = (int) (getPreferredSize().getWidth() * d);
		int h = (int) (getPreferredSize().getHeight() * d);
		if (w < minimumSize.width) {
			preferredSize = minimumSize;
		} else if (w > maximumSize.width) {
			preferredSize = maximumSize;
		} else {
			preferredSize = new Dimension(w, h);
		}
		createShapes();
		getParent().doLayout();
		revalidate();
		repaint();
	}

	public void paintComponent(Graphics g) {
		createShapes();
		Graphics2D gr = (Graphics2D) g;
		gr.setRenderingHints(hints);
		gr.setColor(Color.white);
		gr.fillRect(0, 0, getWidth(), getHeight());
		gr.setColor(Color.black);
		if (layout.equals(GRID))
			drawGrid(gr);
		gr.setStroke(new BasicStroke(lineWidth));
		gr.draw(rect);
		Font font = gr.getFont();
		font = new Font(font.getName(), Font.BOLD, font.getSize() + 1);
		gr.setFont(font);
		FontMetrics fontMetric = gr.getFontMetrics();
		String desc = NB_SENSORS + "=" + model.getNbSensors() + ",  " + NB_TARGETS + "=" + model.getNbTargets() + ",  ";
		desc += KC + "=" + model.getKc() + ",  " + KV + "=" + model.getKv() + ",  " + P_C + "=" + model.getPc() + ",  " + P_V + "=" + model.getPv() + ",  ";
		desc += X_axis + "=" + model.getX_axis() + ",  " + Y_axis + "=" + model.getY_axis();
		gr.drawString(desc, (getWidth() - fontMetric.stringWidth(desc)) / 2, margin - lineWidth);
		for(int id : myTargets.keySet()) {
			myTargets.get(id).paint(gr);
		}
		for(int id : mySensors.keySet()) {
			mySensors.get(id).paint(gr);
		}
		if (pick != null) {
			pick.paint(gr);
		}
		gr.dispose();
	}

	private void createShapes() {
		int m = margin - lineWidth;
		rect = new Rectangle(m, m + m / 2, getWidth() - 2 * m, getHeight() - 2 * m);
		unitX = (rect.width - 2 * lineWidth) / (model.getX_axis() + 1);
		unitY = (rect.height - 2 * lineWidth) / (model.getY_axis() + 1);
		mySensors = new HashMap<Integer, Sensor>();
		myTargets = new HashMap<Integer, Target>();
		for(Element sensor : model.getSensors()) {
			int id = new Integer(sensor.getAttributeValue(ID));
			String name = sensor.getAttributeValue(NAME);
			IntArray neighbors = IntArray.fromString(sensor.getText());
			double xx = new Double(sensor.getAttributeValue(X_axis));
			double yy = new Double(sensor.getAttributeValue(Y_axis));
			double x = rect.x + lineWidth + unitX + (xx * unitX);
			double y = rect.y + lineWidth + unitY + (yy * unitY);
			mySensors.put(id, new Sensor(id, name, x, y, neighbors));
		}
		for(Element target : model.getTargets()) {
			int id = new Integer(target.getAttributeValue(ID));
			String name = target.getAttributeValue(NAME);
			IntArray neighbors = IntArray.fromString(target.getText());
			double xx = new Double(target.getAttributeValue(X_axis));
			double yy = new Double(target.getAttributeValue(Y_axis));
			double x = rect.x + lineWidth + unitX + (xx * unitX);
			double y = rect.y + lineWidth + unitY + (yy * unitY);
			myTargets.put(id, new Target(id, name, x, y, neighbors));
		}
	}

	/**
	 * @param gr
	 */
	private void drawGrid(Graphics2D gr) {
		// ========================================================================
		gr.setStroke(new BasicStroke(0.5f));
		int w = rect.width - 2 * lineWidth;
		int h = rect.height - 2 * lineWidth;
		int unitX = h / (model.getX_axis() + 1);
		int unitY = w / (model.getY_axis() + 1);
		int x = rect.x + lineWidth;
		int y = rect.y + lineWidth;
		for(int r = 1 ; r <= model.getX_axis() ; r++) {
			gr.drawLine(x, y + r * unitX, x + w, y + r * unitX);
		}
		for(int r = 1 ; r <= model.getY_axis() ; r++) {
			gr.drawLine(x + r * unitY, y, x + r * unitY, y + h);
		}
		// ========================================================================
	}

	class Target {

		protected int id;
		protected Shape shape;
		protected String name;
		protected FontMetrics fontMetrics;
		protected int rad;
		protected int len;
		protected int width = 10;
		protected int height = 10;
		protected Color selectedColor = Color.black;
		protected Color selectedFillNeigh = Color.green;
		protected Color selectedFillColor = Color.orange;
		protected Color normalColor = Color.black;
		protected Color normalFillColor = Color.red;
		protected RenderingHints hints;
		protected IntArray neighbors = null;
		protected final float alpha = 0.2f;
		protected boolean selected;

		protected Composite composite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
		protected Composite defaultComposite;
		protected Point2D location;

		public Target(int id, String name, double x, double y, IntArray neighbors) {
			hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			hints.add(new RenderingHints(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY));
			this.id = id;
			this.name = name;
			shape = createMobile(x - width / 2, y - height / 2, width);
			// shape = new Rectangle2D.Double(x-width/2, y-height/2, width, height);
			this.neighbors = neighbors;
			location = new Point2D.Double(x, y);
		}

		public void paint(Graphics g) {
			Graphics2D gr = ((Graphics2D) getGraphics());
			defaultComposite = gr.getComposite();
			fontMetrics = gr.getFontMetrics();
			rad = fontMetrics.getAscent();
			len = fontMetrics.stringWidth(name);
			if (selected) {
				paintZone(gr);
				paintNeighbors(gr);
				paintSelected(gr);
			} else {
				paintNormal(gr);
			}
			gr.dispose();
		}

		/**
		 * 
		 */
		protected void paintNormal(Graphics2D gr) {
			gr.setRenderingHints(hints);
			gr.setStroke(new BasicStroke(1.1f));
			gr.setColor(normalFillColor);
			gr.fill(shape);
			gr.setColor(normalColor);
			gr.draw(shape);
		}

		/**
		 * 
		 */
		protected void paintSelected(Graphics2D gr) {
			gr.setRenderingHints(hints);
			gr.setStroke(new BasicStroke(1.2f));
			gr.setColor(selectedFillColor);
			gr.fill(shape);
			int x = (int) (shape.getBounds2D().getCenterX() - len / 2);
			int y = (int) (shape.getBounds2D().getCenterY() - height - (rad - fontMetrics.getDescent()) / 2);
			Shape zone = new Rectangle2D.Double(x, y - rad, len, (rad - fontMetrics.getDescent()) / 2 + height);
			gr.setColor(Color.white);
			gr.fill(zone);
			gr.setColor(Color.black);
			gr.setStroke(new BasicStroke(1.8f));
			gr.drawString(name, x, y);
			gr.setStroke(new BasicStroke(1.2f));
			gr.draw(shape);
		}

		/**
		 * 
		 */
		protected void paintNeighbors(Graphics2D gr) {
			for(int neig : neighbors) {
				mySensors.get(neig).selected(gr);
			}
		}

		/**
		 * 
		 */
		protected void selected(Graphics2D gr) {
			gr.setRenderingHints(hints);
			gr.setStroke(new BasicStroke(1.8f));
			gr.setColor(selectedFillNeigh);
			gr.fill(shape);
			gr.setColor(selectedColor);
			gr.draw(shape);
		}

		/**
		 * 
		 */
		protected void paintZone(Graphics2D gr) {
			int w, h, x, y;
			Shape zone = null;
			if (layout.equals(REAL)) {
				w = (int) (2 * model.getKv() * unitX + 1.5 * width);
				h = (int) (2 * model.getKv() * unitY + 3 * height);
				x = (int) (location.getX()) - w / 2;
				y = (int) (location.getY()) - h / 2 - height / 2;
				zone = new Ellipse2D.Double(x, y, w, h);
			} else {
				w = (int) (((2 * (model.getKv() - 1) + 1) * unitX) + 1.5 * width);// bounds.getWidth());//width;
				h = (int) (((2 * (model.getKv() - 1) + 1) * unitY) + 3 * height);// bounds.getHeight());//height;
				int xx = (int) ((location.getX() - margin) / unitX);
				int yy = (int) ((location.getY() - 3 * margin / 2) / unitY);
				x = (int) (xx * unitX + margin - ((w - unitX) / 2));
				y = (int) (yy * unitY + 3 * margin / 2 - ((h - unitY) / 2)) - height / 2;
				zone = new Rectangle2D.Double(x, y, w, h);
			}
			gr.setRenderingHints(hints);
			gr.setComposite(composite);
			gr.setColor(Color.yellow);
			gr.fill(zone);
			gr.setColor(Color.black);
			gr.setStroke(new BasicStroke(1f));
			gr.draw(zone);
			gr.setComposite(defaultComposite);
		}

		/**
		 * @param p
		 * @return
		 */
		public boolean contains(Point p) {
			return shape.contains(p);
		}
	}

	class Sensor extends Target {

		public Sensor(int id, String name, double x, double y, IntArray neighbors) {
			super(id, name, x, y, neighbors);
			this.selectedColor = Color.blue;
			this.selectedFillNeigh = Color.cyan;
			this.normalColor = Color.black;
			this.normalFillColor = Color.gray;
			// shape = new Ellipse2D.Double(x-width/2, y-height/2, width, height);
			shape = createSensor(x - width / 2, y - height / 2, width);
		}

		/**
		 * 
		 */
		protected void paintZone(Graphics2D gr) {
			int w = (int) (2 * model.getKc() * unitX + 1.5 * width);// width);
			int h = (int) (2 * model.getKc() * unitY + 3 * height);// bounds.getHeight());//height);
			int x = (int) (location.getX()) - w / 2;
			int y = (int) (location.getY()) - h / 2 - height/3;
			gr.setRenderingHints(hints);
			gr.setColor(Color.yellow);
			gr.setComposite(composite);
			Shape zone = null;
			if (layout.equals(REAL)) {
				zone = new Ellipse2D.Double(x, y, w, h);
			} else {
				zone = new Rectangle2D.Double(x, y, w, h);
			}
			gr.fill(zone);
			gr.setColor(Color.black);
			gr.setStroke(new BasicStroke(1f));
			gr.draw(zone);
			gr.setComposite(defaultComposite);
		}
		
		/**
		 * 
		 */
		@Override
		protected void paintNormal(Graphics2D gr) {
			gr.setRenderingHints(hints);
			gr.setStroke(new BasicStroke(1.1f));
//			gr.setColor(normalFillColor);
//			gr.fill(shape);
			gr.setColor(normalColor);
			gr.draw(shape);
		}
	}

	/**
	 * Creates a triangle shape that points downwards.
	 * 
	 * @param s
	 *            the size factor (equal to half the height of the triangle).
	 * @return A triangle shape.
	 */
	public static Shape createDownTriangle(float x, float y, float s) {
		final GeneralPath p0 = new GeneralPath();
		p0.moveTo(x, y + s);
		p0.lineTo(x + s, y - s);
		p0.lineTo(x - s, y - s);
		p0.closePath();
		return p0;
	}

	/**
	 * Creates a triangle shape that points upwards.
	 * 
	 * @param s the size factor (equal to half the height of the triangle).
	 * @return A triangle shape.
	 */
	public static Shape createUpTriangle(double x, double y, double s) {
		final GeneralPath shape = new GeneralPath();
		shape.moveTo(x, y - s);
		shape.lineTo(x + s, y + s);
		shape.lineTo(x - s, y + s);
		shape.closePath();
		return shape;
	}

	/**
	 * Creates a triangle shape that points upwards.
	 * 
	 * @param s the size factor (equal to half the height of the triangle).
	 * @return A triangle shape.
	 */
	public static Shape create2UpTriangle(double x, double y, double s) {
		final GeneralPath shape = new GeneralPath();
		shape.moveTo(x, y - s);
		shape.lineTo(x + s / 2, y + s);
		shape.lineTo(x - s / 2, y + s);
		shape.closePath();
		return shape;
	}

	/**
	 * Creates a diamond shape.
	 * 
	 * @param s the size factor (equal to half the height of the diamond).
	 * @return A diamond shape.
	 */
	public static Shape createDiamond(double x, double y, double s) {
		final GeneralPath shape = new GeneralPath();
		shape.moveTo(x, y - s);
		shape.lineTo(x + s, y);
		shape.lineTo(x, y + s);
		shape.lineTo(x - s, y);
		shape.closePath();
		return shape;
	}

	/**
	 * Creates a square shape.
	 * 
	 * @param s the size factor (equal to half the height of the square).
	 * @return A square shape.
	 */
	public static Shape createSquare(double x, double y, double s) {
		final GeneralPath shape = new GeneralPath();
		shape.moveTo(x - s, y - s);
		shape.lineTo(x + s, y - s);
		shape.lineTo(x + s, y + s);
		shape.lineTo(x - s, y + s);
		shape.closePath();
		return shape;
	}

	/**
	 * Creates a rectangle shape.
	 * 
	 * @param s the size factor (equal to half the height of the rectangle).
	 * @return A rectangle shape.
	 */
	public static Shape createRectangle(double x, double y, double s) {
		final GeneralPath shape = new GeneralPath();
		shape.moveTo(x - s / 2, y - s);
		shape.lineTo(x + s / 2, y - s);
		shape.lineTo(x + s / 2, y + s);
		shape.lineTo(x - s / 2, y + s);
		shape.closePath();
		return shape;
	}

	/**
	 * Creates a arc shape.
	 * 
	 * @param s the size factor (equal to half the height of the arc).
	 * @return A arc shape.
	 */
	public static Shape createArc(double x, double y, double s, double start, double extent) {
		return new Arc2D.Double(x - s, y - s, 2 * s, 2 * s, start, extent, Arc2D.OPEN);
	}

	public static Shape createSensor(double x, double y, double s) {
		// final GeneralPath shape = new GeneralPath();
		// shape.append(createUpTriangle(x, y+s, s), false);
		// shape.append(createSquare(x, y, s/2), false);
		// return shape;
		final GeneralPath shape = new GeneralPath();
		shape.append(create2UpTriangle(x + s / 2, y + s / 2, s), false);
		shape.append(createArc(x + s / 2, y - s / 2, s - 2 * s / 3, -40, 260), false);
		shape.append(createArc(x + s / 2, y - s / 2, s - s / 3, -50, 280), false);
		return shape;
	}

	/**
	 * Creates a mobile shape.
	 * 
	 * @param s the size factor (equal to half the height of the mobile).
	 * @return A mobile shape.
	 */
	public static Shape createMobile(double x, double y, double s) {
		final GeneralPath shape = new GeneralPath();
		double ss = s / 3;
		shape.moveTo(x - s / 2, y - s);
		shape.lineTo(x - s / 2, y - s - ss);
		shape.lineTo(x - s / 2 + ss / 2, y - s - ss);
		shape.lineTo(x - s / 2 + ss / 2, y - s);
		shape.lineTo(x + s / 2, y - s);
		shape.lineTo(x + s / 2, y + s);
		shape.lineTo(x - s / 2, y + s);
		shape.closePath();
		return shape;
	}
}
