/**
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copyItem of the GNU General Public License along with this program; if not, write to
 * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * @version 2.0 DisChoco Logger.java
 */
package gui.visualizing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.StringTokenizer;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.RepaintManager;

import dischoco.DisChoco;
import gui.Filter;
import gui.LabelAccessory;
import util.Logger;

/**
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class GuiLogger extends Logger implements Printable {

	private JMenuBar menu = null;
	private JMenu fileMenu = null;
	private JMenu editMenu = null;
	private JMenu formatMenu = null;
	private JMenuItem newFileItem = null;
	private JMenuItem openItem = null;
	private JMenuItem printItem = null;
	private JMenuItem saveItem = null;
	private JMenuItem saveAsItem = null;
	private JMenuItem closeItem = null;
	private JMenuItem cutItem = null;
	private JMenuItem copyItem = null;
	private JMenuItem pasteItem = null;
	private JMenuItem findItem = null;
	private JMenuItem selectAll = null;
	private JTextArea textArea = null;
	private JPanel principalPanel = null;
	private JScrollPane scrollpane = null;
	private JCheckBoxMenuItem wordWrap = null;

	private int option;
	private String content = null;
	private String name = null;
	private String word;
	private JFileChooser fileChooser = null;
	private JInternalFrame internalFrame = null;

//	/**
//	 * This method creates the main window of the user interface and displays it.
//	 * @param args
//	 */
//	public static void main(String...args) {
//		for (UIManager.LookAndFeelInfo laf : UIManager.getInstalledLookAndFeels()) {
//			if ("Mac OS X".equals(laf.getName())) {
//				try {
//					UIManager.setLookAndFeel(laf.getClassName());
//				} catch (Exception e) {}
//			} else {
//				if ("Nimbus".equals(laf.getName())) {
//					try {
//						UIManager.setLookAndFeel(laf.getClassName());
//					} catch (Exception e) {}
//				}
//			}
//		}
//		new GuiLogger("Logger");
//	}
//	/**
//	 * Defines the constructor of the Gui logger class
//	 * @param title
//	 */
//	public GuiLogger(String title) {
//		getMainFrame(title);
//	}

	/**
	 * @param fname
	 */
	public GuiLogger(String fname) {
		super(fname);
		getMainFrame(fname);
	}
	/**
	 * 
	 * @param directory
	 * @param fname
	 */
	public GuiLogger(String directory, String fname) {
		super(directory, fname);
		getMainFrame(fname);
	}

	public synchronized void printToLog(String s, int level) {
		super.printToLog(s, level);
		if (level >= log_debug_level) {
			this.textArea.append(s);
//			System.out.print(s);
		}
	}

	public synchronized void printToLogln(String s, int level) {
		super.printToLogln(s, level);
		if (level <= log_debug_level) {
			this.textArea.append(s+"\n");
//			System.out.println(s);
		}
	}
	/**
	 * @param title
	 */
	private JInternalFrame getMainFrame(String title) {
		if(internalFrame == null){
			internalFrame = new JInternalFrame();
			internalFrame.setJMenuBar(getMenu());
			internalFrame.setTitle(title);
			internalFrame.getContentPane().add(getPrincipalPanel(), BorderLayout.CENTER);
			internalFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			internalFrame.setSize(725, 460);
			Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
			int w = internalFrame.getSize().width;
			int h = internalFrame.getSize().height;
			internalFrame.setLocation((dim.width - w) / 2, (dim.height - h) / 2);
			internalFrame.setVisible(true);
		}
		return internalFrame;
	}

	private JMenuBar getMenu(){
		if(menu == null){
			menu = new JMenuBar();
			menu.add(getFileMenu());
			menu.add(getEditMenu());
			menu.add(getFormatMenu());
		}
		return menu;
	}

	/**
	 * @return
	 */
	private JMenu getFileMenu(){
		if(fileMenu == null){
			fileMenu = new JMenu("File");
			fileMenu.setMnemonic('f');
			fileMenu.add(getNewFileItem());
			fileMenu.add(getOpenItem());
			fileMenu.add(getSaveItem());
			fileMenu.add(getSaveAsItem());
			fileMenu.addSeparator();
			fileMenu.add(getPrintItem());
			fileMenu.add(getExitItem());
		}
		return fileMenu;
	}
	
	/**
	 * @return
	 */
	private JMenuItem getNewFileItem() {
		if(newFileItem == null){
			newFileItem = new JMenuItem("New");
			newFileItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent paramActionEvent) {
					newFile();
				}
			});
			newFileItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		}
		return newFileItem;
	}
	/**
	 * @return
	 */
	private JMenuItem getOpenItem() {
		if(openItem == null){
			openItem = new JMenuItem("Open...");
			openItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent paramActionEvent) {
					openFile();
				}
			});
			openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		}
		return openItem;
	}

	/**
	 * @return
	 */
	private JMenuItem getSaveItem() {
		if(saveItem == null){
			saveItem = new JMenuItem("Save");
			saveItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent paramActionEvent) {
					saveFile();
				}
			});
			saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		}
		return saveItem;
	}

	/**
	 * @return
	 */
	private JMenuItem getSaveAsItem() {
		if(saveAsItem == null){
			saveAsItem = new JMenuItem("Save As...");
			saveAsItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent paramActionEvent) {
					saveAsFile();
				}
			});
		}
		return saveAsItem;
	}

	/**
	 * @return
	 */
	private JMenuItem getPrintItem() {
		if(printItem == null){
			printItem = new JMenuItem("Print");
			printItem.addActionListener(new ActionListener(){
				@SuppressWarnings("rawtypes")
				public void actionPerformed(ActionEvent paramActionEvent) {
					javax.swing.SwingWorker worker = new javax.swing.SwingWorker() {
						@Override
						protected Object doInBackground() throws Exception {
							printFile();
							return null;
						}
					};
					worker.execute();
				}
			});
			printItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
		}
		return printItem;
	}

	/**
	 * @return
	 */
	private JMenuItem getExitItem() {
		if(closeItem == null){
			closeItem = new JMenuItem("Exit");
			closeItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent paramActionEvent) {
					exitFile();
				}
			});
			closeItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, ActionEvent.CTRL_MASK));
		}
		return closeItem;
	}

	/**
	 * @return
	 */
	private JMenu getEditMenu() {
		if(editMenu == null){
			editMenu = new JMenu("Edit");
			editMenu.setMnemonic('e');
			editMenu.add(getCopyItem());
			editMenu.add(getCutItem());
			editMenu.add(getPasteItem());
			editMenu.addSeparator();
			editMenu.add(getFindItem());
			editMenu.add(getSelectAllItem());
		}
		return editMenu;
	}

	/**
	 * @return
	 */
	private JMenuItem getCopyItem() {
		if(copyItem == null){
			copyItem = new JMenuItem("Copy");
			copyItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					copyFile();
				}
			});
			copyItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		}
		return copyItem;
	}
	/**
	 * @return
	 */
	private JMenuItem getCutItem() {
		if(cutItem == null){
			cutItem = new JMenuItem("Cut");
			cutItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					cutFile();
				}
			});
			cutItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
		}
		return cutItem;
	}

	/**
	 * @return
	 */
	private JMenuItem getPasteItem() {
		if(pasteItem == null){
			pasteItem = new JMenuItem("Paste");
			pasteItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					pasteFile();
				}
			});
			pasteItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
		}
		return pasteItem;
	}

	/**
	 * @return
	 */
	private JMenuItem getFindItem() {
		if(findItem == null){
			findItem = new JMenuItem("Find");
			findItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					findFile();
				}
			});
			findItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
		}
		return findItem;
	}

	/**
	 * @return
	 */
	private JMenuItem getSelectAllItem() {
		if(selectAll == null){
			selectAll = new JMenuItem("Select All");
			selectAll.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					selectAllFile();
				}
			});
			selectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		}
		return selectAll;
	}

	/**
	 * @return
	 */
	private JMenu getFormatMenu() {
		if(formatMenu == null){
			formatMenu = new JMenu("Format");
			formatMenu.add(getWordWrapItem());
		}
		return formatMenu;
	}

	/**
	 * @return
	 */
	private JMenuItem getWordWrapItem() {
		if(wordWrap == null){
			wordWrap = new JCheckBoxMenuItem("Word Wrap");
			wordWrap.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					wrapFile();
				}
			});
		}
		return wordWrap;
	}

	/**
	 * @return
	 */
	private JPanel getPrincipalPanel() {
		if(principalPanel == null){
			principalPanel = new JPanel();
			principalPanel.setLayout(new BorderLayout());
			principalPanel.add(getScrollpane());
		}
		return principalPanel;
	}

	/**
	 * @return
	 */
	private JScrollPane getScrollpane() {
		if(scrollpane == null){
			scrollpane = new JScrollPane(getTextArea());
		}
		return scrollpane;
	}

	/**
	 * @return
	 */
	public JTextArea getTextArea() {
		if(textArea == null){
			textArea = new JTextArea(25, 65);
			textArea.setLineWrap(false);
			textArea.setWrapStyleWord(false);
		}
		return textArea;
	}

	/* Implementation of the newFile() method. */
	public void newFile() {
		if (!textArea.getText().equals("") && !textArea.getText().equals(content)) {
			if (name == null) {
				/* Shows a Confirm dialog box. */
				option = JOptionPane.showConfirmDialog(null, "Do you want to save the file?");
				if (option == JOptionPane.YES_OPTION) {
					saveAs();
					textArea.setText("");
				}
				if (option == JOptionPane.NO_OPTION) {
					textArea.setText("");
				}
			} else {
				/* Shows a Confirm dialog box. */
				option = JOptionPane.showConfirmDialog(null, "Do you want to save the file?");
				if (option == JOptionPane.YES_OPTION) {
					/* Calls the saveItem() method. */
					save();
					textArea.setText("");
				}
				if (option == JOptionPane.NO_OPTION) {
					textArea.setText("");
				}
			}
		} else {
			textArea.setText("");
		}
		internalFrame.setTitle("Untitled");
	}

	public void openFile() {
		if (!textArea.getText().equals("") && !textArea.getText().equals(content)) {
			if (name == null) {
				option = JOptionPane.showConfirmDialog(null, "Do you want to save the file?");
				if (option == 0) {
					saveAs();
					open();
				}
				if (option == 1) {
					open();
				}
			} else {
				option = JOptionPane.showConfirmDialog(null, "Do you want to save the file?");
				if (option == 0) {
					save();
					open();
				}
				if (option == 1) {
					open();
				}
			}
		} else {
			open();
		}
	}

	public void saveFile() {
		if (name == null) {
			saveAs();
		} else {
			save();
		}
	}

	public void saveAsFile() {
		saveAs();
	}

	public void printFile() {
		print();
	}

	public void exitFile() {
		if (!textArea.getText().equals("") && !textArea.getText().equals(content)) {
			if (name == null) {
				option = JOptionPane.showConfirmDialog(null, "Do you want to save the file?");
				if (option == 0) {
					saveAs();
					internalFrame.dispose();
				}
				if (option == 1) {
					internalFrame.dispose();
				}
			} else {
				option = JOptionPane.showConfirmDialog(null, "Do you want to save the file?");
				if (option == 0) {
					save();
					internalFrame.dispose();
				}
				if (option == 1) {
					internalFrame.dispose();
				}
			}
		} else {
			internalFrame.dispose();
		}
	}

	public void cutFile() {
		textArea.cut();
	}

	public void copyFile() {
		textArea.copy();
	}

	public void pasteFile() {
		textArea.paste();
	}

	public void selectAllFile() {
		textArea.selectAll();
	}

	public void findFile() {
		try {
			word = JOptionPane.showInputDialog("Type the word to find");
			while (textArea.getText().indexOf(word) == -1) {
				JOptionPane.showMessageDialog(null, 
						"Word not found!", "No match", 
						JOptionPane.WARNING_MESSAGE);
				word = JOptionPane.showInputDialog("Type the word to find");
			}
			textArea.select(textArea.getText().indexOf(word), textArea.getText().indexOf(word) + word.length());
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(null, "Search canceled", "Abourted", JOptionPane.WARNING_MESSAGE);
		}
	}

	public void wrapFile() {
		if (wordWrap.getState() == true) {
			textArea.setLineWrap(true);
			textArea.setWrapStyleWord(true);
		} else {
			textArea.setLineWrap(false);
			textArea.setWrapStyleWord(false);
		}
	}

	public void open() {
		Filter filter = new Filter(new String[] {"txt"}, "Logger file (txt)");
		fileChooser = new JFileChooser(DisChoco.filePath);
		fileChooser.setFileFilter(filter);
//		fileChooser.setFileView(new JavaFileView());
		fileChooser.setAccessory(new LabelAccessory(fileChooser));
		fileChooser.setPreferredSize(new Dimension(660, 440));
		int returnVal = fileChooser.showOpenDialog(internalFrame);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			textArea.setText(null);
			try {
				name = fileChooser.getSelectedFile().getPath();
				FileInputStream fin = new FileInputStream(fileChooser.getSelectedFile());
				FileChannel fchan = fin.getChannel();
				long fsize = fchan.size();
				ByteBuffer buff = ByteBuffer.allocate((int) fsize);
				fchan.read(buff);
				buff.rewind();
				String str = new String(buff.array());
				textArea.append(str);
				content = textArea.getText();
				fchan.close();
				fin.close();
			} catch (IOException ioe) {
				System.err.println("I/O Error on Open");
			}
			internalFrame.setTitle(fileChooser.getSelectedFile().getAbsolutePath());
		}
	}

	public void save() {
		try {
			FileOutputStream fout = new FileOutputStream(fileChooser.getSelectedFile());
			PrintWriter pw = new PrintWriter(fout);
			content = textArea.getText();
			StringTokenizer st = new StringTokenizer(content, System.getProperty("line.separator"));
			while (st.hasMoreTokens()) {
				pw.println(st.nextToken());
			}
			pw.close();
			fout.close();
		} catch (IOException ioe) {
			System.err.println("I/O Error on Save");
		}
		internalFrame.setTitle(fileChooser.getSelectedFile().getName());
	}

	public void saveAs() {
		Filter filter = new Filter(new String[] {"txt"}, "Logger file (txt)");
		fileChooser = new JFileChooser(DisChoco.filePath);
		fileChooser.setFileFilter(filter);
//		fileChooser.setFileView(new JavaFileView());
		fileChooser.setAccessory(new LabelAccessory(fileChooser));
		fileChooser.setPreferredSize(new Dimension(660, 440));
		fileChooser.setDialogTitle("Save As");
		int returnVal = fileChooser.showSaveDialog(internalFrame);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			try {
				FileOutputStream fout = new FileOutputStream(fileChooser.getSelectedFile() + ".txt");
				PrintWriter pw = new PrintWriter(fout);
				content = textArea.getText();
				name = fileChooser.getSelectedFile().getPath();
				StringTokenizer st = new StringTokenizer(content, System.getProperty("line.separator"));
				while (st.hasMoreTokens()) {
					pw.println(st.nextToken());
				}
				pw.close();
				fout.close();
			} catch (IOException ioe) {
				System.err.println("I/O Error on Save");
			}
			internalFrame.setTitle(fileChooser.getSelectedFile().getAbsolutePath());
		}
	}
	public void print() {
		PrinterJob printer = PrinterJob.getPrinterJob();
		printer.setPrintable(this);
		if (printer.printDialog()) {
			try {
				printer.print();
			} catch (PrinterException pe) {
				System.out.println("Error: " + pe);
			}
		}
		internalFrame.setVisible(true);
	}

	/**
	 * This method opens a Print dialog box Parameters
	 */
	public int print(Graphics g, PageFormat formatMenu, int index) {
		if (index > 0) {
			return (NO_SUCH_PAGE);
		} else {
			Graphics2D g2d = (Graphics2D) g;
			g2d.translate(formatMenu.getImageableX(), formatMenu.getImageableY());
			RepaintManager manager1 = RepaintManager.currentManager(textArea);
			manager1.setDoubleBufferingEnabled(false);
			textArea.paint(g2d);
			RepaintManager manager2 = RepaintManager.currentManager(textArea);
			manager2.setDoubleBufferingEnabled(true);
			return (PAGE_EXISTS);
		}
	}
}