/**
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with this program; if not, write to
 * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * @version 2.0 DisChoco RandomDisCSPGeneratorPanel.java
 */
package gui.visualizing;

import static simulation.parser.InstanceTokens.NB_AGENTS;
import static simulation.parser.InstanceTokens.NB_CONSTRAINTS;
import static simulation.parser.InstanceTokens.NB_TUPLES;
import gui.graph.presentation.GraphPanel;
import gui.tree.presentation.TreePanel;
import gui.tree.presentation.TreePanel.Orientation;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.HashMap;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import kernel.VarIdentifier;
import kernel.order.AgentComplexity;
import kernel.order.AgentOrdering;
import kernel.order.LexicographicAO;
import kernel.order.MaxDegPlusMinDomAO;
import kernel.order.MinDomOverDegAO;
import kernel.order.RandomAO;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import simulation.parser.InstanceParser;
import simulation.parser.InstanceTokens;
import util.Commands;
import util.graph.UndirectedGraph;
import util.tree.AbstractTree;
import dischoco.DisChoco;

/**
 * @authors <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and <a
 *          href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 */

public class RBDisCSPPanel extends JPanel {

	private static final long serialVersionUID = -4353556871723921003L;
	private JLabel typeLabel = null;
	private JLabel prepocessingLabel = null;
	private JLabel randomDCSPLabel = null;
	private JLabel nbAgLabel = null;
	private JLabel nbAgentsLabel = null;
	private JLabel domLabel = null;
	private JLabel domainSizeLabel = null;
	private JLabel cstsdensityLabel = null;
	private JLabel P1Label = null;
	private JLabel cststightnessLabel = null;
	private JLabel P2Label = null;
	private JLabel nbCstsLabel = null;
	private JLabel nbConstrantsLabel = null;
	private JLabel tuplesLabel = null;
	private JLabel nbTuplesLabel = null;
	private JComboBox agentsOrdering = null;
	private JPanel eastPanel = null;
	private JPanel westPanel = null;
//	private File problem = null;
	private HashMap<VarIdentifier, AgentComplexity> complexities = null;
	private UndirectedGraph<VarIdentifier> graph = null;
	private GraphPanel<VarIdentifier> graphPanel = null;
	private Dimension size = null;
	private final static String TreeAO_PANEL = "TreeAO_Panel";
	private final static String GraphAO_PANEL = "GraphAO_Panel";
	private int nbAgents;
	private int domainSize;
	private double p1;
	private double p2;
	private int nbConstraints;
	private int nbTuples;
	private String benchmark;
	private static final int WEST_WIDTH = 300;
	
	/**
	 * 
	 * @param file
	 * @param d
	 */
	public RBDisCSPPanel(Dimension d) {
		super();
		setSize(d);
		setPreferredSize(d);
		size = d;
		String fileName = new File(DisChoco.filePath).getAbsolutePath();
		InstanceParser parser = new InstanceParser(fileName);
		parser.loadInstance(fileName);
		parser.parse(false);
		complexities = parser.getAgentsComplexity();
		graph = parser.getConstraintGraph();
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		if (fileName.endsWith("xml.bz2")) {
			try {
				Process p = Runtime.getRuntime().exec("bunzip2 -c " + fileName);
				document = sxb.build(p.getInputStream());
				p.waitFor();
				p.exitValue();
				p.destroy();
			} catch (Exception e) {
				System.out.println("Problem with " + fileName);
				e.printStackTrace();
			}
		}
		try { document = sxb.build(DisChoco.filePath);
		} catch (Exception e) {}
		Element gui = document.getRootElement().getChild(Commands.GuiPresentation);
		setPresentation(gui);
		setLayout(new BorderLayout());
		add(getWestPanel(), BorderLayout.WEST);
		add(getEastPanel(), BorderLayout.CENTER);
	}
	
	/**
	 * 
	 * @param width
	 * @param height
	 */
	public RBDisCSPPanel(int width, int height) {
		super();
		this.setSize(width, height);
	}

	public void setPresentation(Element presentation) {
		this.setLayout(null);
		this.benchmark = presentation.getAttributeValue(InstanceTokens.BENCHMARK);
		if(this.benchmark ==  null) {
			this.benchmark = "unknown";
		}
		this.nbAgents = new Integer(presentation.getAttributeValue(NB_AGENTS));
		this.domainSize = new Integer(presentation.getAttributeValue(Commands.DomainSize));
		this.p1 = new Double(presentation.getAttributeValue(Commands.DENSITY))/100;
		this.p2 = new Double(presentation.getAttributeValue(Commands.TIGHTNESS))/100;
		this.nbConstraints = new Integer(presentation.getAttributeValue(NB_CONSTRAINTS));
		this.nbTuples = new Integer(presentation.getAttributeValue(NB_TUPLES));
	}

	/**
	 * This method initializes westPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getWestPanel() {
		if (westPanel == null) {
			westPanel = new JPanel(null);
			Dimension d = new Dimension(WEST_WIDTH, size.height);
			westPanel.setSize(d);
			westPanel.setPreferredSize(d);
			prepocessingLabel = new JLabel("Prepocessing Ordering", JLabel.CENTER);
			prepocessingLabel.setBounds(new Rectangle(50, 15, 200, 22));
			typeLabel = new JLabel(InstanceTokens.BENCHMARK+":", JLabel.RIGHT);
			typeLabel.setBounds(new Rectangle(40, 90, 100, 22));
			randomDCSPLabel = new JLabel(this.benchmark, JLabel.LEFT);
			randomDCSPLabel.setBounds(new Rectangle(150, 90, 150, 22));
			nbAgLabel = new JLabel(Commands.NumberOfAgents+":", JLabel.RIGHT);
			nbAgLabel.setBounds(new Rectangle(25, 135, 150, 22));
			nbAgentsLabel = new JLabel("", JLabel.LEFT);
			nbAgentsLabel.setBounds(new Rectangle(190, 135, 150, 22));
			domLabel = new JLabel(Commands.SizeOfDomain+":", JLabel.RIGHT);
			domLabel.setBounds(new Rectangle(25, 170, 150, 22));
			domainSizeLabel = new JLabel("", JLabel.LEFT);
			domainSizeLabel.setBounds(new Rectangle(190, 170, 150, 22));
			cstsdensityLabel = new JLabel(Commands.DENSITY_OF_GRAPH+":", JLabel.RIGHT);
			cstsdensityLabel.setBounds(new Rectangle(25, 205, 150, 22));
			P1Label = new JLabel("", JLabel.LEFT);
			P1Label.setBounds(new Rectangle(190, 205, 150, 22));
			cststightnessLabel = new JLabel(Commands.CONSTRAINTS_TIGHTNESS+":", JLabel.RIGHT);
			cststightnessLabel.setBounds(new Rectangle(25, 240, 150, 22));
			P2Label = new JLabel("", JLabel.LEFT);
			P2Label.setBounds(new Rectangle(190, 240, 150, 22));
			nbCstsLabel = new JLabel(Commands.NumberOfConstraints+":", JLabel.RIGHT);
			nbCstsLabel.setBounds(new Rectangle(25, 275, 150, 22));
			nbConstrantsLabel = new JLabel("", JLabel.LEFT);
			nbConstrantsLabel.setBounds(new Rectangle(190, 275, 150, 22));
			tuplesLabel = new JLabel(Commands.NumberOfTuples+":", JLabel.RIGHT);
			tuplesLabel.setBounds(new Rectangle(25, 310, 150, 22));
			nbTuplesLabel = new JLabel("", JLabel.LEFT);
			nbTuplesLabel.setBounds(new Rectangle(190, 310, 150, 22));
			westPanel.add(prepocessingLabel);
			westPanel.add(getAgentsOrdering());
			westPanel.add(typeLabel);
			westPanel.add(randomDCSPLabel);
			westPanel.add(nbAgLabel);
			westPanel.add(nbAgentsLabel);
			westPanel.add(domLabel);
			westPanel.add(domainSizeLabel);
			westPanel.add(cstsdensityLabel);
			westPanel.add(cststightnessLabel);
			westPanel.add(P1Label);
			westPanel.add(P2Label);
			westPanel.add(nbCstsLabel);
			westPanel.add(nbConstrantsLabel);
			westPanel.add(tuplesLabel);
			westPanel.add(nbTuplesLabel);
			nbAgentsLabel.setText(nbAgents + "");
			domainSizeLabel.setText(domainSize + "");
			P1Label.setText(p1 + "");
			P2Label.setText(p2 + "");
			nbConstrantsLabel.setText(nbConstraints + "");
			nbTuplesLabel.setText(nbTuples + "");
		}
		return westPanel;
	}

	public JComboBox getAgentsOrdering() {
		if (agentsOrdering == null) {
			agentsOrdering = new JComboBox(AgentOrdering.ORDERINGS);
			agentsOrdering.setBounds(new Rectangle(50, 50, 200, 22));
			agentsOrdering.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						plotEastPanel((String) e.getItem());
					}
				}
			});
			System.out.println("Agent ordering : " + LexicographicAO.buildAgentsOrder(complexities));
			plotGraphAO();
		}
		return agentsOrdering;
	}

	/**
	 * This method initializes figurePanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getEastPanel() {
		if (eastPanel == null) {
			eastPanel = new JPanel(new CardLayout());
			Dimension d = new Dimension(size.width-350, size.height);
			eastPanel.setSize(d);
			eastPanel.setPreferredSize(d);
		}
		return eastPanel;
	}

	/**
	 * This method initializes randomAOPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getGraphPanel() {
		if (graphPanel == null) {
			graphPanel = new GraphPanel<VarIdentifier>(graph, new Dimension(size.width-350, size.height), Orientation.HORIZONTAL);
		}
		return graphPanel;
	}

	private void plotEastPanel(String selectedItem) {
		if (selectedItem.equals(Commands.TreeAO)) {
			plotTreeAO();
		} else if (selectedItem.equals(Commands.LexicoAO)) {
			System.out.println("Agent ordering : " + LexicographicAO.buildAgentsOrder(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.RandomAO)) {
			System.out.println("Agent ordering : " + RandomAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MinDomOverDegAO)) {
			System.out.println("Agent ordering : " + MinDomOverDegAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MaxDegPlusMinDomAO)) {
			System.out.println("Agent ordering : " + MaxDegPlusMinDomAO.buildpriorityTree(complexities));
			plotGraphAO();
		}
	}

	/**
	 * 
	 */
	private void plotGraphAO() {
		getEastPanel().add(getGraphPanel(), GraphAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), GraphAO_PANEL);
	}

	private void plotTreeAO() {
		AbstractTree<VarIdentifier> tree = graph.DFS();
		System.out.println("Agent ordering (BFS) : [" + tree.toBFSString()+"]");
		getEastPanel().add(new TreePanel<VarIdentifier>(tree, getEastPanel().getSize(), Orientation.HORIZONTAL), TreeAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), TreeAO_PANEL);
	}
}
