/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * @version 2.0
 *          DisChoco
 *          SNPanel.java
 */
package gui.visualizing;

import static util.Commands.DISPONIBILITY;
import static util.Commands.DisponibilityOfAgents;
import static simulation.parser.InstanceTokens.NB_AGENTS;
import static util.Commands.NB_AGENTS_PER_MEETING;
import static util.Commands.NB_DAYS;
import static util.Commands.NB_HOURS;
import static util.Commands.NB_MEETINGS;
import static util.Commands.NB_MEETINGS_PER_AGENT;
import static util.Commands.NumberOfAgents;
import static util.Commands.NumberOfDays;
import static util.Commands.NumberOfHours;
import static util.Commands.NumberOfMeetings;
import static util.Commands.NumberOfAgentPerMeetings;
import static util.Commands.NumberOfMeetingsPerAgent;
import static util.Commands.NumberOfXAxis;
import static util.Commands.NumberOfYAxis;
import static util.Commands.P_V;
import static util.Commands.X_axis;
import static util.Commands.Y_axis;
import gui.graph.presentation.GraphPanel;
import gui.tree.presentation.TreePanel;
import gui.tree.presentation.TreePanel.Orientation;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import kernel.VarIdentifier;
import kernel.order.AgentComplexity;
import kernel.order.AgentOrdering;
import kernel.order.LexicographicAO;
import kernel.order.MaxDegPlusMinDomAO;
import kernel.order.MinDomOverDegAO;
import kernel.order.RandomAO;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import simulation.parser.InstanceParser;
import simulation.parser.InstanceTokens;
import util.Commands;
import util.graph.UndirectedGraph;
import util.tree.AbstractTree;
import dischoco.DisChoco;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 * 
 */

public class RandomMSPPanel extends JPanel {

	public static final String fixKeys = P_V;
	private final static String MEETING_SCHEDULING = "Meeting Scheduling";

	private static final long serialVersionUID = -6560016016508986831L;
	private JLabel typeLabel = null;
	private JLabel prepocessingLabel = null;
	private JLabel randomDCSPLabel = null;
	private JLabel nbMeetLabel = null;
	private JLabel nbMeetingsLabel = null;
	private JLabel nbAgLabel = null;
	private JLabel nbAgentsLabel = null;
	private JLabel nbMPALabel = null;
	private JLabel nbMeetingPerAgentLabel = null;
	private JLabel dispALabel = null;
	private JLabel disponibilityOfAgentsLabel = null;
	private JLabel nbDLabel = null;
	private JLabel nbDaysLabel = null;
	private JLabel nbHLabel = null;
	private JLabel nbHoursLabel = null;
	private JPanel westPanel = null;
	private JPanel eastPanel = null;
	private JLabel nbXLabel = null;
	private JLabel nbXAxisLabel = null;
	private JLabel nbYLabel = null;
	private JLabel nbYAxisLabel = null;
	private JComboBox agentsOrdering = null;
	// private File problem = null;
	private HashMap<VarIdentifier, AgentComplexity> complexities = null;
	private UndirectedGraph<VarIdentifier> graph = null;
	private GraphPanel<VarIdentifier> graphPanel = null;
	private Dimension size = new Dimension(820, 400); // @jve:decl-index=0:
	private final static String TreeAO_PANEL = "TreeAO_Panel";
	private final static String GraphAO_PANEL = "GraphAO_Panel";
	private static final int WEST_WIDTH = 300;
	private int nbMeetings;
	private int nbAgents;
	private int nbMeetingsPerAgent;
	private int nbAgentsPerMeeting;
	private int disponibilityOfAgents;
	private int nbDays;
	private int nbHours;
	private int xaxis;
	private int yaxis;

	private String benchmark;

	/**
	 * 
	 * @param file
	 * @param d
	 */
	public RandomMSPPanel(Dimension d) {
		super();
		size = d;
		setSize(size);
		setPreferredSize(size);
		String fileName = new File(DisChoco.filePath).getAbsolutePath();
		InstanceParser parser = new InstanceParser(fileName);
		parser.loadInstance(fileName);
		parser.parse(false);
		parser.convertToSimpleModel();
		complexities = parser.getAgentsComplexity();
		graph = parser.getConstraintGraph();
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		if (fileName.endsWith("xml.bz2")) {
			try {
				Process p = Runtime.getRuntime().exec("bunzip2 -c " + fileName);
				document = sxb.build(p.getInputStream());
				p.waitFor();
				p.exitValue();
				p.destroy();
			} catch (Exception e) {
				System.out.println("Problem with " + fileName);
				e.printStackTrace();
			}
		}
		try {
			document = sxb.build(DisChoco.filePath);
		} catch (Exception e) {}
		Element gui = document.getRootElement().getChild(Commands.GuiPresentation);
		setPresentation(gui);
		setLayout(new BorderLayout());
		add(getWestPanel(), BorderLayout.WEST);
		add(getEastPanel(), BorderLayout.CENTER);
	}

//	  <GuiPresentation name="RandomMSP" type="DisCSP" benchmark="RandomMSP" nbMeetings="7" nbAgents="10" nbMeetingsPerAgent="2" disponibility="100" nbDays="2" nbHours="10" x_axis="3" y_axis="3">
//	    <meetings>
//	      <meeting name="M1">[ 1 3 6 7 ]</meeting>
//	      <meeting name="M2">[ 2 5 7 9 10 ]</meeting>
//	      <meeting name="M3">[ 4 8 ]</meeting>
//	      <meeting name="M4">[ 3 5 ]</meeting>
//	      <meeting name="M5">[ 6 9 ]</meeting>
//	      <meeting name="M6">[ 1 4 8 ]</meeting>
//	      <meeting name="M7">[ 2 10 ]</meeting>
//	    </meetings>
//	    <agents>
//	      <agent name="A1">[ 1 6 ]</agent>
//	      <agent name="A2">[ 2 7 ]</agent>
//	      <agent name="A3">[ 1 4 ]</agent>
//	      <agent name="A4">[ 3 6 ]</agent>
//	      <agent name="A5">[ 2 4 ]</agent>
//	      <agent name="A6">[ 1 5 ]</agent>
//	      <agent name="A7">[ 1 2 ]</agent>
//	      <agent name="A8">[ 3 6 ]</agent>
//	      <agent name="A9">[ 2 5 ]</agent>
//	      <agent name="A10">[ 2 7 ]</agent>
//	    </agents>
//	  </GuiPresentation>
	public void setPresentation(Element presentation) {
		this.setLayout(null);
		this.benchmark = presentation.getAttributeValue(InstanceTokens.BENCHMARK);
		if(this.benchmark ==  null) {
			this.benchmark = "unknown";
		}
		this.nbMeetings = new Integer(presentation.getAttributeValue(NB_MEETINGS));
		this.nbAgents = new Integer(presentation.getAttributeValue(NB_AGENTS));
		String attribute = presentation.getAttributeValue(NB_MEETINGS_PER_AGENT);
		this.nbMeetingsPerAgent = (attribute == null)? 0 : new Integer(attribute);
		attribute = presentation.getAttributeValue(NB_AGENTS_PER_MEETING);
		this.nbAgentsPerMeeting = (attribute == null)? 0 : new Integer(attribute);
		this.disponibilityOfAgents = new Integer(presentation.getAttributeValue(DISPONIBILITY));
		this.nbDays = new Integer(presentation.getAttributeValue(NB_DAYS));
		this.nbHours = new Integer(presentation.getAttributeValue(NB_HOURS));
		attribute = presentation.getAttributeValue(X_axis);
		if(attribute != null) this.xaxis = new Integer(attribute);
		attribute = presentation.getAttributeValue(Y_axis);
		if(attribute != null) this.yaxis = new Integer(attribute);
		//TODO meetings and agents
	}

	/**
	 * This method initializes westPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getWestPanel() {
		if (westPanel == null) {
			westPanel = new JPanel(null);
			Dimension d = new Dimension(WEST_WIDTH, size.height);
			westPanel.setMinimumSize(d);
			westPanel.setMaximumSize(d);
			westPanel.setPreferredSize(d);
			prepocessingLabel = new JLabel("Prepocessing Ordering", JLabel.CENTER);
			prepocessingLabel.setBounds(new Rectangle(50, 15, 200, 22));
			typeLabel = new JLabel(InstanceTokens.BENCHMARK+":", JLabel.RIGHT);
			typeLabel.setBounds(new Rectangle(40, 80, 100, 22));
			randomDCSPLabel = new JLabel(this.benchmark, JLabel.LEFT);
			randomDCSPLabel.setBounds(new Rectangle(150, 80, 150, 22));
			nbMeetLabel = new JLabel(NumberOfMeetings+":", JLabel.RIGHT);
			nbMeetLabel.setBounds(new Rectangle(15, 115, 200, 22));
			nbMeetingsLabel = new JLabel("", JLabel.LEFT);
			nbMeetingsLabel.setBounds(new Rectangle(230, 115, 150, 22));
			nbAgLabel = new JLabel(NumberOfAgents+":", JLabel.RIGHT);
			nbAgLabel.setBounds(new Rectangle(15, 150, 200, 22));
			nbAgentsLabel = new JLabel("", JLabel.LEFT);
			nbAgentsLabel.setBounds(new Rectangle(230, 150, 150, 22));
			nbMPALabel = new JLabel(NumberOfMeetingsPerAgent+":", JLabel.RIGHT);
			nbMPALabel.setBounds(new Rectangle(15, 185, 200, 22));
			nbMeetingPerAgentLabel = new JLabel("", JLabel.LEFT);
			nbMeetingPerAgentLabel.setBounds(new Rectangle(230, 185, 150, 22));
			dispALabel = new JLabel(DisponibilityOfAgents+":", JLabel.RIGHT);
			dispALabel.setBounds(new Rectangle(15, 220, 200, 22));
			disponibilityOfAgentsLabel = new JLabel("", JLabel.LEFT);
			disponibilityOfAgentsLabel.setBounds(new Rectangle(230, 220, 150, 22));
			nbDLabel = new JLabel(NumberOfDays+":", JLabel.RIGHT);
			nbDLabel.setBounds(new Rectangle(15, 255, 200, 22));
			nbDaysLabel = new JLabel("", JLabel.LEFT);
			nbDaysLabel.setBounds(new Rectangle(230, 255, 150, 22));
			nbHLabel = new JLabel(NumberOfHours+":", JLabel.RIGHT);
			nbHLabel.setBounds(new Rectangle(15, 290, 200, 22));
			nbHoursLabel = new JLabel("", JLabel.LEFT);
			nbHoursLabel.setBounds(new Rectangle(230, 290, 150, 22));
			nbXLabel = new JLabel(NumberOfXAxis+":", JLabel.RIGHT);
			nbXLabel.setBounds(new Rectangle(15, 325, 200, 22));
			nbXAxisLabel = new JLabel("", JLabel.LEFT);
			nbXAxisLabel.setBounds(new Rectangle(230, 325, 150, 22));
			nbYLabel = new JLabel(NumberOfYAxis+":", JLabel.RIGHT);
			nbYLabel.setBounds(new Rectangle(15, 360, 200, 22));
			nbYAxisLabel = new JLabel("", JLabel.LEFT);
			nbYAxisLabel.setBounds(new Rectangle(230, 360, 150, 22));
			westPanel.add(prepocessingLabel);
			westPanel.add(getAgentsOrdering());
			westPanel.add(typeLabel);
			westPanel.add(randomDCSPLabel);
			westPanel.add(nbMeetLabel);
			westPanel.add(nbMeetingsLabel);
			westPanel.add(nbAgLabel);
			westPanel.add(nbAgentsLabel);
			westPanel.add(nbMPALabel);
			westPanel.add(dispALabel);
			westPanel.add(nbMeetingPerAgentLabel);
			westPanel.add(disponibilityOfAgentsLabel);
			westPanel.add(nbDLabel);
			westPanel.add(nbDaysLabel);
			westPanel.add(nbHLabel);
			westPanel.add(nbHoursLabel);
			westPanel.add(nbXLabel);
			westPanel.add(nbXAxisLabel);
			westPanel.add(nbYLabel);
			westPanel.add(nbYAxisLabel);
			nbMeetingsLabel.setText(this.nbMeetings + "");
			nbAgentsLabel.setText(this.nbAgents + "");
			if(this.nbMeetingsPerAgent==0) {
				nbMeetingPerAgentLabel.setText(Integer.toString(this.nbAgentsPerMeeting));
				nbMPALabel.setText(NumberOfAgentPerMeetings+":");
				nbXLabel.setText("");
				nbYLabel.setText("");
			} else {
				nbMeetingPerAgentLabel.setText(Integer.toString(this.nbMeetingsPerAgent));
				nbMPALabel.setText(NumberOfMeetingsPerAgent+":");
				nbXAxisLabel.setText(this.xaxis + "");
				nbYAxisLabel.setText(this.yaxis + "");
			}
			disponibilityOfAgentsLabel.setText(this.disponibilityOfAgents + "");
			nbDaysLabel.setText(this.nbDays + "");
			nbHoursLabel.setText(this.nbHours + "");
		}
		return westPanel;
	}

	public JComboBox getAgentsOrdering() {
		if (agentsOrdering == null) {
			List<String> list = new ArrayList<String>();
			list.add(MEETING_SCHEDULING);
			list.addAll(Arrays.asList(AgentOrdering.ORDERINGS));
			agentsOrdering = new JComboBox(list.toArray(new String[list.size()]));
			agentsOrdering.setBounds(new Rectangle(50, 50, 200, 22));
			agentsOrdering.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						plotEastPanel((String) e.getItem());
					}
				}
			});
			plotGraphAO();
		}
		return agentsOrdering;
	}

	/**
	 * This method initializes figurePanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getEastPanel() {
		if (eastPanel == null) {
			eastPanel = new JPanel(new CardLayout());
			Dimension d = new Dimension(size.width - WEST_WIDTH, size.height);
			eastPanel.setPreferredSize(d);
		}
		return eastPanel;
	}

	/**
	 * This method initializes randomAOPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private GraphPanel<VarIdentifier> getGrpahPanel() {
		if (graphPanel == null) {
			graphPanel = new GraphPanel<VarIdentifier>(graph, new Dimension(size.width - WEST_WIDTH, size.height), Orientation.HORIZONTAL);
		}
		return graphPanel;
	}

	/**
	 * 
	 * @param selectedItem
	 */
	private void plotEastPanel(String selectedItem) {
		if (selectedItem.equals(MEETING_SCHEDULING)) {
			//TODO plot table of meetings
			System.out.println("TODO plot table of meetings");
		} else if (selectedItem.equals(Commands.TreeAO)) {
			plotTreeAO();
		} else if (selectedItem.equals(Commands.LexicoAO)) {
			System.out.println("Agent ordering : " + LexicographicAO.buildAgentsOrder(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.RandomAO)) {
			System.out.println("Agent ordering : " + RandomAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MinDomOverDegAO)) {
			System.out.println("Agent ordering : " + MinDomOverDegAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MaxDegPlusMinDomAO)) {
			System.out.println("Agent ordering : " + MaxDegPlusMinDomAO.buildpriorityTree(complexities));
			plotGraphAO();
		}
	}

	/**
	 * 
	 */
	private void plotGraphAO() {
		getEastPanel().add(getGrpahPanel(), GraphAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), GraphAO_PANEL);
	}

	/**
	 * 
	 */
	private void plotTreeAO() {
		AbstractTree<VarIdentifier> tree = graph.DFS();
		System.out.println("Agent ordering (BFS) : [" + tree.toBFSString() + "]");
		getEastPanel().add(new TreePanel<VarIdentifier>(tree, getEastPanel().getSize(), Orientation.HORIZONTAL), TreeAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), TreeAO_PANEL);
	}
} // @jve:decl-index=0:visual-constraint="10,10"
