/** 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 * 
 * @version 2.0
 * DisChoco
 * SNPanel.java
 */ 
package gui.visualizing;

import static util.Commands.GRID;
import static util.Commands.GSensorDCSP;
import static util.Commands.GuiPresentation;
import static util.Commands.KC;
import static util.Commands.KV;
import static util.Commands.NB_SENSORS;
import static util.Commands.NB_TARGETS;
import static util.Commands.P_C;
import static util.Commands.P_V;
import static util.Commands.REAL;
import static util.Commands.Sensors;
import static util.Commands.Targets;
import static util.Commands.X_axis;
import static util.Commands.Y_axis;
import gui.graph.presentation.GraphPanel;
import gui.tree.presentation.TreePanel;
import gui.tree.presentation.TreePanel.Orientation;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import kernel.VarIdentifier;
import kernel.order.AgentComplexity;
import kernel.order.AgentOrdering;
import kernel.order.LexicographicAO;
import kernel.order.MaxDegPlusMinDomAO;
import kernel.order.MinDomOverDegAO;
import kernel.order.RandomAO;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import simulation.parser.InstanceParser;
import simulation.parser.InstanceTokens;
import util.Commands;
import util.graph.UndirectedGraph;
import util.tree.AbstractTree;
import dischoco.DisChoco;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 * 
 */

public class SensorNetworkPanel extends JPanel {
	
	/** 
	 * This program is free software; you can redistribute it and/or modify 
	 * it under the terms of the GNU General Public License as published by 
	 * the Free Software Foundation; either version 2 of the License, or 
	 * (at your option) any later version. 
	 *  
	 * This program is distributed in the hope that it will be useful, 
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
	 * GNU General Public License for more details. 
	 *  
	 * You should have received a copy of the GNU General Public License 
	 * along with this program; if not, write to the Free Software 
	 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
	 * 
	 * @version 2.0
	 * DisChoco
	 * SNPanel.java
	 */ 
	private static final long serialVersionUID = 6052326216599337737L;

	private int x_axis;
	private int y_axis;
	private int nbTargets;
	private int nbSensors;
	private int Kc;
	private int Kv;
	private double Pc;
	private double Pv;
	private JComboBox agentsOrdering;
	private HashMap<VarIdentifier, AgentComplexity> complexities;
	private final static String TreeAO_PANEL = "TreeAO_Panel";
	private final static String GraphAO_PANEL = "GraphAO_Panel";
	private final static String SensorGrid = "SensorGrid";
	private final static String SENSOR_NETWORK  = "Sensor Network";

	private static final int WEST_WIDTH = 300;
	private JPanel westPanel;
	private JPanel eastPanel;
	private JLabel typeLabel;
	private JLabel prepocessingLabel;
	private JLabel sensorDCSPLabel;
	private JLabel nbSLabel;
	private JLabel nbSensorsLabel;
	private JLabel nbTLabel;
	private JLabel nbTargetsLabel;
	private JLabel compatibilityDensityLabel;
	private JLabel PcLabel;
	private JLabel visibilityDensityLabel;
	private JLabel PvLabel;
	private JLabel nbKcLabel;
	private JLabel kcLabel;
	private JLabel nbKvLabel;
	private JLabel kvLabel;

	private UndirectedGraph<VarIdentifier> graph;
	private GraphPanel<VarIdentifier> graphPanel;
	private List<Element> sensors;
	private List<Element> targets;
	private String layout = REAL;

	private Dimension size;

	private String benchmark;
	public static Color sensorColor = Color.blue;
	public static Color targetColor = Color.red;
	
	public SensorNetworkPanel(Dimension d){
		super();
		size = d;
		setPreferredSize(size);
		String fileName = new File(DisChoco.filePath).getAbsolutePath();
		InstanceParser parser = new InstanceParser(fileName);
		parser.loadInstance(fileName);
		parser.parse(false);
		parser.convertToSimpleModel();
		this.complexities = parser.getAgentsComplexity();
		this.graph  = parser.getConstraintGraph();
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		if (fileName.endsWith("xml.bz2")) {
			try {
				Process p = Runtime.getRuntime().exec("bunzip2 -c " + fileName);
				document = sxb.build(p.getInputStream());
				p.waitFor();
				p.exitValue();
				p.destroy();
			} catch (Exception e) {
				System.out.println("Problem with " + fileName);
				e.printStackTrace();
			}
		}
		try { document = sxb.build(DisChoco.filePath);
		} catch (Exception e) {}
		Element gui = document.getRootElement().getChild(GuiPresentation);
		this.setLayout(new BorderLayout());
		setPresentation(gui);
		this.add(getWestPanel(), BorderLayout.WEST);
		this.add(getEastPanel(), BorderLayout.CENTER);
	}
	
	private JPanel getWestPanel() {
		if (westPanel == null) {
			westPanel = new JPanel(null);
			westPanel.setMinimumSize(new Dimension(WEST_WIDTH, size.height));
			westPanel.setMaximumSize(new Dimension(WEST_WIDTH, size.height));
			westPanel.setPreferredSize(new Dimension(WEST_WIDTH, size.height));
			prepocessingLabel = new JLabel("Prepocessing Ordering", JLabel.CENTER);
			prepocessingLabel.setBounds(new Rectangle(50, 15, 200, 22));
			westPanel.add(prepocessingLabel, null);
			westPanel.add(getAgentsOrdering());
			typeLabel = new JLabel(InstanceTokens.BENCHMARK+":", JLabel.RIGHT);
			typeLabel.setBounds(new Rectangle(40, 90, 100, 22));
			westPanel.add(typeLabel, null);
			sensorDCSPLabel = new JLabel(this.benchmark, JLabel.LEFT);
			sensorDCSPLabel.setBounds(new Rectangle(150, 90, 150, 22));
			westPanel.add(sensorDCSPLabel, null);
			nbSLabel = new JLabel("Number of Sensors :", JLabel.RIGHT);
			nbSLabel.setBounds(new Rectangle(25, 135, 150, 22));
			westPanel.add(nbSLabel, null);
			nbSensorsLabel = new JLabel(""+this.nbSensors, JLabel.LEFT);
			nbSensorsLabel.setBounds(new Rectangle(190, 135, 150, 22));
			westPanel.add(nbSensorsLabel, null);
			nbTLabel = new JLabel("Number of Targets :", JLabel.RIGHT);
			nbTLabel.setBounds(new Rectangle(25, 170, 150, 22));
			westPanel.add(nbTLabel, null);
			nbTargetsLabel = new JLabel(""+this.nbTargets, JLabel.LEFT);
			nbTargetsLabel.setBounds(new Rectangle(190, 170, 150, 22));
			westPanel.add(nbTargetsLabel, null);
			compatibilityDensityLabel = new JLabel("compatibility density :", JLabel.RIGHT);
			compatibilityDensityLabel.setBounds(new Rectangle(25, 205, 150, 22));
			westPanel.add(compatibilityDensityLabel, null);
			PcLabel = new JLabel(""+this.Pc, JLabel.LEFT);
			PcLabel.setBounds(new Rectangle(190, 205, 150, 22));
			westPanel.add(PcLabel, null);
			visibilityDensityLabel = new JLabel("visibility density :", JLabel.RIGHT);
			visibilityDensityLabel.setBounds(new Rectangle(25, 240, 150, 22));
			westPanel.add(visibilityDensityLabel, null);
			PvLabel = new JLabel(""+this.Pv, JLabel.LEFT);
			PvLabel.setBounds(new Rectangle(190, 240, 150, 22));
			westPanel.add(PvLabel, null);
			nbKcLabel = new JLabel("Kc :", JLabel.RIGHT);
			nbKcLabel.setBounds(new Rectangle(25, 275, 150, 22));
			westPanel.add(nbKcLabel, null);
			kcLabel = new JLabel(""+this.Kc, JLabel.LEFT);
			kcLabel.setBounds(new Rectangle(190, 275, 150, 22));
			westPanel.add(kcLabel, null);
			nbKvLabel = new JLabel("Kv :", JLabel.RIGHT);
			nbKvLabel.setBounds(new Rectangle(25, 310, 150, 22));
			westPanel.add(nbKvLabel, null);
			kvLabel = new JLabel(this.Kv+"", JLabel.LEFT);
			kvLabel.setBounds(new Rectangle(190, 310, 150, 22));
			westPanel.add(kvLabel, null);
		}
		return westPanel;
	}

	/**
	 * This method initializes figurePanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getEastPanel() {
		if (eastPanel == null) {
			Dimension d = new Dimension(size.width-WEST_WIDTH, size.height);
			eastPanel = new JPanel(new CardLayout());
			eastPanel.setMinimumSize(d);
			eastPanel.setMaximumSize(d);
			eastPanel.setPreferredSize(d);
			eastPanel.setBackground(Color.white);
			eastPanel.add(new JScrollPane(new GridSensor(this, this.layout , d)), SensorGrid);
		}
		return eastPanel;
	}

	public JComboBox getAgentsOrdering() {
		if (agentsOrdering == null) {
			List<String> list = new ArrayList<String>();
			list.add(SENSOR_NETWORK);
			list.addAll(Arrays.asList(AgentOrdering.ORDERINGS));
			agentsOrdering = new JComboBox(list.toArray(new String[list.size()]));
			agentsOrdering.setBounds(new Rectangle(50, 50, 200, 22));
			agentsOrdering.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						plotEastPanel((String) e.getItem());
					}
				}
			});
		}
		return agentsOrdering;
	}

	private void plotEastPanel(String selectedItem) {
		if (selectedItem.equals(Commands.TreeAO)) {
			plotTreeAO();
		} else if (selectedItem.equals(Commands.LexicoAO)) {
			System.out.println("Agent ordering : " + LexicographicAO.buildAgentsOrder(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.RandomAO)) {
			System.out.println("Agent ordering : " + RandomAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MinDomOverDegAO)) {
			System.out.println("Agent ordering : " + MinDomOverDegAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MaxDegPlusMinDomAO)) {
			System.out.println("Agent ordering : " + MaxDegPlusMinDomAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(SENSOR_NETWORK)) {
			this.plotSensorGrid();
		}
	}

	/**
	 * This method initializes randomAOPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getGrpahPanel() {
		if (graphPanel == null) {
			graphPanel = new GraphPanel<VarIdentifier>(graph, new Dimension(674, size.height), Orientation.HORIZONTAL);
		}
		return graphPanel;
	}

	/**
	 * 
	 */
	private void plotSensorGrid() {
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), SensorGrid);
	}

	/**
	 * 
	 */
	private void plotTreeAO() {
		AbstractTree<VarIdentifier> tree = graph.DFS();
		System.out.println("Agent ordering : [" + tree.toDFSString()+"]");
		getEastPanel().add(new TreePanel<VarIdentifier>(tree, getEastPanel().getSize(), Orientation.HORIZONTAL), TreeAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), TreeAO_PANEL);
	}


	/**
	 * 
	 */
	private void plotGraphAO() {
		getEastPanel().add(getGrpahPanel(), GraphAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), GraphAO_PANEL);
	}
	
	public void setPresentation(Element presentation){
		this.benchmark = presentation.getAttributeValue(InstanceTokens.BENCHMARK);
		if(this.benchmark ==  null) {
			this.benchmark = "unknown";
		}
		this.nbTargets = new Integer(presentation.getChild(Targets).getAttributeValue(NB_TARGETS));
		this.nbSensors = new Integer(presentation.getChild(Sensors).getAttributeValue(NB_SENSORS));
		this.Kc = new Integer(presentation.getAttributeValue(KC));
		this.Kv = new Integer(presentation.getAttributeValue(KV));
		this.Pc = new Double(presentation.getAttributeValue(P_C));
		this.Pv = new Double(presentation.getAttributeValue(P_V));
		this.x_axis = new Integer(presentation.getAttributeValue(X_axis));
		this.y_axis = new Integer(presentation.getAttributeValue(Y_axis));
		this.layout = (this.benchmark.equals(GSensorDCSP))?GRID:REAL;
		this.sensors = presentation.getChild(Sensors).getChildren();
		this.targets = presentation.getChild(Targets).getChildren();
	}

	
	/**
	 * @return the x_axis
	 */
	public int getX_axis() {
		return this.x_axis;
	}

	
	/**
	 * @param x_axis the x_axis to set
	 */
	public void setX_axis(int x_axis) {
		this.x_axis = x_axis;
	}

	
	/**
	 * @return the y_axis
	 */
	public int getY_axis() {
		return this.y_axis;
	}

	
	/**
	 * @param y_axis the y_axis to set
	 */
	public void setY_axis(int y_axis) {
		this.y_axis = y_axis;
	}

	
	/**
	 * @return the nbTargets
	 */
	public int getNbTargets() {
		return this.nbTargets;
	}

	
	/**
	 * @param nbTargets the nbTargets to set
	 */
	public void setNbTargets(int nbTargets) {
		this.nbTargets = nbTargets;
	}

	
	/**
	 * @return the nbSensors
	 */
	public int getNbSensors() {
		return this.nbSensors;
	}

	
	/**
	 * @param nbSensors the nbSensors to set
	 */
	public void setNbSensors(int nbSensors) {
		this.nbSensors = nbSensors;
	}

	
	/**
	 * @return the kc
	 */
	public int getKc() {
		return this.Kc;
	}

	
	/**
	 * @param kc the kc to set
	 */
	public void setKc(int kc) {
		this.Kc = kc;
	}

	
	/**
	 * @return the kv
	 */
	public int getKv() {
		return this.Kv;
	}

	
	/**
	 * @param kv the kv to set
	 */
	public void setKv(int kv) {
		this.Kv = kv;
	}

	
	/**
	 * @return the pc
	 */
	public double getPc() {
		return this.Pc;
	}

	
	/**
	 * @param pc the pc to set
	 */
	public void setPc(double pc) {
		this.Pc = pc;
	}

	
	/**
	 * @return the pv
	 */
	public double getPv() {
		return this.Pv;
	}

	
	/**
	 * @param pv the pv to set
	 */
	public void setPv(double pv) {
		this.Pv = pv;
	}

	
	/**
	 * @return the graph
	 */
	public UndirectedGraph<VarIdentifier> getGraph() {
		return this.graph;
	}

	
	/**
	 * @param graph the graph to set
	 */
	public void setGraph(UndirectedGraph<VarIdentifier> graph) {
		this.graph = graph;
	}
	
	/**
	 * @return the sensors
	 */
	public List<Element> getSensors() {
		return this.sensors;
	}
	
	/**
	 * @param sensors the sensors to set
	 */
	public void setSensors(List<Element> sensors) {
		this.sensors = sensors;
	}
	
	/**
	 * @return the targets
	 */
	public List<Element> getTargets() {
		return this.targets;
	}
	
	/**
	 * @param targets the targets to set
	 */
	public void setTargets(List<Element> targets) {
		this.targets = targets;
	}
	
}
