package gui.visualizing;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import benchmark.DataSetTeamcore;

import dischoco.DisChoco;
import simulation.parser.InstanceParser;
import util.Commands;

public class VisualizingPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	final static String EMPTY_PANEL = "empty_Panel";
	final static String Random_DisCSP_PANEL = "Random_DisCSP_Panel";
	final static String Random_MaxDisCSP_PANEL = "Random_MaxDisCSP_Panel";
	final static String Random_DCOP_PANEL = "Random_DCOP_Panel";
	final static String Random_GC_PANEL = "Random_GC_Panel";
	final static String SN_PANEL = "SN_Panel";
	final static String MS_PANEL = "MS_Panel";
	final static String RMS_PANEL = "RMS_Panel";

	final static String Random_SoftGC_PANEL = "Random_SoftGC_Panel";
	
	final static String TreeAO_PANEL = "TreeAO_Panel";
	final static String RandomAO_PANEL = "RandomAO_Panel";
	final static String LexicoAO_PANEL = "LexicoAO_Panel";
	final static String MinDomDegAO_PANEL = "MinDomDegAO_Panel";

	private JPanel emptyPanel = null;
	private JPanel principalPanel = null;
//	private File problem = null;  //  @jve:decl-index=0:

	/**
	 * This is the default constructor
	 */
	public VisualizingPanel() {
		super();
		initialize();
	}

	/**
	 * Initialize the class.
	 */
	private void initialize() {
		this.setLayout(new BorderLayout());
		this.add(getPrincipalPanel(), BorderLayout.CENTER);
		this.setMinimumSize(new Dimension(1024, 520));
		this.setMaximumSize(new Dimension(1024, 520));
		this.setPreferredSize(new Dimension(1024, 520));
	}

	/**
	 * Return the JFrameContentPane property value.
	 * 
	 * @return JPanel
	 */
	private JPanel getPrincipalPanel() {
		if (principalPanel == null) {
			principalPanel = new JPanel();
			principalPanel.setLayout(new CardLayout());
			principalPanel.setMinimumSize(new Dimension(1024, 520));
			principalPanel.setMaximumSize(new Dimension(1024, 520));
			principalPanel.setPreferredSize(new Dimension(1024, 520));
			principalPanel.add(getEmptyPanel(), EMPTY_PANEL);
			principalPanel.setBorder(BorderFactory.createEtchedBorder());
		}
		return principalPanel;
	}
	
	/**
	 * This method initializes westPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getEmptyPanel() {
		if (emptyPanel == null) {
			emptyPanel = new JPanel();
			emptyPanel.setMinimumSize(new Dimension(1024, 520));
			emptyPanel.setMaximumSize(new Dimension(1024, 520));
			emptyPanel.setPreferredSize(new Dimension(1024, 520));
		}
		return emptyPanel;
	}
	public void setProblem() {
		String problemFile = DisChoco.filePath;
		InstanceParser parser = new InstanceParser(problemFile);
		parser.loadInstance(problemFile);
		parser.parse(false);
		String benchmark = parser.getBenchmark();
		if (benchmark.equals(Commands.RandomDisCSP)) { 
			getPrincipalPanel().add(new RBDisCSPPanel(getPrincipalPanel().getSize()), Random_DisCSP_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), Random_DisCSP_PANEL);
		} else if (benchmark.equals(Commands.RandomMaxDisCSP)) { 
			getPrincipalPanel().add(new RBDisCSPPanel(getPrincipalPanel().getSize()), Random_MaxDisCSP_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), Random_MaxDisCSP_PANEL);
		} else if (benchmark.equals(Commands.GSensorDCSP) || benchmark.equals(Commands.SensorDCSP) ) {
			getPrincipalPanel().add(new SensorNetworkPanel(getPrincipalPanel().getSize()), SN_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), SN_PANEL);
		} else if (benchmark.equals(Commands.DMSP)) {
			getPrincipalPanel().add(new MeetingSchedulingPanel(getPrincipalPanel().getSize()), MS_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), MS_PANEL);
		} else if (benchmark.equals(Commands.RMSP) || benchmark.equals(Commands.DIMES)) {
			getPrincipalPanel().add(new RandomMSPPanel(getPrincipalPanel().getSize()), RMS_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), RMS_PANEL);
		} else if (benchmark.equals(Commands.RandomDCOP)) {
			getPrincipalPanel().add(new RBDCOPPanel(getPrincipalPanel().getSize()), Random_DCOP_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), Random_DCOP_PANEL);
		} else if (benchmark.equals(Commands.GraphColoring)) {
			getPrincipalPanel().add(new RBDCOPPanel(getPrincipalPanel().getSize()), Random_GC_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), Random_GC_PANEL);
		} else if (benchmark.equals(Commands.SoftGraphColoring)) {
			getPrincipalPanel().add(new RBDCOPPanel(getPrincipalPanel().getSize()), Random_SoftGC_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), Random_SoftGC_PANEL);
		} else if (benchmark.equals(DataSetTeamcore.TEAMCORE)) {
			getPrincipalPanel().add(new RBDCOPPanel(getPrincipalPanel().getSize()), Random_DCOP_PANEL);
			CardLayout cards = (CardLayout) (getPrincipalPanel().getLayout());
			cards.show(getPrincipalPanel(), Random_DCOP_PANEL);
		} else {
			System.err.println("Todo for : " + benchmark);
		}
	}
}
