package kernel;

import kernel.com.Message;

/**
 * A class for modularly adding functionality/observability to the Master. It is added to Master as a Plugin to allow access to the protected member
 * functions.
 * 
 * @author Redouane EZZAHIR
 * 
 */
public abstract class MasterObserver {
	/**
	 * the master to monitor its execution by the observer
	 */

	protected AbstractMaster master;

	/**
	 * Set the master to observe
	 * 
	 * @param master
	 */

	public void setMaster(AbstractMaster master) {
		this.master = master;
	}

	/**
	 * This function is called after the master executes <code>treatReceivedMsgs()</code>, but before the message is handled.
	 * 
	 * @param msg
	 *            The message the master is trying to receive.
	 * @see AbstractMaster#treatReceivedMsgs()
	 */
	public abstract boolean handleReceivedMessage(Message msg);

	/**
	 * This function is called after the master executes <code>sendMessage()</code>, but before the message is actually sent.
	 * 
	 * @param msg
	 *            The message the agent is trying to send.
	 * 
	 * @see Master#sendMessage(kernel.com.Message)
	 */
	public abstract void handleSendMessage(Message msg);

}
