package kernel;

import kernel.AbstractAgent;
import kernel.VarIdentifier;
import kernel.events.VariableEventTrigger;
import util.IntArray;

/**
 * Interface Variable defines all the methods for a variable to be used on a distributed problem.
 * 
 * @author Redouane EZZAHIR
 */
public interface Variable extends VariableEventTrigger {

	public AbstractAgent getAgentOwner();

	public VarIdentifier getVarIdentifier();

	public int getAgentId();

	public String getName();

	/**
	 * Checks whether the variable is instantiated or not
	 */
	public boolean isInstantiated();

	/**
	 * remove the current assigned value of the variable It unassigns the variable
	 */
	public void desinstanciate(int index);

	/**
	 * Returns the time counter of the last assignment of the variable
	 */
	public int getTimeCounter();

	/**
	 * Set the time counter to the given value
	 */
	public void setTimeCounter(int counter);

	/**
	 * Returns a string that represents the variable
	 */
	public String toString();

	/**
	 * Returns the domain size of the variable
	 * 
	 */
	public int getDomainSize();

	public IntArray getListeners(int evtType);

	public Object getDomain();

}
