package kernel.com;

import kernel.VarIdentifier;
import util.HashList;

/**
 * Interface CommunicationInterface defines the necessary methods for managing message communication: sendMessage, receiveMessage and
 * broadcastMessage.
 * <p>
 * All communication system must implement this interface
 * <p>
 * 
 */
public abstract interface CommunicationInterface {
	/**
	 * Send one message
	 * 
	 * @param msg
	 *            message to send
	 */
	public void sendMessage(Message msg);

	/**
	 * Checks whether the agent of the given ID is already registered or not
	 * 
	 * @param agentId
	 *            an agent ID
	 * @return boolean : true if agent is already inscribed
	 */
	public boolean isRegistered(VarIdentifier agentId);

	/**
	 * Register the agent of the given ID as an known agent by the communication system
	 * 
	 * @param agentId
	 */
	public void register(VarIdentifier agentId);

	/**
	 * Returns all IDs of all registered agents
	 */
	public HashList<VarIdentifier> getRegisteredAgentIds();

	/**
	 * Checks whether the end of search is reached or not
	 */
	public boolean checkEnd();

}
