package kernel.com;

import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.StopException;
import kernel.events.AgentEventListener;
import util.Commands;

/**
 * This class implements MessagesHandler, it processes message if it contains a termination detecting information.
 * 
 * 
 * @author Redouane EZZAHIHR
 * 
 */
public abstract class TerminateMsgHandler extends MessageHandler implements AgentEventListener {

	protected boolean terminate = false;

	/**
	 * Handle a message of termination detection.
	 * 
	 * @return <code>true</code> if the message is consumed, <code>false</code> otherwise.
	 */
	public abstract boolean processTerminationDetection(Message msg) throws StopException;

	/**
	 * Constructs a Handler for Message of termination detection
	 */

	public TerminateMsgHandler(AbstractAgent agent) {
		super(agent, Commands.TERMINATION_DETECTION);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see MessageHandler#processMessage(Message)
	 */
	public void processMessage(Message msg) throws StopException {
		String endState = (String) msg.get(Commands.TERMINATE);
		if (endState != null) {
			this.terminate = true;
			throw new StopException(endState);
		} else {
			processTerminationDetection(msg);
		}
	}

	/**
	 * Use this method to ask Master to broadcast System Message (Termination)
	 * 
	 * @see #whenAgentTerminate()
	 */
	public void askMasterToTerminate(boolean broadcast) {
		if (!this.terminate) {
			Message msg = this.createMessage(Commands.MASTER);
			msg.addContents(Commands.BROADCAST_SYSTEM_MSG, new Boolean(broadcast));
			msg.addContents(Commands.END_STATE, this.myAgent.endState);
			msg.setReceiverId(AbstractMaster.MasterVarId);
			this.myAgent.sendMessage(msg);
			this.terminate = true;
		}
	}
}
