/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package kernel.constraints.integer;

/**
 * The interface defines the methods required on the binary relation. 
 * 
 * @version DisChoco 2.5.3<br/>
 * kernel.constraints.integer.BinRelation.java
 * @author Mohamed Wahbi
 */
public interface BinRelation {

	/**
	 * This method checks the consistency of the couple (<tt>value_x</tt>, <tt>value_y</tt>). 
	 * 
	 * @param value_x : the value of the first variable.
	 * @param value_y : the value of the second variable.
	 * @return true if (value_x, value_y) is a consistent couple.
	 */
	public boolean isConsistent(int value_x, int value_y);
	
	/**
	 * This method returns the number of supports for the given value <tt>value</tt> 
	 * in the domain of the second variable (i.e., y).
	 * 
	 * @param value : the value of the first variable.
	 * @return int : the number of support of the first variable's value. 
	 */
	public int getNbOfSupports(int value);
	
	/**
	 * This method returns the number of conflicts for the given value (i.e., <tt>value</tt>) 
	 * in the domain of the second variable (i.e., y). 
	 * 
	 * @param value : the value of the first variable.
	 * @return int : the number of conflicts of the first variable's value. 
	 */
	public int getNbOfConflicts(int value);


}
