package kernel.constraints.integer;

import java.util.Iterator;
import java.util.List;
import kernel.Constraint;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;

public class ExtensionBinConstraint extends SIntBinConstraint {

	public BinRelation relation;
	
//	/**
//	 * 
//	 * @param type : 
//	 * @param var0 : 
//	 * @param var1 : 
//	 * @param couples
//	 */
//	public SoftExtensionBinConstraint(String type, IntVariable var0, IntExternalVar var1, List<int[]> couples) {
//		super(var0, var1);
//		relation = makeBinaryRelation(var0, var1, couples, type.equals(feasP));
//		this.type = type;
//	}
	
	/**
	 * 
	 * @param type : 
	 * @param var0 : 
	 * @param var1 : 
	 * @param couples
	 */
	public ExtensionBinConstraint(String type, String name, boolean feas, IntVariable var0, IntExternalVar var1, int[][] tuples, boolean transpose) {
		super(var0, var1);
		relation = makeBinaryRelation(feas, var0, var1, tuples, transpose);
		this.name = name;
		this.type = type;
	}
	
	/**
	 * 
	 * @param type : 
	 * @param var0 : 
	 * @param var1 : 
	 * @param couples
	 */
	public ExtensionBinConstraint(String name, boolean feas, IntVariable var0, IntExternalVar var1, int[][] tuples, boolean transpose) {
		super(var0, var1);
		relation = makeBinaryRelation(feas, var0, var1, tuples, transpose);
		this.name = name;
		this.type = feas? Constraint.feasP : Constraint.infeasP;
	}
	
	/**
	 * 
	 * @param v0 : 
	 * @param v1 : 
	 * @param couples : 
	 * @param feas : 
	 * @return
	 */
	public static BinRelation makeBinaryRelation(boolean feas, IntVariable v0, IntExternalVar v1, int[][] tuples, boolean transpose) {
		int n0 = v0.getSup() - v0.getInf() + 1;
		int n1 = v1.getSup() - v1.getInf() + 1;
		int inf0 = v0.getInf();
		int inf1 = v1.getInf();
		BinRelation relation = new CouplesTable(feas, inf0, inf1, n0, n1, transpose);
		for (int[] tuple : tuples) {
			if (tuple.length != 2)
				throw new Error("Wrong dimension : " + tuple.length + " for a tuple");
			((CouplesTable) relation).setCouple(tuple[0], tuple[1]);
		}
		return relation;
	}	/**
	 * 
	 * @param v0 : 
	 * @param v1 : 
	 * @param couples : 
	 * @param feas : 
	 * @return
	 */
	public static BinRelation makeBinaryRelation(IntVariable v0, IntVariable v1, List<int[]> couples, boolean feas, boolean transpose) {
		int n0 = v0.getSup() - v0.getInf() + 1;
		int n1 = v1.getSup() - v1.getInf() + 1;
		int inf0 = v0.getInf();
		int inf1 = v1.getInf();
		BinRelation relation = new CouplesTable(feas, inf0, inf1, n0, n1, transpose);
		Iterator<int[]> it = couples.iterator();
		while (it.hasNext()) {
			int[] tuple = (int[]) it.next();
			if (tuple.length != 2)
				throw new Error("Wrong dimension : " + tuple.length + " for a tuple");
			((CouplesTable) relation).setCouple(tuple[0], tuple[1]);
		}
		return relation;
	}

//	/**
//	 * @param soft
//	 * @param var
//	 * @param var2
//	 * @param couples
//	 * @param costs
//	 * @param defaultCost
//	 */
//	public ExtensionBinConstraint(String type, IntVariable v0, IntExternalVar v1, Integer[] weights) {
//		super(v0, v1);
//		relation = makeBinarySoftRelation(v0, v1, weights);
//		this.type = type;
//	}
//	
//	public static BinRelation makeBinarySoftRelation(IntVariable v0, IntVariable v1, Integer[] weights) {
//		int n0 = v0.getSup() - v0.getInf() + 1;
//		int n1 = v1.getSup() - v1.getInf() + 1;
//		int inf0 = v0.getInf();
//		int inf1 = v1.getInf();
//		return new SoftCouplesTable(inf0, inf1, n0, n1, weights);
//	}
//	/**
//	 * 
//	 * @param v0 : 
//	 * @param v1 : 
//	 * @param couples : 
//	 * @param feas : 
//	 * @return
//	 */
//	public static BinRelation makeBinarySoftRelation(IntVariable v0, IntVariable v1, List<int[]> couples, boolean feas, List<Integer> costs, Integer defaultCost) {
//		int n0 = v0.getSup() - v0.getInf() + 1;
//		int n1 = v1.getSup() - v1.getInf() + 1;
//		int inf0 = v0.getInf();
//		int inf1 = v1.getInf();
//		BinRelation relation = new SoftCouplesTable(feas, inf0, inf1, n0, n1, defaultCost);
//		for(int i = 0 ; i < couples.size() ; i++) {
//			int[] tuple = couples.get(i);
//			if (tuple.length != 2)
//				throw new Error("Wrong dimension : " + tuple.length + " for a tuple");
//			int cost = costs.get(i);
//			((SoftCouplesTable) relation).setCouple(tuple[0], tuple[1], cost);
//		}
//		return relation;
//	}

	@Override
	public void propagate() {
		awakeOnVar(v0);
		awakeOnVar(v1);
	}

	public void awakeOnVar(IntVariable v) {
		Iterator<Integer> it = v.getDomain().iterator();
		while (it.hasNext()) {
			int value = it.next();
			int support = this.findSupport(value, v);
			if (support == Integer.MIN_VALUE) {
				v.removeValue(value, index);
			}
		}
	}

	/**
	 * @param value
	 */
	public int getNumberOfSupport(int value) {
		return this.relation.getNbOfSupports(value);
	}

	/**
	 * @param value
	 * @return
	 */
	public int getNumberOfConflicts(int value) {
		return this.relation.getNbOfConflicts(value);
	}


	/* (non-Javadoc)
	 * @see simple.SimpleConstraint#isConsistent(int, int)
	 */
	@Override
	public boolean isConsistent(int val0, int val1) {
		this.incrementNumConstraintChecks(1);
		return this.relation.isConsistent(val0, val1);
	}
	
	/**
	 * Returns a string representation of this object
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("C_" + this.getIndex());
		sb.append("[ " + this.v0.toString() + "@" + type + "@" + this.v1.toString() + "]  ");
		sb.append(", posted=" + this.isPosted());
		sb.append(", satisfied=" + this.isSatisfied());
//		if(relation instanceof SoftCouplesTable){
//			sb.append(" " + this.getCost());
//		} else {
//			sb.append(" " + this.isSatisfied());
//		}
		return  sb.toString();
	}
//
//	/**
//	 * @return
//	 */
//	private String getCost() {
//		if(this.v0.isInstantiated() && this.v1.isInstantiated()){
//			return Integer.toString(this.relation.getCost(this.v0.getValue(), this.v1.getValue()));
//		}
//		return new String("");
//	}
}
