package kernel.constraints.integer;

import java.util.Iterator;
import kernel.vars.integer.IntDomain;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;

/**
 * The absolute constraint v0 = |v1 + cste| is a binary constraint with v0 = var0, v1 = var1 and cste is an integer constant
 */
public class SAbsolute extends SIntBinConstraint {

	// private boolean propagateFirstCall = true;

	public SAbsolute(IntVariable var0, IntExternalVar var1) {
		super(var0, var1);
	}

	public SAbsolute(IntVariable var0, IntExternalVar var1, int cste) {
		super(var0, var1, cste);
	}

	@Override
	public boolean isConsistent(int val1, int val2) {
		return val1 == Math.abs(val2 + cste);
	}

	@Override
	public void propagate() {
		v0.updateInf(0, index);
		// if (v0.getDomain().isEnumerated()) {
		IntDomain dom0 = v0.getDomain();
		Iterator<Integer> it = dom0.iterator();
		while (it.hasNext()) {
			int val = it.next();
			if (!v1.canBeInstantiatedTo(val - cste) && !v1.canBeInstantiatedTo(-val - cste)) {
				// this.prune(v0, val);
				v0.removeValue(val, index);
			}
		}

		// } else {
		// awakeOnInf(1);
		// awakeOnSup(1);
		// }
		// if (v1.getDomain().isEnumerated()) {
		IntDomain dom1 = v1.getDomain();
		it = dom1.iterator();
		while (it.hasNext()) {
			int val = it.next();
			if (!v0.canBeInstantiatedTo(val + cste) && !v0.canBeInstantiatedTo(-val - cste)) {
				// this.prune(v1, val);
				// this.prune(v1, -val);
				v1.removeValue(val, index);
				v1.removeValue(-val, index);
			}
		}
		// } else {
		// awakeOnInf(0);
		// awakeOnSup(0);
		// }
	}

	@Override
	public void awakeOnVar(IntVariable v) {
		if (v1.equals(v)) {
			v0.updateInf(0, index);
			// if (v0.getDomain().isEnumerated()) {
			IntDomain dom0 = v0.getDomain();
			Iterator<Integer> it = dom0.iterator();
			while (it.hasNext()) {
				int val = it.next();
				if (!v1.canBeInstantiatedTo(val - cste) && !v1.canBeInstantiatedTo(-val - cste)) {
					// this.prune(v0, val);
					v0.removeValue(val, index);
				}
			}
		}

		// } else {
		// awakeOnInf(1);
		// awakeOnSup(1);
		// }
		// if (v1.getDomain().isEnumerated()) {
		if (v0.equals(v)) {
			IntDomain dom1 = v1.getDomain();
			Iterator<Integer> it = dom1.iterator();
			while (it.hasNext()) {
				int val = it.next();
				if (!v0.canBeInstantiatedTo(val + cste) && !v0.canBeInstantiatedTo(-val - cste)) {
					// this.prune(v1, val);
					// this.prune(v1, -val);
					v1.removeValue(val, index);
					v1.removeValue(-val, index);
				}
			}
		}
		// } else {
		// awakeOnInf(0);
		// awakeOnSup(0);
		// }
	}

}
