package kernel.constraints.integer;

import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;

/**
 * 
 * A Class models the external constraint | X - Y | operator C, where X is a single internal variable and Y is an external variable
 * 
 * @author Redouane EZZAHIR
 * 
 */
public class SDistanceXYC extends SIntBinConstraint {

	protected final int operator;

	/**
	 * Constructs a new SDistanceXYC constraint
	 * 
	 * @param var0
	 *            the internal variable
	 * @param var1
	 *            the external variable
	 * @param cste
	 *            a constant
	 * @param oper
	 *            An operator EQ, LT, GT, or NEQ
	 */
	public SDistanceXYC(IntVariable var0, IntExternalVar var1, int cste, int oper) {
		super(var0, var1, cste);
		this.operator = oper;
		if (oper != 0 && oper != 1 && oper != 2 && oper != 3) { throw new Error("operateur inconnu " + oper); }
		this.type = DISTANCE + "XYC";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simple.integer.constraints.SIntBinConstraint#isConsistent(int, int)
	 */
	@Override
	public boolean isConsistent(int internalVal, int val2) {
		this.incrementNumConstraintChecks(1);
		switch (operator) {
			case EQ :
				return Math.abs(internalVal - val2) == this.cste;
			case LT :
				return Math.abs(internalVal - val2) < cste;
			case GT :
				return Math.abs(internalVal - val2) > cste;
			case NEQ :
				return Math.abs(internalVal - val2) != cste;
			default :
				throw new Error("Unknown opperator ! " + operator + ":" + operatorToString());
		}

	}

	/**
	 * Returns a string representation of this object
	 */
	public String toString() {

		return "C_" + this.getIndex() + ":[ |" + this.v0.toString() + " - " + "{" + this.v1.toString() + "} |" + operatorToString() + " " + cste
				+ " ]";
	}

	protected String operatorToString() {
		if (operator == EQ) {
			return "=";
		} else if (operator == LT) {
			return "<";
		} else if (operator == GT) {
			return ">";
		} else if (operator == NEQ) { return "#"; }
		return null;
	}

	@Override
	public void propagate() {
	// TODO
	}

	@Override
	public void awakeOnVar(IntVariable v) {
	// TODO Auto-generated method stub
	}
}
