package kernel.constraints.integer;

import java.util.Iterator;
import kernel.Constraint;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;

public class SEqualXYC extends SIntBinConstraint {

	public SEqualXYC(IntVariable var0, IntExternalVar var1) {
		super(var0, var1);
		this.type = Constraint.eq;
	}

	public SEqualXYC(IntVariable var0, IntExternalVar var1, int cste) {
		super(var0, var1, cste);
		this.type = Constraint.eq;
	}

	@Override
	public boolean isConsistent(int val1, int val2) {
		this.incrementNumConstraintChecks(1);
		return val1 == val2 + cste;
	}

	@Override
	public void propagate() {
		this.reviseV0();
		this.reviseV1();
	}

	/**
	 * Returns a string representation of this object
	 */
	public String toString() {
		return this.getReference() + ":[ " + this.v0.toString() + " = " + this.v1.toString() + " + " + cste + " ]";
	}

	@Override
	public void awakeOnVar(IntVariable v) {
		if (v.equals(v0)) {
			this.reviseV1();
		} else {
			this.reviseV0();
		}
	}

	private void reviseV0() {
		Iterator<Integer> it = v0.getDomain().iterator();
		while (it.hasNext()) {
			int val = it.next();
			this.incrementNumConstraintChecks(1);
			if (!v1.getDomain().contains(val - cste)) {
				v0.removeValue(val, this.index);
			}
		}
	}

	private void reviseV1() {
		Iterator<Integer> it = v1.getDomain().iterator();
		while (it.hasNext()) {
			int val = it.next();
			this.incrementNumConstraintChecks(1);
			if (!v0.getDomain().contains(val + cste))
				v1.removeValue(val, this.index);
		}
	}

}
