package kernel.constraints.integer;

import kernel.AbstractConstraint;
import kernel.AbstractVar;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;
import simple.dcsp.SimpleConstraint;

public abstract class SIntTernConstraint extends AbstractConstraint implements SimpleConstraint {

	protected IntVariable v0;

	protected IntVariable v1;

	protected IntVariable v2;

	final int posInternalVar;

	/**
	 * Checks whether the given triplet is consistent or not.
	 */
	public abstract boolean isConsistent(int val0, int val1, int val2);

	/**
	 * Returns the cost of the given triplet.
	 */
	public abstract int getCostOf(int val0, int val1, int val2);

	public SIntTernConstraint(IntVariable var0, IntVariable var1, IntVariable var2) {
		this.v0 = var0;
		this.v1 = var1;
		this.v2 = var2;
		if (!(v0 instanceof IntExternalVar))
			posInternalVar = 0;
		else if (!(v1 instanceof IntExternalVar))
			posInternalVar = 1;
		else if (!(v1 instanceof IntExternalVar))
			posInternalVar = 2;
		else
			throw new Error("One of the variables of the constuctor arguments must an inernal variable");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simple.SimpleConstraint#getInternalVar()
	 */
	@Override
	public AbstractVar getInternalVar(int idx) {
		if (idx != 0) { throw new Error("second internal variable not allowed for simple model"); }
		if (this.posInternalVar == 0)
			return this.v0;
		if (this.posInternalVar == 1)
			return v1;

		return v2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbExternalVars()
	 */

	@Override
	public int getNbExternalVars() {
		return 2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbInternalVars()
	 */

	@Override
	public int getNbInternalVars() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbVars()
	 */

	@Override
	public int getNbVars() {
		return 3;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#isCompletlyInstantiated()
	 */
	@Override
	public boolean isCompletlyInstantiated() {
		return this.v0.isInstantiated() && this.v1.isInstantiated() && this.v2.isInstantiated();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simple.SimpleConstraint#isConsistent(int)
	 */
	@Override
	public boolean isConsistent(int internalVal) {
		if (posInternalVar == 0 && v1.isInstantiated() && v2.isInstantiated())
			return isConsistent(internalVal, this.v1.getValue(), v2.getValue());
		if (posInternalVar == 1 && v0.isInstantiated() && v2.isInstantiated())
			return isConsistent(this.v0.getValue(), internalVal, v2.getValue());
		if (posInternalVar == 2 && v0.isInstantiated() && v1.isInstantiated())
			return isConsistent(this.v0.getValue(), v1.getValue(), internalVal);
		this.incrementNumConstraintChecks(1);
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getExternalVar(int)
	 */

	@Override
	public AbstractVar getExternalVar(int idx) {
		if (idx == 0)
			return v1;
		else
			return v2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#isSatisfied()
	 */
	@Override
	public boolean isSatisfied() {
		if (!this.isCompletlyInstantiated()) {
			this.incrementNumConstraintChecks(1);
			return true;
		}
		return isConsistent(v0.getValue(), this.v1.getValue(), v2.getValue());
	}

	//	

}
