package kernel.constraints.integer;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

import kernel.AbstractConstraint;
import kernel.AbstractVar;
import kernel.ConstraintsHandler;
import kernel.Variable;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;
import simple.SimpleAgent;
import simple.dcsp.SimpleConstraint;
import simulation.model.expressions.BooleanExpression;
import simulation.model.expressions.VariableExpression;
import util.IntArray;

public class SPrimitiveConstraint extends AbstractConstraint implements SimpleConstraint {

	BooleanExpression predicate;

	/**
	 * the external variable
	 */
	protected IntExternalVar[] y;

	private VariableExpression xExp = null;

	SimpleAgent owner;

	public SPrimitiveConstraint(SimpleAgent owner, BooleanExpression predicate) {
		this.predicate = predicate;
		this.owner = owner;
		if (this.owner != null) {
			int i = 0;
			List<? extends VariableExpression> vars = predicate.getVariableExpressions();
			y = new IntExternalVar[predicate.getNbVariableExpression() - 1];
			for (VariableExpression var : vars) {
				if (var.getVariable() != null) {
					if (owner.getAgentId() != var.getVariable().getAgentId()){
						y[i++] = owner.makeExternalVar((IntVariable) var.getVariable());
					} else {
						xExp = var;
					}
				}
			}
		}
	}

	/**
	 * set the constraint handler as listener
	 * 
	 * @param listener
	 *            : the constraint handler
	 */
	public void setConstraintListener(ConstraintsHandler listener) {
		this.constraintHandler = listener;

	}

	private boolean isAllExternalVarsInstantiated() {
		for (IntExternalVar v : this.y)
			if (!v.isInstantiated())
				return false;
		return true;
	}

	@Override
	public boolean isConsistent(int internalVal) {
		this.incrementNumConstraintChecks(1);
		if (isAllExternalVarsInstantiated()) {
			xExp.setTemporaryValue(internalVal);
			boolean res = this.predicate.evaluate();
			xExp.removeTemporaryValue();
			return res;
		}
		return true;
	}

	@Override
	public boolean isConsistent(Variable v, int val) {
		throw new Error("TODO");
	}

	@Override
	public AbstractVar getExternalVar(int idx) {
		return this.y[idx];
	}

	@Override
	public AbstractVar getInternalVar(int idx) {
		if (this.owner != null)
			return this.owner.var;
		return null;
	}

	@Override
	public int getNbExternalVars() {
		return this.getNbVars() - 1;
	}

	@Override
	public int getNbInternalVars() {
		if (this.owner != null)
			return 1;
		return 0;
	}

	@Override
	public int getNbVars() {
		return this.predicate.getNbVariableExpression();
	}

	@Override
	public boolean isCompletlyInstantiated() {
		return this.owner.var.isInstantiated() && this.isAllExternalVarsInstantiated();
	}

	@Override
	public boolean isSatisfied() {
		return this.predicate.evaluate();
	}

	public String toString() {
		return this.predicate.getFunctionalExpression(0);
	}

	@Override
	public void propagate() {
	// TODO Auto-generated method stub
	}

	@Override
	public void awake() {
		for (Variable v : this.getVars()) {
			if (this.varEventSources.contains(v.getVarIdentifier())) {
				IntArray evtTypes = v.getEventsType(this.index);
				for (int e : evtTypes) {
					this.awakeOnVariableEvent(v, e);
				}
			}
		}
	}

	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {
	// TODO Auto-generated method stub

	}

	@Override
	public AbstractVar getVar(int i) {
		// TODO Auto-generated method stub
		return null;
	}
	
	@Override
	public Collection<AbstractVar> getExternalVars() {
		HashSet<AbstractVar> externalVars = new HashSet<AbstractVar>(Arrays.asList(this.y));
		return Collections.unmodifiableSet(externalVars);
	}

}
