package kernel.constraints.integer;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import kernel.AbstractConstraint;
import kernel.AbstractVar;
import kernel.Variable;
import kernel.vars.integer.IntVariable;
import simple.dcsp.SimpleConstraint;
import util.IntArray;

public class SUnaryIntConstraint extends AbstractConstraint implements SimpleConstraint {

	private IntVariable x;

	private int C = 0;

	/**
	 * @param var
	 * @param Type
	 * @param cste
	 */
	public SUnaryIntConstraint(String type, IntVariable var, int cste) {
		this.type = type;
		this.x = var;
		this.C = cste;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#isSatisfied()
	 */
	public boolean isSatisfied() {
		if (!this.isCompletlyInstantiated()) {
			this.incrementNumConstraintChecks(1);
			return true;
		}
		int xval = this.x.getValue();
		return this.isConsistent(xval);
	}

	/**
	 * @param xval
	 *            the value of the internal variable
	 */
	@Override
	public boolean isConsistent(int xval) {
		this.incrementNumConstraintChecks(1);
		if (type.equals(eq))
			return xval == this.C;
		else if (type.equals(neq))
			return xval != this.C;
		else if (type.equals(lt))
			return xval < this.C;
		else if (type.equals(gt))
			return xval > this.C;
		return true;

	}

	public void propagate() {
		Iterator<Integer> it = this.x.getDomain().iterator();
		while (it.hasNext()) {
			int value = it.next();
			if (!this.isConsistent(value)) {
				// this.x.delete(val, this.index);
				this.x.removeValue(value, this.index);
			}
		}
	}

	@Override
	public void awake() {
		if (this.varEventSources.contains(x.getVarIdentifier())) {
			IntArray evtTypes = x.getEventsType(this.index);
			for (int e : evtTypes) {
				this.awakeOnVariableEvent(x, e);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#isCompletlyInstantiated()
	 */
	public boolean isCompletlyInstantiated() {
		return this.x.isInstantiated();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simple.SimpleConstraint#getInternalVar()
	 */
	public IntVariable getInternalVar(int idx) {
		if (idx != 0){
			throw new IndexOutOfBoundsException("index : " + idx + " must be equals to zero");
		}
		return this.x;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbExternalVars()
	 */
	public int getNbExternalVars() {
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbInternalVars()
	 */
	public int getNbInternalVars() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbVars()
	 */
	public int getNbVars() {
		return 1;
	}

	public String toString() {
		return this.x.toString() + this.type + this.C;
	}

	/**
	 * Not used for the unary constraint
	 * 
	 * @throws UnsupportedOperationException
	 */
	public AbstractVar getExternalVar(int index) {
		throw new UnsupportedOperationException("There is not external variables in unary constraint");
	}

	/**
	 * Not used for the unary constraint
	 */
	public void setActiveOnInternalVars() {
		/* noting */
	}

	/**
	 * Not used for the unary constraint
	 */
	public void setPassiveOnInternalVars() {
		/* noting */
	}

	/**
	 * Not used for the unary constraint
	 */
	public void setActiveOnExternalVars() { 
		/* noting */
	}

	/**
	 * Not used for the unary constraint
	 */
	public void setPassiveOnExternalVars() { 
		/* noting */
	}

	@Override
	public void awakeOnVariableEvent(Variable var, int evtType) {
		/* noting */
	}

	@Override
	public boolean isConsistent(Variable var, int value) {
		if (var.equals(x)){
			return this.isConsistent(value);
		} else {
			throw new Error("The given var " + var + " is different from " + x);
		}
	}

	@Override
	public AbstractVar getVar(int idx) {
		assert (idx == 0) : "index : " + idx + " must be equals to zero";
		return x;
	}

	@Override
	public Collection<AbstractVar> getExternalVars() {
		return Collections.emptySet();
	}

}
