package kernel.constraints.integer.explain;

import java.util.Iterator;
import kernel.Variable;
import kernel.constraints.integer.ExtensionBinConstraint;
import kernel.events.VariableEvent;
import kernel.vars.integer.explain.ExplainedIntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import kernel.vars.integer.explain.ExplainedIntVariable;

public class FBBinConstraint extends ExtensionBinConstraint implements Propagator {

	public FBBinConstraint(String name, boolean feas, ExplainedIntVariable var0, ExplainedIntExternalVar var1, int[][] tuples, boolean transpose) {
		super("FB::", name, feas, var0, var1, tuples, transpose);
	}

	public void awakeOnRestores(ExplainedIntVar var) {
		if (var.equals(v0)) {
			if (v1.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (!isConsistent(value, v1.getValue()))
						var.removeValue(value, index, index);
				}
			}
		} else if (var.equals(v1)) {
			if (v0.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (isConsistent(v0.getValue(), value))
						var.removeValue(value, index, index);
				}
			}
		}
	}

	public void propagate() {

	}

	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {
		this.incrementNumConstraintChecks(1);
		ExplainedIntVar var = (ExplainedIntVar) v;
		switch (evtType) {
			case VariableEvent.INSTANCIATE :
				this.awakeOnInst(var);
			break;
			case VariableEvent.DESINSTANCIATE :
				this.awakeOnDesInst(var);
			break;
			case VariableEvent.REMOVE_VAL :
				this.awakeOnRemovals(var);
			break;
			case VariableEvent.RESTORE_VAL :
				this.awakeOnRestores(var);
			break;
			case VariableEvent.DECREMENT_SUP :
				this.awakeOnDecSup(var);
			break;
			case VariableEvent.INCREMENT_INF :
				this.awakeOnIncrInf(var);
			break;
			default :
				return;
		}

	}

	@Override
	public void awakeOnInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			Iterator<Integer> itv1 = v1.getDomain().iterator();
			while (itv1.hasNext()) {
				int val = itv1.next();
				if (!isConsistent(v0.getValue(), val)) {
					((ExplainedIntVar) v1).removeValue(val, this.index, this.index);
				}
			}
		} else if (var.equals(v1)) {
			Iterator<Integer> itv0 = v0.getDomain().iterator();
			while (itv0.hasNext()) {
				int val = itv0.next();
				if (!isConsistent(val, v1.getValue())) {
					((ExplainedIntVar) v0).removeValue(val, this.index, this.index);
				}
			}
		}
	}

	@Override
	public void awakeOnDesInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			((ExplainedIntVar) v1).removeExplanation(index);
		} else if (var.equals(v1)) {
			((ExplainedIntVar) v0).removeExplanation(index);
		}
	}

	@Override
	public void awakeOnDecSup(ExplainedIntVar var) {}

	@Override
	public void awakeOnIncrInf(ExplainedIntVar var) {}

	@Override
	public void awakeOnRemovals(ExplainedIntVar var) {}

	@Override
	public void setUnPosted() {
		((ExplainedIntVar) v0).removeExplanation(this.index);
		((ExplainedIntVar) v1).removeExplanation(this.index);
	}

}
