package kernel.constraints.integer.explain;

import java.util.Collection;
import java.util.Iterator;

import kernel.Variable;
import kernel.constraints.integer.ExtensionBinConstraint;
import kernel.events.VariableEvent;
import kernel.vars.integer.IntVariable;
import kernel.vars.integer.explain.ExplainedIntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import kernel.vars.integer.explain.ExplainedIntVariable;
import kernel.vars.integer.explain.Explanation;
import util.IntArray;

public class SEAC2001BinConstraint extends ExtensionBinConstraint implements Propagator {

	protected int[] currentSupport0 = null;

	protected int[] currentSupport1 = null;

	protected int offset0;

	protected int offset1;

	public SEAC2001BinConstraint(String name, boolean feas, ExplainedIntVariable var0, ExplainedIntExternalVar var1, int[][] tuples, boolean transpose) {
		super("AC2001::", name, feas, var0, var1, tuples, transpose);
		init(var0, var1);
	}

	// ===================================================================================================

	private void init(IntVariable x0, IntVariable x1) {
		offset0 = x0.getInf();
		offset1 = x1.getInf();
		currentSupport0 = new int[x0.getSup() - x0.getInf() + 1];
		currentSupport1 = new int[x1.getSup() - x1.getInf() + 1];

		for (int i = 0 ; i < currentSupport0.length ; i++) {
			currentSupport0[i] = Integer.MIN_VALUE;
		}

		for (int i = 0 ; i < currentSupport1.length ; i++) {
			currentSupport1[i] = Integer.MIN_VALUE;
		}
	}

	// ===================================================================================================

	@Override
	public void setUnPosted() {
		super.setUnPosted();
		if (v1.isInstantiated())
			((ExplainedIntVar) v0).removeExplanation(this.index);
		if (v0.isInstantiated())
			((ExplainedIntVar) v1).removeExplanation(this.index);
	}

	// ===================================================================================================

	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {
		this.incrementNumConstraintChecks(1);
		ExplainedIntVar var = (ExplainedIntVar) v;
		switch (evtType) {
			case VariableEvent.INSTANCIATE :
				this.awakeOnInst(var);
			break;
			case VariableEvent.DESINSTANCIATE :
				this.awakeOnDesInst(var);
			break;
			case VariableEvent.REMOVE_VAL :
				this.awakeOnRemovals(var);
			break;
			case VariableEvent.RESTORE_VAL :
				this.awakeOnRestores(var);
			break;
			case VariableEvent.DECREMENT_SUP :
				this.awakeOnDecSup(var);
			break;
			case VariableEvent.INCREMENT_INF :
				this.awakeOnIncrInf(var);
			break;
			default :
				return;
		}
	}

	// ===================================================================================================

	@Override
	public void awakeOnInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			Iterator<Integer> itv1 = v1.getDomain().iterator();
			while (itv1.hasNext()) {
				int val = itv1.next();
				this.incrementNumConstraintChecks(1);
				if (!isConsistent(v0.getValue(), val)) {
					((ExplainedIntVar) v1).removeValue(val, index, index);
					currentSupport1[val - offset1] = Integer.MIN_VALUE;
				}
			}
		} else if (var.equals(v1)) {
			Iterator<Integer> itv0 = v0.getDomain().iterator();
			while (itv0.hasNext()) {
				int val = itv0.next();
				this.incrementNumConstraintChecks(1);
				if (!isConsistent(val, v1.getValue())) {
					((ExplainedIntVar) v0).removeValue(val, index, index);
					currentSupport0[val - offset0] = Integer.MIN_VALUE;
				}
			}
		}
	}

	// ===================================================================================================

	/**
	 * updates the support for all values in the domain of v0, and remove unsupported values for v0 @
	 */
	private void reviseV0() {
		Iterator<Integer> itv0 = v0.getDomain().iterator();
		while (itv0.hasNext()) {
			int val0 = itv0.next();
			this.incrementNumConstraintChecks(1);
			if (!v1.canBeInstantiatedTo(currentSupport0[val0 - offset0])) {
				updateSupportVal0(val0);
			}
		}
	}

	// ===================================================================================================

	private void updateSupportVal0(int val0) {
		Iterator<Integer> it = v1.getDomain().iterator();
		boolean update = false;
		while (it.hasNext()) {
			int val1 = it.next();
			if (this.isConsistent(val0, val1)) {
				currentSupport0[val0 - offset0] = val1;
				if (this.currentSupport1[val1 - offset1] == Integer.MIN_VALUE) {
					currentSupport1[val1 - offset1] = val0;
				}
				update = true;
				break;
			}
		}
		if (!update) {
			IntArray removedSupports = new IntArray();
			for(int val1 : v1.getDomain().removedValsIterator()) {
				if (this.isConsistent(val0, val1)) {
					removedSupports.add(val1);
				}
			}
			Collection<Explanation> expl = ((ExplainedIntVar) v1).getExplanations(removedSupports);
			// this.constraintHandler.myAgent.log( "remove "+val0+ " from Dom of "+ v0 +" because "+ expl, 1);
			((ExplainedIntVar) v0).removeValue(val0, this.index, expl);
		}
	}

	// ===================================================================================================

	private void updateSupportVal1(int val1) {
		Iterator<Integer> it = v0.getDomain().iterator();
		boolean update = false;
		while (it.hasNext()) {
			int val0 = it.next();
			if (this.isConsistent(val0, val1)) {
				currentSupport1[val1 - offset1] = val0;
				if (this.currentSupport0[val0 - offset0] == Integer.MIN_VALUE) {
					currentSupport1[val1 - offset0] = val1;
				}
				update = true;
				break;
			}
		}
		if (!update) {
			IntArray removedSupports = new IntArray();
			for(int val0 : v0.getDomain().removedValsIterator()) {
				if (this.isConsistent(val0, val1)) {
					removedSupports.add(val0);
				}
			}
			Collection<Explanation> expl = ((ExplainedIntVar) v0).getExplanations(removedSupports);
			// this.constraintHandler.myAgent.log( "remove "+val1+ " from Dom of "+ v1 +" because "+ expl, 1);
			((ExplainedIntVar) v1).removeValue(val1, this.index, expl);
			// ((ExplainedIntVar) v1).removeValue(val1, this.index, ((ExplainedIntVar) v0).getExplanations(removedSupports));
		}
	}

	// ===================================================================================================

	private void reviseV1() {
		Iterator<Integer> itv1 = v1.getDomain().iterator();
		while (itv1.hasNext()) {
			int val1 = itv1.next();
			if (!v0.canBeInstantiatedTo(currentSupport1[val1 - offset1])) {
				updateSupportVal1(val1);
			}
		}
	}

	// ===================================================================================================

	@Override
	public void awakeOnDesInst(ExplainedIntVar var) {
		// System.err.println(this.getReference()+ " awakeOnInst "+ var);
		if (var.equals(v0)) {
			((ExplainedIntVar) v1).removeExplanation(index);
		} else if (var.equals(v1)) {
			((ExplainedIntVar) v0).removeExplanation(index);
		}
	}

	// ===================================================================================================

	@Override
	public void awakeOnRemovals(ExplainedIntVar var) {
		reviseOnVar(var);
	}

	// ===================================================================================================

	private void reviseOnVar(ExplainedIntVar var) {
		if (var.equals(v0))
			this.reviseV1();
		else
			this.reviseV0();
	}

	// ===================================================================================================

	@Override
	public void awakeOnDecSup(ExplainedIntVar var) {
		reviseOnVar(var);
	}

	// ===================================================================================================

	@Override
	public void awakeOnIncrInf(ExplainedIntVar var) {
		reviseOnVar(var);
	}

	// ===================================================================================================

	@Override
	public void propagate() {
		reviseV0();
		reviseV1();
	}

	// ====================================================================================

	public void awakeOnRestores(ExplainedIntVar var) {
		Iterator<Integer> it = var.getDomain().getRepairIterator();
		// String values ="";
		// while(it.hasNext())
		// values +=it.next()+" ";
		// this.constraintHandler.myAgent.log(this.index + "  awakeOnRestores for "+var+ " "+values+ "    "+v0 +"  "+
		// v0.isInstantiated()+"    "+v1+" "+v1.isInstantiated(), 1);
		if (var.equals(v0)) {
			if (v1.isInstantiated()) {
				it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (!isConsistent(value, v1.getValue()))
						var.removeValue(value, index, index);
				}
			} else {
				it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int val0 = it.next();
					this.updateSupportVal0(val0);
				}
			}
		} else if (var.equals(v1)) {
			if (v0.isInstantiated()) {
				it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (isConsistent(v0.getValue(), value))
						var.removeValue(value, index, index);
				}
			} else {
				it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int val1 = it.next();
					this.updateSupportVal1(val1);
				}
			}
		}
	}

}
