package kernel.constraints.integer.explain;

import java.util.Iterator;
import kernel.Variable;
import kernel.constraints.integer.SDistanceXYC;
import kernel.events.VariableEvent;
import kernel.vars.integer.IntDomain;
import kernel.vars.integer.explain.ExplainedIntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import kernel.vars.integer.explain.ExplainedIntVariable;
import util.IntArray;

/**
 * 
 * A Class models the external constraint | X - Y | operator C, where X is a single internal variable and Y is an external variable
 * 
 * @author Redouane EZZAHIR
 * 
 */
public class SEDistanceXYC extends SDistanceXYC implements Propagator {

	public SEDistanceXYC(ExplainedIntVariable var0, ExplainedIntExternalVar var1, int cste, int oper) {
		super(var0, var1, cste, oper);
	}

	// ========================================================================

	@Override
	public void awakeOnInst(ExplainedIntVar var) {
		if (operator == EQ) {
			if (var.equals(v0)) {
				filterOnInst((ExplainedIntVar) v1, v0.getValue());
			} else if (var.equals(v1)) {
				filterOnInst((ExplainedIntVar) v0, v1.getValue());
			}
		} else if (operator == GT) {
			if (var.equals(v0)) {
				((ExplainedIntVar) v1).removeInterval(v0.getValue() - cste, v0.getValue() + cste, index, index);
			} else if (var.equals(v1)) {
				((ExplainedIntVar) v0).removeInterval(v1.getValue() - cste, v1.getValue() + cste, index, index);
			}
		} else if (operator == LT) {
			if (var.equals(v0)) {
				((ExplainedIntVar) v1).updateInf(v0.getValue() - cste + 1, index, index);
				((ExplainedIntVar) v1).updateSup(v0.getValue() + cste - 1, index, index);
			} else if (var.equals(v1)) {
				((ExplainedIntVar) v0).updateInf(v1.getValue() - cste + 1, index, index);
				((ExplainedIntVar) v0).updateSup(v1.getValue() + cste - 1, index, index);
			}
		} else {
			filterNeq();
		}
	}

	// ========================================================================

	@Override
	public void awakeOnDecSup(ExplainedIntVar var) {
		if (operator == EQ) {
			if (var.equals(v0)) {
				filterOnSup((ExplainedIntVar) v0, (ExplainedIntVar) v1);
			} else if (var.equals(v1)) {
				filterOnSup((ExplainedIntVar) v1, (ExplainedIntVar) v0);
			}
		} else if (operator == GT) {
			if (var.equals(v0)) {
				filterGTonVar((ExplainedIntVar) v0, (ExplainedIntVar) v1);
			} else if (var.equals(v1)) {
				filterGTonVar((ExplainedIntVar) v1, (ExplainedIntVar) v0);
			}
		} else if (operator == LT) {
			if (var.equals(v0)) {
				filterLTonVar((ExplainedIntVar) v0, (ExplainedIntVar) v1);
			} else if (var.equals(v1)) {
				filterLTonVar((ExplainedIntVar) v1, (ExplainedIntVar) v0);
			}
		} else {
			filterNeq();
		}
	}

	// ========================================================================

	@Override
	public void awakeOnIncrInf(ExplainedIntVar var) {
		if (operator == EQ) {
			if (var.equals(v0)) {
				filterOnInf((ExplainedIntVar) v0, (ExplainedIntVar) v1);
			} else if (var.equals(v1)) {
				filterOnInf((ExplainedIntVar) v1, (ExplainedIntVar) v0);
			}
		} else if (operator == GT) {
			if (var.equals(v0)) {
				filterGTonVar((ExplainedIntVar) v0, (ExplainedIntVar) v1);
			} else if (var.equals(v1)) {
				filterGTonVar((ExplainedIntVar) v1, (ExplainedIntVar) v0);
			}
		} else if (operator == LT) {
			if (var.equals(v0)) {
				filterLTonVar((ExplainedIntVar) v0, (ExplainedIntVar) v1);
			} else if (var.equals(v1)) {
				filterLTonVar((ExplainedIntVar) v1, (ExplainedIntVar) v0);
			}
		} else {
			filterNeq();
		}

	}

	// ========================================================================

	/**
	 * In case of a EQ, due to the instantiation to one variable to val @
	 */
	public void filterOnInst(ExplainedIntVar v, int val) {
		if (!v.canBeInstantiatedTo(val + cste)) {
			v.restrictDomainTo(val - cste, index, index);
		} else if (!v.canBeInstantiatedTo(val - cste)) {
			v.restrictDomainTo(val + cste, index, index);
		} else {
			// Iterator<Integer> it = v.getDomain().iterator();
			// for (; it.hasNext();) {
			// int value = it.next();
			// if (value != (val - cste) && value != (val + cste)) {
			// v.removeValue(value, index);
			// }
			// }
			int min = (val - cste);
			int max = (val + cste);
			v.updateInf(min, index, index);
			v.updateSup(max, index, index);
			v.removeInterval(min + 1, max - 1, index, index);
		}
	}

	// ========================================================================

	private void unfilterOnDesInst(ExplainedIntVar vv0, ExplainedIntVar vv1) {
		if (vv1.getDomainSize() == 1) {
			Iterator<Integer> it = vv1.getDomain().originalDomainIterator();
			for ( ; it.hasNext() ;) {
				int value = it.next();
				this.incrementNumConstraintChecks(1);
				if (vv0.canBeInstantiatedTo(value - cste) || vv0.canBeInstantiatedTo(value + cste)) {
					vv1.removeExplanation(value, index);
				}
			}
		}
		if (vv1.getDomainSize() == 2) {
			vv1.removeExplanationForInterval(vv0.getPreviousValue() - cste, vv0.getPreviousValue() + cste, index);
			IntDomain dom = vv1.cloneOriginalDomain();
			int value = dom.getPrevValue(vv0.getPreviousValue() - cste);
			for ( ; value != Integer.MIN_VALUE ; value = dom.getPrevValue(value - 1)) {
				this.incrementNumConstraintChecks(1);
				if (vv0.canBeInstantiatedTo(value - cste) || vv0.canBeInstantiatedTo(value + cste)) {
					vv1.removeExplanation(value, index);
				}
			}
			value = dom.getNextValue(vv0.getPreviousValue() + cste);
			for ( ; value != Integer.MAX_VALUE ; value = dom.getPrevValue(value + 1)) {
				this.incrementNumConstraintChecks(1);
				if (vv0.canBeInstantiatedTo(value - cste) || vv0.canBeInstantiatedTo(value + cste)) {
					vv1.removeExplanation(value, index);
				}
			}
		}
	}

	// ========================================================================

	@Override
	public void awakeOnDesInst(ExplainedIntVar var) {
		if (operator == EQ) {
			if (var.equals(v0)) {
				unfilterOnDesInst((ExplainedIntVar) v1, (ExplainedIntVar) v0);
			} else if (var.equals(v1)) {
				unfilterOnDesInst((ExplainedIntVar) v0, (ExplainedIntVar) v1);
			}
		} else if (operator == GT) {
			if (var.equals(v0)) {
				// v1.removeInterval(v0.getValue() - cste, v0.getValue() + cste,
				// index);
				((ExplainedIntVar) v1).removeExplanationForInterval(v0.getPreviousValue() - cste, v0.getPreviousValue() + cste, index);
			} else if (var.equals(v1)) {
				// v0.removeInterval(v1.getValue() - cste, v1.getValue() + cste,
				// index);
				((ExplainedIntVar) v0).removeExplanationForInterval(v1.getPreviousValue() - cste, v1.getPreviousValue() + cste, index);
			}
		} else if (operator == LT) {
			if (var.equals(v0)) {
				// v1.updateInf( v0.getValue() - cste + 1, index);
				int min = minConsistentValue(((ExplainedIntVar) v1), ((ExplainedIntVar) v0));
				((ExplainedIntVar) v1).removeExplanationForInterval(min, v0.getPreviousValue() - cste + 1, index);
				// v1.updateSup( v0.getValue() + cste - 1, index);
				int max = maxConsistentValue(((ExplainedIntVar) v1), ((ExplainedIntVar) v0));
				((ExplainedIntVar) v1).removeExplanationForInterval(v0.getPreviousValue() + cste - 1, max, index);
			} else if (var.equals(v1)) {
				// v0.updateInf( v1.getValue() - cste + 1, index);
				int min = minConsistentValue(((ExplainedIntVar) v0), ((ExplainedIntVar) v1));
				((ExplainedIntVar) v0).removeExplanationForInterval(min, v1.getPreviousValue() - cste + 1, index);
				// v0.updateSup( v1.getValue() + cste - 1, index);
				int max = maxConsistentValue(((ExplainedIntVar) v0), ((ExplainedIntVar) v1));
				((ExplainedIntVar) v0).removeExplanationForInterval(v1.getPreviousValue() + cste - 1, max, index);
			}
		} else if (operator == NEQ) { // filterNeq(revisev0, revisev1);
			if (var.equals(v0)) {
				((ExplainedIntVar) v1).removeExplanation(v0.getPreviousValue() + cste, index);
				((ExplainedIntVar) v1).removeExplanation(v0.getPreviousValue() - cste, index);
			} else if (var.equals(v1)) {
				((ExplainedIntVar) v0).removeExplanation(v1.getPreviousValue() + cste, index);
				((ExplainedIntVar) v0).removeExplanation(v1.getPreviousValue() - cste, index);
			}
		}
	}

	// ========================================================================

	private int maxConsistentValue(ExplainedIntVar vv0, ExplainedIntVar vv1) {
		IntDomain dom = vv0.cloneOriginalDomain();
		int value = dom.getSup();
		for ( ; value != Integer.MIN_VALUE ; value = dom.getPrevValue(value - 1)) {
			this.incrementNumConstraintChecks(1);
			if (vv1.canBeInstantiatedTo(value - cste) || vv1.canBeInstantiatedTo(value + cste)) { return value; }
		}
		return value;
	}

	// ========================================================================

	private int minConsistentValue(ExplainedIntVar vv0, ExplainedIntVar vv1) {
		IntDomain dom = vv0.cloneOriginalDomain();
		int value = dom.getInf();
		for ( ; value != Integer.MAX_VALUE ; value = dom.getNextValue(value + 1)) {
			this.incrementNumConstraintChecks(1);
			if (vv1.canBeInstantiatedTo(value - cste) || vv1.canBeInstantiatedTo(value + cste)) { return value; }
		}
		return value;
	}

	// ========================================================================

	@Override
	public void awakeOnRemovals(ExplainedIntVar var) {
		Iterator<Integer> it = var.getDomain().getDeltaIterator();
		if (operator == EQ) {
			if (var.equals(v0)) {
				while (it.hasNext()) {
					int x = it.next();
					this.incrementNumConstraintChecks(1);
					if (!v0.canBeInstantiatedTo(x + 2 * cste)) {
						((ExplainedIntVar) v1).removeValue(x + cste, index, var.getExplanation(x + 2 * cste));
					}
					if (!v0.canBeInstantiatedTo(x - 2 * cste)) {
						((ExplainedIntVar) v1).removeValue(x - cste, index, var.getExplanation(x - 2 * cste));
					}
				}
			} else if (var.equals(v1)) {
				while (it.hasNext()) {
					int x = it.next();
					this.incrementNumConstraintChecks(1);
					if (!v1.canBeInstantiatedTo(x + 2 * cste)) {
						((ExplainedIntVar) v0).removeValue(x + cste, index, var.getExplanation(x + 2 * cste));
					}
					if (!v1.canBeInstantiatedTo(x - 2 * cste)) {
						((ExplainedIntVar) v0).removeValue(x - cste, index, var.getExplanation(x - 2 * cste));
					}
				}
			}
		} else if (operator == NEQ) {
			filterNeq();
		}
	}

	// ========================================================================

	/**
	 * In case of a LT, due to a modification on vv0 domain
	 */
	public void filterLTonVar(ExplainedIntVar vv0, ExplainedIntVar vv1) {
		((ExplainedIntVar) vv1).updateInf(vv0.getInf() - cste + 1, index, vv0.getInfExplanations());
		((ExplainedIntVar) vv1).updateSup(vv0.getSup() + cste - 1, index, vv1.getSupExplanations());
	}

	// ========================================================================

	/**
	 * In case of a EQ, due to a modification of the lower bound of vv0 @
	 */
	public void filterOnInf(ExplainedIntVar vv0, ExplainedIntVar vv1) {
		IntDomain dom = vv1.getDomain();
		int end = vv0.getInf() + cste;
		for (int val = vv0.getInf() ; val <= end ; val = dom.getNextValue(val)) {
			this.incrementNumConstraintChecks(1);
			if (!vv0.canBeInstantiatedTo(val - cste) && !vv0.canBeInstantiatedTo(val + cste)) {
				vv1.removeValue(val, index, vv0.getExplanations(new IntArray(new int[] {val - cste, val + cste})));
			}
		}
	}

	// ========================================================================

	/**
	 * In case of a EQ, due to a modification of the upper bound of vv0 @
	 */
	public void filterOnSup(ExplainedIntVar vv0, ExplainedIntVar vv1) {
		IntDomain dom = vv1.getDomain();
		int initval;
		if (vv0.getSup() - cste > vv1.getInf()) {
			initval = dom.getNextValue(vv0.getSup() - cste - 1);
		} else {
			initval = vv1.getInf();
		}
		int val = initval;
		do {
			this.incrementNumConstraintChecks(1);
			if (!vv0.canBeInstantiatedTo(val - cste) && !vv0.canBeInstantiatedTo(val + cste)) {
				vv1.removeValue(val, index, vv0.getExplanations(new IntArray(new int[] {val - cste, val + cste})));
			}
			val = dom.getNextValue(val);
		} while (val <= vv1.getSup() && val > initval);

	}

	// ========================================================================

	@Override
	public void propagate() {
		if (operator == EQ) {
			filterFromVarToVar((ExplainedIntVar) v0, (ExplainedIntVar) v1);
			filterFromVarToVar((ExplainedIntVar) v1, (ExplainedIntVar) v0);
		} else if (operator == GT) {
			filterGT();
		} else if (operator == LT) {
			filterLT();
		} else {
			filterNeq();
		}
	}

	// ========================================================================

	/**
	 * Initial propagation in case of EQ @
	 */
	protected void filterFromVarToVar(ExplainedIntVar var1, ExplainedIntVar var2) {
		Iterator<Integer> it = var1.getDomain().iterator();
		for ( ; it.hasNext() ;) {
			int val = it.next();
			this.incrementNumConstraintChecks(1);
			if (!var2.canBeInstantiatedTo(val - cste) && !var2.canBeInstantiatedTo(val + cste)) {
				var1.removeValue(val, index, var2.getExplanations(new IntArray(new int[] {val - cste, val + cste})));

			}
		}
	}

	// ========================================================================

	protected void filterNeq() {
		if (v0.isInstantiated()) {
			((ExplainedIntVar) v1).removeValue(v0.getValue() + cste, index, index);
			((ExplainedIntVar) v1).removeValue(v0.getValue() - cste, index, index);
		}
		if (v1.isInstantiated()) {
			((ExplainedIntVar) v0).removeValue(v1.getValue() + cste, index, index);
			((ExplainedIntVar) v0).removeValue(v1.getValue() - cste, index, index);
		}
	}

	// ========================================================================

	protected void filterLT() {
		filterLTonVar((ExplainedIntVar) v1, (ExplainedIntVar) v0);
		filterLTonVar((ExplainedIntVar) v0, (ExplainedIntVar) v1);
	}

	// ========================================================================

	protected void filterGT() {
		filterGTonVar((ExplainedIntVar) v0, (ExplainedIntVar) v1);
		filterGTonVar((ExplainedIntVar) v1, (ExplainedIntVar) v0);
	}

	// ========================================================================

	// | X - Y | operator C

	private void filterGTonVar(ExplainedIntVar vv0, ExplainedIntVar vv1) {
		int lbv0 = vv0.getSup() - cste;
		int ubv0 = vv0.getInf() + cste;
		// remove interval [lbv0, ubv0] from domain of vv1
		if(lbv0 <= ubv0 ){ //TODO corrected by Mohamed 2011-04-17
			vv1.removeInterval(lbv0, ubv0, index, vv1.getDomExplanations());
		}
	}

	// ========================================================================
	@Override
	public void setUnPosted() {
		super.setUnPosted();
		if (v1.isInstantiated())
			((ExplainedIntVar) v0).removeExplanation(this.index);
		if (v0.isInstantiated())
			((ExplainedIntVar) v1).removeExplanation(this.index);
	}

	// ========================================================================

	public void awakeOnRestores(ExplainedIntVar var) {
		if (var.equals(v0)) {
			if (v1.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (!isConsistent(value, v1.getValue()))
						var.removeValue(value, index, index);
				}
			}
		} else if (var.equals(v1)) {
			if (v0.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (isConsistent(v0.getValue(), value))
						var.removeValue(value, index, index);
				}
			}
		}
	}

	// ========================================================================

	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {
		this.incrementNumConstraintChecks(1);
		ExplainedIntVar var = (ExplainedIntVar) v;
		switch (evtType) {
			case VariableEvent.INSTANCIATE :
				this.awakeOnInst(var);
			break;
			case VariableEvent.DESINSTANCIATE :
				this.awakeOnDesInst(var);
			break;
			case VariableEvent.REMOVE_VAL :
				this.awakeOnRemovals(var);
			break;
			case VariableEvent.RESTORE_VAL :
				this.awakeOnRestores(var);
			break;
			case VariableEvent.DECREMENT_SUP :
				this.awakeOnDecSup(var);
			break;
			case VariableEvent.INCREMENT_INF :
				this.awakeOnIncrInf(var);
			break;
			default :
				return;
		}
	}
	// ========================================================================

}
