package kernel.constraints.integer.explain;

import java.util.Collection;
import java.util.Iterator;

import kernel.Variable;
import kernel.constraints.integer.SEqualXYC;
import kernel.events.VariableEvent;
import kernel.vars.integer.explain.ExplainedIntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import kernel.vars.integer.explain.ExplainedIntVariable;
import kernel.vars.integer.explain.Explanation;

public class SEEqualXYC extends SEqualXYC implements Propagator {

	public SEEqualXYC(ExplainedIntVariable var0, ExplainedIntExternalVar var1) {
		super(var0, var1);
	}

	public SEEqualXYC(ExplainedIntVariable var0, ExplainedIntExternalVar var1, int cste) {
		super(var0, var1, cste);
	}

	@Override
	public void awakeOnInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			((ExplainedIntVar) v1).restrictDomainTo(v0.getValue() - cste, index, index);
		} else if (var.equals(v1)) {
			((ExplainedIntVar) v0).restrictDomainTo(v1.getValue() + cste, index, index);
		}
	}

	@Override
	public void awakeOnDesInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			((ExplainedIntVar) v1).removeExplanation(index);
		} else if (var.equals(v1)) {
			((ExplainedIntVar) v0).removeExplanation(index);
		}
	}

	@Override
	public void awakeOnRemovals(ExplainedIntVar var) {
		if (var.equals(v0)) {
			Iterator<Integer> it = var.getDomain().getDeltaIterator();
			while (it.hasNext()) {
				int val = it.next();
				this.incrementNumConstraintChecks(1);
				if (v1.canBeInstantiatedTo(val - cste)) {
					((ExplainedIntVar) v1).removeValue(val - cste, index, ((ExplainedIntVar) v0).getExplanation(val));
				}
			}
		} else if (var.equals(v1)) {
			Iterator<Integer> it = var.getDomain().getDeltaIterator();
			while (it.hasNext()) {
				int val = it.next();
				this.incrementNumConstraintChecks(1);
				if (v0.canBeInstantiatedTo(val + cste)) {
					((ExplainedIntVar) v0).removeValue(val + cste, index, ((ExplainedIntVar) v1).getExplanation(val));
				}
			}
		}
	}

//	@Override
//	public void awakeOnIncrInf(ExplainedIntVar var) {
//		if (var.equals(v0)) {
//			((ExplainedIntVar) v1).updateInf(v0.getInf() - cste, this.index, ((ExplainedIntVar) v0).getInfExplanations());
//		} else if (var.equals(v1)) {
//			((ExplainedIntVar) v0).updateInf(v1.getInf() + cste, index, ((ExplainedIntVar) v1).getInfExplanations());
//		}
//	}
//
//	@Override
//	public void awakeOnDecSup(ExplainedIntVar var) {
//		if (var.equals(v0)) {
//			((ExplainedIntVar) v1).updateSup(v0.getSup() - cste, this.index, ((ExplainedIntVar) v0).getSupExplanations());
//		} else if (var.equals(v1)) {
//			((ExplainedIntVar) v0).updateSup(v1.getSup() + cste, this.index, ((ExplainedIntVar) v1).getSupExplanations());
//		}
//	}
	 
	@Override //TODO corrected by Mohamed 2011-04-17
	public void awakeOnIncrInf(ExplainedIntVar var) {
		//TODO corrected by Mohamed
		int inf_v1 = v1.getInf();
		int inf_v0 = v0.getInf();
		if (var.equals(v0) && inf_v0 - cste > inf_v1) {
			((ExplainedIntVar) v1).updateInf(inf_v0 - cste, this.index, ((ExplainedIntVar) v0).getInfExplanations());
		}
		if (var.equals(v1) && inf_v1 + cste > inf_v0) {
			((ExplainedIntVar) v0).updateInf(inf_v1 + cste, this.index, ((ExplainedIntVar) v1).getInfExplanations());
		}
	}

	@Override //TODO corrected by Mohamed 2011-04-17
	public void awakeOnDecSup(ExplainedIntVar var) {
		int sup_v0 = v0.getSup();
		int sup_v1 = v1.getSup();
		if (var.equals(v0) && sup_v0 - cste < sup_v1) {
			((ExplainedIntVar) v1).updateSup(sup_v0 - cste, this.index, ((ExplainedIntVar) v0).getSupExplanations());
		}
		if (var.equals(v1) && sup_v1 + cste < sup_v0) {
			((ExplainedIntVar) v0).updateSup(sup_v1 + cste, this.index, ((ExplainedIntVar) v1).getSupExplanations());
		}
	}

	@Override
	public void propagate() {
		this.awakeOnIncrInf((ExplainedIntVar) v0);
		this.awakeOnDecSup((ExplainedIntVar) v0);
		this.awakeOnIncrInf((ExplainedIntVar) v1);
		this.awakeOnDecSup((ExplainedIntVar) v1);
		// ensure that, holes are also propagated
		Iterator<Integer> it = v0.getDomain().iterator();
		while (it.hasNext()) {
			int val0 = it.next();
			this.incrementNumConstraintChecks(1);
			if (!(v1.canBeInstantiatedTo(val0 - cste))) {
				Collection<Explanation> exp = ((ExplainedIntVar) v1).getExplanation(val0 - cste);
				((ExplainedIntVar) v0).removeValue(val0, index, exp);
			}
		}
		it = v1.getDomain().iterator();
		while (it.hasNext()) {
			int val1 = it.next();
			this.incrementNumConstraintChecks(1);
			if (!(v0.canBeInstantiatedTo(val1 + cste))) {
				Collection<Explanation> exp = ((ExplainedIntVar) v0).getExplanation(val1 + cste);
				((ExplainedIntVar) v1).removeValue(val1, index, exp);
			}
		}
	}

	// ///////////////////////////////////////////////////////////////////////////////

	@Override
	public void setUnPosted() {
		super.setUnPosted();
		if (v1.isInstantiated())
			((ExplainedIntVar) v0).removeExplanation(this.index);
		if (v0.isInstantiated())
			((ExplainedIntVar) v1).removeExplanation(this.index);
	}

	public void awakeOnRestores(ExplainedIntVar var) {
		if (var.equals(v0)) {
			if (v1.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (!isConsistent(value, v1.getValue()))
						var.removeValue(value, index, index);
				}
			} else {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int val0 = it.next();
					this.incrementNumConstraintChecks(1);
					if (!(v1.canBeInstantiatedTo(val0 - cste))) {
						((ExplainedIntVar) v0).removeValue(val0, index, ((ExplainedIntVar) v1).getExplanation(val0 - cste));
					}
				}
			}
		} else if (var.equals(v1)) {
			if (v0.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (!isConsistent(v0.getValue(), value))
						var.removeValue(value, index, index);
				}
			} else {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int val1 = it.next();
					this.incrementNumConstraintChecks(1);
					if (!(v0.canBeInstantiatedTo(val1 + cste))) {
						((ExplainedIntVar) v1).removeValue(val1, index, ((ExplainedIntVar) v0).getExplanation(val1 + cste));
					}
				}
			}
		}
	}

	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {
		this.incrementNumConstraintChecks(1);
		ExplainedIntVar var = (ExplainedIntVar) v;
		switch (evtType) {
			case VariableEvent.INSTANCIATE :
				this.awakeOnInst(var);
			break;
			case VariableEvent.DESINSTANCIATE :
				this.awakeOnDesInst(var);
			break;
			case VariableEvent.REMOVE_VAL :
				this.awakeOnRemovals(var);
			break;
			case VariableEvent.RESTORE_VAL :
				this.awakeOnRestores(var);
			break;
			case VariableEvent.DECREMENT_SUP :
				this.awakeOnDecSup(var);
			break;
			case VariableEvent.INCREMENT_INF :
				this.awakeOnIncrInf(var);
			break;
			default :
				return;
		}

	}

}
