/************************************************************************

 **************************************************************************/

package kernel.constraints.integer.explain;

import java.util.Iterator;
import kernel.Variable;
import kernel.constraints.integer.SGreaterThanEqualXYC;
import kernel.events.VariableEvent;
import kernel.vars.integer.explain.ExplainedIntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import kernel.vars.integer.explain.ExplainedIntVariable;

/**
 * 
 * A Class models the external constraint X >= Y + C where X is a single internal variable and Y is an external variable
 * 
 * @author Redouane EZZAHIR
 * 
 */

public class SEGreaterThanEqualXYC extends SGreaterThanEqualXYC implements Propagator {

	/**
	 * Constructs a new SNotEqualXYC constraint
	 * 
	 * @param var0
	 *            the internal variable
	 * @param var1
	 *            the external variable
	 * @param cste
	 *            a constant
	 */
	public SEGreaterThanEqualXYC(ExplainedIntVariable var0, ExplainedIntExternalVar var1, int cste) {
		super(var0, var1, cste);
	}

	@Override
	public void setUnPosted() {
		super.setUnPosted();
		if (v1.isInstantiated())
			((ExplainedIntVar) v0).removeExplanation(this.index);
		if (v0.isInstantiated())
			((ExplainedIntVar) v1).removeExplanation(this.index);
	}

	@Override
	public void awakeOnDecSup(ExplainedIntVar var) {
		if (var.equals(v0))
			((ExplainedIntVar) v1).updateSup(v0.getSup() - this.cste, this.index, ((ExplainedIntVar) v0).getSupExplanations());
	}

	@Override
	public void awakeOnIncrInf(ExplainedIntVar var) {
		if (var.equals(v1))
			((ExplainedIntVar) v0).updateInf(v1.getInf() + this.cste, index, ((ExplainedIntVar) v1).getInfExplanations());
	}

	@Override
	public void awakeOnInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			((ExplainedIntVar) v1).updateSup(v0.getValue() - this.cste, this.index, this.index);
		} else if (var.equals(v1)) {
			((ExplainedIntVar) v0).updateInf(v1.getValue() + this.cste, index, this.index);
		}
	}

	@Override
	public void awakeOnDesInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			((ExplainedIntVar) v1).removeExplanation(index);
		} else if (var.equals(v1)) {
			((ExplainedIntVar) v0).removeExplanation(index);
		}
	}

	/**
	 * Propagation when the values <code>values</code> from domain of the variable <code>var</code>.
	 */
	@Override
	public void awakeOnRemovals(ExplainedIntVar var) {
		this.awakeOnIncrInf(var);
		this.awakeOnDecSup(var);
	}

	@Override
	public void propagate() {
		this.awakeOnIncrInf((ExplainedIntVar) v1);
		this.awakeOnDecSup((ExplainedIntVar) v0);
	}

	// /////////////////////////////////////////////////////////////////////////////////////////

	public void awakeOnRestores(ExplainedIntVar var) {
		if (var.equals(v0)) {
			if (v1.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (!isConsistent(value, v1.getValue()))
						((ExplainedIntVar) v0).removeValue(value, index, index);
				}
			} else {// X >= Y + C
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					this.incrementNumConstraintChecks(1);
					if (value - cste < v1.getInf())
						((ExplainedIntVar) v0).removeValue(value, index, ((ExplainedIntVar) v1).getInfExplanations());
				}
			}
		} else if (var.equals(v1)) {
			if (v0.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (isConsistent(v0.getValue(), value))
						((ExplainedIntVar) v1).removeValue(value, index, index);
				}
			} else {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					this.incrementNumConstraintChecks(1);
					if (v0.getSup() < value + cste)
						((ExplainedIntVar) v1).removeValue(value, index, ((ExplainedIntVar) v0).getSupExplanations());
				}
			}
		}
	}

	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {
		this.incrementNumConstraintChecks(1);
		ExplainedIntVar var = (ExplainedIntVar) v;
		switch (evtType) {
			case VariableEvent.INSTANCIATE :
				this.awakeOnInst(var);
			break;
			case VariableEvent.DESINSTANCIATE :
				this.awakeOnDesInst(var);
			break;
			case VariableEvent.REMOVE_VAL :
				this.awakeOnRemovals(var);
			break;
			case VariableEvent.RESTORE_VAL :
				this.awakeOnRestores(var);
			break;
			case VariableEvent.DECREMENT_SUP :
				this.awakeOnDecSup(var);
			break;
			case VariableEvent.INCREMENT_INF :
				this.awakeOnIncrInf(var);
			break;
			default :
				return;
		}

	}

	// //////////////////////////////////////////////////////////////////////////

}
