/************************************************************************

 **************************************************************************/

package kernel.constraints.integer.explain;

import java.util.Iterator;
import kernel.Variable;
import kernel.constraints.integer.SNotEqualXYC;
import kernel.events.VariableEvent;
import kernel.vars.integer.explain.ExplainedIntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import kernel.vars.integer.explain.ExplainedIntVariable;

/**
 * 
 * A Class models the external constraint v0 != v1 + C where v0 is a single internal variable and v1 is an external variable
 * 
 * @author R EZZAHIR
 * 
 */
public class SENotEqualXYC extends SNotEqualXYC implements Propagator {

	public SENotEqualXYC(ExplainedIntVariable var0, ExplainedIntExternalVar var1) {
		super(var0, var1);

	}

	public SENotEqualXYC(ExplainedIntVariable var0, ExplainedIntExternalVar var1, int cste) {
		super(var0, var1, cste);
	}

	@Override
	public void setUnPosted() {
		super.setUnPosted();
		if (v1.isInstantiated())
			((ExplainedIntVar) v0).removeExplanation(this.index);
		if (v0.isInstantiated())
			((ExplainedIntVar) v1).removeExplanation(this.index);
	}

	@Override
	public void awakeOnInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			((ExplainedIntVar) v1).removeValue(v0.getValue() - cste, index, index);
		} else if (var.equals(v1)) {
			((ExplainedIntVar) v0).removeValue(v1.getValue() + cste, index, index);
		}
	}

	@Override
	public void awakeOnDesInst(ExplainedIntVar var) {
		if (var.equals(v0)) {
			((ExplainedIntVar) v1).removeExplanation(v0.getPreviousValue() - cste, index);
		} else if (var.equals(v1)) {
			((ExplainedIntVar) v0).removeExplanation(v1.getPreviousValue() + cste, index);
		}
	}

	@Override
	public void awakeOnRemovals(ExplainedIntVar var) {
		if (var.equals(v0)) {
			if (v0.getDomainSize() == 1 && v1.canBeInstantiatedTo(v0.getInf() - cste)) {
				((ExplainedIntVar) v1).removeValue(v0.getInf() - cste, index, ((ExplainedIntVar) v0).getDomExplanations());
			}
		} else if (var.equals(v1)) {
			if (v1.getDomainSize() == 1 && v0.canBeInstantiatedTo(v1.getInf() + cste)) {
				((ExplainedIntVar) v0).removeValue(v1.getInf() + cste, index, ((ExplainedIntVar) v1).getDomExplanations());
			}
		}
	}

	@Override
	public void awakeOnDecSup(ExplainedIntVar var) {
		// if(var.hasEnemeratedDomain())
		this.awakeOnRemovals(var);
	}

	@Override
	public void awakeOnIncrInf(ExplainedIntVar var) {
		// if(var.hasEnemeratedDomain())
		this.awakeOnRemovals(var);
	}

	/**
	 * The initial propagation consist in forward checking @
	 */
	@Override
	public void propagate() {
		this.awakeOnRemovals((ExplainedIntVar) v0);
		this.awakeOnRemovals((ExplainedIntVar) v1);
	}

	// /////////////////////////////////////////////////////////////////////////////////////////

	public void awakeOnRestores(ExplainedIntVar var) {
		if (var.equals(v0)) {
			if (v1.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (!isConsistent(value, v1.getValue()))
						((ExplainedIntVar) v0).removeValue(value, index, index);
				}
			} else if (v1.getDomainSize() == 1) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					this.incrementNumConstraintChecks(1);

					if (v1.getInf() + cste == value)
						((ExplainedIntVar) v0).removeValue(value, index, ((ExplainedIntVar) v1).getDomExplanations());
				}
			}
		} else if (var.equals(v1)) {
			if (v0.isInstantiated()) {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					if (isConsistent(v0.getValue(), value))
						((ExplainedIntVar) v1).removeValue(value, index, index);
				}
			} else {
				Iterator<Integer> it = var.getDomain().getRepairIterator();
				while (it.hasNext()) {
					int value = it.next();
					this.incrementNumConstraintChecks(1);

					if (v0.getInf() - cste == value)
						((ExplainedIntVar) v1).removeValue(value, index, ((ExplainedIntVar) v0).getDomExplanations());
				}
			}
		}
	}

	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {
		this.incrementNumConstraintChecks(1);
		ExplainedIntVar var = (ExplainedIntVar) v;
		switch (evtType) {
			case VariableEvent.INSTANCIATE :
				this.awakeOnInst(var);
			break;
			case VariableEvent.DESINSTANCIATE :
				this.awakeOnDesInst(var);
			break;
			case VariableEvent.REMOVE_VAL :
				this.awakeOnRemovals(var);
			break;
			case VariableEvent.RESTORE_VAL :
				this.awakeOnRestores(var);
			break;
			case VariableEvent.DECREMENT_SUP :
				this.awakeOnDecSup(var);
			break;
			case VariableEvent.INCREMENT_INF :
				this.awakeOnIncrInf(var);
			break;
			default :
				return;
		}

	}

	// ////////////////////////////////////////////////////////////////////////////////////////////
}
