package kernel.constraints.integer.soft;

import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;
import simple.dcop.SimpleSoftConstraint;

public class SoftExtensionBinConstraint extends SoftIntBinConstraint implements SimpleSoftConstraint {

	public SoftCouplesTable relation;
	
	public SoftExtensionBinConstraint(String name, IntVariable v0, IntExternalVar v1, Integer[] weights, boolean transpose) {
 		super(v0, v1);
		relation = makeBinarySoftRelation(name, v0, v1, weights, transpose);
		this.type = name;
	}
	
	public static SoftCouplesTable makeBinarySoftRelation(String name, IntVariable v0, IntVariable v1, Integer[] weights, boolean transpose) {
		int n0 = v0.getSup() - v0.getInf() + 1;
		int n1 = v1.getSup() - v1.getInf() + 1;
		int inf0 = v0.getInf();
		int inf1 = v1.getInf();
		return new SoftCouplesTable(name, inf0, inf1, n0, n1, weights, transpose);
	}
	
	/* (non-Javadoc)
	 * @see simple.SimpleConstraint#evaluate(int)
	 */
	@Override
	public Integer evaluate(int value) {
		if(!this.v1.isInstantiated()) {
			return 0;
		}
		//assert this.v1.isInstantiated() : "The variable "+this.v1+" is not instantiated";
		return this.getCost(value, this.v1.getValue());
	}
	
	/**
	 * Returns a string representation of this object
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("C_" + this.getIndex());
		sb.append("[ " + this.v0.toString() + "@" + type + "@" + this.v1.toString() + "]  ");
		sb.append(", posted=" + this.isPosted());
		sb.append(", cost=" + this.getCost());
		return  sb.toString();
	}

	/**
	 * @return
	 */
	private String getCost() {
		if(this.v0.isInstantiated() && this.v1.isInstantiated()){
			return Integer.toString(this.relation.getCost(this.v0.getValue(), this.v1.getValue()));
		}
		return new String("?");
	}
	
	/* (non-Javadoc) 
	 * @see simple.SimpleSoftConstraint#getCost(int, int)
	 */
	@Override
	public Integer getCost(int x, int y) {
		this.incrementNumConstraintChecks(1);
		return this.relation.getCost(x, y);
	}
	
	/* (non-Javadoc)
	 * @see simple.SimpleSoftConstraint#getMinCost(int)
	 */
	@Override
	public Integer getMinCost(int value) {
		this.incrementNumConstraintChecks(this.relation.getN2());
		return this.relation.getMinCost(value);
	}

	/* (non-Javadoc)
	 * @see simple.SimpleSoftConstraint#getMaxCost(int)
	 */
	@Override
	public Integer getMaxCost(int value) {
		this.incrementNumConstraintChecks(this.relation.getN2());
		return this.relation.getMaxCost(value);
	}
	
}
