/************************************************************************

 **************************************************************************/

package kernel.constraints.integer.soft;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import kernel.AbstractConstraint;
import kernel.AbstractVar;
import kernel.Variable;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;
import simple.dcop.SimpleSoftConstraint;

/**
 * An abstract class for all implementations of (binary) constraints in the simple mode over internal variable and external variable.
 */
public abstract class SoftIntBinConstraint extends AbstractConstraint implements SimpleSoftConstraint {

	protected IntVariable v0;

	protected IntVariable v1;

	/**
	 * A constant
	 */
	protected int cste = 0;

	/**
	 * Constructs a binary external constraint
	 * 
	 * @param var0 An Internal variable or an external variable
	 * @param var1 An external variable when var0 is internal, otherwise internal variable
	 */
	public SoftIntBinConstraint(IntVariable var0, IntExternalVar var1) {
		v0 = var0;
		v1 = var1;
	}

	/**
	 * Constructs a binary external constraint.
	 * 
	 * @param var0 An Internal variable or an external variable
	 * @param var1 An external variable when var0 is internal, otherwise internal variable
	 * @param cste A constant
	 */
	public SoftIntBinConstraint(IntVariable var0, IntExternalVar var1, int cste) {
		this(var0, var1);
		this.cste = cste;
	}

	@Override
	public boolean isConsistent(Variable v, int val) {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see Constraint#getNbVars()
	 */
	public int getNbVars() {
		return 2;
	}

	/*
	 * (non-Javadoc)
	 * @see simple.SimpleConstraint#getInternalVar(int)
	 */
	public IntVariable getInternalVar(int idx) {
		if (idx != 0)
			throw new Error("index must equal to zero idx:" + idx);
		if (v1 instanceof IntExternalVar) {
			return v0;
		} else {
			return v1;
		}
	}

	/*
	 * (non-Javadoc)
	 * @see Constraint#getNbInternalVars()
	 */
	public int getNbInternalVars() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * @see Constraint#getNbExternalVars()
	 */
	public int getNbExternalVars() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * @see Constraint#getExternalVar(int)
	 */
	public AbstractVar getExternalVar(int idx) {
		if (idx != 0)
			throw new Error("index must equal to zero idx:" + idx);
		if (v1 instanceof IntExternalVar) {
			return v1;
		} else {
			return v0;
		}
	}

	/*
	 * (non-Javadoc)
	 * @see Constraint#isSatisfied()
	 */
	public boolean isSatisfied() {
		return true;
	}

	public boolean isCompletlyInstantiated() {
		return v0.isInstantiated() && v1.isInstantiated();
	}

	/**
	 * A simple Forward Checking
	 */
	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {}

	public AbstractVar getVar(int i) {
		if (i == 0)
			return this.v0;
		return v1;
	}

	@Override
	public void awake() {}

	// public abstract void awakeOnVar(IntVariable v);

	@Override
	public void propagate() {
		// awakeOnVar(v0);
		// awakeOnVar(v1);
	}

	@Override
	public Collection<AbstractVar> getExternalVars() {
		HashSet<AbstractVar> externalVars = new HashSet<AbstractVar>();
		externalVars.add(this.v1);
		return Collections.unmodifiableSet(externalVars);
	}

}
