package kernel.constraints.integer.soft;

import java.util.Arrays;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVariable;
import simple.dcop.SimpleSoftConstraint;
import simulation.parser.Toolkit;

public class SoftTuplesConstraint extends SoftIntBinConstraint implements SimpleSoftConstraint {

	private Integer[] weights;
	private int[][] tuples;
	private boolean transpose = false;
	private Integer defaultCost = INFINITY;
	
	public SoftTuplesConstraint(String name, IntVariable v0, IntExternalVar v1, int[][] tuples, Integer[] weights, Integer defaultCost, boolean transpose) {
 		super(v0, v1);
		assert (tuples.length == weights.length) : "Problem with weights length="+weights.length+" <> tuples length="+tuples.length;
		this.type = name;
 		this.tuples = tuples;
 		this.weights = weights;
		this.transpose = transpose;
		this.defaultCost = defaultCost;
	}
	
	/* (non-Javadoc)
	 * @see simple.SimpleConstraint#evaluate(int)
	 */
	@Override
	public Integer evaluate(int value) {
		if(!this.v1.isInstantiated()) {
			return 0;
		}
		return this.getCost(value, this.v1.getValue());
	}
	
	/**
	 * Returns a string representation of this object
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("C_" + this.getIndex());
		sb.append("[ " + this.v0.toString() + "@" + type + "@" + this.v1.toString() + "]  ");
		sb.append(", posted=" + this.isPosted());
		sb.append(", cost=" + this.getCost());
		return  sb.toString();
	}

	/**
	 * @return
	 */
	private String getCost() {
		if(this.v0.isInstantiated() && this.v1.isInstantiated()){
			return Integer.toString(this.getCost(this.v0.getValue(), this.v1.getValue()));
		}
		return new String("?");
	}
	
	/* (non-Javadoc) 
	 * @see simple.SimpleSoftConstraint#getCost(int, int)
	 */
	@Override
	public Integer getCost(int xValue, int yValue) {
		if(this.transpose) {
			return this.computeCostOf(new int[]{yValue, xValue});
		} else {
			return this.computeCostOf(new int[]{xValue, yValue});
		}
	}
	
	/* (non-Javadoc)
	 * @see simple.SimpleSoftConstraint#getMinCost(int)
	 */
	@Override
	public Integer getMinCost(int value) {
		int minCost = MAX_COST;
		for(int yValue : this.v1.getDomain()) {
			minCost = Math.min(minCost, this.getCost(value, yValue));
		}
		return minCost;
	}

	/* (non-Javadoc)
	 * @see simple.SimpleSoftConstraint#getMaxCost(int)
	 */
	@Override
	public Integer getMaxCost(int value) {
		int maxCost = MIN_COST;
		for(int yValue : this.v1.getDomain()) {
			maxCost = Math.max(maxCost, this.getCost(value, yValue));
		}
		return maxCost;
	}

	public int computeCostOf(int[] tuple) {
		this.incrementNumConstraintChecks(1);
		int position = Arrays.binarySearch(this.tuples, tuple, Toolkit.lexicographicComparator);
		return (position >= 0)? this.weights[position] : defaultCost;
	}
}
