package kernel.events;

import kernel.AbstractAgent;

/**
 * A class modeling the event : agent change state. An agent may be in a wait state, idle, or running
 * 
 * @author Redouane Ezzahir
 * 
 */
public class AgentChangeStateEvent extends AgentEvent {

	public final static byte STATE_RUNNING = 1;
	public final static byte STATE_WAITING = 2;
	public final static byte STATE_IDLE = 3;
	/**
	 * A field storing the status of the agent
	 */
	byte agentState = 0;

	/**
	 * Constructs an object AgentChangeStateEvent
	 * 
	 * @param agent
	 *            the agent that changed its state
	 * @param agentState
	 *            the new state which the agent has reached
	 */

	public AgentChangeStateEvent(AbstractAgent agent, byte agentState) {
		super(agent);
		this.agentState = agentState;
	}

	/**
	 * Set the value of agentState to the given state
	 * 
	 * @param state
	 */
	public void setAgentsState(byte state) {
		this.agentState = state;
	}

	/**
	 * Get the state of the agent that generate this event
	 */
	public byte getAgentState() {
		return agentState;
	}
}
