/**************************************************************************

 ***************************************************************************/

package kernel.events;

import kernel.Constraint;
import util.Logger;

/**
 * A class for constraint propagation
 * 
 * @author Redouane EZZAHIR
 * 
 * 
 */
public class ConstraintEvent implements DisChocoEvent<Constraint>, Comparable<ConstraintEvent> {
	/**
	 * The constraint that must be reviewed
	 */
	protected Constraint constraint;

	private boolean initialized;

	/**
	 * Constructs a constraint event
	 * 
	 * @param constraint
	 *            an external constraint to revise
	 * @param priority
	 * @param varEvent
	 *            the event that was occurred on a variable of the <code>constraint</code> scope
	 */
	public ConstraintEvent(Constraint constraint, boolean init, int priority) {
		this.constraint = constraint;
		this.initialized = init;
	}

	@Override
	public Constraint getTouchedObject() {
		return this.constraint;
	}

	@Override
	public boolean isActive() {
		return false;
	}

	@Override
	public int compareTo(ConstraintEvent e) {
		return e.constraint.getPriority() - this.constraint.getPriority();
	}

	/**
	 * Fires the given event to the handlers listening to the event's type. There is a single handler is the constraintHandler
	 */
	public void propagateOneBasicEvent() {
		if (this.initialized) {
			this.constraint.propagate();// .getConstraintsHandler().propagateConstraintEvent(this);
		} else {
			this.constraint.awake();
		}
	}

	/**
	 * delete this event
	 */
	public void clear() {
		if (Logger.LOG_DEBUG) {
			Logger logger = this.constraint.getConstraintsHandler().myAgent.logger;
			logger.printToLogln("Const Awake Event does not need to be cleared !", Logger.INFO_LEVEL);
		}
	}

	public boolean isInitialized() {
		return initialized;
	}

	public void setInitialized(boolean init) {
		this.initialized = init;
	}

	public String toString() {
		return "constEvt [" + this.constraint.toString() + "]";
	}

}
