/*************************************************************************

 **************************************************************************/

package kernel.events;

import java.util.Comparator;
import java.util.Iterator;
import java.util.PriorityQueue;

/**
 * Interface ConstraintEvent
 */
public class ConstraintEventQueue implements EventQueue<ConstraintEvent> {

	/**
	 * A priority queue with all the constraint events.
	 */
	private PriorityQueue<ConstraintEvent> queue;

	/**
	 * The comparator, or null if priority queue uses elements' natural ordering.
	 */
	private final Comparator<? super ConstraintEvent> comparator;

	public ConstraintEventQueue() {
		this(20, null);
	}

	public ConstraintEventQueue(int initialCapacity, Comparator<? super ConstraintEvent> comparator) {
		this.comparator = comparator;
		this.queue = new PriorityQueue<ConstraintEvent>(initialCapacity, this.comparator);
	}

	@Override
	public ConstraintEvent popEvent() {
		ConstraintEvent event = queue.poll();
		return event;
	}

	@Override
	public void propagateOneEvent() {
		ConstraintEvent event = popEvent();
		event.propagateOneBasicEvent();
		if (event.isActive()) {
			this.pushEvent(event);
		}
	}

	@Override
	public boolean pushEvent(ConstraintEvent event) {
		if (this.queue.contains(event))
			this.queue.remove(event); // in order to update priority
		return this.queue.offer(event);
	}

	@Override
	public void remove(ConstraintEvent event) {
		this.queue.remove(event);
	}

	/**
	 * Removes all the events (including the popping one).
	 */

	public void flushEventQueue() {
		for (Iterator<ConstraintEvent> it = queue.iterator() ; it.hasNext() ;) {
			ConstraintEvent event = it.next();
			event.clear();
		}
		queue.clear();
	}

	public int size() {
		return queue.size();
	}

	public String toString() {
		return this.queue.toString();
	}

	/**
	 * Checks if the queue is empty.
	 */
	public boolean isEmpty() {
		return queue.isEmpty();
	}

}
