/************************************************************************

 **************************************************************************/

package kernel.events;

import java.util.EventListener;

import kernel.Variable;

/**
 * Interface ConstraintListener
 */
public interface ConstraintListener extends EventListener {

	/**
	 * It must be defined by user when developing a new algorithm; It defines how processing constraints when running a distributed algorithm.
	 * <p>
	 * For example, In DisCSP Algorithm, we verify the satisfaction of the constraints, we remove no consistent value, etc.
	 * <p>
	 * In DCOP Algorithm, we calculate the costs for each value, etc.
	 * 
	 * @param event
	 *            A constraint event to handle
	 */

	public void propagateVariableEvent(Variable v, int evtType, int evtCause);

	public void propagateConstraintEvent(ConstraintEvent event);

	public void propagate();

}
