package kernel.events;

import java.util.Comparator;

import kernel.Constraint;
import kernel.Variable;

public class DisChocoEngine implements PropagationEngine {
	/**
	 * The different queues for the constraint awake events.
	 */
	protected ConstraintEventQueue constEventQueue;
	/**
	 * The queue with all the variable events.
	 */
	protected VariableEventQueue varEventQueue;

	/**
	 * Constructs a new engine by initializing the event queues.
	 */
	public DisChocoEngine() {
		this.constEventQueue = new ConstraintEventQueue();
		this.varEventQueue = new VariableEventQueue();
	}

	/**
	 * Constructs a new engine by initializing the event queues.
	 */
	public DisChocoEngine(int constrInitCapacity, Comparator<? super ConstraintEvent> constrComparator, int varInitCapacity,
			Comparator<? super VariableEvent> varComparator) {
		this.constEventQueue = new ConstraintEventQueue(constrInitCapacity, constrComparator);
		this.varEventQueue = new VariableEventQueue(varInitCapacity, varComparator);
	}

	public void postEvent(Variable v, int basicEvt, int idx) {
		VariableEvent event = v.getEvent();
		event.recordEventTypeAndCause(basicEvt, idx);
		varEventQueue.pushEvent(event);
		// if(!varEventQueue.pushEvent(event)){
		// event.touchedVar.getAgentOwner().log(event+ " is not pushed  ", Logger.INFO_LEVEL);
		// }
	}

	/**
	 * Posts a constraint awake event.
	 * 
	 * @param constraint
	 *            The constraint that must be awaken.
	 * @param post
	 *            Specifies if the constraint must be initialized (awake instead of propagate).
	 */

	public boolean postConstAwake(Constraint constraint, boolean initialized) {
		ConstraintEvent event = (ConstraintEvent) constraint.getEvent();
		event.setInitialized(initialized);
		return this.constEventQueue.pushEvent(event);
	}

	/**
	 * Returns the variable event queue.
	 */
	public VariableEventQueue getVarEventQueue() {
		return varEventQueue;
	}

	@Override
	public void flushEvents() {
		this.constEventQueue.flushEventQueue();
		this.varEventQueue.flushEventQueue();
	}

	@Override
	public EventQueue<? extends DisChocoEvent<?>> getNextActiveEventQueue() {
		if (!this.constEventQueue.isEmpty())
			return this.constEventQueue;
		if (!this.varEventQueue.isEmpty())
			return this.varEventQueue;
		return null;
	}

	@Override
	public void registerEvent(ConstraintEvent event) {
		event.setInitialized(true);
		this.constEventQueue.pushEvent(event);
	}

	@Override
	public void propagate() {
		EventQueue<? extends DisChocoEvent<?>> q = this.getNextActiveEventQueue();
		while (q != null) {
			q.propagateOneEvent();
			q = this.getNextActiveEventQueue();
		}
		assert (this.varEventQueue.isEmpty() && this.constEventQueue.isEmpty());
	}

}
