package kernel.events;

/**
 * An interface for all the implementations of var queues (Constraint awake, on Variable events for instance).
 */
public interface EventQueue<E> {

	/**
	 * Checks if the queue is empty.
	 */
	public boolean isEmpty();

	/**
	 * Pops the next event to propagate.
	 */
	public E popEvent();

	/**
	 * Adds an event to the queue.
	 * 
	 * @param event
	 */
	public boolean pushEvent(E event);

	/**
	 * Removes all the events and clears all the events if needed.
	 */
	public void flushEventQueue();

	/**
	 * Removes an event.
	 */

	public void remove(E event);

	/**
	 * Propagate one event
	 */
	public void propagateOneEvent();

	/**
	 * returns the number of pending events in the queue
	 * 
	 * @return the number of pending events in the queue
	 */
	public int size();

}
