package kernel.events;

import kernel.vars.integer.explain.ExplainedIntVar;
import util.BitSet;

public class ExplainedIntVarEvent extends IntVariableEvent {

	public ExplainedIntVarEvent(ExplainedIntVar var) {
		super(var);
	}

	@Override
	public void propagateOneBasicEvent() {
		if (this.eventType != EMPTYEVENT) {
			int eventT = this.eventType;
			int evCause = this.cause;

			this.eventType = EMPTYEVENT;
			this.cause = NOCAUSE;

			int mostImportantEvent = BitSet.lowestBit(eventT);
			eventT = BitSet.unsetBit(eventT, mostImportantEvent);

			switch (mostImportantEvent) {
				case DESINSTANCIATE :
					propagateEvent(DESINSTANCIATE, NOCAUSE);
				break;

				case RESTORE_INF :
					propagateRestEvent(RESTORE_INF, NOCAUSE);
				break;
				case RESTORE_SUP :
					propagateRestEvent(RESTORE_SUP, NOCAUSE);
				break;
				case RESTORE_VAL :
					propagateRestEvent(RESTORE_VAL, NOCAUSE);
				break;

				case INSTANCIATE :
					propagateEvent(INSTANCIATE, evCause);
				break;

				case INCREMENT_INF :
					propagateRemEvent(INCREMENT_INF, evCause);
				break;
				case DECREMENT_SUP :
					propagateRemEvent(DECREMENT_SUP, evCause);
				break;
				case REMOVE_VAL :
					propagateRemEvent(REMOVE_VAL, evCause);
				break;
				default :
					System.err.println("warinng : calls propagate one event on empty event! " + mostImportantEvent);
			}
			this.eventType |= eventT;
			if (this.cause == NOCAUSE && this.eventType != EMPTYEVENT) {
				this.cause = evCause;
			}
		} else {
			System.err.println("warinng : calls propagate one event on empty event! " + this.eventType + " :: " + this.cause);
		}

	}

	private void propagateRestEvent(int eventType, int nocause) {
		this.propagateEvent(eventType, nocause);
		((ExplainedIntVar) this.touchedVar).getDomain().releaseRepairDomain();
	}

	// ====================================================================
	/**
	 * Clears the variable event : delegates to the variable event.
	 */
	public void clear() {
		this.eventType = EMPTYEVENT;
		this.cause = NOEVENT;
		((ExplainedIntVar) this.touchedVar).getDomain().releaseRepairDomain();
	}

}
