package kernel.events;

import java.util.Comparator;
import java.util.Iterator;
import java.util.PriorityQueue;

public class VariableEventQueue implements EventQueue<VariableEvent> {

	/**
	 * A priority queue with all the variable events.
	 */
	private PriorityQueue<VariableEvent> queue;

	/**
	 * The comparator, or null if priority queue uses elements' natural ordering.
	 */
	private final Comparator<? super VariableEvent> comparator;

	public VariableEventQueue() {
		this(20, null);
	}

	public VariableEventQueue(int initialCapacity, Comparator<? super VariableEvent> comparator) {
		this.comparator = comparator;
		this.queue = new PriorityQueue<VariableEvent>(initialCapacity, this.comparator);
	}

	/**
	 * Checks if the queue is empty.
	 */
	public boolean isEmpty() {
		return (queue.size() == 0);
	}

	/**
	 * Propagates the most important event
	 */
	@Override
	public void propagateOneEvent() {
		if (!this.isEmpty()) {
			VariableEvent event = popEvent();
			event.propagateOneBasicEvent();
			if (event.isActive()) {
				this.pushEvent(event);
			}
			// if(!event.isActive()) {
			// System.err.println("Event not active in queue "+event+ " => "+this.queue);
			// this.queue.remove(event);
			// }
			// else {
			// event.propagateOneBasicEvent();
			// if(event.isActive()){
			// this.pushEvent(event);
			// }
			// // else{
			// // this.queue.remove(event);
			// // }
			// }
		}
	}

	/**
	 * Adds an event to the queue.
	 */
	@Override
	public boolean pushEvent(VariableEvent event) {
		if (this.queue.contains(event))
			this.queue.remove(event); // in order to update priority
		return this.queue.offer(event);
	}

	/**
	 * Removes an event. This method should not be useful for variable events.
	 */
	@Override
	public void remove(VariableEvent event) {
		this.queue.remove(event);
	}

	/**
	 * Pops an event to propagate.
	 */
	public VariableEvent popEvent() {
		VariableEvent event = this.queue.poll();
		return event;
	}

	/**
	 * Removes all the events (including the popping one).
	 */

	public void flushEventQueue() {
		for (Iterator<VariableEvent> it = queue.iterator() ; it.hasNext() ;) {
			VariableEvent event = it.next();
			event.clear();
		}
		this.queue.clear();
	}

	public int size() {
		return queue.size();
	}

	public String toString() {
		return this.queue.toString();
	}

}
