package kernel.events;

import util.IntArray;

/**
 * An interface for triggering events on the variables
 * 
 * @author Redouane EZZAHIR
 * 
 */

public interface VariableEventTrigger {
	/**
	 * Adds a listener of the variable event (i.e. A constraint)
	 */
	public void addVariableEventListener(int eventType, VariableEventListener listener);

	/**
	 * Removes a listener of the variable event (i.e. A constraint)
	 */
	public void removeVariableEventListener(VariableEventListener listener);

	/**
	 * Removes a listener of the variable event (i.e. A constraint)
	 */
	public void removeVariableEventListener(int eventType, VariableEventListener listener);

	public IntArray getEventsType(int index);

	/**
	 * Returns the number of listeners involving the variable.
	 */
	public int getNumListeners();

	/**
	 * Trigger the event of value change with the given type (Assigns or unassigns value)
	 * 
	 * @param index
	 *            Index of the constraint that causes the event
	 */
	public void fireChangedValue(int event, int index);

	/**
	 * Trigger the event of domain change with the given type (remove or restore value)
	 * 
	 * @param index
	 *            Index of the constraint that causes the event
	 */
	public void fireChangedDomain(int event, int index);

	public VariableEvent getEvent();
}
