package kernel.order;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import kernel.VarIdentifier;
import util.HashList;

/**
 * A class to represent the complicity of an agent : number of local variables, of internal constraints, maximum size of domain, the neighbors
 * 
 * @author Redouane EZZAHIR
 * 
 */

public class AgentComplexity implements Serializable {

	private static final long serialVersionUID = -9038560902573312747L;
	
	private String label = null;
	private int maxDomain;
	private int nbConstraints;
	private int nbLocalVars;
	private HashList<VarIdentifier> neighbors;
	private HashList<VarIdentifier> localAgents;
	private int nbNeighbors;
	private Map<VarIdentifier, String> constraints = null;

	public AgentComplexity() {
		this.maxDomain = 0;
		this.neighbors = new HashList<VarIdentifier>();
		this.localAgents = new HashList<VarIdentifier>();
		this.nbConstraints = 0;
		this.nbLocalVars = 0;
		this.constraints = new HashMap<VarIdentifier, String>();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "[domain=" + this.maxDomain + ", nbVars=" + this.nbLocalVars + 
				", neighbors=" + this.neighbors + ", nbConstraints=" + this.nbConstraints + 
				", localAgents=" + this.localAgents + "]";
	}
	
	/**
	 * @return the maxDomain
	 */
	public int getMaxDomain() {
		return this.maxDomain;
	}

	/**
	 * @param maxDomain the maxDomain to set
	 */
	public void setMaxDomain(int maxDomain) {
		this.maxDomain = maxDomain;
	}

	/**
	 * @return the nbConstraints
	 */
	public int getNumberConstraints() {
		return this.nbConstraints;
	}

	/**
	 * @param nbConstraints the numberConstraints to set
	 */
	public void IncrementNbConstraints() { 
		this.nbConstraints++;
	}

	/**
	 * @param nbConstraints the numberConstraints to set
	 */
	public void setNumberConstraints(int nbConstraints) {
		this.nbConstraints = nbConstraints;
	}

	/**
	 * @return the numberLocalVars
	 */
	public int getNumberLocalVars() {
		return this.nbLocalVars;
	}

	/**
	 * @param numberLocalVars the numberLocalVars to set
	 */
	public void setNumberLocalVars(int numberLocalVars) {
		this.nbLocalVars = numberLocalVars;
	}

	/**
	 * @param nbLocalVars the numberConstraints to set
	 */
	public void IncrementNbLocalVars() { 
		this.nbLocalVars++;
	}

	/**
	 * @return the neighbors
	 */
	public HashList<VarIdentifier> getNeighbors() {
		return this.neighbors;
	}

	/**
	 * @param neighbors the neighbors to set
	 */
	public void setNeighbors(HashList<VarIdentifier> neighbors) {
		this.neighbors = neighbors;
		this.nbNeighbors = neighbors.size();
	}

	/**
	 * @param neighbors the neighbors to set
	 */
	public void addNeighbor(VarIdentifier neighbor, String constraint) {
		if(!this.neighbors.contains(neighbor)){
			this.neighbors.add(neighbor);
			this.constraints.put(neighbor, constraint);
			this.nbNeighbors++;
		}
	}

	/**
	 * @return the localAgents
	 */
	public HashList<VarIdentifier> getLocalAgents() {
		return this.localAgents;
	}

	/**
	 * @param localAgents the localAgents to set
	 */
	public void setLocalAgents(HashList<VarIdentifier> localAgents) {
		this.localAgents = localAgents;
	}

	/**
	 * @return the nbNeighbors
	 */
	public int getNbNeighbors() {
		return this.nbNeighbors;
	}

	/**
	 * @return the label
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * @param label the label to set
	 */
	public void setLabel(String label) {
		this.label = label;
	}

	/**
	 * @param agentId
	 * @return
	 */
	public String getConstraintName(VarIdentifier neighbor) {
		if(this.constraints.containsKey(neighbor)) {
			return this.constraints.get(neighbor);
		}
		return "";
	}
	
}
