package kernel.order;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import kernel.VarIdentifier;
import util.HashList;

/**
 * A Class for Min-Domain Degree ordering
 */
public class MaxDegPlusMinDomAO extends CentralizedAO implements ChainOrder {

	protected HashList<VarIdentifier> agentsId = null;
	protected Comparator<VarIdentifier> comparator = null;

	public MaxDegPlusMinDomAO() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see order.CentralizedAgentOrdering#buildAgentsOrder()
	 */
	public HashList<VarIdentifier> buildAgentsOrder() {
		return this.buildAgentsOrderForComplex();
	}

	/**
	 * 
	 */
	public HashList<VarIdentifier> buildAgentsOrderForComplex() {
		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
		this.agentsId = new HashList<VarIdentifier>();
		for(VarIdentifier varId : this.agentsComplexity.keySet()) {
			this.agentsId.add(varId);
		}
		this.comparator = new Comparator<VarIdentifier>() {

			@Override
			public int compare(VarIdentifier varId1, VarIdentifier varId2) {
				int deg1 = agentsComplexity.get(varId1).getNbNeighbors();
				int deg2 = agentsComplexity.get(varId2).getNbNeighbors();
				int score = deg2 - deg1;
				if (score == 0) {
					int dom1 = agentsComplexity.get(varId1).getMaxDomain();
					int dom2 = agentsComplexity.get(varId2).getMaxDomain();
					score = dom1 - dom2;
				}
				return score;
			}

		};
		Collections.sort(this.agentsId, this.comparator);
		while (this.agentsId.size() != 0) {
			VarIdentifier varId = this.agentsId.remove(0);
			order.add(varId);
			List<VarIdentifier> localAgents = this.agentsComplexity.get(varId).getLocalAgents();
			localAgents.removeAll(order);
			if (!localAgents.isEmpty()) {
				Collections.sort(localAgents, this.comparator);
				order.addAll(localAgents);
			}
		}
		return order;
	}

	/**
	 * @param complexities
	 * @return
	 */
	public static HashList<VarIdentifier> buildpriorityTree(final HashMap<VarIdentifier, AgentComplexity> complexities) {
		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
		HashList<VarIdentifier> varIds = new HashList<VarIdentifier>();
		for(VarIdentifier varId : complexities.keySet()) {
			varIds.add(varId);
		}
		Comparator<VarIdentifier> comparator = new Comparator<VarIdentifier>() {

			@Override
			public int compare(VarIdentifier varId1, VarIdentifier varId2) {
				int deg1 = complexities.get(varId1).getNbNeighbors();
				int deg2 = complexities.get(varId2).getNbNeighbors();
				int score = deg2 - deg1;
				if (score == 0) {
					int dom1 = complexities.get(varId1).getMaxDomain();
					int dom2 = complexities.get(varId2).getMaxDomain();
					score = dom1 - dom2;
				}
				return score;
			}

		};
		Collections.sort(varIds, comparator);
		while (varIds.size() != 0) {
			VarIdentifier varId = varIds.remove(0);
			order.add(varId);
			List<VarIdentifier> localAgents = complexities.get(varId).getLocalAgents();
			localAgents.removeAll(order);
			if (!localAgents.isEmpty()) {
				Collections.sort(localAgents, comparator);
				order.addAll(localAgents);
			}
		}
		return order;
	}

}
