package kernel.order;

import java.util.HashMap;
import java.util.Iterator;
import kernel.VarIdentifier;
import util.HashList;
import util.Tools;

/**
 * A Class for Min-Domain Degree ordering
 */
public class MinDomOverDegAO extends CentralizedAO implements ChainOrder {

	public MinDomOverDegAO() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see order.CentralizedAgentOrdering#buildAgentsOrder()
	 */
	public HashList<VarIdentifier> buildAgentsOrder() {
		return this.buildAgentsOrderForComplex();
	}

	/**
	 * 
	 */
	public HashList<VarIdentifier> buildAgentsOrderForComplex() {
		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
		int n = this.agentsComplexity.keySet().size();
		HashMap<VarIdentifier, Double> dom_deg = new HashMap<VarIdentifier, Double>();
		Iterator<VarIdentifier> it = this.agentsComplexity.keySet().iterator();
		while (it.hasNext()) {
			VarIdentifier agId = it.next();
			AgentComplexity agComp = this.agentsComplexity.get(agId);
			// System.out.println(agId+" : "+agComp.getLocalAgents());
			if (agComp.getNumberConstraints() == 0) {
				dom_deg.put(agId, Double.POSITIVE_INFINITY);
			} else {
				dom_deg.put(agId, ((double) agComp.getMaxDomain()) / agComp.getNumberConstraints());
			}
		}
		while (order.size() < n) {
			VarIdentifier agId = Tools.MinOf(dom_deg);
			order.add(agId);
			dom_deg.remove(agId);
			HashList<VarIdentifier> localAgents = this.agentsComplexity.get(agId).getLocalAgents();
			HashList<VarIdentifier> local_agents = new HashList<VarIdentifier>(localAgents);
			local_agents.removeAll(order);
			while (!order.contains(localAgents)) {
				HashMap<VarIdentifier, Double> local_dom_deg = new HashMap<VarIdentifier, Double>();
				for(VarIdentifier id : local_agents) {
					local_dom_deg.put(id, dom_deg.get(id));
				}
				VarIdentifier localId = Tools.MinOf(local_dom_deg);
				order.add(localId);
				dom_deg.remove(localId);
				local_agents.remove(localId);
			}
		}
		// System.out.println(order);
		// System.exit(0);
		return order;
	}

	/**
	 * @param complexities
	 * @return
	 */
	public static HashList<VarIdentifier> buildpriorityTree(HashMap<VarIdentifier, AgentComplexity> complexities) {
		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
		int n = complexities.keySet().size();
		HashMap<VarIdentifier, Double> dom_deg = new HashMap<VarIdentifier, Double>();
		Iterator<VarIdentifier> it = complexities.keySet().iterator();
		while (it.hasNext()) {
			VarIdentifier agId = it.next();
			AgentComplexity agComp = complexities.get(agId);
			// System.out.println(agId+" : "+agComp.getLocalAgents());
			if (agComp.getNumberConstraints() == 0) {
				dom_deg.put(agId, Double.POSITIVE_INFINITY);
			} else {
				dom_deg.put(agId, ((double) agComp.getMaxDomain()) / agComp.getNumberConstraints());
			}
		}
		while (order.size() < n) {
			VarIdentifier agId = Tools.MinOf(dom_deg);
			order.add(agId);
			dom_deg.remove(agId);
			HashList<VarIdentifier> localAgents = complexities.get(agId).getLocalAgents();
			HashList<VarIdentifier> local_agents = new HashList<VarIdentifier>(localAgents);
			local_agents.removeAll(order);
			while (!order.contains(localAgents)) {
				HashMap<VarIdentifier, Double> local_dom_deg = new HashMap<VarIdentifier, Double>();
				for(VarIdentifier id : local_agents) {
					local_dom_deg.put(id, dom_deg.get(id));
				}
				VarIdentifier localId = Tools.MinOf(local_dom_deg);
				order.add(localId);
				dom_deg.remove(localId);
				local_agents.remove(localId);
			}
		}
		// System.out.println(order);
		// System.exit(0);
		return order;
	}

}
