/************************************************************************

 **************************************************************************/

package kernel.vars.integer;

import kernel.AbstractAgent;
import kernel.ExternalVar;
import kernel.VarIdentifier;
import kernel.Variable;
import util.VarAssignment;

/**
 * A Class models the integer external variables
 * 
 * @author Redouane EZZAHIR
 * 
 */
@SuppressWarnings("serial")
public class IntExternalVar extends IntVariable implements Variable, ExternalVar {

	public IntExternalVar(AbstractAgent owner, IntVar v) {
		this(owner, v.getVarAssignment());
		this.name = v.getName();
		this.domain = v.cloneOriginalDomain();
	}

	/**
	 * Constructs an external variable
	 * 
	 * @param owner
	 *            The agent owning this object
	 * @param varId
	 *            the identifier of the external variable
	 * @param min
	 *            The minimal value in the domain
	 * @param max
	 *            The maximal value in the the domain
	 */

	public IntExternalVar(AbstractAgent owner, VarIdentifier varId, int min, int max) {
		super(owner, min, max);
		if (owner.getAgentId() == varId.getAgentId())
			throw new Error("External variable must have an agent Id different " + "from its owner Id (owner :" + owner.toString()
					+ " VarIdentifier : " + varId);
		this.setVarIdentifier(varId);
		this.name = "X" + varId.toString();
	}

	/**
	 * Constructs an external variable
	 * 
	 * @param owner The agent owning this object
	 * @param varId  the identifier of the external variable
	 * @param vals the domain of the external variable
	 */
	public IntExternalVar(AbstractAgent owner, VarIdentifier varId, String name, Integer[] values) {
		super(owner, name, varId, values);
		if (owner.getAgentId() == varId.getAgentId())
			throw new Error("External variable must have an agent Id different " + "from its owner Id (owner :" + owner.toString()
					+ " VarIdentifier : " + varId);
	}

	/**
	 * Constructs an external variable with unknown domain from a variable assignment
	 * 
	 * @param owner
	 *            The agent owning this object
	 * @param varAss
	 *            the variable assignment of the external variable to create
	 */
	public IntExternalVar(AbstractAgent owner, VarAssignment varAss) {
		this(owner, varAss.getVarIdentifier(), 0, 0);
		this.timeCounter = varAss.getTimeCounter();
		this.currentValue = varAss.getValue();
	}

	/**
	 * Instantiate the variable to the value in the given assignment and update the time counter
	 * 
	 * @param v
	 */
	public void instantiate(VarAssignment v) {
		super.instantiate(v.getValue());
		this.timeCounter = v.getTimeCounter();
	}

}
