package kernel.vars.integer.explain;

import java.util.BitSet;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import kernel.vars.integer.BitSetIntDomain;
import util.IntArray;

public class ExplainedBitSetIntDomain extends BitSetIntDomain implements ExplainedIntDomain {

	/**
	 * Each BitSet of this array stores a list of indices explaining the the deletion of a value from the domain
	 **/
	protected Explanation[] explanations;
	//
	// /**
	// * A linked list for restoration events.
	// */
	//
	// int[] restoreChain;
	//
	// /**
	// * The begin of the <code>restoreChain</code> linked list.
	// */
	//
	// int firstRestValueToBePropagated = -1;
	/**
	 * States weather a value will be restored.
	 */

	BitSet willBeRestored;

	public ExplainedBitSetIntDomain(String name, int min, int max) {
		super(name, min, max);
		this.explanations = new Explanation[this.capacity];
		// this.restoreChain = new int[this.capacity];
		this.willBeRestored = new BitSet();
	}

	public ExplainedBitSetIntDomain(String name, Integer[] values) {
		super(name, values);
		this.explanations = new Explanation[this.capacity];
		// this.restoreChain = new int[this.capacity];
		this.willBeRestored = new BitSet();
	}

	public boolean addValue(int value) {
		if (super.addValue(value)) {
			Explanation[] newExplanations = new Explanation[this.capacity];
			System.arraycopy(this.explanations, 0, newExplanations, 0, explanations.length);
			this.explanations = newExplanations;

			// int[] newrestoreChain = new int[this.capacity];
			// System.arraycopy(this.restoreChain, 0, newrestoreChain, 0,restoreChain.length);
			// this.restoreChain = newrestoreChain;
			return true;
		}
		return false;
	}

	@Override
	public boolean updateInf(int inf, int index) {
		boolean removed = false;
		int newInfIndex = inf - offset;
		for (int i = contents.nextSetBit(0) ; i < newInfIndex && i > -1 ; i = contents.nextSetBit(i + 1))
			if (this.removeIndex(i)) {
				removed = true;
				if (this.explanations[i] == null)
					this.explanations[i] = new Explanation();
				this.explanations[i].add(index);
			}
		return removed;
	}

	@Override
	public boolean updateSup(int sup, int index) {
		boolean removed = false;
		int newsupIndex = sup - offset;
		int i = contents.length() - 1;
		while (i > newsupIndex && i > -1) {
			if (this.removeIndex(i)) {
				removed = true;
				if (this.explanations[i] == null)
					this.explanations[i] = new Explanation();
				this.explanations[i].add(index);
			}
			i--;
		}
		return removed;
	}

	@Override
	public void addExplanation(int val, int index) {
		int i = this.indexOf(val);
		if (i > -1) {
			if (this.explanations[i] == null)
				this.explanations[i] = new Explanation();
			this.explanations[i].add(index);
		}
	}

	@Override
	public boolean remove(int val, int index) {
		int i = this.indexOf(val);
		boolean rem = super.remove(val);
		if (rem) {
			if (this.explanations[i] == null)
				this.explanations[i] = new Explanation();
			this.explanations[i].add(index);
		}
		return rem;

	}

	/**
	 * Removes an explanation index from the list of explanation.
	 * 
	 */
	public boolean removeExplanation(int index) {
		boolean restored = false;
		if (index > -1)
			for (int i = 0 ; i < this.explanations.length ; i++) {
				if (this.explanations[i] != null) {
					if (this.explanations[i].delete(index)) {
						this.explanations[i] = null;
						this.contents.set(i);
						this.removedVals.clear(i);
						if (!this.willBeRestored.get(i)) {
							// verifie d'abord que l'evenement de restauration
							// n'est pas deja la avant de ne l'ajouter.
							// restoreChain[i] = firstRestValueToBePropagated;
							// firstRestValueToBePropagated = i;
							this.willBeRestored.set(i);
							restored = true;
						}
					}
				}
			}
		return restored;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean removeExplanation(int value, int index) {
		boolean restored = false;
		int i = this.indexOf(value);
		if (i > -1 && index > -1) {
			if (this.explanations[i] != null) {
				if (this.explanations[i].delete(index)) {
					this.explanations[i] = null;
					this.contents.set(i);
					this.removedVals.clear(i);
					if (!this.willBeRestored.get(i)) {
						// verifie d'abord que l'evenement de restauration
						// n'est pas deja la avant de ne l'ajouter.
						// restoreChain[i] = firstRestValueToBePropagated;
						// firstRestValueToBePropagated = i;
						this.willBeRestored.set(i);
						restored = true;
					}
				}
			}
		}
		return restored;
	}

	/**
	 * Returns the set of explanations stored for the given value
	 */
	public Collection<Explanation> getExplanation(int val) {
		HashSet<Explanation> e = new HashSet<Explanation>();
		int i = this.indexOf(val);
		if (i < 0 ) //TODO corrected by Mohamed 2011-04-17
			return e;
//			throw new Error("Value :" + val + " out of domain " + this);
		if (this.explanations[i] == null)
			return e;
		e.add(this.explanations[i]);
		return e;
	}

	@Override
	public ExplainedIntDomain cloneOriginalDomain() {
		ExplainedBitSetIntDomain d = new ExplainedBitSetIntDomain(name, offset, offset + this.originalSize);
		d.contents = (BitSet) this.contents.clone();
		d.contents.or(this.removedVals);
		return d;
	}

	/**
	 * Returns an iterator on restored values.
	 */

	public Iterator<Integer> getRepairIterator() {
		return new RepairIntIterator();
	}

	/**
	 * A class for iterate over values of the current domain
	 */

	private class RepairIntIterator implements Iterator<Integer> {
		protected int currentIndex = -1;

		private RepairIntIterator() {}

		public boolean hasNext() {
			return (willBeRestored.nextSetBit(currentIndex + 1) != -1);
		}

		public Integer next() {
			currentIndex = willBeRestored.nextSetBit(currentIndex + 1);
			return currentIndex + offset;
		}

		/* (non-Javadoc)
		 * @see java.util.Iterator#remove()
		 */
		@Override
		public void remove() {
			// TODO Auto-generated method stub
			
		}
	
	}

	/**
	 * If restoration handling completes, the chain must reinitialized to null.
	 */

	public void clearRepairDomain() {
		this.willBeRestored.clear();
	}

	@Override
	public void releaseRepairDomain() {
		this.willBeRestored.clear();
	}

	public String toString() {
		StringBuffer buf = new StringBuffer("");
		for (Explanation e : this.explanations) {
			buf.append(e + "|");
		}
		return super.toString() + "  Exp :[" + buf.toString() + "]";
	}

	@Override
	public Collection<Explanation> getExplanations(IntArray values) {
		HashSet<Explanation> exp = new HashSet<Explanation>();
		for (int val : values) {
			int i = this.indexOf(val);
			if (i == -1)
				throw new Error("Value :" + val + " out of domain " + this);
			if (this.explanations[i] != null)
				exp.add(this.explanations[i]);
		}
		return exp;
	}

	@Override
	public Collection<Explanation> getDomExplanations() {
		Collection<Explanation> e = new HashSet<Explanation>();
		for (Explanation exp : this.explanations) {
			if (exp != null)
				e.add(exp);
		}
		return e;
	}

	@Override
	public Collection<Explanation> getInfExplanations() {
		Collection<Explanation> e = new HashSet<Explanation>();
		for (int i = this.removedVals.nextSetBit(0) ; i >= 0 && i < indexOf(this.getInf()) ; i = this.removedVals.nextSetBit(i + 1)) {
			if (this.explanations[i] != null)
				e.add(this.explanations[i]);
		}
		return e;
	}

	@Override
	public Collection<Explanation> getSupExplanations() {
		Collection<Explanation> e = new HashSet<Explanation>();
		int idxsup = indexOf(getSup());
		for (int i = this.removedVals.nextSetBit(idxsup) ; i >= 0 ; i = this.removedVals.nextSetBit(i + 1)) {
			if (this.explanations[i] != null)
				e.add(this.explanations[i]);
		}
		return e;
	}

}
