package kernel.vars.integer.explain;

import java.util.Collection;
import static simulation.parser.InstanceTokens.DOMAIN_PREFIX;
import kernel.AbstractAgent;
import kernel.AbstractVar;
import kernel.Constraint;
import kernel.VarIdentifier;
import kernel.events.ExplainedIntVarEvent;
import kernel.events.VariableEvent;
import kernel.events.VariableEventListener;
import kernel.vars.integer.IntVariable;
import simple.SingleExplainedVarAgent;
import util.IntArray;

@SuppressWarnings("serial")
public class ExplainedIntVariable extends IntVariable implements ExplainedIntVar {

	public ExplainedIntVariable(IntVariable v) {
		super(v.getAgentOwner(), v.getName(), v.getDomain().toArray());
		this.domain = new ExplainedBitSetIntDomain(DOMAIN_PREFIX + owner.name, this.domain.toArray());
		this.event = new ExplainedIntVarEvent(this);
	}

	public ExplainedIntVariable(AbstractAgent owner, String name, int min, int max) {
		super(owner, name, min, max);
		this.domain = new ExplainedBitSetIntDomain(DOMAIN_PREFIX + owner.name, min, max);
		this.event = new ExplainedIntVarEvent(this);

	}

	public ExplainedIntVariable(AbstractAgent owner, String name, Integer[] values) {
		super(owner, name, values);
		this.domain = new ExplainedBitSetIntDomain(DOMAIN_PREFIX + name, values);
		this.event = new ExplainedIntVarEvent(this);
	}

	public ExplainedIntVariable(AbstractAgent owner, String name, VarIdentifier varId, Integer[] values) {
		super(owner, name, varId, values);
		this.domain = new ExplainedBitSetIntDomain(DOMAIN_PREFIX + name, values);
		this.event = new ExplainedIntVarEvent(this);
	}

	public ExplainedIntVariable(AbstractAgent owner, int min, int max) {
		super(owner, min, max);
		this.domain = new ExplainedBitSetIntDomain(DOMAIN_PREFIX + owner.name, min, max);
		this.event = new ExplainedIntVarEvent(this);

	}

	public ExplainedIntVariable(AbstractAgent owner, Integer[] values) {
		super(owner, values);
		this.domain = new ExplainedBitSetIntDomain(DOMAIN_PREFIX + name, values);
		this.event = new ExplainedIntVarEvent(this);

	}

	/**
	 * Removes a listener (constraint) from the listener list of the variable
	 */
	public void removeVariableEventListener(int eventType, VariableEventListener listener) {
		if (eventType == VariableEvent.DESINSTANCIATE) {
			for (AbstractVar v : ((Constraint) listener).getVars())
				if (this.isInstantiated())
					((ExplainedIntVar) v).removeExplanation(((Constraint) listener).getIndex());
		}
		super.removeVariableEventListener(eventType, listener);
	}

	@Override
	public boolean removeExplanation(int value, int index) {
		boolean restored = false;
		if (((ExplainedIntDomain) this.domain).removeExplanation(value, index)) {
			// this.owner.log(this + "  removeExplanation " + value + "  " + index + "   ", 1);
			this.fireChangedDomain(VariableEvent.RESTORE_VAL, index);
			restored = true;
		}
		return restored;
	}

	@Override
	public boolean removeExplanation(int index) {
		boolean restored = ((ExplainedIntDomain) this.domain).removeExplanation(index);
		if (restored) {
			// this.owner.log(this + "  removeExplanation " + index + " ", 1);
			this.fireChangedDomain(VariableEvent.RESTORE_VAL, index);
		}
		return restored;
	}

	@Override
	public boolean removeExplanationForInterval(int min, int max, int index) {
		boolean restored = false;
		for (int val = min ; val <= max ; val++) {
			if (this.domain.originalDomainContains(val)) {
				restored |= ((ExplainedIntDomain) domain).removeExplanation(val, index);
			}
		}
		if (restored) {
			this.fireChangedDomain(VariableEvent.RESTORE_VAL, index);
		}

		return restored;
	}

	@Override
	public ExplainedIntDomain getDomain() {
		return (ExplainedIntDomain) this.domain;
	}

	public boolean removeValue(int val, int index, int expl) {
		if (this.getDomain().remove(val, expl)) {
			// this.owner.log(this + "  removeValue " + val+ "  because " + expl + " ", 1);
			if (this.isInstantiated()) {
				if (val == this.getValue())
					this.desinstanciate(index);
			}
			this.fireChangedDomain(VariableEvent.REMOVE_VAL, index);
			return true;
		}
		return false;

	}

	@Override
	public boolean updateInf(int inf, int index, int expl) {
		boolean update = this.getDomain().updateInf(inf, expl);
		if (update) {
			if (this.isInstantiated()) {
				if (this.getValue() < inf)
					this.desinstanciate(index);
			}
			this.fireChangedDomain(VariableEvent.INCREMENT_INF, index);
			// this.fireChangedDomain(VariableEvent.REMOVE_VAL, index);
		}
		return update;
	}

	@Override
	public boolean updateSup(int sup, int index, int expl) {
		boolean update = this.getDomain().updateSup(sup, expl);
		if (update) {
			if (this.isInstantiated()) {
				if (this.getValue() > sup)
					this.desinstanciate(index);
			}
			this.fireChangedDomain(VariableEvent.DECREMENT_SUP, index);
			// this.fireChangedDomain(VariableEvent.REMOVE_VAL, index);
		}
		return update;
	}

	@Override
	public boolean removeInterval(int min, int max, int index, int expl) {
		boolean update = false;
		//val >=min && 
		for (int val = this.getNextDomainValue(min - 1) ; val <= max ; val = this.getNextDomainValue(val)){
			update |= this.getDomain().remove(val, expl);
		}	
		if (this.isInstantiated()) {
			if (!this.domain.contains(this.getValue()))
				this.desinstanciate(index);
		}
		if (update)
			this.fireChangedDomain(VariableEvent.REMOVE_VAL, index);
		return update;

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	/*
	 * fix a bug: fixed by Mohamed the 2012/01/08
	 * update inf and update sup afterwards return update inf or update sup 
	 */
	public boolean restrictDomainTo(int val, int index, int expl) {
		boolean updateInf = this.updateInf(val, index, expl);
		boolean updateSup = this.updateSup(val, index, expl);
		return updateInf || updateSup;
	}

	/**
	 * @return a clone object of original domain
	 */
	@Override
	public ExplainedIntDomain cloneOriginalDomain() {
		return this.getDomain().cloneOriginalDomain();
	}

	@Override
	public Collection<Explanation> getExplanations(IntArray vals) {
		return this.getDomain().getExplanations(vals);
	}

	@Override
	public Collection<Explanation> getExplanation(int val) {
		return this.getDomain().getExplanation(val);
	}

	@Override
	public boolean updateInf(int val, int idx) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean updateSup(int val, int idx) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean removeValue(int val, int idx) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean removeInterval(int min, int max, int index) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean restrictDomainTo(int val, int index) {
		throw new UnsupportedOperationException();
	}

	@Override
	public int makeConstraintNogood(int val, Collection<Explanation> expl) {
		SingleExplainedVarAgent agent = (SingleExplainedVarAgent) this.owner;
		int idx = agent.buildNogoodConstraint(this, val, expl);
		return idx;
	}

	@Override
	public int makeConstraintNogood(IntArray vals, Collection<Explanation> expl) {
		SingleExplainedVarAgent agent = (SingleExplainedVarAgent) this.owner;
		int idx = agent.buildNogoodConstraint(this, vals, expl);
		return idx;
	}

	@Override
	public Collection<Explanation> getDomExplanations() {
		return this.getDomain().getDomExplanations();
	}

	@Override
	public Collection<Explanation> getInfExplanations() {
		return this.getDomain().getInfExplanations();
	}

	@Override
	public Collection<Explanation> getSupExplanations() {
		return this.getDomain().getSupExplanations();
	}

	@Override
	public boolean removeValue(int val, int index, Collection<Explanation> expl) {
		int exp = makeConstraintNogood(val, expl);
		return this.removeValue(val, index, exp);
	}

	@Override
	/*
	 * fix a bug: modified by Mohamed && instead of || 
	 */
	public boolean restrictDomainTo(int val, int index, Collection<Explanation> expl) {
		return this.updateInf(val, index, expl) && this.updateSup(val, index, expl);
	}

	@Override
	public boolean removeInterval(int min, int max, int index, Collection<Explanation> expl) {
		IntArray vals = new IntArray();
		for (int val = this.getNextDomainValue(min - 1) ; val <= max ; val = this.getNextDomainValue(val))
			vals.add(val);
		if(vals.size()>0){
			int exp = this.makeConstraintNogood(vals, expl);
			return this.removeInterval(min, max, index, exp);
		} else {
			return false;
		}
	}

	@Override
	public boolean updateInf(int inf, int index, Collection<Explanation> expl) {
		IntArray vals = new IntArray();
		for (int val = this.getInf() ; val < inf ; val = this.getNextDomainValue(val))
			vals.add(val);
		int exp = this.makeConstraintNogood(vals, expl);
		return this.updateInf(inf, index, exp);
	}

	@Override
	public boolean updateSup(int sup, int index, Collection<Explanation> expl) {
		IntArray vals = new IntArray();
		for (int val = this.getSup() ; val > sup ; val = this.getPrevDomainValue(val))
			vals.add(val);
		int exp = this.makeConstraintNogood(vals, expl);
		return this.updateSup(sup, index, exp);
	}

}
