package realisticuse;

import kernel.AbstractAgent;
import kernel.VarIdentifier;
import saci.Config;
import saci.Message;
import saci.MessageHandler;
import util.Commands;
import util.HashList;

public class AgentManager extends SaciComm {

	/**
	 * 
	 */
	private static final long serialVersionUID = -4621757691688324434L;

	AbstractAgent agent;

	String algorithm;

	public AgentManager(String name, Config c) {
		super(name);
		this.config = c;
	}

	/**
	 * adds handlers for DisSolver specific messages
	 **/

	public void addMsgHandlers() {
		// add a message handler to partial-distributed-problem message
		addMessageHandler("partial-problem", new MessageHandler() {
			public boolean processMessage(Message m) {
				String pp = (String) m.get("partial-problem");
				System.out.println(myName + ": I received my partial-problem \n" + pp);
				parsePartialProblem(pp);
				return true;
			}
		});

		// add a message handler to set-algorithm message
		addMessageHandler("set-algorithm", new MessageHandler() {
			public boolean processMessage(Message m) {
				algorithm = (String) m.get("algorithm");
				System.out.println(myName + ": I received the name of algorithm : " + algorithm);
				creatDisChocoAgent();
				return true; // no other message handler gives this message
			}
		});
		// add a message handler to ok-to-start message
		addMessageHandler("ok-to-start", new MessageHandler() {
			public boolean processMessage(Message m) {
				System.out.println(myName + ": I received ok-to-start");
				return true; // no other message handler gives this message
			}
		});
		// add a message handler to msg messages
		addMessageHandler("msg", new MessageHandler() {
			public boolean processMessage(Message m) {
				kernel.com.Message msg = MsgConverter.convertSaciMsg(m);
				System.out.println(myName + ": I received msg :" + msg);
				agent.notifyReception(msg);
				return true;
			}

		});
	}

	protected void creatDisChocoAgent() {
		sendGetStart();
	}

	private void sendGetStart() {
		Message m = this.createMessage("get-start");
		m.put("receiver", "Master");
		m.put("first-name", myName);
		sendSaciMsg(m);
	}

	protected void parsePartialProblem(String pp) {
		setGetAlgorithm();
	}

	private void setGetAlgorithm() {
		Message m = this.createMessage("get-algorithm");
		m.put("receiver", "Master");
		m.put("first-name", myName);
		sendSaciMsg(m);
	}

	@SuppressWarnings("static-access")
	public void run() {
		sendRegistrationMsg();
		int i = 0;
		// try{
		try {
			while (i < 21) {
				i++;
				System.err.print("-");
				Thread.currentThread().sleep(100);
				if (i == 20) {
					kernel.com.Message msg = new kernel.com.Message(Commands.TERMINATION_DETECTION);
					msg.addContents(Commands.TERMINATE, Commands.SUCCESS);
					this.sendMessage(msg);
				}
			}
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// this.getMBox(socName).stop();
		// this.getControlAg().kill();
		// this.destroy();
		// this.stopAg();
		// } catch (RemoteException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }

		System.out.println("\n--------- " + this.myName + ": Good by! --------------");
		// System.exit(0);
	}

	protected void sendRegistrationMsg() {
		Message m = this.createMessage("registration");
		m.put("receiver", "Master");
		m.put("first-name", myName);
		m.put("description", myDiscription);
		sendSaciMsg(m);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String arg[] = {"red"};
		args = arg;
		if (args.length == 0) {
			System.err.println("Illegal number of arguments \n" + "Usage : java AgentManager <agent-name> [<master-host>]  ");
			System.exit(-1);
		}
		Config c = new Config();
		if (args.length == 1) { // master host
			c.set("master", "localhost");
		} else
			c.set("master", args[1]);

		AgentManager agManger = new AgentManager(args[0], c);
		agManger.start();
		// //AgentManager agManger2 = new AgentManager(args[0], c);
		// /agManger.start();
		System.exit(0);
	}

	@Override
	public boolean checkEnd() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public HashList<VarIdentifier> getRegisteredAgentIds() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean isRegistered(VarIdentifier agentId) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void register(VarIdentifier agentId) {
	// TODO Auto-generated method stub
	}

}
