/************************************************************************

 **************************************************************************/

package realisticuse;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import kernel.AbstractMaster;
import kernel.VarIdentifier;
import saci.Message;
import saci.MessageHandler;
import saci.launcher.Command;
import simulation.parser.InstanceParser;
import util.Commands;
import util.HashList;

/**
 * Class DisSolver for realistic use
 */
public class DisSolver extends SaciComm {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7717556078797189642L;
	List<String> readyAgents;
	AbstractMaster master;
	InstanceParser parser;
	protected Properties properties = new Properties();;

	public DisSolver() {
		super("Master");
		readyAgents = new ArrayList<String>();
	}

	protected boolean connectLauncher() {
		if (super.connectLauncher())
			try {
				String socH = launcher.getSocietyHost(config.get("society.name"));
				if (socH == null)
					launcher.createFacilitator(config);
			} catch (Exception e) {
				System.err.println("Error creating facilitator  " + "\n" + e);
				return false;
			}
		return true;
	}

	public void initAg(String[] args) {
		addMsgHandlers();
	}

	void killAgents() {
		try {
			Command command = new Command(Command.KILL_SOCAGS);
			command.addArg("society.name", socName);
			launcher.execCommand(command);

			command = new Command(Command.KILL_SOC);
			command.addArg("society.name", socName);
			launcher.execCommand(command);
		} catch (Exception e) {
			System.err.println("Error killing previous agents");
		}
	}

	/** adds handlers for DisSolver specific messages */
	public void addMsgHandlers() {
		// add a message handler to receive registration messages
		addMessageHandler("registration", new MessageHandler() {
			public boolean processMessage(Message m) {
				String firstName = (String) m.get("first-name");
				String senderDescription = (String) m.get("description");
				System.out.println("Master receives registration message from " + m.getSender() + "\n...Its first name is " + firstName
						+ " with the description : " + senderDescription);
				sendParialDisProlem(firstName, m.getSender());
				return true;
			}
		});

		// add a message handler to get-algorithm messages
		addMessageHandler("get-algorithm", new MessageHandler() {
			public boolean processMessage(Message m) {
				System.out.println("Master : I received get-algorithm from " + m.getSender());
				String algorithm = (String) properties.getProperty("algorithm", "ABT");
				Message mRep = createMessage("set-algorithm");
				mRep.put("receiver", m.getSender());
				mRep.put("algorithm", algorithm);
				sendSaciMsg(mRep);
				return true;
			}
		});

		// add a message handler to get-start messages
		addMessageHandler("get-start", new MessageHandler() {
			public boolean processMessage(Message m) {
				System.out.println("Master receives get-start from " + m.getSender());
				readyAgents.add(m.getSender());
				// if(readyAgents.size()==getNberAgents()){
				breadcastStart();
				// }
				// else{
				// Message mRep = createCompressedMsg("sorry");
				// mRep.put("receiver", m.getSender());
				// mRep.put("ready-agents", readyAgents.toString() );
				// sendMessage(mRep);
				// }
				return true;
			}
		});
		// add a message handler to msg messages
		addMessageHandler("msg", new MessageHandler() {
			public boolean processMessage(Message m) {
				System.out.println("Master receives msg from " + m.getSender());
				kernel.com.Message msg = MsgConverter.convertSaciMsg(m);
				// master.notifyReception(msg);
				terminationDetection(msg);
				return true;
			}

		});
	}

	protected void breadcastStart() {
		Message start = createMessage("ok-to-start");
		// TODO add options
		broadcast(start);
	}

	protected void terminationDetection(kernel.com.Message msg) {
		if (msg.getType().equals(Commands.TERMINATION_DETECTION)) {
			if (msg.contains(Commands.TERMINATE)) {
				// while(!this.master.isFinished()){
				// try {
				// Thread.sleep(1000);
				// } catch (InterruptedException e) {
				// e.printStackTrace();
				// }
				// }
				System.out.println("Master has received a terminate message, I want to terminate also.");
				try {
					try {
						int i = 0;
						while (i < 21) {
							i++;
							System.err.print("| ");
							Thread.sleep(200);
						}
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					this.stopAg();
					this.killAgents();
					launcher.killFacilitator(socName);
					this.destroy();

				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				System.out.println("\n--------- " + this.myName + ": Good by! --------------");
				System.exit(0);
			}
		}
	}

	protected int getNberAgents() {

		return parser.getMapOfAgents().size();
	}

	public void sendParialDisProlem(String firstName, String receiver) {
		Message m = this.createCompressedMsg("partial-problem");
		m.put("receiver", receiver);
		m
				.put(
						"partial-problem",
						"X1 + X2 > 0 ; X1 + X2 > 0 ; \n"
								+ "X1 + X2 > 0 ; X1 + X2 > 0 ; \n"
								+ " X1 + X2 > 0 ;  X1 + X2 > 0 ;\n"
								+ "Relation : <relations nbRelations=\"7\">   <relation name=\"R1\" arity=\"2\" nbTuples=\"12\" semantics=\"conflits\">4 1|1 4|1 2|4 4|1 1|0 2|3 2|4 3|4 2|3 4|2 1|0 3</relation>"
								+ "\n<relation name=\"R2\" arity=\"2\" nbTuples=\"12\" semantics=\"conflits\">2 2|2 0|1 4|1 0|3 1|3 0|0 1|0 4|4 4|3 3|1 1|2 3</relation>"
								+ " \n<relation name=\"R3\" arity=\"2\" nbTuples=\"12\" semantics=\"conflits\">2 1|4 1|3 2|2 0|4 0|4 3|3 1|0 4|0 2|1 0|1 3|0 1</relation>"
								+ "\n  <relation name=\"R4\" arity=\"2\" nbTuples=\"12\" semantics=\"conflits\">2 3|4 2|2 0|4 0|3 2|0 0|1 4|2 4|1 3|4 3|0 4|3 0</relation>"
								+ " \n   <relation name=\"R5\" arity=\"2\" nbTuples=\"12\" semantics=\"conflits\">3 4|1 3|1 1|1 4|3 2|1 0|4 3|2 3|2 4|3 0|2 0|3 1</relation>"
								+ "\n <relation name=\"R6\" arity=\"2\" nbTuples=\"12\" semantics=\"conflits\">4 1|4 4|0 2|0 1|1 3|3 2|4 0|2 1|2 2|4 2|1 1|0 3</relation>"
								+ "\n  <relation name=\"R7\" arity=\"2\" nbTuples=\"12\" semantics=\"conflits\">1 2|4 0|3 3|1 4|4 1|2 2|0 2|1 3|0 1|0 3|1 1|2 3</relation>"
								+ " \n</relations> ");
		this.sendSaciMsg(m);
	}

	public static void main(String[] args) {
		DisSolver solver = new DisSolver();
		if (args.length == 1) { // master host
			solver.config.set("master", args[0]);
		}
		solver.start();
	}

	@Override
	public boolean checkEnd() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public HashList<VarIdentifier> getRegisteredAgentIds() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean isRegistered(VarIdentifier agentId) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void register(VarIdentifier agentId) {
	// TODO Auto-generated method stub

	}

	@Override
	public void sendMessage(kernel.com.Message msg) {
	// TODO Auto-generated method stub

	}

}
