/**
 * DisChcoco is an open-source platform for development and experimentation in
 * distributed constraint programming.
 *The DisChoco project has been proposed by M. Christian BESSIERE (LIRMM/CNRS) 
 *and supported by the Maroc-France PAI project no. MA/05/127.
 * 
 * @author R. EZZAHIR
 *  2005 - 2008
 */

package realisticuse.com;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import saci.Agent;
import saci.Config;
import saci.Message;
import saci.MessageHandler;

/**
 * A MasterAgent is responsible for to start protocol It is also responsible to detected the end of distributed search.
 * 
 * @author R. EZZAHIR
 */

public class SaciMaster extends Agent {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7833760469472289130L;
	protected Properties prop = null;
	protected Thread executor;
	protected HashMap<String, String> agents;
	protected int NbAgents = 0;
	protected HashMap<String, Object> AcceptedMsgInitialEndAgents;
	protected int NbInitialEndAgents;

	public SaciMaster(Properties conf) {
		super();
		prop = conf;
		NbAgents = new Integer(prop.getProperty("NbAgents")).intValue();
		AcceptedMsgInitialEndAgents = new HashMap<String, Object>();
		agents = new HashMap<String, String>();
		executor = (new Thread(this));
	}

	/**
	 * Finalizes the agent
	 */
	public void stopAg() {
		super.stopAg();
	}

	public void Start() {
		executor.start();
	}

	/**
	 * iniciate the agent
	 */
	public void initAg(String[] args) {
		running = true;
		try {
			mbox.addMessageHandler("control", null, null, "", new MessageHandler() {

				public boolean processMessage(Message m) {
					if (m.get("command").equals("getStartOrder")) {
						agents.put(m.getSender(), "" + agents.size());
						if (agents.size() == NbAgents) {
							sendStart();
							return true;
						}
					}
					if (m.get("command").equals("Termination.Detector") && m.get("MessageType").equals("Accepted")) {
						System.out.println(" Accepted " + m.getSender() + " " + m.getReceiver());

						AcceptedMsgInitialEndAgents.put(m.getSender(), m.get("accepted"));
						if (AcceptedMsgInitialEndAgents.size() == NbInitialEndAgents) {
							System.out.println("Send Stop mssage");
							Message msg = createMessage("ChocoComm");
							msg.put("msg::type", "Stop");
							msg.put("condition", "true");
							try {
								mbox.broadcast(msg);
							} catch (RemoteException e) {
								e.printStackTrace();
							}
							stopAg();
							System.exit(0);
						}

					}

					if (m.get("command").equals("InitialEndAgent")) {
						NbInitialEndAgents++;
						return true;
					}

					return false;
				}

			});
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public Message createMessage(String msgId) {
		return new Message("(tell :ontology :content " + msgId + ")");
	}

	public void sendStart() {
		Iterator<String> it = this.agents.keySet().iterator();
		while (it.hasNext()) {
			Message m = createMessage("control");
			m.put("command", "start");
			m.put("receiver", it.next());
			m.put("AgentsNameID", agents);
			try {
				mbox.sendMsg(m);
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}

	}

	public void run() {
		System.out.println("Master is running");
		Config c = new Config();
		c.set("facilitator.hos", prop.getProperty("MasterHostName"));
		c.set("society.name", prop.getProperty("SocietyName", ""));
		try {
			if (enterSoc("Master", c)) {
				initAg(null);
				System.out.println(this.getName() + " in loop wainting for messages ");
				long ResolDuration = Integer.parseInt(prop.getProperty("ResolutionTime")) + Integer.parseInt(prop.getProperty("TimeOut"));
				long startTime = System.currentTimeMillis() / 1000;

				while (running & ((System.currentTimeMillis() / 1000) - startTime < ResolDuration)) {
					Message m = null;
					try {
						m = mbox.polling();

					} catch (Exception e) {
						m = null;
					}
					if (m == null) {
						continue;
					}
				}
			}
		} catch (Exception e) {
			System.err.println("Error=" + e);
		}
	}

}
