package simple;

import java.util.Collection;
import java.util.Iterator;
import kernel.Constraint;
import kernel.VarIdentifier;
import kernel.constraints.integer.SUnaryIntConstraint;
import kernel.constraints.integer.explain.FCBinConstraint;
import kernel.constraints.integer.explain.SEAC2001BinConstraint;
import kernel.constraints.integer.explain.SEDistanceXYC;
import kernel.constraints.integer.explain.SEEqualXYC;
import kernel.constraints.integer.explain.SEGreaterThanEqualXYC;
import kernel.constraints.integer.explain.SEGreaterThanXYC;
import kernel.constraints.integer.explain.SELessThanEqualXYC;
import kernel.constraints.integer.explain.SELessThanXYC;
import kernel.constraints.integer.explain.SENoGoodConstraint;
import kernel.constraints.integer.explain.SENotEqualXYC;
import kernel.constraints.integer.explain.SEUnaryIntConstraint;
import kernel.constraints.integer.soft.SoftExtensionBinConstraint;
import kernel.constraints.integer.soft.SoftTuplesConstraint;
import kernel.constraints.integer.soft.SoftUnaryIntConstraint;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVar;
import kernel.vars.integer.IntVariable;
import kernel.vars.integer.explain.ExplainedIntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import kernel.vars.integer.explain.ExplainedIntVariable;
import kernel.vars.integer.explain.Explanation;
import simple.dcsp.Nogood;
import simple.dcsp.SimpleConstraint;
import simulation.metric.Metric;
import simulation.parser.PVariable;
import util.Instantiation;
import util.IntArray;
import util.VarAssignment;

public abstract class SingleExplainedVarAgent extends SimpleAgent {

	protected int nbInferedNogood = 0;

	protected int ac = Constraint.AC2001;

	// ======================================================================

	public SingleExplainedVarAgent(int agId, String name, String description) {
		super(agId, name, description);
		this.auxiliaryStatistics.put("nbAssignments", new Metric("nbAssignments", 0));
		this.auxiliaryStatistics.put("nbInferedNogood", new Metric("nbInferedNogood", 0));
	}
	
	public SingleExplainedVarAgent(PVariable variable) {
		super(variable);
		this.auxiliaryStatistics.put("nbAssignments", new Metric("nbAssignments", 0));
		this.auxiliaryStatistics.put("nbInferedNogood", new Metric("nbInferedNogood", 0));
	}

	// ====================================================================

	/**
	 * Constructs the single controlled internal variable
	 */
	public IntVariable makeInternalVar(Integer[] values) {
		this.var = new ExplainedIntVariable(this, values);
		//this.var.setVarIdentifier(new VarIdentifier(this.agentId));
		this.addVariable(var.getVarIdentifier(), var);
		return var;
	}

	/**
	 * Constructs the single controlled internal variable
	 */
	public IntVariable makeInternalVar(int min, int max) {
		this.var = new ExplainedIntVariable(this, min, max);
		//this.var.setVarIdentifier(new VarIdentifier(this.agentId));
		this.addVariable(var.getVarIdentifier(), var);
		return this.var;
	}
	
	@Override
	public IntVariable makeInternalVar(String name, int id, Integer[] values) {
		this.var = new ExplainedIntVariable(this, name, new VarIdentifier(this.agentId, id), values);
		this.addVariable(var.getVarIdentifier(), var);
		return this.var;
	}
	
	/**
	 * Constructs the single controlled internal variable
	 */
	@Override
	public IntVariable makeInternalVar(PVariable variable) {
		return this.makeInternalVar(variable.getName(), variable.getVarIdentifier(), variable.getDomain().getValues());
	}


	/**
	 * Constructs the single controlled internal variable
	 */
	@Override
	public IntVariable makeInternalVar(String name, VarIdentifier varId, Integer[] values) {
		this.var = new ExplainedIntVariable(this, name, varId, values);
		this.addVariable(var.getVarIdentifier(), var);
		return var;
	}
	
	// ======================================================================

	public IntExternalVar makeExternalVar(IntVar v) {
		if (!this.containsVariable(v.getVarIdentifier())) {
			IntExternalVar exV = new ExplainedIntExternalVar(this, v, v.getName());
			this.addVariable(exV.getVarIdentifier(), exV);
			return exV;
		}
		return (IntExternalVar) this.getVariable(v.getVarIdentifier());
	}

	// ======================================================================

	public IntExternalVar makeExternalVar(VarAssignment v) {
		VarIdentifier varId = v.getVarIdentifier();
		if (!(this.containsVariable(varId))) {
			IntExternalVar exV = new ExplainedIntExternalVar(this, v);
			this.addVariable(varId, exV);
			return exV;
		}
		return (IntExternalVar) this.getVariable(v.getVarIdentifier());
	}

	// ======================================================================

	public IntExternalVar makeExternalVar(VarIdentifier vId) {
		return this.makeExternalVar(new VarAssignment(vId, Integer.MIN_VALUE));
	}

	// ======================================================================

	public IntExternalVar makeExternalVar(VarIdentifier vId, int min, int max) {
		if (!this.containsVariable(vId)) {
			IntExternalVar exV = new ExplainedIntExternalVar(this, vId, min, max);
			this.addVariable(exV.getVarIdentifier(), exV);
			return exV;
		}
		return (IntExternalVar) this.getVariable(vId);
	}

	@Override
	public IntExternalVar makeExternalVar(String name, VarIdentifier varId, Integer[] values) {
		if (!this.containsVariable(varId)) {
			IntExternalVar exV = new ExplainedIntExternalVar(this, varId, name, values);
			this.addVariable(exV.getVarIdentifier(), exV);
			return exV;
		} else {
			System.out.println(""+this.variables);
			return (IntExternalVar) this.getVariable(varId);
		}
	}

	@Override
	public IntExternalVar makeExternalVar(String name, int agId, int id, Integer[] values) {
		return makeExternalVar(name, new VarIdentifier(agId, id), values);
	}

	@Override
	public IntExternalVar makeExternalVar(PVariable variable) {
		return makeExternalVar(variable.getName(), variable.getVarIdentifier(), variable.getDomain().getValues());
	}
	// ======================================================================

	/**
	 * Make an unary constraint that maps each value to a cost
	 */
	@Override
	public Constraint makeSoftRelation(String name, PVariable variable, Integer[] values, Integer[] weights) {
		//TODO explained soft unary constraint
		this.makeInternalVar(variable);
		Constraint constraint = new SoftUnaryIntConstraint(name, this.var, values, weights);
		this.addConstraint(constraint);
		return constraint;
	}

	// ======================================================================

	/**
	 * Makes unary constraint X = cste
	 * 
	 * @param cste
	 *            A constant
	 */
	public Constraint eqC(int cste) {
		Constraint exC = new SEUnaryIntConstraint(Constraint.eq, var, cste);
		this.addConstraint(exC);
		return exC;
	}

	// ======================================================================
	/**
	 * Makes unary constraint X != cste
	 * 
	 * @param cste
	 *            A constant
	 */
	public Constraint neqC(int cste) {
		Constraint exC = new SEUnaryIntConstraint(Constraint.neq, var, cste);
		this.addConstraint(exC);
		return exC;
	}

	// ======================================================================
	/**
	 * Makes unary constraint X <= cste
	 * 
	 * @param cste
	 *            A constant
	 */
	public Constraint leqC(int cste) {
		Constraint exC = new SEUnaryIntConstraint(Constraint.lt, var, cste + 1);
		this.addConstraint(exC);
		return exC;
	}

	// ======================================================================
	/**
	 * Makes unary constraint X >= cste
	 * 
	 * @param cste
	 *            A constant
	 */

	public Constraint geqC(int cste) {
		Constraint exC = new SUnaryIntConstraint(Constraint.gt, var, (cste + 1));
		this.addConstraint(exC);
		return exC;
	}

	// ========================================================================================================
	/**
	 * Make the constraint X = Y + C when X is the internal variable, Y is the given external variable <code>y</code> , and C is a positive constant.
	 */
	public Constraint eqYplusC(IntExternalVar y, int c) {
		SimpleConstraint exC = new SEEqualXYC((ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, c);
		this.addConstraint(exC);
		return exC;
	}

	// ========================================================================================================
	/**
	 * Make the constraint X != Y + C when X is the internal variable, Y is the given external variable <code>y</code> , and C is a positive constant.
	 */
	public Constraint neqYplusC(IntExternalVar y, int c) {
		SimpleConstraint exC = new SENotEqualXYC(
				(ExplainedIntVariable) this.var, 
				(ExplainedIntExternalVar) y, 
				c);
		this.addConstraint(exC);
		return exC;
	}

	// ========================================================================================================
	/**
	 * Make the constraint X >= Y + C when X is the internal variable, Y is the given external variable <code>y</code>, and C is a constant
	 */
	public Constraint geqYplusC(IntExternalVar y, int c) {
		SimpleConstraint exC = new SEGreaterThanEqualXYC((ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, c);
		this.addConstraint(exC);
		return exC;
	}

	// ========================================================================================================
	/**
	 * Make the constraint X > Y + C when X is the internal variable, Y is the given external variable <code>y</code>, and C is a positive constant.
	 */
	public Constraint gtYplusC(IntExternalVar y, int c) {
		SimpleConstraint exC = new SEGreaterThanXYC((ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, c);
		this.addConstraint(exC);
		return exC;
	}

	// ========================================================================================================
	/**
	 * Make the constraint X <= Y + C when X is the internal variable, Y is the given external variable <code>y</code> , and C is a positive constant.
	 */
	public Constraint leqYplusC(IntExternalVar y, int c) {
		SimpleConstraint exC = new SELessThanEqualXYC((ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, c);
		this.addConstraint(exC);
		return exC;
	}

	// ========================================================================================================
	/**
	 * Make the constraint X < Y + C when X is the internal variable, Y is the given external variable <code>y</code> , and C is a positive constant.
	 */
	public Constraint ltYplusC(IntExternalVar y, int c) {
		SimpleConstraint exC = new SELessThanXYC((ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, c);
		this.addConstraint(exC);
		return exC;
	}
	
	public Constraint infeasiblePair(IntExternalVar y, int[][] tuples, boolean transpose) {
		Constraint exC = null;
		boolean feas = false;
		String name = Constraint.infeasP;
		if (ac == Constraint.FC)
			exC = new FCBinConstraint("FC"+name, feas, (ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, tuples, transpose);
		else if (ac == Constraint.AC2001) {
			exC = new SEAC2001BinConstraint("AC2001"+name, feas, (ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, tuples, transpose);
		}
		this.addConstraint(exC);
		return exC;
	}

	// ========================================================================================================

	/**
	 * Make an extension binary constraints with allowed tuples.
	 */
	public Constraint feasiblePair(IntExternalVar y, int[][] tuples, boolean transpose) {
		Constraint exC = null;
		boolean feas = true;
		String name = Constraint.feasP;
		if (ac == Constraint.FC)
			exC = new FCBinConstraint("FC"+name, feas, (ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, tuples, transpose);
		else if (ac == Constraint.AC2001) {
			exC = new SEAC2001BinConstraint("AC2001"+name, feas, (ExplainedIntVariable) this.var, (ExplainedIntExternalVar) y, tuples, transpose);
		} else
			throw new Error("consistancy method not yet implemented");
		this.addConstraint(exC);
		return exC;
	}

	// ========================================================================================================

	/**
	 * Make the constraint |X - Y| Operator C when X is the internal variable, Y is the given external variable <code>y</code>, and C is a positive
	 * constant.
	 */
	public Constraint distanceXYC(IntExternalVar y, int c, int operator) {
		if (operator == Constraint.GEQ) {
			c--;
			operator = Constraint.GT;
		}
		if (operator == Constraint.LEQ) {
			operator = Constraint.LT;
			c++;
		}
		SimpleConstraint exC = new SEDistanceXYC(this.getMyVar(), (ExplainedIntExternalVar) y, c, operator);
		this.addConstraint(exC);
		return exC;
	}

	// ==========================================================================================================
	// ///////////////////////////////// Nogood Manager /////////////////////////////////////////////////////////
	// ==========================================================================================================

	public ExplainedIntVariable getMyVar() {
		return (ExplainedIntVariable) this.var;
	}

	// ==========================================================================================================

	public Nogood solveMyNogoods() {
//		nbInferedNogood++;
		return this.solveNogoods((ExplainedIntVariable) this.var);
	}

	// ==========================================================================================================

	public Nogood solveNogoods(ExplainedIntVar v) {
		nbInferedNogood++;
		Instantiation nogood = new Instantiation(this.agentOrder);
		Iterator<Integer> it = v.getDomain().originalDomainIterator();
		while (it.hasNext()) {
			int value = it.next();
			Instantiation ng = this.buildBetterNogood((ExplainedIntVar) v, value);
			if (ng != null) {
				nogood.priorityMerge(ng);
			}
		}
		return new Nogood(nogood);
	}

	// ==========================================================================================================

	public Instantiation solveMyNogoodStore() {
		nbInferedNogood++;
		Instantiation nogood = new Instantiation(this.agentOrder);
		Iterator<Integer> it = this.var.getDomain().originalDomainIterator();
		while (it.hasNext()) {
			int value = it.next();
			Instantiation ng = this.buildBetterNogood((ExplainedIntVar) var, value);
			if (ng != null) {
				nogood.priorityMerge(ng);
			}
		}
		return nogood;
	}

	// ==========================================================================================================

	public Nogood makeBetterNogood(ExplainedIntVar var, int val) {
		return this.buildBetterNogood(var, val, var.getDomain().getExplanation(val));
	}

	// ==========================================================================================================
	//TODO if values are removed before
	public int buildNogoodConstraint(ExplainedIntVar v, IntArray values, Collection<Explanation> expl) {
		Constraint c = this.makeNoGoodConstraint(v, buildBetterNogood(v, values.get(0), expl));
		SENoGoodConstraint cn = (SENoGoodConstraint) c;
		for (int i = 1 ; i < values.size() ; i++) {
			cn.addRhsValue(values.get(i));
		}
		return c.getIndex();
	}

	// ==========================================================================================================

	public int buildNogoodConstraint(ExplainedIntVar v, int val, Collection<Explanation> expl) {
		Nogood nogood = buildBetterNogood(v, val, expl);
		Constraint c = this.makeNoGoodConstraint(v, nogood);
		return c.getIndex();
	}

	// ==========================================================================================================

	public Instantiation getBetterEleminator(ExplainedIntVar v, int val) {
//		nbInferedNogood++;
		Instantiation eliminator = new Instantiation(this.agentOrder);
		Collection<Explanation> expl = v.getDomain().getExplanation(val);
		if (expl != null)
			for (Explanation exp : expl)
				eliminator.priorityMerge(this.buildBetterNogood(v, exp));
		return eliminator;
	}

	public Nogood buildBetterNogood(ExplainedIntVar v, int value, Collection<Explanation> expl) {
//		nbInferedNogood++;
		VarAssignment varAss = v.getVarAssignment();
		varAss.setValue(value);
		Nogood nogood = new Nogood(new Instantiation(this.agentOrder), varAss);
		if (expl != null)
			for (Explanation exp : expl)
				nogood.getLhs().priorityMerge(this.buildBetterNogood(v, exp));
		return nogood;
	}

	// ==========================================================================================================

	private Constraint makeNoGoodConstraint(ExplainedIntVar var, Nogood nogood) {
		Constraint c = this.getConstraintNogood(nogood);
		if (c == null) {
			c = new SENoGoodConstraint(var, nogood);
			this.constraintHandler.dynamicAdd(c);
			((SENoGoodConstraint) c).setActive();
			// this.log("From makeNoGoodConstraint  of "+ nogood + " "+ c, 1);
		} else {
			this.constraintHandler.postConstAwake(c, true);
			// this.log("From makeNoGoodConstraint  of "+ nogood + " "+ c, 1);

		}
		return c;
	}

	private Constraint getConstraintNogood(Nogood nogood) {
		nbInferedNogood++;
		Iterator<Constraint> it = this.constraintHandler.getConstraintsIterator();
		Constraint c = null;
		while (it.hasNext()) {
			c = it.next();
			if (c.getConstraintType().equals(Constraint.NOGOOD)) {
				SENoGoodConstraint ngc = (SENoGoodConstraint) c;
				VarIdentifier varId = nogood.getRhs().getVarIdentifier();
				if (ngc.hasRhsId(varId) && ngc.hasLhsEquals(nogood.getLhs())) {
					for (int val : nogood.getRhsValues())
						((SENoGoodConstraint) c).addRhsValue(val);
					// this.log("From getConstraintNogood  of "+ nogood + " "+ c, 1);
					return c;
				}
			}
			c = null;
		}
		// this.log("From getConstraintNogood  of "+ nogood + " "+ c, 1);
		return c;
	}

	// ==========================================================================================================
	public Constraint makeNoGoodConstraint(Nogood nogood) {
		Constraint c = this.getConstraintNogood(nogood);
		if (c == null) {
			c = new SENoGoodConstraint(this, nogood);
			this.constraintHandler.dynamicAdd(c);
			((SENoGoodConstraint) c).setActive();
		} else {
			this.constraintHandler.postConstAwake(c, true);
		}
		// this.log("Save Nogood "+nogood + "  index "+c.getIndex() ,1);
		this.constraintHandler.propagate();
		return c;
	}

//	// ==========================================================================================================
//	public Constraint storeBetterNoGood(Nogood nogood) {
//		int value = nogood.getRhsValue();
//		Constraint c = null;
//		if(this.var.getDomain().contains(value)){
//			c = this.makeNoGoodConstraint(nogood);
//		} else {
//			Instantiation ng = this.buildBetterNogood((ExplainedIntVar) var, value);
//			if(ng.compareHPLV(nogood.getLhs()) < 0){
//				c = this.makeNoGoodConstraint((ExplainedIntVar) this.var, nogood);
//			}
//		}
//		return c;
//	}

	// ==========================================================================================================
	// /////////////////////// Private Methods
	// //////////////////////////////////////////////////////////////////
	// ==========================================================================================================
	protected Instantiation buildBetterNogood(ExplainedIntVar var, int val) {
		Collection<Explanation> exp = var.getDomain().getExplanation(val);
		if (exp == null || exp.isEmpty()) { return new Instantiation(this.agentOrder); }
		return this.buildBetterNogood(var, exp.iterator().next());
	}

	// ==========================================================================================================

	private Instantiation buildBetterNogood(ExplainedIntVar v, Explanation explanation) {
		Instantiation betterNg = new Instantiation(this.agentOrder);
		if (explanation == null || explanation.isEmpty()) { return betterNg; }
		for (int i = explanation.getBitSet().nextSetBit(0) ; i >= 0 ; i = explanation.getBitSet().nextSetBit(i + 1)) {
			Constraint c = this.constraintHandler.getConstraint(i);
			Instantiation ng = c.toInstantiation();
			if (betterNg.isEmpty())
				betterNg = ng;
			else if (ng.compareHPLV(betterNg) > 0) {
				betterNg = ng;
			}
		}
		return betterNg;
	}

	@Override
	public Collection<Metric> getAuxiliaryStatistics() {
		this.auxiliaryStatistics.put("nbAssignments", new Metric("nbAssignments", this.var.getTimeCounter()));
		this.auxiliaryStatistics.put("nbInferedNogood", new Metric("nbInferedNogood", this.nbInferedNogood));
		return this.auxiliaryStatistics.values();
	}

	/**
	 * 
	 */
	@Override
	public Constraint makeConstraintTable(String name, boolean feasible, PVariable v0, PVariable v1, int[][] tuples, boolean transpose) {
		Integer[] values = v1.getDomain().getValues();
		ExplainedIntExternalVar exVar = (ExplainedIntExternalVar) makeExternalVar(v1.getName(), v1.getAgentId(), v1.getId(), values);
		ExplainedIntVariable myVariable = (ExplainedIntVariable) this.var;
		Constraint exC = null;
		if (ac == Constraint.FC)
			exC = new FCBinConstraint("FC"+name, feasible, myVariable, exVar, tuples, transpose);
		else if (ac == Constraint.AC2001) {
			exC = new SEAC2001BinConstraint("AC2001"+name, feasible, myVariable, exVar, tuples, transpose);
		}
		this.addConstraint(exC);
		return exC;
	}
	
	@Override
	public Constraint makeSoftBinRelation(String name, PVariable v0, PVariable v1, Integer[] weights, boolean transpose){
		ExplainedIntExternalVar exVar = (ExplainedIntExternalVar) makeExternalVar(v1);
		ExplainedIntVariable myVariable = (ExplainedIntVariable) this.var;
		Constraint exC = new SoftExtensionBinConstraint(name, myVariable, exVar, weights, transpose);
		this.addConstraint(exC);
		return exC;
	}
	
	@Override
	public Constraint makeWeightedTuplesRelation(String name, PVariable v0, PVariable v1, int[][] tuples, Integer[] weights, int defaultCost, boolean transpose){
		IntExternalVar exVar = makeExternalVar(v1);
		Constraint exC = new SoftTuplesConstraint(name, this.var, exVar, tuples, weights, defaultCost, transpose);
		this.addConstraint(exC);
		return exC;
	}
	
}
