/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/
package simple.dcop;

import static kernel.Constraint.INFINITY;
import java.math.BigInteger;
import java.rmi.UnexpectedException;
import java.util.Iterator;
import kernel.AbstractAgent;
import kernel.Constraint;
import kernel.ConstraintsHandler;

public class DCOPConstraintHandler extends ConstraintsHandler {

	public DCOPConstraintHandler(AbstractAgent agent) {
		super(agent);
	}

	/** 
	 * @param value
	 * @return
	 */
	public Integer localCost(int value) {
		Iterator<Constraint> it = this.getConstraintsIterator();
		BigInteger sum = BigInteger.valueOf(0);
		while (it.hasNext()) {
			Constraint constraint = it.next();
			if (constraint.isPosted()) {
				if(constraint instanceof SimpleSoftConstraint) {
					SimpleSoftConstraint exC =  (SimpleSoftConstraint) constraint;
					BigInteger cost = BigInteger.valueOf(exC.evaluate(value));
					if (cost.compareTo(BigInteger.valueOf(INFINITY)) >= 0){
						return INFINITY;
					}
					sum = sum.add(cost);
					if (sum.compareTo(BigInteger.valueOf(INFINITY)) >= 0){
						return INFINITY;
					}
				} else {
					new UnexpectedException("DCOP can not deal with hard constraint\n"+constraint);
				}
//				else if (constraint.isCompletlyInstantiated()) { 
//					IntVariable myVar = ((SimpleAgent) this.myAgent).var;
//					boolean isConsistent = constraint.isConsistent(myVar, value);
//					if(!isConsistent) {
//						System.out.println(value+" is not consistent because of "+constraint.toString());
//						return INFINITY;
//					}
//				}
			}
		}
		return sum.intValue();
	}

	/**
	 * This method add the cost <i>c1</i> to <i>c2</i>. 
	 * 
	 * @param c1 : the first cost
	 * @param c2 : the second cost
	 * @return Integer <i>c1 + c2</i>
	 */
	public static Integer sumCosts(Integer c1, Integer c2) {
		BigInteger sum = BigInteger.valueOf(c1);
		if (sum.compareTo(BigInteger.valueOf(INFINITY)) >= 0){
			return INFINITY;
		}
		if (c2.compareTo(INFINITY) >= 0) {
			return INFINITY;
		}
		sum = sum.add(BigInteger.valueOf(c2));
		// make sure we don't overflow when dealing with hard constraints
		if (sum.compareTo(BigInteger.valueOf(INFINITY)) >= 0){
			return INFINITY;
		}
		return sum.intValue();
	}


	/**
	 * This method subtracts the cost <i>c2</i> from <i>c1</i>. 
	 * 
	 * @param c1 : the first cost
	 * @param c2 : the second cost
	 * @return Integer <i>c1 - c2</i>
	 */
	public static Integer subCosts(Integer c1, Integer c2) {
		// infinity - infinity = MAX_COST
		if (c1.equals(INFINITY) && c2.equals(INFINITY)) return new Integer(0);
		// infinity - N = infinity, and N - infinity = infinity.
		if (c1.equals(INFINITY) || c2.equals(INFINITY)) return INFINITY;
		else return c1 - c2;
	}
	
//	
//	/**
//	 * This method subtracts the cost <i>c2</i> from <i>c1</i>. 
//	 * 
//	 * @param c1 : the first cost
//	 * @param c2 : the second cost
//	 * @return Integer <i>c1 - c2</i>
//	 */
//	public static Integer subCosts(Integer c1, Integer c2) {
//		BigInteger sub = BigInteger.valueOf(c1);
//		if (sub.compareTo(BigInteger.valueOf(INFINITY)) >= 0 && c2.compareTo(INFINITY) < 0){
//			return INFINITY;
//		}
//		// infinity - infinity = MAX_COST
//		if (c1.equals(INFINITY) && c2.equals(INFINITY)) return new Integer(0);
//		// infinity - N = infinity, and N - infinity = infinity.
//		if (c1.equals(INFINITY) || c2.equals(INFINITY)) return INFINITY;
//		else return c1 - c2;
//	}

}
