package simple.dcop;

import kernel.Constraint;

/**
 * An interface models a soft constraint in simple mode (i.e. when single variable per agent is used).
 * 
 * @version dischoco 2.5.3<br/>
 * simple.SimpleSoftConstraint.java
 * @author Mohamed Wahbi
 */
public interface SimpleSoftConstraint extends Constraint {

	static final Integer MAX_COST = Constraint.INFINITY;
	static final Integer MIN_COST = Integer.MIN_VALUE;

	/**
	 * This method returns the evaluation of the given value regarding the constraint 
	 * @return Integer 
	 */
	public Integer evaluate(int value);

	/**
	 * This method returns the cost of the couple (<tt>value_x</tt>, <tt>value_y</tt>). 
	 * 
	 * @param value_x : the value of the first variable.
	 * @param value_y : the value of the second variable.
	 * @return true if (value_x, value_y) is a consistent couple.
	 */
	public Integer getCost(int xValue, int yValue);
	
	/**
	 * This method returns the minimal cost of the given value, i.e.,  <tt>value</tt>. 
	 * 
	 * @param value : the value of the first variable.
	 * @return Integer : the minimal cost of the given value.
	 */
	public Integer getMinCost(int value);
	
	/**
	 * This method returns the maximal cost of the given value, i.e.,  <tt>value</tt>. 
	 * 
	 * @param value : the value of the first variable.
	 * @return Integer : the maximal cost of the given value.
	 */
	public Integer getMaxCost(int value);
}
