/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/
package simple.dcop.adopt;

import java.io.Serializable;
import kernel.Constraint;
import util.Instantiation;

/** 
 * Class to hold cost information for a given pair of value/child (value, child)
 * 
 * @version dischoco 2.5.3<br/>
 * simple.dcop.adopt.Cost.java
 * @author Mohamed Wahbi
 */
public class Cost implements Serializable {

	private static final long serialVersionUID	= -8633194633703405643L;
	/**
	 * ub(d,x_l): upper bound
	 */
	private Integer ub;
	/** 
	 * lb(d,x_l): lower bound 
	 */
	private Integer lb;
	/** context (d,x_l): context relevant to lb(d,x_l), ub(d,x_l) 
	 */
	private Instantiation context;
	/** 
	 * threshold(d,x_l): threshold I can send to child
	 */
	private Integer th;

	/**
	 *  class constructor
	 */
	public Cost() {
		this.lb = new Integer(0);
		this.ub = Constraint.INFINITY;
		this.context = new Instantiation();
		this.th = new Integer(0);
	}
	/**
	 * The constructor
	 * @param owner the agent owner of the Cost
	 * @param context 
	 * @param lb 
	 * @param ub 
	 */
	public Cost(Instantiation context, Integer lb, Integer ub) {
		this.lb = lb;
		this.ub = ub;
		this.context = context;
		this.th = new Integer(0);
	}
	/**
	 * 
	 */
	public void clear() {
		this.lb = new Integer(0);
		this.ub = Constraint.INFINITY;
		this.context = new Instantiation();
		this.th = new Integer(0); 
	}
	
	public boolean equals(Object obj){
		if(obj instanceof Cost){
			Cost cost = (Cost) obj;
			return ((this.lb.compareTo(cost.lb)==0) && (this.ub.compareTo(cost.ub)==0) 
					&& this.context.equals(cost.context));
		}
		return false;
	}
	
//	public boolean equals(Object obj){
//		if(obj instanceof Cost){
//			Cost cost = (Cost) obj;
//			return ((this.lb.equals(cost.lb)) && (this.ub.equals(cost.ub)) && 
//					this.threshold.equals(cost.threshold) && this.context.equals(cost.context));
//		}
//		return false;
//	}
	
	/**
	 * @return the ub
	 */
	public Integer getUb() {
		return this.ub;
	}
	
	/**
	 * @param ub the ub to set
	 */
	public void setUb(Integer ub) {
		this.ub = ub;
	}
	
	/**
	 * @return the lb
	 */
	public Integer getLb() {
		return this.lb;
	}
	
	/**
	 * @param lb the lb to set
	 */
	public void setLb(Integer lb) {
		this.lb = lb;
	}
	
	/**
	 * @return the context
	 */
	public Instantiation getContext() {
		return this.context.clone();
	}
	
	/**
	 * @param context the context to set
	 */
	public void setContext(Instantiation context) {
		this.context = context;
	}
	
	/**
	 * @return the threshold
	 */
	public Integer getThreshold() {
		return this.th;
	}
	
	/**
	 * @param threshold the threshold to set
	 */
	public void setThreshold(Integer threshold) {
		this.th = threshold;
	}
	

	public String toString() {
		StringBuffer sb = new StringBuffer("[");
		sb.append("LB="+this.lb);
		sb.append(", UB=" + this.ub);
		sb.append(", TH=" + this.th);
		sb.append(", Context="+ this.context.toString());
		sb.append("]");
		return sb.toString();
	}

	/**
	 * @param strcost
	 * @return
	 */
//	@SuppressWarnings("unchecked")
//	public synchronized static Cost fromString(AbstractAgent owner, String strcost) {
//		Cost c = new Cost(owner);
//		String[] str = strcost.split("@");
//		c.lb = Integer.parseInt(str[0]);
//		c.ub = Integer.parseInt(str[1]);
//		c.threshold = Integer.parseInt(str[2]);
//		if (str.length == 4)
//			c.context = Instantiation.fromString(str[3],owner);
//		else 
//			c.context = new Instantiation(owner.agentOrder);
//
//		return c;
//	}
	 
}