/*
 * DisChoco: Distributed Constraint Reasoning Solver
 * Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi
 * DisChoco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * DisChoco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * To contact the authors: <http://dischoco.sourceforge.net/>
 */
package simple.dcop.adopt;

import java.util.ArrayList;
import java.util.Collection;
import dischoco.Protocols;
import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.order.TreeAO;
import simulation.parser.PVariable;
import util.Commands;

public class MasterAdopt extends AbstractMaster {
	
	public static final int originalAdopt = 0; // [Modi et al. 2005]
	public static final int improvedAdopt = 1; // [Gutierrez and Meseguer, 2010]
	public static final int AdoptPlus     = 2; // [Gutierrez and Meseguer, 2010]
	private int version = originalAdopt;
	
	public MasterAdopt(String adopt) {
		super();
		if(adopt.equals(Protocols.AdoptPlus)) {
			version = AdoptPlus;
		} else if(adopt.equals(Protocols.Adopt)) {
			version = improvedAdopt;
		} else if(adopt.equals(Protocols.OriginalAdopt)) {
			version = originalAdopt;
		}
		this.setCentralizedAO(new TreeAO(TreeAO.MaxDeg));
		((TreeAO) this.centralizedAO).orderSwitch = TreeAO.DFSCHAIN;
	}

	@Override
	public AbstractAgent makeAgent(int agentId, String agentName, String description) {
		switch(version) {
			case AdoptPlus : 
				return new AdoptPlusAgent(agentId, agentName, description);
			case improvedAdopt : 
				return new AdoptAgent(agentId, agentName, description);
			case originalAdopt : 
				return new Adopt_ModiAgent(agentId, agentName, description);
			default : return null;
		}
	}

	/* (non-Javadoc)
	 * @see kernel.AbstractMaster#makeAgent(simulation.parser.PVariable)
	 */
	@Override
	public AbstractAgent makeAgent(PVariable variable) {
		switch(version) {
			case AdoptPlus : 
				return new AdoptPlusAgent(variable);
			case improvedAdopt : 
				return new AdoptAgent(variable);
			case originalAdopt : 
				return new Adopt_ModiAgent(variable);
			default : return null;
		}
	}

	@Override
	public Collection<String> getTypesOfMessages() {
		Collection<String> ans = new ArrayList<String>();
		ans.add(Commands.COST);
		ans.add(Commands.VALUE);
		if(version == originalAdopt) {
			ans.add(Commands.THRESHOLD);
		}
		ans.add(Commands.TERMINATION_DETECTION);
		return ans;
	}
}
