/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/
package simple.dcop.afb;

import java.util.ArrayList;
import java.util.Collection;
import kernel.AbstractAgent;
import kernel.AbstractMaster;
import simulation.parser.PVariable;
import dischoco.Protocols;

public class AFBMaster extends AbstractMaster {

	private String protocol;

	public AFBMaster(String protocol) {
		this.protocol = protocol;
	}

	@Override
	public Collection<String> getTypesOfMessages() {
		Collection<String> ans = new ArrayList<String>();
		ans.add(AFBAgent.OK);
		ans.add(AFBAgent.BACK);
		ans.add(AFBAgent.FB);
		ans.add(AFBAgent.LB);
		ans.add(AFBAgent.SOLUTION);
		return ans;
	}

	@Override
	public AbstractAgent makeAgent(int agId, String name, String description) {
		if (this.protocol.equals(Protocols.AFB)) {
			return new AFBAgent(agId, description, description);
		} else if (this.protocol.equals(Protocols.AFB_minC)) {
			return new AFB_minCAgent(agId, description, description);
		} else if (this.protocol.equals(Protocols.AFB_BJ)) {
			return new AFB_BJAgent(agId, description, description);
		}
		return null;
	}

	@Override
	public AbstractAgent makeAgent(PVariable variable) {
		if (this.protocol.equals(Protocols.AFB)) {
			return new AFBAgent(variable);
		} else if (this.protocol.equals(Protocols.AFB_minC)) {
			return new AFB_minCAgent(variable);
		} else if (this.protocol.equals(Protocols.AFB_BJ)) {
			return new AFB_BJAgent(variable);
		} 
		return null;
	}

}
