/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A SimplePARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
 */
package simple.dcop.afb;

import kernel.StopException;
import simple.dcop.DCOPConstraintHandler;
import simulation.parser.PVariable;

public class AFB_minCAgent extends AFBAgent {

	public AFB_minCAgent(int agId, String name, String description) {
		super(agId, name, description);
	}

	public AFB_minCAgent(PVariable variable) {
		super(variable);
	}

	protected void assign() throws StopException {
		clearCollectedLowerBounds();
		int lastIndex = -1;
		if (this.var.isInstantiated()) {
			lastIndex = currentIndexOf(var.getValue());
			unassignCurrentVariable();
		}
		int index = lastIndex + 1;
		boolean found = false;
		while (!found && index < currentDomain.size()) {
			int myValue = this.currentDomain.get(index);
			if (evalute(myValue).compareTo(UB) < 0) {
				assignCurrentVariable(myValue);
				found = true;
			}
			index++;
		}
		if (!found) {
			backtrack();
		}
	}

	protected void reorderDomain() {
		int length = this.currentDomain.size();
		for(int i = 0 ; i < length - 1 ; i++) {
			int value_i = this.currentDomain.get(i);
			for(int j = i + 1 ; j < length ; j++) {
				int value_j = this.currentDomain.get(j);
				if (this.LC(value_j).compareTo(this.LC(value_i)) < 0) {
					this.currentDomain.swapByIndexes(i, j);
					value_i = value_j;
				}
			}
		}
	}

	private void cumputeLocalCosts() {
		for(int i = 0 ; i < this.initialDomain.size() ; i++) {
			int value = this.initialDomain.get(i);
			LC[i] = ((DCOPConstraintHandler) this.constraintHandler).localCost(value);
		}
	}

	protected void processOK(PartialAssignment pa) throws StopException {
		this.partialAssignment = pa;
		unassignCurrentVariable();
		clearLocalCosts();
		updateAgentView(pa.getAssignments());
		clearCollectedLowerBounds();
		cumputeLocalCosts();
		reorderDomain();
		if (pa.getCost().compareTo(this.UB) >= 0) {
			backtrack();
		} else {
			assign();
		}
	}
}
