package simple.dcop.afb;

import java.io.Serializable;

public class TimeStamp implements Serializable {

	private static final long serialVersionUID = 7714730374459221552L;

	private int[] timeStamp;

	public TimeStamp() {
		this.timeStamp = new int[1];
	}

	public TimeStamp(int[] timeStamp) {
		this.timeStamp = timeStamp;
	}

	public TimeStamp(int size) {
		this.timeStamp = new int[size];
	}

	public int size() {
		return this.timeStamp.length;
	}

	public void setTimeStamp(TimeStamp timeStamp) {
		this.timeStamp = timeStamp.getTimeStamp();
	}

	public int compareTimeStamp(TimeStamp timeStamp) {
		return this.compareTimeStamp(timeStamp.timeStamp);
	}

	public int compareTimeStamp(TimeStamp timeStamp, int upto) {
		return this.compareTimeStamp(timeStamp.getTimeStamp(), upto);
	}

	public void update(int i) {
		int oldsize = this.size();
		if (i >= oldsize) {
			this.ensureCapacity(oldsize + 1 + (i - oldsize));
		}
		if (i >= oldsize)
			for (int k = oldsize ; k < i ; k++) {
				this.timeStamp[k] = 0;
			}
		this.timeStamp[i]++;
		for (int k = i + 1 ; k < this.size() ; k++) {
			this.timeStamp[k] = 0;
		}
	}
	
	public void update(int i, int counter) {
		int oldsize = this.size();
		if (i >= oldsize) {
			this.ensureCapacity(oldsize + 1 + (i - oldsize));
		}
		if (i >= oldsize)
			for (int k = oldsize ; k < i ; k++) {
				this.timeStamp[k] = 0;
			}
		this.timeStamp[i] = counter;
		for (int k = i + 1 ; k < this.size() ; k++) {
			this.timeStamp[k] = 0;
		}
	}
	
	private void ensureCapacity(int minCapacity) {
		int[] newtimeStamp = new int[minCapacity];
		System.arraycopy(timeStamp, 0, newtimeStamp, 0, timeStamp.length);
		this.timeStamp = newtimeStamp;
	}

	/**
	 * A time-stamp is an ordered list of counters Ctr1 , Ctr2 ,. . . , Ctrk . When comparing (lexicographically) two time-stamps, the most up to date
	 * is one which is lexicographically greater, that is, the one with greatest value on the first counter on which they differ, if any, otherwise
	 * the longest one.
	 * 
	 * @param timeStamp
	 *            an other time-stamp
	 * @param upto
	 *            Index of last counter to compare with its corresponding in this time-stamp
	 * @return (j+1) : where j is the index of the first newest variable assignment (i.e first counter in <code>timeStamp</code> that is bigger than
	 *         its corresponding counter in this time-stamp -(j+1) where j is the index of the first oldest variable assignment (i.e first counter in
	 *         <code>view</code> that is smaller than its corresponding counter in this time-stamp ; 0 when all counter in <code>timeStamp</code> are
	 *         equals to its corresponding counters in this time-stamp ;
	 */
	public int compareTimeStamp(int[] timeStamp, int upto) {
		int j = 0;
		for ( ; j < this.timeStamp.length && j < timeStamp.length && j <= upto ; j++) {
			if (this.timeStamp[j] > timeStamp[j]) {
				return -(j + 1);
			} else if (this.timeStamp[j] < timeStamp[j]) { 
				return  j + 1; 
			}
		}
		if (j <= upto && this.timeStamp.length < timeStamp.length) {
			return j + 1; 
		}
		return 0;
	}

	/**
	 * @param timeStamp
	 *            : an other time-stamp
	 * @see #compareTimeStamp(Vector<Integer>, int)
	 */
	public int compareTimeStamp(int[] timeStamp) {
		return this.compareTimeStamp(timeStamp, timeStamp.length - 1);
	}

	/**
	 * 
	 * @param timeStamp
	 */
	public void setTimeStamp(int[] timeStamp) {
		this.timeStamp = timeStamp;
	}

	public int[] getTimeStamp() {
		return this.timeStamp;
	}

	public TimeStamp clone() {
		return new TimeStamp(this.timeStamp.clone());
	}

	/**
	 * Returns a string representation of this object
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("[ ");
		for (int i = 0 ; i < timeStamp.length ; i++) {
			
			sb.append(timeStamp[i] + " ");
		}
		sb.append("]");
		return sb.toString();
	}

	public void update(TimeStamp ts) {
		int i = 0;
		for ( ; i < ts.timeStamp.length ; i++) {
			this.timeStamp[i] = ts.timeStamp[i];
		}
		for ( ; i < timeStamp.length ; i++) 
			this.timeStamp[i] = 0;
			
	}

}
