/*
 * DisChoco: Distributed Constraint Reasoning Solver
 * Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi
 * DisChoco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * DisChoco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * To contact the authors: <http://dischoco.sourceforge.net/>
 */
package simple.dcop.bnbadopt;

import java.util.ArrayList;
import java.util.Collection;
import dischoco.Protocols;
import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.order.TreeAO;
import simulation.parser.PVariable;
import util.Commands;

public class MasterBnBAdopt extends AbstractMaster {
	
	public static final int BnBAdopt = 0; // [Yeoh et al. 2008]
	public static final int BnBAdoptPlus = 1; // [Gutierrez and Meseguer, 2010]
	public static final int BnBAdoptPlusDP2 = 2; // [Gutierrez and Meseguer, 2010]
	private int version = BnBAdopt;
	
	public MasterBnBAdopt(String algorithm) {
		super();
		if(algorithm.equals(Protocols.BnBAdoptPlus)) {
			version = BnBAdoptPlus;
		} else if(algorithm.equals(Protocols.BnBAdopt)) {
			version = BnBAdopt;
		} else if(algorithm.equals(Protocols.BnBAdoptPlusDP2)) {
			version = BnBAdoptPlusDP2;
		} 
		this.setCentralizedAO(new TreeAO(TreeAO.MaxDegMinDom));
		((TreeAO) this.centralizedAO).orderSwitch = TreeAO.DFSCHAIN;
	}

	@Override
	public AbstractAgent makeAgent(int agentId, String agentName, String description) {
		switch(version) {
			case BnBAdoptPlus : 
				return new BnBAdoptPlusAgent(agentId, agentName, description);
			case BnBAdoptPlusDP2 : 
				return new BnBAdoptPlusDP2Agent(agentId, agentName, description);
			case BnBAdopt : 
				return new BnBAdoptAgent(agentId, agentName, description);
			default : throw new IllegalArgumentException("version unknown : "+version);
		}
	}


	/* (non-Javadoc)
	 * @see kernel.AbstractMaster#makeAgent(simulation.parser.PVariable)
	 */
	@Override
	public AbstractAgent makeAgent(PVariable variable) {
		switch(version) {
			case BnBAdoptPlus : 
				return new BnBAdoptPlusAgent(variable);
			case BnBAdoptPlusDP2 : 
				return new BnBAdoptPlusDP2Agent(variable);
			case BnBAdopt : 
				return new BnBAdoptAgent(variable);
			default : throw new IllegalArgumentException("version unknown : "+version);
		}
	}
	@Override
	public Collection<String> getTypesOfMessages() {
		Collection<String> ans = new ArrayList<String>();
		ans.add(Commands.COST);
		ans.add(Commands.VALUE);
		ans.add(Commands.TERMINATION_DETECTION);
		if(version == BnBAdoptPlusDP2) {
			ans.add(BnBAdoptPlusDP2Agent.PREPROCESS);
		}
		return ans;
	}
	
}
