package simple.dcsp;

import java.io.Serializable;
import java.util.Comparator;

import kernel.VarIdentifier;
import util.Instantiation;
import util.IntArray;
import util.VarAssignment;

/**
 * A class that models the Nogoods in the simple mode; Nogood : lhs ---> !(rhs)
 */
public class Nogood implements Serializable, Cloneable {

	private static final long serialVersionUID = -3494694602450054170L;
	/**
	 * The left hand side of the nogood
	 */
	protected Instantiation lhs;
	/**
	 * The right hand side of the nogood
	 */
	protected VarIdentifier rhsId = null;

	protected IntArray rhsValues = null;

	/**
	 * Constructs an empty nogood
	 * 
	 * @param comparator
	 *            A comparator for the order of the variables. In general is the agentOrdering, It allows to conserve the order of variable equals to
	 *            the current order
	 */
	public Nogood(Comparator<? super VarAssignment> comparator) {
		if (comparator == null) {
			this.lhs = new Instantiation();
		} else {
			this.lhs = new Instantiation(comparator);
		}
	}

	public Nogood(Instantiation lhs, VarIdentifier varId, IntArray values) {
		this.lhs = lhs;
		this.rhsId = varId;
		// this.rhs = new VarAssignment(varId, values.get(0));
		this.rhsValues = values.clone();
		// this.values.remove(values.get(0));
	}

	/**
	 * Constructs a new nogood with empty lhs
	 * 
	 * @param owner
	 *            the agent ownning this nogood
	 * @param rhs
	 *            the rhs of the nogood
	 */
	public Nogood(VarAssignment rhs) {
		this.lhs = new Instantiation();
		this.setRhs(rhs);
	}

	/**
	 * Constructs a new nogood
	 */
	public Nogood(Instantiation lhs, VarAssignment rhs) {
		this.lhs = lhs;
		this.setRhs(rhs);
	}

	/**
	 * Constructs nogood from Instantiation The last variable is put on the rhs of the nogood. For example if ass = x1=v1, x2=v2, x3=v3 the nogood is
	 * lhs: x1=v1, x2=v2 , rhs: x3=v3
	 * 
	 * @param ass
	 */
	public Nogood(Instantiation ass) {
		if (ass.isEmpty()) {
			this.lhs = ass.clone();
			this.rhsId = null;
		} else {
			this.lhs = ass.clone();
			this.setRhs(this.lhs.remove(lhs.last()));
		}
	}

	/**
	 * checks whether the nogood is empty or not (i.e It checks only rhs if it is empty)
	 */
	public boolean isEmpty() {
		return this.rhsId == null;
	}

	/**
	 * Returns the value of the variable in right hand side of the nogood
	 */
	public int getRhsValue() {
		return this.rhsValues.get(0);
	}

	/**
	 * Returns the value of the variable in right hand side of the nogood
	 */
	public IntArray getRhsValues() {
		return this.rhsValues;
	}

	/**
	 * Set the right hand side of the nogood to the given value
	 */
	public void setRhs(VarAssignment rhs) {
		this.rhsId = rhs.getVarIdentifier();
		this.rhsValues = new IntArray();
		this.rhsValues.add(rhs.getValue());
	}

	/**
	 * Gets the value of the right hand side of the nogood
	 */

	public VarAssignment getRhs() {
		if (this.rhsId == null)
			return null;
		return new VarAssignment(this.rhsId, this.rhsValues.get(0));
	}

	/**
	 * Returns the ID of the agent in the rhs
	 */
	public VarIdentifier getRhsVarIdentifier() {
		return this.rhsId;
	}

//	/**
//	 * Returns the ID of the agent in the rhs
//	 */
//	public int getRhsAgentId() {
//		return this.rhsId.getAgentId();
//	}

	/**
	 *Returns the left hand side of the nogood
	 */
	public Instantiation getLhs() {
		return this.lhs;
	}

	/**
	 * Set the left hand side of the nogood to the given value
	 */
	public void setLhs(Instantiation lhs) {
		this.lhs = lhs;
	}

	/**
	 * Returns a string representing the nogood
	 */

	public String toString() {
		return "{" + this.lhs.toString() + "=>" + this.rhsId + " != " + this.rhsValues + "}";
	}

	public Nogood clone(Comparator<? super VarAssignment> comparator) {
		return new Nogood(this.lhs.clone(comparator), this.rhsId.clone(), this.rhsValues.clone());
	}

	public Nogood clone() {
		return new Nogood(this.lhs.clone(), this.rhsId.clone(), this.rhsValues.clone());
	}

	public Nogood cloneUseingSelfComparator() {
		return new Nogood(this.lhs.cloneUseingSelfComparator(), this.rhsId.clone(), this.rhsValues.clone());
	}
	// /**
	// * Called to build a nogood from its representing string
	// *
	// * @param str
	// * A sting that represent the nogood, it can be received by
	// * message
	// * @param comparator
	// * A comparator of the order of variable. In general is the
	// * agentOrdering, It allows to conserve the order of variable
	// * equals to the current order.
	// */
	// public synchronized static Nogood fromString(String str, Comparator<? super VarAssignment> comparator) {
	// Nogood ng = new Nogood(comparator);
	// str = str.substring(1, str.indexOf("}"));
	// String[] sa = str.split("=>!");
	// ng.lhs = Instantiation.fromString(sa[0], comparator);
	// ng.rhs = VarAssignment.fromString(sa[1]);
	// return ng;
	// }
	//
	// public static Nogood fromString(String strNg) {
	// String str = strNg.substring(1, strNg.indexOf("}"));
	// String[] sa = str.split("=>!");
	// sa[0] += "," + sa[1];
	// return fromString(strNg, new DefaultComparator(sa[0]));
	// }

}
