/************************************************************
 * ABT add permanent link
 *************************************************************/

package simple.dcsp.abtfamily.abt;

import java.util.Iterator;
import kernel.AbstractAgent;
import kernel.Communicator;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.MessageHandler;
import kernel.events.AgentEventListener;
import kernel.events.VariableEvent;
import kernel.vars.integer.IntExternalVar;
import simple.SimpleAgent;
import simple.SingleExplainedVarAgent;
import simple.dcsp.Nogood;
import simple.dcsp.abtfamily.ABT;
import simple.dcsp.abtfamily.IdleEvent;
import simulation.metric.Metric;
import simulation.parser.PVariable;
import util.Commands;
import util.HashList;
import util.Logger;
import util.VarAssignment;
import util.VarEventMask;

public class ABTAgent extends SingleExplainedVarAgent implements ABT {

	public OkMessageHandler okMsgH = null;
	public NogoodsHandler ngdMsgH = null;
	public AddLinkMsgHandler adlMsgH = null;

	/**
	 * @param agentId
	 * @param agName
	 * @param description
	 */
	public ABTAgent(int agentId, String agName, String description) {
		super(agentId, agName, description);
		this.constructCommunicator();
		this.terminateMsgHandler = new ABTTermDetection(this);
		// -------------- Messages Handler ------------------//
		this.okMsgH = new OkMessageHandler(this);
		this.ngdMsgH = new NogoodsHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
		this.initAuxStatistics();
	}

	/**
	 * @param agentId
	 * @param agName
	 * @param description
	 */
	public ABTAgent(PVariable variable) {
		super(variable);
		this.constructCommunicator();
		this.terminateMsgHandler = new ABTTermDetection(this);
		// -------------- Messages Handler ------------------//
		this.okMsgH = new OkMessageHandler(this);
		this.ngdMsgH = new NogoodsHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
		this.initAuxStatistics();
	}

	private void initAuxStatistics() {
		this.auxiliaryStatistics.put(Commands.nbObsoleteNogood, new Metric(Commands.nbObsoleteNogood));
		this.auxiliaryStatistics.put(Commands.nbAssignments, new Metric(Commands.nbAssignments));
		this.auxiliaryStatistics.put(Commands.ObsoleteOk, new Metric(Commands.ObsoleteOk));
		this.auxiliaryStatistics.put(Commands.MSGsSIZE, new Metric(Commands.MSGsSIZE));
		this.auxiliaryStatistics.put(Commands.MaxMSGsSIZE, new Metric(Commands.MaxMSGsSIZE, 0L, Metric.max));
	}

	/**
	 * @param msg
	 */
	public void storeStatisticsFor(Message msg) {
		long size = msg.getSize();
		this.auxiliaryStatistics.get(Commands.MSGsSIZE).value += size;
		if (this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value < size) {
			this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value = size;
		}
	}

	/**
	 * 
	 */
	private void constructCommunicator() {
		this.communicator = new Communicator(this) {

			protected boolean waitForArrivalMessages(long msTimeout) throws InterruptedException {
				logDomains();
				this.waitingForMessage = true;
				boolean arrived = false;
				if (((ABT) this.myAgent).isIdle()) {
					arrived = mailBoxLock.waitUntilTrue(10);
					if (!arrived){
						this.myAgent.fireAgentEvent(new IdleEvent(this.myAgent));
					}
				}
				if (!arrived) {
					arrived = mailBoxLock.waitUntilTrue(msTimeout);
				}
				this.waitingForMessage = false;
				logDomains();
				return arrived;
			}
		};
	}

	@Override
	public void initAgentView() {
		HashList<VarIdentifier> higherNeighbors = this.communicator.getHigherNeighbors();
		for(VarIdentifier varId : higherNeighbors) {
			this.agentView.add((IntExternalVar) this.getExternalVar(varId));
		}
	}

	/*
	 * (non-Javadoc)
	 * @see AbstractAgent#addMsgHandlers()
	 */
	@Override
	public void addMsgHandlers() {
		this.communicator.addMsgHandler(this.okMsgH);
		this.communicator.addMsgHandler(this.ngdMsgH);
		this.communicator.addMsgHandler(this.adlMsgH);
	}

	@Override
	public void configurePropagationEngine() {
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.INSTANCIATE);
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.DESINSTANCIATE);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_THE_LAST).add(VariableEvent.RESTORE_VAL);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_BETWEEN_L_H).add(VariableEvent.RESTORE_VAL);
		this.constraintHandler.update();
	}

	// ////////////////////////////////////////////////////////////////////
	// //////////////////////////// ABT Procedures: //////////////////////
	// ///////////////////////////////////////////////////////////////////
	@Override
	public void startDistributedSearch() throws StopException {
		this.addAgentEventListener((AgentEventListener) this.terminateMsgHandler);
		((ABTTermDetection) this.terminateMsgHandler).sendInitialEndAgentMessage();
		this.checkAgentView();
	}

	/**
	 * choose an instantiation (i.e a value) compatible with the agentView
	 * 
	 * @throws StopException
	 */
	public void checkAgentView() throws StopException {
		int myValue = this.var.getValue();
		if (!this.var.getDomain().contains(myValue)) {
			if (this.valSelector.hasNextValue()) {
				int value = this.valSelector.getNextValue();
				this.var.instantiate(value);
				this.okMsgH.sendOkMessages();
			} else {
				this.backtrack();
			}
		}
	}

	public boolean isIdle() {
		if (this.disSearchStarted) {
			HashList<VarIdentifier> higherNeighbors = this.communicator.getHigherNeighbors();
			return this.var.isInstantiated() && this.agentView.toInstantiation().getVarIds().contains(higherNeighbors);
		}
		return false;
	}

	/**
	 * @throws StopException
	 */
	public void backtrack() throws StopException {
		this.resolveContradiction();
		this.checkAgentView();
	}

	public void resolveContradiction() throws StopException {
		Nogood nogood = this.solveMyNogoods();
		if (nogood.isEmpty()) {
			throw new StopException(Commands.FAILURE);
		}
		this.agentView.clear(nogood.getRhs());
		this.ngdMsgH.sendNogood(nogood);
	}

	/**
	 * Procedure to check if a new link should be added.
	 * 
	 * @param nogood
	 */
	public void checkAddLink(Nogood nogood) {
		Iterator<VarAssignment> it = nogood.getLhs().iterator();
		while (it.hasNext()) {
			VarAssignment ass = it.next();
			VarIdentifier varId = ass.getVarIdentifier();
			if (!this.communicator.isHigherNeighbor(varId)) {
				this.communicator.addNeighbor(varId);
				log("### ADD HIGHER NEIGHBOR: "+varId+" HigherNeighbors="+communicator.getHigherNeighbors(), Logger.DEFAULT_LEVEL);
				this.agentView.add(this.makeExternalVar(ass));
				this.adlMsgH.sendAddLink(ass);
			}
		}
	}

	@Override
	public void sendOkMessage(VarIdentifier agId) {
		this.okMsgH.sendOkMessage(agId);
	}

	public class AddLinkMsgHandler extends MessageHandler {

		public AddLinkMsgHandler(AbstractAgent agent) {
			super(agent, Commands.ADDLINK);
		}

		/**
		 * @param msg
		 */
		public void processMessage(Message msg) throws StopException {
			VarIdentifier senderId = msg.getSenderId();
			VarAssignment myAss = var.getVarAssignment();
			VarAssignment varAss = (VarAssignment) msg.get(Commands.LINK);
			addNeighbor(senderId);
			log("### ADD LOWER NEIGHBOR: "+senderId+" LowerNeighbors="+communicator.getLowerNeighbors(), Logger.DEFAULT_LEVEL);
			if (myAss.getValue() != varAss.getValue()) {/* re-send OK message */
				sendOkMessage(senderId);
			}
		}

		/**
		 * @param ass
		 */
		public void sendAddLink(VarAssignment ass) {
			Message msg = this.createMessage();
			msg.addContents(Commands.LINK, ass);
			msg.setReceiverId(ass.getVarIdentifier());
			storeStatisticsFor(msg);
			sendMessage(msg);
		}
	}

	public class OkMessageHandler extends MessageHandler {

		public OkMessageHandler(AbstractAgent agent) {
			super(agent, Commands.OK);
		}

		/**
		 * @param msg
		 */
		@Override
		public void processMessage(Message msg) throws StopException {
			VarAssignment senderAss = (VarAssignment) msg.get(Commands.OK);
			if (agentView.compareTimeCounter(senderAss) >= 0) {
				agentView.priorityMerge(senderAss);
				checkAgentView();
			} else {
				auxiliaryStatistics.get(Commands.ObsoleteOk).value++;
			}
		}

		/**
		 * 
		 */
		public void sendOkMessages() {
			for(VarIdentifier varId : communicator.getLowerNeighbors()) {
				this.sendOkMessage(varId);
			}
		}

		/**
		 * 
		 */
		public void sendOkMessage(VarIdentifier receiverId) {
			Message msg = this.createMessage();
			msg.addContents(Commands.OK, var.getVarAssignment());
			msg.setReceiverId(receiverId);
			storeStatisticsFor(msg);
			sendMessage(msg);
		}
	}

	/**
	 * for each value we store just one Nogood following this heuristic: when comparing two nogoods, 
	 * select the one with the highest possible lowest variable involved (Nogood Selector HPLV)
	 */
	public class NogoodsHandler extends MessageHandler {

		public NogoodsHandler(AbstractAgent agent) {
			super(agent, Commands.NOGOOD);
		}

		/**
		 * 
		 */
		public void processMessage(Message msg) throws StopException {
			Nogood nogood = (Nogood) msg.get(Commands.NOGOOD);
			int myValue = ((SimpleAgent) this.myAgent).var.getValue();
			if (this.isCoherent(nogood)) {
				checkAddLink(nogood);
				makeNoGoodConstraint(nogood);
				checkAgentView();
			} else { // re-send OK message
				auxiliaryStatistics.get(Commands.nbObsoleteNogood).value++;
				if (nogood.getRhsValue() == myValue) {
					sendOkMessage(msg.getSenderId());
				}
			}
		}

		/**
		 * @param ng
		 */
		public void sendNogood(Nogood ng) {
			Message msg = this.createMessage();
			msg.addContents(Commands.NOGOOD, ng);
			msg.setReceiverId(ng.getRhsVarIdentifier());
			storeStatisticsFor(msg);
			this.myAgent.sendMessage(msg);
		}

		/**
		 * @param nogood
		 */
		public boolean isCoherent(Nogood nogood) {
			int myVal = var.getValue();
			return agentView.compatibleWith(nogood.getLhs()) && nogood.getRhsValue() == myVal;
		}

	}
}
