package simple.dcsp.abtfamily.abt;

import java.util.HashMap;
import java.util.Iterator;
import kernel.AbstractMaster;
import kernel.MasterObserver;
import kernel.VarIdentifier;
import kernel.com.Message;
import util.Commands;
import util.HashList;
import util.Instantiation;

/**
 * termination detection (Silaghi et al. 2000)
 * 
 * @author redouane
 * 
 */
public class TerminationDetector extends MasterObserver {

	protected HashMap<VarIdentifier, Instantiation> acceptedMsgInitialEndAgents;
	protected HashList<VarIdentifier> initialEndAgents;

	public TerminationDetector() {
		super();
		this.acceptedMsgInitialEndAgents = new HashMap<VarIdentifier, Instantiation>();
		this.initialEndAgents = new HashList<VarIdentifier>();
	}

	@Override
	public boolean handleReceivedMessage(Message msg) {
		Instantiation accepted = (Instantiation) msg.get(Commands.ACCEPTED);
		VarIdentifier senderId = msg.getSenderId();
		if (accepted != null) {
			this.acceptedMsgInitialEndAgents.put(senderId, accepted);
			if (this.allAcceptedMsgHaveBeenReceived()) {
				Instantiation solution = areAllcompatible();
				if (solution != null) {
					this.master.setEndState(Commands.SUCCESS);
					this.master.broadcastSystemMessage(AbstractMaster.MasterVarId);
				}
			}
		} else if (msg.get(Commands.INITIAL_END_AGENT) != null) {
			boolean ans = (Boolean) msg.get(Commands.INITIAL_END_AGENT);
			if (ans) {
				this.initialEndAgents.add(senderId);
			}
		}
		return true;
	}

	private boolean allAcceptedMsgHaveBeenReceived() {
		return this.acceptedMsgInitialEndAgents.keySet().containsAll(this.initialEndAgents);
	}

	private Instantiation areAllcompatible() {
		Iterator<VarIdentifier> it = this.acceptedMsgInitialEndAgents.keySet().iterator();
		Instantiation union = new Instantiation();
		while (it.hasNext()) {
			Instantiation ass = this.acceptedMsgInitialEndAgents.get(it.next());
			if (union.compatible(ass)) {
				union.priorityMerge(ass);
			} else {
				return null;
			}
		}
		return union;
	}

	@Override
	public void handleSendMessage(Message msg) {}

}
