package simple.dcsp.abtfamily.abtdac;

import java.util.Iterator;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.events.AgentEventListener;
import kernel.events.VariableEvent;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import simple.dcsp.Nogood;
import simple.dcsp.abtfamily.abt.ABTAgent;
import simple.dcsp.abtfamily.abt.ABTTermDetection;
import util.Commands;
import util.HashList;
import util.VarAssignment;
import util.VarEventMask;

public class ABTdacAgent extends ABTAgent {

	protected DelMsgHandler delMsgH;

	// =================================================================================

	/**
	 * @param agentId
	 * @param agName
	 * @param description
	 */
	public ABTdacAgent(int agentId, String agName, String description) {
		super(agentId, agName, description);
		this.constraintHandler = new ABTdacConstraintHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
	}

	@Override
	public void configurePropagationEngine() {
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.INSTANCIATE);
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.DESINSTANCIATE);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_THE_LAST).add(VariableEvent.RESTORE_VAL);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_BETWEEN_L_H).add(VariableEvent.RESTORE_VAL);
		// cette ligne est pour DAC
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_THE_LAST).add(VariableEvent.REMOVE_VAL);
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_LOWER_THAN_LOCAL_VAR).add(VariableEvent.REMOVE_VAL);
		this.constraintHandler.update();
	}

	@Override
	public void addMsgHandlers() {
		super.addMsgHandlers();
		this.communicator.addMsgHandler(this.delMsgH);
	}

	public void initAgentView() {
		HashList<VarIdentifier> varIds = this.agentOrder.getHigherAgents();
		for(VarIdentifier varId : varIds) {
			if (this.communicator.isNeighbor(varId)) {
				this.agentView.add((IntExternalVar) this.getExternalVar(varId));
			}
		}
	}

	// =================================================================================
	// ABT Procedures
	// =================================================================================
	@Override
	public void startDistributedSearch() throws StopException {
		this.addAgentEventListener((AgentEventListener) this.terminateMsgHandler);
		((ABTTermDetection) this.terminateMsgHandler).sendInitialEndAgentMessage();

		if (this.getMailBoxLock().isTrue())
			this.processSomeMessages();

		this.checkAgentView();
	}

	// =================================================================================
	/**
	 * choose an instantiation (i.e a value) compatible with the agentView
	 * 
	 * @throws StopException
	 */
	public void checkAgentView() throws StopException {
		int myVal = this.var.getValue();
		if (!this.var.getDomain().contains(myVal)) {
			if (this.valSelector.hasNextValue()) {
				int val = this.valSelector.getNextValue();
				this.var.instantiate(val);
				if (var.getPreviousValue() != var.getValue())
					this.okMsgH.sendOkMessages();
			} else {
				this.backtrack();
			}
		}
	}

	// =================================================================================
	public boolean isIdle() {
		if (this.disSearchStarted && this.var.isInstantiated()) {
			HashList<VarIdentifier> hn = communicator.getHigherNeighbors();
			for(VarIdentifier varId : hn) {
				IntExternalVar v = this.agentView.getVar(varId);
				if (!v.isInstantiated())
					return false;
			}
			return true;
		}

		return false;
	}

	// =================================================================================
	/**
	 * @throws StopException
	 */
	public void backtrack() throws StopException {
		this.resolveContradiction();

		this.checkAgentView();
	}

	// =================================================================================

	public void resolveContradiction() throws StopException {
		Nogood nogood = this.solveMyNogoods();
		if (nogood.isEmpty()) {
			throw new StopException(Commands.FAILURE);
		}
		this.agentView.clear(nogood.getRhs());
		this.ngdMsgH.sendNogood(nogood);
	}

	// =================================================================================
	/**
	 * Procedure to check if a new link should be added.
	 * 
	 * @param nogood
	 */
	public void checkAddLink(Nogood nogood) {
		Iterator<VarAssignment> it = nogood.getLhs().iterator();
		while (it.hasNext()) {
			VarAssignment ass = it.next();
			if (!this.communicator.isHigherNeighbor(ass.getVarIdentifier())) {
				this.communicator.addNeighbor(ass.getVarIdentifier());
				this.agentView.add(this.makeExternalVar(ass));
				this.adlMsgH.sendAddLink(ass);
			}
		}
	}

	// =================================================================================

	public void valuesConditionallyDeleted(Nogood nogood) throws StopException {
		if (this.agentView.compatibleWith(nogood.getLhs())) {
			this.makeNoGoodConstraint(nogood);

			this.checkAddLink(nogood);
			this.checkAgentView();
		}

	}

	// =================================================================================

	public void sendConditionalDeldMsg(int val) {
		Nogood ng = this.makeBetterNogood((ExplainedIntVar) var, val);
		VarIdentifier last = ng.getLhs().last().getVarIdentifier();
		HashList<VarIdentifier> neighbors0 = this.communicator.getHigherNeighbors0();
		for(VarIdentifier varId : neighbors0) {
			if (this.agentOrder.chooseHigher(varId, last).equals(last) && !last.equals(varId) && ng.getLhs().size() > 1) {
				this.delMsgH.sendConditionalDeldMsg(ng, varId);
			}
		}
	}

	// =================================================================================

	@Override
	public void sendOkMessage(VarIdentifier agId) {
		this.okMsgH.sendOkMessage(agId);
	}

}
