/***********************************************************************
ABTDO Constraints 2008 with some corrections and add permanent link
 ************************************************************************/

package simple.dcsp.abtfamily.abtdo;

import static simple.dcsp.abtfamily.abtdo.ABTDOMaster.MinDom;
import static simple.dcsp.abtfamily.abtdo.ABTDOMaster.MinDom_Single;
import static simple.dcsp.abtfamily.abtdo.ABTDOMaster.Nogood_triggered;
import static simple.dcsp.abtfamily.abtdo.ABTDOMaster.Random;
import static simple.dcsp.abtfamily.abtdo.ABTDOMaster.Random_Single;
import java.util.HashMap;
import java.util.Iterator;
import kernel.AbstractAgent;
import kernel.Communicator;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.events.AgentEventListener;
import kernel.events.VariableEvent;
import kernel.order.AgentOrdering;
import kernel.vars.integer.IntExternalVar;
import simple.SingleExplainedVarAgent;
import simple.dcsp.Nogood;
import simple.dcsp.abtfamily.IdleEvent;
import simulation.metric.Metric;
import simulation.parser.PVariable;
import util.Commands;
import util.HashList;
import util.Instantiation;
import util.Tools;
import util.VarAssignment;
import util.VarEventMask;

public class ABTDOAgent extends SingleExplainedVarAgent {

	private CurrentOrder currentOrder = null;

	public OkMsgHandler okMsgH = null;
	public NogoodMsgHandler ngdMsgH = null;
	public AddLinkMsgHandler adlMsgH = null;
	public OrderMsgHandler orderMsgH = null;
	public HashMap<VarIdentifier, Integer> domainSizes;
	public HashMap<VarIdentifier, Integer> degrees;
	public short heuristic = 0;

	public static final String CHANGE_ORDER = "ChangeOrder";
	public static final String nbIncompatibleNogood = "nbIncompatibleNogood";

	/**
	 * @param agentId
	 * @param agName
	 * @param description
	 * @param heuristic
	 */
	public ABTDOAgent(int agId, String name, String desc, short heuristic) {
		super(agId, name, desc);
		this.heuristic = heuristic;
		this.domainSizes = new HashMap<VarIdentifier, Integer>();
		this.degrees = new HashMap<VarIdentifier, Integer>();
		this.agentOrder = new AgentOrdering(this) {
			@Override
			public void processOrder(Message msg) {
				super.processOrder(msg);
				HashMap<VarIdentifier, Integer> doms = Tools.mapfromString((String) msg.get(Commands.DomainSize), VarIdentifier.class, Integer.class);
				domainSizes.putAll(doms);
			}
		};
		this.communicator = new ABTDOCommunicator(this);
		// -------------- Messages Handler ------------------//
		this.terminateMsgHandler = new ABTDOTermDetection(this);
		this.okMsgH = new OkMsgHandler(this);
		this.ngdMsgH = new NogoodMsgHandler(this);
		this.orderMsgH = new OrderMsgHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
		this.terminateMsgHandler = new ABTDOTermDetection(this);
		this.initAuxStatistics();
	}

	/**
	 * @param variable
	 * @param heuristic
	 */
	public ABTDOAgent(PVariable variable, short heuristic) {
		super(variable);
		this.heuristic = heuristic;
		this.domainSizes = new HashMap<VarIdentifier, Integer>();
		this.degrees = new HashMap<VarIdentifier, Integer>();
		this.agentOrder = new AgentOrdering(this) {
			@Override
			public void processOrder(Message msg) {
				super.processOrder(msg);
				HashMap<VarIdentifier, Integer> doms = Tools.mapfromString((String) msg.get(Commands.DomainSize), VarIdentifier.class, Integer.class);
				domainSizes.putAll(doms);
			}
		};
		this.communicator = new ABTDOCommunicator(this);
		// -------------- Messages Handler ------------------//
		this.terminateMsgHandler = new ABTDOTermDetection(this);
		this.okMsgH = new OkMsgHandler(this);
		this.ngdMsgH = new NogoodMsgHandler(this);
		this.orderMsgH = new OrderMsgHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
		this.terminateMsgHandler = new ABTDOTermDetection(this);
		this.initAuxStatistics();
	}
	private void initAuxStatistics() {
		this.auxiliaryStatistics.put(Commands.nbObsoleteNogood, new Metric(Commands.nbObsoleteNogood));
		this.auxiliaryStatistics.put(Commands.nbAssignments, new Metric(Commands.nbAssignments));
		this.auxiliaryStatistics.put(Commands.ObsoleteOk, new Metric(Commands.ObsoleteOk));
		this.auxiliaryStatistics.put(CHANGE_ORDER, new Metric(CHANGE_ORDER, 0L, Metric.max));
		this.auxiliaryStatistics.put(nbIncompatibleNogood, new Metric(nbIncompatibleNogood));
		this.auxiliaryStatistics.put(Commands.MSGsSIZE, new Metric(Commands.MSGsSIZE));
		this.auxiliaryStatistics.put(Commands.MaxMSGsSIZE, new Metric(Commands.MaxMSGsSIZE, 0L, Metric.max));
	}

	/**
	 * @param msg
	 */
	public void storeStatisticsFor(Message msg) {
		long size = msg.getSize();
		this.auxiliaryStatistics.get(Commands.MSGsSIZE).value += size;
		if(this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value < size){
			this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value = size;
		}
	}

	// ==========================================================================

	public void initAgentView() {
		HashList<VarIdentifier> varIds = this.communicator.getNeighbors();
		for (VarIdentifier varId : varIds) {
			this.agentView.add((IntExternalVar) this.getExternalVar(varId));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see AbstractAgent#addMsgHandlers()
	 */
	@Override
	public void addMsgHandlers() {
		this.communicator.addMsgHandler(this.okMsgH);
		this.communicator.addMsgHandler(this.ngdMsgH);
		this.communicator.addMsgHandler(this.orderMsgH);
		this.communicator.addMsgHandler(this.adlMsgH);
	}

	@Override
	public void configurePropagationEngine() {
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.INSTANCIATE);
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.DESINSTANCIATE);

		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_THE_LAST).add(VariableEvent.RESTORE_VAL);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_BETWEEN_L_H).add(VariableEvent.RESTORE_VAL);

		this.constraintHandler.update();
	}

	@Override
	public void startDistributedSearch() throws StopException {
		this.currentOrder = new CurrentOrder(this.getOrder());
		this.addAgentEventListener((AgentEventListener) this.terminateMsgHandler);
		((ABTDOTermDetection) this.terminateMsgHandler).setBestOrder(this.currentOrder);
		this.CheckAgentView();
	}

	public boolean isIdle() {
		if (this.disSearchStarted) {
			HashList<VarIdentifier> neighbors = this.communicator.getNeighbors();
			Instantiation view = this.agentView.toInstantiation();
			boolean idle = view.getVarIds().contains(neighbors) && this.var.isInstantiated();
			if (idle) {
				((ABTDOTermDetection) this.terminateMsgHandler).setCurrentOrder(currentOrder);
			}
			return idle;
		}
		return false;
	}

	/**
	 * choose an instantiation (i.e a value) compatible with the agentView
	 * 
	 * @throws StopException
	 */
	public void CheckAgentView() throws StopException {
		int myValue = this.var.getValue();
		if (!this.var.getDomain().contains(myValue)) {
			if (this.valSelector.hasNextValue()) {
				int value = this.valSelector.getNextValue();
				this.var.instantiate(value);
				if (!this.iNogoodTriggered()) {
					this.ChooseNewOrder();
				}
				this.okMsgH.sendOkMessages();
				this.orderMsgH.sendOrderMessages();
			} else {
				this.Backtrack();
			}
		}
	}

	// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * @throws StopException
	 */

	public void Backtrack() throws StopException {
		this.resolveContradiction();
		//		
		this.CheckAgentView();
	}

	// ///////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * 
	 */
	public void resolveContradiction() throws StopException {
		Instantiation nogood = this.solveMyNogoodStore();
		if (nogood.isEmpty()) { throw new StopException(Commands.FAILURE); }
		for (VarAssignment ass : nogood) {
			if (this.agentOrder.isLower(ass.getVarIdentifier())) {
				System.err.println(this.agentId + " try to backtrack AgentView: " + this.agentView + "\n" + this.getOrder() + " \n"
						+ this.constraintHandler.toString() + " :: " + "\n" + nogood + "\n" + this.var.getDomain());
				System.exit(0);
			}
		}
		this.agentView.clear(nogood.last());
		this.ngdMsgH.sendNogood(nogood, this.getVarIdentifier(), nogood.last().getVarIdentifier());
	}

	// ///////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Procedure to check if a new link should be added.
	 * 
	 * @param nogood
	 */
	public void CheckAddLink(Nogood nogood) {
		Iterator<VarAssignment> it = nogood.getLhs().iterator();
		while (it.hasNext()) {
			VarAssignment ass = it.next();
			if (!this.communicator.isNeighbor(ass.getVarIdentifier())) {
				this.communicator.addNeighbor(ass.getVarIdentifier());
				this.agentView.add(this.makeExternalVar(ass));
				this.adlMsgH.sendAddLink(ass);
			}
		}
	}

	public void setDomainSize(VarIdentifier senderId, int domainSize) {
		this.domainSizes.put(senderId, domainSize);
	}

	public boolean iNogoodTriggered() {
		return this.heuristic == Nogood_triggered;
	}

	/**
	 * 
	 * @param oldAgentView
	 * @param order
	 * @param counters
	 */
	public boolean setCurrentOrder(CurrentOrder currentOrder) {
		if (this.currentOrder.compare(currentOrder) < 0) {
			this.currentOrder = currentOrder;
			if (this.agentOrder.setNewOrder(currentOrder.getOrder())) {
				this.auxiliaryStatistics.get(CHANGE_ORDER).value++;
				this.constraintHandler.update();
			}
			return true;
		}
		return false;
	}

	/**
	 * 
	 */
	private void ChooseNewOrder() {
		switch (this.heuristic) {
			case Random :
				this.ChooseRandomOrder();
			break;
			case MinDom :
				this.ChooseMinDomOrder();
			break;
			case Random_Single :
				this.ChooseSingleRandomOrder();
			break;
			case MinDom_Single :
				this.ChooseSingleMinDomOrder();
			break;
			default :
			break;
		}
	}

	public boolean ChangeOrderNG(VarIdentifier generator) {
		if (this.agentOrder.isLower(generator)) {
			CurrentOrder newOrder = this.currentOrder.getNgTriggeredOrder(this.getVarIdentifier(), generator);
			return setCurrentOrder(newOrder);
		}
		return false;
	}

	private void ChooseSingleMinDomOrder() {
		CurrentOrder newOrder = this.currentOrder.getSingleDomainSizeOrder(this.getVarIdentifier(), this.domainSizes);
		this.setCurrentOrder(newOrder);
	}

	private void ChooseSingleRandomOrder() {
		CurrentOrder newOrder = this.currentOrder.getSingleRandomOrder(this.getVarIdentifier());
		this.setCurrentOrder(newOrder);
	}

	private void ChooseMinDomOrder() {
		CurrentOrder newOrder = this.currentOrder.getDomainSizeOrder(this.getVarIdentifier(), this.domainSizes);
		this.setCurrentOrder(newOrder);
	}

	private void ChooseRandomOrder() {
		CurrentOrder newOrder = this.currentOrder.getRandomOrder(this.getVarIdentifier());
		this.setCurrentOrder(newOrder);
	}

	public CurrentOrder getCurrentOrder() {
		return this.currentOrder.clone();
	}



	/**
	 * 
	 */
	class ABTDOCommunicator extends Communicator {
		/**
		 * @param agent
		 */
		public ABTDOCommunicator(AbstractAgent agent) {
			super(agent);
		}

		protected boolean waitForArrivalMessages(long msTimeout) throws InterruptedException {
			logDomains();
			this.waitingForMessage = true;
			boolean arrived = false;
			if (((ABTDOAgent) this.myAgent).isIdle()) {
				arrived = mailBoxLock.waitUntilTrue(5L);
				if (!arrived) {
					this.myAgent.fireAgentEvent(new IdleEvent(this.myAgent));
				}
			}
			if (!arrived) {
				arrived = mailBoxLock.waitUntilTrue(msTimeout);
			}
			this.waitingForMessage = false;
			logDomains();
			return arrived;
		}
		
		public HashList<VarIdentifier> getMyLowerNeighbors(){
			return HashList.intersection(this.getLowerNeighbors(), this.getNeighbors0());
		}
		
		public HashList<VarIdentifier> getMyHigherNeighbors(){
			return HashList.intersection(getHigherNeighbors(), getNeighbors0());
		}
	}
}
