package simple.dcsp.abtfamily.abtdo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.MasterObserver;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.order.AgentComplexity;
import kernel.order.LexicographicAO;
import simulation.parser.PVariable;
import util.Commands;
import util.HashList;

public class ABTDOMaster extends AbstractMaster {

	public final static short Nogood_triggered = 1;
	public final static short MinDom = 2;
	public final static short Random = 3;
	public final static short MinDom_Single = 4;
	public final static short Random_Single = 5;

	public short heuristic = Nogood_triggered;

	public ABTDOMaster(short heuristic) {
		this.setCentralizedAO(new LexicographicAO());
		this.addObserver(new TerminationDetector());
		this.heuristic = heuristic;
	}

	public AbstractAgent makeAgent(int agId, String name, String desc) {
		return new ABTDOAgent(agId, name, desc, this.heuristic);
	}

	public AbstractAgent makeAgent(PVariable variable) {
		return new ABTDOAgent(variable, this.heuristic);
	}

	public void broadcast(Message msg, VarIdentifier agentId) {
		if (msg.getType().equals(Commands.AO)) {
			HashMap<VarIdentifier, AgentComplexity> ac = this.centralizedAO.getAgentsComplexity();
			Map<VarIdentifier, Integer> domainSizes = new HashMap<VarIdentifier, Integer>();
			for(Map.Entry<VarIdentifier, AgentComplexity> entry : ac.entrySet()) {
				domainSizes.put(entry.getKey(), entry.getValue().getMaxDomain());
			}
			msg.addContents(Commands.DomainSize, domainSizes.toString());
		}
		super.broadcast(msg, agentId);
	}

	public HashList<VarIdentifier> getOrder() {
		for (MasterObserver observer : this.observers) {
			if (observer instanceof TerminationDetector) { 
				return ((TerminationDetector) observer).bestOrder.getOrder();
			}
		}
		return new HashList<VarIdentifier>();
	}

	/* (non-Javadoc)
	 * @see kernel.AbstractMaster#getTypesOfMessages()
	 */
	@Override
	public Collection<String> getTypesOfMessages() {
		Collection<String> ans = new ArrayList<String>();
		ans.add(Commands.OK);
		ans.add(Commands.NOGOOD);
		ans.add(Commands.ADDLINK);
		ans.add(Commands.ORDER);
		return ans;
	}

}
