package simple.dcsp.abtfamily.abtdo;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Random;
import kernel.VarIdentifier;
import util.HashList;
/**
 * 
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class CurrentOrder implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8522009754008718289L;
	private HashList<VarIdentifier> order;
	private HashMap<VarIdentifier, Integer> counters;

	public CurrentOrder(HashList<VarIdentifier> order) {
		this.order = order;
		this.counters = new HashMap<VarIdentifier, Integer>();
		for (VarIdentifier agId : this.order) {
			this.counters.put(agId, 0);
		}
	}

	public CurrentOrder(HashList<VarIdentifier> order, HashMap<VarIdentifier, Integer> counters) {
		this.order = (HashList<VarIdentifier>) order.clone();
		this.counters = counters;
	}

	@SuppressWarnings("unchecked")
	public CurrentOrder clone() {
		HashMap<VarIdentifier, Integer> counters = (HashMap<VarIdentifier, Integer>) this.counters.clone();
		return new CurrentOrder((HashList<VarIdentifier>) order.clone(), counters);
	}

	public HashList<VarIdentifier> getOrder() {
		return this.order;
	}

	public String toString() {
		String order = "[";
		for (VarIdentifier agId : this.order) {
			int counter = this.counters.get(agId);
			order += "(" + agId + "," + counter + ");";
		}
		order = order.substring(0, order.length() - 1);
		order += "]";
		return order;
	}


	public void setNewOrder(VarIdentifier myId, HashList<VarIdentifier> newOrder) {
		this.order = newOrder;
		this.updateCounters(myId);
	}

	public void updateCounters(VarIdentifier myId) {
		HashList<VarIdentifier> lowerAgents = this.order.getElementsAfter(myId);
		for (VarIdentifier lower : lowerAgents) {
			this.counters.put(lower, 0);
		}
		this.counters.put(myId, this.counters.get(myId) + 1);
	}

	/**
	 * 
	 * @param currentOrder
	 * @return <b>(i+1)</b> if <code>this</code> order is the newest</br> <b>0</b> if this order and <code>currentOrder</code> are equal</br>
	 *         <b>-(i+1)</b> if <code>currentOrder</code> is the newest</br>
	 */
	public int compare(CurrentOrder currentOrder) {
		for (int i = 0 ; i < this.order.size() ; i++) {
			VarIdentifier varId1 = this.order.get(i);
			int counter1 = this.counters.get(varId1);
			VarIdentifier varId2 = currentOrder.order.get(i);
			int counter2 = currentOrder.counters.get(varId2);
			if (counter1 > counter2) { return (i + 1); }
			if (counter1 < counter2) { return -(i + 1); }
		}
		return 0;
	}

	public CurrentOrder getNgTriggeredOrder(VarIdentifier myId, VarIdentifier varId) {
		CurrentOrder newOrder = this.clone();
		int myRank = newOrder.order.indexOf(myId);
		int agRank = newOrder.order.indexOf(varId);

		for (int i = agRank ; i > myRank + 1 ; i--) {
			newOrder.order.add(i, newOrder.order.get(i - 1));
		}
		newOrder.order.add(myRank + 1, varId);
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public CurrentOrder getRandomOrder(VarIdentifier myId) {
		CurrentOrder newOrder = this.clone();
		HashList<VarIdentifier> lowerAgents = newOrder.order.getElementsAfter(myId);
		Random rand = new Random(System.currentTimeMillis());
		for (int i = newOrder.order.indexOf(myId) + 1 ; i < newOrder.order.size() ; i++) {
			int index = rand.nextInt(lowerAgents.size());
			VarIdentifier varId = lowerAgents.get(index);
			lowerAgents.remove(varId);
			newOrder.order.add(i, varId);
		}
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public CurrentOrder getDomainSizeOrder(VarIdentifier myId, HashMap<VarIdentifier, Integer> domainSizes) {
		CurrentOrder newOrder = this.clone();
		HashList<VarIdentifier> lowerAgents = newOrder.order.getElementsAfter(myId);
		VarIdentifier[] agents = sort(lowerAgents, domainSizes);
		for (int i = 0 ; i < lowerAgents.size() ; i++) {
			VarIdentifier varId = agents[i];
			newOrder.order.add(i + newOrder.order.indexOf(myId) + 1, varId);
		}
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public CurrentOrder getSingleRandomOrder(VarIdentifier myId) {
		CurrentOrder newOrder = this.clone();
		Random rand = new Random(System.currentTimeMillis());
		HashList<VarIdentifier> lowerAgents = newOrder.order.getElementsAfter(myId);
		int index = rand.nextInt(lowerAgents.size());
		VarIdentifier varId = lowerAgents.get(index);
		int myRank = newOrder.order.indexOf(myId);
		int agRank = newOrder.order.indexOf(varId);
		for (int i = agRank ; i > myRank + 1 ; i--) {
			newOrder.order.add(i, newOrder.order.get(i - 1));
		}
		newOrder.order.add(myRank + 1, varId);
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public CurrentOrder getSingleDomainSizeOrder(VarIdentifier myId, HashMap<VarIdentifier, Integer> domainSizes) {
		CurrentOrder newOrder = this.clone();
		HashList<VarIdentifier> lowerAgents = newOrder.order.getElementsAfter(myId);
		if (lowerAgents.size() == 0) {
			newOrder.updateCounters(myId);
			return newOrder;
		}
		VarIdentifier[] agents = sort((HashList<VarIdentifier>)lowerAgents.clone(), domainSizes);
		VarIdentifier varId = agents[0];
		int myRank = newOrder.order.indexOf(myId);
		int agRank = newOrder.order.indexOf(varId);
		for (int i = agRank ; i > myRank + 1 ; i--) {
			newOrder.order.add(i, newOrder.order.get(i - 1));
		}
		newOrder.order.add(myRank + 1, varId);
		newOrder.updateCounters(myId);
		return newOrder;
	}

	public static VarIdentifier[] sort(HashList<VarIdentifier> agents, HashMap<VarIdentifier, Integer> domainSizes) {
		VarIdentifier[] ans = new VarIdentifier[agents.size()];
		for (int i = 0 ; i < ans.length ; i++) {
			VarIdentifier minId = agents.get(0);
			int minSize = domainSizes.get(minId);
			for (VarIdentifier agId : agents) {
				if (minSize > domainSizes.get(agId)) {
					minSize = domainSizes.get(agId);
					minId = agId;
				}
			}
			ans[i] = minId;
			agents.remove(minId);
		}
		return ans;
	}

	public boolean equals(Object obj) {
		CurrentOrder arg = (CurrentOrder) obj;
		return (arg.order.equals(this.order) && this.compare(arg) == 0);
	}

//	public static void main(String[] agrs) {
//		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
//		VarIdentifier v1 = new VarIdentifier(1);
//		VarIdentifier v2 = new VarIdentifier(2);
//		VarIdentifier v3 = new VarIdentifier(3);
//		VarIdentifier v4 = new VarIdentifier(4);
//		VarIdentifier v5 = new VarIdentifier(5);
//		order.add(v1);
//		order.add(v2);
//		order.add(v3);
//		order.add(v4);
//		order.add(v5);
//		HashMap<VarIdentifier, Integer> domainSizes = new HashMap<VarIdentifier, Integer>();
//		domainSizes.put(v1, 5);
//		domainSizes.put(v2, 2);
//		domainSizes.put(v3, 3);
//		domainSizes.put(v4, 4);
//		domainSizes.put(v5, 1);
//		System.out.println("order: "+order + "\n" + "domainSizes: "+domainSizes);
//		System.out.println("order: "+order);
//		VarIdentifier[] agents = sort(order, domainSizes);
//		System.out.print("agents: ");
//		for (int i = 0 ; i < agents.length ; i++) {
//			System.out.print(agents[i] + " ");
//		}
//		System.out.println();
//	}

	public HashMap<VarIdentifier, Integer> getCounters() {
		return this.counters;
	}

}
