package simple.dcsp.abtfamily.abtdoretro;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import kernel.AbstractAgent;
import kernel.Communicator;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.events.AgentEventListener;
import kernel.events.VariableEvent;
import kernel.order.AgentOrdering;
import kernel.vars.integer.IntExternalVar;
import simple.SingleExplainedVarAgent;
import simple.dcsp.abtfamily.IdleEvent;
import simulation.metric.Metric;
import simulation.parser.PVariable;
import util.Commands;
import util.HashList;
import util.Instantiation;
import util.Logger;
import util.Tools;
import util.VarAssignment;
import util.VarEventMask;

public class ABTDORetroAgent extends SingleExplainedVarAgent {
	
	public static final String CHANGE_ORDER = "ChangeOrder";
	public static final String nbIncompatibleNogood = "nbIncompatibleNogood";

	public int K = 0;

	public final static String DomainSize = "dom";
	public final static String Stampe = "stampe";
	public final static String Generator = "generator";

	public OkMsgHandler okMsgH = null;
	public NogoodsHandler ngdMsgH = null;
	public AddLinkMsgHandler adlMsgH = null;
	public OrderMsgHandler orderMsgH = null;

	public short heuristic;
	private CurrentOrder currentOrder = null;
	public HashMap<VarIdentifier, Integer> domainSizes;
	public HashMap<VarIdentifier, Integer> degrees;

	private ArrayList<Instantiation> constraints;

	/**
	 * @param agentId
	 * @param agName
	 * @param description
	 */
	public ABTDORetroAgent(int agId, String name, String desc, short heuristic) {
		super(agId, name, desc);
		this.heuristic = heuristic;
		this.constrectorHelper();
	}

	public ABTDORetroAgent(int agId, String name, String desc, int K, short heuristic) {
		super(agId, name, desc);
		this.K = K;
		this.heuristic = heuristic;
		this.constrectorHelper();
		this.initAuxStatistics();
	}

	public ABTDORetroAgent(PVariable variable, int K, short heuristic) {
		super(variable);
		this.K = K;
		this.heuristic = heuristic;
		this.constrectorHelper();
		this.initAuxStatistics();
	}

	private void initAuxStatistics() {
		this.auxiliaryStatistics.put(Commands.nbObsoleteNogood, new Metric(Commands.nbObsoleteNogood));
		this.auxiliaryStatistics.put(Commands.nbAssignments, new Metric(Commands.nbAssignments));
		this.auxiliaryStatistics.put(Commands.ObsoleteOk, new Metric(Commands.ObsoleteOk));
		this.auxiliaryStatistics.put(CHANGE_ORDER, new Metric(CHANGE_ORDER, 0L, Metric.max));
		this.auxiliaryStatistics.put(nbIncompatibleNogood, new Metric(nbIncompatibleNogood));
		this.auxiliaryStatistics.put(Commands.MSGsSIZE, new Metric(Commands.MSGsSIZE));
		this.auxiliaryStatistics.put(Commands.MaxMSGsSIZE, new Metric(Commands.MaxMSGsSIZE, 0L, Metric.max));
	}

	/**
	 * @param msg
	 */
	public void storeStatisticsFor(Message msg) {
		long size = msg.getSize();
		this.auxiliaryStatistics.get(Commands.MSGsSIZE).value += size;
		if(this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value < size){
			this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value = size;
		}
	}

	private void constrectorHelper() {
		this.domainSizes = new HashMap<VarIdentifier, Integer>();
		this.degrees = new HashMap<VarIdentifier, Integer>();
		this.constraints = new ArrayList<Instantiation>();
		this.agentOrder = new AgentOrdering(this) {
			@Override
			public void processOrder(Message msg) {
				super.processOrder(msg);
				Map<VarIdentifier, Integer> doms = Tools.mapfromString((String) msg.get(Commands.DomainSize), VarIdentifier.class, Integer.class);
				domainSizes.putAll(doms);
			}
		};
		this.communicator = new ABTDORetroCommunicator(this);
		// -------------- Messages Handler ------------------//
		this.okMsgH = new OkMsgHandler(this);
		this.ngdMsgH = new NogoodsHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
		this.orderMsgH = new OrderMsgHandler(this);;
		this.terminateMsgHandler = new ABTDOTermDetection(this);
	}
	
	public void initAgentView() {
		HashList<VarIdentifier> varIds = this.communicator.getNeighbors();
		for (VarIdentifier varId : varIds) {
			this.agentView.add((IntExternalVar) this.getExternalVar(varId));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see AbstractAgent#addMsgHandlers()
	 */
	@Override
	public void addMsgHandlers() {
		this.communicator.addMsgHandler(this.okMsgH);
		this.communicator.addMsgHandler(this.ngdMsgH);
		this.communicator.addMsgHandler(this.orderMsgH);
		this.communicator.addMsgHandler(this.adlMsgH);
	}

	@Override
	public void configurePropagationEngine() {
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.INSTANCIATE);
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.DESINSTANCIATE);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_THE_LAST).add(VariableEvent.RESTORE_VAL);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_BETWEEN_L_H).add(VariableEvent.RESTORE_VAL);
		this.constraintHandler.update();
	}

	@Override
	protected void startDistributedSearch() throws StopException {
		this.currentOrder = new CurrentOrder(this.getOrder());
		this.addAgentEventListener((AgentEventListener) this.terminateMsgHandler);
		((ABTDOTermDetection) this.terminateMsgHandler).setBestOrder(this.currentOrder);
		this.CheckAgentView();
	}

	public boolean isIdle() {
		if (this.disSearchStarted) {
			HashList<VarIdentifier> higherN = this.communicator.getHigherNeighbors();
			Instantiation view = this.agentView.getContextsOfHigherAgents();
			boolean idle = view.getVarIds().contains(higherN) && this.var.isInstantiated();
			if (idle) {
				((ABTDOTermDetection) this.terminateMsgHandler).setCurrentOrder(this.currentOrder);
			}
			return idle;
		}
		return false;
	}

	// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// //////////////////////////// ABT Procedures: ///////////////////////////////////////////////////////////////
	// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// /////////////////////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * choose an instantiation (i.e a value) compatible with the agentView
	 * 
	 * @throws StopException
	 */
	public void CheckAgentView() throws StopException {
		this.domainSizes.put(this.getVarIdentifier(), this.var.getDomainSize());
		int myValue = this.var.getValue();
		if (!this.var.getDomain().contains(myValue)) {
			// if my current assignment is not consistent with all
			// higher priority assignments in myAgentView
			if (this.valSelector.hasNextValue()) {
				int value = this.valSelector.getNextValue();
				this.var.instantiate(value);
				this.okMsgH.sendOkMessages();
			} else {
				// no value in myDomain is consistent with all higher
				// priority assignments in myAgentView
				this.Backtrack();
			}
		}
	}

	// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * @throws StopException
	 */

	public void Backtrack() throws StopException {
		Instantiation nogood = this.solveMyNogoodStore();
		if (nogood.isEmpty()) { throw new StopException(Commands.FAILURE); }
		VarAssignment lhs = nogood.last().clone();
		VarIdentifier target = lhs.getVarIdentifier();
		VarIdentifier generator = this.getVarIdentifier();
		HashList<VarIdentifier> conflictsVarIds = nogood.getVarIds();
		conflictsVarIds.remove(target);
		VarIdentifier sencondLower = this.agentOrder.getLowerAgent(conflictsVarIds);
		CurrentOrder newOrder = null;
		if (nogood.size() > this.K) {
			newOrder = this.ChooseNewOrder(target, sencondLower);
			this.ngdMsgH.sendNogood(nogood, generator, target);
		} else {
			if (!this.constraints.contains(nogood)) {
				this.constraints.add(nogood);
				for (VarIdentifier agId : conflictsVarIds) {
					this.ngdMsgH.sendNogood(nogood, generator, agId);
				}
			}
			newOrder = this.ChooseNewOrder(target, null);
		}
		this.setCurrentOrder(newOrder);
		this.orderMsgH.sendOrderMessages();
		this.agentView.clear(lhs);
		this.CheckAgentView();
	}

	/**
	 * 
	 * @param sencondLower
	 * @return
	 */
	private CurrentOrder ChooseNewOrder(VarIdentifier target, VarIdentifier sencondLower) {
		return this.currentOrder.getNewOrder(this.heuristic, this.getVarIdentifier(), target, sencondLower, this.domainSizes);
	}

	// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Procedure to check if a new link should be added.
	 * 
	 * @param nogood
	 */
	public void CheckAddLink(Instantiation nogood) {
		nogood.remove(this.var.getVarAssignment());
		Iterator<VarAssignment> it = nogood.iterator();
		while (it.hasNext()) {
			VarAssignment ass = it.next();
			if (!this.communicator.isNeighbor(ass.getVarIdentifier())) {
				this.communicator.addNeighbor(ass.getVarIdentifier());
				this.agentView.add(this.makeExternalVar(ass));
				this.adlMsgH.sendAddLink(ass);
			}
		}
	}

	// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// /////////////////////////////////////////////////////////////////////////////////////////////////////////////
	public void setDomainSize(VarIdentifier varId, int domainSize) {
		this.domainSizes.put(varId, domainSize);
	}

	public void store(Instantiation nogood) {
		if (!this.constraints.contains(nogood)) {
			this.constraints.add(nogood);
		}
	}

	/**
	 * 
	 * @param currentOrder
	 * @return
	 */
	public boolean setCurrentOrder(CurrentOrder currentOrder) {
		if (this.currentOrder.compare(currentOrder) < 0) {
			this.currentOrder = currentOrder;
			if (this.agentOrder.setNewOrder(currentOrder.getOrder())) {
				this.auxiliaryStatistics.get(CHANGE_ORDER).value++;
				this.constraintHandler.update();
				this.log("myAgentView: " + this.agentView + "" + this.constraintHandler, Logger.INFO_LEVEL);
			}
			return true;
		}
		return false;
	}

	public CurrentOrder getCurrentOrder() {
		return this.currentOrder.clone();
	}
	
	/**
	 * 
	 */
	class ABTDORetroCommunicator extends Communicator {
		/**
		 * @param agent
		 */
		public ABTDORetroCommunicator(AbstractAgent agent) {
			super(agent);
		}

		protected boolean waitForArrivalMessages(long msTimeout) throws InterruptedException {
			logDomains();
			this.waitingForMessage = true;
			boolean arrived = false;
			if (((ABTDORetroAgent) this.myAgent).isIdle()) {
				arrived = mailBoxLock.waitUntilTrue(5L);
				if (!arrived) {
					this.myAgent.fireAgentEvent(new IdleEvent(this.myAgent));
				}
			}
			if (!arrived) {
				arrived = mailBoxLock.waitUntilTrue(msTimeout);
			}
			this.waitingForMessage = false;
			logDomains();
			return arrived;
		}
		
		public HashList<VarIdentifier> getMyLowerNeighbors(){
			return HashList.intersection(this.getLowerNeighbors(), this.getNeighbors0());
		}
		
		public HashList<VarIdentifier> getMyHigherNeighbors(){
			return HashList.intersection(this.getHigherNeighbors(), getNeighbors0());
		}
	}
}
