package simple.dcsp.abtfamily.abtdoretro;

import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.MessageHandler;
import simple.SimpleAgent;
import simple.dcsp.Nogood;
import util.Commands;
import util.Instantiation;
import util.VarAssignment;

/**
 * for each value we store just one Nogood following this heuristic: when comparing two nogoods, select the one with the highest possible lowest
 * variable involved (Nogood Selector HPLV)
 */

public class NogoodsHandler extends MessageHandler {

	public NogoodsHandler(ABTDORetroAgent agent) {
		super(agent, Commands.NOGOOD);
	}

	/**
	 * 
	 */
	public void processMessage(Message msg) throws StopException {
		VarIdentifier senderId = msg.getSenderId();
		Instantiation ng = ((Instantiation) msg.get(Commands.NOGOOD)).clone(this.myAgent.agentOrder);
		VarIdentifier generator = (VarIdentifier) msg.get(ABTDORetroAgent.Generator);
		int myValue = ((SimpleAgent) this.myAgent).var.getValue();
		VarIdentifier target = ng.last().getVarIdentifier();
		int ngValue = ng.getVarAssignment(((ABTDORetroAgent) this.myAgent).var.getVarIdentifier()).getValue();
		if (ng.size() <= ((ABTDORetroAgent) this.myAgent).K) {
			((ABTDORetroAgent) this.myAgent).CheckAddLink(ng.clone());
			((ABTDORetroAgent) this.myAgent).store(ng);
			if (this.isCoherent(ng)) {
				if (target.equals(this.myAgent.getVarIdentifier())) {
					Nogood nogood = new Nogood(ng);
					((ABTDORetroAgent) this.myAgent).makeNoGoodConstraint(nogood);
					((ABTDORetroAgent) this.myAgent).CheckAgentView();
				}
			} else if (myValue == ngValue && generator.equals(senderId)) {
				this.myAgent.auxiliaryStatistics.get(Commands.nbObsoleteNogood).value++;
				((ABTDORetroAgent) this.myAgent).okMsgH.sendOkMessage(senderId);
			}
		} else {
			if (this.isCoherent(ng)) {
				if (target.equals(this.myAgent.getVarIdentifier())) {
					((ABTDORetroAgent) this.myAgent).CheckAddLink(ng.clone());
					Nogood nogood = new Nogood(ng);
					((ABTDORetroAgent) this.myAgent).makeNoGoodConstraint(nogood);
					((ABTDORetroAgent) this.myAgent).CheckAgentView();
				} else {
					this.myAgent.auxiliaryStatistics.get(ABTDORetroAgent.nbIncompatibleNogood).value++;
					this.sendNogood(ng, generator, target);
					if (generator.equals(senderId)) {
						((ABTDORetroAgent) this.myAgent).okMsgH.sendOkMessage(senderId);
					}
				}
			} else if (myValue == ngValue && generator.equals(senderId)) {
				this.myAgent.auxiliaryStatistics.get(Commands.nbObsoleteNogood).value++;
				((ABTDORetroAgent) this.myAgent).okMsgH.sendOkMessage(senderId);
			}
		}
	}

	/**
	 * 
	 * @param nogood
	 * @param receiverId
	 */
	public void sendNogood(Instantiation nogood, VarIdentifier generator, VarIdentifier target) {
		Message msg = this.createMessage();
		msg.addContents(Commands.NOGOOD, nogood.clone());
		msg.addContents(ABTDORetroAgent.Generator, generator);
		msg.setReceiverId(target);
		((ABTDORetroAgent) this.myAgent).storeStatisticsFor(msg);
		this.myAgent.sendMessage(msg);
	}

	/**
	 * @param nogood
	 * @return
	 */
	public boolean isCoherent(Instantiation nogood) {
		VarAssignment myAss = ((ABTDORetroAgent) this.myAgent).var.getVarAssignment();
		return this.myAgent.agentView.compatibleWith(nogood) && nogood.compatible(myAss);
	}

}
