package simple.dcsp.abtfamily.abtdoretro;

import java.util.HashMap;
import java.util.Iterator;

import kernel.AbstractMaster;
import kernel.MasterObserver;
import kernel.VarIdentifier;
import kernel.com.Message;
import util.Commands;
import util.HashList;
import util.Instantiation;

/**
 * termination detection (Silaghi et al. 2000)
 * 
 * @author redouane
 * 
 */
public class TerminationDetector extends MasterObserver {

	protected HashMap<VarIdentifier, Instantiation> acceptedMsg;
	protected HashMap<VarIdentifier, CurrentOrder> currentOrders = null;

	protected HashList<VarIdentifier> initialEndAgents = new HashList<VarIdentifier>();
	protected CurrentOrder bestOrder = null;

	public TerminationDetector() {
		super();
		this.acceptedMsg = new HashMap<VarIdentifier, Instantiation>();
		this.currentOrders = new HashMap<VarIdentifier, CurrentOrder>();
	}

	@Override
	public boolean handleReceivedMessage(Message msg) {
		if (msg.getType().equals(Commands.ORDER)) {
			this.ProcessOrder(msg);
			this.checkEnd();
		}
		if (msg.getType().equals(Commands.TERMINATION_DETECTION)) {
			Instantiation accepted = (Instantiation) msg.get(Commands.ACCEPTED);// Instantiation.fromString((String) msg.get(Commands.ACCEPTED),
			// null);
			CurrentOrder rcvOrder = (CurrentOrder) msg.get(Commands.ORDER); // CurrentOrder.fromString((String) msg.get(Commands.ORDER));
			VarIdentifier senderId = (VarIdentifier) msg.get(Commands.ID);
			this.acceptedMsg.remove(senderId);
			this.currentOrders.put(senderId, rcvOrder);
			this.acceptedMsg.put(senderId, accepted);
			this.setBestOrder(rcvOrder);
			this.deleteObsoleteInformations();
			this.checkEnd();
		}
		return true;
	}

	private void deleteObsoleteInformations() {
		Iterator<VarIdentifier> it = this.currentOrders.keySet().iterator();
		while (it.hasNext()) {
			VarIdentifier agId = it.next();
			CurrentOrder order = this.currentOrders.get(agId);
			int cmp = this.bestOrder.compare(order);
			int index = this.bestOrder.getOrder().indexOf(agId);
			if (cmp != 0 && Math.abs(cmp) <= index + 1) {
				it.remove();
				this.acceptedMsg.remove(agId);
			}
		}
	}

	private void ProcessOrder(Message msg) {
		CurrentOrder rcvOrder = (CurrentOrder) msg.get(Commands.ORDER); // CurrentOrder.fromString(""+msg.get(Commands.ORDER));
		this.setBestOrder(rcvOrder);
	}

	private HashList<VarIdentifier> computeInitialEndAgents() {
		this.initialEndAgents = new HashList<VarIdentifier>();
		for (VarIdentifier varId : this.bestOrder.getOrder()) {
			if (this.GammaPlus(varId).size() == 0) {
				this.initialEndAgents.add(varId);
			}
		}
		return this.initialEndAgents;
	}

	/**
	 * 
	 * @param agId
	 * @return
	 */
	private HashList<VarIdentifier> GammaPlus(VarIdentifier varId) {
		HashList<VarIdentifier> higherAgents = this.bestOrder.getOrder().getElementsBefor(varId);
		HashList<VarIdentifier> neighbours = this.master.getNeighbors(varId);
		return HashList.intersection(higherAgents, neighbours);
	}

	/**
	 * 
	 * @return
	 */
	private boolean checkEnd() {
		this.computeInitialEndAgents();
		if (!(this.currentOrders.keySet().containsAll(initialEndAgents) && this.acceptedMsg.keySet().containsAll(initialEndAgents))) {
			return false;
		}
		// if(Logger.LOG_DEBUG){
		// this.master.log("\nI check the end : ",1);
		// this.master.log("IEAgendts : "+this.initialEndAgents,1);
		// this.master.log("bestOrder : "+this.bestOrder,1);
		// this.master.log("acceptedMsg : "+this.acceptedMsg,1);
		// this.master.log("orders    : "+this.currentOrders+"\n",1);
		// }
		Instantiation solution = new Instantiation();
		for (VarIdentifier varId : this.initialEndAgents) {
			if (!this.currentOrders.containsKey(varId) || !this.acceptedMsg.containsKey(varId)) {
				// if(Logger.LOG_DEBUG){
				// this.master.log("==>  orders or acceptedMsg do not contains "+agId,1);
				// }
				return false;
			}
			CurrentOrder order = this.currentOrders.get(varId);
			int cmp = this.bestOrder.compare(order);
			if (cmp != 0 && Math.abs(cmp) <= this.bestOrder.getOrder().indexOf(varId) + 1) {
				// if(Logger.LOG_DEBUG){
				// this.master.log("==>  problem in orders \n"+agId+" => "+order +" and \n"+this.bestOrder,1);
				// }
				return false;
			}
			Instantiation accepted = this.acceptedMsg.get(varId);
			if (solution.compatible(accepted)) {
				solution.priorityMerge(accepted);
			} else {
				// if(Logger.LOG_DEBUG){
				// this.master.log("non compatible accepted "+solution+" \n"+accepted,1);
				// }
				return false;
			}
		}
		// if(Logger.LOG_DEBUG){
		// this.master.log("terminate with SUCCESS : "+solution, 1);
		// }
		this.master.setEndState(Commands.SUCCESS);
		this.master.broadcastSystemMessage(AbstractMaster.MasterVarId);
		return true;
	}

	@Override
	public void handleSendMessage(Message msg) {}

	public void setBestOrder(CurrentOrder currentOrder) {
		if (this.bestOrder == null) {
			this.bestOrder = currentOrder;
			// if(Logger.LOG_DEBUG){
			// this.master.log("bestOrder:\n"+this.bestOrder,1);
			// }
		} else {
			if (this.bestOrder.compare(currentOrder) < 0) {
				this.bestOrder = currentOrder;
				// if(Logger.LOG_DEBUG){
				// this.master.log("bestOrder:\n"+this.bestOrder,1);
				// }
			}
		}
	}

	/**
	 * 
	 * @param currentOrder
	 * @return <!--
	 * 
	 *         <pre>
	 * 		<b>(i+1)</b> if <i>this</i> order is the newest</br>
	 * 		<b>0</b> if this order and <i>currentOrder</i> are equal</br>
	 * 		<b>-(i+1)</b> if <i>currentOrder</i> is the newest</br>
	 * <!--
	 * </pre>
	 */
	public int compare(HashList<VarIdentifier> order1, HashMap<VarIdentifier, Integer> counters1, HashList<VarIdentifier> order2, HashMap<VarIdentifier, Integer> counters2) {
		for (int i = 0 ; i < order1.size() ; i++) {
			VarIdentifier agId1 = order1.get(i);
			int counter1 = counters1.get(agId1);
			VarIdentifier agId2 = order2.get(i);
			int counter2 = counters2.get(agId2);
			if (counter1 > counter2) {
				return (i + 1);
			} else {
				if (counter1 < counter2) {
					return -(i + 1);
				} else {
					if (agId1.compareTo(agId2) > 0) { return -(i + 1); }
					if (agId1.compareTo(agId2) < 0) { return (i + 1); }
				}
			}
		}
		return 0;
	}
}
