package simple.dcsp.abtfamily.abtdoretroz;

import java.util.HashMap;

import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.TerminateMsgHandler;
import kernel.events.AgentEvent;
import simple.SimpleAgent;
import simple.dcsp.abtfamily.IdleEvent;
import util.BooleanLock;
import util.Commands;
import util.HashList;
import util.Instantiation;
import util.Logger;
import util.VarAssignment;

public class ABTDOTermDetection extends TerminateMsgHandler {

	protected HashMap<VarIdentifier, Instantiation> acceptedMsg = null;
	protected HashMap<VarIdentifier, CurrentOrder> currentOrders = null;
	protected BooleanLock whenIdleLock = new BooleanLock(false);
	protected Instantiation prevSentAccepted = null;
	protected CurrentOrder prevSentOrder = null;
	protected CurrentOrder bestOrder = null;

	public ABTDOTermDetection(AbstractAgent agent) {
		super(agent);
		this.acceptedMsg = new HashMap<VarIdentifier, Instantiation>();
		this.currentOrders = new HashMap<VarIdentifier, CurrentOrder>();
	}

	/**
	 * 
	 */
	public boolean processTerminationDetection(Message msg) throws StopException {
		Instantiation accepted = (Instantiation) msg.get(Commands.ACCEPTED);
		// Instantiation.fromString((String) msg.get(Commands.ACCEPTED),
		// this.myAgent.agentOrder);
		CurrentOrder rcvOrder = (CurrentOrder) msg.get(Commands.ORDER);// CurrentOrder.fromString((String) msg.get(Commands.ORDER));
		VarIdentifier senderId = msg.getSenderId();
		this.currentOrders.put(senderId, rcvOrder);
		this.acceptedMsg.put(senderId, accepted);
		this.setBestOrder(rcvOrder);
		HashList<VarIdentifier> lowerNeighbors = this.myAgent.communicator.getLowerNeighbors();
		lowerNeighbors = HashList.intersection(lowerNeighbors, this.myAgent.communicator.getNeighbors0());
		if (this.currentOrders.size() == lowerNeighbors.size()) {
			this.tryToSendAccepted();
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.TerminateMsgHandler#handleAgentEvent(events.AgentEvent )
	 */

	@Override
	public void handleAgentEvent(AgentEvent agentEvent) {
		if (agentEvent instanceof IdleEvent && !this.myAgent.isFinished()) {
			this.tryToSendAccepted();
		}
		if (agentEvent instanceof kernel.events.AgentFinishedEvent) {
			this.askMasterToTerminate(true);
		}
	}

	/**
	 * 
	 */
	public void tryToSendAccepted() {
		CurrentOrder myCurrentOrder = ((ZABTDORetroAgent) this.myAgent).getCurrentOrder();
		this.myAgent.log(
				"I try to send an accepted msg \n"+
				"myCurrentOrder: "+myCurrentOrder+"\n"+
				"Orders   : "+this.currentOrders+"\n"+
				"Accepted : "+this.acceptedMsg+"\n"+
				"bestOrder: "+this.bestOrder, Logger.INFO_LEVEL);
		Instantiation accepted = this.allAcceptedMsgHaveBeenReceived();
		if (accepted != null) {
			if (this.ISendTheSame(accepted, myCurrentOrder)) {
				this.myAgent.log("I sent the same "+accepted+" ", Logger.INFO_LEVEL);
				return;
			}
			this.prevSentAccepted = accepted.clone();
			this.prevSentOrder = this.bestOrder.clone();
			HashList<VarIdentifier> higherNeighbors = this.myAgent.communicator.getHigherNeighbors();
			higherNeighbors = HashList.intersection(higherNeighbors, this.myAgent.communicator.getNeighbors0());
			if (higherNeighbors.size() == 0) { // am I an initial end agent
				this.sendAccepted(AbstractMaster.MasterVarId);
			} else {
				for (VarIdentifier higher : higherNeighbors) {
					this.sendAccepted(higher);
				}
			}
		}
	}

	/**
	 * 
	 * @param accepted
	 * @param myCurrentOrder
	 * @param myCurrentCounters
	 * @return
	 */
	private boolean ISendTheSame(Instantiation accepted, CurrentOrder currentOrder) {
		if (this.prevSentAccepted != null && this.prevSentAccepted.equals(accepted) && 
				this.prevSentOrder != null && this.prevSentOrder.equals(currentOrder)) { return true; }
		return false;
	}

	/**
	 * 
	 * @param receiverId
	 */
	protected void sendAccepted(VarIdentifier receiverId) {
		Message msg = null;
		msg = this.createMessage();
		msg.addContents(Commands.ACCEPTED, this.prevSentAccepted.clone());
		msg.addContents(Commands.ORDER, this.prevSentOrder.clone());
		msg.setReceiverId(receiverId);
		this.myAgent.sendMessage(msg);
	}

	protected synchronized Instantiation allAcceptedMsgHaveBeenReceived() {
		CurrentOrder currentOrder = ((ZABTDORetroAgent) this.myAgent).getCurrentOrder();
		this.setBestOrder(currentOrder);
		int cmp = this.bestOrder.compare(currentOrder);
		int myRank = this.myAgent.agentOrder.getMyRank();
		if (cmp != 0 && Math.abs(cmp) <= myRank + 1) {
			this.myAgent.log("my Order is not compatible with the best order", Logger.INFO_LEVEL);
			return null;
		}
		Instantiation myView = this.myAgent.agentView.getContextsOfHigherAgents();
		VarAssignment myAssign = ((SimpleAgent) this.myAgent).var.getVarAssignment();
		myView.add(myAssign);
		HashList<VarIdentifier> lowerNeighbors = this.myAgent.communicator.getLowerNeighbors();
		lowerNeighbors = HashList.intersection(lowerNeighbors, this.myAgent.communicator.getNeighbors0());
		for (VarIdentifier lower : lowerNeighbors) {
			if (!this.currentOrders.containsKey(lower) || !this.acceptedMsg.containsKey(lower)) {
				this.myAgent.log("I do not yet received and accepted msg from :"+lower+" ", Logger.INFO_LEVEL);
				return null;
			}
			cmp = this.bestOrder.compare(this.currentOrders.get(lower));
			int lowerRank = this.bestOrder.getOrder().indexOf(lower);
			if (cmp != 0 && Math.abs(cmp) <= lowerRank + 1) {
				this.myAgent.log("the rank of "+lower+" is not coherent with the best order", Logger.INFO_LEVEL);
				return null;
			}
			Instantiation view = this.acceptedMsg.get(lower);
			if (myView.compatible(view)) {
				myView.priorityMerge(view);
			} else {
				this.myAgent.log("The view of "+lower+" is not comatible :"+view, Logger.INFO_LEVEL);
				return null;
			}
		}
		return myView;
	}

	public void setBestOrder(CurrentOrder currentOrder) {
		if (this.bestOrder == null) {
			this.bestOrder = currentOrder;
		} else {
			if (this.bestOrder.compare(currentOrder) < 0) {
				this.bestOrder = currentOrder;
			}
		}
	}
}
