package simple.dcsp.abtfamily.abtdoretroz;

import static simple.dcsp.abtfamily.abtdoretroz.ZABTDORetroMaster.After_Second_Last;
import static simple.dcsp.abtfamily.abtdoretroz.ZABTDORetroMaster.Dom_Pos_All;
import static simple.dcsp.abtfamily.abtdoretroz.ZABTDORetroMaster.Dom_Pos_Single;
import static simple.dcsp.abtfamily.abtdoretroz.ZABTDORetroMaster.Highest;
import static simple.dcsp.abtfamily.abtdoretroz.ZABTDORetroMaster.MinDomain;

import java.io.Serializable;
import java.util.HashMap;
import kernel.VarIdentifier;

import util.HashList;

public class CurrentOrder implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1573387988364872830L;
	private HashList<VarIdentifier> order;
	private HashMap<VarIdentifier, Integer> counters;

	public CurrentOrder(HashList<VarIdentifier> order) {
		this.order = order;
		this.counters = new HashMap<VarIdentifier, Integer>();
		for (VarIdentifier varId : this.order) {
			this.counters.put(varId, 0);
		}
	}

	public CurrentOrder(HashList<VarIdentifier> order, HashMap<VarIdentifier, Integer> counters) {
		this.order = order;
		this.counters = counters;
	}

	@SuppressWarnings("unchecked")
	public CurrentOrder clone() {
		HashMap<VarIdentifier, Integer> counters = (HashMap<VarIdentifier, Integer>) this.counters.clone();
		return new CurrentOrder((HashList<VarIdentifier>) this.order.clone(), counters);
	}

	public HashList<VarIdentifier> getOrder() {
		return this.order;
	}

	public String toString() {
		String order = "[";
		for (VarIdentifier agId : this.order) {
			int counter = this.counters.get(agId);
			order += "(" + agId + "," + counter + ");";
		}
		order = order.substring(0, order.length() - 1);
		order += "]";
		return order;
	}

	public void setNewOrder(VarIdentifier myId, HashList<VarIdentifier> newOrder) {
		this.order = newOrder;
		HashList<VarIdentifier> lowerAgents = newOrder.getElementsAfter(myId);
		for (VarIdentifier lower : lowerAgents) {
			this.counters.put(lower, 0);
		}
		this.counters.put(myId, this.counters.get(myId) + 1);
	}

	public void updateCounters(VarIdentifier myId) {
		HashList<VarIdentifier> lowerAgents = this.order.getElementsAfter(myId);
		for (VarIdentifier lower : lowerAgents) {
			this.counters.put(lower, 0);
		}
		this.counters.put(myId, this.counters.get(myId) + 1);
	}

//	/**
//	 * 
//	 * @param currentOrder
//	 * @return <!--
//	 * 
//	 *         <pre>
//	 * 		<b>(i+1)</b> if <i>this</i> order is the newest</br>
//	 * 		<b>0</b> if this order and <i>currentOrder</i> are equal</br>
//	 * 		<b>-(i+1)</b> if <i>currentOrder</i> is the newest</br>
//	 * <!--
//	 * </pre>
//	 */
//	public int compare(CurrentOrder currentOrder) {
//		for (int i = 0 ; i < this.order.size() ; i++) {
//			int agId1 = this.order.get(i);
//			int counter1 = this.counters.get(agId1);
//			int agId2 = currentOrder.order.get(i);
//			int counter2 = currentOrder.counters.get(agId2);
//			if (counter1 > counter2) {
//				return (i + 1);
//			} else {
//				if (counter1 < counter2) {
//					return -(i + 1);
//				} else {
//					if (agId1 > agId2) { return -(i + 1); }
//					if (agId1 < agId2) { return (i + 1); }
//				}
//			}
//		}
//		return 0;
//	}
	/**
	 * 
	 * @param currentOrder
	 * @return <b>(i+1)</b> if <code>this</code> order is the newest</br> <b>0</b> if this order and <code>currentOrder</code> are equal</br>
	 *         <b>-(i+1)</b> if <code>currentOrder</code> is the newest</br>
	 */
	public int compare(CurrentOrder currentOrder) {
		for (int i = 0 ; i < this.order.size() ; i++) {
			VarIdentifier agId1 = this.order.get(i);
			int counter1 = this.counters.get(agId1);
			VarIdentifier agId2 = currentOrder.order.get(i);
			int counter2 = currentOrder.counters.get(agId2);
			if (counter1 > counter2) { return (i + 1); }
			if (counter1 < counter2) { return -(i + 1); }
		}
		return this.getOrder().compare(currentOrder.getOrder());
	}
	
	public static VarIdentifier[] sort(HashList<VarIdentifier> agents, HashMap<VarIdentifier, Integer> domainSizes) {
		VarIdentifier[] ans = new VarIdentifier[agents.size()];
		for (int i = 0 ; i < ans.length ; i++) {
			VarIdentifier minId = agents.get(0);
			int minSize = domainSizes.get(minId);
			for (VarIdentifier agId : agents) {
				if (minSize > domainSizes.get(agId)) {
					minSize = domainSizes.get(agId);
					minId = agId;
				}
			}
			ans[i] = minId;
			agents.remove(minId);
		}
		return ans;
	}

	public boolean equals(Object obj) {
		CurrentOrder currentOrder = (CurrentOrder) obj;
		return (currentOrder.order.equals(this.order) && this.compare(currentOrder) == 0);
	}

	public HashMap<VarIdentifier, Integer> getCounters() {
		return this.counters;
	}

	/**
	 * 
	 * @param heuristic
	 * @param myId
	 * @param lower
	 * @param lower2
	 * @param D
	 * @return
	 */
	public CurrentOrder getNewOrder(short heuristic, VarIdentifier agentId, VarIdentifier target, VarIdentifier secondlower, HashMap<VarIdentifier, Integer> domainSizes) {
		switch (heuristic) {
			case MinDomain :
				return this.getMinDomainOrder(agentId, target, secondlower, domainSizes);
			case After_Second_Last :
				return this.getAfterOrder(agentId, secondlower);
			case Dom_Pos_Single :
				return this.getPositionSingleOrder(agentId, target, secondlower, domainSizes);
			case Dom_Pos_All :
				return this.getPositionAllOrder(agentId, target, secondlower, domainSizes);
			case Highest :
				return this.getHighestOrder(agentId, target, secondlower, domainSizes);
		}
		return null;
	}

	private CurrentOrder getHighestOrder(VarIdentifier agentId, VarIdentifier target, VarIdentifier secondlower, HashMap<VarIdentifier, Integer> domainSizes) {
		// TODO Auto-generated method stub
		return null;
	}

	private CurrentOrder getPositionAllOrder(VarIdentifier agentId, VarIdentifier target, VarIdentifier secondlower, HashMap<VarIdentifier, Integer> domainSizes) {
		// TODO Auto-generated method stub
		return null;
	}

	private CurrentOrder getPositionSingleOrder(VarIdentifier agentId, VarIdentifier target, VarIdentifier secondlower, HashMap<VarIdentifier, Integer> domainSizes) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * 
	 * @param myId
	 * @param agId
	 * @return
	 */
	public CurrentOrder getAfterOrder(VarIdentifier myId, VarIdentifier agId) {
		CurrentOrder newOrder = this.clone();
		int myRank = newOrder.order.indexOf(myId);
		int agRank = newOrder.order.indexOf(agId);
		VarIdentifier afterRank = newOrder.order.get(agRank + 1);
		int myNewCounter = newOrder.counters.get(afterRank) + 1;
		HashList<VarIdentifier> lowerAgents = newOrder.order.getElementsAfter(myId);
		for (VarIdentifier lower : lowerAgents) {
			newOrder.counters.put(lower, 0);
		}
		for (int i = myRank ; i > agRank + 1 ; i--) {
			VarIdentifier id = newOrder.order.get(i - 1);
			newOrder.order.add(i, id);
			newOrder.counters.put(id, 0);
		}
		newOrder.order.add(agRank + 1, myId);
		newOrder.counters.put(myId, myNewCounter);
		return newOrder;
	}

	/**
	 * 
	 * @param myId
	 * @param lower
	 * @param secondlower
	 * @param D
	 * @return
	 */
	private CurrentOrder getMinDomainOrder(VarIdentifier myId, VarIdentifier lower, VarIdentifier secondlower, HashMap<VarIdentifier, Integer> D) {
		CurrentOrder newOrder = this.getAfterOrder(myId, lower);
		int secondRank = newOrder.order.indexOf(secondlower);
		int myRank = newOrder.order.indexOf(myId);
		int myD = D.get(myId);
		for (int i = myRank ; i > secondRank + 1 ; i--) {
			VarIdentifier agId = newOrder.order.get(i - 1);
			int agD = D.get(agId);
			int agCounter = this.counters.get(agId);
			if (myD < agD) {
				newOrder.order.add(i, agId);
				newOrder.counters.put(agId, 0);
				newOrder.order.add(i - 1, myId);
				newOrder.counters.put(myId, agCounter + 1);
			} else {
				break;
			}
		}
		return newOrder;

	}
	
}
