package simple.dcsp.abtfamily.abtdoretroz;

import java.util.HashMap;
import java.util.Iterator;
import kernel.AbstractMaster;
import kernel.MasterObserver;
import kernel.VarIdentifier;
import kernel.com.Message;
import util.Commands;
import util.HashList;
import util.Instantiation;
import util.Logger;

/**
 * termination detection (Silaghi et al. 2000)
 * 
 * @author redouane
 * 
 */
public class TerminationDetector extends MasterObserver {

	protected HashMap<VarIdentifier, Instantiation> acceptedMsg;
	protected HashMap<VarIdentifier, CurrentOrder> currentOrders = null;

	protected HashList<VarIdentifier> initialEndAgents = new HashList<VarIdentifier>();
	protected CurrentOrder bestOrder = null;

	public TerminationDetector() {
		super();
		this.acceptedMsg = new HashMap<VarIdentifier, Instantiation>();
		this.currentOrders = new HashMap<VarIdentifier, CurrentOrder>();
	}

	@Override
	public boolean handleReceivedMessage(Message msg) {
		if (msg.getType().equals(Commands.ORDER)) {
			this.ProcessOrder(msg);
			this.checkEnd();
		}
		if (msg.getType().equals(Commands.TERMINATION_DETECTION)) {
			Instantiation accepted = (Instantiation) msg.get(Commands.ACCEPTED);
			CurrentOrder rcvOrder = (CurrentOrder) msg.get(Commands.ORDER);
			VarIdentifier senderId = (VarIdentifier) msg.get(Commands.ID);
			this.acceptedMsg.remove(senderId);
			this.currentOrders.put(senderId, rcvOrder);
			this.acceptedMsg.put(senderId, accepted);
			this.setBestOrder(rcvOrder);
			this.deleteObsoleteInformations();
			this.checkEnd();
		}
		return true;
	}

	private void deleteObsoleteInformations() {
		Iterator<VarIdentifier> it = this.currentOrders.keySet().iterator();
		while (it.hasNext()) {
			VarIdentifier agId = it.next();
			CurrentOrder order = this.currentOrders.get(agId);
			int cmp = this.bestOrder.compare(order);
			int index = this.bestOrder.getOrder().indexOf(agId);
			if (cmp != 0 && Math.abs(cmp) <= index + 1) {
				it.remove();
				this.acceptedMsg.remove(agId);
			}
		}
	}

	private void ProcessOrder(Message msg) {
		CurrentOrder rcvOrder = (CurrentOrder) msg.get(Commands.ORDER); // CurrentOrder.fromString(""+msg.get(Commands.ORDER));
		this.setBestOrder(rcvOrder);
	}

	private HashList<VarIdentifier> computeInitialEndAgents() {
		this.initialEndAgents = new HashList<VarIdentifier>();
		for(VarIdentifier agId : this.bestOrder.getOrder()) {
			if (this.GammaPlus(agId).size() == 0) {
				this.initialEndAgents.add(agId);
			}
		}
		this.master.log("Roots agents : " + this.initialEndAgents + " ", Logger.INFO_LEVEL);
		return this.initialEndAgents;
	}

	/**
	 * 
	 * @param agId
	 * @return
	 */
	private HashList<VarIdentifier> GammaPlus(VarIdentifier agId) {
		HashList<VarIdentifier> higherAgents = this.bestOrder.getOrder().getElementsBefor(agId);
		HashList<VarIdentifier> neighbours = this.master.getNeighbors(agId);
		return HashList.intersection(higherAgents, neighbours);
	}

	/**
	 * 
	 * @return
	 */
	private boolean checkEnd() {
		this.computeInitialEndAgents();
		if (!(this.currentOrders.keySet().containsAll(initialEndAgents) && this.acceptedMsg.keySet().containsAll(initialEndAgents))) {
			return false;
		}
		this.master.log("I check the end\n" + "Roots agents : " + this.initialEndAgents + "\n" + "bestOrder : " + this.bestOrder + "acceptedMsg : "
				+ this.acceptedMsg + "orders    : " + this.currentOrders, Logger.INFO_LEVEL);

		Instantiation solution = new Instantiation();
		for(VarIdentifier agId : this.initialEndAgents) {
			if (!this.currentOrders.containsKey(agId) || !this.acceptedMsg.containsKey(agId)) {
				this.master.log("orders or acceptedMsg do not contains " + agId, Logger.INFO_LEVEL);
				return false;
			}
			CurrentOrder order = this.currentOrders.get(agId);
			int cmp = this.bestOrder.compare(order);
			if (cmp != 0 && Math.abs(cmp) <= this.bestOrder.getOrder().indexOf(agId) + 1) {
				this.master.log("problem in orders \n" + agId + " => " + order + " and \n" + this.bestOrder, Logger.INFO_LEVEL);
				return false;
			}
			Instantiation accepted = this.acceptedMsg.get(agId);
			if (solution.compatible(accepted)) {
				solution.priorityMerge(accepted);
			} else {
				this.master.log("non compatible accepted " + solution + " \n" + accepted, Logger.INFO_LEVEL);
				return false;
			}
		}
		this.master.log("terminate with SUCCESS : " + solution, Logger.INFO_LEVEL);
		this.master.setEndState(Commands.SUCCESS);
		this.master.broadcastSystemMessage(AbstractMaster.MasterVarId);
		return true;
	}

	@Override
	public void handleSendMessage(Message msg) {}

	public void setBestOrder(CurrentOrder currentOrder) {
		if (this.bestOrder == null) {
			this.bestOrder = currentOrder;
			this.master.log("The best order " + currentOrder + " ", Logger.INFO_LEVEL);
		} else {
			if (this.bestOrder.compare(currentOrder) < 0) {
				this.bestOrder = currentOrder;
				this.master.log("The best order " + currentOrder + " ", Logger.INFO_LEVEL);
			}
		}
	}

}
