package simple.dcsp.abtfamily.abtdoretroz;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.MasterObserver;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.order.AgentComplexity;
import kernel.order.LexicographicAO;
import simulation.parser.PVariable;
import util.Commands;
import util.HashList;

public class ZABTDORetroMaster extends AbstractMaster {

	public final static short MinDomain = 1;
	public final static short After_Second_Last = 2;
	public final static short Dom_Pos_Single = 3;
	public final static short Dom_Pos_All = 4;
	public final static short Highest = 5;

	protected short heuristic = MinDomain;
	protected int K = 0;

	public long nbObsoleteOk = 0L;
	public long nbObsoleteNogood = 0L;
	public long nbAssignments = 0L;
	public long changeOrder = 0L;
	public long nbIncompatibleNogood = 0L;
	public int nbRcvdStatistics = 0;

	public ZABTDORetroMaster(short heuristic) {
		this.setCentralizedAO(new LexicographicAO());
		this.addObserver(new TerminationDetector());
		this.heuristic = heuristic;
	}

	public ZABTDORetroMaster(short heuristic, int K) {
		this.setCentralizedAO(new LexicographicAO());
		this.addObserver(new TerminationDetector());
		this.heuristic = heuristic;
		this.K = K;
	}

	public AbstractAgent makeAgent(int agId, String name, String desc) {
		return new ZABTDORetroAgent(agId, name, desc, this.K, this.heuristic);
	}

	/* (non-Javadoc)
	 * @see kernel.AbstractMaster#makeAgent(simulation.parser.PVariable)
	 */
	@Override
	public AbstractAgent makeAgent(PVariable variable) {
		return new ZABTDORetroAgent(variable, this.K, this.heuristic);
	}

	public void broadcast(Message msg, VarIdentifier agentId) {
		if (msg.getType().equals(Commands.AO)) {
			HashMap<VarIdentifier, AgentComplexity> ac = this.centralizedAO.getAgentsComplexity();
			HashList<VarIdentifier> ao = null;
			try {
				ao = HashList.fromString((String) msg.get(Commands.ORDER), VarIdentifier.class);
			} catch (Exception e) {
				e.printStackTrace();
			}
			for (VarIdentifier agId : ao) {
				msg.addContents(agId + "", ac.get(agId).getMaxDomain());
			}
		}
		super.broadcast(msg, agentId);
	}

	public HashList<VarIdentifier> getOrder() {
		for (MasterObserver observer : this.observers) {
			if (observer instanceof TerminationDetector) { 
				return ((TerminationDetector) observer).bestOrder.getOrder(); 
			}
		}
		return new HashList<VarIdentifier>();
	}

	/* (non-Javadoc)
	 * @see kernel.AbstractMaster#getTypesOfMessages()
	 */
	@Override
	public Collection<String> getTypesOfMessages() {
		Collection<String> ans = new ArrayList<String>();
		ans.add(Commands.OK);
		ans.add(Commands.NOGOOD);
		ans.add(Commands.ADDLINK);
		ans.add(Commands.ORDER);
		return ans;
	}

}
