package simple.dcsp.afcfamily.afc.domdeg;

import java.util.HashMap;
import java.util.Iterator;
import kernel.AbstractAgent;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.MessageHandler;
import kernel.order.AgentOrdering;
import simple.dcsp.afcfamily.afc.AFCAgent;
import util.Commands;
import util.HashList;
import util.Instantiation;

public class AFCDomDegAgent extends AFCAgent {

	long myCounter = 0;
	HashMap<VarIdentifier, Long> counters;
	HashMap<VarIdentifier, Integer> domainSizes;

	OrderMsgHandler orderMsgH = null;

	private int heuristic = AFCMinDomMaster.Dom_Dyn_Deg;

	public AFCDomDegAgent(int agentId, String agName, String description, int heuristic) {
		super(agentId, agName, description);
		this.domainSizes = new HashMap<VarIdentifier, Integer>();
		this.counters = new HashMap<VarIdentifier, Long>();
		this.orderMsgH = new OrderMsgHandler(this);
		this.agentOrder = new AFCDomDegAO(this);
		this.heuristic = heuristic;
	}

	// ==========================================================================
	protected void receiveCPA(Instantiation cpa, long sc, boolean fromBackMsg) throws StopException {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(cpa, !fromBackMsg)) {
				this.agentView.clearExcept(cpa);
				// this.agentView.priorityMerge(notOk);
				this.agentViewConsistent = true;
			}
		}
		super.receiveCPA(cpa, sc, fromBackMsg);
	}

	// ==========================================================================
	public void forwardCheck(Instantiation cpa, long sc) {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(cpa, false)) {
				this.agentView.clearExcept(cpa);
				// this.agentView.priorityMerge(cpa);
				this.agentViewConsistent = true;
			}
			int currDomainSize = this.var.getDomainSize();
			super.forwardCheck(cpa, sc);
			if (this.agentViewConsistent && this.var.getDomainSize() != currDomainSize)
				this.orderMsgH.sendDomainSizeMsgs(cpa);
		}
	}

	// ==========================================================================
	public void processNotOk(Instantiation notOk, long sc) {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(notOk, false)) {
				this.agentView.clearExcept(notOk);
			}
		}
		super.processNotOk(notOk, sc);
	}

	// ==========================================================================
	public boolean updateAgentOrdering(Instantiation cpa, boolean ImNext) {
		HashList<VarIdentifier> newOrder = cpa.getVarIds();
		if (ImNext)
			newOrder.add(this.getVarIdentifier());
		HashList<VarIdentifier> others = this.agentOrder.getOrder();
		others.removeAll(newOrder);
		newOrder.addAll(others);
		if (this.agentOrder.setNewOrder(newOrder)) {
			this.agentView.ensureOnlyHigherAgentsInView();
			this.constraintHandler.update();
			return true;
		}
		return false;

	}

	// ================================================================================
	// ===============================================================================
	public boolean isMinDom() {
		return this.heuristic == AFCDomDegMaster.MinDom;
	}

	public boolean isDomStaticDeg() {
		return this.heuristic == AFCDomDegMaster.Dom_Static_Deg;
	}

	public boolean isDomDynDeg() {
		return this.heuristic == AFCDomDegMaster.Dom_Dyn_Deg;
	}

	// ===============================================================================

	public VarIdentifier getNextAgent() {
		HashList<VarIdentifier> assigned = this.agentView.toInstantiation().getVarIds();
		assigned.add(this.getVarIdentifier());
		HashList<VarIdentifier> unassigned = this.agentOrder.getOrder();
		unassigned.removeAll(assigned);
		if (unassigned.size() == 0) {
			return null;
		}
		if (this.isMinDom()) {
			return sortMinDom(unassigned)[0];
		} else {
			return sortMinDomDeg(unassigned)[0];
		}
	}

	// ===============================================================================
	public VarIdentifier[] sortMinDom(HashList<VarIdentifier> unassigned) {
		VarIdentifier[] ans = new VarIdentifier[unassigned.size()];
		for(int i = 0 ; i < ans.length ; i++) {
			VarIdentifier minId = unassigned.get(0);
			int minSize = domainSizes.get(minId);
			for(VarIdentifier agId : unassigned) {
				if (minSize > domainSizes.get(agId)) {
					minSize = domainSizes.get(agId);
					minId = agId;
				}
			}
			ans[i] = minId;
			unassigned.remove(minId);
		}
		return ans;
	}

	public VarIdentifier[] sortMinDomDeg(HashList<VarIdentifier> unassigned) {
		VarIdentifier[] ans = new VarIdentifier[unassigned.size()];
		double minDomDeg, domDeg;
		for(int i = 0 ; i < ans.length ; i++) {
			VarIdentifier minId = unassigned.get(0);
			int deg = deg(minId, unassigned);
			if (deg == 0) {
				minDomDeg = Double.POSITIVE_INFINITY;
			} else {
				minDomDeg = ((double) domainSizes.get(minId)) / deg;
			}
			for(VarIdentifier agId : unassigned) {
				deg = deg(agId, unassigned);
				if (deg == 0) {
					domDeg = Double.POSITIVE_INFINITY;
				} else {
					domDeg = ((double) domainSizes.get(agId)) / deg;
				}
				if (minDomDeg > domDeg) {
					minDomDeg = domDeg;
					minId = agId;
				}
			}
			ans[i] = minId;
			unassigned.remove(minId);
		}
		return ans;
	}

	private int deg(VarIdentifier agId, HashList<VarIdentifier> unassigned) {
		if (this.isDomStaticDeg()) {
			return this.agentOrder.getAgentsGraph().getNeighbors(agId).size();
		}
		Iterator<VarIdentifier> it = this.agentOrder.getNeighborsOf(agId).iterator();
		int deg = 0;
		while (it.hasNext()) {
			if (unassigned.contains(it.next())) {
				deg++;
			}
		}
		return deg;
	}

	// ==============================================================================

	public class OrderMsgHandler extends MessageHandler {

		public OrderMsgHandler(AbstractAgent agent) {
			super(agent, Commands.ORDER);
		}

		public void processMessage(Message msg) throws StopException {
			VarIdentifier senderId = msg.getSenderId();
			int domSize = (Integer) msg.get(Commands.DomainSize);
			long counter = (Long) msg.get(Commands.COUNTER);
			if (counters.containsKey(senderId)) {
				if (counters.get(senderId) <= counter) {
					counters.put(senderId, counter);
					domainSizes.put(senderId, domSize);
				}
			} else {
				counters.put(senderId, counter);
				domainSizes.put(senderId, domSize);
			}
		}

		public void sendDomainSizeMsgs(Instantiation cpa) {
			HashList<VarIdentifier> unassigned = this.myAgent.agentOrder.getOrder();
			unassigned.removeAll(cpa.getVarIds());
			unassigned.remove(getVarIdentifier());
			for(VarIdentifier varId : unassigned) {
				this.sendDomainSize(varId);
			}
		}

		public void sendDomainSize(VarIdentifier agId) {
			myCounter++;
			int domSize = getMyVar().getDomainSize();
			Message msg = createMessage();
			msg.addContents(Commands.DomainSize, domSize);
			msg.addContents(Commands.COUNTER, myCounter);
			msg.setReceiverId(agId);
			sendMessage(msg);
		}

	}

	// ==========================================================================================
	public class AFCDomDegAO extends AgentOrdering {

		public AFCDomDegAO(AbstractAgent agent) {
			super(agent);
		}

		public void processOrder(Message msg) {
			super.processOrder(msg);
			for(VarIdentifier agId : this.agentsOrder) {
				domainSizes.put(agId, (Integer) msg.get(agId + ""));
				counters.put(agId, 0L);
			}
		}
	}

}
