package simple.dcsp.afcfamily.afc.estimDom;

import java.util.HashMap;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.events.VariableEvent;
import kernel.order.AgentOrdering;
import kernel.vars.integer.IntExternalVar;
import simple.SimpleAgent;
import simple.dcsp.afcfamily.afc.AFCAgent;
import util.Commands;
import util.HashList;
import util.Instantiation;
import util.VarEventMask;

public class AFCEstimatedAgent extends AFCAgent {

	private static final short MinDom = 1;
	private static final short MinDomDeg = 2;

	private short type;

	long myCounter = 0;
	HashMap<VarIdentifier, Integer> lbDomais;
	HashMap<VarIdentifier, Integer> ubDomais;

	HashMap<VarIdentifier, Integer> LB;
	HashMap<VarIdentifier, Integer> UB;

	public AFCEstimatedAgent(int agentId, String agName, String description, short type) {
		super(agentId, agName, description);
		this.type = type;
		this.lbDomais = new HashMap<VarIdentifier, Integer>();
		this.ubDomais = new HashMap<VarIdentifier, Integer>();
		this.cpaMsgH = new EstCPAMsgHandler(this);
		this.agentOrder = new AgentOrdering(this) {
			public void processOrder(Message msg) {
				super.processOrder(msg);
				for (VarIdentifier varId : this.agentsOrder) {
					lbDomais.put(varId, (Integer) msg.get(varId + ""));
					ubDomais.put(varId, (Integer) msg.get(varId + ""));
				}
			}
		};
	}

	@Override
	public void configurePropagationEngine() {
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_FIRST).add(VariableEvent.INSTANCIATE);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_FIRST).add(VariableEvent.DESINSTANCIATE);
		super.configurePropagationEngine();
	}

	// ==========================================================================

	public void sendCPA(Instantiation CPA) throws StopException {
		this.computeBounds(CPA);
		super.sendCPA(CPA);
	}

	// ==========================================================================
	protected void receiveCPA(Instantiation cpa, long sc, boolean fromBackMsg) throws StopException {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(cpa, !fromBackMsg)) {
				this.agentView.clearExcept(cpa);
				// this.agentView.priorityMerge(notOk);
				this.agentViewConsistent = true;
			}
		}
		super.receiveCPA(cpa, sc, fromBackMsg);
	}

	// ==========================================================================
	public void forwardCheck(Instantiation cpa, long sc) {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(cpa, false)) {
				this.agentView.clearExcept(cpa);
				// this.agentView.priorityMerge(cpa);
				this.agentViewConsistent = true;
			}
			super.forwardCheck(cpa, sc);
		}
	}

	// ==========================================================================
	public void processNotOk(Instantiation notOk, long sc) {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(notOk, false)) {
				this.agentView.clearExcept(notOk);
			}
		}
		super.processNotOk(notOk, sc);
	}

	// ==========================================================================
	public boolean updateAgentOrdering(Instantiation cpa, boolean ImNext) {
		HashList<VarIdentifier> newOrder = cpa.getVarIds();
		if (ImNext)
			newOrder.add(this.getVarIdentifier());
		HashList<VarIdentifier> others = this.agentOrder.getOrder();
		others.removeAll(newOrder);
		newOrder.addAll(others);
		if (this.agentOrder.setNewOrder(newOrder)) {
			this.agentView.ensureOnlyHigherAgentsInView();
			this.constraintHandler.update();
			return true;
		}
		return false;

	}

	// ==========================================================================

	protected void computeBounds(Instantiation cpa) {
		this.LB = new HashMap<VarIdentifier, Integer>();
		this.UB = new HashMap<VarIdentifier, Integer>();
		HashList<VarIdentifier> unassigned = this.agentOrder.getOrder();
		unassigned.removeAll(cpa.getVarIds());
		for (VarIdentifier varId : unassigned) {
			IntExternalVar exV = (IntExternalVar) this.getExternalVar(varId);
			if (exV != null && this.communicator.isNeighbor(varId)) {
				int ub = this.ubDomais.get(varId);
				UB.put(varId, Math.min(ub, exV.getDomainSize()));
				int conflict = exV.getOriginalDomainSize() - exV.getDomainSize();
				int lb = this.lbDomais.get(varId) - conflict;
				LB.put(varId, Math.min(exV.getDomainSize(), lb));

			} else {
				LB.put(varId, this.lbDomais.get(varId));
				UB.put(varId, this.ubDomais.get(varId));
			}
		}
	}

	// ====================================================================================
	public VarIdentifier getNextAgent() {
		switch (this.type) {
			case MinDom :
				return this.getMinDomainNextAgent();
			case MinDomDeg :
				return this.getDomDegNextAgent();
		}
		return null;
	}

	/**
	 * @param bounds
	 * @return next agent to assign
	 */
	public VarIdentifier getMinDomainNextAgent() {
		if (UB.isEmpty())
			return null;
		for (VarIdentifier varId : UB.keySet()) {
			if (UB.get(varId) == 0) {
				return varId;
			} else {
				boolean selected = true;
				for (VarIdentifier id : LB.keySet()) {
					if (!varId.equals(id)) {
						if (UB.get(varId) > LB.get(id)) {
							selected = false;
							break;
						}
					}
				}
				if (selected)
					return varId;
			}
		}
		VarIdentifier IdOfMinLb = null;
		int minLb = Integer.MAX_VALUE;
		for (VarIdentifier varId : LB.keySet()) {
			if (LB.get(varId) < minLb) {
				IdOfMinLb = varId;
				minLb = LB.get(varId);
			}
		}
		return IdOfMinLb;
	}

	// ===============================================================================
	/**
	 * @param bounds
	 * @return next agent to assign
	 */
	public VarIdentifier getDomDegNextAgent() {
		HashList<VarIdentifier> assigned = this.agentView.toInstantiation().getVarIds();
		assigned.add(this.getVarIdentifier());
		HashList<VarIdentifier> unassigned = this.agentOrder.getOrder();
		unassigned.removeAll(assigned);
		if (unassigned.size() == 0 || UB.isEmpty())
			return null;
		HashMap<VarIdentifier, Integer> degrees = this.getDegrees(unassigned);

		for (VarIdentifier agId : UB.keySet()) {
			int deg = degrees.get(agId);
			double upperBound = UB.get(agId);
			if (upperBound == 0) {
				return agId;
			} else {
				if (deg == 0)
					upperBound = Double.POSITIVE_INFINITY;
				else
					upperBound /= deg;
				boolean selected = true;
				for (VarIdentifier id : LB.keySet()) {
					int degree = degrees.get(id);
					double lowerBound = LB.get(id);
					if (degree == 0)
						lowerBound = Double.POSITIVE_INFINITY;
					else
						lowerBound /= degree;
					if (!agId.equals(id)) {
						if (upperBound > lowerBound) {
							selected = false;
							break;
						}
					}
				}
				if (selected) { return agId; }
			}
		}
		VarIdentifier IdOfMinLb = null;
		double minLb = Double.POSITIVE_INFINITY;
		for (VarIdentifier agId : LB.keySet()) {
			int deg = degrees.get(agId);
			double lowerBound = LB.get(agId);
			if (deg == 0)
				lowerBound = Double.POSITIVE_INFINITY;
			else
				lowerBound /= deg;
			if (lowerBound < minLb) {
				IdOfMinLb = agId;
				minLb = LB.get(agId);
			}
		}
		return IdOfMinLb;
	}

	// ===============================================================================
	private HashMap<VarIdentifier, Integer> getDegrees(HashList<VarIdentifier> unassigned) {
		HashMap<VarIdentifier, Integer> map = new HashMap<VarIdentifier, Integer>();
		for (VarIdentifier varId : UB.keySet()) {
			map.put(varId, ddegree(varId, unassigned));
		}
		return map;
	}

	// ===============================================================================
	private int ddegree(VarIdentifier agId, HashList<VarIdentifier> unassigned) {
		int deg = 0;
		Iterable<VarIdentifier> neighbors = this.agentOrder.getAgentsGraph().getNeighborsData(agId);
		for (VarIdentifier id : neighbors)
			if (unassigned.contains(id)) {
				deg++;
			}
		return deg;
	}

	// ===============================================================================

	public class EstCPAMsgHandler extends CPAMsgHandler {

		public EstCPAMsgHandler(SimpleAgent algo) {
			super(algo);
		}

		@SuppressWarnings("unchecked")
		public void processMessage(Message msg) throws StopException {
			HashMap<VarIdentifier, Integer> lbDomais = (HashMap<VarIdentifier, Integer>) msg.get("LB");
			HashMap<VarIdentifier, Integer> ubDomais = (HashMap<VarIdentifier, Integer>) msg.get("UB");
			lbDomais.putAll(lbDomais);
			ubDomais.putAll(ubDomais);
			super.processMessage(msg);
		}

		public void sendCPAMessage(Instantiation cpa, VarIdentifier receiverId, long sc) {
			Message msg = createMessage();
			msg.addContents(Commands.CPA, cpa.clone());
			msg.addContents("SC", sc);
			msg.addContents("LB", LB);
			msg.addContents("UB", UB);
			msg.setReceiverId(receiverId);
			this.myAgent.sendMessage(msg);
		}
	}
}
