package simple.dcsp.afcfamily.afc.triggerednogood;

import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import simple.dcsp.afcfamily.afc.AFCAgent;
import util.HashList;
import util.Instantiation;

public class AFCTriggeredNogoodAgent extends AFCAgent {

	protected VarIdentifier lastBackTriggeredId = null;

	public AFCTriggeredNogoodAgent(int agentId, String agName, String description) {
		super(agentId, agName, description);
		this.backCPAMsgH = new BtBackCPAMsgHandler(this);
	}

	// ==========================================================================

	public VarIdentifier getNextAgent() {
		if (this.lastBackTriggeredId != null) { // && this.agentOrder.isLower(lastBackTriggeredId)){
			return this.lastBackTriggeredId;
		}
		return this.agentOrder.getNextOf(this.getVarIdentifier());
	}

	// ==========================================================================
	protected void receiveCPA(Instantiation cpa, long sc, boolean fromBackMsg) throws StopException {
		if (sc > this.stepCounter) {
			this.lastBackTriggeredId = null;
			if (this.updateAgentOrdering(cpa, !fromBackMsg)) {
				this.agentView.clearExcept(cpa);
				// this.agentView.priorityMerge(notOk);
				this.agentViewConsistent = true;
			}
		}
		super.receiveCPA(cpa, sc, fromBackMsg);
	}

	// ==========================================================================
	public void forwardCheck(Instantiation cpa, long sc) {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(cpa, false)) {
				this.agentView.clearExcept(cpa);
				// this.agentView.priorityMerge(cpa);
				this.agentViewConsistent = true;
			}
			super.forwardCheck(cpa, sc);
		}
	}

	// ==========================================================================
	public void processNotOk(Instantiation notOk, long sc) {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(notOk, false)) {
				this.agentView.clearExcept(notOk);
			}
		}
		super.processNotOk(notOk, sc);
	}

	// ==========================================================================
	public boolean updateAgentOrdering(Instantiation cpa, boolean ImNext) {
		HashList<VarIdentifier> newOrder = cpa.getVarIds();
		if (ImNext) {
			newOrder.add(this.getVarIdentifier());
		}
		HashList<VarIdentifier> others = this.agentOrder.getOrder();
		others.removeAll(newOrder);
		newOrder.addAll(others);
		if (this.agentOrder.setNewOrder(newOrder)) {
			this.agentView.ensureOnlyHigherAgentsInView();
			this.constraintHandler.update();
			return true;
		}
		return false;

	}

	// =======================================================================
	public class BtBackCPAMsgHandler extends BackCPAMsgHandler {

		public BtBackCPAMsgHandler(AFCTriggeredNogoodAgent agent) {
			super(agent);
		}

		@Override
		public void processMessage(Message msg) throws StopException {
			lastBackTriggeredId = msg.getSenderId();
			super.processMessage(msg);
		}
	}
	// ===================================================================================
}
