package simple.dcsp.afcfamily.afcng.afctree;

import kernel.StopException;
import kernel.VarIdentifier;
import kernel.order.TreeDisAO;
import kernel.vars.integer.IntExternalVar;
import simple.dcsp.afcfamily.afcng.AFCngAgent;
import util.HashList;
import util.Instantiation;
import util.VarAssignment;

public class AFCTreeAgent extends AFCngAgent {

	private int myLevel;

	public AFCTreeAgent(int agentId, String agName, String description) {
		super(agentId, agName, description);
		this.terminateMsgHandler = new AFCTreeTermMsgHandler(this);
		this.agentOrder = new TreeDisAO(this);
	}
	
	/* (non-Javadoc)
	 * @see kernel.AbstractAgent#initAgentView()
	 */
	@Override
	public void initAgentView() {
		HashList<VarIdentifier> agents = ((TreeDisAO) this.agentOrder).getAncestors();
		for (VarIdentifier varId : agents) {
			VarAssignment var = new VarAssignment(varId, Integer.MIN_VALUE);
			IntExternalVar extVar = this.makeExternalVar(var);
			this.agentView.add(extVar);
		}
		this.myLevel = agents.size();
	}

	/**
	 * This method from {@link kernel.AbstractAgent} starts the distributed search
	 * 
	 * @throws StopExeption
	 */
	@Override
	public void startDistributedSearch() throws StopException {
		if (((TreeDisAO) this.agentOrder).getParentId() == null) {
			this.assignCPA();
		}
	}

	// ==========================================================================

	public void checkAssign(VarIdentifier agentId) throws StopException {
		if (this.isMyParent(agentId)) {
			this.assignCPA();
		}
	}

	// ==========================================================================

	private boolean isMyParent(VarIdentifier agId) {
		return agId.equals(((TreeDisAO) this.agentOrder).getParentId());
	}

	// ==========================================================================

	public void sendCPA(Instantiation cpa) throws StopException {
		HashList<VarIdentifier> linkedDescendants = this.communicator.getLowerNeighbors();
		linkedDescendants.addAll(((TreeDisAO) this.agentOrder).getChildrenIds());
		this.timeStamp.setTimeStamp(cpa.getTimeStamp());
		for (VarIdentifier receiverId : linkedDescendants) {
			this.cpaMsgH.sendCPAMessage(cpa, receiverId, this.getVarIdentifier());
		}
		if (linkedDescendants.size() == 0) {
			((AFCTreeTermMsgHandler) this.terminateMsgHandler).sendAccepted();
		}
	}

	public void incrementMyStamp() {
		this.timeStamp.update(myLevel, this.var.getTimeCounter());
	}

}
