package simple.dcsp.afcfamily.afcng.afctree;

import java.util.HashMap;
import java.util.Iterator;

import kernel.AbstractAgent;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.order.TreeDisAO;
import simple.SimpleAgent;
import simple.dcsp.afcfamily.TermMsgHandler;
import util.Commands;
import util.Instantiation;
import util.VarAssignment;

public class AFCTreeTermMsgHandler extends TermMsgHandler {

	int rcvdAccepted = 0;
	private HashMap<VarIdentifier, Instantiation> acceptedMsg;

	public AFCTreeTermMsgHandler(AbstractAgent agent) {
		super(agent);
		this.acceptedMsg = new HashMap<VarIdentifier, Instantiation>();
	}

	public boolean processTerminationDetection(Message msg) throws StopException {
		if (msg.contains(Commands.ACCEPTED)) {
			Instantiation accepted = (Instantiation) msg.get(Commands.ACCEPTED);
			VarIdentifier senderId = msg.getSenderId();
			// if(Logger.LOG_DEBUG){
			// this.myAgent.log("ACCEPTED ==> "+msg, Logger.INFO_LEVEL);
			// }
			if (!this.acceptedMsg.containsKey(senderId)) {
				this.acceptedMsg.put(senderId, accepted);
			} else {
				Instantiation stored = this.acceptedMsg.get(senderId);
				if (accepted.compareTimeCounters(stored) > 0) {
					this.acceptedMsg.put(senderId, accepted);
				}
			}
			// if(Logger.LOG_DEBUG){
			// this.myAgent.log("allAccepted ==> "+this.acceptedMsg, Logger.INFO_LEVEL);
			// }
			int size = this.acceptedMsg.size();
			if (size == ((TreeDisAO) this.myAgent.agentOrder).getChildrenIds().size()) {
				VarAssignment myAssign = ((SimpleAgent) this.myAgent).var.getVarAssignment();
				Iterator<VarIdentifier> it = this.acceptedMsg.keySet().iterator();
				while (it.hasNext()) {
					VarIdentifier child = it.next();
					accepted = this.acceptedMsg.get(child);
					if (!accepted.compatible(myAssign) || !this.myAgent.agentView.compatibleWith(accepted)) {
						// if(Logger.LOG_DEBUG){
						// this.myAgent.log("Incompatibility ("+child+") : "+this.acceptedMsg, Logger.INFO_LEVEL);
						// }
						return false;
					}
				}
				VarIdentifier parentId = ((TreeDisAO) this.myAgent.agentOrder).getParentId();
				if (parentId != null) {
					this.sendAccepted();
				} else {
					throw new StopException(Commands.SUCCESS);
				}
				return true;
			}
		}
		return false;
	}

	public void sendAccepted() {
		VarIdentifier parentId = ((TreeDisAO) this.myAgent.agentOrder).getParentId();
		Instantiation accepted = this.myAgent.agentView.toInstantiation();
		Message msg = this.createMessage();
		msg.addContents(Commands.ACCEPTED, accepted);
		msg.setReceiverId(parentId);
		this.myAgent.sendMessage(msg);
	}

	public void resetRcvdAcceptedCounter() {
		this.rcvdAccepted = 0;
	}

}
