package simple.dcsp.afcfamily.afcng.approximateDom;

import java.util.HashMap;
import java.util.Iterator;
import kernel.Constraint;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.constraints.integer.ExtensionBinConstraint;
import kernel.order.AgentOrdering;
import simple.dcsp.afcfamily.afcng.DynamicAFCngAgent;
import simple.dcsp.afcfamily.afcng.TimeStamp;
import util.HashList;
import util.Instantiation;
import util.Tools;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class AFCngApproximateMinDomAgent extends DynamicAFCngAgent {

	private static final String LowerBound = "LB";
	private static final String UpperBound = "UB";
	private short type;

	/**
	 * the number of conflicts for each value on each neighbor domain. 
	 */
	private HashMap<Integer, HashMap<VarIdentifier, Integer>> nbConflicts = null;

	private HashMap<VarIdentifier, Integer> lbDomais;
	private HashMap<VarIdentifier, Integer> ubDomais;
	private HashMap<VarIdentifier, Integer> LB;
	private HashMap<VarIdentifier, Integer> UB;
	private HashMap<VarIdentifier, Integer> initialDomains = null;

	public AFCngApproximateMinDomAgent(int agentId, String agName, String description, short type) {
		super(agentId, agName, description);
		this.type = type;
		this.lbDomais = new HashMap<VarIdentifier, Integer>();
		this.ubDomais = new HashMap<VarIdentifier, Integer>();
		this.initialDomains = new HashMap<VarIdentifier, Integer>();
		this.cpaMsgH = new EstCPAMsgHandler(this);
		this.nbConflicts = new HashMap<Integer, HashMap<VarIdentifier, Integer>>();
		this.agentOrder = new AgentOrdering(this) {

			public void processOrder(Message msg) {
				super.processOrder(msg);
				for(VarIdentifier agId : this.agentsOrder) {
					// init by domain sizes
					int dom = (Integer) msg.get(agId + "");
					lbDomais.put(agId, 0);
					ubDomais.put(agId, 0);
					initialDomains.put(agId, dom);
				}
			}
		};
	}

	@Override
	public void configurePropagationEngine() {
		// this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_FIRST).add(VariableEvent.INSTANCIATE);
		// this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_FIRST).add(VariableEvent.DESINSTANCIATE);
		super.configurePropagationEngine();
		for(int value : this.var.getDomain()) {
			this.nbConflicts.put(value, new HashMap<VarIdentifier, Integer>());
		}
		Iterator<Constraint> it = this.constraintHandler.getConstraintsIterator();
		while (it.hasNext()) {
			Constraint c = it.next();
			if (c instanceof ExtensionBinConstraint) {
				ExtensionBinConstraint constraint = ((ExtensionBinConstraint) c);
				for(int value : this.var.getDomain()) {
					VarIdentifier varId = constraint.getExternalVar(0).getVarIdentifier();
					int nbConflict = constraint.getNumberOfConflicts(value);
					this.nbConflicts.get(value).put(varId, nbConflict);
				}
			} else {
				// TODO get number of conflicts
				// System.out.println("Unkwon number of Conflicts.\n"+this.agentId+" : "+c);
				// System.exit(0);
			}
		}
	}

	/**
	 * 
	 */
	@Override
	public void sendCPA(Instantiation cpa) throws StopException {
		this.computeBounds(cpa);
		super.sendCPA(cpa);
	}

	/**
	 * min_i is set to the maximum number of inconsistent values of variable x_i 
	 * with the value of every assigned variable.
	 * 
	 * max_i is set to the sum of all inconsistent values of variable x_i with 
	 * respect to each assigned variable
	 * 
	 * Then, the next variable to be assigned is chosen as follows: 
	 * (i) if there is xi such that min_i >= min{d, max_j}, forall x_j unassigned, 
	 * select x_i (where d is the domain size); 
	 * (ii) otherwise, select the variable with maximum max_j .
	 * 
	 */
	private void computeBounds(Instantiation cpa) {
		this.LB = new HashMap<VarIdentifier, Integer>();
		this.UB = new HashMap<VarIdentifier, Integer>();
		HashList<VarIdentifier> unassigned = this.agentOrder.getOrder();
		unassigned.removeAll(cpa.getVarIds());
		int myValue = this.var.getValue();
		for(VarIdentifier lower : unassigned) {
			if (this.communicator.isNeighbor(lower)) {
				// compute the new lower bound
				// lb_j <-- max(lb_j, conflicts<x_i,x_j>)
				int nbConflict = this.nbConflicts.get(myValue).get(lower);
				int lb = this.lbDomais.get(lower);
				LB.put(lower, Math.max(lb, nbConflict));
				// ub_j <-- min(|Dj|,ub_j + conflicts<x_i,x_j>)
				int ub = this.ubDomais.get(lower) + nbConflict;
				UB.put(lower, Math.min(initialDomains.get(lower), ub));
			} else {
				LB.put(lower, this.lbDomais.get(lower));
				UB.put(lower, this.ubDomais.get(lower));
			}
		}
	}

	/**
	 * get the next agent to which xi must send the (to assign) 
	 */
	public VarIdentifier getNextAgent() {// TODO
		String s =
				"nbConflicts: " + this.nbConflicts + "\n" + "LB: " + this.LB + "\n" + "UB: " + this.UB + "\n" + "lbDomais: " + this.lbDomais + "\n"
						+ "ubDomais: " + this.ubDomais + "\n";
		this.log(s, util.Logger.INFO_LEVEL);
		switch (this.type) {
			case AFCngApproximateMinDomMaster.MinDom :
				return this.getMinDomainNextAgent();
			case AFCngApproximateMinDomMaster.MinDomDeg :
				return this.getDomDegNextAgent();
			default :
				return null;
		}
	}

	/**
	 * Then, the next variable to be assigned is chosen as follows: 
	 * (i) if there is xi such that min_i >= min{d, max_j}, forall x_j unassigned, 
	 * select x_i (where d is the domain size); 
	 * (ii) otherwise, select the variable with maximum max_j .
	 * 
	 * @return next agent to assign
	 */
	public VarIdentifier getMinDomainNextAgent() {// TODO
		if (UB.isEmpty())
			return null;
		VarIdentifier idMinUB = Tools.MinOf(UB);
		VarIdentifier idmaxLB = Tools.MaxOf(LB);
		if (LB.get(idmaxLB) > UB.get(idMinUB)) {
			return idmaxLB;
		} else {
			return Tools.MinOf(LB);
		}
	}

	/**
	 * @param bounds
	 * @return next agent to assign
	 */
	public VarIdentifier getDomDegNextAgent() {
		HashList<VarIdentifier> assigned = this.agentView.toInstantiation().getVarIds();
		assigned.add(this.getVarIdentifier());
		HashList<VarIdentifier> unassigned = this.agentOrder.getOrder();
		unassigned.removeAll(assigned);
		if (unassigned.size() == 0 || UB.isEmpty())
			return null;
		HashMap<VarIdentifier, Integer> degrees = this.getDegrees(unassigned);
		for(VarIdentifier agId : UB.keySet()) {
			int deg = degrees.get(agId);
			double upperBound = UB.get(agId);
			if (upperBound == 0) {
				return agId;
			} else {
				if (deg == 0) {
					upperBound = Double.POSITIVE_INFINITY;
				} else {
					upperBound /= deg;
				}
				boolean selected = true;
				for(VarIdentifier id : LB.keySet()) {
					int degree = degrees.get(id);
					double lowerBound = LB.get(id);
					if (degree == 0) {
						lowerBound = Double.POSITIVE_INFINITY;
					} else {
						lowerBound /= degree;
					}
					if (agId != id) {
						if (upperBound > lowerBound) {
							selected = false;
							break;
						}
					}
				}
				if (selected) {
					return agId;
				}
			}
		}
		VarIdentifier IdOfMinLb = null;
		double minLb = Double.POSITIVE_INFINITY;
		for(VarIdentifier agId : LB.keySet()) {
			int deg = degrees.get(agId);
			double lowerBound = LB.get(agId);
			if (deg == 0)
				lowerBound = Double.POSITIVE_INFINITY;
			else
				lowerBound /= deg;
			if (lowerBound < minLb) {
				IdOfMinLb = agId;
				minLb = LB.get(agId);
			}
		}
		return IdOfMinLb;
	}

	private HashMap<VarIdentifier, Integer> getDegrees(HashList<VarIdentifier> unassigned) {
		HashMap<VarIdentifier, Integer> map = new HashMap<VarIdentifier, Integer>();
		for(VarIdentifier agId : UB.keySet()) {
			map.put(agId, ddegree(agId, unassigned));
		}
		return map;
	}

	private int ddegree(VarIdentifier agId, HashList<VarIdentifier> unassigned) {
		int deg = 0;
		Iterable<VarIdentifier> neighbors = this.agentOrder.getAgentsGraph().getNeighborsData(agId);
		for(VarIdentifier id : neighbors)
			if (unassigned.contains(id)) {
				deg++;
			}
		return deg;
	}

	// ====================================================================================
	//
	// ====================================================================================

	/**
	 *  Approximated MinDom CPA message handler
	 */
	public class EstCPAMsgHandler extends CPAMsgHandler {

		public EstCPAMsgHandler(AFCngApproximateMinDomAgent algo) {
			super(algo);
		}

		@SuppressWarnings("unchecked")
		public void processMessage(Message msg) throws StopException {
			VarIdentifier nextId = (VarIdentifier) msg.get(NEXT_ID);
			TimeStamp ts = (TimeStamp) msg.get(TIMESTAMP);
			if (timeStamp.compareTimeStamp(ts) > 0) {
				if (this.myAgent.getVarIdentifier().equals(nextId)) {
					HashMap<VarIdentifier, Integer> lb = (HashMap<VarIdentifier, Integer>) msg.get(LowerBound);
					HashMap<VarIdentifier, Integer> ub = (HashMap<VarIdentifier, Integer>) msg.get(UpperBound);
					lbDomais.putAll(lb);
					ubDomais.putAll(ub);
				}
				Instantiation cpa = (Instantiation) msg.get(CPA);
				receiveCPA(ts, cpa, nextId);
			}
		}

		/**
		 * 
		 */
		@Override
		public void sendCPAMessage(Instantiation cpa, VarIdentifier receiverId, VarIdentifier nextVarId) {
			if (!receiverId.equals(nextVarId)) {
				super.sendCPAMessage(cpa, receiverId, nextVarId);
			} else {
				Message msg = createMessage();
				msg.addContents(CPA, cpa.clone());
				msg.addContents(NEXT_ID, nextVarId);
				msg.addContents(TIMESTAMP, timeStamp.clone());
				msg.addContents(LowerBound, LB);
				msg.addContents(UpperBound, UB);
				msg.setReceiverId(receiverId);
				sendMessage(msg);
			}
		}
	}
}
