package simple.dcsp.afcfamily.ddbj;

import java.util.Iterator;
import java.util.TreeMap;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.MessageHandler;
import kernel.events.VariableEvent;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.explain.ExplainedIntDomain;
import kernel.vars.integer.explain.ExplainedIntVar;
import simple.SimpleAgent;
import simple.SingleExplainedVarAgent;
import simple.dcsp.Nogood;
import simple.dcsp.afcfamily.TermMsgHandler;
import simulation.metric.Metric;
import simulation.parser.PVariable;
import util.Commands;
import util.HashList;
import util.Instantiation;
import util.VarAssignment;
import util.VarEventMask;

public class DDBJAgent extends SingleExplainedVarAgent {

	public static final String TIMESTAMP = "TimeStamp";
	public static final String OBSOLETE_CPA = "obsoleteCPA";
	public static final String OBSOLETE_FC = "obsoleteFC";
	public static final String FC = Commands.FC_CPA;
	public static final String CPA = Commands.CPA;
	public static final String NOT_OK = "not_ok";
	public static final String BACKTRACK = Commands.BACKTRACK;
	public static final String PC = "PC";

	private CPAMsgHandler cpaMsgH;
	private FCMsgHandler fcmsgH;
	private BackCPAMsgHandler backCPAMsgH;
	private NotOKMsgHandler notOkMsgH;
	private PCMsgHandler pcMsgH;
	
	protected TreeMap<VarIdentifier, Integer> agentsCounters = null; //TODO
	protected TimeStamp timeStamp;
	protected boolean agentViewConsistent = true;
	
	/*
	Problem dans l'algorithme dans les lignes 12 et 13 de receiveNG()
	si un agent reoit un Not_ok il se dsinstancie (pb: dans le cas ou il est suuivi d'un BT)
	 */

	public DDBJAgent(int agId, String name, String description) {
		super(agId, name, description);
		this.valSelector = new MinConflictValSelector(this);
		this.agentsCounters = new TreeMap<VarIdentifier, Integer>(); 
		this.terminateMsgHandler = new TermMsgHandler(this);
		this.cpaMsgH = new CPAMsgHandler(this);
		this.fcmsgH = new FCMsgHandler(this);
		this.backCPAMsgH = new BackCPAMsgHandler(this);
		this.notOkMsgH = new NotOKMsgHandler(this);
		this.pcMsgH = new PCMsgHandler(this);
		this.timeStamp = new TimeStamp();
		this.initAuxStatistics();
	}
	
	public DDBJAgent(PVariable variable) {
		super(variable);
		this.valSelector = new MinConflictValSelector(this);
		this.agentsCounters = new TreeMap<VarIdentifier, Integer>(); 
		this.terminateMsgHandler = new TermMsgHandler(this);
		this.cpaMsgH = new CPAMsgHandler(this);
		this.fcmsgH = new FCMsgHandler(this);
		this.backCPAMsgH = new BackCPAMsgHandler(this);
		this.notOkMsgH = new NotOKMsgHandler(this);
		this.pcMsgH = new PCMsgHandler(this);
		this.timeStamp = new TimeStamp();
		this.initAuxStatistics();
	}

	private void initAuxStatistics() {
		this.auxiliaryStatistics.put(Commands.nbObsoleteNogood, new Metric(Commands.nbObsoleteNogood));
		this.auxiliaryStatistics.put(Commands.nbAssignments, new Metric(Commands.nbAssignments));
		this.auxiliaryStatistics.put(OBSOLETE_CPA, new Metric(OBSOLETE_CPA));
		this.auxiliaryStatistics.put(OBSOLETE_FC, new Metric(OBSOLETE_FC));
	}

	// ===========================================================================
	// ///////////////// Initialization ////////////////////////////
	// ===========================================================================

	@Override
	public void configurePropagationEngine() {
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.INSTANCIATE);
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.DESINSTANCIATE);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_THE_LAST).add(VariableEvent.RESTORE_VAL);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_BETWEEN_L_H).add(VariableEvent.RESTORE_VAL);
		this.constraintHandler.update();
	}

	// ==========================================================================

	@Override
	public void addMsgHandlers() {
		this.communicator.addMsgHandler(this.cpaMsgH);
		this.communicator.addMsgHandler(this.fcmsgH);
		this.communicator.addMsgHandler(this.backCPAMsgH);
		this.communicator.addMsgHandler(this.notOkMsgH);
		this.communicator.addMsgHandler(this.pcMsgH);
	}

	// ==========================================================================

	public void initAgentView() {
		HashList<VarIdentifier> agents = this.getOrder();
		agents.remove(this.getVarIdentifier());
		for (VarIdentifier agId : agents) {
			if (!this.communicator.isNeighbor(agId)) {
				VarAssignment var = new VarAssignment(new VarIdentifier(agId), Integer.MIN_VALUE);
				IntExternalVar extVar = this.makeExternalVar(var);
				this.agentView.add(extVar);
			} else {
				this.agentView.add((IntExternalVar) this.getExternalVar(agId));
			}
		}
	}

	// ==========================================================================
	//   
	//  DDBJ Procedure
	// 
	// ==========================================================================

	/**
	 * Start distributed search from the initializing agent
	 * 
	 * @throws StopExeption
	 */
	public void startDistributedSearch() throws StopException {
		((MinConflictValSelector) this.valSelector).initialize();
		this.initAgentCounters();
		if (this.agentOrder.getMyRank() == 0) {
			this.AssignCPA();
		}
	}

	// ==========================================================================

	/**
	 * 
	 */
	private void initAgentCounters() {
		for(VarIdentifier agId : this.agentOrder.getOrder()){
			this.agentsCounters.put(agId, 0);
		}
	}

	/**
	 * this function is called when ever we need to instantiate the current partial assignment, if doesn't succeed it launches a this.Backtrack
	 * procedure
	 */
	public void AssignCPA() throws StopException {
		if (this.valSelector.hasNextValue()) {
			int value = this.valSelector.getNextValue();
			this.var.instantiate(value);
			Instantiation cpa = this.agentView.toInstantiation();
			cpa.add(this.var.getVarAssignment());
			this.updateTimeStamp(cpa.size()-1);
			this.sendCPA(cpa);
		} else {
			this.agentViewConsistent = false;
			this.agentView.clearExclusiveFrom(this.solveMyNogoods().getRhs());
			this.Backtrack();
		}
	}

	// ==========================================================================

	public void sendCPA(Instantiation cpa) throws StopException {
		VarIdentifier nextVarId = this.getNextAgent(cpa);
		if (nextVarId == null) { throw new StopException(Commands.SUCCESS); }
		this.cpaMsgH.sendCPAMessage(cpa, nextVarId);
		this.fcmsgH.sendFCmessages(cpa, nextVarId);
	}

	public VarIdentifier getNextAgent(Instantiation cpa) {
		HashList<VarIdentifier> assigned = cpa.getVarIds(); 
		VarIdentifier IdOfMaxPC = null;
		int maxPC = Integer.MIN_VALUE;
		for (VarIdentifier agId : this.agentsCounters.keySet()) {
			if (!assigned.contains(agId) && this.agentsCounters.get(agId) > maxPC) {
				IdOfMaxPC = agId;
				maxPC = this.agentsCounters.get(agId);
			}
		}
		return IdOfMaxPC;
	}

	public void Backtrack() throws StopException {
		if (this.agentOrder.getMyRank() == 0) {
			throw new StopException(Commands.FAILURE);
		}
		this.agentViewConsistent = false;
		Instantiation cpa = this.agentView.toInstantiation();
		VarIdentifier backTo = cpa.last().getVarIdentifier();
		this.backCPAMsgH.sendBack(cpa, backTo);
		this.decrementCounterOf(backTo);
		this.notOkMsgH.sendNotOks(cpa);
		if (this.var.isInstantiated()) {
			this.var.desinstanciate(VariableEvent.NOCAUSE);
		}
		Instantiation assignments = this.getPotentielConflictAgents();
		this.pcMsgH.sendPC(assignments);
	}

	/**
	 * @return
	 */
	private Instantiation getPotentielConflictAgents() {
		Instantiation PCAs = new Instantiation();
		ExplainedIntDomain domain = ((ExplainedIntVar) this.var).getDomain();
		for(int value : domain.removedValsIterator()) {
			Instantiation ng = super.buildBetterNogood((ExplainedIntVar) this.var, value);
			if(!ng.isEmpty()){
				PCAs.add(ng.get(0));
			}
		}
		return PCAs;
	}

	// ===========================================================================
	public boolean updateAgentView(Instantiation cpa) {
		this.agentView.clearExcept(cpa);
		for (VarAssignment v : cpa) {
			this.agentView.priorityMerge(v);
			if (var.getDomainSize() == 0) { return false; }
		}
		return true;
	}
	
	// ==========================================================================
	protected void processCPA(Instantiation cpa) throws StopException {
		Instantiation view = agentView.toInstantiation();
		if (!agentViewConsistent && cpa.contains(view)) {
			return;
		}
		this.agentViewConsistent = true;
		if (this.updateAgentView(cpa)) {
			this.AssignCPA();
		} else {
			this.Backtrack();
		}
	}

	// ==========================================================================
	public void processFC(Instantiation cpa) throws StopException {
		if (!this.agentViewConsistent) {
			Instantiation view = agentView.toInstantiation();
			if (!cpa.contains(view)) {
				this.agentViewConsistent = true;
			}
		}
		if (this.agentViewConsistent) {
			if (!this.updateAgentView(cpa)) {
				this.Backtrack();
			}
		}
	}
	
	// ==========================================================================
	protected void processBackCPA(Instantiation cpa, TimeStamp ts) throws StopException {
		if (this.var.isInstantiated()) {
			if (timeStamp.compareTimeStamp(ts) >= 0) {
				this.makeNoGoodConstraint(new Nogood(cpa));
				this.AssignCPA();
			}
		}
	}

	/**
	 * @param sender
	 */
	private void incrementCounterOf(VarIdentifier agId) {
		int cmp = agentsCounters.get(agId) + 1;
		this.agentsCounters.put(agId, cmp);
	}

	/**
	 * @param sender
	 */
	private void decrementCounterOf(VarIdentifier agId) {
		int cmp =  agentsCounters.get(agId) - 1;
		this.agentsCounters.put(agId, cmp);
	}

	// ==========================================================================
	public void processNotOk(Instantiation notOk, TimeStamp ts) throws StopException {
		Instantiation view = this.agentView.toInstantiation();
		if (view.contains(notOk)) {
			this.agentView.clearExcept(notOk);
			this.agentView.priorityMerge(notOk);
			this.agentViewConsistent = false;
		} else if (timeStamp.compareTimeStamp(ts) > 0) {
			this.timeStamp.setTimeStamp(ts);
			HashList<VarIdentifier> newOrdering = notOk.getVarIds();
			newOrdering.add(getVarIdentifier());
			this.agentOrder.updateOrder(newOrdering);
			this.agentViewConsistent = false;
			if (!this.updateAgentView(notOk)) {
				this.Backtrack();
			}
			if (this.var.isInstantiated()) {
				this.var.desinstanciate(VariableEvent.NOCAUSE);
			}
		}
//		if (this.var.isInstantiated()) {
//			this.var.desinstanciate(VariableEvent.NOCAUSE);
//		}
	}

	// ===========================================================================
	public class CPAMsgHandler extends MessageHandler {

		public CPAMsgHandler(SimpleAgent algo) {
			super(algo, CPA);
		}

		public void processMessage(Message msg) throws StopException {
			Instantiation cpa = (Instantiation) msg.get(CPA);
			TimeStamp ts = (TimeStamp) msg.get(TIMESTAMP);		
			if (timeStamp.compareTimeStamp(ts) > 0) {
				timeStamp.setTimeStamp(ts);
				HashList<VarIdentifier> newOrdering = cpa.getVarIds();
				newOrdering.add(getVarIdentifier());
				agentOrder.updateOrder(newOrdering);
				constraintHandler.update();
				processCPA(cpa);
			} else {
				auxiliaryStatistics.get(OBSOLETE_CPA).value++;
			}
		}

		public void sendCPAMessage(Instantiation cpa, VarIdentifier receiverId) {
			Message msg = createMessage();
			msg.addContents(CPA, cpa.clone());
			msg.addContents(TIMESTAMP, timeStamp.clone());
			msg.setReceiverId(receiverId);
			sendMessage(msg);
		}

	}

	// ========================================================================
	public class FCMsgHandler extends MessageHandler {

		public FCMsgHandler(DDBJAgent agent) {
			super(agent, FC);
		}

		@Override
		public void processMessage(Message msg) throws StopException {
			Instantiation cpa = (Instantiation) msg.get(FC);
			TimeStamp ts = (TimeStamp) msg.get(TIMESTAMP);		
			if (timeStamp.compareTimeStamp(ts) > 0) {
				timeStamp.setTimeStamp(ts);
				HashList<VarIdentifier> newOrdering = cpa.getVarIds();
				newOrdering.add(getVarIdentifier());
				agentOrder.updateOrder(newOrdering);
				constraintHandler.update();
				processFC(cpa);
			} else {
				auxiliaryStatistics.get(OBSOLETE_FC).value++;
			}
		}

		public void sendFCmessages(Instantiation cpa, VarIdentifier nextVarId) {
			HashList<VarIdentifier> lowerneighbors = communicator.getLowerNeighbors();
			if(lowerneighbors.contains(nextVarId)){
				lowerneighbors.remove(nextVarId);
			}
			for (VarIdentifier lower : lowerneighbors) {
				Message msg = createMessage();
				msg.addContents(FC, cpa.clone());
				msg.addContents(TIMESTAMP, timeStamp.clone());
				msg.setReceiverId(lower);
				sendMessage(msg);
			}
		}
	}

	// ==========================================================================
	public class NotOKMsgHandler extends MessageHandler {

		public NotOKMsgHandler(DDBJAgent agent) {
			super(agent, NOT_OK);
		}

		@Override
		public void processMessage(Message msg) throws StopException {
			Instantiation notOk = (Instantiation) msg.get(NOT_OK);
			TimeStamp ts = (TimeStamp) msg.get(TIMESTAMP);
			processNotOk(notOk, ts);
		}

		public void sendNotOks(Instantiation notOk) {
			HashList<VarIdentifier> lowerneighbors = communicator.getNeighborsLowerThan(notOk.last().getVarIdentifier());
			for (VarIdentifier lower : lowerneighbors) {
				sendNotOk(notOk.clone(), lower);
			}
		}

		private void sendNotOk(Instantiation notOk, VarIdentifier agId) {
			Message msg = this.createMessage();
			msg.addContents(NOT_OK, notOk);
			msg.addContents(TIMESTAMP, timeStamp.clone());
			msg.setReceiverId(agId);
			sendMessage(msg);
		}
	}

	// ==============================================================================
	public class BackCPAMsgHandler extends MessageHandler {

		public BackCPAMsgHandler(DDBJAgent agent) {
			super(agent, BACKTRACK);
		}

		@Override
		public void processMessage(Message msg) throws StopException {
			VarIdentifier senderId = msg.getSenderId();
			Instantiation cpa = (Instantiation) msg.get(CPA);
			TimeStamp ts = (TimeStamp) msg.get(TIMESTAMP);
			//*********************************************
			int value = (Integer) cpa.last().getValue();
			((MinConflictValSelector) valSelector).increment(value);
			incrementCounterOf(senderId);
			//*********************************************
			processBackCPA(cpa, ts);
		}

		public void sendBack(Instantiation cpa, VarIdentifier backTo) {
			Message msg = this.createMessage();
			msg.addContents(CPA, cpa);
			msg.addContents(TIMESTAMP, timeStamp.clone());
			msg.setReceiverId(backTo);
			sendMessage(msg);
		}
	}

	// ==============================================================================
	public class PCMsgHandler extends MessageHandler {

		public PCMsgHandler(DDBJAgent agent) {
			super(agent, PC);
		}

		@Override
		public void processMessage(Message msg) throws StopException {
//			System.err.println("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n");
			VarIdentifier senderId = msg.getSenderId();
			int value = (Integer) msg.get(Commands.VALUE);
			incrementCounterOf(senderId);
			((MinConflictValSelector) valSelector).increment(value);
		}
		
		/**
		 * @param assignments
		 */
		public void sendPC(Instantiation assignments) {
			Iterator<VarAssignment> it = assignments.iterator();
			while(it.hasNext()){
				VarAssignment varAss = it.next();
				VarIdentifier agId = varAss.getVarIdentifier();
				int value = varAss.getValue();
				this.sendPCMessage(agId, value);
				incrementCounterOf(agId);
				((MinConflictValSelector) valSelector).increment(value);
			}
		}

		/**
		 * @param agId
		 * @param value
		 */
		private void sendPCMessage(VarIdentifier agId, int value) {
			Message msg = this.createMessage();
			msg.addContents(Commands.VALUE, value);
			msg.setReceiverId(agId);
			sendMessage(msg);
		}
	}
	
	// ===============================================================================
//	public void incrementMyStamp() {
//		this.timeStamp.update(this.agentOrder.getMyRank());
//	}
	private void updateTimeStamp(int index) {
		this.timeStamp.update(index, this.var.getTimeCounter());
	}
}
