package simple.dcsp.agile;

import java.io.Serializable;
import kernel.VarIdentifier;
import util.HashList;
import util.Instantiation;
import util.Tools;

public class StateOfAgent implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -469787414974796044L;
	private Instantiation agentView = null;
	private HashList<VarIdentifier> order = null;
	private Double[] tv = null;

	public StateOfAgent(HashList<VarIdentifier> order, Double[] tv) {
		this.order = order;
		this.tv = tv;
		this.agentView = new Instantiation();
	}

	public StateOfAgent(HashList<VarIdentifier> order, Double[] tv, Instantiation agentView) {
		this(order, tv);
		this.agentView = agentView;
	}

	public void setAgentView(HashList<VarIdentifier> order, Double[] tv, Instantiation agentView) {
		if (areAcceptable(this.order, this.tv, order, tv) <= 0) {
			this.setOrder(order, tv);
			this.agentView = agentView;
		}
	}

	/**
	 * 
	 * @param order1
	 * @param tv1
	 * @param order2
	 * @param tv2
	 * @return <b><i>+(i+1)</i></b> if <b>tv1</b> is the best (tv1< tv2)<br/>
	 *         <b><i> 0 </i></b> if <b>tv1</b> and <b>tv2</b> are equal (tv1==tv2)<br/>
	 *         <b><i>-(i+1)</i></b> if <b>tv2</b> is the best</b> (tv1>tv2)<br/>
	 */
	public static int areAcceptable(HashList<VarIdentifier> order1, Double[] tv1, HashList<VarIdentifier> order2, Double[] tv2) {
		int ans = Tools.compare(tv1, tv2);
		switch (ans) {
			case 0 : 
				return order1.compare(order2);
			default :
				return ans;

		}
	}

	public Instantiation getAgentView() {
		return this.agentView;
	}

	public HashList<VarIdentifier> getOrder() {
		return this.order;
	}

	public Double[] getTV() {
		return this.tv.clone();
	}

	public void setOrder(HashList<VarIdentifier> order, Double[] tv) {
		if (areAcceptable(this.order, this.tv, order, tv) <= 0) {
			this.order = order;
			this.tv = tv;
		} else {
			throw new Error("\nThere are somethings wrong:" + "\n" + "myOrder: " + this.order + " and myTV : " + Tools.arrayToString(this.tv)
					+ "order: " + order + " and my tv : " + Tools.arrayToString(tv));
		}
	}

	public boolean isCompatible(StateOfAgent state) {
		return (this.order.compare(state.getOrder()) == 0) && 
				Tools.compare(this.tv, state.getTV()) == 0 && 
				this.agentView.compatible(state.getAgentView());
	}

	public void priorityMerge(Instantiation agentView) {
		this.agentView.priorityMerge(agentView);
	}

	public String toString() {
		String ans = "";
		ans += this.order + "@" + Tools.arrayToString(this.tv) + "@" + this.agentView;
		return ans;
	}

	public StateOfAgent clone() {
		return new StateOfAgent((HashList<VarIdentifier>)this.order.clone(), tv.clone(), this.agentView.cloneUseingSelfComparator());
	}

	public boolean equals(Object obj) {
		StateOfAgent state = (StateOfAgent) obj;
		return (this.order.compare(state.getOrder()) == 0) && (Tools.compare(this.tv, state.getTV()) == 0)
				&& this.agentView.contains(state.getAgentView()) && state.getAgentView().contains(this.agentView);
	}

	public void update(StateOfAgent state) {
		int acceptable = areAcceptable(this.order, this.tv, state.getOrder(), state.getTV());
		if (acceptable < 0) {
			this.order = state.getOrder();
			this.tv = state.getTV();
			this.agentView = state.getAgentView();
		} else {
			if (acceptable == 0) {
				if (this.agentView.compareTimeCounters(state.getAgentView()) <= 0) {
					this.agentView = state.getAgentView();
				}
			}
		}
	}

}
