/**
 * 
 */
package simple.dcsp.agile.exp;

import static simple.dcsp.agile.exp.AgileMaster.DomDeg;
import static simple.dcsp.agile.exp.AgileMaster.DomWDeg;
import static simple.dcsp.agile.exp.AgileMaster.MinDom;
import static simple.dcsp.agile.exp.AgileMaster.Topological_Deg;
import static simple.dcsp.agile.exp.AgileMaster.Topological_ID;
import static simple.dcsp.agile.exp.AgileMaster.Topological_MaxSucc;
import java.util.BitSet;
import java.util.HashMap;
import java.util.Map;
import kernel.VarIdentifier;
import util.HashList;
import util.Instantiation;

public class Explanations {
	private BitSet[] lhs = null;
	private int[] rhs = null;
	private Map<VarIdentifier, Integer> domains = new HashMap<VarIdentifier, Integer>();
	private Map<VarIdentifier, Integer> degrees = new HashMap<VarIdentifier, Integer>();
	private Map<VarIdentifier, Integer> weights = new HashMap<VarIdentifier, Integer>();
	private final HashList<VarIdentifier> agentIds;
	private HashList<VarIdentifier> newOrder = null;
	private Double[] newTV = null;
	private Instantiation newNogood = null;
	private short heuristic = MinDom;
	
	public Explanations(HashList<VarIdentifier> agentIds, Map<VarIdentifier, Integer> domains, Map<VarIdentifier, Integer> degrees, Map<VarIdentifier, Integer> weights, short heuristic) {
		this.heuristic = heuristic;
		this.agentIds = (HashList<VarIdentifier>) agentIds.clone();
		this.domains.putAll(domains);
		this.degrees.putAll(degrees);
		this.weights.putAll(weights);
		int length = agentIds.size();
		this.rhs = new int[length];
		this.lhs = new BitSet[length];
		for(VarIdentifier varId : agentIds){
			int index = agentIds.indexOf(varId);
			this.lhs[index] = new BitSet(length);
			this.removeExplanationOf(varId);
		}
		this.newTV = new Double[length];
	}
	
	/**
	 * 
	 */
	public Explanations clone() {
		HashList<VarIdentifier> agentIds = (HashList<VarIdentifier>) this.agentIds.clone();
		return new Explanations(agentIds, this.domains, this.degrees, weights, 
				this.heuristic, this.getLhsClone(), this.rhs.clone());
	}

	/**
	 * @return
	 */
	private BitSet[] getLhsClone() {
		BitSet[] clone = new BitSet[this.lhs.length];
		for(VarIdentifier agId : agentIds){
			int index = agentIds.indexOf(agId);
			clone[index] = (BitSet) this.lhs[index].clone();
		}
		return clone;
	}
	/**
	 * 
	 * @param agentIds
	 * @param maxDomSize
	 * @param degrees
	 * @param heuristic
	 * @param agentView
	 * @param lhs
	 * @param rhs
	 */
	public Explanations(HashList<VarIdentifier> agentIds, Map<VarIdentifier, Integer> domains, Map<VarIdentifier, Integer> degrees, Map<VarIdentifier, Integer> weights, short heuristic, BitSet[] lhs, int[] rhs) {
		this.heuristic = heuristic;
		this.agentIds = agentIds;
		this.domains.putAll(domains);
		this.degrees.putAll(degrees);
		this.weights.putAll(weights);
		this.lhs = lhs;
		this.rhs = rhs;
	}

	public void updateMaxDomainSize(VarIdentifier varId, int d) {
		int value = Math.min(this.getMaxDomainSize(varId), d);
		this.domains.put(varId, value);
	}

	public int getMaxDomainSize(VarIdentifier agId) {
		return this.domains.get(agId);
	}

	public void removeExplanationOf(VarIdentifier varId) {
		int index = agentIds.indexOf(varId);
		this.lhs[index].clear();
		this.rhs[index] = this.domains.get(varId);
	}

	public HashList<VarIdentifier> getLhs(VarIdentifier varId) {
		int index = agentIds.indexOf(varId);
		HashList<VarIdentifier> predecessor = new HashList<VarIdentifier>();
		for(int i=this.lhs[index].nextSetBit(0) ; i>-1 ; i=this.lhs[index].nextSetBit(i+1)){
			predecessor.add(agentIds.get(i));
		}
		return predecessor;
	}

	public int getRhs(VarIdentifier varId) {
		int index = agentIds.indexOf(varId);
		return this.rhs[index];
	}
	
	public void setRhs(VarIdentifier agId, int rhs) {
		int index =this.agentIds.indexOf(agId);
		this.rhs[index] = rhs;
	}
	
	public void setExplanation(VarIdentifier agId, HashList<VarIdentifier> lhs, int rhs) {
		this.setRhs(agId, rhs);
		this.setLhs(agId, lhs);
	}

	/**
	 * @param agId
	 * @param lhs
	 */
	private void setLhs(VarIdentifier agId, HashList<VarIdentifier> lhs) {
		int index = agentIds.indexOf(agId);
		this.lhs[index].clear();
		for(VarIdentifier pre : lhs){
			int p = agentIds.indexOf(pre);
			this.lhs[index].set(p);
		}
	}

	public HashList<VarIdentifier> getNewOrder() {
		return this.newOrder;
	}

	public void setNewOrder(HashList<VarIdentifier> newOrder) {
		this.newOrder = newOrder;
	}

	public Double[] getNewTV() {
		return this.newTV.clone();
	}

	public void setNewTV(Double[] tv) {
		this.newTV = tv;
	}

	public Instantiation getNewNG() {
		return this.newNogood;
	}

	public void setNewNG(Instantiation newNG) {
		this.newNogood = newNG;
	}

	public void clear() {
		this.newOrder = null;
		this.newTV = null;
		this.newNogood = null;
	}

	/**
	 * remove all explanations incompatible with order
	 */
	public int removeIncompatibleLhsWithOrder(HashList<VarIdentifier> order) {
		int nbChecks = 0;
		for(VarIdentifier varId : agentIds){
			HashList<VarIdentifier> higherAgents = order.getElementsBefor(varId);// this.getHigherAgents(order, agId);
			HashList<VarIdentifier> predecessors = this.getLhs(varId);
			if (!higherAgents.contains(predecessors)) {
				this.removeExplanationOf(varId);
			}
			nbChecks++;
		}
		return nbChecks;
	}
	
	/**
	 * remove all explanations incompatible with order
	 */
	public void removeExplanationOfAgents(HashList<VarIdentifier> varIds) {
		for (VarIdentifier varId : varIds) {
			this.removeExplanationOf(varId);
		}
	}

	public HashList<VarIdentifier> getTopologicalOrder(short topologicalType) {
		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
		HashMap<VarIdentifier, HashList<VarIdentifier>> successors = this.getGraphSuccessor();
		HashMap<VarIdentifier, HashList<VarIdentifier>> graph = this.getGraphPredecessor();
		while (graph.keySet().size() > 0) {
			HashList<VarIdentifier> roots = this.getRoots(graph);
			VarIdentifier selected = null;
			double min = Integer.MAX_VALUE;
			double rhs = 0;
			switch (topologicalType) {
				case Topological_ID : {
					for (VarIdentifier agId : roots) {
						int index = agentIds.indexOf(agId);
						if (this.heuristic == MinDom) {
							rhs = this.getRhs(agId);
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.rhs[index]) / this.degrees.get(agId);
						} else if (this.heuristic == DomWDeg) {
							rhs = ((double) this.rhs[index]) / (this.weights.get(agId)*this.degrees.get(agId));
						}
						if ((min > rhs) || ((min == rhs) && (agId.compareTo(selected) < 0))) {
							min = this.getRhs(agId);
							selected = agId;
						}
					}
					break;
				}
				case Topological_Deg : {
					int deg = Integer.MIN_VALUE;
					for (VarIdentifier varId : roots) {
						int index = agentIds.indexOf(varId);
						if (this.heuristic == MinDom) {
							rhs = this.rhs[index];
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.rhs[index]) / this.degrees.get(varId);
						} else if (this.heuristic == DomWDeg) {
							rhs = ((double) this.rhs[index]) / (this.weights.get(varId)*this.degrees.get(varId));
						}
						if (min > rhs || (min == rhs && (this.degrees.get(varId) > deg))) {
							min = this.rhs[index];
							deg = this.degrees.get(varId);
							selected = varId;
						}
					}
					break;
				}
				case Topological_MaxSucc : {
					for (VarIdentifier varId : roots) {
						int index = agentIds.indexOf(varId);
						if (this.heuristic == MinDom) {
							rhs = this.getRhs(varId);
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.rhs[index]) / this.degrees.get(varId);
						} else if (this.heuristic == DomWDeg) {
							rhs = ((double) this.rhs[index]) / (this.weights.get(varId)*this.degrees.get(varId));
						}
						if (min > rhs || (min == rhs && successors.get(varId).size() > successors.get(selected).size())) {
							min = this.getRhs(varId);
							selected = varId;
						}
					}
					break;
				}
			}
			order.add(selected);
			this.remove(graph, selected);
		}
//		if (order.size() != this.agentIds.size()) { throw new Error("\nacyclic graph : \n" + this); }
		return order;
	}
	/**
	 * 
	 * @param logger 
	 * @return
	 */
	public HashList<VarIdentifier> getBestOrder() {
		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
		HashMap<VarIdentifier, HashList<VarIdentifier>> graph = this.getGraphPredecessor();
		while (order.size() < this.agentIds.size()) {
			HashList<VarIdentifier> allRoots = this.getRoots(graph);
			HashList<VarIdentifier> S1 = this.getMinRoots(allRoots);
			while (S1.size() != 0) {
				if (S1.size() > 1) {
					int d = this.getRhs(S1.get(0));
					HashList<VarIdentifier> S2 = this.getAgentsWithMinRhs(graph, d);
					if (S2.size() == 0) {
						VarIdentifier selected = S1.get(0);
						order.add(selected);
						this.remove(graph, selected);
						break;
					}
					HashList<VarIdentifier> S3 = this.getRootsWithMinPredecessor(graph, S2);
					HashList<VarIdentifier> S4 = this.getMinRoots(S3);
					if (S4.size() == 1) {
						VarIdentifier selected = S4.get(0);
						HashList<VarIdentifier> predecessors = new HashList<VarIdentifier>(graph.get(selected));
						for (VarIdentifier pred : graph.get(selected)) {
							predecessors.addAll(graph.get(pred));
						}
						while (predecessors.size() > 0) {
							VarIdentifier pred = predecessors.get(0);
							predecessors.remove(pred);
							if (order.contains(graph.get(pred))) {
								order.add(pred);
								this.remove(graph, pred);
							} else {
								predecessors.add(pred);
							}
						}
						order.add(selected);
						this.remove(graph, selected);
						break;
					} else {
						S1 = S4;
					}
				} else {
					VarIdentifier selected = S1.get(0);
					order.add(selected);
					this.remove(graph, selected);
					break;
				}
			}
		}
		return order;
	}

	/**
	 * 
	 * @param graph
	 * @param agents
	 * @return
	 */
	private HashList<VarIdentifier> getRootsWithMinPredecessor(HashMap<VarIdentifier, HashList<VarIdentifier>> graph, HashList<VarIdentifier> agents) {
		int min = Integer.MAX_VALUE;
		for (VarIdentifier agId : agents) {
			if (graph.get(agId).size() < min) {
				min = graph.get(agId).size();
			}
		}
		HashList<VarIdentifier> ans = new HashList<VarIdentifier>();
		for (VarIdentifier agId : agents) {
			if (graph.get(agId).size() == min) {
				ans.add(agId);
			}
		}
		return ans;
	}

	/**
	 * 
	 * @param graph
	 * @param d
	 * @return
	 */
	private HashList<VarIdentifier> getAgentsWithMinRhs(HashMap<VarIdentifier, HashList<VarIdentifier>> graph, int d) {
		HashList<VarIdentifier> agents = new HashList<VarIdentifier>();
		for (VarIdentifier agId : graph.keySet()) {
			if (this.getRhs(agId) < d) {
				agents.add(agId);
				for (VarIdentifier pre : graph.get(agId)) {
					if (this.getRhs(pre) != d) {
						agents.remove(agId);
						break;
					}
				}
			}
		}
		return agents;
	}

	/**
	 * 
	 * @param roots
	 * @return
	 */
	private HashList<VarIdentifier> getMinRoots(HashList<VarIdentifier> allRoots) {
		int min = Integer.MAX_VALUE;
		for (VarIdentifier agId : allRoots) {
			if (this.getRhs(agId) < min) {
				min = this.getRhs(agId);
			}
		}
		HashList<VarIdentifier> roots = new HashList<VarIdentifier>();
		for (VarIdentifier agId : allRoots) {
			if (this.getRhs(agId) == min) {
				roots.add(agId);
			}
		}
		return roots;
	}

	/**
	 * 
	 * @param graph
	 * @param selected
	 */
	private void remove(HashMap<VarIdentifier, HashList<VarIdentifier>> graph, VarIdentifier selected) {
		for (VarIdentifier agId : graph.keySet()) {
			if (graph.get(agId).contains(selected)) {
				graph.get(agId).remove(selected);
			}
		}
		graph.remove(selected);
	}

	/**
	 * 
	 * @param graph
	 *            is the graph of predecessors
	 * @return
	 */
	private HashList<VarIdentifier> getRoots(HashMap<VarIdentifier, HashList<VarIdentifier>> graph) {
		HashList<VarIdentifier> roots = new HashList<VarIdentifier>();
		for (VarIdentifier agId : graph.keySet()) {
			if ((graph.get(agId).size() == 0)) {
				roots.add(agId);
			}
		}
		return roots;
	}

	/**
	 * 
	 * @return
	 */
	private HashMap<VarIdentifier, HashList<VarIdentifier>> getGraphSuccessor() {
		HashMap<VarIdentifier, HashList<VarIdentifier>> graph = new HashMap<VarIdentifier, HashList<VarIdentifier>>();
		for (VarIdentifier agId : this.agentIds) {
			graph.put(agId, new HashList<VarIdentifier>());
		}
		for (VarIdentifier agId : this.agentIds) {
			HashList<VarIdentifier> predecessors = this.getLhs(agId);
			for (VarIdentifier pre : predecessors) {
				graph.get(pre).add(agId);
			}
		}
		return graph;
	}

	/**
	 * 
	 * @return
	 */
	private HashMap<VarIdentifier, HashList<VarIdentifier>> getGraphPredecessor() {
		HashMap<VarIdentifier, HashList<VarIdentifier>> graph = new HashMap<VarIdentifier, HashList<VarIdentifier>>();
		for (VarIdentifier agId : this.agentIds) {
			HashList<VarIdentifier> predecessors = this.getLhs(agId);
			graph.put(agId, predecessors);
		}
		return graph;
	}

	public boolean isAcyclic() {
		for (VarIdentifier agId : this.agentIds) {
			HashList<VarIdentifier> predecessors = this.getLhs(agId);
			for (VarIdentifier pred : predecessors) {
				if (this.getLhs(pred).contains(agId)) {
					System.err.println("*******************************************\n" + agId +
							" :: predecessors" + predecessors + " :: pred: "
							+ pred + "\n" + this);
					return true;
				}
			}
		}
		return false;
	}

	public String toString(VarIdentifier varId) {
		int index = agentIds.indexOf(varId);
		return varId + " :: " + lhs[index]+" --> "+this.rhs[index];
	}

	public String toString() {
		String ans = "";
		for (VarIdentifier agId : this.agentIds) {
			ans += this.toString(agId) + "\n";
		}
		return ans;
	}

	public Double[] computeTV(HashList<VarIdentifier> order) {
		Double[] tv = new Double[order.size()];
		for (int i = 0 ; i < order.size() ; i++) {
			VarIdentifier varId = order.get(i);
			int index = agentIds.indexOf(varId);
			if (this.heuristic == MinDom) {
				tv[i] = new Double(this.getRhs(varId));
			} else if (this.heuristic == DomDeg){
				tv[i] = ((double) this.rhs[index]) / this.degrees.get(varId);
			} else if (this.heuristic == DomWDeg){
				tv[i] = ((double) this.rhs[index]) / (this.weights.get(varId)*this.degrees.get(varId));
			}
		}
		return tv;
	}
	
	/**
	 * @param agentId
	 */
	public int removeExplanationsContainning(VarIdentifier rhs) {
		int nbChecks = 0;
		int i = agentIds.indexOf(rhs);
		for(VarIdentifier varId : this.agentIds){
			int index = agentIds.indexOf(varId);
			if (this.lhs[index].get(i)) {
				this.removeExplanationOf(varId);
				nbChecks++;
			}
		}
		return nbChecks;
	}
	/**
	 * @param agId
	 * @param weight
	 */
	public void setWeights(VarIdentifier varId, int weight) {
		this.weights.put(varId, weight);
	}

}
