package simple.dcsp.agile.exp;

import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.MessageHandler;
import simple.dcsp.Nogood;
import util.Commands;
import util.HashList;
import util.Instantiation;
import util.Tools;
import util.VarAssignment;

/**
 * for each value we store just one Nogood following this heuristic: when comparing two nogoods, select the one with the highest possible lowest
 * variable involved (Nogood Selector HPLV)
 */

public class NogoodMsgHandler extends MessageHandler {

	public NogoodMsgHandler(AgileAgent agent) {
		super(agent, Commands.NOGOOD);
	}

	/**
	 * 
	 */
	public void processMessage(Message msg) throws StopException {
		VarIdentifier senderId = msg.getSenderId();
		HashList<VarIdentifier> order = HashList.fromString((String) msg.get(Commands.ORDER), VarIdentifier.class);
		Double[] TV = Tools.arrayFromString((String) msg.get(AgileAgent.TV), Double.class);
		int weight = (Integer) msg.get(Commands.WEIGHT);
		((AgileAgent) this.myAgent).setWeights(senderId, weight);
		((AgileAgent) this.myAgent).checkOrder(order, TV);
		Nogood ng = (Nogood) msg.get(Commands.NOGOOD);
		((AgileAgent) this.myAgent).updateAgentView(ng.getLhs());
		if (this.isCoherent(ng.getLhs(), ng.getRhs()) && this.isCompatibleWithMyOrder(ng)) {
			// ExplainedIntVar v = (ExplainedIntVar) ((AgileABTAgent) this.myAgent).var;
			((AgileAgent) this.myAgent).makeNoGoodConstraint(ng);
			((AgileAgent) this.myAgent).CheckAddLink(ng.getLhs());
		} else {
			this.myAgent.auxiliaryStatistics.get(Commands.nbObsoleteNogood).value++;
			if (!((AgileAgent) this.myAgent).IchangeMyOrder) {
				boolean value = ng.getRhsValue() == ((AgileAgent) this.myAgent).var.getValue();
				boolean isLower = this.myAgent.agentOrder.isLower(senderId);
				if (value && isLower) {
					Instantiation lhs = ((AgileAgent) this.myAgent).getExplanationLhs();
					int rhs = ((AgileAgent) this.myAgent).getExplanationRhs();
					((AgileAgent) this.myAgent).okMsgH.sendOkMessage(senderId, lhs, rhs);
				}
			}
		}
		((AgileAgent) this.myAgent).CheckAgentView();
	}

	/**
	 * @param nogood
	 */
	public void sendNogood(Nogood nogood) {
		Message msg = this.createMessage();
		HashList<VarIdentifier> order = this.myAgent.getOrder();
		Double[] tv = ((AgileAgent) this.myAgent).myTV.clone();
		msg.addContents(Commands.NOGOOD, nogood);
		msg.addContents(Commands.ORDER, order.toString());
		msg.addContents(AgileAgent.TV, Tools.arrayToString(tv));
		msg.addContents(Commands.WEIGHT, ((AgileAgent) this.myAgent).getMyWeight());
		msg.setReceiverId(nogood.getRhsVarIdentifier());
		((AgileAgent)this.myAgent).storeStatisticsFor(msg);
		this.myAgent.sendMessage(msg);
	}

	/**
	 * @param ng
	 * @param rhs
	 * @return
	 */
	private boolean isCompatibleWithMyOrder(Nogood ng) {
		HashList<VarIdentifier> higherAgs = this.myAgent.agentOrder.getHigherAgents();
		return higherAgs.contains(ng.getLhs().getVarIds());
	}

	/**
	 * @param rhs
	 * @param nogood
	 * @return
	 */
	public boolean isCoherent(Instantiation lhs, VarAssignment rhs) {
		int myValue = ((AgileAgent) this.myAgent).var.getValue();
		return this.myAgent.agentView.compatibleWith(lhs) && rhs.getValue() == myValue;
	}

}
