/************************************************************
Agile-ABT with termination value as heuristic for changing the 
total order among agents, and using the explanations defined 
as a conjunction of the stored nogoods. 
 *************************************************************/

package simple.dcsp.agileabt.explanation;

import static simple.dcsp.agileabt.explanation.AgileABTMaster.All;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.MinDom;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Neighbors;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.OrderNeigh;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.OrderOnly;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Topological_ID;
import java.util.HashMap;
import java.util.Iterator;
import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.AgentObserver;
import kernel.Communicator;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.MessageHandler;
import kernel.events.AgentEventListener;
import kernel.events.VariableEvent;
import kernel.order.AgentOrdering;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import simple.SingleExplainedVarAgent;
import simple.dcsp.Nogood;
import simple.dcsp.abtfamily.IdleEvent;
import simulation.metric.Metric;
import simulation.metric.MetricListener;
import simulation.parser.PVariable;
import util.Commands;
import util.HashList;
import util.Instantiation;
import util.Tools;
import util.VarAssignment;
import util.VarEventMask;

public class AgileABTAgent extends SingleExplainedVarAgent {
	//FIXME TV contient 0 de l'agent qui backtrack
	public static final String Explanation = "exp";
	public static final String ORDER = Commands.ORDER;
	public static final String OK = Commands.OK;
	public static final String NOGOOD = Commands.NOGOOD;
	public static final String DOMAIN_SIZE = Commands.DomainSize;
	public static final String TERMINATION_VALUE = "TV";
	public static final String ADDLINK = Commands.ADDLINK;
	public static final String LINK = "link";
	public static final String State = "state";
	public static final String EXP_CHECKS = "ExpChecks";
	public static final String NCEXP_CHECKS = "NCExpChecks";
	public static final String CHANGE_TARGET = "ChangeTarget";
	public static final String CHANGE_ORDER = "ChangeOrder";
	public static final String KEEP_TARGET = "KeepTarget";
	public static final String IMPROVE_TV = "ImproveTV";
	public static final String UPDATES_FROM_LHS = "UpdatesFromLhs";
	public static final String IMPROVE_TOPOLOGICAL_TV = "ImproveTopologicalTV";

	private short topologicalType = Topological_ID;

	private short heuristic = MinDom;

	public OkMsgHandler okMsgH = null;
	public NogoodMsgHandler ngdMsgH = null;
	public OrderMsgHandler orderMsgH = null;
	public AddLinkMsgHandler adlMsgH = null;

	public Explanations E;
	public Double[] myTV = null;
	private HashMap<VarIdentifier, Integer> domains;
	private HashMap<VarIdentifier, Integer> degrees;

	public boolean onlyOrder = false;
	public boolean onlyToNeighbors = false;
	public boolean IchangeMyOrder = false;
	public boolean topologicalOrder = false;
	private ExplanationObserver expChecksObserver;

	/**
	 * @param agentId
	 * @param agName
	 * @param description
	 * @param dom
	 * @param type
	 * @param topological
	 * @param topologicalType
	 */
	public AgileABTAgent(int agId, String name, String desc, short heuristic, short type, boolean topological, short topologicalType) {
		super(agId, name, desc);
		this.setType(type);
		this.heuristic = heuristic;
		this.topologicalOrder = topological;
		this.topologicalType = topologicalType;
		this.domains = new HashMap<VarIdentifier, Integer>();
		this.degrees = new HashMap<VarIdentifier, Integer>();
		this.agentOrder = new AgileABTAO(this);
		this.constructCommunicator();
		// -------------- Messages Handler ------------------//
		this.okMsgH = new OkMsgHandler(this);
		this.ngdMsgH = new NogoodMsgHandler(this);
		this.orderMsgH = new OrderMsgHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
		this.terminateMsgHandler = new AgileABTTermDetection(this);
		this.initAuxStatistics();
		this.expChecksObserver = new ExplanationObserver(NCEXP_CHECKS);
		this.addObserver(this.expChecksObserver);
	}
	
	/**
	 * @param agentId
	 * @param agName
	 * @param description
	 * @param dom
	 * @param type
	 * @param topological
	 * @param topologicalType
	 */
	public AgileABTAgent(PVariable variable, short heuristic, short type, boolean topological, short topologicalType) {
		super(variable);
		this.setType(type);
		this.heuristic = heuristic;
		this.topologicalOrder = topological;
		this.topologicalType = topologicalType;
		this.domains = new HashMap<VarIdentifier, Integer>();
		this.degrees = new HashMap<VarIdentifier, Integer>();
		this.agentOrder = new AgileABTAO(this);
		this.constructCommunicator();
		// -------------- Messages Handler ------------------//
		this.okMsgH = new OkMsgHandler(this);
		this.ngdMsgH = new NogoodMsgHandler(this);
		this.orderMsgH = new OrderMsgHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
		this.terminateMsgHandler = new AgileABTTermDetection(this);
		this.initAuxStatistics();
		this.expChecksObserver = new ExplanationObserver(NCEXP_CHECKS);
		this.addObserver(this.expChecksObserver);
	}

	private void initAuxStatistics() {
		this.auxiliaryStatistics.put(Commands.nbObsoleteNogood, new Metric(Commands.nbObsoleteNogood));
		this.auxiliaryStatistics.put(Commands.nbAssignments, new Metric(Commands.nbAssignments));
		this.auxiliaryStatistics.put(Commands.ObsoleteOk, new Metric(Commands.ObsoleteOk));
		this.auxiliaryStatistics.put(CHANGE_ORDER, new Metric(CHANGE_ORDER, 0L, Metric.max));
		this.auxiliaryStatistics.put(CHANGE_TARGET, new Metric(CHANGE_TARGET));
		this.auxiliaryStatistics.put(KEEP_TARGET, new Metric(KEEP_TARGET));
		this.auxiliaryStatistics.put(IMPROVE_TV, new Metric(IMPROVE_TV));
		this.auxiliaryStatistics.put(UPDATES_FROM_LHS, new Metric(UPDATES_FROM_LHS));
		this.auxiliaryStatistics.put(IMPROVE_TOPOLOGICAL_TV, new Metric(IMPROVE_TOPOLOGICAL_TV));
		this.auxiliaryStatistics.put(EXP_CHECKS, new Metric(EXP_CHECKS));
		this.auxiliaryStatistics.put(Commands.MSGsSIZE, new Metric(Commands.MSGsSIZE));
		this.auxiliaryStatistics.put(Commands.MaxMSGsSIZE, new Metric(Commands.MaxMSGsSIZE, 0L, Metric.max));
	}

	/**
	 * @param msg
	 */
	public void storeStatisticsFor(Message msg) {
		long size = msg.getSize();
		this.auxiliaryStatistics.get(Commands.MSGsSIZE).value += size;
		if(this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value < size){
			this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value = size;
		}
	}

	private void setType(short type) {
		switch (type) {
			case All :
				this.onlyToNeighbors = false;
				this.onlyOrder = false;
			break;
			case OrderOnly :
				this.onlyToNeighbors = false;
				this.onlyOrder = true;
			break;
			case Neighbors :
				this.onlyToNeighbors = true;
				this.onlyOrder = false;
			break;
			case OrderNeigh :
				this.onlyToNeighbors = true;
				this.onlyOrder = true;
			break;
		}
	}

	/**
	 * 
	 */
	private void constructCommunicator() {
		this.communicator = new Communicator(this) {
			protected boolean waitForArrivalMessages(long msTimeout) throws InterruptedException {
				logDomains();
				this.waitingForMessage = true;
				boolean arrived = false;
				if (((AgileABTAgent) this.myAgent).isIdle()) {
					arrived = mailBoxLock.waitUntilTrue(10);
					if (!arrived) {
						this.myAgent.fireAgentEvent(new IdleEvent(this.myAgent));
					}
				}
				if (!arrived) {
					arrived = mailBoxLock.waitUntilTrue(msTimeout);
				}
				this.waitingForMessage = false;
				logDomains();
				return arrived;
			}
		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see AbstractAgent#addMsgHandlers()
	 */
	@Override
	public void addMsgHandlers() {
		this.communicator.addMsgHandler(this.okMsgH);
		this.communicator.addMsgHandler(this.ngdMsgH);
		this.communicator.addMsgHandler(this.orderMsgH);
		this.communicator.addMsgHandler(this.adlMsgH);
	}

	@Override
	public void configurePropagationEngine() {
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.INSTANCIATE);
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.DESINSTANCIATE);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_THE_LAST).add(VariableEvent.RESTORE_VAL);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_BETWEEN_L_H).add(VariableEvent.RESTORE_VAL);
		this.constraintHandler.update();
	}

	public void initAgentView() {
		HashList<VarIdentifier> agents = this.getOrder();
		agents.remove(this.getVarIdentifier());
		for (VarIdentifier agId : agents) {
			if (!this.communicator.isNeighbor(agId)) {
				VarAssignment var = new VarAssignment(agId, Integer.MIN_VALUE);
				IntExternalVar extVar = this.makeExternalVar(var);
				this.agentView.add(extVar);
			} else{
				this.agentView.add((IntExternalVar) this.getExternalVar(agId));
			}
		}
	}

	public boolean isIdle() {
		if (this.disSearchStarted) {
			HashList<VarIdentifier> higherN = this.communicator.getHigherNeighbors();
			Instantiation view = this.agentView.getContextsOfHigherAgents();
			boolean idle = view.getVarIds().contains(higherN) && this.var.isInstantiated();
			if (idle) {
				view.add(this.var.getVarAssignment());
				((AgileABTTermDetection) this.terminateMsgHandler).SetMyState(this.getOrder(), getMyTV(), view);
			}
			return idle;
		}
		return false;
	}

	@Override
	protected void startDistributedSearch() throws StopException {
		this.explanationsConstructor();
		this.addAgentEventListener((AgentEventListener) this.terminateMsgHandler);
		((AgileABTTermDetection) this.terminateMsgHandler).initialize(this.getOrder(), getMyTV());
		this.CheckAgentView();
	}

	// ================================================================================================//
	private void explanationsConstructor() {
		this.E = new Explanations(this.getOrder(), this.domains, this.degrees, this.heuristic);
		this.myTV = E.computeTV(this.getOrder());
	}

	// ================================================================================================//
	/**
	 * 
	 */
	public Explanation getMyExplanation() {
		Instantiation lhs = this.solveMyNogoodStore();
		int rhs = this.var.getDomainSize();
		Explanation myExp = new Explanation(lhs, rhs);
		this.E.setExplanation(this.getVarIdentifier(), myExp);
		return myExp;
	}

	// ================================================================================================//
	/**
	 * @return
	 */
	protected boolean isMyExplanationChanged() {
		if (this.onlyOrder) { return !this.onlyOrder; }
		Explanation oldExp = this.E.getExplanation(this.getVarIdentifier());
		Explanation newExp = this.getMyExplanation();
		return !oldExp.equals(newExp);
	}

	// ================================================================================================//
	/**
	 * choose an instantiation (i.e a value) compatible with the agentView
	 * 
	 * @throws StopException
	 */
	public void CheckAgentView() throws StopException {
		this.constraintHandler.propagate();
		int value = this.var.getValue();
		if (!this.var.getDomain().contains(value)) {
			if (this.valSelector.hasNextValue()) {
				int myValue = this.valSelector.getNextValue();
				this.var.instantiate(myValue);
				this.okMsgH.sendOkMessages();
			} else {
				this.Backtrack();
			}
		} else {
			if (this.IchangeMyOrder || this.isMyExplanationChanged()) {
				this.okMsgH.sendOkMessages();
			}
		}
	}

	// ================================================================================================//
	/**
	 * Procedure to check if a new link should be added.
	 * 
	 * @param nogood
	 */
	public void CheckAddLink(Instantiation nogood) {
		if (this.onlyToNeighbors) {
			Iterator<VarAssignment> it = nogood.iterator();
			while (it.hasNext()) {
				VarAssignment ass = it.next();
				if (!this.communicator.isNeighbor(ass.getVarIdentifier())) {
					this.communicator.addNeighbor(ass.getVarIdentifier());
					this.agentView.add(this.makeExternalVar(ass));
					this.adlMsgH.sendAddLink(ass);
				}
			}
		}
	}

	// ================================================================================================//
	/**
	 * 
	 */
	private boolean isCompatibleWithMyOrder(VarIdentifier agId, Instantiation instantiation) {
		HashList<VarIdentifier> predecessors = instantiation.getVarIds();
		HashList<VarIdentifier> higherAgents = this.agentOrder.getAgentsHigherThan(agId);
		return higherAgents.contains(predecessors);
	}

	// ================================================================================================//
	/**
	 * If the received explanation is newest take it
	 */
	public boolean updateExplanation(VarIdentifier agId, Explanation exp) {
		boolean viewCompatible = this.agentView.compatibleWith(exp.getLhs());
		boolean orderCompatible = this.isCompatibleWithMyOrder(agId, exp.getLhs());
		if (viewCompatible && orderCompatible) {// && this.agentOrder.isHigher(agId)){
			this.E.setExplanation(agId, exp);
			if (exp.getLhs().isEmpty()) {
				this.E.updateMaxDomainSize(agId, exp.getRhs());
			}
			return true;
		} else {
			this.E.removeExplanationOf(agId);
		}
		return false;
	}

	// ================================================================================================//
	public void updateAgentView(Instantiation inst) {
		Iterator<VarAssignment> it = inst.iterator();
		while (it.hasNext()) {
			VarAssignment varAss = it.next();
			if (this.agentView.compareTimeCounter(varAss) > 0 && this.agentOrder.isHigher(varAss.getVarIdentifier())) {
				this.agentView.priorityMerge(varAss);
				this.auxiliaryStatistics.get(UPDATES_FROM_LHS).value++;
			}
		}
		this.E.removeExplanationsIncompatibleWith(this.agentView.toInstantiation());
	}

	// ================================================================================================//
	public void updateAgentView(Instantiation inst, VarAssignment ass) {
		if (this.agentView.compareTimeCounter(ass) >= 0) {
			this.agentView.priorityMerge(ass);
			this.E.removeExplanationOf(ass.getVarIdentifier());
		} else {
			this.auxiliaryStatistics.get(Commands.ObsoleteOk).value++;
		}
		this.updateAgentView(inst);
	}

	// ================================================================================================//
	private boolean setNewOrder(HashList<VarIdentifier> order, Double[] tv) {
		System.arraycopy(tv, 0, this.myTV, 0, tv.length);
		if (this.agentOrder.setNewOrder(order)) {
			this.auxiliaryStatistics.get(CHANGE_ORDER).value++;
			this.IchangeMyOrder = true;
			this.E.refresh(this.agentOrder, getVarIdentifier());
			this.constraintHandler.update();
			return true;
		}
		return false;
	}

	// ================================================================================================//
	/**
	 * @param
	 * @return <!--
	 * 
	 *         <pre>
	 * 		<b><i>true</i></b> if <i>{order,tv}</i> are better than <i>{myOrder,myTV}</i>
	 * 		<b><i>false</i></b> otherwise 
	 * <!--
	 * </pre>
	 */
	private boolean isItBestOrder(HashList<VarIdentifier> myOrder, Double myTV[], HashList<VarIdentifier> order, Double[] tv) {
		int cmp = Tools.compare(myTV, tv);
		if (cmp > 0)
			return false;
		if (cmp < 0)
			return true;
		if (myOrder.compare(order) < 0) {
			return true;
		} else {
			return false;
		}
	}

	// ================================================================================================//
	public boolean checkOrder(HashList<VarIdentifier> order, Double tv[]) {
		if (this.isItBestOrder(this.getOrder(), this.myTV, order, tv)) {
			return this.setNewOrder(order, tv);
		}
		return false;
	}

	// ================================================================================================//
	private void removeAssignment(VarAssignment varAss) {
		this.agentView.clear(varAss);
		this.E.remove(varAss);
	}

	protected void incrementNumConstraintChecks(int CCs) {
		for (AgentObserver observer : observers) {
			if (observer instanceof MetricListener) {
				((MetricListener) observer).handleIncrementNbConstraintChecks(CCs);
			}
		}
		this.expChecksObserver.handleIncrementNbConstraintChecks(CCs);
	}

	// ================================================================================================//
	public Explanations updateExplanations(Explanations E, Instantiation ngLhs, VarAssignment x_i, HashMap<Integer, Instantiation> NGs) {
		Explanations E_i = E.clone();
		int nbChecks = E_i.remove(x_i);
		this.expChecksObserver.handleIncrementNbConstraintChecks(nbChecks / 3);
		this.auxiliaryStatistics.get(EXP_CHECKS).value += nbChecks;
		VarIdentifier i = x_i.getVarIdentifier();
		Instantiation lhs_i = new Instantiation();
		lhs_i.priorityMerge(E_i.getLhs(i).clone());
		lhs_i.priorityMerge(ngLhs);
		int rhs_i = E_i.getRhs(i) - 1;
		E_i.setExplanation(i, new Explanation(lhs_i, rhs_i));
		Instantiation myLhs = ngLhs.clone();
		int myRhs = this.getDomainSize(NGs);
		E_i.setExplanation(this.getVarIdentifier(), new Explanation(myLhs, myRhs));
		return E_i;
	}

	// ================================================================================================//
	private HashMap<Integer, Instantiation> removeFrom(HashMap<Integer, Instantiation> NGs, VarAssignment xi) {
		HashMap<Integer, Instantiation> ans = new HashMap<Integer, Instantiation>();
		for (int value : NGs.keySet()) {
			Instantiation ng = NGs.get(value);
			if ((ng.contains(xi.getVarIdentifier()) && ng.compatible(xi)) || !ng.contains(xi.getVarIdentifier())) {
				ans.put(value, ng);
			}
		}
		return ans;
	}

	// ================================================================================================//
	private Explanations chooseVariableOrder(Explanations E, Instantiation nogood, HashMap<Integer, Instantiation> NGs) throws StopException {
		if (nogood.isEmpty()) { throw new StopException(Commands.FAILURE); }
		Explanations selE = null, E_i = null;
		HashList<VarIdentifier> order_i = null;
		Double[] TV_i = this.getMaximumTV(), selTV = this.getMaximumTV();
		VarAssignment x_i = null;
		Instantiation lhs = null, ng_i = null;
		HashMap<Integer, Instantiation> NGs_i = null;
		for (int i = 0 ; i < nogood.size() ; i++) {
			x_i = nogood.get(i);
			VarIdentifier agId = x_i.getVarIdentifier();
			lhs = nogood.clone();
			lhs.remove(x_i);
			NGs_i = this.removeFrom(NGs, x_i);
			E_i = this.updateExplanations(E, lhs, x_i, NGs_i);
			if (E_i.getRhs(agId) == 0) { // It not necessary to backtrack to agentId
				ng_i = E_i.getLhs(agId);
				E_i = this.chooseVariableOrder(E_i, ng_i, NGs_i);
				order_i = E_i.getNewOrder();
				TV_i = E_i.getNewTV();
				ng_i = E_i.getNewNG();
			} else {
				order_i = E_i.getTopologicalOrder(this.topologicalType);
				TV_i = E_i.computeTV(order_i);
				// if (!this.topologicalOrder) {
				// double[] topologicalTV = TV_i.clone();
				// order_i = E_i.getBestOrder();
				// TV_i = E_i.computeTV(order_i);
				// if (Utility.compare(topologicalTV, TV_i) < 0) {
				// this.auxiliaryStatistics.get(IMPROVE_TOPOLOGICAL_TV).value++;
				// }
				// }
				ng_i = nogood.clone();
			}
			if (Tools.compare(selTV, TV_i) < 0) {
				selE = E_i;
				selTV = TV_i;
				selE.setNewOrder(order_i);
				selE.setNewTV(selTV);
				selE.setNewNG(ng_i);
			}
		}
		return selE;
	}

	// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	public void Backtrack() throws StopException {
		HashMap<Integer, Instantiation> nogoods = this.getNogoods();
		Instantiation nogood = this.solveMyNogoodStore();
		if (nogood.isEmpty()) { throw new StopException(Commands.FAILURE); }
		VarIdentifier target = nogood.last().getVarIdentifier();
		this.E.removeExplanationOf(this.getVarIdentifier());
		Explanations E = this.chooseVariableOrder(this.E, nogood, nogoods);
		if (Tools.compare(this.myTV, E.getNewTV()) < 0) {
			// a new order is found
			this.E = E;
			this.setNewOrder(E.getNewOrder(), E.getNewTV());
			Nogood ng = new Nogood(E.getNewNG().clone(this.agentOrder));
			this.ngdMsgH.sendNogood(ng);
			VarIdentifier backTo = ng.getRhsVarIdentifier();
			this.removeAssignment(ng.getRhs());
			this.sendOrderMessages(backTo);
			if (ng.getRhsVarIdentifier().equals(target)) {
				this.auxiliaryStatistics.get(KEEP_TARGET).value++;
			} else {
				this.auxiliaryStatistics.get(CHANGE_TARGET).value++;
			}
			this.auxiliaryStatistics.get(IMPROVE_TV).value++;
		} else {
			Nogood ng = new Nogood(E.getNewNG().clone(this.agentOrder));
			this.ngdMsgH.sendNogood(ng);
			this.auxiliaryStatistics.get(KEEP_TARGET).value++;
			this.removeAssignment(ng.getRhs());
		}
		this.E.clear();
		this.CheckAgentView();
	}

	/**
	 * 
	 */
	private void sendOrderMessages(VarIdentifier exclude) {
		this.orderMsgH.broadcastOrder(exclude);
	}

	private HashMap<Integer, Instantiation> getNogoods() {
		HashMap<Integer, Instantiation> nogoods = new HashMap<Integer, Instantiation>();
		Iterator<Integer> it = this.var.getDomain().originalDomainIterator();
		while (it.hasNext()) {
			int value = it.next();
			Instantiation ng = this.makeBetterNogood((ExplainedIntVar) var, value).getLhs();
			nogoods.put(value, ng);
		}
		return nogoods;
	}

	protected int getDomainSize(HashMap<Integer, Instantiation> nogoods) {
		return this.domains.get(this.getVarIdentifier()) - nogoods.size();
	}

	public Double[] getMyTV() {
		return this.myTV.clone();
	}

	public void setDom(VarIdentifier agId, int domain) {
		this.domains.put(agId, domain);
	}

	public void setDeg(VarIdentifier agId, int degree) {
		this.degrees.put(agId, degree);
	}

	private Double[] getMaximumTV() {
		Double[] tv = new Double[this.getOrder().size()];
		for (int i = 0 ; i < tv.length ; i++) {
			tv[i] = Double.MAX_VALUE;
		}
		return tv;
	}

	// =========================================================================================================================/
	public class AgileABTAO extends AgentOrdering {
		public AgileABTAO(AbstractAgent agent) {
			super(agent);
		}

		public void processOrder(Message msg) {
			super.processOrder(msg);
			HashMap<VarIdentifier, Integer> doms = Tools.mapfromString((String) msg.get(DOMAIN_SIZE), VarIdentifier.class, Integer.class);
			domains.putAll(doms);
			for (VarIdentifier agId : this.agentsOrder) {
				int degree = this.graph.getNeighbors(agId).size();
				setDeg(agId, degree);
			}
		}
	}
	
	public class OkMsgHandler extends MessageHandler {

		public OkMsgHandler(AgileABTAgent agent) {
			super(agent, OK);
		}

		/**
		 * @param msg
		 */
		public void processMessage(Message msg) throws StopException {
			VarAssignment ass = (VarAssignment) msg.get(OK);
			HashList<VarIdentifier> order = HashList.fromString((String) msg.get(ORDER), VarIdentifier.class);
			Double[] TV = Tools.arrayFromString((String) msg.get(TERMINATION_VALUE), Double.class);
			((AgileABTAgent) this.myAgent).checkOrder(order, TV);
			Explanation exp = (Explanation) msg.get(Explanation);
			updateAgentView(exp.getLhs(), ass);
			updateExplanation(ass.getVarIdentifier(), exp);
			CheckAgentView();
		}

		/**
		 * 
		 */
		public void sendOkMessages() {
			Explanation explanation = getMyExplanation();
			HashList<VarIdentifier> receivers = null;
			if (onlyToNeighbors) {
				receivers = communicator.getLowerNeighbors();
			} else {
				receivers = agentOrder.getLowerAgents();
			}
			for(VarIdentifier lower : receivers) {
				this.sendOkMessage(lower, explanation);
			}
			IchangeMyOrder = false;
		}

		/**
		 * 
		 * @param neig
		 * @param myExp
		 */
		public void sendOkMessage(VarIdentifier neig, Explanation myExp) {
			HashList<VarIdentifier> myOrder = getOrder();
			String myTV = Tools.arrayToString(getMyTV());
			Message msg = this.createMessage();
			msg.addContents(OK, var.getVarAssignment());
			msg.addContents(Explanation, myExp);
			msg.addContents(ORDER, myOrder.toString());
			msg.addContents(TERMINATION_VALUE, myTV);
			msg.setReceiverId(neig);
			storeStatisticsFor(msg);
			sendMessage(msg);
		}

	}

	public class OrderMsgHandler extends MessageHandler {

		public OrderMsgHandler(AbstractAgent agent) {
			super(agent, ORDER);
		}

		@Override
		public void processMessage(Message msg) throws StopException {
			HashList<VarIdentifier> order = HashList.fromString((String) msg.get(ORDER), VarIdentifier.class);
			Double[] tv = Tools.arrayFromString((String) msg.get(TERMINATION_VALUE), Double.class);
			checkOrder(order, tv);
			CheckAgentView();
		}

		/**
		 * 
		 * @param agId
		 */
		public void broadcastOrder(VarIdentifier exclude) {
			HashList<VarIdentifier> order = getOrder();
			String TV = Tools.arrayToString(getMyTV());
			String newOrder = order.toString();
			order.remove(exclude);
			order.remove(getVarIdentifier());
			for(VarIdentifier receiver : order) {
				this.sendOrderMessage(newOrder, TV, receiver);
			}
			this.sendOrderMessage(newOrder, TV, AbstractMaster.MasterVarId);
		}

		/**
		 * @param order
		 * @param TV
		 * @param receiver
		 */
		protected void sendOrderMessage(String order, String tv, VarIdentifier receiver) {
			Message msg = createMessage();
			msg.addContents(ORDER, order);
			msg.addContents(TERMINATION_VALUE, tv);
			msg.setReceiverId(receiver);
			storeStatisticsFor(msg);
			sendMessage(msg);
		}
	}
	

	public class NogoodMsgHandler extends MessageHandler {

		public NogoodMsgHandler(AgileABTAgent agent) {
			super(agent, NOGOOD);
		}

		/**
		 * 
		 */
		public void processMessage(Message msg) throws StopException {
			VarIdentifier senderId = msg.getSenderId();
			HashList<VarIdentifier> order = HashList.fromString((String) msg.get(ORDER), VarIdentifier.class);
			Double[] TV = Tools.arrayFromString((String) msg.get(TERMINATION_VALUE), Double.class);
			checkOrder(order, TV);
			Nogood ng = (Nogood) msg.get(NOGOOD);
			updateAgentView(ng.getLhs());
			if (this.isCoherent(ng.getLhs(), ng.getRhs()) && this.isCompatibleWithMyOrder(ng)) {
				makeNoGoodConstraint(ng);
				CheckAddLink(ng.getLhs());
			} else {
				auxiliaryStatistics.get(Commands.nbObsoleteNogood).value++;
				if (!IchangeMyOrder) {
					boolean value = ng.getRhsValue() == var.getValue();
					boolean isLower = agentOrder.isLower(senderId);
					if (value && isLower) {
						Explanation myExp = getMyExplanation();
						okMsgH.sendOkMessage(senderId, myExp);
					}
				}
			}
			CheckAgentView();
		}

		/**
		 * @param nogood
		 */
		public void sendNogood(Nogood nogood) {
			String myTV = Tools.arrayToString(getMyTV());
			Message msg = this.createMessage();
			msg.addContents(NOGOOD, nogood);
			msg.addContents(ORDER, getOrder().toString());
			msg.addContents(TERMINATION_VALUE, myTV);
			msg.setReceiverId(nogood.getRhsVarIdentifier());
			storeStatisticsFor(msg);
			sendMessage(msg);
		}

		/**
		 * @param ng
		 * @param rhs
		 * @return
		 */
		private boolean isCompatibleWithMyOrder(Nogood ng) {
			HashList<VarIdentifier> higherAgs = agentOrder.getHigherAgents();
			return higherAgs.contains(ng.getLhs().getVarIds());
		}

		/**
		 * @param rhs
		 * @param nogood
		 * @return
		 */
		public boolean isCoherent(Instantiation lhs, VarAssignment rhs) {
			int myValue = var.getValue();
			return agentView.compatibleWith(lhs) && rhs.getValue() == myValue;
		}
	}

	public class AddLinkMsgHandler extends MessageHandler {

		public AddLinkMsgHandler(AbstractAgent agent) {
			super(agent, ADDLINK);
		}

		/**
		 * @param msg
		 */
		public void processMessage(Message msg) throws StopException {
			VarAssignment myAss = (VarAssignment) msg.get(LINK);
			VarIdentifier senderId = msg.getSenderId();
			communicator.addNeighbor(senderId);
			if (myAss.getValue() != var.getValue()) { /* re-send OK message */
				Explanation myExp = getMyExplanation();
				okMsgH.sendOkMessage(senderId, myExp);
			}
		}

		/**
		 * 
		 * @param ass
		 */
		public void sendAddLink(VarAssignment ass) {
			Message msg = this.createMessage();
			VarAssignment myAssignment = var.getVarAssignment();
			msg.addContents(LINK, ass);
			msg.addContents(OK, myAssignment);
			msg.setReceiverId(ass.getVarIdentifier());
			storeStatisticsFor(msg);
			sendMessage(msg);
		}
	}
	
}
