package simple.dcsp.agileabt.explanation;

import java.util.HashMap;
import java.util.Iterator;
import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.TerminateMsgHandler;
import kernel.events.AgentEvent;
import simple.dcsp.abtfamily.IdleEvent;
import simple.dcsp.agileabt.StateOfAgent;
import util.BooleanLock;
import util.HashList;
import util.Instantiation;
import util.Logger;
import util.Tools;
import util.VarAssignment;

public class AgileABTTermDetection extends TerminateMsgHandler {

	protected HashMap<VarIdentifier, StateOfAgent> stateOfAgents = null;
	protected BooleanLock whenIdleLock = new BooleanLock(false);
	protected StateOfAgent prevSentAccepted = null;
	protected HashList<VarIdentifier> lastReceivers;
	protected HashList<VarIdentifier> bestOrder;
	protected Double[] bestTV = null;

	public AgileABTTermDetection(AbstractAgent agent) {
		super(agent);
		this.stateOfAgents = new HashMap<VarIdentifier, StateOfAgent>();
	}

	public void initialize(HashList<VarIdentifier> order, Double[] tv ) {
		this.stateOfAgents.put(this.myAgent.getVarIdentifier(), new StateOfAgent(order, tv));
		this.bestTV = tv;
		this.SetBestOrder(order, tv);
		this.prevSentAccepted = null;
		this.lastReceivers = new HashList<VarIdentifier>();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.TerminateMsgHandler#processTerminationDetection(.com.Message)
	 */
	public boolean processTerminationDetection(Message msg) throws StopException {
		VarIdentifier senderId = msg.getSenderId();
		StateOfAgent receivedState = (StateOfAgent) msg.get(AgileABTAgent.State);
		this.setState(senderId, receivedState);
		HashList<VarIdentifier> lowerNeighbors = this.myAgent.communicator.getLowerNeighbors();
		lowerNeighbors = HashList.intersection(lowerNeighbors, this.myAgent.communicator.getNeighbors0());
		if (this.stateOfAgents.keySet().containsAll(lowerNeighbors)) {
			this.tryToSendAccepted();
		} else {
			this.myAgent.log("####Not yet received from all lower neighbors "+lowerNeighbors, Logger.FINE_LEVEL);
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.TerminateMsgHandler#handleAgentEvent(events.AgentEvent )
	 */
	@Override
	public void handleAgentEvent(AgentEvent agentEvent) {
		if (agentEvent instanceof IdleEvent && !this.myAgent.isFinished()) {
			this.tryToSendAccepted();
		}
		if (agentEvent instanceof kernel.events.AgentFinishedEvent) {
			this.askMasterToTerminate(true);
		}
	}

	/**
	 * 
	 * @param agId
	 * @param receivedState
	 */
	public void setState(VarIdentifier agId, StateOfAgent state) {
		HashList<VarIdentifier> order = state.getOrder();
		Double[] tv = state.getTV();
		int cmp = StateOfAgent.areAcceptable(this.bestOrder, this.bestTV, order, tv);
		this.myAgent.log("@@@@ cmp "+cmp+" I try to set the state of "+agId+", the state "+state, Logger.FINE_LEVEL);
		if (cmp < 0) {
			this.myAgent.log("@@@@ I set a new best order from "+agId+" the state "+state, Logger.FINE_LEVEL);
			this.SetBestOrder(order, tv);
			this.stateOfAgents.put(agId, state);
			this.myAgent.log("@@@@ I added the state of "+agId+" to the states map "+this.stateOfAgents, Logger.FINE_LEVEL);
		} else if(cmp==0) {
			this.stateOfAgents.put(agId, state);
			this.myAgent.log("@@@@ I added the state of "+agId+" to the states map "+this.stateOfAgents, Logger.FINE_LEVEL);
		} else {
			this.myAgent.log("$$$$ cmp="+cmp+" bestOrder="+this.bestOrder+"@"+Tools.arrayToString(this.bestTV)+"\n"+
					"is better than order="+order+"@"+Tools.arrayToString(tv), Logger.FINE_LEVEL);
		}
	}

	/**
	 * 
	 * @param order
	 * @param tv
	 */
	public void SetBestOrder(HashList<VarIdentifier> order, Double[] tv) {
		this.bestOrder = order;
		System.arraycopy(tv, 0, this.bestTV, 0, tv.length);
		Iterator<VarIdentifier> it = this.stateOfAgents.keySet().iterator();
		while (it.hasNext()) {
			VarIdentifier agId = it.next();
			StateOfAgent state = this.stateOfAgents.get(agId);
			if (StateOfAgent.areAcceptable(this.bestOrder, this.bestTV, state.getOrder(), state.getTV()) > 0) {
				this.myAgent.log("####I remove the state of "+agId+", i.e., "+state+"\nbecause is worst than best order "+
						this.bestOrder+"@"+Tools.arrayToString(this.bestTV), Logger.FINE_LEVEL);
				it.remove();
			}
		}
		this.myAgent.log("####After setting the best order " + order + "@" + Tools.arrayToString(this.bestTV) 
				+ " " + this.stateOfAgents, Logger.FINE_LEVEL);
	}

	/**
	 * 
	 */
	public void tryToSendAccepted() {
		this.myAgent.log("####I try to send a state to my higher neighbors", Logger.FINE_LEVEL);
		StateOfAgent state = this.buildAccepted();
		if (state != null) {
			if (this.IalrealdySentTheSame(state)) {
				this.myAgent.log("####I alrealdy sent the same state "+state, Logger.FINE_LEVEL);
				return;
			}
			this.prevSentAccepted = state;
			this.lastReceivers = new HashList<VarIdentifier>();
			HashList<VarIdentifier> higherNeighbors = this.myAgent.communicator.getHigherNeighbors();
			higherNeighbors = HashList.intersection(higherNeighbors, this.myAgent.communicator.getNeighbors0());
			this.myAgent.log("####I should send my new state "+state+" to all my higher neighbors "+higherNeighbors, Logger.FINE_LEVEL);
			if (higherNeighbors.size() == 0) {
				this.lastReceivers.add(AbstractMaster.MasterVarId);
				this.sendAccepted(state, AbstractMaster.MasterVarId);
			} else {
				this.lastReceivers = new HashList<VarIdentifier>(higherNeighbors);
				for (VarIdentifier highNeig : higherNeighbors) {
					this.sendAccepted(state, highNeig);
				}
			}
		}
	}

	/**
	 * 
	 * @param order
	 * @param tv
	 * @param agentView
	 */
	public void SetMyState(HashList<VarIdentifier> order, Double[] tv, Instantiation agentView) {
		StateOfAgent myState = new StateOfAgent(order, tv, agentView);
		this.setState(this.myAgent.getVarIdentifier(), myState);
	}

	/**
	 * 
	 * @param accepted
	 * @return
	 */
	private boolean IalrealdySentTheSame(StateOfAgent accepted) {
		boolean samePrev = this.prevSentAccepted != null && this.prevSentAccepted.equals(accepted);
		HashList<VarIdentifier> higherNeighbors = this.myAgent.communicator.getHigherNeighbors();
		higherNeighbors = HashList.intersection(higherNeighbors, this.myAgent.communicator.getNeighbors0());
		boolean sameReceiver = false;
		if (higherNeighbors.size() == 0) {
			sameReceiver = this.lastReceivers.contains(AbstractMaster.MasterVarId) && this.lastReceivers.size() == 1;
		} else {
			sameReceiver = this.lastReceivers.equals(higherNeighbors);
		}
		return (samePrev && sameReceiver);
	}

	/**
	 * 
	 * @return
	 */
	protected synchronized StateOfAgent buildAccepted() {
		if (!this.stateOfAgents.containsKey(this.myAgent.getVarIdentifier())) {
			if (!((AgileABTAgent) this.myAgent).isIdle()) {
				this.myAgent.log("####I tried to build state but I am not Idle", Logger.FINE_LEVEL);
				return null;
			}
			Instantiation myView = this.myAgent.agentView.getContextsOfHigherAgents();
			VarAssignment myAss = ((AgileABTAgent) this.myAgent).var.getVarAssignment();
			myView.add(myAss);
			HashList<VarIdentifier> myOrder = ((AgileABTAgent) this.myAgent).getOrder();
			Double[] myTV = ((AgileABTAgent) this.myAgent).getMyTV();
			this.SetMyState(myOrder, myTV, myView);
			if (!this.stateOfAgents.containsKey(this.myAgent.getVarIdentifier())) {
				this.myAgent.log("####state of agents dosn't contains my Key "+this.stateOfAgents, Logger.FINE_LEVEL);
				return null;
			}
		}
		StateOfAgent accepted = this.stateOfAgents.get(this.myAgent.getVarIdentifier()).clone();
		HashList<VarIdentifier> lowerNeighbors = this.myAgent.communicator.getLowerNeighbors();
		lowerNeighbors = HashList.intersection(lowerNeighbors, this.myAgent.communicator.getNeighbors0());
		for (VarIdentifier lower : lowerNeighbors) {
			if (!this.stateOfAgents.containsKey(lower)) {
				this.myAgent.log("####I didn't received a state from "+lower+" "+this.stateOfAgents, Logger.FINE_LEVEL);
				return null;
			}
			StateOfAgent stateOfAgent = this.stateOfAgents.get(lower);
			if (accepted.isCompatible(stateOfAgent)) {
				accepted.priorityMerge(stateOfAgent.getAgentView());
			} else {
				this.myAgent.log("####Agents' states are not compatible "+this.stateOfAgents, Logger.FINE_LEVEL);
				return null;
			}
		}
		return accepted;
	}

	/**
	 * 
	 * @param accepted
	 * @param receiverId
	 */
	protected void sendAccepted(StateOfAgent state, VarIdentifier receiverId) {
		Message msg = this.createMessage();
		msg.addContents(AgileABTAgent.State, state.clone());
		msg.setReceiverId(receiverId);
		this.myAgent.sendMessage(msg);
	}

	/**
	 * 
	 */
	public String toString() {
		String ans = "\nOrder ==> \n" + this.bestOrder + "@" + Tools.arrayToString(this.bestTV) + "\n";
		Iterator<VarIdentifier> it = this.stateOfAgents.keySet().iterator();
		while (it.hasNext()) {
			VarIdentifier agId = it.next();
			StateOfAgent state = this.stateOfAgents.get(agId);
			ans += "" + agId + "=" + state.toString() + "\n";
		}
		return ans;
	}
}
