/**
 * 
 */
package simple.dcsp.agileabt.explanation;

import java.io.Serializable;
import kernel.VarIdentifier;
import kernel.order.AgentOrdering;
import simple.SimpleAgent;
import util.HashList;
import util.Instantiation;

public class Explanation implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7805801284263701760L;
	private Instantiation lhs;
	private int rhs;

	public Explanation(SimpleAgent agent, int rhs) {
		this.lhs = new Instantiation(agent.agentOrder);
		this.setRhs(rhs);
	}

	public Explanation(int rhs) {
		this.lhs = new Instantiation();
		this.setRhs(rhs);
	}

	public Explanation(Instantiation lhs, int rhs) {
		this.setLhs(lhs);
		this.setRhs(rhs);
	}

	public void setRhs(int rhs) {
		this.rhs = rhs;
		if (this.rhs < 0) { throw new Error("the rhs of an explanation cannot be less than 0"); }
	}

	public int getRhs() {
		return this.rhs;
	}

	public void setLhs(Instantiation lhs) {
		this.lhs = lhs.cloneUseingSelfComparator();
	}

	public Instantiation getLhs() {
		return this.lhs.clone();
	}

	public void clear(int max) {
		this.lhs = new Instantiation();
		this.setRhs(max);
	}

	public HashList<VarIdentifier> getPredecessors() {
		return this.lhs.getVarIds();
	}

	public String toString() {
		return this.lhs + "=>" + this.rhs;
	}

	public boolean equals(Object obj) {
		Explanation exp = (Explanation) obj;
		return (exp.getRhs() == this.rhs && (exp.getLhs().equals(this.lhs)));
	}

	public void refresh(AgentOrdering agentOrder) {
		this.lhs = this.lhs.clone(agentOrder);
	}

	public Explanation clone() {
		return new Explanation(this.lhs.cloneUseingSelfComparator(), this.rhs);
	}
}
