package simple.dcsp.agileabt.explanation;

import java.util.ArrayList;
import java.util.Collection;

import kernel.AgentObserver;
import kernel.StopException;
import kernel.com.Message;
import kernel.events.AgentEvent;
import kernel.events.AgentFinishedEvent;
import simulation.metric.Metric;
import simulation.metric.MetricListener;
import util.Commands;
import util.Logger;

/**
 * A class for measuring the number of non concurrent explanations checks
 * 
 * @author Mohamed Wahbi
 * 
 */
public class ExplanationObserver extends AgentObserver implements MetricListener {

	/**
	 * 
	 */
	long nbNCExpChecks = 0;

	/**
	 * Constructs an observer for Explanations Checks
	 */
	public ExplanationObserver(String name) {
		super(name);
	}

	/**
	 * Updates the current Non-Concurrent constraint checks to be the maximum between its value and the value attached in the message <code>msg</code>
	 * 
	 * @throws StopException
	 */
	@Override
	public boolean handleReceivedMessage(Message msg) throws StopException {
		if (msg.contains(AgileABTAgent.NCEXP_CHECKS)) {
			long senderExpChecks = (Long) msg.get(AgileABTAgent.NCEXP_CHECKS);
			this.nbNCExpChecks = Math.max(this.nbNCExpChecks, senderExpChecks);
			return true;
		}
		return false;
	}

	/**
	 * Attach the Current number of NCCCs of the observed agent to the message <code>msg</code>
	 */
	@Override
	public void handleSendMessage(Message msg) {
		if (this.myAgent.isDisSearchStarted()) {
			msg.addContents(AgileABTAgent.NCEXP_CHECKS, this.nbNCExpChecks);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void handleIncrementNbConstraintChecks(int n) {
		this.nbNCExpChecks += n;
	}

	@Override
	public MetricListener clone() {
		MetricListener clone = null;
		try {
			clone = (ExplanationObserver) super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return clone;
	}

	@Override
	public Collection<Metric> getEndStatistics() {
		ArrayList<Metric> m = new ArrayList<Metric>();
		m.add(new Metric(AgileABTAgent.NCEXP_CHECKS, this.nbNCExpChecks, Commands.MAX));
		return m;
	}

	/**
	 * This function Handles the event <code> {@link kernel.events.AgentFinishedEvent} </code> It print in the logger the end value of
	 * AgentC_Cost (NCCC)
	 */
	@Override
	public void handleAgentEvent(AgentEvent event) {
		if (event instanceof AgentFinishedEvent) {
			this.myAgent.log("AgentC_Cost (" + AgileABTAgent.NCEXP_CHECKS + ") : " + this.nbNCExpChecks, Logger.INFO_LEVEL);
		}
	}

	@Override
	public void handleIncrementCycle() {}

}
