/**
 * 
 */
package simple.dcsp.agileabt.explanation;

import static simple.dcsp.agileabt.explanation.AgileABTMaster.DomDeg;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.MinDom;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Topological_Deg;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Topological_ID;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Topological_MaxSucc;

import java.util.BitSet;
import java.util.HashMap;
import java.util.Map;

import kernel.VarIdentifier;
import kernel.order.AgentOrdering;
import util.HashList;
import util.Instantiation;
import util.VarAssignment;

public class MyExplanations {

	private Instantiation agentView = null;
	private BitSet[] lhs = null;
	private int[] rhs = null;
	private Map<VarIdentifier, Integer> maxDomSize = null;
	private Map<VarIdentifier, Integer> degrees = null;
	private HashList<VarIdentifier> agentIds = null;
	private HashList<VarIdentifier> newOrder = null;
	private double[] newTV = null;
	private Instantiation newNogood = null;
	private short heuristic = MinDom;
	
	public MyExplanations(HashList<VarIdentifier> agentIds, HashMap<VarIdentifier, Integer> maxDomSize, HashMap<VarIdentifier, Integer> degrees, short heuristic) {
		this.heuristic = heuristic;
		this.agentIds = new HashList<VarIdentifier>(agentIds);
		this.maxDomSize = new HashMap<VarIdentifier, Integer>(maxDomSize);
		this.degrees = new HashMap<VarIdentifier, Integer>(maxDomSize);
		this.lhs = new BitSet[agentIds.size()+1];
		for(VarIdentifier agId : agentIds){
			this.removeExplanationOf(agId);
		}
	}
	/**
	 * 
	 */
	public MyExplanations clone() {
		return new MyExplanations(this.agentIds, this.maxDomSize, this.degrees, this.heuristic, 
				this.agentView, this.lhs.clone(), this.rhs.clone());
	}

	/**
	 * 
	 * @param agentIds
	 * @param maxDomSize
	 * @param degrees
	 * @param heuristic
	 * @param agentView
	 * @param lhs
	 * @param rhs
	 */
	public MyExplanations(HashList<VarIdentifier> agentIds, Map<VarIdentifier, Integer> maxDomSize, Map<VarIdentifier, Integer> degrees, short heuristic, Instantiation agentView, BitSet[] lhs, int[] rhs) {
		this.heuristic = heuristic;
		this.agentIds = agentIds;
		this.maxDomSize = maxDomSize;
		this.degrees = degrees;
		this.agentView = agentView;
		this.lhs = lhs;
		this.rhs = rhs;
	}
	

	public void updateMaxDomainSize(VarIdentifier agId, int d) {
		int value = Math.min(this.getMaxDomainSize(agId), d);
		this.maxDomSize.put(agId, value);
	}

	public int getMaxDomainSize(VarIdentifier agId) {
		return this.maxDomSize.get(agId);
	}

	public void removeExplanationOf(VarIdentifier agId) {
		int index = this.agentIds.indexOf(agId);
		this.lhs[index] = new BitSet(agentIds.size()+1);
		this.rhs[index] = this.maxDomSize.get(agId);
	}

	public Instantiation getLhs(VarIdentifier agId) {
		Instantiation lhs = new Instantiation();
		int index = agentIds.indexOf(agId);
		if(!this.lhs[index].isEmpty()){
			for(int i=1 ; i>=0 ; i=this.lhs[index].nextSetBit(i)){
				lhs.add(this.agentView.getVarAssignment(new VarIdentifier(i)));
			}
		}
		return lhs;
	}
	
	public HashList<VarIdentifier> getPredecessors(VarIdentifier agId) {
		HashList<VarIdentifier> predecessor = new HashList<VarIdentifier>();
//		if(!this.lhs[agId].isEmpty()){
//			for(int i=1 ; i>=0 ; i=this.lhs[agId].nextSetBit(i)){
//				predecessor.add(i);
//			}
//		}
		int index = this.agentIds.indexOf(agId);
		for(int i=1 ; i>=0 ; i=this.lhs[index].nextSetBit(i)){
			predecessor.add(this.agentIds.get(i));
		}
		return predecessor;
	}

	public int getRhs(VarIdentifier agId) {
		int index = agentIds.indexOf(agId);
		return this.rhs[index];
	}

	public void setRhs(VarIdentifier varId, int rhs) {
		int index = agentIds.indexOf(varId);
		this.rhs[index] = rhs;
	}
	
	public void setExplanation(VarIdentifier varId, Instantiation lhs, int rhs) {
		this.setRhs(varId, rhs);
		this.setLhs(varId, lhs);
	}

	/**
	 * @param agId
	 * @param lhs
	 */
	private void setLhs(VarIdentifier varId, Instantiation lhs) {
		if(this.agentView.compatible(lhs)){
			int index = agentIds.indexOf(varId);
			this.lhs[index].clear();
			for(VarIdentifier pre : lhs.getVarIds()){
				int p = agentIds.indexOf(pre);
				this.lhs[index].set(p);
			}
		} else {
			throw new Error("\n agentView: "+agentView+"\n is not compatibe with lhs of ("+varId+") :"+lhs);
		}
	}

	public HashList<VarIdentifier> getNewOrder() {
		return this.newOrder;
	}

	public void setNewOrder(HashList<VarIdentifier> newOrder) {
		this.newOrder = newOrder;
	}

	public double[] getNewTV() {
		return this.newTV;
	}

	public void setNewTV(double[] newTV) {
		this.newTV = newTV;
	}

	public Instantiation getNewNG() {
		return this.newNogood;
	}

	public void setNewNG(Instantiation newNG) {
		this.newNogood = newNG;
	}

	public void clear() {
		this.newOrder = null;
		this.newTV = null;
		this.newNogood = null;
	}

	/**
	 * remove all explanations incompatible with order
	 */
	public int removeIncompatibleLhsWithOrder(HashList<VarIdentifier> order) {
		int nbChecks = 0;
		for(VarIdentifier agId : agentIds){
			HashList<VarIdentifier> higherAgents = order.getElementsBefor(agId);// this.getHigherAgents(order, agId);
			HashList<VarIdentifier> predecessors = this.getPredecessors(agId);
			if (!higherAgents.contains(predecessors)) {
				this.removeExplanationOf(agId);
			}
			nbChecks++;
		}
		return nbChecks;
	}
	
	/**
	 * remove all explanations incompatible with order
	 */
	public void removeExplanationOfAgents(HashList<VarIdentifier> agents) {
		for (VarIdentifier agId : agents) {
			this.removeExplanationOf(agId);
		}
	}

	/**
	 * 
	 */
	public int refresh(AgentOrdering agentOrder, VarIdentifier myId) {
		return this.removeIncompatibleLhsWithOrder(agentOrder.getOrder());
	}

	public int remove(VarAssignment varAss) {
		int nbCheck = 0;
		for(VarIdentifier agId : agentIds){
			int index = agentIds.indexOf(agId);
			VarIdentifier var = varAss.getVarIdentifier();
			int i = agentIds.indexOf(var);
			if (this.lhs[index].get(i)) {
				this.removeExplanationOf(agId);
				nbCheck++;
			}
		}
		this.agentView.remove(varAss);
		return nbCheck;
	}

	public HashList<VarIdentifier> getTopologicalOrder(short topologicalType) {
		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
		HashMap<VarIdentifier, HashList<VarIdentifier>> successors = this.getGraphSuccessor();
		HashMap<VarIdentifier, HashList<VarIdentifier>> graph = this.getGraphPredecessor();
		while (graph.keySet().size() > 0) {
			HashList<VarIdentifier> roots = this.getRoots(graph);
			VarIdentifier selected = null;
			double min = Integer.MAX_VALUE;
			double rhs = 0;
			switch (topologicalType) {
				case Topological_ID : {
					for (VarIdentifier agId : roots) {
						int index = agentIds.indexOf(agId);
						if (this.heuristic == MinDom) {
							rhs = this.getRhs(agId);
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.rhs[index]) / this.degrees.get(agId);
						}
						if ((min > rhs) || ((min == rhs) && (agId.compareTo(selected) < 0))) {
							min = this.getRhs(agId);
							selected = agId;
						}
					}
					break;
				}
				case Topological_Deg : {
					int deg = Integer.MIN_VALUE;
					for (VarIdentifier agId : roots) {
						int index = agentIds.indexOf(agId);
						if (this.heuristic == MinDom) {
							rhs = this.rhs[index];
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.rhs[index]) / this.degrees.get(agId);
						}
						if (min > rhs || (min == rhs && (this.degrees.get(agId) > deg))) {
							min = this.rhs[index];
							deg = this.degrees.get(agId);
							selected = agId;
						}
					}
					break;
				}
				case Topological_MaxSucc : {
					for (VarIdentifier agId : roots) {
						int index = agentIds.indexOf(agId);
						if (this.heuristic == MinDom) {
							rhs = this.getRhs(agId);
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.rhs[index]) / this.degrees.get(agId);
						}
						if (min > rhs || (min == rhs && successors.get(agId).size() > successors.get(selected).size())) {
							min = this.getRhs(agId);
							selected = agId;
						}
					}
					break;
				}
			}
			order.add(selected);
			this.remove(graph, selected);
		}
		if (order.size() != this.agentIds.size()) { throw new Error("\nacyclic graph : \n" + this); }
		return order;
	}

	/**
	 * 
	 * @return
	 */
	public HashList<VarIdentifier> getBestOrder() {
		HashList<VarIdentifier> order = new HashList<VarIdentifier>();
		HashMap<VarIdentifier, HashList<VarIdentifier>> graph = this.getGraphPredecessor();
		while (order.size() < this.agentIds.size()) {
			HashList<VarIdentifier> allRoots = this.getRoots(graph);
			HashList<VarIdentifier> S1 = this.getMinRoots(allRoots);
			while (S1.size() != 0) {
				if (S1.size() > 1) {
					int d = this.getRhs(S1.get(0));
					HashList<VarIdentifier> S2 = this.getAgentsWithMinRhs(graph, d);
					if (S2.size() == 0) {
						VarIdentifier selected = S1.get(0);
						order.add(selected);
						this.remove(graph, selected);
						break;
					}
					HashList<VarIdentifier> S3 = this.getRootsWithMinPredecessor(graph, S2);
					HashList<VarIdentifier> S4 = this.getMinRoots(S3);
					if (S4.size() == 1) {
						VarIdentifier selected = S4.get(0);
						HashList<VarIdentifier> predecessors = graph.get(selected);
						for (VarIdentifier pred : graph.get(selected)) {
							predecessors.addAll(graph.get(pred));
						}
						while (predecessors.size() > 0) {
							VarIdentifier pred = predecessors.get(0);
							predecessors.remove(pred);
							if (order.contains(graph.get(pred))) {
								order.add(pred);
								this.remove(graph, pred);
							} else {
								predecessors.add(pred);
							}
						}
						order.add(selected);
						this.remove(graph, selected);
						break;
					} else {
						S1 = S4;
					}
					// S1 = S4;
				} else {
					VarIdentifier selected = S1.get(0);
					order.add(selected);
					this.remove(graph, selected);
					break;
				}
			}
		}
		return order;
	}

	/**
	 * 
	 * @param graph
	 * @param agents
	 * @return
	 */
	private HashList<VarIdentifier> getRootsWithMinPredecessor(HashMap<VarIdentifier, HashList<VarIdentifier>> graph, HashList<VarIdentifier> agents) {
		int min = Integer.MAX_VALUE;
		for (VarIdentifier agId : agents) {
			if (graph.get(agId).size() < min) {
				min = graph.get(agId).size();
			}
		}
		HashList<VarIdentifier> ans = new HashList<VarIdentifier>();
		for (VarIdentifier agId : agents) {
			if (graph.get(agId).size() == min) {
				ans.add(agId);
			}
		}
		return ans;
	}

	/**
	 * 
	 * @param graph
	 * @param d
	 * @return
	 */
	private HashList<VarIdentifier> getAgentsWithMinRhs(HashMap<VarIdentifier, HashList<VarIdentifier>> graph, int d) {
		HashList<VarIdentifier> agents = new HashList<VarIdentifier>();
		for (VarIdentifier agId : graph.keySet()) {
			if (this.getRhs(agId) < d) {
				agents.add(agId);
				for (VarIdentifier pre : graph.get(agId)) {
					if (this.getRhs(pre) != d) {
						agents.remove(agId);
						break;
					}
				}
			}
		}
		return agents;
	}

	/**
	 * 
	 * @param roots
	 * @return
	 */
	private HashList<VarIdentifier> getMinRoots(HashList<VarIdentifier> allRoots) {
		int min = Integer.MAX_VALUE;
		for (VarIdentifier agId : allRoots) {
			if (this.getRhs(agId) < min) {
				min = this.getRhs(agId);
			}
		}
		HashList<VarIdentifier> roots = new HashList<VarIdentifier>();
		for (VarIdentifier agId : allRoots) {
			if (this.getRhs(agId) == min) {
				roots.add(agId);
			}
		}
		return roots;
	}

	/**
	 * 
	 * @param graph
	 * @param selected
	 */
	private void remove(HashMap<VarIdentifier, HashList<VarIdentifier>> graph, VarIdentifier selected) {
		for (VarIdentifier agId : graph.keySet()) {
			if (graph.get(agId).contains(selected)) {
				graph.get(agId).remove(selected);
			}
		}
		graph.remove(selected);
	}

	/**
	 * 
	 * @param graph
	 *            is the graph of predecessors
	 * @return
	 */
	private HashList<VarIdentifier> getRoots(HashMap<VarIdentifier, HashList<VarIdentifier>> graph) {
		HashList<VarIdentifier> roots = new HashList<VarIdentifier>();
		for (VarIdentifier agId : graph.keySet()) {
			if ((graph.get(agId).size() == 0)) {
				roots.add(agId);
			}
		}
		return roots;
	}

	/**
	 * 
	 * @return
	 */
	private HashMap<VarIdentifier, HashList<VarIdentifier>> getGraphSuccessor() {
		HashMap<VarIdentifier, HashList<VarIdentifier>> graph = new HashMap<VarIdentifier, HashList<VarIdentifier>>();
		for (VarIdentifier agId : this.agentIds) {
			graph.put(agId, new HashList<VarIdentifier>());
		}
		for (VarIdentifier agId : this.agentIds) {
			HashList<VarIdentifier> predecessors = this.getPredecessors(agId);
			for (VarIdentifier pre : predecessors) {
				graph.get(pre).add(agId);
			}
		}
		return graph;
	}

	/**
	 * 
	 * @return
	 */
	private HashMap<VarIdentifier, HashList<VarIdentifier>> getGraphPredecessor() {
		HashMap<VarIdentifier, HashList<VarIdentifier>> graph = new HashMap<VarIdentifier, HashList<VarIdentifier>>();
		for (VarIdentifier agId : this.agentIds) {
			HashList<VarIdentifier> predecessors = this.getPredecessors(agId);
			graph.put(agId, predecessors);
		}
		return graph;
	}

	public boolean isAcyclic() {
		for (VarIdentifier agId : this.agentIds) {
			HashList<VarIdentifier> predecessors = this.getPredecessors(agId);
			for (VarIdentifier pred : predecessors) {
				if (this.getPredecessors(pred).contains(agId)) {
					System.err.println("*******************************************\n" + agId +
							" :: predecessors" + predecessors + " :: pred: "
							+ pred + "\n" + this);
					return true;
				}
			}
		}
		return false;
	}

	public String toString(VarIdentifier agId) {
		int index = agentIds.indexOf(agId);
		return agId + " :: " + lhs[index]+" --> "+this.rhs[index];
	}

	public String toString() {
		String ans = "";
		for (VarIdentifier agId : this.agentIds) {
			ans += this.toString(agId) + "\n";
		}
		return ans;
	}

	public void removeExplanationsIncompatibleWith(Instantiation inst) {
		for(VarIdentifier agId : this.agentIds ){
			if (!inst.compatible(this.getLhs(agId))) {
				this.removeExplanationOf(agId);
			}
		}
	}

	public double[] computeTV(HashList<VarIdentifier> order) {
		double[] tv = new double[order.size()];
		for (int i = 0 ; i < order.size() ; i++) {
			VarIdentifier agId = order.get(i);
			if (this.heuristic == MinDom) {
				tv[i] = this.getRhs(agId);
			} else {
				int index = this.agentIds.indexOf(agId);
				tv[i] = ((double) this.rhs[index]) / this.degrees.get(agId);
			}
		}
		return tv;
	}

}
