package simple.dcsp.dba;

import java.util.Iterator;
import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.com.TerminateMsgHandler;
import kernel.events.AgentEvent;
import kernel.vars.integer.IntExternalVar;
import util.Commands;
import util.HashList;
import util.VarAssignment;

public class DBATerminateMsgHandler extends TerminateMsgHandler {

	boolean found = true;

	public DBATerminateMsgHandler(AbstractAgent agent) {
		super(agent);
	}

	@Override
	public boolean processTerminationDetection(Message msg) throws StopException {
		// this.myAgent.log(msg.toString(), 1);
		found = Boolean.parseBoolean((String) msg.get("SOLUTION-FOUND"));
		VarAssignment sol = (VarAssignment) msg.get("SOLUTION");
		HashList<VarIdentifier> ids = HashList.fromString(""+ msg.get("AGENTS-INFORMED"), VarIdentifier.class);
		if (found) {
			if (((DBAagent) this.myAgent).var.getValue() != sol.getValue()) {
				throw new Error(this.myAgent.getAgentName() + " Solution is different from my value " + ((DBAagent) this.myAgent).var + " sol: "
						+ sol);
			} else {
				if (this.myAgent.agentOrder.getNumAgents() != ids.size())
					this.forwardSolution(ids);
				throw new StopException(Commands.SUCCESS);
			}
		} else {
			if (this.myAgent.agentOrder.getNumAgents() != ids.size())
				this.forwardSolution(ids);
			throw new StopException(Commands.FAILURE);
		}

	}

	private void forwardSolution(HashList<VarIdentifier> rcvdIds) {
		Iterator<IntExternalVar> it = this.myAgent.agentView.iterator();
		HashList<VarIdentifier> ids = this.myAgent.communicator.getNeighbors();
		ids = HashList.union(rcvdIds, ids);
		while (it.hasNext()) {
			VarAssignment ass = it.next().getVarAssignment();
			if (!rcvdIds.contains(ass.getVarIdentifier())) {
				this.sendSolution(ass, ids);
			}
		}
	}

	public void sendSolution() {
		Iterator<IntExternalVar> it = this.myAgent.agentView.iterator();
		HashList<VarIdentifier> ids = this.myAgent.communicator.getNeighbors();
		ids.add(this.myAgent.getVarIdentifier());
		while (it.hasNext()) {
			VarAssignment v = it.next().getVarAssignment();
			this.sendSolution(v, ids);
		}
		this.notifyTerminateForMaster();
	}

	private void notifyTerminateForMaster() {
		Message msg = this.createMessage();
		msg.addContents(Commands.END_STATE, found? Commands.SUCCESS : Commands.FAILURE);
		msg.setReceiverId(AbstractMaster.MasterVarId);
		this.myAgent.sendMessage(msg);
	}

	public void sendNoSolution() {
		this.found = false;
		this.sendSolution();
	}

	private void sendSolution(VarAssignment sol, HashList<VarIdentifier> ids) {
		Message msg = this.createMessage();
		msg.addContents("SOLUTION-FOUND", found + "");
		msg.addContents("SOLUTION", sol);
		msg.addContents("AGENTS-INFORMED", ids);
		msg.setReceiverId(sol.getVarIdentifier());
		this.myAgent.sendMessage(msg);
	}

	@Override
	public void handleAgentEvent(AgentEvent agentEvent) {}
}
