/************************************************************************

 **************************************************************************/

package simulation;

import java.util.HashMap;

import kernel.order.CentralizedAO;

/**
 * An interface to implement a distributed constraint programming solver for simulation.
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */

public interface DisCPSolver {
	/**
	 * Called to start resolution
	 */
	public void solve();

	/**
	 * Called to stop running of the solver
	 */
	public void stoptRunning();

	/**
	 * Returns the network latency, used for simulate the massage delay
	 */
	public int getNetworkLatency();

	/**
	 * Set the value of network latency (delay cost coefficient) to the given value
	 * 
	 * @param msgDelayCostCoefficient
	 */
	public void setNetworkLatency(int msgDelayCostCoefficient);

	/**
	 * Set the max number of non concurrent constraint checks (NCCCs) to the given value
	 * 
	 * @param cutOffNCCCs
	 *            the cut-off of NCCCs
	 */
	public void setCutOffNCCCs(long maxNCCCs);

	// /**
	// * Set the max CBR time allowed to solve the problem to the given value.
	// *
	// * @param maxCBR
	// * the Max Cycle Based Run Time
	// */
	// public void setCutOffCBR(long maxCBR);

	/**
	 * to perform a simulation of fast communication environment (Multiprocessors)
	 */
	public void setSlowCommunication();

	/**
	 * to perform a simulation of slow communication (Internet, LAN, ..)
	 */
	public void setFastCommunication();

	/**
	 * Called to add for each agent a NCCC Observer
	 * 
	 * @see {@link metric.NCCCObserver}
	 */
	public void addNCCCMetric();
	
	/**
	 * Called to add for each agent a NCCPUTime Observer
	 * 
	 * @see {@link metric.NCCTObserver}
	 */
	public void NCCPUTimeObserver();

	// /**
	// * Called to add for each agent a CBR Observer
	// *
	// * @see {@link simulation.metric.CBRObserver}
	// */
	// public void addCBRMetric();

	public void addCommunicationMetric();

	// public void addNCCTMetric();

	public void setMessageDelay(int minCCs, int maxCCs);

	/**
	 * Set the centralized agent ordering plugin
	 * 
	 * @param centralizedAO
	 */
	public void setCentralizedAO(CentralizedAO centralizedAO);

	public HashMap<String, Double> getStatistics();

	public String getEndState();

	String getGlobalSolution();

}
