/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/
package simulation;

import java.io.File;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import simulation.parser.InstanceTokens;
import util.Commands;
import benchmark.DataSetTeamcore;
import benchmark.Generator;
import benchmark.graphcoloring.URGCGenerator;
import benchmark.graphcoloring.URSoftGraphColoringGenerator;
import benchmark.meetingscheduling.DCOPMeetingScheduling;
import benchmark.meetingscheduling.DCSPMeetingScheduling;
import benchmark.nqueen.NQueenGenerator;
import benchmark.random.ComplexDisCSPGenerator;
import benchmark.random.DCOPGenerator;
import benchmark.random.DisCSPGenerator;
import benchmark.random.MaxDisCSPGenerator;
import benchmark.sensornetwork.SensorDCSPGenerator;

/**
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">Mohamed Wahbi</a>
 */
public class Experimentation {
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// The GNU GPL copyright notice
		System.err.println(Commands.copyrightNotice);
//		util.Logger.LOG_DEBUG = true;
//		util.Logger.log_debug_level = util.Logger.INFO_LEVEL;
////		String path = ""; 
////		path = "./samples/SoftGC/plus2.xml";
////		String[] arg = {path}; 
////		args = arg;
		new Experimentation(args);
	}

	public Experimentation(String[] args) {
		File file = new File(args[0]);
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(file);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
		Element root = document.getRootElement();
		String benchmark = root.getAttributeValue(InstanceTokens.BENCHMARK);
		Generator experimentation = null;
		if (benchmark.equals(Commands.RandomDisCSP)) {
			experimentation = new DisCSPGenerator(file);
		} else if (benchmark.equals(Commands.RandomDCOP)) {
			experimentation = new DCOPGenerator(file);
		} else if (benchmark.equals(Commands.RandomMaxDisCSP)) {
			experimentation = new MaxDisCSPGenerator(file);
		} else if (benchmark.equals(Commands.GraphColoring)) {
			experimentation = new URGCGenerator(file);
		} else if (benchmark.equals(Commands.NQueen)) {
			experimentation = new NQueenGenerator(file);
		} else if (benchmark.equals(Commands.ComplexRandomTKC)) {
			experimentation = new ComplexDisCSPGenerator(file);
		} else if (benchmark.equals(Commands.GSensorDCSP) || benchmark.equals(Commands.SensorDCSP)) {
			experimentation = new SensorDCSPGenerator(file);
		} else if (benchmark.equals(Commands.RMSP)) {
			experimentation = new DCSPMeetingScheduling(file);
		} else if (benchmark.equals(Commands.DIMES)) {
			experimentation = new DCOPMeetingScheduling(file);
		} else if (benchmark.equals(Commands.SoftGraphColoring)) {
			experimentation = new URSoftGraphColoringGenerator(file);
		} else if (benchmark.equals(DataSetTeamcore.TEAMCORE)) {
			experimentation = new DataSetTeamcore(file);
		} else {
			throw new IllegalArgumentException("Unkown benchmark: "+benchmark);
		}
		experimentation.run();
	}
}
