/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/
package simulation;

import gui.tree.presentation.TreePanel;
import gui.tree.presentation.TreePanel.Orientation;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Scanner;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import dischoco.Protocols;
import kernel.VarIdentifier;
import kernel.order.AgentOrdering;
import simulation.parser.InstanceParser;
import simulation.parser.PConstraint;
import simulation.parser.PExtensionConstraint;
import simulation.parser.PVariable;
import util.Commands;
import util.Instantiation;
import util.Tools;
import util.VarAssignment;
import util.graph.Edge;
import util.graph.UndirectedGraph;
import util.tree.AbstractTree;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 */
public class Main {
	// https://svn.code.sf.net/p/dischoco/code/ 
	// https://dischoco.svn.sourceforge.net/svnroot/dischoco/ 
	// nohup java -ea -Xmx1024m -cp dischoco.jar simulation.Experimentation RMSP/afc.xml > msafc.txt & 
	// -ea -Xms128m -Xmx512m -XX:MaxPermSize=512m
	// -Xns10m 
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// The GNU GPL copyright notice
		System.err.println(Commands.copyrightNotice);
		int nbExecution = 1;
		long maxNCCCs = (long) 1.0E8; 
		boolean solvable = true;
		String protocol = "";
		String problem = "";
		String ordering = "";

		protocol = Protocols.ABT;
		ordering = Commands.MinDomOverDegAO;
		problem = "./problem.xml";
//		util.Logger.LOG_DEBUG = true;
		
		run(problem, protocol, ordering, nbExecution, solvable, maxNCCCs);
	}
	
	public static void drawtree(final String priorityTree) {
		for (UIManager.LookAndFeelInfo laf : UIManager.getInstalledLookAndFeels()) {
			if ("Mac OS X".equals(laf.getName())) {
				try {
					UIManager.setLookAndFeel(laf.getClassName());
				} catch (Exception e) {}
			} else {
				if ("Nimbus".equals(laf.getName())) {
					try {
						UIManager.setLookAndFeel(laf.getClassName());
					} catch (Exception e) {}
				}
			}
		}
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JFrame frame = new JFrame();
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				frame.setTitle(Commands.DisChoco_TITLE);
				Dimension dimension = new Dimension(400, 400); // new Dimension(1024, 720)
				frame.setMinimumSize(dimension);
				frame.setMaximumSize(dimension);
				frame.setPreferredSize(dimension);
				Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
				// Determine the new location of the window
				int w = frame.getSize().width;
				int h = frame.getSize().height;
				int x = (dim.width - w) / 2;
				int y = (dim.height - h) / 2;
				// Move the window
				frame.setLocation(x, y);
				AbstractTree<VarIdentifier> pt = AbstractTree.fromString(priorityTree, VarIdentifier.class);
				TreePanel<VarIdentifier> treePanel = new TreePanel<VarIdentifier>(pt, dimension, Orientation.VERTICAL);
				frame.getContentPane().add(treePanel, BorderLayout.CENTER);
				frame.pack();
				frame.setVisible(true);
			}
		});
	}
	
	/**
	 * 
	 * @param problem 
	 * @param protocol 
	 * @param ordering 
	 * @param nbExecution 
	 * @param solvable 
	 * @param maxNCCCs 
	 */
	protected static void run(String problem, String protocol, String ordering, int nbExecution, boolean solvable, long maxNCCCs) {
		for (int i = 1 ; i <= nbExecution ; i++) {
			DisProblem disProb = new DisProblem(problem, protocol);
			DisSolverSimulator solver = new DisSolverSimulator(disProb);
			solver.addCommunicationMetric();
			solver.setCutOffNCCCs(maxNCCCs);
//			solver.addNCCTMetric();
//			solver.setSlowCommunication();
			solver.setCentralizedAO(AgentOrdering.getCentralizedAO(ordering));
			solver.solve();
			if (solver.getEndState().equals(Commands.SUCCESS)) {
				String s = "------------------------------------------------";
				System.err.println(i + " " + protocol + " with "+ordering+ "\n" + s + s + s);
				//System.err.println(solver.getGlobalSolution());
				System.err.println(solver.getSolution());
				HashMap<String, Double> runStatistics = solver.getStatistics();
				ArrayList<String> keys = disProb.master.getStatisticsKeys();
				System.err.println(Tools.toString(keys) + "\n" + Tools.statisticsToString(keys, runStatistics));
				System.err.println(s + s + s  + "\n");
				if (!solvable) {
					break;
				}
			} else {
				if (solver.getEndState().equals(Commands.FAILURE)) {
					String s = "*************************************************";
					System.err.println(i + " " + protocol + " with "+ordering+ "\n" + s + s + s);
					//System.err.println(solver.getGlobalSolution());
					System.err.println(solver.getSolution());
					HashMap<String, Double> runStatistics = solver.getStatistics();
					ArrayList<String> keys = disProb.master.getStatisticsKeys();
					System.err.println(Tools.toString(keys) + "\n" + Tools.statisticsToString(keys, runStatistics));
					System.err.println(s + s + s + "\n");
					if (solvable) {
						break;
					}
				} else {
					String s = "================================================";
					System.err.println(i + " " + protocol + " with "+ordering+ "\n" + s + s + s);
					//System.err.println(solver.getGlobalSolution());
					System.err.println(solver.getSolution());
					HashMap<String, Double> runStatistics = solver.getStatistics();
					ArrayList<String> keys = disProb.master.getStatisticsKeys();
					System.err.println(Tools.toString(keys) + "\n" + Tools.statisticsToString(keys, runStatistics));
					System.err.println(s + s + s + "\n");
					break;
				}
			}
		}
	}

	/**
	 * 
	 * @param problemFile
	 * @param solution
	 * @return
	 */
	public static boolean isSolution(String problemFile, Instantiation solution) {
		InstanceParser parser = new InstanceParser(problemFile);
		parser.loadInstance(problemFile);
		parser.parse(false);
		Map<String, PConstraint> constraints = parser.getMapOfConstraints();
		Iterator<String> it = constraints.keySet().iterator();
		while (it.hasNext()) {
			PConstraint constraint = constraints.get(it.next());
			PVariable[] scope = constraint.getScope();
			int agId1 = scope[0].getAgent().getId();
			int agId2 = scope[1].getAgent().getId();
			int val1 = solution.getVarAssignment(new VarIdentifier(agId1)).getValue();
			int val2 = solution.getVarAssignment(new VarIdentifier(agId2)).getValue();
			int[][] tuples = ((PExtensionConstraint) constraint).getRelation().getTuples();
			for (int i = 0 ; i < tuples.length ; i++) {
				int v1 = tuples[i][0];
				int v2 = tuples[i][1];
				if (val1 == v1 && val2 == v2) {
					String s = "================================================";
					System.err.println("\n" + s + s + s + s);
					System.err.println("The constraint " + constraint.getName() + " is not satisfied " + " \t==>  " + agId1 + "=" + val1 + " <> "
							+ agId2 + "=" + val2);
					System.err.println(solution);
					System.err.println(s + s + s + s + "\n");
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * @param problem
	 * @param path
	 * @param name
	 */
	public static void toCSPXML(String problem, String path, String name){
		InstanceParser parser = new InstanceParser(problem);
		parser.loadInstance(problem);
		parser.parse(false);
		parser.convertToCSPModel();
		parser.saveXML(path, name);
	}

	/**
	 * @param problem
	 */
	protected static void test(String problem) {
		JFrame frame = new JFrame();
		JScrollPane center = null;	
		JLabel content = null;
		frame.setTitle("DisChoco: Visualizing the meeting scheduling problem");
		InstanceParser parser = new InstanceParser(problem);
		parser.loadInstance(problem);
		parser.parse(false);
		String fileName = problem.substring(0, problem.length()-4)+".dot";
		String dotCode = parser.toDOT();
		File dotFile = new File(fileName);
		try {
			PrintWriter writer = new PrintWriter(dotFile);
			writer.println(dotCode);
			writer.close();
//			dotFile.deleteOnExit();	//Assures the temporary file is deleted when JVM exits
		} 
		catch (FileNotFoundException e) {
			System.err.println("Could not create DOT output file with name " + fileName);
			e.printStackTrace();
		}
		ImageIcon currentDotImage = createDOTfigure(fileName);

		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		//Image could not be created; an error message is shown in the Frame
		if(currentDotImage == null) {
			content = new JLabel("Could not create graphic from .dot file");
			frame.setSize(dim.width/2, dim.height/2);
		}

		//Image is displayed as a content of the label; Frame size is set according to the size of the image
		else {
			content = new JLabel(currentDotImage);
			int width = Math.max(300, Math.min(dim.width - 100, currentDotImage.getIconWidth() + 50));
			int height = Math.max(100, Math.min(dim.height - 150, currentDotImage.getIconHeight() + 100));
			frame.setSize(width, height);
		}
		// Determine the new location of the window
		int w = frame.getSize().width;
		int h = frame.getSize().height;
		int x = (dim.width - w) / 2;
		int y = (dim.height - h) / 2;
		// Move the window
		center = new JScrollPane(content);
		frame.add(center, BorderLayout.CENTER);
		frame.setLocation(x, y);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	
	/**
	 * 
	 * @param dotFileName
	 * @return ImageIcon 
	 */
	private static ImageIcon createDOTfigure(String dotFileName) {
		ImageIcon dotImageIcon = null;
		int status = -1;
		File input = new File(dotFileName);
		try {
			Process p = Runtime.getRuntime().exec(new String[] {Commands.DOT, "-Tgif", input.getAbsolutePath(), "-O"});
			final Scanner err = new Scanner(p.getErrorStream());
			new Thread(new Runnable() {
				public void run() {
					while(err.hasNextLine()) {
						System.err.println(err.nextLine());
					}
				}
			}).start();
			status = p.waitFor();	//0 if creation of image was successful
		} 
		catch (IOException e) {
			if(e.getMessage().contains("Cannot run program")) {
				System.err.println("Can not run \"" + Commands.DOT + "\". Probably, the program is not installed, or it is not on the search path.");
			}
			else {
				e.printStackTrace();
			}
		} catch (InterruptedException e) {}

		if(status == 0) {
			dotImageIcon = new ImageIcon(dotFileName + ".gif");
			//Image needs to be stored, as scaling should be done from this image in order not to loose too much quality
//			Image dotImage = dotImageIcon.getImage();
			new File(dotFileName + ".gif").deleteOnExit();	//Assures the temporary .gif file will be deleted when JVM exits
		}
		
		try {
			Process p = Runtime.getRuntime().exec(new String[] {Commands.DOT, "-Tpdf", input.getAbsolutePath(), "-O"});
			final Scanner err = new Scanner(p.getErrorStream());
			new Thread(new Runnable() {
				public void run() {
					while(err.hasNextLine()) {
						System.err.println(err.nextLine());
					}
				}
			}).start();
			status = p.waitFor();	//0 if creation of image was successful
		}
		catch (IOException e) {
			if(e.getMessage().contains("Cannot run program")) {
				System.err.println("Can not run \"" + Commands.DOT + "\". Probably, the program is not installed, or it is not on the search path.");
			}
			else {
				e.printStackTrace();
			}
		} catch (InterruptedException e) {}
		return dotImageIcon;
	}

	/**
	 * 
	 */
	public static void generateRandomDCOP() {
		int instances = 25, n=10, d=10, 
				minP1=20, maxP1=80, stepP1=20, 
				minCost=0, maxCost=100;
		File path = new File("./samples/RandomDCOP/10_10");
		benchmark.random.DCOPGenerator g = new benchmark.random.DCOPGenerator(path, Commands.TKC, 
				instances, n, d, minP1, maxP1, stepP1, minCost, maxCost);
		g.generate();
		System.exit(1);
	}

	/**
	 * @param problem
	 */
	protected static void drawSolution(String problem, String solutionStr, int kind) {
		JFrame frame = new JFrame();
		JScrollPane center = null;	
		JLabel content = null;
		frame.setTitle("DisChoco: Visualizing the meeting scheduling problem");
		InstanceParser parser = new InstanceParser(problem);
		parser.loadInstance(problem);
		parser.parse(false);
		String fileName = problem.substring(0, problem.length()-4)+".dot";
		String dotCode = "";
		Instantiation solution = Instantiation.fromString(solutionStr);
		UndirectedGraph<VarIdentifier> g = parser.getConstraintGraph();
		UndirectedGraph<VarAssignment> graph = new UndirectedGraph<VarAssignment>();
		for(Edge<VarIdentifier> edge : g.getEdges()) {
			VarIdentifier source = edge.getSource().getData();
			VarAssignment sourceVarAss = solution.getVarAssignment(source);
			VarIdentifier target = edge.getTarget().getData();
			VarAssignment targetVarAss = solution.getVarAssignment(target);
			String label = edge.getLabel();
			double weight = edge.getWeight();
			graph.addEdge(sourceVarAss, targetVarAss, label, weight);
		}
		if(kind == 1) {
			dotCode = graph.toDOTString();
		} else if(kind == 2) {
			AbstractTree<VarAssignment> tree = graph.DFS();
			dotCode = tree.toDOTString();
		}
		File dotFile = new File(fileName);
		try {
			PrintWriter writer = new PrintWriter(dotFile);
			writer.println(dotCode);
			writer.close();
//			dotFile.deleteOnExit();	//Assures the temporary file is deleted when JVM exits
		} 
		catch (FileNotFoundException e) {
			System.err.println("Could not create DOT output file with name " + fileName);
			e.printStackTrace();
		}
		ImageIcon currentDotImage = createDOTfigure(fileName);

		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		//Image could not be created; an error message is shown in the Frame
		if(currentDotImage == null) {
			content = new JLabel("Could not create graphic from .dot file");
			frame.setSize(dim.width/2, dim.height/2);
		}

		//Image is displayed as a content of the label; Frame size is set according to the size of the image
		else {
			content = new JLabel(currentDotImage);
			int width = Math.max(300, Math.min(dim.width - 100, currentDotImage.getIconWidth() + 50));
			int height = Math.max(100, Math.min(dim.height - 150, currentDotImage.getIconHeight() + 100));
			frame.setSize(width, height);
		}
		// Determine the new location of the window
		int w = frame.getSize().width;
		int h = frame.getSize().height;
		int x = (dim.width - w) / 2;
		int y = (dim.height - h) / 2;
		// Move the window
		center = new JScrollPane(content);
		frame.add(center, BorderLayout.CENTER);
		frame.setLocation(x, y);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

}
