package simulation;

import java.util.ArrayList;
import java.util.HashMap;
import dischoco.Protocols;

import util.Commands;
import util.Instantiation;
import util.Tools;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 * 
 */
public class Run {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// The GNU GPL copyright notice
		System.err.println(Commands.copyrightNotice);
		String problem = null;
		String protocol = null;
		args = new String[]{Protocols.AFCtree, "./problem.xml"};
		String commType = "f";
		int nbExecution = 1;
		if (args.length >= 2) {
			protocol = args[0];
			problem = args[1];
			System.out.println("protocol: "+protocol+" problem: "+problem);
			if (args.length >= 3) {
				commType = args[2];
				if (args.length >= 4) {
					util.Logger.LOG_DEBUG = new Boolean(args[3]);
					if (args.length >= 5) {
						util.Logger.log_debug_level = new Integer(args[4]);
						if (args.length >= 6) {
							nbExecution = new Integer(args[5]);
						}
					}
				}
			}
		} else {
			HowToUse();
		}
		for (int i = 1 ; i <= nbExecution ; i++) {
			DisProblem disProb = new DisProblem(problem, protocol);
			DisSolverSimulator solver = new DisSolverSimulator(disProb);
			if (commType.equals("s")) {
				solver.setSlowCommunication();
			}
			solver.addCommunicationMetric();
			solver.addNCCCMetric();
			solver.solve();
			if (solver.getEndState().equals(Commands.SUCCESS)) {
				String s = "------------------------------------------------";
				System.out.println(i + "\n" + s + s);
				Instantiation solution = solver.getSolution();
				System.out.println(solution);
				HashMap<String, Double> runStatistics = solver.getStatistics();
				ArrayList<String> keys = disProb.master.getStatisticsKeys();
				System.out.println(keys + "\n" + Tools.statisticsToString(keys, runStatistics));
				System.out.println(s + s + "\n");
			} else {
				if (solver.getEndState().equals(Commands.FAILURE)) {
					String s = "*************************************************";
					System.out.println(i + "\n" + s + s );
					System.out.println(solver.getGlobalSolution());
					HashMap<String, Double> runStatistics = solver.getStatistics();
					ArrayList<String> keys = disProb.master.getStatisticsKeys();
					System.out.println(keys + "\n" + Tools.statisticsToString(keys, runStatistics));
					System.out.println(s + s + "\n");
				} else {
					String s = "================================================";
					System.out.println(i + "\n" + s + s);
					System.out.println(solver.getGlobalSolution());
					HashMap<String, Double> runStatistics = solver.getStatistics();
					ArrayList<String> keys = disProb.master.getStatisticsKeys();
					System.out.println(keys + "\n" + Tools.statisticsToString(keys, runStatistics));
					System.out.println(s + s + "\n");
					break;
				}
			}
		}
	}

	/**
	 * 
	 */
	private static void HowToUse() {
		String cmd = "java -cp jar " ;
		System.out.println("======================== how to use DisChoco2 ==========================");
		System.out.println("1) " + cmd + "protocol problem.xml");
		System.out.println("2) " + cmd + "protocol problem.xml c");
		System.out.println("3) " + cmd + "protocol problem.xml c log");
		System.out.println("4) " + cmd + "protocol problem.xml c log l");
		System.out.println("5) " + cmd + "protocol problem.xml c log l n");
		System.out.println("6) " + cmd + "protocol problem.xml c log l n o");

		System.out.println("c\t= s simulate slow communication");
		System.out.println("c\t= f simulate fast communication");

		System.out.println("log\t= true to activate loggers");
		System.out.println("log\t= false to do not activate loggers");

		System.out.println("l\t= level of deboggin by default 1");

		System.out.println("n\t= number of execution");
		// System.out.println("o=preprocessing ordering");

		System.out.println("========================================================================");
		System.exit(-1);
	}
	
}
