package simulation.metric;

//import java.io.Serializable;

import java.util.Collection;

import kernel.AbstractAgent;
import kernel.ConstraintsHandler;

/**
 * An interface allows the class instances that implement this interface to calculate the performance of the algorithm execution
 * 
 * @author Redouane EZZAHIR
 */
public interface MetricListener extends Cloneable { // , Serializable {
	/**
	 * Called when an agent increments its number of constraint checks.
	 * 
	 * @param n
	 *            number of constraint checks
	 * @see ConstraintsHandler#incrementNumConstraintChecks()
	 */
	public void handleIncrementNbConstraintChecks(int n);

	/**
	 * All implementing classes should call this function once at the end of each cycle.
	 * 
	 * @see AbstractAgent#incrementCycle()
	 */
	public void handleIncrementCycle();

	/**
	 * 
	 * @return
	 */
	public MetricListener clone();

	public Collection<Metric> getEndStatistics();

}
