package simulation.metric;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import kernel.MasterObserver;
import kernel.com.Message;
import util.Commands;

public class StatisticsMasterObserver extends MasterObserver {

	private HashMap<Class<? extends MetricListener>, MetricListener> metrics = null;
	private HashMap<String, Double> statistics;
	private int nbMsgSendMaster = 0;
	private int nbMsgRcvdMaster = 0;
	private double nbAgentOrderingMsg = 0;

	public StatisticsMasterObserver() {
		this.statistics = new HashMap<String, Double>();
		this.statistics.put(Commands.MSGs, 0D);
	}

	@SuppressWarnings("unchecked")
	@Override
	public boolean handleReceivedMessage(Message msg) {
//		System.out.println(msg);
		String msgType = msg.getType();
		if (msgType.equals(Commands.AO)) {
			this.nbAgentOrderingMsg++;
		}
		if(!msgType.equals(Commands.STATISTICS)){
			this.nbMsgRcvdMaster++;	
		}
		if (msgType.equals(Commands.STATISTICS)) {
			ArrayList<Metric> stat = (ArrayList<Metric>) msg.get(Commands.STATISTICS);
			for (Metric m : stat) {
				if (!this.statistics.containsKey(m.name)) {
					this.statistics.put(m.name, m.value);
				} else {
					if (m.operator.equals(Metric.plus)) {
						this.statistics.put(m.name, statistics.get(m.name) + m.value);
					} else if (m.operator.equals(Metric.moins)) {
						this.statistics.put(m.name, statistics.get(m.name) - m.value);
					} else if (m.operator.equals(Metric.max)) {
						this.statistics.put(m.name, Math.max(statistics.get(m.name), m.value));
					} else if (m.operator.equals(Metric.min)) {
						this.statistics.put(m.name, Math.min(statistics.get(m.name), m.value));
					}
				}
			}
			this.master.incrNumRcvdStatistics();
			return true;
		}
		return false;
	}

	@Override
	public void handleSendMessage(Message msg) {
//		System.out.println(msg);
		this.nbMsgSendMaster++;
		if (msg.getType().equals(Commands.AO) && this.metrics != null) {
			this.nbAgentOrderingMsg++;
			Vector<MetricListener> vectMetric = new Vector<MetricListener>();
			Iterator<Class<? extends MetricListener>> it = this.metrics.keySet().iterator();
			while (it.hasNext()) {
				MetricListener m = this.metrics.get(it.next());
				vectMetric.add(m.clone());
			}
			msg.addContents(Commands.METRIC, vectMetric);
		}
	}

	public HashMap<String, Double> getStatistics() {
		this.statistics.put(Commands.MSGs, this.statistics.get(Commands.MSGs) + this.nbMsgRcvdMaster + this.nbMsgSendMaster);
		this.statistics.put(Commands.AO, this.nbAgentOrderingMsg);
		return this.statistics;
	}

	public boolean addMetric(MetricListener metric) {
		if (this.metrics == null) {
			this.metrics = new HashMap<Class<? extends MetricListener>, MetricListener>();
		}
		Class<? extends MetricListener> metricClass = metric.getClass();
		if (this.metrics.containsKey(metricClass)) {
			this.metrics.remove(metricClass);
			this.metrics.put(metricClass, metric);
			return false;
		}
		this.metrics.put(metricClass, metric);
		return true;
	}

}
