package simulation.model;

import static simulation.model.expressions.ExpressionFactory.abs;
import static simulation.model.expressions.ExpressionFactory.minus;
import java.util.ArrayList;
import java.util.List;
import kernel.Constraint;
import kernel.constraints.integer.SPrimitiveConstraint;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.IntVar;
import kernel.vars.integer.IntVariable;
import simple.SimpleAgent;
import simulation.model.expressions.AbsIntExpression;
import simulation.model.expressions.BooleanExpression;
import simulation.model.expressions.EqualIntExpression;
import simulation.model.expressions.GreateThanExpression;
import simulation.model.expressions.GreaterOrEqualExpression;
import simulation.model.expressions.IntConstantExpression;
import simulation.model.expressions.IntVariableExpression;
import simulation.model.expressions.IntegerExpression;
import simulation.model.expressions.LessOrEqualIntExpression;
import simulation.model.expressions.LessThanExpression;
import simulation.model.expressions.MinusIntExpression;
import simulation.model.expressions.NotEqualIntExpression;
import simulation.model.expressions.PlusIntExpression;
import simulation.model.expressions.VariableExpression;

public class ConstraintFactory {

	public static final Constraint TRUE = new SPrimitiveConstraint(null, new BooleanExpression() {
		@Override
		public Boolean evaluate() {
			return true;
		}

		@Override
		public String getSymbol() {
			return "True";
		}

		@Override
		public List<VariableExpression> getVariableExpressions() {
			return new ArrayList<VariableExpression>();
		}

		@Override
		public boolean isReady() {
			return true;
		}

		@Override
		public String getFunctionalExpression(int fromIndex) {
			return "True";
		}

		@Override
		public int getNbVariableExpression() {
			return 0;
		}
	});

	public static final Constraint FALSE = new SPrimitiveConstraint(null, new BooleanExpression() {
		@Override
		public Boolean evaluate() {
			return false;
		}

		@Override
		public String getSymbol() {
			return "False";
		}

		@Override
		public List<VariableExpression> getVariableExpressions() {
			return new ArrayList<VariableExpression>();
		}

		@Override
		public boolean isReady() {
			return true;
		}

		@Override
		public String getFunctionalExpression(int fromIndex) {
			return "False";
		}

		@Override
		public int getNbVariableExpression() {
			return 0;
		}
	});

	// //////////////////////////////////////////////////////////////////////////
	// //////////////
	// ////////////////////////// Boolean constraint
	// /////////////////////////////////////////
	// //////////////////////////////////////////////////////////////////////////
	// ////////////

	public static Constraint and(SimpleAgent agent, Constraint constraint1, Constraint constraint2) {
		throw new UnsupportedOperationException("not yet implemented");
		// if(constraint1.getOwnerId()!=constraint2.getOwnerId() &&
		// constraint1.getOwnerId()!=agent.getAgentId())
		// throw new
		// Error("Can not cread AND constraint using a not owned constraint");
		// AndExpression andExpr = new AndExpression(new
		// ConstraintExpression(constraint1), new
		// ConstraintExpression(constraint2));
		// return makePrimitiveConstraint(agent, andExpr);
	}

	public static Constraint ifOnlyIf(SimpleAgent agent, Constraint constraint1, Constraint constraint2) {
		throw new UnsupportedOperationException("not yet implemented");
		// if(constraint1.getOwnerId()!=constraint2.getOwnerId() &&
		// constraint1.getOwnerId()!=agent.getAgentId())
		// throw new
		// Error("Can not cread ifOnlyIf constraint using a not owned constraint"
		// );
		// IfOnlyIfExpression ifOnlyIf = new IfOnlyIfExpression(new
		// ConstraintExpression(constraint1), new
		// ConstraintExpression(constraint2));
		// return makePrimitiveConstraint(agent, ifOnlyIf);
	}

	public static Constraint not(SimpleAgent agent, Constraint constraint) {
		throw new UnsupportedOperationException("not yet implemented");
		// //return constraint.getOpposite();
		// return makePrimitiveConstraint(agent, new NotExpression(new
		// ConstraintExpression(constraint)));
	}

	public static Constraint or(SimpleAgent agent, Constraint constraint1, Constraint constraint2) {
		throw new UnsupportedOperationException("not yet implemented");
		// if(constraint1.getOwnerId()!=constraint2.getOwnerId() &&
		// constraint1.getOwnerId()!=agent.getAgentId())
		// throw new
		// Error("Can not cread OR constraint using a not owned constraint");
		// return makePrimitiveConstraint(agent, new OrExpression(new
		// ConstraintExpression(constraint1), new
		// ConstraintExpression(constraint2)));
	}

	// //////////////////////////////////////////////////////////////////////////
	// /
	// ///////////////////// Constraints from expressions
	// //////////////////////////
	// /////////////////////////////////////////////////////////////////////////

	private static Constraint makePrimitiveConstraint(SimpleAgent agent, BooleanExpression expression) {
		Constraint exC = new SPrimitiveConstraint(agent, expression);
		System.err.println("Warning: " + exC + "\n is a primitive constraint, it may strongly decreases the performances!\n");

		return exC;
	}

	// //////////////////////////////////////////////////////////////////////////
	// ///////////////
	// ///////////////////////// equality constraint
	// ////////////////////////////////////////////
	// //////////////////////////////////////////////////////////////////////////
	// ////////////////
	public static Constraint eq(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		Constraint exC = creatEqConstraint(Constraint.EQ, agent, v1, v2);
		return exC;
	}

	public static Constraint neq(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		Constraint exC = creatEqConstraint(Constraint.NEQ, agent, v1, v2);
		return exC;
	}

	public static Constraint geq(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		Constraint exC = creatEqConstraint(Constraint.GEQ, agent, v1, v2);
		return exC;
	}

	public static Constraint leq(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		Constraint exC = creatEqConstraint(Constraint.LEQ, agent, v1, v2);
		return exC;
	}

	public static Constraint gt(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		Constraint exC = creatEqConstraint(Constraint.GT, agent, v1, v2);
		return exC;
	}

	public static Constraint lt(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		Constraint exC = creatEqConstraint(Constraint.LT, agent, v1, v2);
		return exC;
	}

	private static Constraint creatEqConstraint(int type, SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		int n1 = v1.getNbVariableExpression();
		int n2 = v2.getNbVariableExpression();
		int art = n1 + n2;
		switch (art) {
			case 0 :
				return creatConstPredicate(type, v1, v2);
			case 1 :
				return createEqConst(type, agent, v1, v2);
			case 2 : {
				if (n1 == n2)
					return creatBinEq(type, agent, v1, v2);
				else {
					Constraint exC = buildDistance(type, agent, v1, v2);
					if (exC != null)
						return exC;
					IntVariableExpression ve1 = (IntVariableExpression) v1.getVariableExpressions().get(0);
					IntVariableExpression ve2 = (IntVariableExpression) v2.getVariableExpressions().get(0);
					IntVariable x1 = (IntVariable) ve1.getVariable();
					IntVariable x2 = (IntVariable) ve2.getVariable();
					if (Math.max(x1.getDomainSize(), x2.getDomainSize()) < 30) { return makeExtentionConstraint(agent, ve1, ve2,
							new EqualIntExpression(v1, v2)); }
					return makePrimitiveConstraint(agent, new EqualIntExpression(v1, v2));
				}
			}
			default :
				return makePrimitiveConstraint(agent, new EqualIntExpression(v1, v2));
		}
	}

	private static Constraint makeExtentionConstraint(SimpleAgent agent, 
			IntVariableExpression v0, IntVariableExpression v1, BooleanExpression exp) {
		IntExternalVar exV = null;
		IntVariable myVar = null;
		boolean transpose = false;
		if (v0.getVariable().equals(agent.var)) {
			myVar = ((IntVariable) v0.getVariable());
			exV = agent.makeExternalVar((IntVar) v1.getVariable());
		} else {
			transpose = true;
			myVar = ((IntVariable) v1.getVariable());
			exV = agent.makeExternalVar((IntVar) v0.getVariable());
		}
		int offset = myVar.getDomain().getInf();
		int domSZ = myVar.getDomainSize();
		int[][] tuples = new int[domSZ][];
		for (int myValue : myVar.getDomain()) {
			if(transpose){
				v1.setTemporaryValue(myValue);
			} else {
				v0.setTemporaryValue(myValue);
			}
			List<Integer> allowed = new ArrayList<Integer>();
			for (int exValue : exV.getDomain()) {
				if(transpose){
					v0.setTemporaryValue(exValue);
				} else {
					v1.setTemporaryValue(exValue);
				}
				if (exp.evaluate()) {
					allowed.add(exValue);
				}
			}
			tuples[myValue - offset] = util.Tools.toArray(allowed);
		}
		throw new UnsupportedOperationException("not yet implemented");
		//return agent.makeConstraintTable("Expression", true, myVar, exV, tuples, false);
	}

	private static Constraint buildDistance(int type, SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		if (v1 instanceof AbsIntExpression && v2 instanceof IntConstantExpression) { return buildDis(type, agent, (AbsIntExpression) v1,
				(IntConstantExpression) v2); }
		if (v2 instanceof AbsIntExpression && v1 instanceof IntConstantExpression) { return buildDis(type, agent, (AbsIntExpression) v2,
				(IntConstantExpression) v1); }
		return null;
	}

	private static Constraint buildDis(int type, SimpleAgent agent, AbsIntExpression v1, IntConstantExpression v2) {
		IntegerExpression exp = v1.getExpression();
		if (exp instanceof MinusIntExpression) {
			if (((MinusIntExpression) exp).getLeft() instanceof IntVariableExpression
					&& ((MinusIntExpression) exp).getRight() instanceof IntVariableExpression) {
				IntVariable var1 = (IntVariable) v1.getVariableExpressions().get(0).getVariable();
				IntVariable var2 = (IntVariable) v1.getVariableExpressions().get(1).getVariable();

				IntExternalVar exv = null;
				if (var1.equals(agent.var)) {
					exv = agent.makeExternalVar(var2);
				} else {
					exv = agent.makeExternalVar(var1);
				}
				int cste = v2.evaluate().intValue();
				return agent.distanceXYC(exv, cste, type);
			}
		}
		return null;

	}

	private static Constraint creatBinEq(int type, SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		// ----------------------------------------------------------------------
		// -------------------------------------//
		if (v1 instanceof IntVariableExpression && v2 instanceof IntVariableExpression) { // eq(x,y)
			return creatBinEq0(type, agent, v1, v2);
		}// end eq(x,y) constraint
		// ----------------------------------------------------------------------
		// -------------------------------------//
		if (v1 instanceof IntVariableExpression && (v2 instanceof PlusIntExpression || v2 instanceof MinusIntExpression)) { // eq
			// (
			// x
			// ,
			// y
			// +
			// C
			// )
			return creatBinEq1(type, agent, v1, v2);
		}
		if (v2 instanceof IntVariableExpression && (v1 instanceof PlusIntExpression || v1 instanceof MinusIntExpression)) { return creatBinEq1(type,
				agent, v2, v1); } // eq(x,y+c)
		// ----------------------------------------------------------------------
		// -------------------------------------//
		if ((v1 instanceof PlusIntExpression || v1 instanceof MinusIntExpression)
				&& (v2 instanceof PlusIntExpression || v2 instanceof MinusIntExpression)) { // eq
			// (
			// x
			// +
			// c1
			// ,
			// y+c2)
			// to
			// eq(x,
			// y+c)
			return creatBinEq2(type, agent, v2, v1);
		}
		// ----------------------------------------------------------------------
		// -------------------------------------//
		if (v1 instanceof IntVariableExpression && v2 instanceof AbsIntExpression && type == Constraint.EQ) { return creatAbsolute(type, agent,
				(IntVariableExpression) v1, (AbsIntExpression) v2); }// end x= |y+c| constraint
		// ----------------------------------------------------------------------
		// -------------------------------------//
		if (v1 instanceof AbsIntExpression && v2 instanceof IntVariableExpression && type == Constraint.EQ) { return creatAbsolute(type, agent,
				(IntVariableExpression) v2, (AbsIntExpression) v1); }// end x= |y+c| constraint
		return makePrimitiveConstraint(agent, new EqualIntExpression(v1, v2));
	}

	private static Constraint creatAbsolute(int type, SimpleAgent agent, IntVariableExpression v1, AbsIntExpression v2) {
		IntVariable var1 = (IntVariable) ((IntVariableExpression) v1).getVariable();
		IntVariable var2 = (IntVariable) v2.getVariableExpressions().get(0).getVariable();
		IntExternalVar exv = null;
		if (var1.equals(agent.var)) {
			exv = agent.makeExternalVar(var2);
		} else {
			exv = agent.makeExternalVar(var1);
		}
		int cste = 0;
		IntegerExpression exp = v2.getExpression();
		if (exp instanceof IntVariableExpression)
			return agent.eqAbsYplusC(exv, cste);

		if (exp instanceof PlusIntExpression) {
			if (((PlusIntExpression) exp).getLeft() instanceof IntConstantExpression)
				cste = (Integer) ((PlusIntExpression) exp).getLeft().evaluate();
			else
				cste = (Integer) ((PlusIntExpression) exp).getRight().evaluate();
		} else if (exp instanceof MinusIntExpression) {
			if (((MinusIntExpression) exp).getLeft() instanceof IntConstantExpression)
				cste = -(Integer) ((MinusIntExpression) exp).getLeft().evaluate();
			else
				cste = -(Integer) ((MinusIntExpression) exp).getRight().evaluate();
		} else {
			return makePrimitiveConstraint(agent, new EqualIntExpression(v1, v2));
		}
		return agent.eqAbsYplusC(exv, cste);

	}

	private static Constraint creatBinEq2(int type, SimpleAgent agent, IntegerExpression v2, IntegerExpression v1) {
		int cste = 0;
		if (v1 instanceof PlusIntExpression) {
			if (((PlusIntExpression) v1).getLeft() instanceof IntConstantExpression)
				cste -= (Integer) ((PlusIntExpression) v1).getLeft().evaluate();
			else
				cste -= (Integer) ((PlusIntExpression) v1).getRight().evaluate();
		} else {
			if (((MinusIntExpression) v1).getLeft() instanceof IntConstantExpression)
				cste += (Integer) ((MinusIntExpression) v1).getLeft().evaluate();
			else
				cste += (Integer) ((MinusIntExpression) v1).getRight().evaluate();
		}
		if (v2 instanceof PlusIntExpression) {
			if (((PlusIntExpression) v2).getLeft() instanceof IntConstantExpression)
				cste += (Integer) ((PlusIntExpression) v2).getLeft().evaluate();
			else
				cste += (Integer) ((PlusIntExpression) v2).getRight().evaluate();
		} else {
			if (((MinusIntExpression) v2).getLeft() instanceof IntConstantExpression)
				cste -= (Integer) ((MinusIntExpression) v2).getLeft().evaluate();
			else
				cste -= (Integer) ((MinusIntExpression) v2).getRight().evaluate();
		}
		IntVariable var1 = (IntVariable) ((IntVariableExpression) v1).getVariable();
		IntVariable var2 = (IntVariable) ((IntVariableExpression) v2).getVariable();
		IntExternalVar exv = null;
		if (var1.equals(agent.var)) {
			exv = agent.makeExternalVar(var2);
		} else {
			exv = agent.makeExternalVar(var1);
			if (type != Constraint.NEQ && type != Constraint.EQ) {
				type = -type;
			}
		}
		switch (type) {
			case Constraint.NEQ :
				return agent.neqYplusC(exv, cste);
			case Constraint.EQ :
				return agent.eqYplusC(exv, cste);
			case Constraint.LEQ :
				return agent.leqYplusC(exv, cste);
			case Constraint.GEQ :
				return agent.geqYplusC(exv, cste);
			case Constraint.LT :
				return agent.ltYplusC(exv, cste);
			case Constraint.GT :
				return agent.gtYplusC(exv, cste);
			default :
				throw new Error("Unknown type of equality constraint!");
		}
	}

	private static Constraint creatBinEq0(int type, SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		IntVariable var1 = (IntVariable) ((IntVariableExpression) v1).getVariable();
		IntVariable var2 = (IntVariable) ((IntVariableExpression) v2).getVariable();
		if (!(var1.equals(agent.var) || var2.equals(agent.var)))
			throw new Error("An agent can not creat a constraint of an other agent " + var1 + " " + var2 + " " + v1 + " " + v2);
		IntExternalVar exv = null;
		if (var1.equals(agent.var)) {
			exv = agent.makeExternalVar(var2);
		} else {
			exv = agent.makeExternalVar(var1);
			if (type != Constraint.NEQ && type != Constraint.EQ) {
				type = -type;
			}
		}
		switch (type) {
			case Constraint.NEQ :
				return agent.neqY(exv);
			case Constraint.EQ :
				return agent.eqY(exv);
			case Constraint.LEQ :
				return agent.leqY(exv);
			case Constraint.GEQ :
				return agent.geqY(exv);
			case Constraint.LT :
				return agent.ltY(exv);
			case Constraint.GT :
				return agent.gtY(exv);
			default :
				throw new Error("Unknown type of equality constraint!");
		}
	}

	private static Constraint creatBinEq1(int type, SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		IntVariable var1 = (IntVariable) v1.getVariableExpressions().get(0).getVariable();
		IntVariable var2 = (IntVariable) v2.getVariableExpressions().get(0).getVariable();
		int cste = 0;
		if (v2 instanceof PlusIntExpression) {
			if (((PlusIntExpression) v2).getLeft() instanceof IntConstantExpression)
				cste = (Integer) ((PlusIntExpression) v2).getLeft().evaluate();
			else
				cste = (Integer) ((PlusIntExpression) v2).getRight().evaluate();
		} else {
			if (((MinusIntExpression) v2).getLeft() instanceof IntConstantExpression)
				cste = -(Integer) ((MinusIntExpression) v2).getLeft().evaluate();
			else
				cste = -(Integer) ((MinusIntExpression) v2).getRight().evaluate();
		}
		IntExternalVar exv = null;
		if (var1.equals(agent.var)) {
			exv = agent.makeExternalVar(var2);
		} else {
			exv = agent.makeExternalVar(var1);
			cste = -cste;
			if (type != Constraint.NEQ && type != Constraint.EQ) {
				type = -type;
			}
		}
		switch (type) {
			case Constraint.NEQ :
				return agent.neqYplusC(exv, cste);
			case Constraint.EQ :
				return agent.eqYplusC(exv, cste);
			case Constraint.LEQ :
				return agent.leqYplusC(exv, cste);
			case Constraint.GEQ :
				return agent.geqYplusC(exv, cste);
			case Constraint.LT :
				return agent.ltYplusC(exv, cste);
			case Constraint.GT :
				return agent.gtYplusC(exv, cste);
			default :
				throw new Error("Unknown type of equality constraint!");
		}

	}

	private static Constraint createEqConst(int type, SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		if (!(((VariableExpression) v1).getVariable().equals(agent.var) || ((VariableExpression) v2).getVariable().equals(agent.var)))
			throw new Error("An agent can not creat a constraint of an other agent");

		int cste;

		if (v1 instanceof IntVariableExpression)
			cste = v2.evaluate().intValue();
		else {
			cste = v1.evaluate().intValue();
			if (type != Constraint.NEQ && type != Constraint.EQ) {
				type = -type;
			}
		}
		switch (type) {
			case Constraint.NEQ :
				return agent.neqC(cste);
			case Constraint.EQ :
				return agent.eqC(cste);
			case Constraint.LEQ :
				return agent.leqC(cste);
			case Constraint.GEQ :
				return agent.geqC(cste);
			case Constraint.LT :
				return agent.ltC(cste);
			case Constraint.GT :
				return agent.gtC(cste);
			default :
				throw new Error("Unknown type of equality constraint!");
		}
	}

	private static Constraint creatConstPredicate(int type, IntegerExpression v1, IntegerExpression v2) {
		boolean b = false;
		switch (type) {
			case Constraint.NEQ :
				b = (v1.evaluate().intValue() != v2.evaluate().intValue());
			break;
			case Constraint.EQ :
				b = (v1.evaluate().intValue() == v2.evaluate().intValue());
			break;
			case Constraint.LEQ :
				b = (v1.evaluate().intValue() <= v2.evaluate().intValue());
			break;
			case Constraint.GEQ :
				b = (v1.evaluate().intValue() >= v2.evaluate().intValue());
			break;
			case Constraint.LT :
				b = (v1.evaluate().intValue() < v2.evaluate().intValue());
			break;
			case Constraint.GT :
				b = (v1.evaluate().intValue() > v2.evaluate().intValue());
			break;
			default :
				throw new Error("Unknown type of equality constraint!");
		}
		if (b)
			return TRUE;
		else
			return FALSE;
	}

	// //////////////////////////////////////////////////////////////////////////
	// //////////////////////////////////
	// ///////////////////////// Distance constraints
	// ////////////////////////////////////////////////////////////
	// //////////////////////////////////////////////////////////////////////////
	// ////////////////////////////////

	public static Constraint distanceEQ(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2, int cste) {
		Constraint exC = distance(Constraint.EQ, agent, v1, v2, cste);
		return exC;
	}

	public static Constraint distanceNEQ(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2, int cste) {
		Constraint exC = distance(Constraint.NEQ, agent, v1, v2, cste);
		return exC;
	}

	public static Constraint distanceGT(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2, int cste) {
		Constraint exC = distance(Constraint.GT, agent, v1, v2, cste);
		return exC;
	}

	public static Constraint distanceLT(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2, int cste) {
		Constraint exC = distance(Constraint.LT, agent, v1, v2, cste);
		return exC;
	}

	private static Constraint distance(int type, SimpleAgent agent, IntegerExpression v1, IntegerExpression v2, int cste) {
		int arrity = v1.getNbVariableExpression() + v2.getNbVariableExpression();
		if (arrity == 0)
			return creatConstPredicate2(type, v2, v2, cste);

		if (v1 instanceof IntVariableExpression && v2 instanceof IntVariableExpression) {
			IntVariable var1 = (IntVariable) ((IntVariableExpression) v1).getVariable();
			IntVariable var2 = (IntVariable) ((IntVariableExpression) v2).getVariable();
			if (!(var1.equals(agent.var) || var2.equals(agent.var)))
				throw new Error("An agent can not creat a constraint of an other agent");
			IntExternalVar exv = null;
			if (var1.equals(agent.var))
				exv = agent.makeExternalVar(var2);
			else
				exv = agent.makeExternalVar(var1);

			return agent.distanceXYC(exv, cste, type);
		}

		return makePrimitiveConstraint(agent, makeBoolExpression(type, abs(minus(v1, v2)), cste));
	}

	private static BooleanExpression makeBoolExpression(int type, IntegerExpression e, int cste) {
		switch (type) {
			case Constraint.NEQ :
				return new NotEqualIntExpression(e, cste);
			case Constraint.EQ :
				return new EqualIntExpression(e, cste);
			case Constraint.LEQ :
				return new LessOrEqualIntExpression(e, cste);
			case Constraint.GEQ :
				return new GreaterOrEqualExpression(e, cste);
			case Constraint.LT :
				return new LessThanExpression(e, cste);
			case Constraint.GT :
				return new GreateThanExpression(e, cste);
			default :
				throw new Error("Unknown operator of comparaison !");
		}
	}

	private static Constraint creatConstPredicate2(int type, IntegerExpression v1, IntegerExpression v2, int cste) {
		boolean b = false;
		int d = Math.abs((v1.evaluate().intValue() - v2.evaluate().intValue()));
		switch (type) {
			case Constraint.NEQ :
				b = d != cste;
			break;
			case Constraint.EQ :
				b = d == cste;
			break;
			case Constraint.LEQ :
				b = d <= cste;
			break;
			case Constraint.GEQ :
				b = d >= cste;
			break;
			case Constraint.LT :
				b = d < cste;
			break;
			case Constraint.GT :
				b = d > cste;
			break;
			default :
				throw new Error("Unknown type of equality constraint!");
		}
		if (b)
			return TRUE;
		else
			return FALSE;
	}

	public static Constraint distanceEQ(SimpleAgent agent, IntegerExpression v, IntegerExpression v2, IntegerExpression v3) {
		throw new Error("TODO");
	}

	public static Constraint distanceGT(SimpleAgent agent, IntegerExpression v, IntegerExpression v2, IntegerExpression v3) {
		throw new Error("TODO");
	}

	public static Constraint distanceLT(SimpleAgent agent, IntegerExpression v, IntegerExpression v2, IntegerExpression v3) {
		throw new Error("TODO");
	}

	public static Constraint oppositeSign(SimpleAgent agent, IntegerExpression v, IntegerExpression v2) {
		throw new Error("TODO");
	}

	public static Constraint sameSign(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2) {
		throw new Error("TODO");
	}

	public static Constraint min(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2, IntegerExpression v3) {
		throw new Error("TODO");
	}

	public static Constraint max(SimpleAgent agent, IntegerExpression v1, IntegerExpression v2, IntegerExpression v3) {
		throw new Error("TODO");
	}

}
