package simulation.model.expressions;

import java.util.ArrayList;
import java.util.List;

public abstract class BinaryExpression implements Expression {

	protected Expression fLeft;

	protected Expression fRight;

	public BinaryExpression(Expression left, Expression right) {
		super();
		setLeft(left);
		setRight(right);
	}

	public Expression getLeft() {
		return fLeft;
	}

	public void setLeft(Expression expression) {
		fLeft = expression;
	}

	public Expression getRight() {
		return fRight;
	}

	public void setRight(Expression expression) {
		fRight = expression;
	}

	@Override
	public List<? extends VariableExpression> getVariableExpressions() {
		List<VariableExpression> all = new ArrayList<VariableExpression>();
		all.addAll(getLeft().getVariableExpressions());
		all.addAll(getRight().getVariableExpressions());
		return all;
	}

	@Override
	public int getNbVariableExpression() {
		return this.fLeft.getNbVariableExpression() + this.fRight.getNbVariableExpression();
	}

	public String toString() {
		return getLeft().toString() + " " + getSymbol() + " " + getRight().toString();
	}

	public boolean isReady() {
		return this.fLeft.isReady() && this.fRight.isReady();

	}

	@Override
	public String getFunctionalExpression(int fromIndex) {
		int leftSize = this.fLeft.getVariableExpressions().size();
		return this.fLeft.getFunctionalExpression(fromIndex) + "," + this.fRight.getFunctionalExpression(fromIndex + leftSize);
	}

}
