package simulation.parser;

public class InstanceTokens {
	
	public static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
	public static final String INSTANCE_SCHEMA = "XDisCSP_1_0.xsd";
//	public static final String INSTANCE_STYLESHEET_2_1 = "instance_2_1.xsl";
	public static final String SAT = "SAT";
	public static final String UNSAT = "UNSAT";

	public static final String SOL = "SOL";
	public static final String INSTANCE = "instance";
	public static final String INSTANCES = "instances";
	public static final String WHITE_SPACE = " \t\n\r\f";
	public static final String BRACKETS = "[]{}";

	/**
	 * Used as an attribute to identify domains, variables, relations, predicates and constraints.
	 */
	public static final String NAME = "name";

	//description 	presentation
	public static final String DESCRIPTION = "description";
	public static final String PRESENTATION = "presentation";
	public static final String MAX_CONSTRAINT_ARITY = "maxConstraintArity";
	public static final String MIN_VIOLATED_CONSTRAINTS = "minViolatedConstraints";
	public static final String SOLUTION = "solution";
	public static final String NB_SOLUTIONS = "nbSolutions";
	public static final String FORMAT = "format";
	public static final String XDisCSP_1_0 = "XDisCSP 1.0";
	public static final String TYPE = "type";
	public static final String DisCSP = "DisCSP";
	public static final String DCOP = "DCOP";
	public static final String MaxDisCSP = "MaxDisCSP";
	public static final String BENCHMARK = "benchmark";
	public static final String SATISFACTION = "SATISFACTION";
	public static final String OPTIMIZATION = "OPTIMIZATION";
	public static final String HYBRID = "HYBRID";
	
	// agents
	public static final String AGENT = "agent";
	public static final String AGENTS = "agents";
	public static final String NB_AGENTS = "nbAgents";
	public static final String ID = "id";

	// domains
	public static final String DOMAINS = "domains";
	public static final String NB_DOMAINS = "nbDomains";
	public static final String DOMAIN = "domain";
	public static final String NB_VALUES = "nbValues";
	public static final String VALUES = "values";
	public static final String VALUE_SEPARATOR = " ";
	public static final String DISCRETE_INTERVAL_START = "";
	public static final String DISCRETE_INTERVAL_SEPARATOR = "..";
	public static final String DISCRETE_INTERVAL_END = "";
	public static final String VARIABLE_IDENTIFIER_SEPARATOR = ".";

	// variables
	public static final String VARIABLES = "variables";
	public static final String NB_VARIABLES = "nbVariables";
	public static final String VARIABLE = "variable";

	// relations
	public static final String RELATIONS = "relations";
	public static final String NB_RELATIONS = "nbRelations";
	public static final String RELATION = "relation";
	public static final String SUPPORTS = "supports";
	public static final String NB_SUPPORTS = "nbSupports";
	public static final String CONFLICTS = "conflicts";
	public static final String TUPLES_SEPARATOR = "|";
	public static final String SOFT = "soft";
	public static final String INFINITY = "infinity";
	public static final String DEFAULT_COST = "defaultCost";
	public static final String COST_SEPARATOR = ":";
	public static final String NB_CONFLICTS = "nbConflicts";
	public static final String ARITY = "arity";
	public static final String NB_TUPLES = "nbTuples";
	public static final String SEMANTICS = "semantics";

	// functions
	public static final String FUNCTIONS = "functions";
	public static final String NB_FUNCTIONS = "nbFunctions";
	public static final String FUNCTION = "function";

	// predicates
	public static final String PREDICATES = "predicates";
	public static final String NB_PREDICATES = "nbPredicates";
	public static final String PREDICATE = "predicate";
	public static final String EXPRESSION = "expression";
	public static final String FUNCTIONAL = "functional";
	public static final String POSTFIX_EXPRESSION = "postfixExpression";
	public static final String PARAMETERS = "parameters";
	public static final String LIST = "list";

	// global constraints
	public static final String PREFIX_GLOBAL = "global:";
	public static final String ALL_DIFFERENT = "allDifferent";
	public static final String WEIGHTED_SUM = "weightedSum";
	public static final String CUMULATIVE = "cumulative";
	public static final String ELEMENT = "element";
	public static final String NIL = "nil";
	
	public static String getGlobalNameOf(String s) {
		return PREFIX_GLOBAL + s;
	}

	public static String getLowerCaseGlobalNameOf(String s) {
		return PREFIX_GLOBAL + s.toLowerCase();
	}

	// constraints
	public static final String CONSTRAINTS = "constraints";
	public static final String NB_CONSTRAINTS = "nbConstraints";
	public static final String INITIAL_COST = "initialCost";
	public static final String MINIMAL_COST = "minimalCost";
	public static final String MAXIMAL_COST = "maximalCost";
	public static final String CONSTRAINT = "constraint";
	public static final String SCOPE = "scope";
	public static final String REFERENCE = "reference";
	public static final String CST = "cst";
	public static final String VAR = "var";
	public static final String VALUE = "value";

	// names of elements
	public static final String AGENT_PREFIX = "A";
	public static final String DOMAIN_PREFIX = "D";
	public static final String VARIABLE_PREFIX = "X";
	public static final String RELATION_PREFIX = "R";
	public static final String FUNCTION_PREFIX = "F";
	public static final String PREDICATE_PREFIX = "P";
	public static final String CONSTRAINT_PREFIX = "C";
	public static final String PARAMETER_PREFIX = "X";
	public static final String SENSOR_PREFIX = "S";
	public static final String TARGET_PREFIX = "T";
	public static final String MEETING_PREFIX = "M";
	public static final String MODEL = "model";
	public static final String CATEGORY = "category";
	public static final String OWNER_ID = "ownerId";
	public static final String SIMPLE_MODEL = "Simple";
	public static final String COMPLEX_MODEL = "Complex";
	public static final String TKC = "TKC";
	public static final String PKC = "PKC";
	public static final String CONSTRAINT_MODEL = "constraintModel";
	public static final String EXTENSION = "Extention";
	public static final String INTENSION = "Intention";
	public static final String NB_INSTANCES = "nbInstances";
	
	public static String getAgentNameFor(int id) {
		return AGENT_PREFIX + id;
	}

	public static String getDomainNameFor(int id) {
		return DOMAIN_PREFIX + id;
	}

	public static String getVariableNameFor(int id) {
		return VARIABLE_PREFIX + id;
	}

	public static String getVariableNameFor(String varId) {
		return VARIABLE_PREFIX + varId;
	}

	public static String getRelationNameFor(int id) {
		return RELATION_PREFIX + id;
	}

	public static String getFunctionNameFor(int id) {
		return FUNCTION_PREFIX + id;
	}

	public static String getPredicateNameFor(int id) {
		return PREDICATE_PREFIX + id;
	}

	public static String getConstraintNameFor(int id) {
		return CONSTRAINT_PREFIX + id;
	}

	public static String getConstraintNameFor(String name) {
		return CONSTRAINT_PREFIX + name;
	}

	public static String getParameterNameFor(int id) {
		return PARAMETER_PREFIX + id;
	}
	
	public static String getSensorNameFor(int id) {
		return SENSOR_PREFIX + id;
	}
	
	public static String getTargetNameFor(int id) {
		return TARGET_PREFIX + id;
	}

	public static String getMeetingNameFor(int id) {
		return MEETING_PREFIX + id;
	}

	public static String getMeetingNameFor(String varId) {
		return MEETING_PREFIX + varId;
	}
	
	public static boolean isCanonicalName(String prefix, String name) {
		return name.startsWith(prefix) && Toolkit.isInteger(name.substring(prefix.length()));
	}

    /**
     * Returns a string representation of the contents of the specified array.
     * The string representation consists of a list of the array's elements,
     * enclosed in square brackets (<tt>"[]"</tt>).  Adjacent elements are
     * separated by the characters <tt>", "</tt> (a comma followed by a
     * space).  Elements are converted to strings as by
     * <tt>String.valueOf(int)</tt>.  Returns <tt>"null"</tt> if <tt>a</tt> is
     * <tt>null</tt>.
     *
     * @param a the array whose string representation to return
     * @return a string representation of <tt>a</tt>
     * @since 1.5
     */
	public static <T> String domainToString(T[] values) {
		if (values == null)
			return "null";
		int length = values.length - 1;
		if (length == -1)
			return "";
		StringBuilder sb = new StringBuilder();
		for(int i = 0 ; ; i++) {
			sb.append(values[i]);
			if (i == length){
				return sb.toString();
			}
			sb.append(" ");
		}
	}
}
