package simulation.parser;


public abstract class PConstraint {

	protected String name;

	protected int ownerId = -1;

	protected PVariable[] scope;

	public String getName() {
		return name;
	}

	public PVariable[] getScope() {
		return scope;
	}

	public int getArity() {
		return scope.length;
	}

	/**
	 * @param name
	 *            the name of the constraint
	 * @param scope
	 *            The variables in the constraint scope
	 */
	public PConstraint(String name, PVariable[] scope) {
		this.name = name;
		this.scope = scope;
	}

	public void setOwnerId(int ownerId) {
		this.ownerId = ownerId;
	}

	public String toString() {
		String s = "<constraint name=\"" + this.name + "\" arity=\"" + scope.length;
		if (ownerId != -1) {
			s += "OwnerId=\"" + this.ownerId + "\"";
		}
		s += "\" scope=\"";
		for (int i = 0 ; i < scope.length ; i++)
			s += ((i == 0)? "" : " ") + scope[i].getName();
		s += "\"";
		return s;
	}

	public PAgent[] getScopeAgent() {
		PAgent[] scopeAg = new PAgent[this.scope.length];
		for (int i = 0 ; i < this.scope.length ; i++)
			scopeAg[i] = this.scope[i].getAgent();
		return scopeAg;
	}

	public abstract void makeConstraint();
	
}
