package simulation.parser;

import java.util.Arrays;

public class PDomain {

	String name;

	private Integer[] values;

	public PDomain(String name, Integer[] values) {
		this.name = name;
		this.values = values;
	}

	public String getName() {
		return this.name;
	}

	public Integer[] getValues() {
		return this.values;
	}

	public int getNbValues() {
		return this.values.length;
	}

	public boolean equals(Object o) {
		PDomain d = (PDomain) o;
		if (values.length != d.values.length) return false;
		int i = 0;
		for (int v : values) {
			if (d.values[i++] != v) return false;
		}
		return true;
	}

	public int size() {
		return this.values.length;
	}

	public int getMaxAbsoluteValue() {
		return Math.max(Math.abs(this.values[0]), Math.abs(this.values[values.length - 1]));
	}

	public boolean contains(int value) {
		return Arrays.binarySearch(this.values, value) >= 0;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("<").append(InstanceTokens.DOMAIN).append(" ");// "<domain "
		sb.append(InstanceTokens.NAME).append("=\"").append(this.name).append("\" ");
		sb.append(InstanceTokens.NB_VALUES).append("=\"").append(this.values.length).append("\">");
		sb.append(getStringListOfValues());
		sb.append("</").append(InstanceTokens.DOMAIN).append(">\n");
		return sb.toString();
	}

	public String getStringListOfValues() {
		int previousValue = values[0];
		boolean startedInterval = false;
		StringBuffer sb = new StringBuffer();
		for (int i = 1 ; i < values.length ; i++) {
			int currentValue = values[i];
			if (currentValue != previousValue + 1) {
				if (startedInterval) {
					sb.append(previousValue + InstanceTokens.DISCRETE_INTERVAL_END);
					startedInterval = false;
				} else
					sb.append(previousValue);
				sb.append(InstanceTokens.VALUE_SEPARATOR);
			} else {
				if (!startedInterval) {
					sb.append(InstanceTokens.DISCRETE_INTERVAL_START + previousValue + InstanceTokens.DISCRETE_INTERVAL_SEPARATOR);
					startedInterval = true;
				}
			}
			previousValue = currentValue;
		}
		if (startedInterval){
			sb.append(previousValue + InstanceTokens.DISCRETE_INTERVAL_END);
		} else {
			sb.append(previousValue);
		}
		return sb.toString();
	}

	public boolean controlValueRanging(int min, int max) {
		for (int v : values)
			if (v < min || v > max) return false;
		return true;
	}

}
