package simulation.parser;

import kernel.AbstractAgent;
import simple.SimpleAgent;

public class PExtensionConstraint extends PConstraint {

	private PRelation relation;

	public PRelation getRelation() {
		return relation;
	}

	public PExtensionConstraint(String name, PVariable[] scope, PRelation relation) {
		super(name, scope);
		this.relation = relation;
	}

	public int getMaximalCost() {
		return relation.getMaximalCost();
	}

	public long computeCostOf(int[] tuple) {
		return relation.computeCostOf(tuple);
	}

	public String toString() {
		return super.toString() + " reference=\"" + relation.getName() + "\"/>";
	}

	/**
	 * 
	 */
	public void makeConstraint() {
		int arity = this.relation.getArity();
		if (arity == 1) {
			PVariable variable = this.scope[0];
			AbstractAgent owner = variable.getDisChocoAgent();
			this.relation.makeUnaryConstraint(name, owner, variable);
		} else if (arity == 2) {
			for(PVariable variable : getScope()) {
				AbstractAgent agent = variable.getDisChocoAgent();
				if(ownerId==-1 || agent.getAgentId() == this.ownerId){
					this.makeConstraint(agent);
				}
			}
		} else { 
			throw new UnsupportedOperationException("\nNot Yet implemented for global constraints");
		}
	}
	
	private void makeConstraint(AbstractAgent owner) {
		//this.relation.makeBinaryConstraint(this.name, owner, this.scope);
		if (owner instanceof SimpleAgent) {
			this.relation.makeBinaryConstraint(this.name, (SimpleAgent) owner, this.scope);
		} else {
			throw new UnsupportedOperationException("Complex agents are not yet implemented !!!");
			// TODO
		}
	}

//	private void makeSBinaryConstraint(SimpleAgent agent) {	
//		int[][] tuples = this.relation.getTuples();
//		String semantic = this.relation.getSemantics();
//		boolean feas = false;
//		if (semantic.equals(InstanceTokens.SUPPORTS))
//			feas = true;
//		java.util.ArrayList<int[]> couples = new java.util.ArrayList<int[]>();
//		int agIdx = agent.getAgentId();
//		for (int i = 0 ; i < this.relation.getNbTuples() ; i++) {
//			int[] tuple = new int[2];
//			tuple[agIdx] = tuples[i][0];
//			tuple[1 - agIdx] = tuples[i][1];
//			couples.add(tuple);
//		}
//		int vId = this.getScope()[1 - agIdx].getId();
//		int agId = this.getScope()[1 - agIdx].getAgent().getId();
//		kernel.VarIdentifier extVarId = new kernel.VarIdentifier(agId, vId);
//		int[] values = this.getScope()[1 - agIdx].getDomain().getValues();
//		kernel.vars.integer.IntExternalVar extVar = agent.makeExternalVar(this.getScope()[1 - agIdx].getName(), extVarId, values);
//		kernel.Constraint exC = null;
//		if (this.relation instanceof PSoftRelation) {
//			//TODO
//			// int[] costs = ((PSoftRelation) this.relation).getWeights();
//			// if (feas)
//			// exC = agent.feasiblePair(extVar, couples, costs);
//			// else
//			// exC = agent.infeasiblePair(extVar, couples, costs);
//		} else {
//			if (feas) {
//				exC = agent.feasiblePair(extVar, couples);
//			} else {
//				exC = agent.infeasiblePair(extVar, couples);
//			}
//		}
//		exC.setReference(this.name);
//	}

}
