package simulation.parser;

import java.util.StringTokenizer;

public class PFunction {

	protected String formalParametersExpression;
	protected String name;
	protected String[] formalParameters;
	protected String functionalExpression;
	protected String[] unversalPostfixExpression;

	public String getName() {
		return name;
	}

	public String[] getFormalParameters() {
		return formalParameters;
	}

	public String[] getUniversalPostfixExpression() {
		return unversalPostfixExpression;
	}

	public PFunction(String name, String formalParametersExpression, String functionalExpression) {
		this.name = name;
		this.formalParameters = extractFormalParameters(formalParametersExpression, true);
		this.functionalExpression = functionalExpression;
		this.unversalPostfixExpression = buildUniversalPostfixExpression(functionalExpression, formalParameters);
		this.formalParametersExpression = formalParametersExpression;
	}

	/*
	 * Builds a universal postfix expression, i.e. a postfix expression such that the name of all formal parameters are put in a canonical form (as
	 * defined by XMLInstanceRepresentation.getParameterNameFor(position))
	 */
	public static String[] buildUniversalPostfixExpression(String functionalExpression, String formalParametersExpression) {
		return buildUniversalPostfixExpression(functionalExpression, extractFormalParameters(formalParametersExpression));
	}

	public static String[] buildUniversalPostfixExpression(String functionalExpression, String[] formalParameters) {
		StringTokenizer st = new StringTokenizer(functionalExpression, InstanceTokens.WHITE_SPACE + "(),");
		boolean[] found = new boolean[formalParameters.length];
		String postfixExpression = buildUniversalPostfixExpression(st, formalParameters, found);
		// System.out.println(functionalExpression + " " + postfixExpression);
		for (int i = 0 ; i < found.length ; i++)
			if (!found[i])
				throw new IllegalArgumentException("Formal parameter " + i + " not found in the given expression");
		st = new StringTokenizer(postfixExpression);
		String[] tokens = new String[st.countTokens()];
		for (int i = 0 ; i < tokens.length ; i++)
			tokens[i] = st.nextToken();
		return tokens;
	}

	private static String buildUniversalPostfixExpression(StringTokenizer st, String[] formalParameters, boolean[] found) {
		String token = st.nextToken();
		if (Toolkit.isInteger(token))
			return token;
		int position = Toolkit.searchFirstStringOccurrenceIn(token, formalParameters);
		if (position != -1) {
			found[position] = true;
			return InstanceTokens.getParameterNameFor(position);
		}
		String s = token;
		int arity = PredicateTokens.getArityOf(token);
		for (int i = 0 ; i < arity ; i++)
			s = buildUniversalPostfixExpression(st, formalParameters, found) + " " + s;
		return s;
	}

	public static String[] extractFormalParameters(String formalParametersExpression, boolean controlRedundancy) {
		StringTokenizer st = new StringTokenizer(formalParametersExpression);
		int cpt = 0;
		String[] formalParameters = new String[st.countTokens() / 2];
		while (st.hasMoreTokens()) {
			st.nextToken(); // curently, only int is authorized as type, so we
			// can discard this information
			String token = st.nextToken();
			if (controlRedundancy)
				for (int j = 0 ; j < cpt ; j++)
					if (formalParameters[j].equals(token))
						return null;
			formalParameters[cpt++] = token;
		}
		return formalParameters;
	}

	public static String[] extractFormalParameters(String formalParametersExpression) {
		return extractFormalParameters(formalParametersExpression, false);
	}

	public boolean equals(Object o) {
		PFunction f = (PFunction) o;
		return (this.functionalExpression + " " + this.formalParametersExpression)
				.equals(f.functionalExpression + " " + f.formalParametersExpression);
	}

	public String toString() {
		String s = "<function name=\"" + name + "\">\n";
		s += "\t\t\t<parameters>";
		s += this.formalParametersExpression;
		s += "</parameters>\n";
		s += "\t\t\t<expression>\n";
		s += "\t\t\t\t<functional>";
		s += functionalExpression;
		s += "</functional>\n";
		s += "\t\t\t</expression>\n";
		s += "\t\t</function>\n";
		return s;
	}
}
