package simulation.parser;

import static simulation.model.ConstraintFactory.FALSE;
import static simulation.model.ConstraintFactory.TRUE;
import static simulation.model.ConstraintFactory.and;
import static simulation.model.ConstraintFactory.distanceEQ;
import static simulation.model.ConstraintFactory.distanceGT;
import static simulation.model.ConstraintFactory.distanceLT;
import static simulation.model.ConstraintFactory.distanceNEQ;
import static simulation.model.ConstraintFactory.eq;
import static simulation.model.ConstraintFactory.geq;
import static simulation.model.ConstraintFactory.gt;
import static simulation.model.ConstraintFactory.ifOnlyIf;
import static simulation.model.ConstraintFactory.leq;
import static simulation.model.ConstraintFactory.lt;
import static simulation.model.ConstraintFactory.max;
import static simulation.model.ConstraintFactory.min;
import static simulation.model.ConstraintFactory.neq;
import static simulation.model.ConstraintFactory.not;
import static simulation.model.ConstraintFactory.oppositeSign;
import static simulation.model.ConstraintFactory.or;
import static simulation.model.ConstraintFactory.sameSign;
import static simulation.model.expressions.ExpressionFactory.abs;
import static simulation.model.expressions.ExpressionFactory.div;
import static simulation.model.expressions.ExpressionFactory.ifThenElse;
import static simulation.model.expressions.ExpressionFactory.max;
import static simulation.model.expressions.ExpressionFactory.min;
import static simulation.model.expressions.ExpressionFactory.minus;
import static simulation.model.expressions.ExpressionFactory.mod;
import static simulation.model.expressions.ExpressionFactory.mult;
import static simulation.model.expressions.ExpressionFactory.neg;
import static simulation.model.expressions.ExpressionFactory.plus;
import static simulation.model.expressions.ExpressionFactory.pow;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;
import kernel.AbstractAgent;
import kernel.Constraint;
import kernel.vars.integer.IntVariable;
import simple.SimpleAgent;
import simulation.model.expressions.ConstraintExpression;
import simulation.model.expressions.IntConstantExpression;
import simulation.model.expressions.IntVariableExpression;
import simulation.model.expressions.IntegerExpression;

public class PIntensionConstraint extends PConstraint {

	private PFunction function; // a predicate is a kind of function - so
	// function may be a PPredicate

	protected String effectiveParametersExpression;

	private String[] universalPostfixExpression;

	public PFunction getFunction() {
		return function;
	}

	public String[] getUniversalPostfixExpression() {
		return universalPostfixExpression;
	}

	public PIntensionConstraint(String name, PVariable[] scope, PFunction function, String effectivePE) {
		super(name, scope);
		this.function = function;
		this.effectiveParametersExpression = effectivePE;
		String[] variableNames = new String[scope.length];
		for (int i = 0 ; i < variableNames.length ; i++)
			variableNames[i] = scope[i].getName();
		this.universalPostfixExpression = buildNewUniversalPostfixExpression(
				function.getUniversalPostfixExpression(), effectiveParametersExpression, variableNames);

	}

	public static String[] extractUniversalEffectiveParameters(String effectiveParametersExpression, String[] variableNames) {
		// System.out.println(" effective = " + effectiveParametersExpression);
		// System.out.println(" variableNames = " +
		// Toolkit.buildStringFromTokens(variableNames));
		StringTokenizer st = new StringTokenizer(effectiveParametersExpression);
		String[] effectiveParameters = new String[st.countTokens()];
		for (int i = 0 ; i < effectiveParameters.length ; i++) {
			String token = st.nextToken();
			if (!Toolkit.isInteger(token)) {
				int position = Toolkit.searchFirstStringOccurrenceIn(token, variableNames);
				if (position == -1)
					throw new IllegalArgumentException();
				token = InstanceTokens.getParameterNameFor(position);
			}
			effectiveParameters[i] = token;
		}
		return effectiveParameters;
	}

	/*
	 * Builds a new universal postfix expression, i.e. a postfix expression such that the name of all formal parameters are put in a canonical form
	 * (as defined by XMLInstanceRepresentation.getParameterNameFor(position)) after passing the effective parameters.
	 */
	public static String[] buildNewUniversalPostfixExpression(String[] universalPostfixExpression, String effectiveParametersExpression,
			String[] variableNames) {
		return buildNewUniversalPostfixExpression(universalPostfixExpression, extractUniversalEffectiveParameters(effectiveParametersExpression,
				variableNames));
	}

	private static String[] buildNewUniversalPostfixExpression(String[] universalPostfixExpression, String[] universalEffectiveParameters) {

		String[] tokens = new String[universalPostfixExpression.length];
		for (int i = 0 ; i < tokens.length ; i++) {
			String token = universalPostfixExpression[i];
			if (token.startsWith(InstanceTokens.PARAMETER_PREFIX))
				tokens[i] = universalEffectiveParameters[Integer.parseInt(token.substring(InstanceTokens.PARAMETER_PREFIX.length()))];
			else
				tokens[i] = token;
		}
		return tokens;
	}

	public String toString() {
		String s = super.toString();
		s += " reference=\"" + function.getName() + "\">\n";
		s += "\t\t\t<parameters>";
		s += effectiveParametersExpression;
		s += "</parameters>\n";
		s += "\t\t</constraint>\n";
		return s;
	}

	public void makeConstraint() {
		for(PVariable variable : getScope()) {
			AbstractAgent agent = variable.getDisChocoAgent();
			if(ownerId==-1 || agent.getAgentId() == this.ownerId){
				this.makeConstraint(agent);
			}
		}
	}

	private void makeConstraint(AbstractAgent agent) {
		if (agent instanceof SimpleAgent) {
			this.makeSConstraint((SimpleAgent) agent);
		} else {
			throw new UnsupportedOperationException("Not yet implemneted for complex problems");
		}
	}

	private void makeSConstraint(SimpleAgent agent) {
		String[] pp = this.getUniversalPostfixExpression();
		Deque<Object> q = new ArrayDeque<Object>();
		for (int i = 0 ; i < pp.length ; i++) {
			String val = pp[i];
			if (!(boolExp.containsKey(val) || constExp.containsKey(val) || intExp.containsKey(val) || boolPartExp.containsKey(val))) {
				try {
					q.addFirst(new IntConstantExpression(Integer.valueOf(val)));
				} catch (NumberFormatException e) {
					// It must be a variable
					int idx = Integer.parseInt(val.substring(1));
					IntVariable x = this.scope[idx].makeIntVariable(agent);
					IntVariableExpression var = new IntVariableExpression(x);
					q.addFirst(var);
				}
			} else if (boolExp.containsKey(val)) {

				BooleanExp b = (BooleanExp) boolExp.get(val);
				Constraint[] is = new Constraint[b.arity];
				for (int j = 0 ; j < b.arity ; j++) {
					is[j] = (Constraint) q.removeFirst();
				}
				q.addFirst(createBool(agent, b, is));

			} else if (constExp.containsKey(val)) {
				ConstExp b = (ConstExp) constExp.get(val);
				IntegerExpression[] is = new IntegerExpression[b.arity];
				for (int j = 0 ; j < b.arity ; j++) {
					is[j] = (IntegerExpression) q.removeFirst();
				}
				Constraint c = createExp(agent, b, is);
				q.addFirst(c);

			} else if (intExp.containsKey(val)) {
				IntExp b = (IntExp) intExp.get(val);
				IntegerExpression[] is = new IntegerExpression[b.arity];
				for (int j = 0 ; j < b.arity ; j++) {
					is[j] = (IntegerExpression) q.removeFirst();
				}
				q.addFirst(createInt(b, is));

			} else if (boolPartExp.containsKey(val)) {
				BooleanPartExp b = (BooleanPartExp) boolPartExp.get(val);
				Constraint c = (Constraint) q.removeFirst();
				IntegerExpression[] iss = new IntegerExpression[b.arity];
				for (int j = 0 ; j < b.arity ; j++) {
					iss[j] = (IntegerExpression) q.removeFirst();
				}
				q.addFirst(createBoolPart(b, c, iss));

			} else {
				throw new Error("I don't know what to do with that :" + val);
			}
		}

	}

	public static IntegerExpression createInt(IntExp expr, IntegerExpression... intExps) {
		switch (expr) {
			case add :
				return plus(intExps[0], intExps[1]);
			case div :
				return div(intExps[0], intExps[1]);
			case mod :
				return mod(intExps[0], intExps[1]);
			case mul :
				return mult(intExps[0], intExps[1]);
			case neg :
				return neg(intExps[0]);
			case pow :
				return pow(intExps[0], intExps[1]);
			case sub :
				return minus(intExps[0], intExps[1]);
			case abs :
				return abs(intExps[0]);
			case max :
				return max(intExps[0], intExps[1]);
			case min :
				return min(intExps[0], intExps[1]);
			default :
				return null;
		}
	}

	private IntegerExpression createBoolPart(BooleanPartExp b, Constraint c, IntegerExpression... iss) {
		switch (b) {
			case ifte :
				return ifThenElse(new ConstraintExpression(c), iss[0], iss[1]);
			default :
				return null;
		}
	}

	private Constraint createBool(SimpleAgent agent, BooleanExp b, Constraint... i) {
		switch (b) {
			case and :
				if (i.length == 2) { return and(agent, i[0], i[1]); }
				throw new Error("todo");
			case iff :
				if (i.length == 2) { return ifOnlyIf(agent, i[0], i[1]); }
				throw new Error("todo");
			case not :
				if (i.length == 1) { return not(agent, i[0]); }
				throw new Error("todo");
			case or :
				if (i.length == 2) { return or(agent, i[0], i[1]); }
				throw new Error("todo");
			case xor :
				if (i.length == 2) {
					// A XOR B = (A ET non B) OU (non A ET B).
					return or(agent, and(agent, i[0], not(agent, i[1])), and(agent, i[1], not(agent, i[0])));
				}
				throw new Error("todo");
			default :
				return null;
		}
	}

	private Constraint createExp(SimpleAgent agent, ConstExp b, IntegerExpression... i) {

		switch (b) {
			case eq :
				return eq(agent, i[0], i[1]);
			case fals :
				return FALSE;
			case ge :
				return geq(agent, i[0], i[1]);
			case gt :
				return gt(agent, i[0], i[1]);
			case le :
				return leq(agent, i[0], i[1]);
			case lt :
				return lt(agent, i[0], i[1]);
			case ne :
				return neq(agent, i[0], i[1]);
			case tru :
				return TRUE;
			case distanceEQ :
				if (i[2] instanceof IntConstantExpression)
					return distanceEQ(agent, i[0], i[1], ((IntConstantExpression) i[2]).evaluate());
				else
					return distanceEQ(agent, i[0], i[1], i[2]);
			case distanceNEQ :
				if (i[2] instanceof IntConstantExpression)
					return distanceNEQ(agent, i[0], i[1], ((IntConstantExpression) i[2]).evaluate());
				else
					throw new Error("todo");
			case distanceGT :
				if (i[2] instanceof IntConstantExpression)
					return distanceGT(agent, i[0], i[1], ((IntConstantExpression) i[2]).evaluate());
				else
					return distanceGT(agent, i[0], i[1], i[2]);
			case distanceLT :
				if (i[2] instanceof IntConstantExpression)
					return distanceLT(agent, i[0], i[1], ((IntConstantExpression) i[2]).evaluate());
				else
					return distanceLT(agent, i[0], i[1], (i[2]));
			case oppSign :
				return oppositeSign(agent, i[0], i[1]);
			case sameSign :
				return sameSign(agent, i[0], i[1]);
			case minDisChoco :
				return min(agent, i[1], i[2], i[0]);
			case maxDisChoco :
				return max(agent, i[1], i[2], i[0]);
			default :
				return null;
		}
	}

	public enum ConstExp {
		tru("true", 0), 
		fals("false", 0), 
		eq("eq", 2), 
		ne("ne", 2), 
		ge("ge", 2), 
		gt("gt", 2), 
		le("le", 2), 
		lt("lt", 2), 
		distanceEQ("distEQ", 3), 
		distanceNEQ("distNEQ", 3), 
		distanceGT("distGT", 3), 
		distanceLT("distLT", 3), 
		oppSign("oppSign", 2), 
		sameSign("sameSign", 2), 
		minDisChoco("minDisChoco", 3), 
		maxDisChoco("maxDisChoco", 3);

		String name;
		int arity;

		private ConstExp(String name, int arity) {
			this.name = name;
			this.arity = arity;
		}
	}

	private static Map<String, ConstExp> constExp = new HashMap<String, ConstExp>();
	static {
		constExp.put("eq", ConstExp.eq);
		constExp.put("ne", ConstExp.ne);
		constExp.put("ge", ConstExp.ge);
		constExp.put("gt", ConstExp.gt);
		constExp.put("le", ConstExp.le);
		constExp.put("lt", ConstExp.lt);
		constExp.put("true", ConstExp.tru);
		constExp.put("false", ConstExp.fals);
		constExp.put("distEQ", ConstExp.distanceEQ);
		constExp.put("distNEQ", ConstExp.distanceNEQ);
		constExp.put("distGT", ConstExp.distanceGT);
		constExp.put("distLT", ConstExp.distanceLT);
		constExp.put("oppSign", ConstExp.oppSign);
		constExp.put("sameSign", ConstExp.sameSign);
		constExp.put("minChoco", ConstExp.minDisChoco);
		constExp.put("maxChoco", ConstExp.maxDisChoco);

	};

	protected enum BooleanExp {
		and("and", 2), not("not", 1), or("or", 2), xor("xor", 2), iff("iff", 2);
		String name;
		int arity;

		private BooleanExp(String name, int arity) {
			this.name = name;
			this.arity = arity;
		}
	}

	private static Map<String, BooleanExp> boolExp = new HashMap<String, BooleanExp>();
	static {
		boolExp.put("and", BooleanExp.and);
		boolExp.put("not", BooleanExp.not);
		boolExp.put("or", BooleanExp.or);
		boolExp.put("xor", BooleanExp.xor);
		boolExp.put("iff", BooleanExp.iff);
	};

	protected enum BooleanPartExp {
		ifte("if", 2);
		String name;
		int arity;

		private BooleanPartExp(String name, int arity) {
			this.name = name;
			this.arity = arity;
		}
	}

	private static Map<String, BooleanPartExp> boolPartExp = new HashMap<String, BooleanPartExp>();
	static {
		boolPartExp.put("if", BooleanPartExp.ifte);
	};

	protected enum IntExp {
		neg("neg", 1), add("add", 2), sub("sub", 2), mul("mul", 2), div("div", 2), mod("mod", 2), pow("pow", 2), abs("abs", 1), min("min", 2), max(
				"max", 2);
		String name;
		int arity;

		private IntExp(String name, int arity) {
			this.name = name;
			this.arity = arity;
		}
	}

	private static Map<String, IntExp> intExp = new HashMap<String, IntExp>();
	static {
		intExp.put("neg", IntExp.neg);
		intExp.put("add", IntExp.add);
		intExp.put("sub", IntExp.sub);
		intExp.put("mul", IntExp.mul);
		intExp.put("div", IntExp.div);
		intExp.put("mod", IntExp.mod);
		intExp.put("pow", IntExp.pow);
		intExp.put("abs", IntExp.abs);
		intExp.put("min", IntExp.min);
		intExp.put("max", IntExp.max);
	};

}
