package simulation.parser;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.Arrays;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import kernel.AbstractAgent;

public class PRelation {

	protected String name;

	protected int nbTuples;

	protected int arity;

	protected String semantics;

	protected int[][] tuples;

	public PRelation(String name, int arity, int nbTuples, String semantics) {
		this.name = name;
		this.arity = arity;
		this.nbTuples = nbTuples;
		this.semantics = semantics;
	}

	public PRelation(String name, int arity, int nbTuples, String semantics, int[][] tuples) {
		this(name, arity, nbTuples, semantics);
		this.tuples = tuples;
		Arrays.sort(this.tuples , Toolkit.lexicographicComparator);
	}

	public String getName() {
		return name;
	}

	public int getArity() {
		return arity;
	}

	public String getSemantics() {
		return semantics;
	}

	public int[][] getTuples() {
		return tuples;
	}

	public int getMaximalCost() {
		return 1;
	}

	protected int computeCostOf(int[] tuple) {
		int position = Arrays.binarySearch(tuples, tuple, Toolkit.lexicographicComparator);
		if (semantics.equals(InstanceTokens.SUPPORTS)){
			return position >= 0? 0 : 1;
		}
		return position >= 0? 1 : 0;
	}

	public boolean isSimilarTo(int arity, int nbTuples, String semantics, int[][] tuples) {
		if (this.arity != arity || this.nbTuples != nbTuples)
			return false;
		if (!this.semantics.equals(semantics))
			return false;
		for (int i = 0 ; i < tuples.length ; i++)
			for (int j = 0 ; j < tuples[i].length ; j++)
				if (this.tuples[i][j] != tuples[i][j])
					return false;
		return true;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("<").append(InstanceTokens.RELATION).append(" ");// "<relation "
		sb.append(InstanceTokens.NAME).append("=\"").append(this.name).append("\" "); // name=this.name
		sb.append(InstanceTokens.ARITY).append("=\"").append(this.arity).append("\" "); // arity=this.arity
		sb.append(InstanceTokens.SEMANTICS).append("=\"").append(this.semantics).append("\" "); // semantics=this.semantics
		sb.append(InstanceTokens.NB_TUPLES).append("=\"").append(this.nbTuples).append("\""); // nbTuples=this.nbTuples
		sb.append(">");
//		for (int i = 0 ; i < nbTuples - 1 ; i++) {
//			for (int j = 0 ; j < arity ; j++) {
//				sb.append(tuples[i][j]);
//				if(j < arity - 1) sb.append(' ');
//			}
//			sb.append(InstanceTokens.TUPLES_SEPARATOR);
//		}
//		for(int j = 0 ; j < arity ; j++) {
//			sb.append(tuples[nbTuples - 1][j]);
//			if(j < arity - 1) sb.append(" ");
//		}
		sb.append(getTuplesToString());
		sb.append("</").append(InstanceTokens.RELATION).append(">\n");// "</relation>"
		return sb.toString();
	}

	public String getTuplesToString() {
		StringBuffer sb = new StringBuffer();
		for (int i = 0 ; i < nbTuples - 1 ; i++) {
			for (int j = 0 ; j < arity ; j++) {
				sb.append(tuples[i][j]);
				if(j < arity - 1) sb.append(' ');
			}
			sb.append(InstanceTokens.TUPLES_SEPARATOR);
		}
		for(int j = 0 ; j < arity ; j++) {
			sb.append(tuples[nbTuples - 1][j]);
			if(j < arity - 1) sb.append(" ");
		}
		return sb.toString();
	}

	public int getNbTuples() {
		return this.nbTuples;
	}

	/**
	 * @param agent
	 * @param scope
	 * @param name
	 */
	public void makeBinaryConstraint(String name, AbstractAgent owner, PVariable[] scope) {
		if(owner.getAgentId() == scope[0].getAgentId()){
			this.makeBinaryConstraint(name, owner, scope[0], scope[1], false);
		} else {
			this.makeBinaryConstraint(name, owner, scope[1], scope[0], true);
		}
	}

	/**
	 * @param owner
	 * @param v0
	 * @param v1
	 * @param name
	 */
	protected void makeBinaryConstraint(String name, AbstractAgent owner, PVariable v0, PVariable v1, boolean transpose) {
		boolean feasible = semantics.equals(InstanceTokens.SUPPORTS);
		owner.makeConstraintTable(name, feasible, v0, v1, tuples, transpose);
	}
	
	/**
	 * 
	 * @param scope
	 * @param name
	 * @param count
	 * @return
	 */
	public JInternalFrame makeConstraintInternalFrame(PVariable[] scope, String name, int count) {
				// title, resizable, closable, maximizable, iconifiable
		String title = "Constraint "+name+" [";
		int i=0;
		while(i<scope.length-1) {
			title += scope[i].getName()+", ";
			i++;
		}
		title += scope[i].getName()+"]";
		JInternalFrame frame = new JInternalFrame(title, true, false, true, true);
		frame.add(this.makeConstraintTable(scope, name));
		frame.setBounds(70*count, 5*count, scope.length*150, 400);
		return frame;
	}

	public JPanel makeConstraintTable(PVariable[] scope, String name) {
        String[] columnNames = getColumnNames(scope, name);
        String[][] data = getData();
		JTable table = new JTable(data, columnNames);
		table.setRowHeight(22);
		table.setIntercellSpacing(new Dimension(1, 1));
		table.getTableHeader().setReorderingAllowed(false);
		table.getTableHeader().setResizingAllowed(false);
		table.setShowVerticalLines(true);
		table.setShowHorizontalLines(true);
		table.setGridColor(Color.gray);
		table.setBounds(0, 0, scope.length*150, 400);
		return makeTitledPanel("Constraint "+name, new javax.swing.JScrollPane(table));//table);
	}

	public JTable getConstraintTable(PVariable[] scope, String name) {
        String[] columnNames = getColumnNames(scope, name);
        RelationTableModel model = new RelationTableModel(columnNames, tuples, getWeights());
		JTable table = new JTable(model);
		table.setRowHeight(22);
		table.setIntercellSpacing(new Dimension(1, 1));
		table.getTableHeader().setReorderingAllowed(false);
		table.getTableHeader().setResizingAllowed(false);
		table.setShowVerticalLines(true);
		table.setShowHorizontalLines(true);
		table.setGridColor(Color.gray);
		table.setBounds(0, 0, scope.length*150, 400);
		for(int column = 0 ; column < table.getColumnCount() ; column++) {
			TableColumn tm = table.getColumnModel().getColumn(column);
			tm.setCellRenderer(new gui.debug.RelationRenderer());
		}
		return table;
	}

	/**
	 * @return
	 */
	private String[][] getData() {
		String[][] data = new String[tuples.length][];
		for(int i=0; i<tuples.length ; i++) {
			data[i] = new String[tuples[i].length+1];
			int j=0;
			while(j<tuples[i].length) {
				data[i][j] = Integer.toString(tuples[i][j]);
				j++;
			}
			data[i][j] = Integer.toString(computeCostOf(tuples[i]));
		}
		return data;
	}

	/**
	 * @return
	 */
	private int[] getWeights() {
		int[] weights = new int[tuples.length];
		for(int i=0; i<tuples.length ; i++) {
			try {
				Integer cost = computeCostOf(tuples[i]);
				System.out.print(Arrays.toString(tuples[i])+"="+cost+" ");
				weights[i] = cost;
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
		System.out.println("");
		return weights;
	}

	/**
	 * @return
	 */
	protected String[] getColumnNames(PVariable[] variables, String name) {
		String[] columnNames = new String[variables.length+1];
		int i=0;
		while( i<variables.length) {
			columnNames[i] = variables[i].getName();
			i++;
		}
		columnNames[i] = name;
		return columnNames;
	}

	private static JPanel makeTitledPanel(String title, JComponent component) {
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(component, BorderLayout.CENTER);
		panel.setBorder(BorderFactory.createTitledBorder(title));
		return panel;
	}
	
	public class RelationTableModel extends AbstractTableModel {
		private static final long serialVersionUID = 1L;
		private int[][] tuples = null;
		private int[] weights = null;
		private String[] headers = null;

		public RelationTableModel(String[] headers, int[][] tuples, int[] weights) {
			super();
			this.headers = headers;
			this.tuples = tuples;
			this.weights = weights;
		}

		public int getRowCount() {
			return tuples.length;
		}

		public int getColumnCount() {
			return headers.length;
		}

		public String getColumnName(int columnIndex) {
			return headers[columnIndex];
		}

		/*
		 * (non-Javadoc)
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			if (columnIndex < headers.length - 1) {
				return tuples[rowIndex][columnIndex];
			} else if (columnIndex == (headers.length - 1)) { return weights[rowIndex]; }
			return null;
		}

		@Override
		public Class<?> getColumnClass(int columnIndex) {
			return Integer.class;
		}

		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return false;
		}

		public boolean lookForRow(int[] tuple) {
			int row = Arrays.binarySearch(tuples, tuple, Toolkit.lexicographicComparator);
			if (row > 0) {
				return true; 
			}
			return false;
		}

		public int binarySearch(int[] tuple) {
			return Arrays.binarySearch(tuples, tuple, Toolkit.lexicographicComparator);
		}
	}

	/**
	 * @param name
	 * @param agent
	 * @param variable
	 */
	public void makeUnaryConstraint(String name, AbstractAgent owner, PVariable variable) {
		throw new UnsupportedOperationException("Not yet implemented");
	}
}

