/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package util;

/**
 * Implements utilities for handling bit sets
 */
public class BitSet {

	/**
	 * tests whether the i-th bit of an search is on or off
	 */
	public static boolean getBit(int n, int i) {
		return (((n >> i) & 1) == 1);
	}

	/**
	 * sets the i-th bit of an search (does nothing if it is already on)
	 */
	public static int setBit(int n, int i) {
		if (!getBit(n, i))
			n += (1 << i);
		return n;
	}

	/**
	 * sets the i-th bit of an search (does nothing if it is already on)
	 */
	public static int unsetBit(int n, int i) {
		if (getBit(n, i)) {
			n -= (1 << i);
		}
		return n;
	}

	/**
	 * returns the heaviest bit on (-1 if none)
	 */
	public static int getHeavierBit(int n) {
		for (int i = 31 ; i >= 0 ; i--) {
			if (getBit(n, i))
				return i;
		}
		return -1;
	}

	/**
	 * i inclusive
	 * 
	 * @param n
	 * @param i
	 */
	public static int lowestBit(int n, int i) {
		for (int j = i ; j <= 31 ; j++) {
			if (getBit(n, j))
				return j;
		}
		return 32;
	}

	public static int lowestBit(int n) {
		return lowestBit(n, 0);
	}

	public static String toBitString(int n) {
		int tab[] = new int[32];
		for (int i = 0 ; i < 32 ; i++) {
			if (getBit(n, i))
				tab[31 - i] = 1;
		}
		return Tools.arrayToString(tab);
	}

}
