/*
 * DisChoco: Distributed Constraint Reasoning Solver
 * Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi
 * DisChoco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * DisChoco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * To contact the authors: <http://dischoco.sourceforge.net/>
 */
package util;

import java.io.Serializable;
import java.util.Comparator;
import java.util.StringTokenizer;
import kernel.VarIdentifier;

public class DefaultComparator implements Comparator<VarAssignment>, Serializable {

	private static final long serialVersionUID = -2034287212234330234L;

	private HashList<VarIdentifier> order;

	public DefaultComparator(String string) {
		this.order = this.extractVarIds(string);
	}

	@Override
	public int compare(VarAssignment v1, VarAssignment v2) {
		return order.indexOf(v1.getVarIdentifier()) - order.indexOf(v2.getVarIdentifier());
	}

	public HashList<VarIdentifier> extractVarIds(String string) {
		HashList<VarIdentifier> ids = new HashList<VarIdentifier>();
		if (string.contains("[")) {
			string = string.substring(string.indexOf("[") + 1, string.indexOf("]"));
		}
		StringTokenizer tok = new StringTokenizer(string, ",");
		while (tok.hasMoreTokens()) {
			VarIdentifier id = VarAssignment.fromString(tok.nextToken()).getVarIdentifier();
			ids.add(id);
		}
		return ids;
	}

	public void update(VarIdentifier varId) {
		if (order.indexOf(varId) == -1){
			order.add(varId);
		}
	}

}
