/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package util;

import java.util.Iterator;

/**
 * An interface for collections over native integers. (among other things, this is useful for iterating domains) Note that it does not extend
 * Iterator, as we iterate over native int, and not Object.
 */

public interface IntIterator extends Iterator<Integer> {
	/**
	 * Returns <tt>true</tt> if the iteration has more elements. (In other words, returns <tt>true</tt> if <tt>next</tt> would return an element
	 * rather than throwing an exception.)
	 * 
	 * @return <tt>true</tt> if the getIterator has more elements.
	 */
	boolean hasNext();

	/**
	 * Returns the next element in the iteration.
	 * 
	 * @return the next element in the iteration.
	 * @throws java.util.NoSuchElementException
	 *             iteration has no more elements.
	 */
	Integer next();

	/**
	 * Removes from the underlying collection the last element returned by the getIterator (optional operation). This method can be called only once
	 * per call to <tt>next</tt>. The behavior of an getIterator is unspecified if the underlying collection is modified while the iteration is in
	 * progress in any way other than by calling this method.
	 * 
	 * @throws java.lang.UnsupportedOperationException
	 *             if the <tt>remove</tt> operation is not supported by this Iterator.
	 * @throws java.lang.IllegalStateException
	 *             if the <tt>next</tt> method has not yet been called, or the <tt>remove</tt> method has already been called after the last call to
	 *             the <tt>next</tt> method.
	 */
	void remove();
}
