/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package util;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

public class Logger {

	private PrintWriter nLog = null;

	private String name;

	private boolean active = false;

	public static boolean LOG_DEBUG = false;

	public static Integer ERROR_LEVEL 			= new Integer(0);
	public static Integer INFO_LEVEL 			= new Integer(1);
	public static Integer LEVEL_TRACE_MESSAGE 	= new Integer(2);
	public static Integer DEFAULT_LEVEL 		= new Integer(3);
	public static Integer FINE_LEVEL 			= new Integer(4);
	public static Integer LEVEL_TRACE_FUNCTION_1 = new Integer(5);
	public static Integer LEVEL_TRACE_FUNCTION_2 = new Integer(6);

	public static Integer log_debug_level = DEFAULT_LEVEL;

	public Logger(String fname) {
		this.name = fname + ".txt";
		String path = "."+File.separator ;
		try {
			File f = new File(path + "logs");
			if (!f.exists()) { f.mkdir(); }
		} catch (Exception e) { e.printStackTrace(); }
		try {
			Runtime.getRuntime().exec("del " + path + "logs" + File.separator + fname);
			Runtime.getRuntime().exec("rm " + path + "logs" + File.separator + fname);
		} catch (Exception e) {}
		try {
			FileWriter ffile = new FileWriter(path + "logs" + File.separator + fname);
			PrintWriter pwtr = new PrintWriter(ffile);
			this.nLog = pwtr;
		} catch (IOException e) {
			System.err.println("Error opening " + path + "logs" + File.separator + fname);
		}
	}

	/**
	 * 
	 * @param directory
	 * @param fname
	 */
	public Logger(String directory, String fname) {
		this.name = fname;
//		log_debug_level = DEFAULT_LEVEL;
		String path = directory;
		if (!path.endsWith(File.separator)) {
			path += File.separator ;
		}
		try {
			File f = new File(path);
			if (!f.exists()) {
				f.mkdir();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			FileWriter ffile = new FileWriter(path + File.separator + fname, false);
			PrintWriter pwtr = new PrintWriter(ffile, true);
			nLog = pwtr;
		} catch (IOException ee) {
			System.err.println(path + File.separator + fname);
			ee.printStackTrace();
		}
	}

	public synchronized void printToLog(String s, Integer level) {
		if (level.compareTo(log_debug_level) <= 0) {
			this.nLog.print(s);
			this.nLog.flush();
			System.out.print(s);
		}
	}

	public synchronized void printToLog(String s) {
		printToLog(s, DEFAULT_LEVEL);
	}

	public synchronized void printToLogln(String s) {
		printToLogln(s, DEFAULT_LEVEL);
	}

	public synchronized void printToLogln(String s, Integer level) {
		if (level.compareTo(log_debug_level) <= 0) {
			this.nLog.println(s);
			this.nLog.flush();
			System.out.println(s);
		}
	}

	public void activate() {
		this.active = true;
	}

	public boolean isActivated() {
		return (this.active || LOG_DEBUG);
	}

	public void close() {
		this.nLog.close();
	}

	public String getFileName() {
		return this.name;
	}

	public PrintWriter getPrintWriter() {
		return this.nLog;
	}

}
