/*
DisChoco: Distributed Constraint Reasoning Solver 
Copyright (C) 2007-2013 Redouane Ezzahir & Mohamed Wahbi 

DisChoco is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DisChoco is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

To contact the authors: <http://dischoco.sourceforge.net/>
*/ 
package util;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.StringWriter;
import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.math.BigInteger;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.BitSet;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.Vector;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

/**
 * Class Utility
 */
@SuppressWarnings("unchecked")
public class Tools {
	
	private static String[] alphabet = { 
		"a", "b", "c", "d", "e", "f", "g",
		"h", "i", "j", "k", "l", "m", "n", 
		"o", "p", "q", "r", "s", "t", "u", 
		"v", "w", "x", "y", "z" };
	
	private static String[] ALPHABET = { 
		"A", "B", "C", "D", "E", "F", "G",
		"H", "I", "J", "K", "L", "M", "N", 
		"O", "P", "Q", "R", "S", "T", "U", 
		"V", "W", "X", "Y", "Z" };

	/**
	 * Return current time as a string for printing
	 */
	public synchronized static String TimeString() {
		Date d = new Date(System.currentTimeMillis());
		return (d.toString());
	}

	/**
	 * 
	 */
	public static HashMap<String, Double> merge(HashMap<String, Double> stat1, HashMap<String, Double> stat2) {
		HashMap<String, Double> statistics = new HashMap<String, Double>();// (HashMap<String, Long>) stat2.clone();
		Iterator<String> it = stat1.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			Double value = stat1.get(key);
			if (value == null)
				value = 0D;
			statistics.put(key, value);
		}
		it = stat2.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			Double value = stat2.get(key);
			if (value == null)
				value = 0D;
			if (statistics.containsKey(key)) {

				statistics.put(key, value + statistics.get(key));
			} else {
				statistics.put(key, value);
			}
		}
		return statistics;
	}

	/**
	 * @param arr
	 * @return the transpose of this matrix
	 */
	public synchronized static int[][] intArrayTranspose(int arr[][]) {
		int rez[][] = new int[arr[0].length][arr.length];
		for (int i = 0 ; i < arr[0].length ; i++) {
			for (int j = 0 ; j < arr.length ; j++) {
				rez[i][j] = arr[j][i];
			}
		}
		return rez;
	}

	public synchronized static final int combine_hash(int x, int y) {
		int rc = 0;
		for (int i = 7 ; i >= 0 ; --i) {
			if (i == 4)
				x = y;
			rc = rc * 21 + (char) x;
			x >>>= 8;
		}
		return rc;
	}

	public static HashMap<Integer, Integer> IntegerHashMapfromString(String str) {
		HashMap<Integer, Integer> result = new HashMap<Integer, Integer>();
		str.trim();
		str = str.substring(1);
		str = str.substring(0, str.indexOf('}')).replaceAll(" ", "");
		StringTokenizer tok = new StringTokenizer(str.trim(), ",");
		while (tok.hasMoreTokens()) {
			String element = tok.nextToken();
			String[] keyval = element.split("=");
			result.put(Integer.parseInt(keyval[0]), Integer.parseInt(keyval[1]));
		}
		return result;
	}

	public synchronized static Vector<Integer> IntegerVectorFromString(String line) {
		Vector<Integer> res = null;
		if (line != null) {
			try {
				res = new Vector<Integer>();
				StringTokenizer st = new StringTokenizer(line);
				// reading integers
				while (st.hasMoreElements()) {
					Integer tmp = Integer.valueOf(st.nextToken(", \t\n\r\f"));
					res.add(tmp);
				}
			} catch (Exception e) {
				e.printStackTrace();
				return null;
			}
		}
		return res;
	}

	// comma or space/tab separated
	public synchronized static int[] IntegerArrayFromString(String line, String separator) {
		int[] values = null;
		if (line != null) {
			String str = line.trim();
			if (str.startsWith("["))
				str = str.substring(1);
			if (str.endsWith("]"))
				str = str.substring(0, str.length() - 1);
			str = str.trim();
			if (!str.equalsIgnoreCase("")) {
				// System.out.println("Gonna build context from:|" + str + "|");
				StringTokenizer tok = new StringTokenizer(str, separator);
				values = new int[tok.countTokens()];
				int i = 0;
				while (tok.hasMoreTokens()) {
					values[i++] = Integer.parseInt(tok.nextToken().trim());
				}
			}
		}
		return values;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static int compareArrays(int[] x, int[] y) {
		int i = 0;
		if (x.length != y.length)
			return Integer.MAX_VALUE;
		// System.out.println("S1 = " + x.toString());
		// System.out.println("S2 = " + y.toString());
		for (int j = 0 ; j < x.length ; j++) {
			if (x[j] != y[j])
				i++;
		}
		return i;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static double arrayAverage(double[] array) {
		double rez = 0;
		for (int i = 0 ; i < array.length ; i++) {
			rez += array[i];
		}
		rez = rez / array.length;
		return rez;
	}

	// --------------------------------------------------------------------------------------------------------------------//

	public static int indexOfMax(int[] data) {
		int indexOfMax = 0; // initial guess
		if (data.length > 1)
			for (int i = 1 ; i < data.length ; i++)
				if (data[i] > data[indexOfMax])
					indexOfMax = i; // found a smaller value
		return indexOfMax;
	}

	// --------------------------------------------------------------------------------------------------------------------//

	public synchronized static int indexOfMin(int[] data) {
		int indexOfMin = 0; // initial guess
		if (data.length > 1)
			for (int i = 1 ; i < data.length ; i++)
				if (data[i] < data[indexOfMin])
					indexOfMin = i; // found a smaller value
		return indexOfMin;
	}

	// --------------------------------------------------------------------------------------------------------------------//

	public synchronized static int indexOfMin(double[] data) {
		int indexOfMin = 0; // initial guess
		for (int i = 1 ; i < data.length ; i++)
			if (data[i] < data[indexOfMin])
				indexOfMin = i; // found a smaller value
		return indexOfMin;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	/**
	 *@return min value from the gevin array;
	 */
	public synchronized static double arrayMinimum(double[] array) {
		double min = Double.POSITIVE_INFINITY;
		for (int i = 0 ; i < array.length ; i++) {
			if (min > array[i])
				min = array[i];
		}
		return min;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static double arrayMaximum(double[] array) {
		double max = Double.NEGATIVE_INFINITY;
		for (int i = 0 ; i < array.length ; i++) {
			if (max < array[i])
				max = array[i];
		}
		return max;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static int arrayAverage(int[] array) {
		int rez = 0;
		for (int i = 0 ; i < array.length ; i++) {
			rez += array[i];
		}
		rez = rez / array.length;
		return rez;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	/**
	 *@return min value from the gevin array;
	 */
	public synchronized static int arrayMinimum(int[] array) {
		return array[indexOfMin(array)];
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static int arrayMaximum(int[] array) {
		int max = Integer.MIN_VALUE;
		for (int i = 0 ; i < array.length ; i++) {
			if (max < array[i])
				max = array[i];
		}
		return max;
	}

	public static int sum(int[] array) {
		int res = 0;
		for (int i = 0 ; i < array.length ; i++)
			res += array[i];
		return res;
	}

	public static int sum(int[] array, int to) {
		int res = 0;
		for (int i = 0 ; i < array.length && i < to ; i++)
			res += array[i];
		return res;
	}
	
	// ---------------------------------------------------------------------------------------//
	public synchronized static boolean isAllArrayTrue(boolean[] array) {
		for (int i = 0 ; i < array.length ; i++) {
			if (!array[i])
				return false;
		}
		return true;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(boolean[] array) {
		return Arrays.toString(array);
	}

	public synchronized static TreeSet<Integer> arrayToTreeSet(int[] array) {
		TreeSet<Integer> treeSet = new TreeSet<Integer>();
		for (int i = 0 ; i < array.length ; i++) {
			treeSet.add(array[i]);
		}
		return treeSet;
	}

	public synchronized static int[] arrayFromTreeSet(TreeSet<Integer> treeSet) {
		int[] array = new int[treeSet.size()];
		Iterator<Integer> it = treeSet.iterator();
		int i = 0;
		while (it.hasNext()) {
			array[i] = it.next();
			i++;
		}
		return array;
	}
	
	/**
	 * 
	 */
	public static float Homogeneity(HashMap<String, Long> statP2, float mediane) {
		float homogeneity = 0;
		Iterator<String> it = statP2.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			float value = statP2.get(key);
			homogeneity += (long) (Math.pow(Math.abs((value - mediane)), 2));
		}
		return homogeneity;
	}

	/**
	 * 
	 */
	public static float Average(HashMap<String, Long> statP2) {
		float mediane = 0;
		Iterator<String> it = statP2.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			mediane += statP2.get(key);
		}
		return mediane / statP2.size();
	}

	/**
	 * 
	 */
	public static void average(HashMap<String, Double> stat, int n) {
		Iterator<String> it = stat.keySet().iterator();
		while (it.hasNext()) {
			String key = it.next();
			stat.put(key, stat.get(key) / n);
		}
	}

	/**
	 * 
	 */
	public static int[] average(int[] stat, int n) {
		int[] ans = stat.clone();
		for (int i = 0 ; i < ans.length ; i++) {
			ans[i] /= n;
		}
		return ans;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(Object[] array) {
		return Arrays.toString(array);
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(Object[] array, int start, int end) {
		if (array == null)
			return "";
		StringBuffer sb = new StringBuffer();
		if (end - start >= 0) {
			for (int i = start ; i < end ; i++) {
				sb.append(array[i] + ",");
			}
			sb.append(array[end]);
		}
		return sb.toString();
	}

	public synchronized static int[] IntegerArrayFromString(String str) {
		if (str == null)
			return null;// new int[0];
		if (str.startsWith("["))
			str = str.substring(1, str.indexOf("]")); // remove [ and ]
		StringTokenizer tok = new StringTokenizer(str);
		int[] array = new int[tok.countTokens()];
		int i = 0;
		while (tok.hasMoreTokens()) {
			array[i++] = Integer.parseInt(tok.nextToken());
		}
		return array;
	}

	public synchronized static short[] ShortArrayFromString(String str) {
		if (str.startsWith("["))
			str = str.substring(1, str.indexOf("]")); // remove [ and ]
		if (str == null)
			return null;// new int[0];
		StringTokenizer tok = new StringTokenizer(str);
		short[] array = new short[tok.countTokens()];
		int i = 0;
		while (tok.hasMoreTokens()) {
			array[i++] = Short.parseShort(tok.nextToken());
		}
		return array;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(int[] array, int start, int end) {
		if (array == null)
			return "null";
		StringBuffer sb = new StringBuffer();
		if (end - start >= 0) {
			for (int i = start ; i < end ; i++) {
				sb.append(array[i] + " ");
			}
			sb.append(array[end]);
		}
		return sb.toString();
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(short[] array, int start, int end) {
		if (array == null)
			return "null";
		StringBuffer sb = new StringBuffer();
		if (end - start >= 0) {
			for (int i = start ; i < end ; i++) {
				sb.append(array[i] + " ");
			}
			sb.append(array[end]);
		}
		return sb.toString();
	}
	
	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(int[] array) {
		return Arrays.toString(array);
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(short[] array) {
		return Arrays.toString(array);
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(int[][] array) {
		if (array == null)
			return "null";
		StringBuffer sb = new StringBuffer();
		sb.append("[");
		for (int i = 0 ; i < array.length ; i++) {
			sb.append(arrayToString(array[i]) + "\n");
		}
		sb.append("]");
		return sb.toString();
	}
	
	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static int[] addArrays(int[] v1, int[] v2) {
		int minLength = Math.min(v1.length, v2.length);
		int[] rez = null;
		if (v1.length < v2.length) {
			rez = new int[v2.length];
			System.arraycopy(v2, 0, rez, 0, v2.length);
		} else {
			rez = new int[v1.length];
			System.arraycopy(v1, 0, rez, 0, v1.length);
		}
		for (int i = 0 ; i < minLength ; i++) {
			rez[i] = v1[i] + v2[i];
		}
		return rez;
	}
	
	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static double[][] copyDoubleArray(double[][] src, double[][] dest) {
		if (src.length != dest.length || src[0].length != dest[0].length)
			return null;
		for (int i = 0 ; i < src.length ; i++)
			for (int j = 0 ; j < src[0].length ; j++)
				dest[i][j] = src[i][j];
		return src;
	}

	// -----------------------------------------------------------------------------------------------------------------------//

	public synchronized static Vector<Integer> arrayToVector(int[] src) {
		if (src == null)
			return null;
		Vector<Integer> dest = new Vector<Integer>();
		for (int i = 0 ; i < src.length ; i++)
			dest.add(new Integer(src[i]));
		return dest;
	}
	
	public synchronized static int[] generatePermutation(Random randomGenerator, int[] perm) {
		// int[] perm = new int[howm];
		Vector<Integer> orig = new Vector<Integer>();
		for (int i = 0 ; i < perm.length ; i++) {
			orig.add(perm[i]);
		}
		boolean firstOk = false;
		for (int i = 0 ; i < perm.length ; i++) {
			int which = randomGenerator.nextInt(perm.length - i);
			if (!firstOk) {
				if (i == which) {
					i--;
					continue;
				} else
					firstOk = true;
			}
			perm[i] = ((Integer) orig.elementAt(which)).intValue();
			orig.remove(which);
		}
		return perm;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String indent(String src, int level) {
		String[] parts = src.split("\n");
		StringBuffer rez = new StringBuffer();
		String indentul = new String();
		for (int i = 0 ; i < level ; i++)
			indentul += "";
		for (int i = 0 ; i < parts.length ; i++)
			rez.append(indentul + parts[i] + "\n");
		return rez.toString();
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static int findInArray(Object[] arr, Object targ) {
		for (int i = 0 ; i < arr.length ; i++) {
			// System.out.print("Comparing " + arr[i] + " with " + targ);
			if (arr[i].equals(targ)) {
				// System.out.println("-> " +i);
				return i;
			}
		}
		// System.out.println("-> -1");
		return -1;
	}
	
	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static boolean arrayContains(Object[] arr, Object targ) {
		// System.out.print("Searching " + targ + " in " + arrayToString(arr));
		if (findInArray(arr, targ) >= 0) {
			// System.out.println("->true");
			return true;
		} else {
			// System.out.println("->false");
			return false;
		}
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static boolean arrayContains(int[] array, int element) {
		if (findInArray(array, element) >= 0) {
			return true;
		} else {
			return false;
		}
	}
	
	// -----------------------------------------------------------------------------------------------------------------//
	public static int[] setMinus(int[] unu, int[] doi) {
		ArrayList<Integer> arrl = new ArrayList<Integer>();
		for (int i = 0 ; i < unu.length ; i++) {
			if (Tools.findInArray(doi, unu[i]) == -1)
				arrl.add(unu[i]);
		}
		// System.out.print("Set minus: " + arrayToString(unu) + " \\ " +
		// arrayToString(doi));
		// System.out.println("->" + arrayToString(arrl.toArray()));

		int[] res = new int[arrl.size()];
		for (int i = 0 ; i < arrl.size() ; i++) {
			res[i] = arrl.get(i);
		}
		return res;
	}
	
	public synchronized static String[] intersection(String[] unu, String[] doi) {
		ArrayList<String> arrl = new ArrayList<String>();
		for (String e : unu) {
			if (Tools.arrayContains(doi, e))
				arrl.add(e);
		}
		String[] ans = new String[arrl.size()];
		for (int i = 0 ; i < arrl.size() ; i++) {
			ans[i] = arrl.get(i);
		}
		return ans;
	}

	public synchronized static String[] union(String[] unu, String[] doi) {
		ArrayList<String> array = new ArrayList<String>();
		for (String e : unu) {
			array.add(e);
		}
		for (String e : doi) {
			if (!array.contains(e)) {
				array.add(e);
			}
		}
		String[] ans = new String[array.size()];
		for (int i = 0 ; i < array.size() ; i++) {
			ans[i] = array.get(i);
		}
		return ans;
	}

	// -----------------------------------------------------------------------------------------------------------------//

	/**
	 * @param arr
	 * @param id
	 * @return the first index where the specified int appears in this array, or -1 if not found
	 */
	public synchronized static int findInArray(int[] arr, int id) {
		for (int i = 0 ; i < arr.length ; i++) {
			if (arr[i] == id)
				return i;
		}
		return -1;
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static void copyArray(Object[] srcarray, Object[] destarray) {
		copyArrayOffset(srcarray, destarray, 0);
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static void copyArrayOffset(Object[] srcarray, Object[] destarray, int offset) {
		if (srcarray.length + offset > destarray.length) {
			System.out.println("Arrays size mismatch: source has " + srcarray.length + " elements and destination has " + destarray.length
					+ " elements.");
			System.exit(1);
		}
		for (int i = 0 ; i < srcarray.length ; i++)
			destarray[i + offset] = srcarray[i];
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static void copyArray(int[] srcarray, int[] destarray) {
		copyArrayOffset(srcarray, destarray, 0);
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static void copyArrayOffset(int[] srcarray, int[] destarray, int offset) {
		if (srcarray.length + offset > destarray.length) {
			System.out.println("Arrays size mismatch: source has " + srcarray.length + " elements and destination has " + destarray.length
					+ " elements.");
			System.exit(1);
		}
		for (int i = 0 ; i < srcarray.length ; i++)
			destarray[i + offset] = srcarray[i];
	}

	// -----------------------------------------------------------------------------------------------------------------//
	/*
	 * public synchronized static Value ValuefromString(String stringValue) { stringValue = stringValue.trim(); if(ParameterTypes.isInt(stringValue))
	 * return new IntValue(Integer.parseInt(stringValue)); else if(stringValue.startsWith("[")){ String bounds[] = stringValue.split(","); double inf
	 * = Double.parseDouble(bounds[0].substring(1)); bounds[1] = bounds[1].split("]")[0]; double sup = Double.parseDouble( bounds[1]); return new
	 * RealValue(inf,sup); } return new nullValue(); }
	 */
	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static BitSet BitSetFromString(String str) { // {1, 3,
		// 323 }
		BitSet b = new BitSet();
		str.trim();
		str = str.substring(1);
		str = str.substring(0, str.indexOf('}'));
		StringTokenizer tok = new StringTokenizer(str.trim(), ",");
		while (tok.hasMoreTokens())
			b.set(Integer.parseInt(tok.nextToken().trim()), true);

		return b;
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public static String convertXMLFileToString(String fileName) {
		try {
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
			InputStream inputStream = new FileInputStream(new File(fileName));
			org.w3c.dom.Document doc = documentBuilderFactory.newDocumentBuilder().parse(inputStream);
			StringWriter stw = new StringWriter();
			Transformer serializer = TransformerFactory.newInstance().newTransformer();
			serializer.transform(new DOMSource(doc), new StreamResult(stw));
			return stw.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public static HashMap<Integer, Integer> mapIntegerIntegerfromString(String str) {
		HashMap<Integer, Integer> ans = new HashMap<Integer, Integer>();
		String map = str.substring(str.indexOf("{") + 1, str.indexOf("}"));// remove { and }
		StringTokenizer tok = new StringTokenizer(map, ",");
		while (tok.hasMoreTokens()) {
			StringTokenizer t = new StringTokenizer(tok.nextToken(), "=");
			int key = Integer.parseInt(t.nextToken().replaceAll(" ", ""));
			int value = Integer.parseInt(t.nextToken().replaceAll(" ", ""));
			ans.put(key, value);
		}
		return ans;
	}
	
	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static final long Factorial(long n) {
		BigInteger fac = BigInteger.ONE;
		for (int i = 1 ; i <= n ; i++) {
			fac = fac.multiply(BigInteger.valueOf(i));
		}
		long f = fac.longValue();
		if (f == 0) { return 1; }
		return f;
	}

	// -----------------------------------------------------------------------------------------------------------------//
	public synchronized static final long Arrangement(long k, long n) {
		if (k > n) { return 0; }
		if (k == 1)
			return n;
		if (k == n)
			return Factorial(n);
		long res = 0;
		res = Factorial(n) / Factorial(n - k);
		return res;
	}

	/**
	 * combinaison de k parmi n
	 * 
	 * @param k
	 * @param n
	 */
	public synchronized static final long C(long n, long k) {
		if (k == 1)
			return n;
		return Arrangement(k, n) / Factorial(k);
	}

	public static int indexOf(int[] data, int element) {
		for (int i = 0 ; i < data.length ; i++) {
			if (data[i] == element) { return i; }
		}
		return -1;
	}

	public static int[] subArray(int[] data, int from, int to) {
		int[] array = new int[to - from];
		for (int i = 0 ; i < array.length ; i++) {
			array[i] = data[i + from];
		}
		return array;
	}

	public static String statisticsToString(ArrayList<String> keys, HashMap<String, Double> statistics) {
		DecimalFormat df = new DecimalFormat("#.0");
	    df.setGroupingUsed(false);
	    df.setGroupingSize(3);
	    df.setMaximumFractionDigits(0);
//		DecimalFormat df = new DecimalFormat("0.##"); //""
//		df.setMinimumFractionDigits(2); 
//		df.setDecimalSeparatorAlwaysShown(false);
//		DecimalFormat df = new DecimalFormat() ; 
//		df.setMaximumFractionDigits(3); //arrondi  2 chiffres apres la virgules 
	    String res = df.format(statistics.get(Commands.MSGs)) + ", " + 
				df.format(statistics.get(Commands.NCCCs)) + ", " + 
				df.format(statistics.get(Commands.GNCCCs)) + ", " + 
				df.format(statistics.get(Commands.NCCPUTime)) + ", " + 
				df.format(statistics.get(Commands.CCs)) + ", " + 
				df.format(statistics.get(Commands.SOLVABLE)) + ", ";
		for (String key : keys) {
			if (!key.equals(Commands.NCCCs) && !key.equals(Commands.GNCCCs) && !key.equals(Commands.NCCPUTime) 
					&& !key.equals(Commands.CCs) && !key.equals(Commands.MSGs) && !key.equals(Commands.SOLVABLE)) {
				if (statistics.containsKey(key)) {
					String value = df.format(statistics.get(key)); 
					res += value.replace(',', '.') + ", "; 
				} else {
					res += "0, ";
				}
			}
		}
		return res;
	}

	public static String statisticsKeyToString(HashMap<String, Long> runStatistics) {
		String keys = Commands.MSGs + ", " + Commands.NCCCs + ", " + Commands.GNCCCs + ", " + 
		Commands.NCCPUTime + ", " + Commands.CCs + ", " + Commands.SOLVABLE + ", ";
		Iterator<String> itr = runStatistics.keySet().iterator();
		while (itr.hasNext()) {
			String key = itr.next();
			if (!key.equals(Commands.NCCCs) && !key.equals(Commands.GNCCCs) && !key.equals(Commands.NCCPUTime) && !key.equals(Commands.CCs) 
					&& !key.equals(Commands.MSGs) && !key.equals(Commands.SOLVABLE)) {
				keys += key;
				if (itr.hasNext()) {
					keys += ", ";
				}
			}
		}
		return keys;
	}

	public static String dataKeyToString(HashMap<String, Integer> keysMap) {
		String keys = Commands.MSGs + "," + Commands.NCCCs + "," + Commands.GNCCCs +  "," + Commands.NCCPUTime + "," + Commands.CCs 
		+ "," + Commands.SOLVABLE + ",";
		Iterator<String> itr = keysMap.keySet().iterator();
		while (itr.hasNext()) {
			String key = itr.next();
			if (!key.equals(Commands.NCCCs) && !key.equals(Commands.GNCCCs) && !key.equals(Commands.NCCPUTime) && !key.equals(Commands.CCs) 
					&& !key.equals(Commands.MSGs) && !key.equals(Commands.SOLVABLE)) {
				keys += key;
				if (itr.hasNext()) {
					keys += ",";
				}
			}
		}
		return keys;
	}

	/**
	 * We compare two arrays, which are supposed to have the same length
	 * 
	 * @param array1
	 * @param array2
	 * @return <b><i>+(i+1)</i></b> if <b>array1</b> is the lowest (<b> <i>array1 < array2</i></b> )<br/>
	 *         <b><i> 0</i></b> if <b>array1</b> and <b>array2</b> are equal<br/>
	 *         <b><i>-(i+1)</i></b> if <b>array2</b> is the best(<b><i>array1 > array2</i></b>)<br/>
	 */
	public static int compare(Double array1[], Double[] array2) {
		for(int i = 0 ; i < array1.length ; i++) {
			if (array2[i] > array1[i]) {
				return i + 1;
			} else if (array2[i] < array1[i]) {
				return -(i + 1);
			}
		}
		return 0;
	}

	// -----------------------------------------------------------------------------------------------------------------------//
	public synchronized static String arrayToString(double[] array) {
		return Arrays.toString(array);
	}

//	public synchronized static double[] DoubleArrayFromString(String str) {
//		if (str.startsWith("["))
//			str = str.substring(1, str.indexOf("]")); // remove [ and ]
//		if (str == null)
//			return null;// new int[0];
//		StringTokenizer tok = new StringTokenizer(str);
//		double[] array = new double[tok.countTokens()];
//		int i = 0;
//		while (tok.hasMoreTokens()) {
//			array[i++] = Double.parseDouble(tok.nextToken());
//		}
//		return array;
//	}

	/**
	 * @param keys
	 * @return
	 */
	public static String toString(ArrayList<String> keys) {
		String str = keys.toString();
		return str.substring(1, str.length()-1);
	}

	/**
	 * @param domDeg
	 * @return
	 */
	public static int MinOf(HashMap<Integer, Double> map, IntArray elements) {
		int min = map.keySet().iterator().next();
		for(int key : elements){
			double value = map.get(key);
			if(value < map.get(min)){
				min = key ; 
			}
		}
		return min;
	}

	/**
	 * @param domDeg
	 * @return
	 */
	public static <T, N extends Number> T MinOf(HashMap<T, N> map) {
		T minKey = map.keySet().iterator().next();
		for(T key : map.keySet()){
			N value = map.get(key);
			if(value.doubleValue() < map.get(minKey).doubleValue()){
				minKey = key ;
			}
		}
		return minKey;
	}
	
	/**
	 * @param domDeg
	 * @return
	 */
	public static <T, N extends Number> T MaxOf(HashMap<T, N> map) {
		T max = map.keySet().iterator().next();
		for(T key : map.keySet()){
			N value = map.get(key);
			if(value.doubleValue() > map.get(max).doubleValue()){
				max = key ; 
			}
		}
		return max;
	}
	
	/**
	 * @param domDeg
	 * @return
	 */
	public static <T, N extends Comparable<? super N>> N MinValueOf(HashMap<T, N> map) {
		return Collections.min(new ArrayList<N>(map.values()));
	}
	
	/**
	 * @param domDeg
	 * @return
	 */
	public static <T, N extends Comparable<N>> N MaxValueOf(HashMap<T, N> map) {
		return Collections.max(new ArrayList<N>(map.values()));
	}

	public static HashMap<Integer, IntArray> mapIntegerIntArrayfromString(String str) {
		str = str.substring(str.indexOf("{") + 1, str.indexOf("}"));// remove { and }
		HashMap<Integer, IntArray> agNeig = new HashMap<Integer, IntArray>();
		StringTokenizer tok = new StringTokenizer(str, ",");
		while (tok.hasMoreTokens()) {
			StringTokenizer t = new StringTokenizer(tok.nextToken(), "=");
			int agId = Integer.parseInt(t.nextToken().replaceAll(" ", ""));
			IntArray neig = IntArray.fromString(t.nextToken());
			agNeig.put(agId, neig);
			
		}
		return agNeig;
	}
	
	public static <K, V> HashMap<K, V> mapfromString(String string, Class<K> keyClass, Class<V> valueClass) {
		string = string.substring(string.indexOf("{") + 1, string.indexOf("}"));// remove { and }
		HashMap<K, V> agNeig = new HashMap<K, V>();
		try {
			StringTokenizer tok = new StringTokenizer(string, ",");
			Constructor<K> keyConstructor = keyClass.getConstructor(new Class[] {String.class});
			Constructor<V> valueConstructor = valueClass.getConstructor(new Class[] {String.class});
			while (tok.hasMoreTokens()) {
				StringTokenizer t = new StringTokenizer(tok.nextToken(), "=");
				String keyString = t.nextToken().replaceAll(" ", "");
				Object[] args = new Object[] {keyString};
				K key  = keyConstructor.newInstance(args);
				String valueString = t.nextToken();
				args = new Object[] {valueString};
				V value  = valueConstructor.newInstance(args);
				agNeig.put(key, value);
				
			}
		} catch (Exception e) {
		}
		return agNeig;
	}
	
	public static int[] toArray(List<Integer> list) {
		int[] array = new int[list.size()];
		for(int i = 0 ; i < array.length ; i++) {
			array[i] = list.get(i).intValue();
		}
		return array;
	}

	public synchronized static <T> T[] arrayFromString(String string, Class<T> clazz) {
		if (string.startsWith("["))
			string = string.substring(1, string.indexOf("]")); // remove [ and ]
		if (string == null)
			return null;// new int[0];
		StringTokenizer tok = new StringTokenizer(string, ",");
		try {
			Constructor<T> constructor = clazz.getConstructor(new Class[] {String.class});
			T[] array = (T[]) Array.newInstance(clazz, tok.countTokens());
			int i = 0;
			while (tok.hasMoreTokens()) {
				Object[] args = new Object[] {tok.nextToken().trim()};
				T element  = constructor.newInstance(args);
				array[i++] = element;
			}
			return array;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static int tryParseInt(String str) {
		for(String s : alphabet) {
			str = str.replaceAll(s, "");
		}
		for(String s : ALPHABET) {
			str = str.replaceAll(s, "");
		}
		str = str.replaceAll("[\\-\\+\\.\\^:,]","");
		return Integer.parseInt(str);
	}


//	public static void main(String[] args) {
//		// LinkedList<String> box = new LinkedList<String>();
//		// box.addLast("A");
//		// box.addLast("B");
//		// box.addLast("C");
//		// box.addLast("D");
//		// box.addLast("E");
//		// Iterator<String> it = box.iterator();
//		// System.out.println(box);
//		// while (it.hasNext()) {
//		// System.out.println(it.next());
//		// }
//		int[] tab = {1, 5, 9, 6, 3};
//		String toString = Tools.arrayToString(tab);
//		System.out.println(toString);
//		Short[] array = Tools.arrayFromString(toString, Short.class);
//		System.out.println(Tools.arrayToString(array));
////		TreeSet<Integer> treeSet = Tools.arrayToTreeSet(tab);
////		System.out.println(treeSet.toString());
////		int[] tab1 = Tools.arrayFromTreeSet(treeSet);
////		System.out.println(Tools.arrayToString(tab1));
//	}
}
